@@ -90,20 +90,20 @@ time_t ModMArch::end()
 
 void ModMArch::put( vector<TMess::SRec> &mess )
 {
-    unsigned long long t_cnt = SYS->shrtCnt();
+    long long t_cnt = TSYS::curTime();
 
     ResAlloc res(m_res,false);
 
-    if(!run_st) throw TError(nodePath().c_str(),_("Archive no started!"));
+    if(!run_st) throw TError(nodePath().c_str(),_("Archive is not started!"));
     for( unsigned i_m = 0; i_m < mess.size(); i_m++)
     {
 	if( !chkMessOK(mess[i_m].categ,mess[i_m].level) ) continue;
 
 	int i_arh;
-	for( i_arh = 0; i_arh < arh_s.size(); i_arh++) 
+	for( i_arh = 0; i_arh < arh_s.size(); i_arh++)
 	    if( !arh_s[i_arh]->err() && mess[i_m].time >= arh_s[i_arh]->begin() )
 	    {
-		if( i_arh == 0 && 
+		if( i_arh == 0 &&
 		    ((m_max_size && arh_s[i_arh]->size() > m_max_size*1024) ||
 		     (mess[i_m].time >= arh_s[i_arh]->begin()+m_time_size*24*60*60)) ) break;
 		try{ arh_s[i_arh]->put(mess[i_m]); }
@@ -112,16 +112,18 @@ void ModMArch::put( vector<TMess::SRec> &mess )
 		i_arh = -1;
 		break;
 	    }
-	//- If going a new data then create new file -
+	//> If going a new data then create new file
 	if( i_arh >= 0 )
 	{
 	    res.release();
 
 	    res.request(true);
-	    //-- Create new Archive --
+	    //>> Create new Archive
 	    char c_buf[30];
 	    time_t tm = time(NULL);
-	    strftime(c_buf,sizeof(c_buf),"/%F %T.msg",localtime(&tm));
+	    struct tm tm_tm;
+	    localtime_r(&tm,&tm_tm);
+	    strftime(c_buf,sizeof(c_buf),"/%F %T.msg",&tm_tm);
 	    try{ arh_s.push_front( new MFileArch( addr()+c_buf, mess[i_m].time, this, Mess->charset(), useXML() ) ); }
 	    catch(TError err)
 	    {
@@ -129,18 +131,18 @@ void ModMArch::put( vector<TMess::SRec> &mess )
 		return;
 	    }
 	    res.release();
-	    //-- Allow parallel read access --
+	    //>> Allow parallel read access
 	    res.request(false);
 	    arh_s[0]->put(mess[i_m]);
 	}
     }
-    tm_calc = 1.0e3*((double)(SYS->shrtCnt()-t_cnt))/((double)SYS->sysClk());
+    tm_calc = 1e-3*(TSYS::curTime()-t_cnt);
 }
 
 void ModMArch::get( time_t b_tm, time_t e_tm, vector<TMess::SRec> &mess, const string &category, char level )
 {
     if( e_tm <= b_tm ) return;
-    if(!run_st) throw TError(nodePath().c_str(),_("Archive no started!"));
+    if(!run_st) throw TError(nodePath().c_str(),_("Archive is not started!"));
 
     ResAlloc res(m_res,false);
     for( int i_arh = arh_s.size()-1; i_arh >= 0 && mess.size() < TArchiveS::max_req_mess; i_arh-- )
@@ -274,17 +276,19 @@ void ModMArch::cntrCmdProc( XMLNode *opt )
     if( opt->name() == "info" )
     {
 	TMArchivator::cntrCmdProc(opt);
-	ctrMkNode("fld",opt,-1,"/prm/st/fsz",_("Archive files size (kB)"),0444,"root","Archive",1,"tp","real");
+	ctrMkNode("fld",opt,-1,"/prm/st/fsz",_("Archivator files size (kB)"),0444,"root","Archive",1,"tp","real");
 	ctrMkNode("fld",opt,-1,"/prm/st/tarch",_("Archiving time (msek)"),0444,"root","Archive",1,"tp","real");
-	if(ctrMkNode("area",opt,1,"/bs",_("Additional options"),0444,"root","Archive"))
+	ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),0664,"root","Archive",2,
+	    "tp","str","help",_("Path to directory for archivator's of messages files."));
+	if(ctrMkNode("area",opt,-1,"/prm/add",_("Additional options"),0444,"root","Archive"))
 	{
-	    ctrMkNode("fld",opt,-1,"/bs/xml",cfg("FSArchXML").fld().descr(),0664,"root","Archive",1,"tp","bool");
-	    ctrMkNode("fld",opt,-1,"/bs/sz",cfg("FSArchMSize").fld().descr(),0664,"root","Archive",1,"tp","dec");
-	    ctrMkNode("fld",opt,-1,"/bs/fl",cfg("FSArchNFiles").fld().descr(),0664,"root","Archive",1,"tp","dec");
-	    ctrMkNode("fld",opt,-1,"/bs/len",cfg("FSArchTmSize").fld().descr(),0664,"root","Archive",1,"tp","dec");
-	    ctrMkNode("fld",opt,-1,"/bs/pcktm",cfg("FSArchPackTm").fld().descr(),0664,"root","Archive",1,"tp","dec");
-	    ctrMkNode("fld",opt,-1,"/bs/tm",cfg("FSArchTm").fld().descr(),0664,"root","Archive",1,"tp","dec");
-	    ctrMkNode("comm",opt,-1,"/bs/chk_nw",_("Check archivator directory now"),0660,"root","Archive");
+	    ctrMkNode("fld",opt,-1,"/prm/add/xml",cfg("FSArchXML").fld().descr(),0664,"root","Archive",1,"tp","bool");
+	    ctrMkNode("fld",opt,-1,"/prm/add/sz",cfg("FSArchMSize").fld().descr(),0664,"root","Archive",1,"tp","dec");
+	    ctrMkNode("fld",opt,-1,"/prm/add/fl",cfg("FSArchNFiles").fld().descr(),0664,"root","Archive",1,"tp","dec");
+	    ctrMkNode("fld",opt,-1,"/prm/add/len",cfg("FSArchTmSize").fld().descr(),0664,"root","Archive",1,"tp","dec");
+	    ctrMkNode("fld",opt,-1,"/prm/add/pcktm",cfg("FSArchPackTm").fld().descr(),0664,"root","Archive",1,"tp","dec");
+	    ctrMkNode("fld",opt,-1,"/prm/add/tm",cfg("FSArchTm").fld().descr(),0664,"root","Archive",1,"tp","dec");
+	    ctrMkNode("comm",opt,-1,"/prm/add/chk_nw",_("Check archivator directory now"),0660,"root","Archive");
 	}
 	return;
     }
@@ -293,37 +297,37 @@ void ModMArch::cntrCmdProc( XMLNode *opt )
     string a_path = opt->attr("path");
     if( a_path == "/prm/st/fsz" && ctrChkNode(opt) )		opt->setText(TSYS::real2str((double)size()/1024.,6));
     else if( a_path == "/prm/st/tarch" && ctrChkNode(opt) )	opt->setText(TSYS::real2str(tm_calc,6));
-    else if( a_path == "/bs/xml" )
+    else if( a_path == "/prm/add/xml" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText( useXML() ? "1" : "0" );
 	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	setUseXML( atoi(opt->text().c_str()) );
     }
-    else if( a_path == "/bs/sz" )
+    else if( a_path == "/prm/add/sz" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText(TSYS::int2str( maxSize() ));
 	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	setMaxSize( atoi(opt->text().c_str()) );
     }
-    else if( a_path == "/bs/fl" )
+    else if( a_path == "/prm/add/fl" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText(TSYS::int2str( numbFiles() ));
 	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	setNumbFiles( atoi(opt->text().c_str()) );
     }
-    else if( a_path == "/bs/len" )
+    else if( a_path == "/prm/add/len" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText(TSYS::int2str( timeSize() ));
 	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	setTimeSize( atoi(opt->text().c_str()) );
     }
-    else if( a_path == "/bs/pcktm" )
+    else if( a_path == "/prm/add/pcktm" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText(TSYS::int2str( packTm() ));
 	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	setPackTm( atoi(opt->text().c_str()) );
     }
-    else if( a_path == "/bs/tm" )
+    else if( a_path == "/prm/add/tm" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText(TSYS::int2str( checkTm() ));
 	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	setCheckTm( atoi(opt->text().c_str()) );
     }
-    else if( a_path == "/bs/chk_nw" && ctrChkNode(opt,"set",0660,"root","Archive",SEQ_WR) )	checkArchivator(true);
+    else if( a_path == "/prm/add/chk_nw" && ctrChkNode(opt,"set",0660,"root","Archive",SEQ_WR) )	checkArchivator(true);
     else TMArchivator::cntrCmdProc(opt);
 }
 
@@ -392,10 +396,10 @@ void MFileArch::attach( const string &iname, bool full )
     m_pack = mod->filePack(name());
     m_acces = time(NULL);
 
-    //- Check archive and unpack if want -
+    //> Check archive and unpack if want
     if( m_pack )
     {
-	//-- Get file info from DB --
+	//>> Get file info from DB
 	TConfig c_el(&mod->packFE());
 	c_el.cfg("FILE").setS(name());
 	if(SYS->db().at().dataGet(mod->filesDB(),mod->nodePath()+"Pack/",c_el))
@@ -407,13 +411,9 @@ void MFileArch::attach( const string &iname, bool full )
 
 	    if( !m_xml || (m_xml && !full) )
 	    {
-	        //--- Get file size ---
+	        //>>> Get file size
 	        int hd = open(name().c_str(),O_RDONLY);
-		if( hd > 0 )
-		{
-		    m_size = lseek(hd,0,SEEK_END);
-		    close(hd);
-		}
+		if( hd > 0 )	{ m_size = lseek(hd,0,SEEK_END); close(hd); }
 		return;
 	    }
 	}
@@ -427,20 +427,20 @@ void MFileArch::attach( const string &iname, bool full )
     try
     {
 	char s_char[9];
-	//- Check to plain text archive -
+	//> Check to plain text archive
 	if( fgets(buf,sizeof(buf),f) == NULL )
 	    throw TError(owner().nodePath().c_str(),_("File %s header error!"),name().c_str());
 	string s_tmpl = mod->modId()+"%*s %8s %x %x";
 	if( sscanf(buf,s_tmpl.c_str(),s_char,&m_beg,&m_end) == 3 )
 	{
-	    //-- Attach plain text archive file --
+	    //>> Attach plain text archive file
 	    m_chars = s_char;
 	    m_xml = false;
 	    m_load = true;
 	    fseek(f,0,SEEK_END);
 	    m_size = ftell(f);
 
-	    //-- Delete Node tree --
+	    //>> Delete Node tree
 	    if( m_node ) delete m_node;
 	    m_node = NULL;
 
@@ -465,7 +465,7 @@ void MFileArch::attach( const string &iname, bool full )
 		m_node->load(s_buf);
 		if( m_node->name() != mod->modId() )
 		{
-		    mess_err(owner().nodePath().c_str(),_("No my Archive file: <%s>"),name().c_str());
+                    mess_err(owner().nodePath().c_str(),_("Archive file: <%s> is not mine."),name().c_str());
 		    m_node->clear();
 		    m_err = true;
 		    return;
@@ -541,7 +541,7 @@ void MFileArch::attach( const string &iname, bool full )
 
 void MFileArch::put( TMess::SRec mess )
 {
-    if( m_err ) throw TError(owner().nodePath().c_str(),_("Put message to error Archive file!"));
+    if( m_err ) throw TError(owner().nodePath().c_str(),_("Inserting message to an error Archive file!"));
     if( m_pack ) { m_name = mod->unPackArch(name()); m_pack = false; }
     if( !m_load )
     {
@@ -549,7 +549,7 @@ void MFileArch::put( TMess::SRec mess )
 	if( m_err || !m_load )
 	{
 	    m_err = true;
-	    throw TError(owner().nodePath().c_str(),_("Archive file '%s' isn't attaching!"),m_name.c_str());
+	    throw TError(owner().nodePath().c_str(),_("Archive file '%s' isn't attached!"),m_name.c_str());
 	}
     }
 
@@ -565,10 +565,11 @@ void MFileArch::put( TMess::SRec mess )
 
 	XMLNode *cl_node = m_node->childIns(i_ch,"m");
 	cl_node->setAttr("tm",TSYS::int2str(mess.time,TSYS::Hex));
+	cl_node->setAttr("tmu",TSYS::int2str(mess.utime));
 	cl_node->setAttr("lv",TSYS::int2str(mess.level));
 	cl_node->setAttr("cat",mess.categ);
 	cl_node->setText(mess.mess);
-	if( mess.time > m_end ) 
+	if( mess.time > m_end )
 	{
 	    m_end = mess.time;
 	    m_node->setAttr("End",TSYS::int2str(m_end,TSYS::Hex));
@@ -578,15 +579,15 @@ void MFileArch::put( TMess::SRec mess )
     else
     {
 	char buf[STR_BUF_LEN];
-	//- Check to empty category and message
+	//> Check to empty category and message
 	if( !mess.categ.size() )	mess.categ = " ";
 	if( !mess.mess.size() )		mess.mess = " ";
-	//- Open file -
+	//> Open file
 	FILE *f = fopen(m_name.c_str(),"r+");
 	if( f == NULL ) { m_err = true; return; }
 	if( mess.time >= m_end )
 	{
-	    //- Update header -
+	    //> Update header
 	    if( mess.time != m_end )
 	    {
 		m_end = mess.time;
@@ -594,8 +595,8 @@ void MFileArch::put( TMess::SRec mess )
 		    mod->modId().c_str(),mod->modInfo("Version").c_str(),m_chars.c_str(),m_beg,m_end);
 		fwrite(buf,strlen(buf),1,f);
 	    }
-	    //- Prepare and put mess to end file -
-	    snprintf(buf,sizeof(buf),"%8x %d %s %s\n",mess.time,mess.level,
+	    //> Prepare and put mess to end file
+	    snprintf(buf,sizeof(buf),"%x:%d %d %s %s\n",mess.time,mess.utime,mess.level,
 		Mess->codeConvOut(m_chars,TSYS::strEncode(mess.categ,TSYS::Custom," \n\t%")).c_str(),
 		Mess->codeConvOut(m_chars,TSYS::strEncode(mess.mess,TSYS::Custom," \n\t%")).c_str());
 	    fseek(f,0,SEEK_END);
@@ -604,12 +605,12 @@ void MFileArch::put( TMess::SRec mess )
 	else
 	{
 	    time_t m_tm = 0;
-	    //- Put into file -
-	    //-- Get want position --
+	    //> Put into file
+	    //>> Get want position
 	    long c_off = cacheGet(mess.time);
 	    if(c_off) fseek(f,c_off,SEEK_SET);
 	    else fgets(buf,sizeof(buf),f);
-	    //-- Check mess records --
+	    //>> Check mess records
 	    int pass_cnt = 0;
 	    time_t last_tm = 0;
 	    while(fgets(buf,sizeof(buf),f) != NULL)
@@ -618,12 +619,12 @@ void MFileArch::put( TMess::SRec mess )
 		if( m_tm > mess.time )
 		{
 		    int prev_m_len = strlen(buf);
-		    //--- Prepare message ---
-		    snprintf(buf,sizeof(buf),"%8x %d %s %s\n",mess.time,mess.level,
+		    //>>> Prepare message
+		    snprintf(buf,sizeof(buf),"%x:%d %d %s %s\n",mess.time,mess.utime,mess.level,
 			Mess->codeConvOut(m_chars,TSYS::strEncode(mess.categ,TSYS::Custom," \n\t%")).c_str(),
 			Mess->codeConvOut(m_chars,TSYS::strEncode(mess.mess,TSYS::Custom," \n\t%")).c_str());
 		    string s_buf = buf;
-		    //--- Move tail at a new message size ---
+		    //>>> Move tail at a new message size
 		    int mv_beg = ftell(f)-prev_m_len;
 		    fseek(f,0,SEEK_END);
 		    int mv_end = ftell(f);
@@ -638,17 +639,17 @@ void MFileArch::put( TMess::SRec mess )
 			mv_end-=sizeof(buf);
 		    }
 		    while(beg_cur!=mv_beg);
-		    //--- Write a new message ---
+		    //>>> Write a new message
 		    fseek(f,mv_beg,SEEK_SET);
 		    fwrite(s_buf.c_str(),s_buf.size(),1,f);
 		    cacheUpdate(mess.time,s_buf.size());
-		    //--- Put last value to cache ---
+		    //>>> Put last value to cache
 		    cacheSet(mess.time,mv_beg,true);
 		    break;
 		}
 		else if((pass_cnt++) > CACHE_POS && m_tm != last_tm)
 		{
-		    //--- Add too big position to cache ---
+		    //>>> Add too big position to cache
 		    cacheSet(m_tm,ftell(f)-strlen(buf));
 		    pass_cnt = 0;
 		}
@@ -665,12 +666,12 @@ void MFileArch::get( time_t b_tm, time_t e_tm, vector<TMess::SRec> &mess, const
 {
     TMess::SRec b_rec;
 
-    if( m_err ) throw TError(owner().nodePath().c_str(),_("Get messages from error Archive file!"));
+    if( m_err ) throw TError(owner().nodePath().c_str(),_("Getting messages from an error Archive file!"));
     if( m_pack ){ m_name = mod->unPackArch(name()); m_pack = false; }
     if( !m_load )
     {
 	attach( m_name );
-	if( m_err || !m_load )	throw TError(owner().nodePath().c_str(),_("Archive file isn't attaching!"));
+	if( m_err || !m_load )	throw TError(owner().nodePath().c_str(),_("Archive file isn't attached!"));
     }
 
     ResAlloc res(m_res,false);
@@ -680,23 +681,22 @@ void MFileArch::get( time_t b_tm, time_t e_tm, vector<TMess::SRec> &mess, const
     {
 	for( int i_ch = 0; i_ch < m_node->childSize() && mess.size() < TArchiveS::max_req_mess; i_ch++)
 	{
-	    //- Find messages -
+	    //> Find messages
 	    b_rec.time  = strtol( m_node->childGet(i_ch)->attr("tm").c_str(),(char **)NULL,16);
+	    b_rec.utime  = atoi( m_node->childGet(i_ch)->attr("tmu").c_str() );
 	    b_rec.categ = m_node->childGet(i_ch)->attr("cat");
 	    b_rec.level = (TMess::Type)atoi( m_node->childGet(i_ch)->attr("lv").c_str() );
 	    b_rec.mess  = m_node->childGet(i_ch)->text();
-	    if( b_rec.time >= b_tm && b_rec.time < e_tm && b_rec.level >= level && TMess::chkPattern(b_rec.categ,category) )
+	    if( b_rec.time >= b_tm && b_rec.time <= e_tm && b_rec.level >= level && TMess::chkPattern(b_rec.categ,category) )
 	    {
 		bool equal = false;
 		int i_p = mess.size();
 		for( int i_m = mess.size()-1; i_m >= 0; i_m-- )
 		{
-		    if( mess[i_m].time > b_rec.time )	i_p = i_m;
-		    else if( b_rec.time == mess[i_m].time && 
-			    b_rec.level == mess[i_m].level &&
-			    b_rec.mess == mess[i_m].mess )
+		    if( FTM(mess[i_m]) > FTM(b_rec) ) i_p = i_m;
+		    else if( FTM(mess[i_m]) == FTM(b_rec) && b_rec.level == mess[i_m].level && b_rec.mess == mess[i_m].mess )
 		    { equal = true; break; }
-		    else if( mess[i_m].time < b_rec.time ) break;
+		    else if( FTM(mess[i_m]) < FTM(b_rec) ) break;
 		}
 		if( !equal ) mess.insert(mess.begin()+i_p,b_rec);
 	    }
@@ -705,39 +705,40 @@ void MFileArch::get( time_t b_tm, time_t e_tm, vector<TMess::SRec> &mess, const
     else
     {
 	char buf[STR_BUF_LEN];
-	//- Open file -
+	//> Open file
 	FILE *f = fopen(m_name.c_str(),"r");
 	if( f == NULL ) { m_err = true; return; }
 
-	//- Get want position -
+	//> Get want position
 	long c_off = cacheGet(b_tm);
 	if(c_off) fseek(f,c_off,SEEK_SET);
 	else fgets(buf,sizeof(buf),f);
-	//- Check mess records -
+	//> Check mess records
 	int pass_cnt = 0;
 	time_t last_tm = 0;
 	while( fgets(buf,sizeof(buf),f) != NULL && mess.size() < TArchiveS::max_req_mess )
 	{
-	    sscanf(buf,"%x %d",&b_rec.time,&b_rec.level);
+	    char stm[51]; int off = 0;
+	    sscanf(buf,"%50s %d",stm,&b_rec.level);
+	    b_rec.time = strtol(TSYS::strSepParse(stm,0,':',&off).c_str(),NULL,16);
+	    b_rec.utime = atoi(TSYS::strSepParse(stm,0,':',&off).c_str());
 	    if( b_rec.time >= e_tm ) break;
 	    if( b_rec.time >= b_tm )
 	    {
 		if( b_rec.level < level ) continue;
 		char m_cat[1001], m_mess[1001];
-		sscanf(buf,"%*x %*d %1000s %1000s",m_cat,m_mess);
+		sscanf(buf,"%*x:%*d %*d %1000s %1000s",m_cat,m_mess);
 		b_rec.categ = TSYS::strDecode(Mess->codeConvIn(m_chars,m_cat),TSYS::HttpURL);
 		b_rec.mess  = TSYS::strDecode(Mess->codeConvIn(m_chars,m_mess),TSYS::HttpURL);
 		if( !TMess::chkPattern(b_rec.categ,category) ) continue;
-		//-- Check to equal messages and inserting --
+		//>> Check to equal messages and inserting
 		bool equal = false;
 		int i_p = mess.size();
 		for( int i_m = mess.size()-1; i_m >= 0; i_m-- )
-		    if( mess[i_m].time > b_rec.time )   i_p = i_m;
-		    else if( b_rec.time == mess[i_m].time &&
-			    b_rec.level == mess[i_m].level &&
-			    b_rec.mess == mess[i_m].mess )
-			{ equal = true; break; }
-		    else if( mess[i_m].time < b_rec.time ) break;
+		    if( FTM(mess[i_m]) > FTM(b_rec) )   i_p = i_m;
+		    else if( FTM(mess[i_m]) == FTM(b_rec) && b_rec.level == mess[i_m].level && b_rec.mess == mess[i_m].mess )
+		    { equal = true; break; }
+		    else if( FTM(mess[i_m]) < FTM(b_rec) ) break;
 		if( !equal ) mess.insert(mess.begin()+i_p,b_rec);
 	    }
 	    else if((pass_cnt++) > CACHE_POS && b_rec.time != last_tm)
@@ -761,33 +762,33 @@ void MFileArch::check( bool free )
 	    int hd = open( m_name.c_str(),O_RDWR|O_TRUNC );
 	    if(hd > 0 )
 	    {
-		string x_cf = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" + m_node->save(XMLNode::BrOpenPrev);
+		string x_cf = m_node->save(XMLNode::XMLHeader|XMLNode::BrOpenPrev);
 		m_size = x_cf.size();
 		write(hd,x_cf.c_str(),m_size);
 		close(hd);
 		m_write = false;
 	    }
 	}
-	//- Free memory of XML-archive after 10 minets -
+	//> Free memory of XML-archive after 10 minets
 	if( time(NULL) > m_acces + owner().packTm()*30 || free )
 	{
 	    m_node->clear();
 	    m_load = false;
 	}
     }
-    //- Check for pack archive file -
+    //> Check for pack archive file
     if( !m_err && !m_pack && owner().packTm() && (time(NULL) > m_acces + owner().packTm()*60) && ((xmlM() && !m_load) || !xmlM()) )
     {
 	m_name = mod->packArch(name());
 	m_pack = true;
-	//-- Get file size --
+	//>> Get file size
 	int hd = open(name().c_str(),O_RDONLY);
 	if( hd > 0 )
 	{
 	    m_size = lseek(hd,0,SEEK_END);
 	    close(hd);
 	}
-	//-- Write info to DB --
+	//>> Write info to DB
 	TConfig c_el(&mod->packFE());
 	c_el.cfg("FILE").setS(name());
 	c_el.cfg("BEGIN").setS(TSYS::int2str(begin(),TSYS::Hex));
