@@ -59,16 +59,16 @@ void ModVArch::setValPeriod( double iper )
 
 void ModVArch::start( )
 {
-    //- Start getting data cycle -
+    //> Start getting data cycle
     TVArchivator::start();
 
-    //- First scan dir. Load and connect archive files -
-    checkArchivator(true);
+    //> First scan dir. Load and connect archive files
+    try{ checkArchivator(true); } catch( TError err ) { stop( ); throw; }
 }
 
 void ModVArch::stop( )
 {
-    //- Stop getting data cicle an detach archives -
+    //> Stop getting data cicle an detach archives
     TVArchivator::stop();
 }
 
@@ -187,7 +187,7 @@ void ModVArch::checkArchivator( bool now )
 
     //-- Scan files of attached archives --
     ResAlloc res(a_res,false);
-    for( map<string,TVArchEl*>::iterator iel = archEl.begin(); iel != archEl.end(); iel++ )
+    for( map<string,TVArchEl*>::iterator iel = archEl.begin(); iel != archEl.end(); ++iel )
 	((ModVArchEl *)iel->second)->checkArchivator( now );
 
     chkANow = false;
@@ -258,7 +258,7 @@ void ModVArch::expArch(const string &arch_nm, time_t beg, time_t end, const stri
 	{
 	    long long end_tm = c_tm+buf_sz*buf_per;
 	    end_tm = vmin(end_tm,(long long)end*1000000);
-	    SYS->archive().at().valAt(arch_nm).at().getVal(buf,c_tm,end_tm,workId());
+	    SYS->archive().at().valAt(arch_nm).at().getVals(buf,c_tm,end_tm,workId());
 
 	    //-- Check scale --
 	    for( ; c_tm <= buf.end(); c_tm+=buf_per )
@@ -279,7 +279,7 @@ void ModVArch::expArch(const string &arch_nm, time_t beg, time_t end, const stri
 	{
 	    long long end_tm = c_tm+buf_sz*buf_per;
 	    end_tm = vmin(end_tm,(long long)end*1000000);
-	    SYS->archive().at().valAt(arch_nm).at().getVal(buf,c_tm,end_tm,workId());
+	    SYS->archive().at().valAt(arch_nm).at().getVals(buf,c_tm,end_tm,workId());
 	
 	    for( ; c_tm <= buf.end(); c_tm+=buf_per, val_cnt++ )
 	    {
@@ -308,7 +308,7 @@ void ModVArch::expArch(const string &arch_nm, time_t beg, time_t end, const stri
 	{
 	    long long end_tm = c_tm+buf_sz*buf_per;
 	    end_tm = vmin(end_tm,(long long)end*1000000);
-	    SYS->archive().at().valAt(arch_nm).at().getVal(buf,c_tm,end_tm,workId());
+	    SYS->archive().at().valAt(arch_nm).at().getVals(buf,c_tm,end_tm,workId());
 	    for(; c_tm <= buf.end(); c_tm+=buf_per )
 	    {
 		sprintf(c_val,"%g\n",buf.getR(&c_tm,true));
@@ -332,14 +332,16 @@ void ModVArch::cntrCmdProc( XMLNode *opt )
     if( opt->name() == "info" )
     {
 	TVArchivator::cntrCmdProc(opt);
-	if(ctrMkNode("area",opt,1,"/bs",_("Additional options"),0444,"root","Archive"))
+	ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),0664,"root","Archive",2,
+	    "tp","str","help",_("Path to directory for archivator's of values files."));
+	if(ctrMkNode("area",opt,-1,"/prm/add",_("Additional options"),0444,"root","Archive"))
 	{
-	    ctrMkNode("fld",opt,-1,"/bs/tm",cfg("FSArchTmSize").fld().descr(),0664,"root","Archive",1,"tp","real");
-	    ctrMkNode("fld",opt,-1,"/bs/fn",cfg("FSArchNFiles").fld().descr(),0664,"root","Archive",1,"tp","dec");
-	    ctrMkNode("fld",opt,-1,"/bs/round",cfg("FSArchRound").fld().descr(),0664,"root","Archive",1,"tp","real");
-	    ctrMkNode("fld",opt,-1,"/bs/pcktm",cfg("FSArchPackTm").fld().descr(),0664,"root","Archive",1,"tp","dec");
-	    ctrMkNode("fld",opt,-1,"/bs/tmout",cfg("FSArchTm").fld().descr(),0664,"root","Archive",1,"tp","dec");
-	    ctrMkNode("comm",opt,-1,"/bs/chk_nw",_("Check archivator directory now"),0660,"root","Archive");
+	    ctrMkNode("fld",opt,-1,"/prm/add/tm",cfg("FSArchTmSize").fld().descr(),0664,"root","Archive",1,"tp","real");
+	    ctrMkNode("fld",opt,-1,"/prm/add/fn",cfg("FSArchNFiles").fld().descr(),0664,"root","Archive",1,"tp","dec");
+	    ctrMkNode("fld",opt,-1,"/prm/add/round",cfg("FSArchRound").fld().descr(),0664,"root","Archive",1,"tp","real");
+	    ctrMkNode("fld",opt,-1,"/prm/add/pcktm",cfg("FSArchPackTm").fld().descr(),0664,"root","Archive",1,"tp","dec");
+	    ctrMkNode("fld",opt,-1,"/prm/add/tmout",cfg("FSArchTm").fld().descr(),0664,"root","Archive",1,"tp","dec");
+	    ctrMkNode("comm",opt,-1,"/prm/add/chk_nw",_("Check archivator directory now"),0660,"root","Archive");
 	}
 	ctrMkNode("list",opt,-1,"/arch/arch/3",_("Files size (Mb)"),0444,"root","root",1,"tp","real");
 	if(ctrMkNode("comm",opt,-1,"/arch/exp",_("Export"),0660))
@@ -354,31 +356,32 @@ void ModVArch::cntrCmdProc( XMLNode *opt )
     }
     //- Process command to page -
     string a_path = opt->attr("path");
-    if( a_path == "/bs/tm" )
+    if( a_path == "/prm/add/tm" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText(TSYS::real2str( fileTimeSize(), 6 ));
 	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	setFileTimeSize( atof(opt->text().c_str()) );
     }
-    else if( a_path == "/bs/fn" )
+    else if( a_path == "/prm/add/fn" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText(TSYS::int2str( fileNumber() ));
 	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	setFileNumber( atoi(opt->text().c_str()) );
     }
-    else if( a_path == "/bs/round" )
+    else if( a_path == "/prm/add/round" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText(TSYS::real2str( roundProc(), 6 ));
 	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	setRoundProc( atof(opt->text().c_str()) );
     }
-    else if( a_path == "/bs/pcktm" )
+    else if( a_path == "/prm/add/pcktm" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText(TSYS::int2str( packTm() ));
 	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	setPackTm( atoi(opt->text().c_str()) );
     }
-    else if( a_path == "/bs/tmout" )
+    else if( a_path == "/prm/add/tmout" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) )	opt->setText(TSYS::int2str( checkTm() ));
 	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) )	setCheckTm( atoi(opt->text().c_str()) );
     }
+    else if( a_path == "/prm/add/chk_nw" && ctrChkNode(opt,"set",0660,"root","Archive",SEQ_WR) )	checkArchivator(true);
     else if( a_path == "/arch/arch" && ctrChkNode(opt) )
     {
 	//-- Fill Archives table --
@@ -388,12 +391,12 @@ void ModVArch::cntrCmdProc( XMLNode *opt )
 	XMLNode *f_size = ctrMkNode("list",opt,-1,"/arch/arch/3","");
 
 	ResAlloc res(a_res,false);
-	for( map<string,TVArchEl*>::iterator iel = archEl.begin(); iel != archEl.end(); iel++ )
+	for( map<string,TVArchEl*>::iterator iel = archEl.begin(); iel != archEl.end(); ++iel )
 	{
 	    if(n_arch)	n_arch->childAdd("el")->setText(iel->second->archive().id());
 	    if(n_per)	n_per->childAdd("el")->setText(TSYS::real2str((double)iel->second->archive().period()/1000000.,6));
 	    if(n_size)	n_size->childAdd("el")->setText(TSYS::int2str(iel->second->archive().size()));
-	    if(f_size)	f_size->childAdd("el")->setText(TSYS::real2str((double)((ModVArchEl *)iel->second)->size()/1024.,6));
+	    if(f_size)	f_size->childAdd("el")->setText(TSYS::real2str((double)((ModVArchEl *)iel->second)->size()/1024.,4,'f'));
 	}
     }
     else if( a_path == "/arch/lst" && ctrChkNode(opt) )
@@ -408,7 +411,6 @@ void ModVArch::cntrCmdProc( XMLNode *opt )
 	opt->childAdd("el")->setText("ascii");
 	opt->childAdd("el")->setText("wav");
     }
-    else if( a_path == "/bs/chk_nw" && ctrChkNode(opt,"set",0660,"root","Archive",SEQ_WR) )	checkArchivator(true);
     else if( a_path == "/arch/exp" && ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )
 	expArch(ctrId(opt,"arch")->text(),
 		atoi(ctrId(opt,"beg")->text().c_str()),
@@ -422,7 +424,7 @@ void ModVArch::cntrCmdProc( XMLNode *opt )
 //* FSArch::ModVArchEl - Value archive element    *
 //*************************************************
 ModVArchEl::ModVArchEl( TVArchive &iachive, TVArchivator &iarchivator ) :
-    TVArchEl(iachive,iarchivator)
+    TVArchEl(iachive,iarchivator), realEnd(0)
 {
 
 }
@@ -551,7 +553,10 @@ long long ModVArchEl::end()
     ResAlloc res(m_res,false);
     for( int i_a = arh_f.size()-1; i_a >= 0; i_a-- )
 	if( !arh_f[i_a]->err() )
-	    return arh_f[i_a]->end();
+	{
+	    if( !realEnd ) realEnd = arh_f[i_a]->endData();
+	    return vmin(arh_f[i_a]->end(),realEnd);
+	}
 
     return 0;
 }
@@ -566,11 +571,8 @@ long long ModVArchEl::begin()
     return 0;
 }
 
-void ModVArchEl::getVal( TValBuf &buf, long long ibeg, long long iend )
+void ModVArchEl::getValsProc( TValBuf &buf, long long ibeg, long long iend )
 {
-    iend = vmin( iend, end() );
-    ibeg = vmax( ibeg, begin() );
-
     ResAlloc res(m_res,false);
     for( int i_a = 0; i_a < arh_f.size(); i_a++ )
 	if( ibeg > iend ) break;
@@ -578,69 +580,40 @@ void ModVArchEl::getVal( TValBuf &buf, long long ibeg, long long iend )
 	{
 	    for( ; ibeg < arh_f[i_a]->begin(); ibeg+=(long long)(archivator().valPeriod()*1000000.) )
 		buf.setI(EVAL_INT,ibeg);
-	    arh_f[i_a]->getVal(buf,ibeg,vmin(iend,arh_f[i_a]->end()));
+	    arh_f[i_a]->getVals(buf,ibeg,vmin(iend,arh_f[i_a]->end()));
 	    ibeg = arh_f[i_a]->end()+1;
 	}
     for( ; ibeg <= iend; ibeg+=(long long)(archivator().valPeriod()*1000000.) )
 	buf.setI(EVAL_INT,ibeg);
 }
 
-string ModVArchEl::getS( long long *tm, bool up_ord )
+TVariant ModVArchEl::getValProc( long long *tm, bool up_ord )
 {
-    long long itm = tm?*tm:SYS->curTime();
+    long long itm = tm ? *tm : SYS->curTime();
+    long long per;
     ResAlloc res(m_res,false);
     for( int i_a = 0; i_a < arh_f.size(); i_a++ )
 	if( !arh_f[i_a]->err() && (
 		(up_ord && itm <= arh_f[i_a]->end() && itm > arh_f[i_a]->begin()-arh_f[i_a]->period()) ||
 		(!up_ord && itm < arh_f[i_a]->end()+arh_f[i_a]->period() && itm >= arh_f[i_a]->begin()) ) )
-	    return arh_f[i_a]->getS(up_ord?arh_f[i_a]->maxPos()-(arh_f[i_a]->end()-itm)/arh_f[i_a]->period():(itm-arh_f[i_a]->begin())/arh_f[i_a]->period());
-    return EVAL_STR;
-}
-
-double ModVArchEl::getR( long long *tm, bool up_ord )
-{
-    long long itm = tm?*tm:SYS->curTime();
-    ResAlloc res(m_res,false);
-    for( int i_a = 0; i_a < arh_f.size(); i_a++ )
-	if( !arh_f[i_a]->err() && (
-		(up_ord && itm <= arh_f[i_a]->end() && itm > arh_f[i_a]->begin()-arh_f[i_a]->period()) ||
-		(!up_ord && itm < arh_f[i_a]->end()+arh_f[i_a]->period() && itm >= arh_f[i_a]->begin()) ) )
-	    return arh_f[i_a]->getR(up_ord?arh_f[i_a]->maxPos()-(arh_f[i_a]->end()-itm)/arh_f[i_a]->period():(itm-arh_f[i_a]->begin())/arh_f[i_a]->period());
+	{
+	    if(tm) { per = arh_f[i_a]->period(); *tm = (itm/per)*per+((up_ord&&itm%per)?per:0); }
+	    return arh_f[i_a]->getVal(up_ord?arh_f[i_a]->maxPos()-(arh_f[i_a]->end()-itm)/arh_f[i_a]->period():(itm-arh_f[i_a]->begin())/arh_f[i_a]->period());
+	}
+    if(tm) { per = (long long)(archivator().valPeriod()*1000000.); *tm = (itm>=begin()||itm<=end()) ? (itm/per)*per+((up_ord&&itm%per)?per:0) : 0; }
     return EVAL_REAL;
 }
 
-int ModVArchEl::getI( long long *tm, bool up_ord )
+void ModVArchEl::setValsProc( TValBuf &buf, long long beg, long long end )
 {
-    long long itm = tm?*tm:SYS->curTime();
-    ResAlloc res(m_res,false);
-    for( int i_a = 0; i_a < arh_f.size(); i_a++ )
-	if( !arh_f[i_a]->err() && (
-		(up_ord && itm <= arh_f[i_a]->end() && itm > arh_f[i_a]->begin()-arh_f[i_a]->period()) ||
-		(!up_ord && itm < arh_f[i_a]->end()+arh_f[i_a]->period() && itm >= arh_f[i_a]->begin()) ) )
-	    return arh_f[i_a]->getI(up_ord?arh_f[i_a]->maxPos()-(arh_f[i_a]->end()-itm)/arh_f[i_a]->period():(itm-arh_f[i_a]->begin())/arh_f[i_a]->period());
-    return EVAL_INT;
-}
-
-char ModVArchEl::getB( long long *tm, bool up_ord )
-{
-    long long itm = tm?*tm:SYS->curTime();
-    ResAlloc res(m_res,false);
-    for( int i_a = 0; i_a < arh_f.size(); i_a++ )
-	if( !arh_f[i_a]->err() && (
-		(up_ord && itm <= arh_f[i_a]->end() && itm > arh_f[i_a]->begin()-arh_f[i_a]->period()) ||
-		(!up_ord && itm < arh_f[i_a]->end()+arh_f[i_a]->period() && itm >= arh_f[i_a]->begin()) ) )
-	    return arh_f[i_a]->getB(up_ord?arh_f[i_a]->maxPos()-(arh_f[i_a]->end()-itm)/arh_f[i_a]->period():(itm-arh_f[i_a]->begin())/arh_f[i_a]->period());
-    return EVAL_BOOL;
-}
-
-void ModVArchEl::setVal( TValBuf &buf, long long beg, long long end )
-{
-    //- Check border -
+    //> Check border
     if( !buf.vOK(beg,end) )	return;
     beg = vmax(beg,buf.begin());
     end = vmin(end,buf.end());
 
-    //- Put values to files -
+    realEnd = vmax(realEnd,buf.end());
+
+    //> Put values to files
     ResAlloc res(m_res,false);
     for( int i_a = 0; i_a < arh_f.size(); i_a++ )
 	if( !arh_f[i_a]->err() && beg <= end )
@@ -651,7 +624,9 @@ void ModVArchEl::setVal( TValBuf &buf, long long beg, long long end )
 		//--- Create new file(s) for old data ---
 		char c_buf[30];
 		time_t tm = beg/1000000;
-		strftime(c_buf,sizeof(c_buf)," %F %T.val",localtime(&tm));
+		struct tm tm_tm;
+		localtime_r(&tm,&tm_tm);
+		strftime(c_buf,sizeof(c_buf)," %F %T.val",&tm_tm);
 		string AName = archivator().addr()+"/"+archive().id()+c_buf;
 
 		long long n_end;	//New file end position
@@ -667,7 +642,7 @@ void ModVArchEl::setVal( TValBuf &buf, long long beg, long long end )
 	    if( beg <= arh_f[i_a]->end() && end >= arh_f[i_a]->begin() )
 	    {
 		long long n_end = (end > arh_f[i_a]->end())?arh_f[i_a]->end():end;
-		arh_f[i_a]->setVal(buf,beg,n_end);
+		arh_f[i_a]->setVals(buf,beg,n_end);
 		beg = n_end+(long long)(archivator().valPeriod()*1000000.);
 	    }
 	}
@@ -678,13 +653,15 @@ void ModVArchEl::setVal( TValBuf &buf, long long beg, long long end )
 	//-- Create new file(s) for new data --
 	char c_buf[30];
 	time_t tm = beg/1000000;
-	strftime(c_buf,sizeof(c_buf)," %F %T.val",localtime(&tm));
+	struct tm tm_tm;
+	localtime_r(&tm,&tm_tm);
+	strftime(c_buf,sizeof(c_buf)," %F %T.val",&tm_tm);
 	string AName = archivator().addr()+"/"+archive().id()+c_buf;
 
 	long long n_end = beg+(long long)(((ModVArch &)archivator()).fileTimeSize()*60.*60.*1000000.);
 	arh_f.push_back(new VFileArch(AName,beg,n_end,(long long)(archivator().valPeriod()*1000000.),archive().valType(),this) );
 	n_end = (end > n_end)?n_end:end;
-	arh_f[arh_f.size()-1]->setVal(buf,beg,n_end);
+	arh_f[arh_f.size()-1]->setVals(buf,beg,n_end);
 	beg = n_end+(long long)(archivator().valPeriod()*1000000.);
     }
 }
@@ -887,8 +864,7 @@ void VFileArch::attach( const string &name )
 	if( hd == -1 )	throw TError(owner().archivator().nodePath().c_str(),_("Archive file <%s> no openned!"),name.c_str());
 	m_size = lseek(hd,0,SEEK_END);
 	mpos = (end()-begin())/period();
-	if( !m_pack && cur_tm >= begin() && cur_tm <= end() )
-	    repairFile(hd);
+	if( !m_pack && cur_tm >= begin() && cur_tm <= end() ) repairFile(hd);
 	close(hd);
 	res.release();
 
@@ -898,13 +874,13 @@ void VFileArch::attach( const string &name )
 	    {
 		case TFld::Integer:
 		{
-		    int tval = getI((cur_tm-begin())/period());
+		    int tval = getVal((cur_tm-begin())/period()).getI();
 		    owner().prev_val.assign((char*)&tval,vSize);
 		    break;
 		}
 		case TFld::Real:
 		{
-		    double tval = getR((cur_tm-begin())/period());
+		    double tval = getVal((cur_tm-begin())/period()).getR();
 		    owner().prev_val.assign((char*)&tval,vSize);
 		}
 		break;
@@ -948,7 +924,41 @@ void VFileArch::check( )
     }
 }
 
-void VFileArch::getVal( TValBuf &buf, long long beg, long long end )
+long long VFileArch::endData( )
+{
+    if( getVal(mpos).getS() != EVAL_STR ) return end();
+
+    ResAlloc res(m_res,false);
+    if( m_err ) throw TError(owner().archivator().nodePath().c_str(),_("Archive file error!"));
+
+    if( m_pack )
+    {
+	res.request(true);
+	m_name = mod->unPackArch(m_name);
+	m_pack = false;
+	res.request(false);
+    }
+
+    //> Open archive file
+    int hd = open(name().c_str(),O_RDONLY);
+    if( hd <= 0 ) { m_err = true; return end(); }
+
+    //> Find last value offset
+    int last_off = calcVlOff(hd,mpos);
+    int curPos = mpos;
+    for( int d_win = curPos/2; d_win > 3; d_win/=2 )
+	if( calcVlOff(hd,curPos-d_win) == last_off ) curPos-=d_win;
+    while( curPos > 0 && calcVlOff(hd,curPos) == last_off ) curPos--;
+
+    //> Free file resource and close file
+    close(hd);
+    m_acces = time(NULL);
+    res.release();
+
+    return begin() + curPos*period();
+}
+
+void VFileArch::getVals( TValBuf &buf, long long beg, long long end )
 {
     int vpos_beg, vpos_end, voff_beg, vlen_beg, voff_end, vlen_end;
     char *pid_b, *val_b;
@@ -991,7 +1001,7 @@ void VFileArch::getVal( TValBuf &buf, long long beg, long long end )
 	read(hd,pid_b,i_end-i_beg);
 	//-- Calc end offset --
 	voff_end=voff_beg;
-	for(int i_pos = vpos_beg+1; i_pos <= vpos_end; i_pos++ )
+	for( int i_pos = vpos_beg+1; i_pos <= vpos_end; i_pos++ )
 	    voff_end += vSize*(bool)((0x01<<(i_pos%8))&pid_b[(i_pos/8)-(vpos_beg/8)]);
 	//-- Get value block --
 	i_beg=voff_beg;
@@ -1044,7 +1054,7 @@ void VFileArch::getVal( TValBuf &buf, long long beg, long long end )
 	switch(type())
 	{
 	    case TFld::Boolean:
-		buf.setB((bool)*(val_b+voff_beg),begin()+vpos_beg*period());
+		buf.setB((char)*(val_b+voff_beg),begin()+vpos_beg*period());
 		break;
 	    case TFld::Integer:
 		buf.setI(*(int*)(val_b+voff_beg),begin()+vpos_beg*period());
@@ -1076,7 +1086,7 @@ void VFileArch::getVal( TValBuf &buf, long long beg, long long end )
     free(val_b);
 }
 
-string VFileArch::getS( int vpos )
+TVariant VFileArch::getVal( int vpos )
 {
     ResAlloc res(m_res,false);
     if( m_err ) throw TError(owner().archivator().nodePath().c_str(),_("Archive file error!"));
@@ -1088,125 +1098,44 @@ string VFileArch::getS( int vpos )
 	res.release();
     }
 
+    m_acces = time(NULL);
+    res.request(false);
+    //>Open archive file
+    int hd = open(name().c_str(),O_RDONLY);
+    if( hd <= 0 ) { m_err = true; return EVAL_REAL; }
+
     switch(type())
     {
 	case TFld::Boolean:
-	{ char vl = getB(vpos); return (vl==EVAL_BOOL)?EVAL_STR:TSYS::int2str((bool)vl); }
+	{
+	    char rez = *(char*)getValue(hd,calcVlOff(hd,vpos),sizeof(char)).c_str();
+	    close(hd);
+	    return rez;
+	}
 	case TFld::Integer:
-	{ int vl = getI(vpos); return (vl==EVAL_INT)?EVAL_STR:TSYS::int2str(vl); }
+	{
+	    int rez = *(int*)getValue(hd,calcVlOff(hd,vpos),sizeof(int)).c_str();
+	    close(hd);
+	    return rez;
+	}
 	case TFld::Real:
-	{ double vl = getR(vpos); return (vl==EVAL_REAL)?EVAL_STR:TSYS::real2str(vl); }
+	{
+	    double rez = *(double*)getValue(hd,calcVlOff(hd,vpos),sizeof(double)).c_str();
+	    close(hd);
+	    return rez;
+	}
 	case TFld::String:
-	    m_acces = time(NULL);
-	    res.request(false);
-	    //- Open archive file -
-	    int hd = open(name().c_str(),O_RDONLY);
-	    if( hd <= 0 ) { m_err = true; return EVAL_STR; }
+	{
 	    int v_sz;
 	    int v_off = calcVlOff(hd,vpos,&v_sz);
 	    string rez = getValue(hd,v_off,v_sz);
 	    close(hd);
 	    return rez;
+	}
     }
 }
 
-double VFileArch::getR( int vpos )
-{
-    ResAlloc res(m_res,false);
-    if( m_err ) throw TError(owner().archivator().nodePath().c_str(),_("Archive file error!"));
-    if( m_pack )
-    {
-	res.request(true);
-	m_name = mod->unPackArch(m_name);
-	m_pack = false;
-	res.release();
-    }
-
-    switch(type())
-    {
-	case TFld::Boolean:
-	{ char vl = getB(vpos); return (vl==EVAL_BOOL)?EVAL_REAL:(bool)vl; }
-	case TFld::Integer:
-	{ int vl = getI(vpos); return (vl==EVAL_INT)?EVAL_REAL:(double)vl; }
-	case TFld::String:
-	{ string vl = getS(vpos); return (vl==EVAL_STR)?EVAL_REAL:atof(vl.c_str()); }
-	case TFld::Real:
-	    m_acces = time(NULL);
-	    res.request(false);
-	    //- Open archive file -
-	    int hd = open(name().c_str(),O_RDONLY);
-	    if( hd <= 0 ) { m_err = true; return EVAL_REAL; }
-	    double rez = *(double*)getValue(hd,calcVlOff(hd,vpos),sizeof(double)).c_str();
-	    close(hd);
-	    return rez;
-    }
-}
-
-int VFileArch::getI( int vpos )
-{
-    ResAlloc res(m_res,false);
-    if( m_err ) throw TError(owner().archivator().nodePath().c_str(),_("Archive file error!"));
-    if( m_pack )
-    {
-	res.request(true);
-	m_name = mod->unPackArch(m_name);
-	m_pack = false;
-	res.release();
-    }
-
-    switch(type())
-    {
-	case TFld::Boolean:
-	{ char vl = getB(vpos); return (vl==EVAL_BOOL)?EVAL_INT:(bool)vl; }
-	case TFld::String:
-	{ string vl = getS(vpos); return (vl==EVAL_STR)?EVAL_INT:atoi(vl.c_str()); }
-	case TFld::Real:
-	{ double vl = getR(vpos); return (vl==EVAL_REAL)?EVAL_INT:(int)vl; }
-	case TFld::Integer:
-	    m_acces = time(NULL);
-	    res.request(false);
-	    //- Open archive file -
-	    int hd = open(name().c_str(),O_RDONLY);
-	    if( hd <= 0 ) { m_err = true; return EVAL_INT; }
-	    int rez = *(int*)getValue(hd,calcVlOff(hd,vpos),sizeof(int)).c_str();
-	    close(hd);
-	    return rez;
-    }
-}
-
-char VFileArch::getB( int vpos )
-{
-    ResAlloc res(m_res,false);
-    if( m_err ) throw TError(owner().archivator().nodePath().c_str(),_("Archive file error!"));
-    if( m_pack )
-    {
-	res.request(true);
-	m_name = mod->unPackArch(m_name);
-	m_pack = false;
-	res.release();
-    }
-
-    switch(type())
-    {
-	case TFld::Integer:
-	{ int vl = getI(vpos); return (vl==EVAL_INT)?EVAL_BOOL:(bool)vl; }
-	case TFld::String:
-	{ string vl = getS(vpos); return (vl==EVAL_STR)?EVAL_BOOL:(bool)atoi(vl.c_str()); }
-	case TFld::Real:
-	{ double vl = getR(vpos); return (vl==EVAL_REAL)?EVAL_BOOL:(bool)vl; }
-	case TFld::Boolean:
-	    m_acces = time(NULL);
-	    res.request(false);
-	    //- Open archive file -
-	    int hd = open(name().c_str(),O_RDONLY);
-	    if( hd <= 0 ) { m_err = true; return EVAL_BOOL; }
-	    char rez = *(char*)getValue(hd,calcVlOff(hd,vpos),sizeof(char)).c_str();
-	    close(hd);
-	    return rez;
-    }
-}
-
-void VFileArch::setVal( TValBuf &buf, long long ibeg, long long iend )
+void VFileArch::setVals( TValBuf &buf, long long ibeg, long long iend )
 {
     int vpos_beg, vpos_end, vdif;
     string val_b, value, value_first, value_end;       //Set value
@@ -1464,7 +1393,7 @@ int VFileArch::calcVlOff( int hd, int vpos, int *vsz, bool wr )
 	    b_sz = vmin((vpos/8)-(cach_pos/8)+1,sizeof(buf));
 	    read(hd,buf,b_sz);
 	}
-	for(int i_ps = cach_pos; i_ps <= vpos; i_ps++)
+	for( int i_ps = cach_pos; i_ps <= vpos; i_ps++ )
 	{
 	    int rest = i_ps%8;
 	    if( !rest )
@@ -1491,10 +1420,10 @@ int VFileArch::calcVlOff( int hd, int vpos, int *vsz, bool wr )
 	else voff = sizeof(FHead)+mpos*vSize;
 	lseek(hd,sizeof(FHead)+cach_pos*vSize,SEEK_SET);
 
-	for(int i_ps = cach_pos; i_ps <= vpos; i_ps++)
+	for( int i_ps = cach_pos; i_ps <= vpos; i_ps++ )
 	{
 	    int pk_vl = 0;
-	    for(int i_e = 0; i_e < vSize; i_e++)
+	    for( int i_e = 0; i_e < vSize; i_e++ )
 	    {
 		if( ++i_bf >= b_sz )
 		{
@@ -1611,7 +1540,7 @@ void VFileArch::setPkVal( int hd, int vpos, int vl )
     }
 }
 
-void VFileArch::repairFile(int hd, bool fix)
+void VFileArch::repairFile( int hd, bool fix )
 {
     int v_sz;
     if( !m_pack )
@@ -1686,4 +1615,3 @@ void VFileArch::cacheDrop( int pos )
     if( cach_pr_wr.pos >= pos )
 	cach_pr_wr.off = cach_pr_wr.pos = cach_pr_wr.vsz = 0;
 }
-
