@@ -40,7 +40,7 @@
 #define MOD_NAME	"DB DBF"
 #define MOD_TYPE	"BD"
 #define VER_TYPE	VER_BD
-#define VERSION		"1.9.0"
+#define VERSION		"2.0.1"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"BD modul. Allow support of the *.dbf files, version 3.0."
 #define LICENSE		"GPL"
@@ -122,7 +122,7 @@ void BDMod::load_( )
 	next_opt = getopt_long( SYS->argc, ( char *const * ) SYS->argv, short_opt, long_opt, NULL );
 	switch ( next_opt )
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case -1:  break;
 	}
     }
@@ -160,7 +160,7 @@ void MBD::enable( )
     char *rez = getcwd(buf,sizeof(buf));
     if( chdir(addr().c_str()) != 0 && mkdir(addr().c_str(),S_IRWXU|S_IRGRP|S_IROTH) != 0 )
 	throw TError(TSYS::DBInit,nodePath().c_str(),_("Error create DB directory <%s>!"),addr().c_str());
-    if( rez && chdir(buf) ) throw TError(TSYS::DBInit,nodePath().c_str(),_("Restore previous directory as curent is error."));
+    if( rez && chdir(buf) ) throw TError(TSYS::DBInit,nodePath().c_str(),_("Restore previous directory as current error."));
 
     TBD::enable( );
 }
@@ -189,7 +189,7 @@ void MBD::allowList( vector<string> &list )
 TTable *MBD::openTable( const string &nm, bool create )
 {
     if( !enableStat() )
-	throw TError(TSYS::DBOpenTable,nodePath().c_str(),_("Error open table <%s>. DB disabled."),nm.c_str());
+	throw TError(TSYS::DBOpenTable,nodePath().c_str(),_("Error open table <%s>. DB is disabled."),nm.c_str());
     return new MTable(nm,this,create);
 }
 
@@ -201,7 +201,7 @@ void MBD::cntrCmdProc( XMLNode *opt )
 	TBD::cntrCmdProc(opt);
 	ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),0664,"root","BD",2,
 	    "tp","str","help",
-	    _("For DBF address DB is directory which contain files of tables (*.dbf).\n"
+	    _("For DBF address DB is the directory which contains files of tables (*.dbf).\n"
 	      "For example: /opt/dbf ."));
 	return;
     }
@@ -253,52 +253,39 @@ void MTable::postDisable(int flag)
     }
 }
 
+MBD &MTable::owner( )	{ return (MBD&)TTable::owner(); }
+
 bool MTable::fieldSeek( int i_ln, TConfig &cfg )
 {
     int i_clm;
 
     ResAlloc res(m_res,false);
 
-    i_ln = findKeyLine(cfg,i_ln);
+    i_ln = findKeyLine(cfg,i_ln,true);
     if( i_ln < 0 ) return false;
 
-    //- Get config fields list -
+    //> Get config fields list
     vector<string> cf_el;
     cfg.cfgList(cf_el);
 
-    //- Write data to cfg -
+    //> Write data to cfg
     for( int i_cf = 0; i_cf < cf_el.size(); i_cf++ )
     {
 	TCfg &e_cfg = cfg.cfg(cf_el[i_cf]);
 
-	//-- Find collumn --
+	//>> Find collumn
 	db_str_rec *fld_rec;
 	for(i_clm = 0;(fld_rec = basa->getField(i_clm)) != NULL;i_clm++)
 	    if( cf_el[i_cf].substr(0,10) == fld_rec->name ) break;
 	if(fld_rec == NULL) continue;
 
-	//-- Get table volume --
+	//>> Get table volume
 	string val;
-	if( basa->GetFieldIt( i_ln, i_clm, val ) < 0) 
+	if( basa->GetFieldIt( i_ln, i_clm, val ) < 0)
 	    throw TError(TSYS::DBInernal,nodePath().c_str(),_("Cell error!"));
 
-	//-- Write value --
-	switch(e_cfg.fld().type())
-	{
-	    case TFld::String:
-	    {
-		//--- Remove spaces from end ---
-		int i;
-		for(i = val.size(); i > 0; i--) if(val[i-1]!=' ') break;
-		if(i != (int)val.size()) val.resize(i);
-
-		e_cfg.setS(Mess->codeConvIn(codepage.c_str(),val));
-		break;
-	    }
-	    case TFld::Integer:	e_cfg.setI(atoi(val.c_str()));	break;
-	    case TFld::Real:    e_cfg.setR(atof(val.c_str()));	break;
-	    case TFld::Boolean:	e_cfg.setB((val.c_str()[0] == 'T')?true:false);	break;
-	}
+	//>> Write value
+	setVal(e_cfg,val);
     }
 
     return true;
@@ -308,50 +295,34 @@ void MTable::fieldGet( TConfig &cfg )
 {
     int i_ln, i_clm;
 
-    //- Alloc resource -
+    //> Alloc resource
     ResAlloc res(m_res,false);
 
-    //- Get key line -
+    //> Get key line
     i_ln = findKeyLine( cfg );
-    if( i_ln < 0 ) throw TError(TSYS::DBRowNoPresent,nodePath().c_str(),_("Field no present!"));
+    if( i_ln < 0 ) throw TError(TSYS::DBRowNoPresent,nodePath().c_str(),_("Field is not present!"));
 
-    //- Get config fields list -
+    //> Get config fields list
     vector<string> cf_el;
     cfg.cfgList(cf_el);
 
-    //- Write data to cfg -
+    //> Write data to cfg
     for( int i_cf = 0; i_cf < cf_el.size(); i_cf++ )
     {
 	TCfg &e_cfg = cfg.cfg(cf_el[i_cf]);
 
-	//-- Find collumn --
+	//>> Find collumn
 	db_str_rec *fld_rec;
 	for(i_clm = 0;(fld_rec = basa->getField(i_clm)) != NULL;i_clm++)
 	    if( cf_el[i_cf].substr(0,10) == fld_rec->name ) break;
 	if(fld_rec == NULL) continue;
 
-	//-- Get table volume --
+	//>> Get table volume
 	string val;
-	if( basa->GetFieldIt( i_ln, i_clm, val ) < 0) 
-	    throw TError(TSYS::DBInernal,nodePath().c_str(),_("Cell error!"));	
-
-	//-- Write value --
-	switch(e_cfg.fld().type())
-	{
-	    case TFld::String:
-	    {
-		//--- Remove spaces from end ---
-		int i;
-		for(i = val.size(); i > 0; i--) if(val[i-1]!=' ') break;
-		if(i != (int)val.size()) val.resize(i);
-
-		e_cfg.setS(Mess->codeConvIn(codepage.c_str(),val));
-		break;
-	    }
-	    case TFld::Integer:	e_cfg.setI(atoi(val.c_str()));	break;
-	    case TFld::Real:    e_cfg.setR(atof(val.c_str()));	break;
-	    case TFld::Boolean:	e_cfg.setB((val.c_str()[0] == 'T')?true:false);	break;
-	}
+	if( basa->GetFieldIt( i_ln, i_clm, val ) < 0)
+	    throw TError(TSYS::DBInernal,nodePath().c_str(),_("Cell error!"));
+	//>> Write value
+	setVal(e_cfg,val);
     }
 }
 
@@ -359,25 +330,25 @@ void MTable::fieldSet( TConfig &cfg )
 {
     int i_ln, i_clm;
 
-    //- Alloc resource -
+    //> Alloc resource
     ResAlloc res(m_res,true);
 
-    //- Get config fields list -
+    //> Get config fields list
     vector<string> cf_el;
     cfg.cfgList(cf_el);
 
-    //- Check and fix structure of table -
+    //> Check and fix structure of table
     for( int i_cf = 0; i_cf < cf_el.size(); i_cf++ )
     {
 	TCfg &e_cfg = cfg.cfg(cf_el[i_cf]);
 
-	//-- Find collumn --
+	//>> Find collumn
 	db_str_rec *fld_rec;
 	for(i_clm = 0;(fld_rec = basa->getField(i_clm)) != NULL;i_clm++)
 	    if( cf_el[i_cf].substr(0,10) == fld_rec->name ) break;
 	if(fld_rec == NULL)
 	{
-	    //-- Create new collumn --
+	    //>> Create new collumn
 	    db_str_rec n_rec;
 
 	    fieldPrmSet( e_cfg, n_rec );
@@ -386,7 +357,7 @@ void MTable::fieldSet( TConfig &cfg )
 	}
 	else
 	{
-	    //-- Check collumn parameters --
+	    //>> Check collumn parameters
 	    switch(e_cfg.fld().type())
 	    {
 		case TFld::String:
@@ -407,11 +378,11 @@ void MTable::fieldSet( TConfig &cfg )
 	    db_str_rec n_rec;
 
 	    fieldPrmSet( e_cfg, n_rec );
-	    if( basa->setField(i_clm,&n_rec) < 0 ) 
+	    if( basa->setField(i_clm,&n_rec) < 0 )
 		throw TError(TSYS::DBInernal,nodePath().c_str(),_("Column error!"));
 	}
     }
-    //- Del no used collumn -
+    //> Del no used collumn
     db_str_rec *fld_rec;
     for(i_clm = 0;(fld_rec = basa->getField(i_clm)) != NULL;i_clm++)
     {
@@ -422,38 +393,22 @@ void MTable::fieldSet( TConfig &cfg )
 	    if( basa->DelField(i_clm) < 0 )
 		throw TError(TSYS::DBClose,nodePath().c_str(),_("Delete field error!"));
     }
-    //- Get key line -
+    //> Get key line
     i_ln = findKeyLine( cfg );
     if( i_ln < 0 ) i_ln = basa->CreateItems(-1);
-    //- Write data to bd -
+    //> Write data to bd
     for( int i_cf = 0; i_cf < cf_el.size(); i_cf++ )
     {
 	TCfg &e_cfg = cfg.cfg(cf_el[i_cf]);
-	
-	//-- Find collumn --
+
+	//>> Find collumn
 	db_str_rec *fld_rec;
 	for(i_clm = 0;(fld_rec = basa->getField(i_clm)) != NULL;i_clm++)
 	    if( cf_el[i_cf].substr(0,10) == fld_rec->name ) break;
 	if(fld_rec == NULL) continue;
 
-	//-- Prepare value --
-	string val;
-	switch(e_cfg.fld().type())
-	{
-	    case TFld::String:	val = Mess->codeConvOut(codepage,e_cfg.getS());	break;
-	    case TFld::Integer:	val = SYS->int2str(e_cfg.getI());break;
-	    case TFld::Real:
-	    {
-		char str[200];
-		snprintf(str,sizeof(str),"%*.*f",fld_rec->len_fild,fld_rec->dec_field,e_cfg.getR());
-		val = str;
-		break;
-	    }
-	    case TFld::Boolean:	val = (e_cfg.getB() == true)?"T":"F";	break;
-	}
-	
-	//-- Set table volume --
-	if( basa->ModifiFieldIt( i_ln, i_clm,val.c_str() ) < 0 )
+	//>> Set table volume
+	if( basa->ModifiFieldIt( i_ln, i_clm, getVal(e_cfg,fld_rec).c_str() ) < 0 )
 	    throw TError(TSYS::DBInernal,nodePath().c_str(),_("Cell error!"));
     }
 
@@ -468,7 +423,7 @@ void MTable::fieldDel( TConfig &cfg )
     //- Get key line -
     bool i_ok = false;
     int i_ln;
-    while((i_ln = findKeyLine(cfg)) >= 0)
+    while((i_ln = findKeyLine(cfg,0,true)) >= 0)
     {
 	if( basa->DeleteItems(i_ln,1) < 0 )
 	    throw TError(TSYS::DBInernal,nodePath().c_str(),_("Line error!"));
@@ -476,57 +431,55 @@ void MTable::fieldDel( TConfig &cfg )
 	i_ok = true;
 	m_modify = true;
     }
-    if( !i_ok ) throw TError(TSYS::DBInernal,nodePath().c_str(),_("Field no present!"));
+    if( !i_ok ) throw TError(TSYS::DBInernal,nodePath().c_str(),_("Field is not present!"));
 }
 
-int MTable::findKeyLine( TConfig &cfg, int cnt )
+int MTable::findKeyLine( TConfig &cfg, int cnt, bool useKey )
 {
     int i_ln, i_clm, i_cnt = 0;
 
-    //- Get config fields list -
+    mLstUse = time(NULL);
+
+    //> Get config fields list
     vector<string> cf_el;
     cfg.cfgList(cf_el);
-    //- Left only keys into list -
+    //> Left only keys into list
     for( int i_cf = 0; i_cf < cf_el.size(); )
 	if( cfg.cfg(cf_el[i_cf]).fld().flg()&TCfg::Key ) i_cf++;
 	else cf_el.erase(cf_el.begin()+i_cf);
 
-    //- Find want field -
+    //> Find want field
     for( i_ln = 0; i_ln < basa->GetCountItems(  ); i_ln++ )
     {
 	int cnt_key = 0;
 	for( int i_cf = 0; i_cf < cf_el.size(); i_cf++ )
 	{
-	    if( !cfg.cfg(cf_el[i_cf]).getS().size() )
-	    {
-		cnt_key++;
-		continue;
-	    }
+	    if( useKey && !cfg.cfg(cf_el[i_cf]).keyUse( ) )	{ cnt_key++; continue; }
 	    //string key = cfg.cfg(cf_el[i_cf]).name();
-	    //- Check key -
-	    //-- Find collumn --
+	    //> Check key
+	    //>> Find collumn
 	    db_str_rec *fld_rec;
 	    for(i_clm = 0;(fld_rec = basa->getField(i_clm)) != NULL;i_clm++)
 		if( cf_el[i_cf].substr(0,10) == fld_rec->name ) break;
-	    if(fld_rec == NULL) 
-		throw TError(TSYS::DBInernal,nodePath().c_str(),_("Key column <%s> no present!"),cf_el[i_cf].c_str());
-	    //-- Get table volume --
+	    if(fld_rec == NULL)
+		throw TError(TSYS::DBInernal,nodePath().c_str(),_("Key column <%s> is not present!"),cf_el[i_cf].c_str());
+	    //>> Get table volume
 	    string val;
-	    if( basa->GetFieldIt( i_ln, i_clm, val ) < 0) 
+	    if( basa->GetFieldIt( i_ln, i_clm, val ) < 0)
 		throw TError(TSYS::DBInernal,nodePath().c_str(),_("Cell error!"));
-	    //-- Remove spaces from end --
+	    //>> Remove spaces from end
 	    int i;
 	    for(i = val.size(); i > 0; i--) if(val[i-1]!=' ') break;
 	    if(i != (int)val.size()) val.resize(i);
-	    //-- Compare value --
-	    if( val != cfg.cfg(cf_el[i_cf]).getS() )
+	    //>> Compare value
+	    if( val != getVal(cfg.cfg(cf_el[i_cf])) )
 	    {
 		cnt_key = 0;
 		break;
 	    }
 	    cnt_key++;
 	}
-	if(cnt_key && cnt <= i_cnt++) break;
+	if( cnt_key && cnt <= i_cnt++ ) break;
     }
     if(i_ln >= basa->GetCountItems(  )) return -1;
 
@@ -569,3 +522,40 @@ void MTable::save( )
     basa->SaveFile((char *)n_table.c_str());
     m_modify = false;
 }
+
+string MTable::getVal( TCfg &cfg, db_str_rec *fld_rec )
+{
+    switch(cfg.fld().type())
+    {
+	case TFld::String:	return Mess->codeConvOut(codepage,cfg.getS());
+	case TFld::Integer:	return TSYS::int2str(cfg.getI());
+	case TFld::Real:
+	{
+	    if( !fld_rec ) return TSYS::real2str(cfg.getR());
+
+	    char str[200];
+	    snprintf(str,sizeof(str),"%*.*f",fld_rec->len_fild,fld_rec->dec_field,cfg.getR());
+	    return str;
+	}
+	case TFld::Boolean:	return cfg.getB()?"T":"F";
+    }
+    return "";
+}
+
+void MTable::setVal( TCfg &cfg, const string &val )
+{
+    switch( cfg.fld().type() )
+    {
+	case TFld::String:
+	{
+	    //> Remove spaces from end
+	    int i;
+	    for( i = val.size(); i > 0; i--) if(val[i-1]!=' ' ) break;
+	    cfg.setS(Mess->codeConvIn(codepage.c_str(),val.substr(0,i)));
+	    break;
+	}
+	case TFld::Integer:	cfg.setI(atoi(val.c_str()));	break;
+	case TFld::Real:	cfg.setR(atof(val.c_str()));	break;
+	case TFld::Boolean:	cfg.setB(val.c_str()[0]=='T');	break;
+    }
+}
