@@ -66,15 +66,18 @@ class MTable : public TTable
 	void fieldSet( TConfig &cfg );
 	void fieldDel( TConfig &cfg );
 
-	MBD &owner() { return (MBD&)TTable::owner(); }
+	MBD &owner( );
 
     private:
 	//Private methods
 	void postDisable(int flag);
 	void save( );
-	int  findKeyLine( TConfig &cfg, int cnt = 0 );
+	int  findKeyLine( TConfig &cfg, int cnt = 0, bool useKey = false );
 	void fieldPrmSet( TCfg &e_cfg, db_str_rec &n_rec );
 
+	string getVal( TCfg &cfg, db_str_rec *fld_rec = NULL );
+	void   setVal( TCfg &cfg, const string &vl );
+
 	//Private atributes
 	string n_table;
 	string codepage;
