@@ -33,7 +33,7 @@
 #define MOD_NAME	"DB FireBird"
 #define MOD_TYPE	"BD"
 #define VER_TYPE	VER_BD
-#define VERSION		"0.8.0"
+#define VERSION		"0.9.1"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"DB modul. Allow support of the DB FireBird."
 #define LICENSE		"GPL"
@@ -76,7 +76,7 @@ BDMod::BDMod(string name)
     mod		= this;
 }
 
-BDMod::~BDMod()
+BDMod::~BDMod( )
 {
 
 }
@@ -103,7 +103,7 @@ void BDMod::load_( )
 {
     //- Load parameters from command line -
     int next_opt;
-    char *short_opt="h";
+    const char *short_opt="h";
     struct option long_opt[] =
     {
 	{"help"    ,0,NULL,'h'},
@@ -116,7 +116,7 @@ void BDMod::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -225,7 +225,7 @@ void MBD::allowList( vector<string> &list )
 TTable *MBD::openTable( const string &inm, bool create )
 {
     if( !enableStat() )
-	throw TError(TSYS::DBOpenTable,nodePath().c_str(),_("Error open table <%s>. DB disabled."),inm.c_str());
+	throw TError(TSYS::DBOpenTable,nodePath().c_str(),_("Error open table <%s>. DB is disabled."),inm.c_str());
 
     return new MTable(inm,this,create);
 }
@@ -273,8 +273,6 @@ void MBD::sqlReq( isc_tr_handle *itrans, const string &ireq, vector< vector<stri
     if( tbl ) tbl->clear();
     if(!enableStat()) return;
 
-    //printf("TEST 20: %s\n",ireq.c_str());
-
     ResAlloc res(conn_res,true);
 
     XSQLDA  *out_sqlda = (XSQLDA *)malloc(XSQLDA_LENGTH(10));
@@ -443,7 +441,7 @@ void MBD::cntrCmdProc( XMLNode *opt )
 	TBD::cntrCmdProc(opt);
 	ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),0664,"root","BD",2,
 	    "tp","str","help",
-	    _("FireBird address to DB writed as: [<file>;<user>;<pass>].\n"
+	    _("FireBird address to DB must be written as: [<file>;<user>;<pass>].\n"
 	      "Where:\n"
 	      "  file - full DB file;\n"
 	      "  user - DB user;\n"
@@ -471,7 +469,7 @@ MTable::MTable(string inm, MBD *iown, bool create ) : TTable(inm), trans(0)
     //- Get table structure description -
     getStructDB( tblStrct );
     if( tblStrct.size() <= 1 )
-	throw TError(TSYS::DBOpenTable,nodePath().c_str(),_("Table '%s' no present."),name().c_str());
+	throw TError(TSYS::DBOpenTable,nodePath().c_str(),_("Table '%s' is not present."),name().c_str());
 }
 
 MTable::~MTable(  )
@@ -490,6 +488,8 @@ void MTable::postDisable(int flag)
     }
 }
 
+MBD &MTable::owner()	{ return (MBD&)TTable::owner(); }
+
 void MTable::getStructDB( vector< vector<string> > &tblStrct )
 {
     //- Get generic data structure -
@@ -519,6 +519,7 @@ void MTable::getStructDB( vector< vector<string> > &tblStrct )
 void MTable::fieldStruct( TConfig &cfg )
 {
     if( tblStrct.empty() ) throw TError(TSYS::DBTableEmpty,nodePath().c_str(),_("Table is empty."));
+    mLstUse = time(NULL);
 
     for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
     {
@@ -544,12 +545,13 @@ bool MTable::fieldSeek( int row, TConfig &cfg )
     vector< vector<string> > tbl;
 
     if( tblStrct.empty() ) throw TError(TSYS::DBTableEmpty,nodePath().c_str(),_("Table is empty."));
+    mLstUse = time(NULL);
 
     owner().transCommit(&trans);
-    //- Make WHERE -
+    //> Make WHERE
     string req = "SELECT FIRST 1 SKIP "+TSYS::int2str(row)+" ";
     string req_where = "WHERE ";
-    //-- Add use keys to list --
+    //>> Add use keys to list
     bool first_sel = true;
     bool next = false;
     for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
@@ -557,11 +559,11 @@ bool MTable::fieldSeek( int row, TConfig &cfg )
 	string sid = tblStrct[i_fld][0];
 	if( !cfg.cfgPresent(sid) ) continue;
 	TCfg &u_cfg = cfg.cfg(sid);
-	if( u_cfg.fld().flg()&TCfg::Key && u_cfg.getS().size() )
+	if( u_cfg.fld().flg()&TCfg::Key && u_cfg.keyUse() )
 	{
-	    if( !next ) next = true; 
+	    if( !next ) next = true;
 	    else req_where=req_where+"AND ";
-	    req_where=req_where+"\""+mod->sqlReqCode(sid,'"')+"\"='"+mod->sqlReqCode(u_cfg.getS())+"' ";
+	    req_where=req_where+"\""+mod->sqlReqCode(sid,'"')+"\"='"+mod->sqlReqCode(getVal(u_cfg))+"' ";
 	}
 	else if( u_cfg.fld().flg()&TCfg::Key || u_cfg.view() )
 	{
@@ -571,24 +573,14 @@ bool MTable::fieldSeek( int row, TConfig &cfg )
 	}
     }
 
-    //- Request -
+    //> Request
     req = req+" FROM \""+mod->sqlReqCode(name(),'"')+"\" "+((next)?req_where:"");
     owner().sqlReq( req, &tbl );
     if( tbl.size() < 2 ) return false;
-    //- Processing of query -
+    //> Processing of query
     for( int i_fld = 0; i_fld < tbl[0].size(); i_fld++ )
-    {
-	if( !cfg.cfgPresent(tbl[0][i_fld]) ) continue;
-	string val = tbl[1][i_fld];
-	TCfg &u_cfg = cfg.cfg(tbl[0][i_fld]);
-	switch(u_cfg.fld().type())
-	{
-	    case TFld::String:	u_cfg.setS(val); break;
-	    case TFld::Integer:	u_cfg.setI(atoi(val.c_str()));	break;
-	    case TFld::Real:	u_cfg.setR(atof(val.c_str()));	break;
-	    case TFld::Boolean:	u_cfg.setB(atoi(val.c_str()));	break;
-	}
-    }
+	if( cfg.cfgPresent(tbl[0][i_fld]) )
+	    setVal(cfg.cfg(tbl[0][i_fld]),tbl[1][i_fld]);
 
     return true;
 }
@@ -598,12 +590,13 @@ void MTable::fieldGet( TConfig &cfg )
     vector< vector<string> > tbl;
 
     if( tblStrct.empty() ) throw TError(TSYS::DBTableEmpty,nodePath().c_str(),_("Table is empty."));
+    mLstUse = time(NULL);
 
     owner().transCommit(&trans);
-    //- Prepare request -
+    //> Prepare request
     string req = "SELECT ";
     string req_where;
-    //-- Add fields list to queue --
+    //>> Add fields list to queue
     bool first_sel = true;
     bool next_wr = false;
     for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
@@ -613,8 +606,7 @@ void MTable::fieldGet( TConfig &cfg )
 	if( u_cfg.fld().flg()&TCfg::Key )
 	{
 	    if( !next_wr ) next_wr = true; else req_where=req_where+"AND ";
-	    req_where=req_where+"\""+mod->sqlReqCode(tblStrct[i_fld][0],'"')+"\"='"+
-				     mod->sqlReqCode(u_cfg.getS())+"'";
+	    req_where=req_where+"\""+mod->sqlReqCode(tblStrct[i_fld][0],'"')+"\"='"+mod->sqlReqCode(getVal(u_cfg))+"'";
 	}
 	else if( u_cfg.view() )
 	{
@@ -625,52 +617,42 @@ void MTable::fieldGet( TConfig &cfg )
     }
     req = req+" FROM \""+mod->sqlReqCode(name(),'"')+"\" WHERE "+req_where;
 
-    //- Query -
+    //> Query
     owner().sqlReq( req, &tbl );
-    if( tbl.size() < 2 ) throw TError(TSYS::DBRowNoPresent,nodePath().c_str(),_("Row no present."));
+    if( tbl.size() < 2 ) throw TError(TSYS::DBRowNoPresent,nodePath().c_str(),_("Row is not present."));
 
-    //- Processing of query -
+    //> Processing of query
     for( int i_fld = 0; i_fld < tbl[0].size(); i_fld++ )
-    {
-	if( !cfg.cfgPresent(tbl[0][i_fld]) ) continue;
-	string val = tbl[1][i_fld];
-	TCfg &u_cfg = cfg.cfg(tbl[0][i_fld]);
-	switch(u_cfg.fld().type())
-	{
-	    case TFld::String:	u_cfg.setS(val); break;
-	    case TFld::Integer:	u_cfg.setI(atoi(val.c_str()));	break;
-	    case TFld::Real:	u_cfg.setR(atof(val.c_str()));	break;
-	    case TFld::Boolean:	u_cfg.setB(atoi(val.c_str()));	break;
-	}
-    }
+	if( cfg.cfgPresent(tbl[0][i_fld]) )
+	    setVal(cfg.cfg(tbl[0][i_fld]),tbl[1][i_fld]);
 }
 
 void MTable::fieldSet( TConfig &cfg )
 {
     vector< vector<string> > tbl;
 
+    mLstUse = time(NULL);
     if( tblStrct.empty() ) fieldFix(cfg);
 
     owner().transOpen(&trans);
-    //- Get config fields list -
+    //> Get config fields list
     vector<string> cf_el;
     cfg.cfgList(cf_el);
 
-    //- Get present fields list -
+    //> Get present fields list
     string req_where = "WHERE ";
-    //-- Add key list to queue --
+    //>> Add key list to queue
     bool next = false;
     for( int i_el = 0; i_el < cf_el.size(); i_el++ )
     {
 	TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
 	if( !(u_cfg.fld().flg()&TCfg::Key) ) continue;
 	if( !next ) next = true; else req_where=req_where+"AND ";
-	req_where=req_where+"\""+mod->sqlReqCode(cf_el[i_el],'"')+"\"='"+
-				 mod->sqlReqCode(u_cfg.getS())+"' ";
+	req_where=req_where+"\""+mod->sqlReqCode(cf_el[i_el],'"')+"\"='"+mod->sqlReqCode(getVal(u_cfg))+"' ";
     }
 
-    //- Prepare query -
-    //-- Add new record --
+    //> Prepare query
+    //>> Add new record
     string reqi = "INSERT INTO \""+mod->sqlReqCode(name(),'"')+"\" ";
     string ins_name, ins_value;
     next = false;
@@ -686,21 +668,13 @@ void MTable::fieldSet( TConfig &cfg )
 	    ins_value=ins_value+",";
 	}
 	ins_name=ins_name+"\""+mod->sqlReqCode(cf_el[i_el],'"')+"\" ";
-	string val;
-	switch(u_cfg.fld().type())
-	{
-	    case TFld::String:	val = u_cfg.getS();			break;
-	    case TFld::Integer:	val = SYS->int2str(u_cfg.getI());	break;
-	    case TFld::Real:	val = SYS->real2str(u_cfg.getR());	break;
-	    case TFld::Boolean:	val = SYS->int2str(u_cfg.getB());	break;
-	}
-	ins_value=ins_value+"'"+mod->sqlReqCode(val)+"' ";
+	ins_value=ins_value+"'"+mod->sqlReqCode(getVal(u_cfg))+"' ";
     }
     reqi = reqi + "("+ins_name+") VALUES ("+ins_value+")";
     try{ owner().sqlReq( &trans, reqi ); }
     catch(TError err)
     {
-	//-- Update present record --
+	//>> Update present record
 	string requ = "UPDATE \""+mod->sqlReqCode(name(),'"')+"\" SET ";
 	next = false;
 	for( int i_el = 0; i_el < cf_el.size(); i_el++ )
@@ -709,15 +683,7 @@ void MTable::fieldSet( TConfig &cfg )
 	    if( u_cfg.fld().flg()&TCfg::Key || !u_cfg.view() ) continue;
 
 	    if( !next ) next = true; else requ=requ+",";
-	    string val;
-	    switch(u_cfg.fld().type())
-	    {
-		case TFld::String:	val = u_cfg.getS();	break;
-		case TFld::Integer:	val = SYS->int2str(u_cfg.getI());	break;
-		case TFld::Real:	val = SYS->real2str(u_cfg.getR());	break;
-		case TFld::Boolean:	val = SYS->int2str(u_cfg.getB());	break;
-	    }
-	    requ=requ+"\""+mod->sqlReqCode(cf_el[i_el],'"')+"\"='"+mod->sqlReqCode(val)+"' ";
+	    requ=requ+"\""+mod->sqlReqCode(cf_el[i_el],'"')+"\"='"+mod->sqlReqCode(getVal(u_cfg))+"' ";
 	}
 	requ = requ + req_where;
 	try{ owner().sqlReq( &trans, requ ); }
@@ -733,24 +699,24 @@ void MTable::fieldSet( TConfig &cfg )
 void MTable::fieldDel( TConfig &cfg )
 {
     if( tblStrct.empty() ) throw TError(TSYS::DBTableEmpty,nodePath().c_str(),_("Table is empty."));
+    mLstUse = time(NULL);
 
     owner().transOpen(&trans);
-    //- Get config fields list -
+    //> Get config fields list
     vector<string> cf_el;
     cfg.cfgList(cf_el);
 
-    //- Prepare request -
+    //> Prepare request
     string req = "DELETE FROM \""+mod->sqlReqCode(name(),'"')+"\" WHERE ";
-    //-- Add key list to queue --
+    //>> Add key list to queue
     bool next = false;
     for( int i_el = 0; i_el < cf_el.size(); i_el++ )
     {
 	TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
-	if( u_cfg.fld().flg()&TCfg::Key && u_cfg.getS().size() )
+	if( u_cfg.fld().flg()&TCfg::Key && u_cfg.keyUse() )
 	{
 	    if( !next ) next = true; else req=req+"AND ";
-	    req=req+"\""+mod->sqlReqCode(cf_el[i_el],'"')+"\"='"+
-			 mod->sqlReqCode(u_cfg.getS())+"' ";
+	    req=req+"\""+mod->sqlReqCode(cf_el[i_el],'"')+"\"='"+mod->sqlReqCode(getVal(u_cfg))+"' ";
 	}
     }
     owner().sqlReq( &trans, req );
@@ -841,3 +807,26 @@ void MTable::fieldFix( TConfig &cfg )
 
     owner().transOpen(&trans);
 }
+
+string MTable::getVal( TCfg &cfg )
+{
+    switch( cfg.fld().type() )
+    {
+	case TFld::String:	return cfg.getS();
+	case TFld::Integer:	return SYS->int2str(cfg.getI());
+	case TFld::Real:	return SYS->real2str(cfg.getR());
+	case TFld::Boolean:	return SYS->int2str(cfg.getB());
+    }
+    return "";
+}
+
+void MTable::setVal( TCfg &cfg, const string &val )
+{
+    switch( cfg.fld().type() )
+    {
+	case TFld::String:	cfg.setS(val);	break;
+	case TFld::Integer:	cfg.setI(atoi(val.c_str()));	break;
+	case TFld::Real:	cfg.setR(atof(val.c_str()));	break;
+	case TFld::Boolean:	cfg.setB(atoi(val.c_str()));	break;
+    }
+}
