@@ -35,7 +35,7 @@
 #define MOD_NAME	"DB MySQL"
 #define MOD_TYPE	"BD"
 #define VER_TYPE	VER_BD
-#define VERSION		"1.4.0"
+#define VERSION		"1.6.0"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"BD modul. Allow support of the BD MySQL."
 #define MOD_LICENSE	"GPL"
@@ -117,7 +117,7 @@ void BDMod::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -204,7 +204,7 @@ void MBD::allowList( vector<string> &list )
 TTable *MBD::openTable( const string &inm, bool create )
 {
     if( !enableStat() )
-	throw TError(TSYS::DBOpen,nodePath().c_str(),_("Error open table <%s>. DB disabled."),inm.c_str());
+	throw TError(TSYS::DBOpen,nodePath().c_str(),_("Error open table <%s>. DB is disabled."),inm.c_str());
 
     return new MTable(inm,this,create);
 }
@@ -272,7 +272,7 @@ void MBD::cntrCmdProc( XMLNode *opt )
 	TBD::cntrCmdProc(opt);
 	ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),0664,"root","BD",2,
 	    "tp","str","help",
-	    _("MySQL DB address writed in form: [<host>;<user>;<pass>;<db>;<port>;<u_sock>].\n"
+	    _("MySQL DB address must be written as: [<host>;<user>;<pass>;<db>;<port>;<u_sock>].\n"
 	      "Where:\n"
 	      "  host - MySQL server hostname;\n"
 	      "  user - DB user name;\n"
@@ -334,9 +334,12 @@ void MTable::postDisable(int flag)
     }
 }
 
+MBD &MTable::owner()	{ return (MBD&)TTable::owner(); }
+
 void MTable::fieldStruct( TConfig &cfg )
 {
     if( tblStrct.empty() ) throw TError(TSYS::DBTableEmpty,nodePath().c_str(),_("Table is empty!"));
+    mLstUse = time(NULL);
 
     for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
     {
@@ -358,6 +361,8 @@ void MTable::fieldStruct( TConfig &cfg )
 	    cfg.elem().fldAdd( new TFld(sid.c_str(),sid.c_str(),TFld::Real,flg,(TSYS::int2str(pr1)+"."+TSYS::int2str(pr2)).c_str()) );
 	else if( tblStrct[i_fld][1] == "tinyint(1)" )
 	    cfg.elem().fldAdd( new TFld(sid.c_str(),sid.c_str(),TFld::Boolean,flg,"1") );
+	else if( tblStrct[i_fld][1] == "datetime" )
+	    cfg.elem().fldAdd( new TFld(sid.c_str(),sid.c_str(),TFld::Integer,flg|TFld::DateTimeDec,"10") );
     }
 }
 
@@ -366,52 +371,58 @@ bool MTable::fieldSeek( int row, TConfig &cfg )
     vector< vector<string> > tbl;
 
     if( tblStrct.empty() ) throw TError(TSYS::DBTableEmpty,nodePath().c_str(),_("Table is empty!"));
+    mLstUse = time(NULL);
 
-    //- Make SELECT and WHERE -
+    string sid;
+    string lang2Code = cfg.lang2Code().empty() ? Mess->lang2Code() : cfg.lang2Code();
+    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && lang2Code != Mess->lang2CodeBase());
+    //> Make SELECT and WHERE
     string req = "SELECT ";
     string req_where = "WHERE ";
-    //- Add use keys to list -
+    //> Add use keys to list
     bool first_sel = true;
     bool next = false;
     for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
     {
-	string sid = tblStrct[i_fld][0];
-	if( !cfg.cfgPresent(sid) ) continue;
-	TCfg &u_cfg = cfg.cfg(sid);
-	if( u_cfg.fld().flg()&TCfg::Key && u_cfg.getS().size() )
+	sid = tblStrct[i_fld][0];
+	TCfg *u_cfg = cfg.at(sid,true);
+	if( !u_cfg && isVarTextTransl && sid.size() > 3 && sid.substr(0,3) == (lang2Code+"#") )
 	{
-	    if( !next ) next = true;
-	    else req_where=req_where+"AND ";
-	    req_where=req_where+"`"+TSYS::strEncode(sid,TSYS::SQL)+"`='"+
-			TSYS::strEncode(u_cfg.getS(),TSYS::SQL)+"' ";
+	    u_cfg = cfg.at(sid.substr(3),true);
+	    if( u_cfg && !(u_cfg->fld().flg()&TCfg::TransltText && !u_cfg->noTransl()) ) continue;
 	}
-	else if( u_cfg.fld().flg()&TCfg::Key || u_cfg.view() )
+	if( !u_cfg ) continue;
+
+	if( u_cfg->fld().flg()&TCfg::Key && u_cfg->keyUse() )
 	{
-	    if( first_sel ) req=req+"`"+TSYS::strEncode(sid,TSYS::SQL)+"`";
-	    else req=req+",`"+TSYS::strEncode(sid,TSYS::SQL)+"`";
+	    req_where = req_where + (next?"AND `":"`") + TSYS::strEncode(sid,TSYS::SQL) + "`='" + TSYS::strEncode(getVal(*u_cfg),TSYS::SQL) + "' ";
+	    next = true;
+	}
+	else if( u_cfg->fld().flg()&TCfg::Key || u_cfg->view() )
+	{
+	    req = req + (first_sel?"`":",`") + TSYS::strEncode(sid,TSYS::SQL) + "`";
 	    first_sel = false;
 	}
     }
 
-    //- Request -
+    //> Request
     if( first_sel ) return false;
-    req = req + " FROM `"+TSYS::strEncode(owner().bd,TSYS::SQL)+"`.`"+TSYS::strEncode(name(),TSYS::SQL)+"` "+
-		 ((next)?req_where:"") +" LIMIT "+TSYS::int2str(row)+",1";
+    req = req + " FROM `" + TSYS::strEncode(owner().bd,TSYS::SQL) + "`.`" + TSYS::strEncode(name(),TSYS::SQL) + "` " +
+	((next)?req_where:"") + " LIMIT " + TSYS::int2str(row) + ",1";
     owner().sqlReq( req, &tbl );
     if( tbl.size() < 2 ) return false;
     for( int i_fld = 0; i_fld < tbl[0].size(); i_fld++ )
     {
-	if( !cfg.cfgPresent(tbl[0][i_fld]) ) continue;
-	string val = tbl[1][i_fld];
-	TCfg &u_cfg = cfg.cfg(tbl[0][i_fld]);
-	switch(u_cfg.fld().type())
+	sid = tbl[0][i_fld];
+	TCfg *u_cfg = cfg.at(sid,true);
+	if( u_cfg ) setVal(*u_cfg,tbl[1][i_fld]);
+	else if( isVarTextTransl && sid.size() > 3 && sid.substr(0,3) == (lang2Code+"#") && tbl[1][i_fld].size() )
 	{
-	    case TFld::String:	u_cfg.setS(val);		break;
-	    case TFld::Integer:	u_cfg.setI(atoi(val.c_str()));	break;
-	    case TFld::Real:	u_cfg.setR(atof(val.c_str()));	break;
-	    case TFld::Boolean:	u_cfg.setB(atoi(val.c_str()));	break;
+	    u_cfg = cfg.at(sid.substr(3),true);
+	    if( u_cfg ) setVal(*u_cfg,tbl[1][i_fld]);
 	}
     }
+
     return true;
 }
 
@@ -420,50 +431,56 @@ void MTable::fieldGet( TConfig &cfg )
     vector< vector<string> > tbl;
 
     if( tblStrct.empty() ) throw TError(TSYS::DBTableEmpty,nodePath().c_str(),_("Table is empty!"));
+    mLstUse = time(NULL);
 
-    //- Prepare request -
+    string sid;
+    string lang2Code = cfg.lang2Code().empty() ? Mess->lang2Code() : cfg.lang2Code();
+    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && lang2Code != Mess->lang2CodeBase());
+    //> Prepare request
     string req = "SELECT ";
     string req_where;
-    //-- Add fields list to queue --
+    //>> Add fields list to queue
     bool first_sel = true;
     bool next_wr = false;
     for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
     {
-	if( !cfg.cfgPresent(tblStrct[i_fld][0]) ) continue;
-	TCfg &u_cfg = cfg.cfg(tblStrct[i_fld][0]);
-	if( u_cfg.fld().flg()&TCfg::Key )
+	sid = tblStrct[i_fld][0];
+	TCfg *u_cfg = cfg.at(sid,true);
+	if( !u_cfg && isVarTextTransl && sid.size() > 3 && sid.substr(0,3) == (lang2Code+"#") )
 	{
-	    if( !next_wr ) next_wr = true; else req_where=req_where+"AND ";
-	    req_where=req_where+"`"+TSYS::strEncode(tblStrct[i_fld][0],TSYS::SQL)+"`='"+
-		    TSYS::strEncode(u_cfg.getS(),TSYS::SQL)+"' ";
+	    u_cfg = cfg.at(sid.substr(3),true);
+	    if( u_cfg && !(u_cfg->fld().flg()&TCfg::TransltText && !u_cfg->noTransl()) ) continue;
 	}
-	else if( u_cfg.view() )
+	if( !u_cfg ) continue;
+
+	if( u_cfg->fld().flg()&TCfg::Key )
 	{
-	    if( first_sel ) req=req+"`"+TSYS::strEncode(tblStrct[i_fld][0],TSYS::SQL)+"`";
-	    else req=req+",`"+TSYS::strEncode(tblStrct[i_fld][0],TSYS::SQL)+"`";
+	    req_where = req_where + (next_wr?"AND `":"`") + TSYS::strEncode(sid,TSYS::SQL) + "`='"  +TSYS::strEncode(getVal(*u_cfg),TSYS::SQL) + "' ";
+	    next_wr = true;
+	}
+	else if( u_cfg->view() )
+	{
+	    req = req + (first_sel?"`":",`") + TSYS::strEncode(sid,TSYS::SQL) + "`";
 	    first_sel = false;
 	}
     }
-    req = req+" FROM `"+TSYS::strEncode(owner().bd,TSYS::SQL)+"`.`"+
-	TSYS::strEncode(name(),TSYS::SQL)+"` WHERE "+req_where;
+    req = req + " FROM `" + TSYS::strEncode(owner().bd,TSYS::SQL) + "`.`" + TSYS::strEncode(name(),TSYS::SQL) + "` WHERE " + req_where;
 
-    //- Query -
+    //> Query
     //printf("TEST 01: query: <%s>\n",req.c_str());
     owner().sqlReq( req, &tbl );
-    if( tbl.size() < 2 ) throw TError(TSYS::DBRowNoPresent,nodePath().c_str(),_("Row no present!"));
+    if( tbl.size() < 2 ) throw TError(TSYS::DBRowNoPresent,nodePath().c_str(),_("Row is not present!"));
 
-    //- Processing of query -
+    //> Processing of query
     for( int i_fld = 0; i_fld < tbl[0].size(); i_fld++ )
     {
-	if( !cfg.cfgPresent(tbl[0][i_fld]) ) continue;
-	string val = tbl[1][i_fld];
-	TCfg &u_cfg = cfg.cfg(tbl[0][i_fld]);
-	switch(u_cfg.fld().type())
+	sid = tbl[0][i_fld];
+	TCfg *u_cfg = cfg.at(sid,true);
+	if( u_cfg ) setVal(*u_cfg,tbl[1][i_fld]);
+	else if( isVarTextTransl && sid.size() > 3 && sid.substr(0,3) == (lang2Code+"#") && tbl[1][i_fld].size() )
 	{
-	    case TFld::String:	u_cfg.setS(val);		break;
-	    case TFld::Integer:	u_cfg.setI(atoi(val.c_str()));	break;
-	    case TFld::Real:	u_cfg.setR(atof(val.c_str()));	break;
-	    case TFld::Boolean:	u_cfg.setB(atoi(val.c_str()));	break;
+	    u_cfg = cfg.at(sid.substr(3),true);
+	    if( u_cfg ) setVal(*u_cfg,tbl[1][i_fld]);
 	}
     }
 }
@@ -473,35 +490,36 @@ void MTable::fieldSet( TConfig &cfg )
     vector< vector<string> > tbl;
 
     if( tblStrct.empty() ) throw TError(TSYS::DBTableEmpty,nodePath().c_str(),_("Table is empty!"));
+    mLstUse = time(NULL);
 
-    //- Get config fields list -
+    string sid, sval;
+    string lang2Code = cfg.lang2Code().empty() ? Mess->lang2Code() : cfg.lang2Code();
+    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && lang2Code != Mess->lang2CodeBase());
+    //> Get config fields list
     vector<string> cf_el;
     cfg.cfgList(cf_el);
 
-    //- Get present fields list -
+    //> Get present fields list
     string req_where = "WHERE ";
-    //-- Add key list to query --
+    //>> Add key list to query
     bool next = false;
     for( int i_el = 0; i_el < cf_el.size(); i_el++ )
     {
 	TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
 	if( !(u_cfg.fld().flg()&TCfg::Key) ) continue;
-	if( !next ) next = true; else req_where=req_where+"AND ";
-	req_where=req_where+"`"+TSYS::strEncode(cf_el[i_el],TSYS::SQL)+"`='"+
-				TSYS::strEncode(u_cfg.getS(),TSYS::SQL)+"' ";
+	req_where = req_where + (next?"AND `":"`") + TSYS::strEncode(cf_el[i_el],TSYS::SQL) + "`='" + TSYS::strEncode(getVal(u_cfg),TSYS::SQL) + "' ";
+	next = true;
     }
 
-    //- Prepare query -
-    //-- Try for get already present field --
-    string req = "SELECT 1 FROM `"+TSYS::strEncode(owner().bd,TSYS::SQL)+"`.`"+
-	TSYS::strEncode(name(),TSYS::SQL)+"` "+req_where;
+    //> Prepare query
+    //>> Try for get already present field
+    string req = "SELECT 1 FROM `" + TSYS::strEncode(owner().bd,TSYS::SQL) + "`.`" + TSYS::strEncode(name(),TSYS::SQL) + "` " + req_where;
     try{ owner().sqlReq( req, &tbl ); }
     catch(TError err)	{ fieldFix(cfg); owner().sqlReq( req ); }
     if( tbl.size() < 2 )
     {
-	//-- Add new record --
-	req = "INSERT INTO `"+TSYS::strEncode(owner().bd,TSYS::SQL)+"`.`"+
-	    TSYS::strEncode(name(),TSYS::SQL)+"` ";
+	//>> Add new record
+	req = "INSERT INTO `" + TSYS::strEncode(owner().bd,TSYS::SQL) + "`.`" + TSYS::strEncode(name(),TSYS::SQL) + "` ";
 	string ins_name, ins_value;
 	next = false;
 	for( int i_el = 0; i_el < cf_el.size(); i_el++ )
@@ -509,52 +527,35 @@ void MTable::fieldSet( TConfig &cfg )
 	    TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
 	    if( !(u_cfg.fld().flg()&TCfg::Key) && !u_cfg.view() ) continue;
 
-	    if( !next ) next = true;
-	    else
-	    {
-		ins_name=ins_name+",";
-		ins_value=ins_value+",";
-	    }
-	    ins_name=ins_name+"`"+TSYS::strEncode(cf_el[i_el],TSYS::SQL)+"` ";
-	    string val;
-	    switch(u_cfg.fld().type())
-	    {
-		case TFld::String:	val = u_cfg.getS();	break;
-		case TFld::Integer:	val = SYS->int2str(u_cfg.getI());	break;
-		case TFld::Real:	val = SYS->real2str(u_cfg.getR());	break;
-		case TFld::Boolean:	val = SYS->int2str(u_cfg.getB());	break;
-	    }
-	    ins_value=ins_value+"'"+TSYS::strEncode(val,TSYS::SQL)+"' ";
+	    bool isTransl = (u_cfg.fld().flg()&TCfg::TransltText && isVarTextTransl && !u_cfg.noTransl());
+	    sid = isTransl ? (lang2Code+"#"+cf_el[i_el]) : cf_el[i_el];
+	    ins_name = ins_name + (next?",`":"`") + TSYS::strEncode(sid,TSYS::SQL) + "` ";
+	    sval = getVal(u_cfg); //if( isTransl && sval.empty() ) sval = " ";
+	    ins_value = ins_value + (next?",'":"'") + TSYS::strEncode(sval,TSYS::SQL) + "' ";
+	    next = true;
 	}
 	req = req + "("+ins_name+") VALUES ("+ins_value+")";
     }
     else
     {
-	//-- Update present record --
-	req = "UPDATE `"+TSYS::strEncode(owner().bd,TSYS::SQL)+"`.`"+
-	    TSYS::strEncode(name(),TSYS::SQL)+"` SET ";
+	//>> Update present record
+	req = "UPDATE `" + TSYS::strEncode(owner().bd,TSYS::SQL) + "`.`" + TSYS::strEncode(name(),TSYS::SQL) + "` SET ";
 	next = false;
 	for( int i_el = 0; i_el < cf_el.size(); i_el++ )
 	{
 	    TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
 	    if( u_cfg.fld().flg()&TCfg::Key || !u_cfg.view() ) continue;
 
-	    if( !next ) next = true; else req=req+",";
-	    string val;
-	    switch(u_cfg.fld().type())
-	    {
-		case TFld::String:	val = u_cfg.getS();			break;
-		case TFld::Integer:	val = SYS->int2str(u_cfg.getI());	break;
-		case TFld::Real:	val = SYS->real2str(u_cfg.getR());	break;
-		case TFld::Boolean:	val = SYS->int2str(u_cfg.getB());	break;
-	    }
-	    req=req+"`"+TSYS::strEncode(cf_el[i_el],TSYS::SQL)+"`='"+
-			TSYS::strEncode(val,TSYS::SQL)+"' ";
+	    bool isTransl = (u_cfg.fld().flg()&TCfg::TransltText && isVarTextTransl && !u_cfg.noTransl());
+	    sid = isTransl ? (lang2Code+"#"+cf_el[i_el]) : cf_el[i_el];
+	    sval = getVal(u_cfg); //if( isTransl && sval.empty() ) sval = " ";
+	    req = req + (next?",`":"`") + TSYS::strEncode(sid,TSYS::SQL) + "`='" + TSYS::strEncode(sval,TSYS::SQL) + "' ";
+	    next = true;
 	}
 	req = req + req_where;
     }
 
-    //- Query -
+    //> Query
     //printf("TEST 02: query: <%s>\n",req.c_str());
     try{ owner().sqlReq( req ); }
     catch(TError err)	{ fieldFix(cfg); owner().sqlReq( req ); }
@@ -563,24 +564,23 @@ void MTable::fieldSet( TConfig &cfg )
 void MTable::fieldDel( TConfig &cfg )
 {
     if( tblStrct.empty() ) throw TError(TSYS::DBTableEmpty,nodePath().c_str(),_("Table is empty!"));
+    mLstUse = time(NULL);
 
-    //- Get config fields list -
+    //> Get config fields list
     vector<string> cf_el;
     cfg.cfgList(cf_el);
 
-    //- Prepare request -
-    string req = "DELETE FROM `"+TSYS::strEncode(owner().bd,TSYS::SQL)+"`.`"+
-	TSYS::strEncode(name(),TSYS::SQL)+"` WHERE ";
-    //-- Add key list to query --
+    //> Prepare request
+    string req = "DELETE FROM `" + TSYS::strEncode(owner().bd,TSYS::SQL) + "`.`" + TSYS::strEncode(name(),TSYS::SQL) + "` WHERE ";
+    //>> Add key list to query
     bool next = false;
     for( int i_el = 0; i_el < cf_el.size(); i_el++ )
     {
 	TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
-	if( u_cfg.fld().flg()&TCfg::Key && u_cfg.getS().size() )
+	if( u_cfg.fld().flg()&TCfg::Key && u_cfg.keyUse() )
 	{
-	    if( !next ) next = true; else req=req+"AND ";
-	    req=req+"`"+TSYS::strEncode(cf_el[i_el],TSYS::SQL)+"`='"+
-			TSYS::strEncode(u_cfg.getS(),TSYS::SQL)+"' ";
+	    req = req + (next?"AND `":"`") + TSYS::strEncode(cf_el[i_el],TSYS::SQL) + "`='" + TSYS::strEncode(getVal(u_cfg),TSYS::SQL) + "' ";
+	    next = true;
 	}
     }
 
@@ -593,42 +593,50 @@ void MTable::fieldFix( TConfig &cfg )
 
     if( tblStrct.empty() ) throw TError(TSYS::DBTableEmpty,nodePath().c_str(),_("Table is empty!"));
 
-    //- Get config fields list -
+    string lang2Code = cfg.lang2Code().empty() ? Mess->lang2Code() : cfg.lang2Code();
+    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && lang2Code != Mess->lang2CodeBase());
+    //> Get config fields list
     vector<string> cf_el;
     cfg.cfgList(cf_el);
 
-    //- Prepare request for fix structure -
-    string req = "ALTER TABLE `"+TSYS::strEncode(owner().bd,TSYS::SQL)+"`.`"+
-	TSYS::strEncode(name(),TSYS::SQL)+"` DROP PRIMARY KEY, ";
+    //> Prepare request for fix structure
+    string req = "ALTER TABLE `" + TSYS::strEncode(owner().bd,TSYS::SQL) + "`.`" + TSYS::strEncode(name(),TSYS::SQL) + "` DROP PRIMARY KEY, ";
 
     string pr_keys;
     for( int i_cf = 0; i_cf < cf_el.size(); i_cf++ )
     {
 	TCfg &u_cfg = cfg.cfg(cf_el[i_cf]);
-	//-- Check primary key --
+	//>> Check primary key
 	if( u_cfg.fld().flg()&TCfg::Key )
 	{
-	    if( !next_key ) next_key = true;
-	    else pr_keys=pr_keys+",";
-	    pr_keys=pr_keys+"`"+TSYS::strEncode(u_cfg.name(),TSYS::SQL)+"`";
+	    pr_keys = pr_keys + (next_key?",`":"`") + TSYS::strEncode(u_cfg.name(),TSYS::SQL) + "`";
+	    next_key = true;
 	}
-	
-	int i_fld;
+
+	int i_fld, tsz;
 	for( i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
-	    if( cf_el[i_cf] == tblStrct[i_fld][0] )	break;
+	    if( cf_el[i_cf] == tblStrct[i_fld][0] ) break;
+	/*{
+	    int fsz = cf_el[i_cf].size();
+	    int tsz = fsz - tblStrct[i_fld][0].size();
+	    if( (tsz == 0 && cf_el[i_cf] == tblStrct[i_fld][0]) || (tsz == 3 && cf_el[i_cf] == tblStrct[i_fld][0].substr(0,fsz-3) && tblStrct[i_fld][0][fsz] == '#' ) )
+		break;
+	}*/
+	//>> Change field
+	string f_tp;
 	if( i_fld < tblStrct.size() )
 	{
-	    string f_tp;
 	    switch(u_cfg.fld().type())
 	    {
 		    if( fieldLen(u_cfg.fld().len()) < 256 || u_cfg.fld().flg()&TCfg::Key )
-			f_tp = "varchar("+TSYS::int2str(vmax(1,vmin((u_cfg.fld().flg()&TCfg::Key)?150:255,fieldLen(u_cfg.fld().len()))))+")";
+			f_tp = "varchar("+TSYS::int2str(vmax(1,vmin((u_cfg.fld().flg()&TCfg::Key)?200:255,fieldLen(u_cfg.fld().len()))))+")";
 		    else if( fieldLen(u_cfg.fld().len()) < 65536 )
 			f_tp = "text";
 		    else f_tp = "mediumtext";
 		    break;
 		case TFld::Integer:
-		    f_tp = "int("+TSYS::int2str(vmax(1,u_cfg.fld().len()))+")";
+		    if( u_cfg.fld().flg()&TFld::DateTimeDec )	f_tp = "datetime";
+		    else f_tp = "int("+TSYS::int2str(vmax(1,u_cfg.fld().len()))+")";
 		    break;
 		case TFld::Real:
 		    f_tp = "double("+TSYS::int2str(vmax(3,u_cfg.fld().len()))+","+TSYS::int2str(vmax(2,u_cfg.fld().dec()))+")";
@@ -637,34 +645,64 @@ void MTable::fieldFix( TConfig &cfg )
 		    f_tp = "tinyint(1)";
 		    break;
 	    }
-	    if( tblStrct[i_fld][1] == f_tp )	continue;
-	    //-- Change field --
-	    if( !next ) next = true; else req=req+",";
-	    req=req+"CHANGE `"+TSYS::strEncode(cf_el[i_cf],TSYS::SQL)+"` `"+TSYS::strEncode(cf_el[i_cf],TSYS::SQL)+"` ";
-	    fieldPrmSet(u_cfg,(i_cf>0)?cf_el[i_cf-1]:"",req);
+	    if( tblStrct[i_fld][1] != f_tp )
+	    {
+		req = req + (next?",CHANGE `":"CHANGE `") + TSYS::strEncode(cf_el[i_cf],TSYS::SQL) + "` `" + TSYS::strEncode(cf_el[i_cf],TSYS::SQL) + "` ";
+		next = true;
+		fieldPrmSet(u_cfg,(i_cf>0)?cf_el[i_cf-1]:"",req);
+	    }
 	}
+	//>> Add field
 	else
 	{
-	    if( !next ) next = true; else req=req+",";
-	    //-- Add field --
-	    req=req+"ADD `"+TSYS::strEncode(cf_el[i_cf],TSYS::SQL)+"` ";
+	    req = req + (next?",ADD `":"ADD `") + TSYS::strEncode(cf_el[i_cf],TSYS::SQL) + "` ";
+	    next = true;
 	    fieldPrmSet(u_cfg,(i_cf>0)?cf_el[i_cf-1]:"",req);
 	}
-    }
-    //- DROP fields -
-    for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
-	if( !cfg.cfgPresent(tblStrct[i_fld][0]) )
+	//> Check other languages
+	if( u_cfg.fld().flg()&TCfg::TransltText )
 	{
-	    if( !next ) next = true; else req=req+",";
-	    req=req+"DROP `"+TSYS::strEncode(tblStrct[i_fld][0],TSYS::SQL)+"` ";
+	    bool col_cur = false;
+	    for( int i_c = i_fld; i_c < tblStrct.size(); i_c++ )
+		if( tblStrct[i_c][0].size() > 3 && tblStrct[i_c][0].substr(2) == ("#"+cf_el[i_cf]) )
+		{
+		    if( tblStrct[i_c][1] != f_tp )
+		    {
+			req = req + (next?",CHANGE `":"CHANGE `") + TSYS::strEncode(tblStrct[i_c][0],TSYS::SQL) + "` `" + TSYS::strEncode(tblStrct[i_c][0],TSYS::SQL) + "` ";
+			next = true;
+			fieldPrmSet(u_cfg,(i_cf>0)?cf_el[i_cf-1]:"",req);
+		    }
+		    if( tblStrct[i_c][0].substr(0,2) == lang2Code ) col_cur = true;
+		}
+	    if( !col_cur && isVarTextTransl )
+	    {
+		req = req + (next?",ADD `":"ADD `") + TSYS::strEncode(lang2Code+"#"+cf_el[i_cf],TSYS::SQL) + "` ";
+		next = true;
+		fieldPrmSet(u_cfg,(i_cf>0)?cf_el[i_cf-1]:"",req);
+	    }
 	}
-    req=req+",ADD PRIMARY KEY ("+pr_keys+") ";
+    }
+    //> DROP fields
+    for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
+    {
+	int i_cf;
+	for( i_cf = 0; i_cf < cf_el.size(); i_cf++ )
+	    if( cf_el[i_cf] == tblStrct[i_fld][0] ||
+		    (cfg.cfg(cf_el[i_cf]).fld().flg()&TCfg::TransltText && tblStrct[i_fld][0].size() > 3 && tblStrct[i_fld][0].substr(2) == ("#"+cf_el[i_cf])) )
+		break;
+	if( i_cf >= cf_el.size() )
+	{
+	    req = req + (next?",DROP `":"DROP `") + TSYS::strEncode(tblStrct[i_fld][0],TSYS::SQL) + "` ";
+	    next = true;
+	}
+    }
+    req = req + ",ADD PRIMARY KEY (" + pr_keys + ") ";
 
     if( next )
     {
 	owner().sqlReq( req );
-	//- Update structure information -
-	req ="DESCRIBE `"+TSYS::strEncode(owner().bd,TSYS::SQL)+"`.`"+TSYS::strEncode(name(),TSYS::SQL)+"`";
+	//> Update structure information
+	req = "DESCRIBE `" + TSYS::strEncode(owner().bd,TSYS::SQL) + "`.`" + TSYS::strEncode(name(),TSYS::SQL) + "`";
 	owner().sqlReq( req, &tblStrct );
     }
 }
@@ -676,13 +714,16 @@ void MTable::fieldPrmSet( TCfg &cfg, const string &last, string &req )
     {
 	case TFld::String:
 	    if( fieldLen(cfg.fld().len()) < 256 || cfg.fld().flg()&TCfg::Key )
-		req=req+"varchar("+SYS->int2str(vmax(1,vmin((cfg.fld().flg()&TCfg::Key)?150:255,fieldLen(cfg.fld().len()))))+") NOT NULL DEFAULT '"+cfg.fld().def()+"' ";
+		req=req+"varchar("+SYS->int2str(vmax(1,vmin((cfg.fld().flg()&TCfg::Key)?200:255,fieldLen(cfg.fld().len()))))+") "+
+			((cfg.fld().flg()&TCfg::Key)?"BINARY":"")+" NOT NULL DEFAULT '"+cfg.fld().def()+"' ";
 	    else if( fieldLen(cfg.fld().len()) < 65536 )
 		req=req+"text NOT NULL ";// DEFAULT '"+cfg.fld().def()+"' ";
 	    else req=req+"mediumtext NOT NULL ";// DEFAULT '"+cfg.fld().def()+"' ";
 	    break;
 	case TFld::Integer:
-	    req=req+"int("+SYS->int2str(vmax(1,cfg.fld().len()))+") NOT NULL DEFAULT '"+TSYS::int2str(atoi(cfg.fld().def().c_str()))+"' ";
+	    if( cfg.fld().flg()&TFld::DateTimeDec )
+	    	req=req+"datetime NOT NULL DEFAULT '"+UTCtoSQL(atoi(cfg.fld().def().c_str()))+"' ";
+	    else req=req+"int("+SYS->int2str(vmax(1,cfg.fld().len()))+") NOT NULL DEFAULT '"+TSYS::int2str(atoi(cfg.fld().def().c_str()))+"' ";
 	    break;
 	case TFld::Real:
 	    req=req+"double("+SYS->int2str(vmax(3,cfg.fld().len()))+","+SYS->int2str(vmax(2,cfg.fld().dec()))+") NOT NULL DEFAULT '"+TSYS::real2str(atof(cfg.fld().def().c_str()))+"' ";
@@ -695,3 +736,48 @@ void MTable::fieldPrmSet( TCfg &cfg, const string &last, string &req )
     //if( last.size() )	req=req+"AFTER `"+last+"` ";
 }
 
+string MTable::getVal( TCfg &cfg )
+{
+    switch( cfg.fld().type() )
+    {
+	case TFld::String:	return cfg.getS();
+	case TFld::Integer:
+	    if( cfg.fld().flg()&TFld::DateTimeDec )	return UTCtoSQL(cfg.getI());
+	    else		return SYS->int2str(cfg.getI());
+	case TFld::Real:	return SYS->real2str(cfg.getR());
+	case TFld::Boolean:	return SYS->int2str(cfg.getB());
+    }
+    return "";
+}
+
+void MTable::setVal( TCfg &cfg, const string &val )
+{
+    switch( cfg.fld().type() )
+    {
+	case TFld::String:	cfg.setS(val);	break;
+	case TFld::Integer:
+	    if( cfg.fld().flg()&TFld::DateTimeDec )	cfg.setI(SQLtoUTC(val));
+	    else cfg.setI(atoi(val.c_str()));
+	    break;
+	case TFld::Real:	cfg.setR(atof(val.c_str()));	break;
+	case TFld::Boolean:	cfg.setB(atoi(val.c_str()));	break;
+    }
+}
+
+string MTable::UTCtoSQL( time_t val )
+{
+    struct tm tm_tm;
+    localtime_r(&val,&tm_tm);
+    char buf[255];
+    int rez = strftime( buf, sizeof(buf), "%Y-%m-%d %H:%M:%S", &tm_tm );
+
+    return (rez>0) ? string(buf,rez) : "";
+}
+
+time_t MTable::SQLtoUTC( const string &val )
+{
+    struct tm stm;
+    strptime(val.c_str(),"%Y-%m-%d %H:%M:%S",&stm);
+
+    return mktime(&stm);
+}
