@@ -35,7 +35,7 @@
 #define MOD_NAME	"DB SQLite"
 #define MOD_TYPE	"BD"
 #define VER_TYPE	VER_BD
-#define VERSION		"1.4.0"
+#define VERSION		"1.6.0"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"BD modul. Allow support of the BD SQLite."
 #define LICENSE		"GPL"
@@ -118,7 +118,7 @@ void BDMod::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -203,7 +203,7 @@ void MBD::allowList( vector<string> &list )
 TTable *MBD::openTable( const string &inm, bool create )
 {
     if( !enableStat() )
-	throw TError(TSYS::DBOpenTable,nodePath().c_str(),_("Error open table <%s>. DB disabled."),inm.c_str());
+	throw TError(TSYS::DBOpenTable,nodePath().c_str(),_("Error open table <%s>. DB is disabled."),inm.c_str());
 
     return new MTable(inm,this,create);
 }
@@ -233,11 +233,11 @@ void MBD::sqlReq( const string &ireq, vector< vector<string> > *tbl )
     }
     //- Put request -
     rc = sqlite3_get_table(m_db,Mess->codeConvOut(cd_pg.c_str(),req).c_str(),&result, &nrow, &ncol, &zErrMsg );
-    if( rc != SQLITE_OK ) 
+    if( rc != SQLITE_OK )
     {
 	//-- Fix transaction --
 	if( trans_reqs>1 && (commCnt-1)<0 )	commCnt=trans_reqs;
-	throw TError(TSYS::DBRequest,nodePath().c_str(),_("Get table error: %s"),zErrMsg);
+	throw TError(100+rc,nodePath().c_str(),_("Getting table error: %s"),zErrMsg);
     }
     if( tbl && ncol > 0 )
     {
@@ -266,7 +266,7 @@ void MBD::cntrCmdProc( XMLNode *opt )
 	TBD::cntrCmdProc(opt);
 	ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),0664,"root","BD",2,
 		    "tp","str","help",
-		    _("SQLite DB address writet as: [<FileDBPath>;<nTransReq>].\n"
+		    _("SQLite DB address must be written as: [<FileDBPath>;<nTransReq>].\n"
 		      "Where:\n"
 		      "  FileDBPath - full path to DB file (./oscada/Main.db);\n"
 		      "  nTransReq - number requests into transaction (default 1)."));
@@ -295,7 +295,7 @@ MTable::MTable(string inm, MBD *iown, bool create ) : TTable(inm)
 	req ="PRAGMA table_info('"+mod->sqlReqCode(name())+"');";
 	owner().sqlReq( req, &tblStrct );
     }
-    catch(...) { if( !create ) throw; }	
+    catch(...) { if( !create ) throw; }
 }
 
 MTable::~MTable(  )
@@ -312,9 +312,12 @@ void MTable::postDisable(int flag)
     }
 }
 
+MBD &MTable::owner()	{ return (MBD&)TTable::owner(); }
+
 void MTable::fieldStruct( TConfig &cfg )
 {
     if( tblStrct.empty() ) throw TError(TSYS::DBTableEmpty,nodePath().c_str(),_("Table is empty."));
+    mLstUse = time(NULL);
 
     for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
     {
@@ -336,48 +339,55 @@ bool MTable::fieldSeek( int row, TConfig &cfg )
     vector< vector<string> > tbl;
 
     if( tblStrct.empty() ) throw TError(TSYS::DBTableEmpty,nodePath().c_str(),_("Table is empty."));
+    mLstUse = time(NULL);
 
-    //- Make WHERE -
+    string sid;
+    string lang2Code = cfg.lang2Code().empty() ? Mess->lang2Code() : cfg.lang2Code();
+    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && lang2Code != Mess->lang2CodeBase());
+    //> Make WHERE
     string req = "SELECT ";
     string req_where = "WHERE ";
-    //-- Add use keys to list --
+    //>> Add use keys to list
     bool first_sel = true;
     bool next = false;
     for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
     {
-	string sid = tblStrct[i_fld][1];
-	if( !cfg.cfgPresent(sid) ) continue;
-	TCfg &u_cfg = cfg.cfg(sid);
-	if( u_cfg.fld().flg()&TCfg::Key && u_cfg.getS().size() )
+	sid = tblStrct[i_fld][1];
+	TCfg *u_cfg = cfg.at(sid,true);
+	if( !u_cfg && isVarTextTransl && sid.size() > 3 && sid.substr(0,3) == (lang2Code+"#") )
 	{
-	    if( !next ) next = true; 
-	    else req_where=req_where+"AND ";
-	    req_where=req_where+"\""+mod->sqlReqCode(sid,'"')+"\"='"+mod->sqlReqCode(u_cfg.getS())+"' ";
+	    u_cfg = cfg.at(sid.substr(3),true);
+	    if( u_cfg && !(u_cfg->fld().flg()&TCfg::TransltText && !u_cfg->noTransl()) ) continue;
 	}
-	else if( u_cfg.fld().flg()&TCfg::Key || u_cfg.view() )
+	if( !u_cfg ) continue;
+
+	if( u_cfg->fld().flg()&TCfg::Key && u_cfg->keyUse() )
 	{
-	    if( first_sel ) req=req+"\""+mod->sqlReqCode(sid,'"')+"\"";
-	    else req=req+",\""+mod->sqlReqCode(sid,'"')+"\"";
+	    req_where = req_where + (next?"AND \"":"\"") + mod->sqlReqCode(sid,'"') + "\"='" + mod->sqlReqCode(getVal(*u_cfg)) + "' ";
+	    next = true;
+	}
+	else if( u_cfg->fld().flg()&TCfg::Key || u_cfg->view() )
+	{
+	    req = req + (first_sel?"\"":",\"")+mod->sqlReqCode(sid,'"')+"\"";
 	    first_sel = false;
 	}
     }
 
-    //- Request -
-    req = req+" FROM '"+mod->sqlReqCode(name())+"' "+((next)?req_where:"")+" LIMIT "+TSYS::int2str(row)+",1;";
+    //> Request
+    if( first_sel ) return false;
+    req = req + " FROM '" + mod->sqlReqCode(name()) + "' " + ((next)?req_where:"") + " LIMIT " +  TSYS::int2str(row) + ",1;";
     owner().sqlReq( req, &tbl );
     if( tbl.size() < 2 ) return false;
-    //- Processing of query -
+    //> Processing of query
     for( int i_fld = 0; i_fld < tbl[0].size(); i_fld++ )
     {
-	if( !cfg.cfgPresent(tbl[0][i_fld]) ) continue;
-	string val = tbl[1][i_fld];
-	TCfg &u_cfg = cfg.cfg(tbl[0][i_fld]);
-	switch(u_cfg.fld().type())
+	sid = tbl[0][i_fld];
+	TCfg *u_cfg = cfg.at(sid,true);
+	if( u_cfg ) setVal(*u_cfg,tbl[1][i_fld]);
+	else if( isVarTextTransl && sid.size() > 3 && sid.substr(0,3) == (lang2Code+"#") && tbl[1][i_fld].size() )
 	{
-	    case TFld::String:	u_cfg.setS(val); break;
-	    case TFld::Integer:	u_cfg.setI(atoi(val.c_str()));	break;
-	    case TFld::Real:	u_cfg.setR(atof(val.c_str()));	break;
-	    case TFld::Boolean:	u_cfg.setB(atoi(val.c_str()));	break;
+	    u_cfg = cfg.at(sid.substr(3),true);
+	    if( u_cfg ) setVal(*u_cfg,tbl[1][i_fld]);
 	}
     }
 
@@ -389,48 +399,55 @@ void MTable::fieldGet( TConfig &cfg )
     vector< vector<string> > tbl;
 
     if( tblStrct.empty() ) throw TError(TSYS::DBTableEmpty,nodePath().c_str(),_("Table is empty."));
+    mLstUse = time(NULL);
 
-    //- Prepare request -
+    string sid;
+    string lang2Code = cfg.lang2Code().empty() ? Mess->lang2Code() : cfg.lang2Code();
+    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && lang2Code != Mess->lang2CodeBase());
+    //> Prepare request
     string req = "SELECT ";
     string req_where;
-    //-- Add fields list to queue --
+    //>> Add fields list to queue
     bool first_sel = true;
     bool next_wr = false;
     for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
     {
-	if( !cfg.cfgPresent(tblStrct[i_fld][1]) ) continue;
-	TCfg &u_cfg = cfg.cfg(tblStrct[i_fld][1]);
-	if( u_cfg.fld().flg()&TCfg::Key )
+	sid = tblStrct[i_fld][1];
+	TCfg *u_cfg = cfg.at(sid,true);
+	if( !u_cfg && isVarTextTransl && sid.size() > 3 && sid.substr(0,3) == (lang2Code+"#") )
 	{
-	    if( !next_wr ) next_wr = true; else req_where=req_where+"AND ";
-	    req_where=req_where+"\""+mod->sqlReqCode(tblStrct[i_fld][1],'"')+"\"='"+
-				     mod->sqlReqCode(u_cfg.getS())+"'";
+	    u_cfg = cfg.at(sid.substr(3),true);
+	    if( u_cfg && !(u_cfg->fld().flg()&TCfg::TransltText && !u_cfg->noTransl()) ) continue;
 	}
-	else if( u_cfg.view() )
+	if( !u_cfg ) continue;
+
+	if( u_cfg->fld().flg()&TCfg::Key )
 	{
-	    if( first_sel ) req=req+"\""+mod->sqlReqCode(tblStrct[i_fld][1],'"')+"\"";
-		    else req=req+",\""+mod->sqlReqCode(tblStrct[i_fld][1],'"')+"\"";
+	    req_where = req_where + (next_wr?"AND \"":"\"") + mod->sqlReqCode(sid,'"') + "\"='" + mod->sqlReqCode(getVal(*u_cfg)) + "'";
+	    next_wr = true;
+	}
+	else if( u_cfg->view() )
+	{
+	    req = req + (first_sel?"\"":",\"") + mod->sqlReqCode(sid,'"') + "\"";
 	    first_sel = false;
 	}
     }
-    req = req+" FROM '"+mod->sqlReqCode(name())+"' WHERE "+req_where+";";
+    req = req + " FROM '" + mod->sqlReqCode(name()) + "' WHERE " + req_where + ";";
 
-    //- Query -
+    //> Query
     owner().sqlReq( req, &tbl );
-    if( tbl.size() < 2 ) throw TError(TSYS::DBRowNoPresent,nodePath().c_str(),_("Row no present."));
+    if( tbl.size() < 2 ) throw TError(TSYS::DBRowNoPresent,nodePath().c_str(),_("Row is not present."));
 
-    //- Processing of query -
+    //> Processing of query
     for( int i_fld = 0; i_fld < tbl[0].size(); i_fld++ )
     {
-	if( !cfg.cfgPresent(tbl[0][i_fld]) ) continue;
-	string val = tbl[1][i_fld];
-	TCfg &u_cfg = cfg.cfg(tbl[0][i_fld]);
-	switch(u_cfg.fld().type())
+	sid = tbl[0][i_fld];
+	TCfg *u_cfg = cfg.at(sid,true);
+	if( u_cfg ) setVal(*u_cfg,tbl[1][i_fld]);
+	else if( isVarTextTransl && sid.size() > 3 && sid.substr(0,3) == (lang2Code+"#") && tbl[1][i_fld].size() )
 	{
-	    case TFld::String:	u_cfg.setS(val); break;
-	    case TFld::Integer:	u_cfg.setI(atoi(val.c_str()));	break;
-	    case TFld::Real:	u_cfg.setR(atof(val.c_str()));	break;
-	    case TFld::Boolean:	u_cfg.setB(atoi(val.c_str()));	break;
+	    u_cfg = cfg.at(sid.substr(3),true);
+	    if( u_cfg ) setVal(*u_cfg,tbl[1][i_fld]);
 	}
     }
 }
@@ -440,32 +457,39 @@ void MTable::fieldSet( TConfig &cfg )
     vector< vector<string> > tbl;
 
     if( tblStrct.empty() ) fieldFix(cfg);
+    mLstUse = time(NULL);
 
-    //- Get config fields list -
+    string sid, sval;
+    string lang2Code = cfg.lang2Code().empty() ? Mess->lang2Code() : cfg.lang2Code();
+    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && lang2Code != Mess->lang2CodeBase());
+    //> Get config fields list
     vector<string> cf_el;
     cfg.cfgList(cf_el);
 
-    //- Get present fields list -
+    //> Get present fields list
     string req_where = "WHERE ";
-    //-- Add key list to queue --
+    //>> Add key list to queue
     bool next = false;
     for( int i_el = 0; i_el < cf_el.size(); i_el++ )
     {
 	TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
 	if( !(u_cfg.fld().flg()&TCfg::Key) ) continue;
-	if( !next ) next = true; else req_where=req_where+"AND ";
-	req_where=req_where+"\""+mod->sqlReqCode(cf_el[i_el],'"')+"\"='"+
-				 mod->sqlReqCode(u_cfg.getS())+"' ";
+	req_where = req_where + (next?"AND \"":"\"") + mod->sqlReqCode(cf_el[i_el],'"') + "\"='" + mod->sqlReqCode(getVal(u_cfg)) + "' ";
+	next = true;
     }
 
-    //- Prepare query -
-    string req = "SELECT 1 FROM '"+mod->sqlReqCode(name())+"' "+req_where+";";
+    //> Prepare query
+    string req = "SELECT 1 FROM '" + mod->sqlReqCode(name()) + "' " + req_where + ";";
     try{ owner().sqlReq( req, &tbl ); }
-    catch(TError err)	{ fieldFix(cfg); owner().sqlReq( req ); }
+    catch(TError err)
+    {
+	if( (err.cod-100) == SQLITE_READONLY )	return;
+	fieldFix(cfg); owner().sqlReq( req );
+    }
     if( tbl.size() < 2 )
     {
-	//-- Add new record --
-	req = "INSERT INTO '"+mod->sqlReqCode(name())+"' ";
+	//> Add new record
+	req = "INSERT INTO '" + mod->sqlReqCode(name()) + "' ";
 	string ins_name, ins_value;
 	next = false;
 	for( int i_el = 0; i_el < cf_el.size(); i_el++ )
@@ -473,81 +497,76 @@ void MTable::fieldSet( TConfig &cfg )
 	    TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
 	    if( !(u_cfg.fld().flg()&TCfg::Key) && !u_cfg.view() ) continue;
 
-	    if( !next ) next = true;
-	    else
-	    {
-		ins_name=ins_name+",";
-		ins_value=ins_value+",";
-	    }
-	    ins_name=ins_name+"\""+mod->sqlReqCode(cf_el[i_el],'"')+"\" ";
-	    string val;
-	    switch(u_cfg.fld().type())
-	    {
-		case TFld::String:	val = u_cfg.getS();			break;
-		case TFld::Integer:	val = SYS->int2str(u_cfg.getI());	break;
-		case TFld::Real:	val = SYS->real2str(u_cfg.getR());	break;
-		case TFld::Boolean:	val = SYS->int2str(u_cfg.getB());	break;
-	    }
-	    ins_value=ins_value+"'"+mod->sqlReqCode(val)+"' ";
+	    bool isTransl = (u_cfg.fld().flg()&TCfg::TransltText && isVarTextTransl && !u_cfg.noTransl());
+	    sid = isTransl ? (lang2Code+"#"+cf_el[i_el]) : cf_el[i_el];
+	    ins_name = ins_name + (next?",\"":"\"") + mod->sqlReqCode(sid,'"') + "\" ";
+	    sval = getVal(u_cfg); //if( isTransl && sval.empty() ) sval = " ";
+	    ins_value = ins_value + (next?",'":"'") + mod->sqlReqCode(sval) + "' ";
+	    next = true;
 	}
 	req = req + "("+ins_name+") VALUES ("+ins_value+")";
     }
     else
     {
-	//-- Update present record --
-	req = "UPDATE '"+mod->sqlReqCode(name())+"' SET ";
+	//> Update present record
+	req = "UPDATE '" + mod->sqlReqCode(name()) + "' SET ";
 	next = false;
 	for( int i_el = 0; i_el < cf_el.size(); i_el++ )
 	{
 	    TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
 	    if( u_cfg.fld().flg()&TCfg::Key || !u_cfg.view() ) continue;
 
-	    if( !next ) next = true; else req=req+",";
-	    string val;
-	    switch(u_cfg.fld().type())
-	    {
-		case TFld::String:	val = u_cfg.getS();			break;
-		case TFld::Integer:	val = SYS->int2str(u_cfg.getI());	break;
-		case TFld::Real:	val = SYS->real2str(u_cfg.getR());	break;
-		case TFld::Boolean:	val = SYS->int2str(u_cfg.getB());	break;
-	    }
-	    req=req+"\""+mod->sqlReqCode(cf_el[i_el],'"')+"\"='"+mod->sqlReqCode(val)+"' ";
+	    bool isTransl = (u_cfg.fld().flg()&TCfg::TransltText && isVarTextTransl && !u_cfg.noTransl());
+	    sid = isTransl ? (lang2Code+"#"+cf_el[i_el]) : cf_el[i_el];
+	    sval = getVal(u_cfg); //if( isTransl && sval.empty() ) sval = " ";
+	    req = req + (next?",\"":"\"") + mod->sqlReqCode(sid,'"') + "\"='" + mod->sqlReqCode(sval) + "' ";
+	    next = true;
 	}
 	req = req + req_where;
     }
     req += ";";
 
-    //- Query -
+    //> Query
     //printf("TEST 02: query: <%s>\n",req.c_str());
     try{ owner().sqlReq( req ); }
-    catch(TError err)	{ fieldFix(cfg); owner().sqlReq( req ); }
+    catch(TError err)
+    {
+	if( (err.cod-100) == SQLITE_READONLY )	return;
+	fieldFix(cfg); owner().sqlReq( req );
+    }
     //printf("TEST 01b: End from set\n");
 }
 
 void MTable::fieldDel( TConfig &cfg )
 {
     if( tblStrct.empty() ) throw TError(TSYS::DBTableEmpty,nodePath().c_str(),_("Table is empty."));
+    mLstUse = time(NULL);
 
-    //- Get config fields list -
+    //> Get config fields list
     vector<string> cf_el;
     cfg.cfgList(cf_el);
 
-    //- Prepare request -
+    //> Prepare request
     string req = "DELETE FROM '"+mod->sqlReqCode(name())+"' WHERE ";
-    //-- Add key list to queue --
+    //>> Add key list to queue
     bool next = false;
     for( int i_el = 0; i_el < cf_el.size(); i_el++ )
     {
 	TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
-	if( u_cfg.fld().flg()&TCfg::Key && u_cfg.getS().size() )
+	if( u_cfg.fld().flg()&TCfg::Key && u_cfg.keyUse() )
 	{
-	    if( !next ) next = true; else req=req+"AND ";
-	    req=req+"\""+mod->sqlReqCode(cf_el[i_el],'"')+"\"='"+
-			 mod->sqlReqCode(u_cfg.getS())+"' ";
+	    req = req + (next?"AND \"":"\"") + mod->sqlReqCode(cf_el[i_el],'"') + "\"='" + mod->sqlReqCode(getVal(u_cfg)) + "' ";
+	    next = true;
 	}
     }
     req += ";";
-    owner().sqlReq( req );
+    try{ owner().sqlReq( req ); }
+    catch( TError err )
+    {
+	if( (err.cod-100) == SQLITE_READONLY )
+	    throw TError(TSYS::DBReadOnly,nodePath().c_str(),_("Deletion is not permited. Data base is read only.\n"));
+	throw;
+    }
 }
 
 void MTable::fieldFix( TConfig &cfg )
@@ -557,77 +576,109 @@ void MTable::fieldFix( TConfig &cfg )
 
     string req;
 
-    //- Get config fields list -
+    string lang2Code = cfg.lang2Code().empty() ? Mess->lang2Code() : cfg.lang2Code();
+    bool isVarTextTransl = (!Mess->lang2CodeBase().empty() && lang2Code != Mess->lang2CodeBase());
+    //> Get config fields list
     vector<string> cf_el;
     cfg.cfgList(cf_el);
 
     if( !tblStrct.empty() )
     {
-	//- Check structure -
+	//> Check structure
 	bool next = false;
-	//-- Check present fields and find new fields --
+	//>> Check present fields and find new fields
 	for( int i_cf = 0; i_cf < cf_el.size(); i_cf++ )
 	{
+	    TCfg &u_cfg = cfg.cfg(cf_el[i_cf]);
 	    int i_fld;
 	    for( i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
-	        if( cf_el[i_cf] == tblStrct[i_fld][1] )
+		if( cf_el[i_cf] == tblStrct[i_fld][1] )
 		{
-		    if( !next ) next = true; else all_flds+=",";
-		    all_flds = all_flds+"\""+mod->sqlReqCode(cf_el[i_cf],'"')+"\"";
-		    switch(cfg.cfg(cf_el[i_cf]).fld().type())
-		    {
-			case TFld::String:	if( tblStrct[i_fld][2] != "TEXT")	fix = true;	break;
-			case TFld::Integer: case TFld::Boolean:
+		    all_flds = all_flds + (next?",\"":"\"") + mod->sqlReqCode(tblStrct[i_fld][1],'"') + "\"";
+		    next = true;
+		    if( !fix )
+			switch( u_cfg.fld().type() )
+			{
+			    case TFld::String:	if( tblStrct[i_fld][2] != "TEXT")	fix = true;	break;
+			    case TFld::Integer: case TFld::Boolean:
 						if( tblStrct[i_fld][2] != "INTEGER")	fix = true;	break;
-			case TFld::Real:	if( tblStrct[i_fld][2] != "DOUBLE" )	fix = true;	break;
-			default: fix = true;
+			    case TFld::Real:	if( tblStrct[i_fld][2] != "DOUBLE" )	fix = true;	break;
+			    default: fix = true;
+			}
+		    //> Put other languages to all list an find column for current language
+		    if( u_cfg.fld().flg()&TCfg::TransltText )
+		    {
+			bool col_cur = false;
+			for( int i_c = i_fld; i_c < tblStrct.size(); i_c++ )
+			    if( tblStrct[i_c][1].size() > 3 && tblStrct[i_c][1].substr(2) == ("#"+cf_el[i_cf]) )
+			    {
+				all_flds = all_flds + ",\"" + mod->sqlReqCode(tblStrct[i_c][1],'"') + "\"";
+				if( tblStrct[i_c][1].substr(0,2) == lang2Code ) col_cur = true;
+			    }
+			if( !col_cur && isVarTextTransl ) fix = true;
 		    }
 		    break;
 		}
 	    if( i_fld >= tblStrct.size() ) fix = true;
 	}
-	//-- Check delete fields --
-	for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
+	//>> Check delete fields
+	for( int i_fld = 1; i_fld < tblStrct.size() && !fix; i_fld++ )
 	{
 	    int i_cf;
 	    for( i_cf = 0; i_cf < cf_el.size(); i_cf++ )
-		if( cf_el[i_cf] == tblStrct[i_fld][1] ) break;
+		if( cf_el[i_cf] == tblStrct[i_fld][1] ||
+			(cfg.cfg(cf_el[i_cf]).fld().flg()&TCfg::TransltText && tblStrct[i_fld][1].size() > 3 && tblStrct[i_fld][1].substr(2) == ("#"+cf_el[i_cf])) )
+		    break;
 	    if( i_cf >= cf_el.size() ) fix = true;
 	}
 
 	if( !fix ) return;	//Structure OK!
-	//-- Fix structure --
-	//--- Move data to temporary DB ---
+	//>> Fix structure
+	//>>> Move data to temporary DB
 	req = "CREATE TEMPORARY TABLE 'temp_"+mod->sqlReqCode(name())+"'("+all_flds+");"
 	    "INSERT INTO 'temp_"+mod->sqlReqCode(name())+"' SELECT "+all_flds+" FROM '"+mod->sqlReqCode(name())+"';"
 	    "DROP TABLE '"+mod->sqlReqCode(name())+"';";
 	owner().sqlReq( req );
     }
 
-    //- Create new table -
+    //> Create new table
     req ="CREATE TABLE '"+mod->sqlReqCode(name())+"' (";
     bool next = false;
     bool next_key = false;
-    string pr_keys;
+    string pr_keys, tpCfg;
     for( int i_cf = 0; i_cf < cf_el.size(); i_cf++ )
     {
-	if( !next ) next = true; else req=req+",";
 	TCfg &cf = cfg.cfg(cf_el[i_cf]);
-	req = req+"\""+mod->sqlReqCode(cf_el[i_cf],'"')+"\" ";
-	//-- Type param --
+	req = req + (next?",\"":"\"") + mod->sqlReqCode(cf_el[i_cf],'"') + "\" ";
+	next = true;
+	//>> Type param
 	switch(cf.fld().type())
 	{
-	    case TFld::String:	req+="TEXT DEFAULT '"+mod->sqlReqCode(cf.fld().def())+"' ";	break;
+	    case TFld::String:	tpCfg = "TEXT DEFAULT '" + mod->sqlReqCode(cf.fld().def()) + "' ";	break;
 	    case TFld::Integer: case TFld::Boolean:
-				req+="INTEGER DEFAULT '"+mod->sqlReqCode(cf.fld().def())+"' ";	break;
-	    case TFld::Real:	req+="DOUBLE DEFAULT '"+mod->sqlReqCode(cf.fld().def())+"' ";	break;
+				tpCfg = "INTEGER DEFAULT '" + mod->sqlReqCode(cf.fld().def()) + "' ";	break;
+	    case TFld::Real:	tpCfg = "DOUBLE DEFAULT '" + mod->sqlReqCode(cf.fld().def()) + "' ";	break;
 	}
-	//-- Primary key --
-	if( cf.fld().flg()&TCfg::Key )
+	req += tpCfg;
+
+	//> Other languages for translation process
+	if( cf.fld().flg()&TCfg::TransltText )
 	{
-	    if( !next_key ) next_key = true;
-	    else pr_keys=pr_keys+",";
-	    pr_keys=pr_keys+"\""+mod->sqlReqCode(cf_el[i_cf],'"')+"\"";
+	    bool col_cur = false;
+	    for( int i_c = 1; i_c < tblStrct.size(); i_c++ )
+		if( tblStrct[i_c][1].size() > 3 && tblStrct[i_c][1].substr(2) == ("#"+cf_el[i_cf]) )
+		{
+		    req = req + ",\"" + mod->sqlReqCode(tblStrct[i_c][1],'"') + "\" "+tpCfg;
+		    if( tblStrct[i_c][1].substr(0,2) == lang2Code ) col_cur = true;
+		}
+	    if( !col_cur && isVarTextTransl )
+		req = req + ",\"" + mod->sqlReqCode(lang2Code+"#"+cf_el[i_cf],'"') + "\" "+tpCfg;
+	}
+	//>> Primary key
+	else if( cf.fld().flg()&TCfg::Key )
+	{
+	    pr_keys = pr_keys + (next_key?",\"":"\"") + mod->sqlReqCode(cf_el[i_cf],'"') + "\"";
+	    next_key = true;
 	}
     }
     req += ", PRIMARY KEY ("+pr_keys+"));";
@@ -648,3 +699,26 @@ void MTable::fieldFix( TConfig &cfg )
     req ="PRAGMA table_info('"+mod->sqlReqCode(name())+"');";
     owner().sqlReq( req, &tblStrct );
 }
+
+string MTable::getVal( TCfg &cfg )
+{
+    switch( cfg.fld().type() )
+    {
+	case TFld::String:	return cfg.getS();
+	case TFld::Integer:	return SYS->int2str(cfg.getI());
+	case TFld::Real:	return SYS->real2str(cfg.getR());
+	case TFld::Boolean:	return SYS->int2str(cfg.getB());
+    }
+    return "";
+}
+
+void MTable::setVal( TCfg &cfg, const string &val )
+{
+    switch( cfg.fld().type() )
+    {
+	case TFld::String:	cfg.setS(val);	break;
+	case TFld::Integer:	cfg.setI(atoi(val.c_str()));	break;
+	case TFld::Real:	cfg.setR(atof(val.c_str()));	break;
+	case TFld::Boolean:	cfg.setB(atoi(val.c_str()));	break;
+    }
+}
