@@ -54,13 +54,16 @@ class MTable : public TTable
 	void fieldSet( TConfig &cfg );
 	void fieldDel( TConfig &cfg );
 
-	MBD &owner()	{ return (MBD&)TTable::owner(); }
+	MBD &owner( );
 
     private:
 	//Private methods
-	void postDisable(int flag);
+	void postDisable( int flag );
 	void fieldFix( TConfig &cfg );
 
+	string getVal( TCfg &cfg );
+	void   setVal( TCfg &cfg, const string &vl );
+
 	//Private attributes
 	vector< vector<string> > tblStrct;
 };
