@@ -32,7 +32,7 @@ using namespace Virtual;
 //*************************************************
 //* Block: Function block                         *
 //*************************************************
-Block::Block( const string &iid, Contr *iown ) : 
+Block::Block( const string &iid, Contr *iown ) :
     TCntrNode(iown), TConfig( &((TipContr &)iown->owner()).blockE() ), TValFunc(iid+"_block",NULL),
     m_enable(false), m_process(false), id_freq(-1), id_start(-1), id_stop(-1),
     m_id(cfg("ID").getSd()), m_name(cfg("NAME").getSd()), m_descr(cfg("DESCR").getSd()),
@@ -77,28 +77,28 @@ void Block::postDisable( int flag )
     {
 	if( flag )
 	{
-	    //- Delete block from BD -
+	    //> Delete block from BD
 	    string tbl = owner().DB()+"."+owner().cfg("BLOCK_SH").getS();
-	    SYS->db().at().dataDel(tbl,mod->nodePath()+owner().cfg("BLOCK_SH").getS(),*this);
+	    SYS->db().at().dataDel(tbl,mod->nodePath()+owner().cfg("BLOCK_SH").getS(),*this,true);
 
-	    //- Delete block's IO from BD -
+	    //> Delete block's IO from BD
 	    TConfig cfg(&owner().owner().blockIOE());
 	    tbl=tbl+"_io";
-	    cfg.cfg("BLK_ID").setS(id());   //Delete all block id records
-	    cfg.cfg("ID").setS("");
+	    cfg.cfg("BLK_ID").setS(id(),true);   //Delete all block id records
 	    SYS->db().at().dataDel(tbl,mod->nodePath()+owner().cfg("BLOCK_SH").getS()+"_io",cfg);
 	}
     }catch(TError err)
     { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
 }
 
-string Block::name( )
-{
-    return m_name.size() ? m_name : id();
-}
+Contr &Block::owner( )	{ return *(Contr *)nodePrev(); }
+
+string Block::name( )	{ return m_name.size() ? m_name : id(); }
 
 void Block::load_( )
 {
+    if( !SYS->chkSelDB(owner().DB()) ) return;
+
     string bd = owner().DB()+"."+owner().cfg("BLOCK_SH").getS();
     SYS->db().at().dataGet(bd,mod->nodePath()+owner().cfg("BLOCK_SH").getS(),*this);
 
@@ -142,11 +142,10 @@ void Block::loadIO( const string &blk_db, const string &blk_id )
 	}
 
 	cfg.cfg("ID").setS(func()->io(i_ln)->id());
-	if(!SYS->db().at().dataGet(bd,mod->nodePath()+bd_tbl,cfg))
-	    continue;
-	//- Value -
+	if( !SYS->db().at().dataGet(bd,mod->nodePath()+bd_tbl,cfg) ) continue;
+	//> Value
 	setS(i_ln,cfg.cfg("VAL").getS());
-	//- Config of link -
+	//> Config of link
 	setLink(i_ln,SET,(LnkT)cfg.cfg("TLNK").getI(),cfg.cfg("LNK").getS());
     }
 }
@@ -375,10 +374,10 @@ void Block::calc( bool first, bool last )
     }catch(TError err)
     {
 	err_cnt++;
-	lnk_res.resReleaseR( );
-	throw TError(nodePath().c_str(),_("Error read block's <%s> links."),id().c_str());
+	lnk_res.resRelease( );
+	throw TError(nodePath().c_str(),_("Error reading block's <%s> links."),id().c_str());
     }
-    lnk_res.resReleaseR( );
+    lnk_res.resRelease( );
 
     //- Calc function -
     try
@@ -386,11 +385,7 @@ void Block::calc( bool first, bool last )
 	TValFunc::calc( );
 	modif();
     }
-    catch(TError err)
-    {
-	err_cnt++;
-	throw TError(nodePath().c_str(),_("Error calc block <%s>."),id().c_str());
-    }
+    catch(TError err) { err_cnt++; throw; }
 
     //- Put values to output links -
     lnk_res.resRequestR( );
@@ -423,18 +418,41 @@ void Block::calc( bool first, bool last )
     }catch(TError err)
     {
 	err_cnt++;
-	lnk_res.resReleaseR( );
-	throw TError(nodePath().c_str(),_("Error write block's <%s> links."),id().c_str());
+	lnk_res.resRelease( );
+	throw TError(nodePath().c_str(),_("Error writing block's <%s> links."),id().c_str());
     }
-    lnk_res.resReleaseR( );
+    lnk_res.resRelease( );
     err_cnt=0;
 }
 
 void Block::cntrCmdProc( XMLNode *opt )
 {
-    //- Get page info -
+    //> Service commands process
+    string a_path = opt->attr("path");
+    if( a_path.substr(0,6) == "/serv/" )
+    {
+	if( a_path == "/serv/attr" )
+	{
+	    if( !enable() || !func() ) throw TError(nodePath().c_str(),_("Block disabled or error."));
+	    if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEQ_RD) )
+		for( int i_a = 0; i_a < ioSize(); i_a++ )
+		    opt->childAdd("a")->setAttr("id",func()->io(i_a)->id())->setText(getS(i_a));
+	    if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEQ_WR) )
+		for( int i_a = 0; i_a < opt->childSize(); i_a++ )
+		{
+		    int io_id = -1;
+		    if( opt->childGet(i_a)->name() != "a" || (io_id=ioId(opt->childGet(i_a)->attr("id"))) < 0 ) continue;
+		    setS(io_id,opt->childGet(i_a)->text());
+		}
+	}
+	else TCntrNode::cntrCmdProc(opt);
+	return;
+    }
+
+    //> Get page info
     if( opt->name() == "info" )
     {
+	TCntrNode::cntrCmdProc(opt);
 	ctrMkNode("oscada_cntr",opt,-1,"/",_("Block: ")+id(),0664,"root","root");
 	if(ctrMkNode("area",opt,-1,"/blck",_("Block")))
 	{
@@ -447,7 +465,7 @@ void Block::cntrCmdProc( XMLNode *opt )
 	    if(ctrMkNode("area",opt,-1,"/blck/cfg",_("Config")))
 	    {
 		ctrMkNode("fld",opt,-1,"/blck/cfg/id",_("Id"),0444,"root","root",1,"tp","str");
-		ctrMkNode("fld",opt,-1,"/blck/cfg/name",_("Name"),0664,"root","root",1,"tp","str");
+		ctrMkNode("fld",opt,-1,"/blck/cfg/name",_("Name"),0664,"root","root",2,"tp","str","len","50");
 		ctrMkNode("fld",opt,-1,"/blck/cfg/descr",_("Description"),0664,"root","root",3,"tp","str","cols","90","rows","4");
 		ctrMkNode("fld",opt,-1,"/blck/cfg/toen",_("To enable"),0664,"root","root",1,"tp","bool");
 		ctrMkNode("fld",opt,-1,"/blck/cfg/toprc",_("To process"),0664,"root","root",1,"tp","bool");
@@ -460,7 +478,7 @@ void Block::cntrCmdProc( XMLNode *opt )
 	    if(ctrMkNode("area",opt,-1,"/lio",_("IO")))
 	    {
 		ctrMkNode("area",opt,-1,"/lio/show",_("Show"));
-		ctrMkNode("fld",opt,-1,"/lio/show/hide",_("Hiden"),0664,"root","root",1,"tp","bool");
+		ctrMkNode("fld",opt,-1,"/lio/show/hide",_("Hidden"),0664,"root","root",1,"tp","bool");
 		if(ctrMkNode("area",opt,-1,"/lio/io",_("IO")))
 		{
 		    vector<string> list;
@@ -468,7 +486,7 @@ void Block::cntrCmdProc( XMLNode *opt )
 		    for( int i_io = 0; i_io < list.size(); i_io++ )
 		    {
 			int id = ioId(list[i_io]);
-			if( ioHide(id) && !atoi(TBDS::genDBGet(nodePath()+"showHide","0",opt->attr("user")).c_str()) ) continue;
+			if( ioHide(id) && !atoi(TBDS::genDBGet(owner().nodePath()+"showHide","0",opt->attr("user")).c_str()) ) continue;
 			const char *tip;
 			switch(ioType(id))
 			{
@@ -485,7 +503,7 @@ void Block::cntrCmdProc( XMLNode *opt )
 	    if(ctrMkNode("area",opt,-1,"/lnk",_("Links")))
 	    {
 		ctrMkNode("area",opt,-1,"/lnk/show",_("Show"));
-		ctrMkNode("fld",opt,-1,"/lnk/show/hide",_("Hiden"),0664,"root","root",1,"tp","bool");
+		ctrMkNode("fld",opt,-1,"/lnk/show/hide",_("Hidden"),0664,"root","root",1,"tp","bool");
 		if(ctrMkNode("area",opt,-1,"/lnk/io",_("IO")))
 		{
 		    vector<string> list;
@@ -495,7 +513,7 @@ void Block::cntrCmdProc( XMLNode *opt )
 		    {
 			int id = ioId(list[i_io]);
 
-			if( ioHide(id) && !atoi(TBDS::genDBGet(nodePath()+"showHide","0",opt->attr("user")).c_str()) ) continue;		
+			if( ioHide(id) && !atoi(TBDS::genDBGet(owner().nodePath()+"showHide","0",opt->attr("user")).c_str()) ) continue;
 
 			//-- Add link's type --
 			ctrMkNode("fld",opt,-1,(string("/lnk/io/1|")+list[i_io]).c_str(),
@@ -508,8 +526,7 @@ void Block::cntrCmdProc( XMLNode *opt )
 	}
 	return;
     }
-    //- Process command to page -
-    string a_path = opt->attr("path");
+    //> Process command to page
     if( a_path == "/blck/st/en" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(enable()?"1":"0");
@@ -566,8 +583,8 @@ void Block::cntrCmdProc( XMLNode *opt )
     }
     else if( (a_path == "/lio/show/hide" || a_path == "/lnk/show/hide") && enable() )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(TBDS::genDBGet(nodePath()+"showHide","0",opt->attr("user")));
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	TBDS::genDBSet(nodePath()+"showHide",opt->text(),opt->attr("user"));
+	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(TBDS::genDBGet(owner().nodePath()+"showHide","0",opt->attr("user")));
+	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	TBDS::genDBSet(owner().nodePath()+"showHide",opt->text(),opt->attr("user"));
     }
     else if( a_path.substr(0,7) == "/lio/io" && enable() )
     {
