@@ -44,7 +44,7 @@
 #define MOD_NAME	"Block based calculator"
 #define MOD_TYPE	"DAQ"
 #define VER_TYPE	VER_CNTR
-#define VERSION		"1.1.0"
+#define VERSION		"1.2.0"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Allow block based calculator."
 #define LICENSE		"GPL"
@@ -120,7 +120,7 @@ void TipContr::load_()
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -130,7 +130,7 @@ void TipContr::load_()
 
 void TipContr::postEnable( int flag )
 {
-    TModule::postEnable( flag );
+    TTipDAQ::postEnable( flag );
 
     //Controllers BD structure
     fldAdd( new TFld("PRM_BD",_("Parameters table"),TFld::String,TFld::NoFlag,"30","system") );
@@ -141,19 +141,19 @@ void TipContr::postEnable( int flag )
 
     //Add parameter types
     int t_prm = tpParmAdd("std","PRM_BD",_("Standard"));
-    tpPrmAt(t_prm).fldAdd( new TFld("IO",_("Blocks' IOs"),TFld::String,TFld::FullText|TCfg::NoVal,"200") );
+    tpPrmAt(t_prm).fldAdd( new TFld("IO",_("Blocks' IOs"),TFld::String,TFld::FullText|TCfg::TransltText|TCfg::NoVal,"200") );
 
     //Blok's db structure
-    blk_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"10") );
-    blk_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TFld::NoFlag,"20") );
-    blk_el.fldAdd( new TFld("DESCR",_("Description"),TFld::String,TFld::NoFlag,"100") );
+    blk_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"20") );
+    blk_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TCfg::TransltText,"50") );
+    blk_el.fldAdd( new TFld("DESCR",_("Description"),TFld::String,TCfg::TransltText,"300") );
     blk_el.fldAdd( new TFld("FUNC",_("Function"),TFld::String,TFld::NoFlag,"75") );
     blk_el.fldAdd( new TFld("EN",_("To enable"),TFld::Boolean,TFld::NoFlag,"1","0") );
     blk_el.fldAdd( new TFld("PROC",_("To process"),TFld::Boolean,TFld::NoFlag,"1","0") );
 
     //IO blok's db structure
-    blkio_el.fldAdd( new TFld("BLK_ID",_("Blok's ID"),TFld::String,TCfg::Key,"10") );
-    blkio_el.fldAdd( new TFld("ID",_("IO ID"),TFld::String,TCfg::Key,"10") );
+    blkio_el.fldAdd( new TFld("BLK_ID",_("Blok's ID"),TFld::String,TCfg::Key,"20") );
+    blkio_el.fldAdd( new TFld("ID",_("IO ID"),TFld::String,TCfg::Key,"20") );
     blkio_el.fldAdd( new TFld("TLNK",_("Link's type"),TFld::Integer,TFld::NoFlag,"2") );
     blkio_el.fldAdd( new TFld("LNK",_("Link"),TFld::String,TFld::NoFlag,"50") );
     blkio_el.fldAdd( new TFld("VAL",_("Link's value"),TFld::String,TFld::NoFlag,"20") );
@@ -203,7 +203,7 @@ Contr::Contr( string name_c, const string &daq_db, ::TElem *cfgelem) :
     m_bl = grpAdd("blk_");
 }
 
-Contr::~Contr()
+Contr::~Contr( )
 {
 
 }
@@ -233,6 +233,13 @@ TCntrNode &Contr::operator=( TCntrNode &node )
     return *this;
 }
 
+string Contr::getStatus( )
+{
+    string rez = TController::getStatus( );
+    if( startStat() && !redntUse( ) ) rez += TSYS::strMess(_("Calc time %g us. "),tm_calc);
+    return rez;
+}
+
 void Contr::postDisable(int flag)
 {
     if( run_st ) stop();
@@ -255,8 +262,12 @@ void Contr::postDisable(int flag)
     TController::postDisable(flag);
 }
 
+TipContr &Contr::owner( )	{ return (TipContr&)TController::owner( ); }
+
 void Contr::load_( )
 {
+    if( !SYS->chkSelDB(DB()) ) return;
+
     TController::load_( );
 
     //- Load block's configuration -
@@ -264,8 +275,7 @@ void Contr::load_( )
     c_el.cfgViewAll(false);
     string bd = DB()+"."+cfg("BLOCK_SH").getS();
 
-    int fld_cnt = 0;
-    while( SYS->db().at().dataSeek(bd,mod->nodePath()+cfg("BLOCK_SH").getS(),fld_cnt++,c_el) )
+    for( int fld_cnt = 0; SYS->db().at().dataSeek(bd,mod->nodePath()+cfg("BLOCK_SH").getS(),fld_cnt++,c_el); )
     {
 	string id = c_el.cfg("ID").getS();
 	if( !chldPresent(m_bl,id) )
@@ -274,7 +284,6 @@ void Contr::load_( )
 	    ((TConfig &)blkAt(id).at()) = c_el;
 	}
 	blkAt(id).at().load();
-	c_el.cfg("ID").setS("");
     }
 }
 
@@ -288,7 +297,7 @@ void Contr::enable_( )
 	try{ blkAt(lst[i_l]).at().setEnable(true); }
 	catch(TError err)
 	{
-	    mess_warning(err.cat.c_str(),"%s",err.mess.c_str()); 
+	    mess_warning(err.cat.c_str(),"%s",err.mess.c_str());
 	    mess_warning(nodePath().c_str(),_("Enable block <%s> error."),lst[i_l].c_str());
 	}
 }
@@ -303,7 +312,7 @@ void Contr::disable_( )
 	try{ blkAt(lst[i_l]).at().setEnable(false); }
 	    catch(TError err)
 	    {
-		mess_warning(err.cat.c_str(),"%s",err.mess.c_str()); 
+		mess_warning(err.cat.c_str(),"%s",err.mess.c_str());
 		mess_warning(nodePath().c_str(),_("Enable block <%s> error."),lst[i_l].c_str());
 	    }
 }
@@ -318,7 +327,7 @@ void Contr::start_( )
 	try{ blkAt(lst[i_l]).at().setProcess(true); }
 	catch(TError err)
 	{
-	    mess_warning(err.cat.c_str(),"%s",err.mess.c_str()); 
+	    mess_warning(err.cat.c_str(),"%s",err.mess.c_str());
 	    mess_warning(nodePath().c_str(),_("Process block <%s> error."),lst[i_l].c_str());
 	}
 
@@ -337,7 +346,7 @@ void Contr::start_( )
 	pthread_create(&calcPthr,&pthr_attr,Contr::Task,this);
 	pthread_attr_destroy(&pthr_attr);
 	if( TSYS::eventWait(prc_st, true, nodePath()+"start",5) )
-	    throw TError(nodePath().c_str(),_("Acquisition task no started!"));
+	    throw TError(nodePath().c_str(),_("Acquisition task is not started!"));
     }
 }
 
@@ -349,7 +358,7 @@ void Contr::stop_( )
 	endrun_req = true;
 	pthread_kill( calcPthr, SIGALRM );
 	if( TSYS::eventWait(prc_st,false,nodePath()+"stop",5) )
-	    throw TError(nodePath().c_str(),_("Acquisition task no stoped!"));
+	    throw TError(nodePath().c_str(),_("Acquisition task is not stopped!"));
 	pthread_join( calcPthr, NULL );
     }
 
@@ -363,10 +372,12 @@ void Contr::stop_( )
 
 void *Contr::Task( void *icontr )
 {
-    long long work_tm, last_tm = 0;
-    struct timespec get_tm;
     Contr &cntr = *(Contr *)icontr;
 
+#if OSC_DEBUG >= 2
+    mess_debug(cntr.nodePath().c_str(),_("Thread <%u> is started. TID: %ld"),pthread_self(),(long int)syscall(224));
+#endif
+
     cntr.endrun_req = false;
     cntr.prc_st = true;
 
@@ -376,11 +387,11 @@ void *Contr::Task( void *icontr )
     while(true)
     {
 	//Check calk time
-	unsigned long long t_cnt = SYS->shrtCnt();
+	long long t_cnt = TSYS::curTime();
 
 	cntr.hd_res.resRequestR( );
-	for(unsigned i_it = 0; i_it < cntr.m_iter; i_it++)
-	    for(unsigned i_blk = 0; i_blk < cntr.clc_blks.size(); i_blk++)
+	for( unsigned i_it = 0; i_it < cntr.m_iter && !cntr.redntUse(); i_it++ )
+	    for( unsigned i_blk = 0; i_blk < cntr.clc_blks.size(); i_blk++ )
 	    {
 		try{ cntr.clc_blks[i_blk].at().calc(is_start,is_stop); }
 		catch(TError err)
@@ -389,25 +400,19 @@ void *Contr::Task( void *icontr )
 		    string blck = cntr.clc_blks[i_blk].at().id();
 		    mess_err(cntr.nodePath().c_str(),_("Block <%s> calc error."),blck.c_str());
 		    if( cntr.clc_blks[i_blk].at().errCnt() < 10 ) continue;
-		    cntr.hd_res.resReleaseR( );
-		    mess_err(cntr.nodePath().c_str(),_("Block <%s> stoped."),blck.c_str());
+		    cntr.hd_res.resRelease( );
+		    mess_err(cntr.nodePath().c_str(),_("Block <%s> is stoped."),blck.c_str());
 		    cntr.blkAt(blck).at().setProcess(false);
 		    cntr.hd_res.resRequestR( );
 		}
 	    }
-	cntr.hd_res.resReleaseR( );
+	cntr.hd_res.resRelease( );
 
-	cntr.tm_calc = 1.0e6*((double)(SYS->shrtCnt()-t_cnt))/((double)SYS->sysClk());
+	cntr.tm_calc = TSYS::curTime()-t_cnt;
 
 	if(is_stop) break;
 
-	//Calc next work time and sleep
-	clock_gettime(CLOCK_REALTIME,&get_tm);
-	work_tm = (((long long)get_tm.tv_sec*1000000000+get_tm.tv_nsec)/((long long)cntr.period()*1000000) + 1)*(long long)cntr.period()*1000000;
-	if(work_tm == last_tm)	work_tm+=(long long)cntr.period()*1000000;	//Fix early call
-	last_tm = work_tm;
-	get_tm.tv_sec = work_tm/1000000000; get_tm.tv_nsec = work_tm%1000000000;
-	clock_nanosleep(CLOCK_REALTIME,TIMER_ABSTIME,&get_tm,NULL);
+	TSYS::taskSleep((long long)cntr.period()*1000000);
 
 	if(cntr.endrun_req) is_stop = true;
 	is_start = false;
@@ -418,6 +423,35 @@ void *Contr::Task( void *icontr )
     return NULL;
 }
 
+void Contr::redntDataUpdate( bool firstArchiveSync )
+{
+    TController::redntDataUpdate(firstArchiveSync);
+
+    vector<string> bls; blkList(bls);
+
+    //> Request for template's attributes values
+    XMLNode req("CntrReqs"); req.setAttr("path",nodePath(0,true));
+    for( int i_b = 0; i_b < bls.size(); i_b++ )
+    {
+	if( !blkAt(bls[i_b]).at().enable( ) ) continue;
+	req.childAdd("get")->setAttr("path","/blk_"+bls[i_b]+"/%2fserv%2fattr");
+    }
+
+    //> Send request to first active station for this controller
+    try{ owner().owner().rdStRequest(workId(),req); }
+    catch(TError err) { return; }
+
+    //> Redirect respond to local parameters
+    req.setAttr("path","/");
+    for( int i_b = 0; i_b < req.childSize(); i_b++ )
+    {
+	if( atoi(req.childGet(i_b)->attr("err").c_str()) ) { req.childDel(i_b--); continue; }
+	req.childGet(i_b)->setName("set");
+    }
+    cntrCmd(&req);
+}
+
+
 TParamContr *Contr::ParamAttach( const string &name, int type )
 {
     return new Prm(name,&owner().tpPrmAt(type));
@@ -433,14 +467,12 @@ void Contr::blkProc( const string & id, bool val )
 {
     unsigned i_blk;
 
-    ResAlloc res(hd_res,true); 
+    ResAlloc res(hd_res,true);
     for( i_blk = 0; i_blk < clc_blks.size(); i_blk++)
 	if( clc_blks[i_blk].at().id() == id ) break;
 
-    if( val && i_blk >= clc_blks.size() )
-	clc_blks.push_back(blkAt(id));
-    if( !val && i_blk < clc_blks.size()	)
-	clc_blks.erase(clc_blks.begin()+i_blk);
+    if( val && i_blk >= clc_blks.size() ) clc_blks.push_back(blkAt(id));
+    if( !val && i_blk < clc_blks.size() ) clc_blks.erase(clc_blks.begin()+i_blk);
 }
 
 void Contr::cntrCmdProc( XMLNode *opt )
@@ -449,19 +481,17 @@ void Contr::cntrCmdProc( XMLNode *opt )
     if( opt->name() == "info" )
     {
 	TController::cntrCmdProc(opt);
-	ctrMkNode("grp",opt,-1,"/br/blk_",_("Block"),0664,"root","root",1,"idm","1");
+	ctrMkNode("grp",opt,-1,"/br/blk_",_("Block"),0664,"root","root",2,"idm","1","idSz","20");
 	if(ctrMkNode("area",opt,-1,"/scheme",_("Blocks scheme")))
 	{
-	    ctrMkNode("fld",opt,-1,"/scheme/ctm",_("Calk time (usek)"),0444,"root","root",1,"tp","real");
-	    ctrMkNode("list",opt,-1,"/scheme/sch",_("Blocks"),0664,"root","root",4,"tp","br","idm","1","s_com","add,del","br_pref","blk_");
 	    ctrMkNode("fld",opt,-1,"/scheme/nmb",_("Number"),0444,"root","root",1,"tp","str");
+	    ctrMkNode("list",opt,-1,"/scheme/sch",_("Blocks"),0664,"root","root",5,"tp","br","idm","1","s_com","add,del","br_pref","blk_","idSz","20");
 	}
 	return;
     }
     //Process command to page
     string a_path = opt->attr("path");
-    if( a_path == "/scheme/ctm" && ctrChkNode(opt) )	opt->setText(TSYS::real2str(tm_calc,6));
-    else if( a_path == "/br/blk_" || a_path == "/scheme/sch" )
+    if( a_path == "/br/blk_" || a_path == "/scheme/sch" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
 	{
@@ -472,8 +502,8 @@ void Contr::cntrCmdProc( XMLNode *opt )
 	}
 	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )
 	{
-	    blkAdd(opt->attr("id"));
-	    blkAt(opt->attr("id")).at().setName(opt->text());
+	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::ID);
+	    blkAdd(vid); blkAt(vid).at().setName(opt->text());
 	}
 	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )	chldDel(m_bl,opt->attr("id"),-1,1);
     }
@@ -487,9 +517,7 @@ void Contr::cntrCmdProc( XMLNode *opt )
 	    if( blkAt(lst[i_b]).at().enable( ) )	enCnt++;
 	    if( blkAt(lst[i_b]).at().process( ) )	prcCnt++;
 	}
-	char buf[100];
-	snprintf(buf,sizeof(buf),_("All: %d; Enabled: %d; Process: %d"),lst.size(),enCnt,prcCnt);
-	opt->setText(buf);
+	opt->setText(TSYS::strMess(_("All: %d; Enabled: %d; Process: %d"),lst.size(),enCnt,prcCnt));
     }
     else TController::cntrCmdProc(opt);
 }
@@ -515,11 +543,13 @@ void Prm::postEnable( int flag )
     if(!vlElemPresent(&v_el))	vlElemAtt(&v_el);
 }
 
+Contr &Prm::owner( )	{ return (Contr&)TParamContr::owner( ); }
+
 void Prm::enable()
 {
     if( enableStat() )  return;
 
-    //- Check and delete no used fields -
+    //> Check and delete no used fields
     for(int i_fld = 0; i_fld < v_el.fldSize(); i_fld++)
     {
 	string fel;
@@ -533,7 +563,7 @@ void Prm::enable()
 	}
     }
 
-    //- Init elements -
+    //> Init elements
     vector<string> pls;
     AutoHD<Block> blk;
     int io;
@@ -552,18 +582,19 @@ void Prm::enable()
 
 	unsigned	flg = TVal::DirWrite|TVal::DirRead;
 	TFld::Type	tp  = TFld::String;
+	string		def;
 	switch( blk.at().ioType(io) )
 	{
-	    case IO::String:	tp = TFld::String;	break;
-	    case IO::Integer:	tp = TFld::Integer;	break;
-	    case IO::Real:	tp = TFld::Real;	break;
-	    case IO::Boolean:	tp = TFld::Boolean;	break;
+	    case IO::String:	tp = TFld::String; def = EVAL_STR; break;
+	    case IO::Integer:	tp = TFld::Integer; def = TSYS::int2str(EVAL_INT);	break;
+	    case IO::Real:	tp = TFld::Real; def = TSYS::real2str(EVAL_REAL);	break;
+	    case IO::Boolean:	tp = TFld::Boolean; def = TSYS::int2str(EVAL_BOOL);	break;
 	}
 	if( !v_el.fldPresent(aid) || v_el.fldAt(v_el.fldId(aid)).type() != tp ||
 	    v_el.fldAt(v_el.fldId(aid)).flg() != flg )
 	{
 	    if(v_el.fldPresent(aid)) v_el.fldDel(v_el.fldId(aid));
-	    v_el.fldAdd( new TFld(aid.c_str(),"",tp,flg) );
+	    v_el.fldAdd( new TFld(aid.c_str(),"",tp,flg,"",def.c_str()) );
 	}
 	int el_id = v_el.fldId(aid);
 	v_el.fldAt(el_id).setDescr( anm.empty() ? blk.at().func()->io(io)->name() : anm );
@@ -572,7 +603,7 @@ void Prm::enable()
 	pls.push_back(aid);
     }
 
-    //- Check and delete no used attrs -
+    //> Check and delete no used attrs
     for( int i_fld = 0; i_fld < v_el.fldSize(); i_fld++ )
     {
 	int i_p;
@@ -594,9 +625,21 @@ void Prm::disable()
     TParamContr::disable();
 }
 
-void Prm::vlSet( TVal &val )
+void Prm::vlSet( TVal &val, const TVariant &pvl )
 {
     if( !enableStat() )	return;
+
+    //> Send to active reserve station
+    if( owner().redntUse( ) )
+    {
+	if( val.getS() == pvl.getS() ) return;
+	XMLNode req("set");
+	req.setAttr("path",nodePath(0,true)+"/%2fserv%2fattr")->childAdd("el")->setAttr("id",val.name())->setText(val.getS());
+	SYS->daq().at().rdStRequest(owner().workId(),req);
+	return;
+    }
+
+    //> Direct write
     try
     {
 	AutoHD<Block> blk = ((Contr &)owner()).blkAt(TSYS::strSepParse(val.fld().reserve(),0,'.'));
@@ -618,26 +661,32 @@ void Prm::vlGet( TVal &val )
     if( val.name() == "err" )
     {
 	if( !owner().startStat() )
-	    val.setS(_("2:Controller stoped"),0,true);
+	    val.setS(_("2:Controller is stoped"),0,true);
 	else if( !enableStat() )
-	    val.setS(_("1:Parameter disabled"),0,true);
+	    val.setS(_("1:Parameter is disabled"),0,true);
 	else val.setS("0",0,true);
 	return;
     }
 
+    if( owner().redntUse( ) ) return;
+
     try
     {
-	if( !enableStat() ) return;
+	//if( !enableStat() ) return;
 	AutoHD<Block> blk = ((Contr &)owner()).blkAt(TSYS::strSepParse(val.fld().reserve(),0,'.'));
 	int io_id = blk.at().ioId(TSYS::strSepParse(val.fld().reserve(),1,'.'));
 	if( io_id < 0 )	disable();
 	else
 	    switch(val.fld().type())
 	    {
-		case TFld::String:	val.setS( enableStat() ? blk.at().getS(io_id) : EVAL_STR, 0, true );	break;
-		case TFld::Integer:	val.setI( enableStat() ? blk.at().getI(io_id) : EVAL_INT, 0, true );	break;
-		case TFld::Real:	val.setR( enableStat() ? blk.at().getR(io_id) : EVAL_REAL, 0, true );	break;
-		case TFld::Boolean:	val.setB( enableStat() ? blk.at().getB(io_id) : EVAL_BOOL, 0, true );	break;
+		case TFld::String:
+		    val.setS( (enableStat()&&owner().startStat()) ? blk.at().getS(io_id) : EVAL_STR, 0, true );	break;
+		case TFld::Integer:
+		    val.setI( (enableStat()&&owner().startStat()) ? blk.at().getI(io_id) : EVAL_INT, 0, true );	break;
+		case TFld::Real:
+		    val.setR( (enableStat()&&owner().startStat()) ? blk.at().getR(io_id) : EVAL_REAL, 0, true );break;
+		case TFld::Boolean:
+		    val.setB( (enableStat()&&owner().startStat()) ? blk.at().getB(io_id) : EVAL_BOOL, 0, true );break;
 	    }
     }catch(TError err) { disable(); }
 }
@@ -658,13 +707,13 @@ void Prm::cntrCmdProc( XMLNode *opt )
     {
 	TParamContr::cntrCmdProc(opt);
 	ctrMkNode("fld",opt,-1,"/prm/cfg/IO",cfg("IO").fld().descr(),0664,"root","root",1,
-	    "help",_("Attributes configuration list. List writed by lines in format: [<blk>.<blk_io>:<aid>:<anm>]\n"
+	    "help",_("Attributes configuration list. List must be written by lines in format: [<blk>.<blk_io>:<aid>:<anm>]\n"
 	    "Where:\n"
 	    "  blk - block identifier from block's scheme;\n"
 	    "  blk_io - block's parameter from block's scheme;\n"
 	    "  aid - created attribute identifier;\n"
 	    "  anm - created attribute name.\n"
-	    "If 'aid' or 'anm' no set then it will be generated from selected block's parameter."));
+	    "If 'aid' or 'anm' are not set they will be generated from selected block's parameter."));
 	return;
     }
     //Process command to page
