@@ -55,7 +55,7 @@ class Prm : public TParamContr
 	void enable( );
 	void disable( );
 
-	Contr &owner( )		{ return (Contr&)TParamContr::owner( ); }
+	Contr &owner( );
 
     protected:
 	//Protected methods
@@ -65,7 +65,7 @@ class Prm : public TParamContr
         //Private methods
 	void postEnable( int flag );
 
-	void vlSet( TVal &val );
+	void vlSet( TVal &val, const TVariant &pvl );
 	void vlGet( TVal &val );
 	void vlArchMake( TVal &val );
 
@@ -89,11 +89,13 @@ class Contr: public TController
 
 	TCntrNode &operator=( TCntrNode &node );
 
+	string getStatus( );
+
 	TParamContr *ParamAttach( const string &name, int type );
 	int period( )				{ return vmax(1,m_per); }
 	int iterate( )				{ return m_iter; }
 
-	//- Scheme's functions -
+	//> Scheme's functions
 	void blkList( vector<string> &ls )	{ chldList(m_bl,ls); }
 	bool blkPresent( const string &id )	{ return chldPresent(m_bl,id); }
 	void blkAdd( const string &id );
@@ -102,7 +104,9 @@ class Contr: public TController
 
 	Res &res( )				{ return hd_res; }
 
-	TipContr &owner( )			{ return (TipContr&)TController::owner( ); }
+	TipContr &owner( );
+
+	void redntDataUpdate( bool firstArchiveSync = false );
 
     protected:
 	//Protected methods
@@ -113,7 +117,7 @@ class Contr: public TController
 	void stop_( );
 	void cntrCmdProc( XMLNode *opt );	//Control interface command process
 
-	//- Process stat -
+	//> Process stat
 	void blkProc( const string & id, bool val );
 
 	void postDisable(int flag);
@@ -163,6 +167,8 @@ class TipContr: public TTipDAQ
 	void cntrCmdProc( XMLNode *opt );       //Control interface command process
 	void load_( );
 
+	bool redntAllow( )	{ return true; }
+
     private:
 	//Private methods
 	void postEnable( int flag );
