@@ -39,7 +39,7 @@
 #define MOD_NAME	"Diamond DA boards"
 #define MOD_TYPE	"DAQ"
 #define VER_TYPE	VER_CNTR
-#define VERSION		"1.1.0"
+#define VERSION		"1.2.0"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Allow access to Diamond systems DA boards. Include support of Athena board."
 #define LICENSE		"GPL"
@@ -66,7 +66,7 @@ extern "C"
 using namespace Diamond;
 
 //*************************************************
-//* TTpContr                                      * 
+//* TTpContr                                      *
 //*************************************************
 TTpContr::TTpContr( string name ) :
     m_init(false), elem_ai("AI"), elem_ao("AO"), elem_di("DI"), elem_do("DO")
@@ -81,11 +81,6 @@ TTpContr::TTpContr( string name ) :
     mSource	= name;
 
     mod		= this;
-
-    //- Init DSCAD -
-    if( dscInit( DSC_VERSION ) != DE_NONE )
-	mess_err(mod->nodePath().c_str(),_("dscInit error."));
-    else m_init = true;
 }
 
 TTpContr::~TTpContr()
@@ -96,7 +91,12 @@ TTpContr::~TTpContr()
 
 void TTpContr::postEnable( int flag )
 {
-    TModule::postEnable( flag );
+    TTipDAQ::postEnable( flag );
+
+    //- Init DSCAD -
+    if( dscInit( DSC_VERSION ) != DE_NONE )
+	mess_err(mod->nodePath().c_str(),_("dscInit error."));
+    else m_init = true;
 
     //- Controler's bd structure -
     fldAdd( new TFld("BOARD",_("Diamond system board"),TFld::Integer,TFld::Selected,"2","25",
@@ -173,6 +173,8 @@ TMdContr::~TMdContr()
 
 }
 
+TTpContr &TMdContr::owner( )	{ return (TTpContr&)TController::owner(); }
+
 TParamContr *TMdContr::ParamAttach( const string &name, int type )
 {
     return new TMdPrm(name,&owner().tpPrmAt(type));
@@ -189,7 +191,7 @@ void TMdContr::start_( )
 {
     //- Check inited of Diamond API -
     if( !mod->drvInitOk() )
-	throw TError(nodePath().c_str(),_("DSC driver not inited!"));
+	throw TError(nodePath().c_str(),_("DSC driver is not initialized!"));
 
     //- DSC strucures init -
     ERRPARAMS errorParams;
@@ -233,7 +235,7 @@ void TMdContr::start_( )
 	pthread_create(&ad_dsc_pthr,&pthr_attr,AD_DSCTask,this);
 	pthread_attr_destroy(&pthr_attr);
 	if( TSYS::eventWait(ad_dsc_st, true, nodePath()+"addsc_task_start",5) )
-	    throw TError(nodePath().c_str(),_("AD DSC driver task no started!"));
+	    throw TError(nodePath().c_str(),_("AD DSC driver task is not started!"));
     }
 }
 
@@ -242,11 +244,11 @@ void TMdContr::stop_( )
     //- Close AI DAQ task -
     if( ad_dsc_st )
     {
-        endrun_req_ad_dsc = true;
-        pthread_kill( ad_dsc_pthr, SIGALRM );
-        if( TSYS::eventWait(ad_dsc_st,false,nodePath()+"addsc_task_stop",5) )
-            throw TError(nodePath().c_str(),_("AD DSC task no stoped!"));
-        pthread_join( ad_dsc_pthr, NULL );
+	endrun_req_ad_dsc = true;
+	pthread_kill( ad_dsc_pthr, SIGALRM );
+	if( TSYS::eventWait(ad_dsc_st,false,nodePath()+"addsc_task_stop",5) )
+	    throw TError(nodePath().c_str(),_("AD DSC task is not stopped!"));
+	pthread_join( ad_dsc_pthr, NULL );
     }
 
     if( !dataEmul() )	dscFreeBoard(dscb);
@@ -292,6 +294,10 @@ void *TMdContr::AD_DSCTask( void *param )
     cntr.endrun_req_ad_dsc = false;
     cntr.ad_dsc_st = true;
 
+#if OSC_DEBUG >= 2
+    mess_debug(cntr.nodePath().c_str(),_("Thread <%u> is started. TID: %ld"),pthread_self(),(long int)syscall(224));
+#endif
+
     //- DSC strucures init -
     BYTE result;
     ERRPARAMS errorParams;
@@ -320,8 +326,8 @@ void *TMdContr::AD_DSCTask( void *param )
 	}
 	//- Generic data -
 	int convRate = 2*(cntr.cfg("ADCONVRATE").getI()/2);
-	
-	if(!cntr.dataEmul())
+
+	if( !cntr.dataEmul() )
 	{
 	    if(ai_prm.size())
 	    {
@@ -344,10 +350,9 @@ void *TMdContr::AD_DSCTask( void *param )
 		dscaioint.dump_threshold = dscaioint.num_conversions/2;
 		dscaioint.fifo_enab = 1;
 		dscaioint.fifo_depth = dscaioint.num_conversions;
-		while(dscaioint.fifo_depth>46)
+		while( dscaioint.fifo_depth>46 )
 		{
-		    if( dscaioint.fifo_depth > 2*46 )
-			dscaioint.fifo_depth/=10;
+		    if( dscaioint.fifo_depth > 2*46 ) dscaioint.fifo_depth/=10;
 		    else dscaioint.fifo_depth/=2;
 		}
 		dscaioint.sample_values = (DSCSAMPLE*)malloc( sizeof(DSCSAMPLE) * dscaioint.num_conversions );
@@ -362,16 +367,23 @@ void *TMdContr::AD_DSCTask( void *param )
 		dscs.op_type = OP_TYPE_INT;
 	    }
 
-	    while(!cntr.endrun_req_ad_dsc)
+	    while( !cntr.endrun_req_ad_dsc )
 	    {
 		dscGetStatus(cntr.dscb, &dscs);
 		if( prev_trans < 0 ) prev_trans = dscs.transfers;
-		if( dscs.transfers != prev_trans )
+		if( dscs.transfers != prev_trans && !cntr.redntUse() )
 		{
-		    if(!vtm)	vtm = SYS->curTime()-1000000;
+		    //> Init current time and check for current time correction
+		    long long cTm = TSYS::curTime();
+		    if( !vtm || fabs((vtm+1000000)-cTm) > 1e6 )
+		    {
+			if( vtm ) mess_warning(cntr.nodePath().c_str(),_("DSC card's counter run from system time is corrected."));
+			vtm = cTm-1000000;
+		    }
+
 		    int v_a_step;
 		    int p_cnt = p_end-p_beg+1;
-		    for(int i_p = 0; i_p < ai_prm.size(); i_p++ )
+		    for( int i_p = 0; i_p < ai_prm.size(); i_p++ )
 		    {
 			if( !cntr.present(ai_prm[i_p]) )	continue;
 			AutoHD<TMdPrm> prm = cntr.at(ai_prm[i_p]);
@@ -381,7 +393,7 @@ void *TMdContr::AD_DSCTask( void *param )
 			int voff = (dscs.transfers+dscaioint.dump_threshold)%dscaioint.num_conversions;
 			//- Get code -
 			AutoHD<TVal> val = prm.at().vlAt("code");
-			if(!val.at().arch().freeStat() && val.at().arch().at().srcMode() == TVArchive::PassiveAttr)
+			if( !val.at().arch().freeStat() && val.at().arch().at().srcMode() == TVArchive::PassiveAttr )
 			{
 			    v_a_step = vmax(1,val.at().arch().at().period()*(int)dscaioint.conversion_rate/1000000);
 			    for( int i_smpl = 0; i_smpl < dscaioint.conversion_rate; i_smpl+=v_a_step )
@@ -401,24 +413,24 @@ void *TMdContr::AD_DSCTask( void *param )
 			//for( int i_smpl = 0; i_smpl < dscaioint.conversion_rate; i_smpl++ )
 			//	printf("Canal %d(%d): %xh\n",prm.at().cnl(),voff+prm.at().cnl()-p_beg+i_smpl*(p_end-p_beg+1),dscaioint.sample_values[voff+prm.at().cnl()-p_beg+i_smpl*(p_end-p_beg+1)]);
 		    }
-		    prev_trans = dscs.transfers;
-		    vtm+=1000000;
+		    vtm += 1000000;
 		}
+		prev_trans = dscs.transfers;
 		clock_nanosleep(CLOCK_REALTIME,0,&get_tm,NULL);
 	    }
 	}
 	else
-	    while(!cntr.endrun_req_ad_dsc)
+	    while( !cntr.endrun_req_ad_dsc )
 	    {
-		if(!vtm)	vtm = SYS->curTime()-1000000;
+		if( !vtm ) vtm = SYS->curTime()-1000000;
 		int v_a_step;
 		int p_cnt = p_end-p_beg+1;
-		for(int i_p = 0; i_p < ai_prm.size(); i_p++ )
+		for( int i_p = 0; i_p < ai_prm.size() && !cntr.redntUse(); i_p++ )
 		{
 		    if( !cntr.present(ai_prm[i_p]) )	continue;
 		    AutoHD<TMdPrm> prm = cntr.at(ai_prm[i_p]);
 		    int p_cnl = prm.at().cnl();
-		    if(prm.at().type() != TMdPrm::AI || p_cnl < p_beg || p_cnl > p_end || !prm.at().enableStat() )
+		    if( prm.at().type() != TMdPrm::AI || p_cnl < p_beg || p_cnl > p_end || !prm.at().enableStat() )
 		        continue;
 		    //- Get code -
 		    AutoHD<TVal> val = prm.at().vlAt("code");
@@ -527,6 +539,8 @@ void TMdPrm::postEnable( int flag )
     else if( TParamContr::type().name == "d_prm" )	setType(DI);
 }
 
+TMdContr &TMdPrm::owner( )	{ return (TMdContr&)TParamContr::owner(); }
+
 void TMdPrm::load_( )
 {
     cfgViewAll(true);
@@ -597,10 +611,21 @@ bool TMdPrm::cfgChange( TCfg &i_cfg )
     return true;
 }
 
-void TMdPrm::vlSet( TVal &val )
+void TMdPrm::vlSet( TVal &val, const TVariant &pvl )
 {
     if( !owner().startStat() || !enableStat() )	return;
 
+    //> Send to active reserve station
+    if( owner().redntUse( ) )
+    {
+	if( val.getS() == pvl.getS() ) return;
+	XMLNode req("set");
+	req.setAttr("path",nodePath(0,true)+"/%2fserv%2fattr")->childAdd("el")->setAttr("id",val.name())->setText(val.getS());
+	SYS->daq().at().rdStRequest(owner().workId(),req);
+	return;
+    }
+
+    //> Direct write
     switch(type())
     {
 	case AO:
@@ -621,7 +646,7 @@ void TMdPrm::vlSet( TVal &val )
 		dscGetLastError(&errorParams);
 		mess_err(nodePath().c_str(),_("dscDAConvert error: %s %s"),dscGetErrorString(errorParams.ErrCode),errorParams.errstring );
 	    }
-	    owner().ao_res.resReleaseW( );
+	    owner().ao_res.resRelease( );
 	    break;
 	}
 	case DO:
@@ -635,7 +660,7 @@ void TMdPrm::vlSet( TVal &val )
 		dscGetLastError(&errorParams);
 		mess_err(nodePath().c_str(),_("dscDIOOutputBit error: %s %s"),dscGetErrorString(errorParams.ErrCode),errorParams.errstring );
 	    }
-	    owner().dio_res.resReleaseW( );
+	    owner().dio_res.resRelease( );
 	}
     }
 }
@@ -646,14 +671,16 @@ void TMdPrm::vlGet( TVal &val )
     if( aid == 0 )
     {
 	if( !owner().startStat() )
-	    val.setS(_("2:Controller stoped"),0,true);
+	    val.setS(_("2:Controller is stoped"),0,true);
 	else if( !enableStat() )
-	    val.setS(_("1:Parameter disabled"),0,true);
+	    val.setS(_("1:Parameter is disabled"),0,true);
 	else val.setS("0",0,true);
 	return;
     }
     if( !owner().startStat() || !enableStat() )	{ val.setS(EVAL_STR,0,true); return; }
 
+    if( owner().redntUse( ) ) return;
+
     switch(type())
     {
 	case AI:
@@ -683,7 +710,7 @@ void TMdPrm::vlGet( TVal &val )
 			mess_err(nodePath().c_str(),_("dscADSample error: %s %s"), dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
 		    }
 		    gval = smpl;
-		    owner().ai_res.resReleaseW( );
+		    owner().ai_res.resRelease( );
 		}
 	    }
 	    switch( aid )
@@ -712,7 +739,7 @@ void TMdPrm::vlGet( TVal &val )
 			mess_err(nodePath().c_str(),_("dscDIOInputBit error: %s %s"), dscGetErrorString(errorParams.ErrCode),errorParams.errstring );
 		    }
 		    gval = i_bt;
-		    owner().dio_res.resReleaseW( );
+		    owner().dio_res.resRelease( );
 		}
 	    }
 	    val.setB(gval,0,true);
