@@ -60,13 +60,13 @@ class TMdPrm : public TParamContr
 	int	cnl( )		{ return m_cnl; }
 	void	setType( Type val );
 
-	TMdContr &owner( )	{ return (TMdContr &)TParamContr::owner(); }
+	TMdContr &owner( );
 
     protected:
 	//Methods
 	void load_( );
 	bool cfgChange( TCfg &cfg );
-	void vlSet( TVal &val );
+	void vlSet( TVal &val, const TVariant &pvl );
 	void vlGet( TVal &val );
 	void vlArchMake( TVal &val );
 
@@ -101,7 +101,7 @@ class TMdContr: public TController
 
 	TParamContr *ParamAttach( const string &name, int type );
 
-	TTpContr &owner( )	{ return (TTpContr&)TController::owner(); }
+	TTpContr &owner( );
 
     protected:
 	//Methods
@@ -150,6 +150,10 @@ class TTpContr: public TTipDAQ
 	TElem &elemDI( )	{ return elem_di; }
 	TElem &elemDO( )	{ return elem_do; }
 
+    protected:
+	//Attribute
+	bool redntAllow( )	{ return true; }
+
     private:
 	//Attributes
 	bool	m_init;
