@@ -34,14 +34,14 @@ Func *JavaLikeCalc::p_fnc;
 //*************************************************
 //* Func: Function                                *
 //*************************************************
-Func::Func( const char *id, const char *name ) :
-    TConfig(&mod->elFnc()), TFunction(id), parse_res(mod->parseRes( )),
-    m_name(cfg("NAME").getSd()), m_descr(cfg("DESCR").getSd()),
+Func::Func( const char *iid, const char *name ) :
+    TConfig(&mod->elFnc()), TFunction(iid), parse_res(mod->parseRes( )),
+    mName(cfg("NAME").getSd()), mDescr(cfg("DESCR").getSd()),
     max_calc_tm(cfg("MAXCALCTM").getId()),prg_src(cfg("FORMULA").getSd())
 {
-    cfg("ID").setS(id);
-    m_name = name;
-    if( !m_name.size() ) m_name = id;
+    cfg("ID").setS(id());
+    mName = name;
+    if( !mName.size() ) mName = id();
 }
 
 Func::~Func( )
@@ -56,7 +56,7 @@ void Func::postEnable( int flag )
 
 void Func::preDisable( int flag )
 {
-    if( m_tval ) { delete m_tval; m_tval = NULL; }
+    if( mTVal ) { delete mTVal; mTVal = NULL; }
 }
 
 void Func::postDisable( int flag )
@@ -72,12 +72,12 @@ void Func::postDisable( int flag )
 
 Lib &Func::owner( )
 {
-    return *((Lib*)nodePrev()); 
+    return *((Lib*)nodePrev());
 }
 
 string Func::name( )
 {
-    return m_name.size()?m_name:id();
+    return mName.size()?mName:id();
 }
 
 TCntrNode &Func::operator=( TCntrNode &node )
@@ -89,7 +89,7 @@ TCntrNode &Func::operator=( TCntrNode &node )
     *(TFunction *)this = *(TFunction*)src_n;
 
     //- Set to DB -
-    cfg("ID").setS(m_id);
+    cfg("ID").setS(mId);
 
     if( src_n->startStat( ) && !startStat( ) )	setStart( true );
 }
@@ -100,18 +100,18 @@ Func &Func::operator=(Func &func)
     *(TFunction *)this = (TFunction&)func;
 
     //- Set to DB -
-    cfg("ID").setS(m_id);
+    cfg("ID").setS(mId);
 }
 
 void Func::setName( const string &nm )
 {
-    m_name = nm;
+    mName = nm;
     if(!owner().DB().empty()) modif();
 }
 
 void Func::setDescr( const string &dscr )
 {
-    m_descr = dscr;
+    mDescr = dscr;
     if(!owner().DB().empty()) modif();
 }
 
@@ -129,7 +129,7 @@ void Func::setProg( const string &prg )
 
 void Func::load_( )
 {
-    if( owner().DB().empty() )	return;
+    if( owner().DB().empty() || (!SYS->chkSelDB(owner().DB())) )	return;
 
     SYS->db().at().dataGet(owner().fullDB(),mod->nodePath()+owner().tbl(),*this);
 
@@ -140,14 +140,13 @@ void Func::loadIO( )
 {
     TConfig cfg(&mod->elFncIO());
 
-    int fld_cnt=0;
     vector<string> u_pos;
-    cfg.cfg("F_ID").setS(id());
-    while( SYS->db().at().dataSeek(owner().fullDB()+"_io",mod->nodePath()+owner().tbl()+"_io",fld_cnt++,cfg) )
+    cfg.cfg("F_ID").setS(id(),true);
+    for( int fld_cnt=0; SYS->db().at().dataSeek(owner().fullDB()+"_io",mod->nodePath()+owner().tbl()+"_io",fld_cnt++,cfg); )
     {
 	string sid = cfg.cfg("ID").getS();
 
-	//- Position storing -
+	//> Position storing
 	int pos = cfg.cfg("POS").getI();
 	while( u_pos.size() <= pos )    u_pos.push_back("");
 	u_pos[pos] = sid;
@@ -155,17 +154,15 @@ void Func::loadIO( )
 	if( ioId(sid) < 0 )
 	    ioIns( new IO(sid.c_str(),"",IO::Real,IO::Default), pos );
 
-	//- Set values -
+	//> Set values
 	int id = ioId(sid);
 	io(id)->setName(cfg.cfg("NAME").getS());
 	io(id)->setType((IO::Type)cfg.cfg("TYPE").getI());
 	io(id)->setFlg(cfg.cfg("MODE").getI());
 	io(id)->setDef(cfg.cfg("DEF").getS());
 	io(id)->setHide(cfg.cfg("HIDE").getB());
-
-	cfg.cfg("ID").setS("");
     }
-    //- Position fixing -
+    //> Position fixing
     for( int i_p = 0; i_p < u_pos.size(); i_p++ )
     {
 	if( u_pos[i_p].empty() ) continue;
@@ -192,50 +189,46 @@ void Func::saveIO( )
     string io_bd = owner().fullDB()+"_io";
     string io_cfgpath = mod->nodePath()+owner().tbl()+"_io/";
 
-    //- Save allow IO -
-    cfg.cfg("F_ID").setS(id());
+    //> Save allow IO
+    cfg.cfg("F_ID").setS(id(),true);
     for( int i_io = 0; i_io < ioSize(); i_io++ )
     {
+	if( io(i_io)->flg()&Func::SysAttr ) continue;
 	cfg.cfg("ID").setS(io(i_io)->id());
 	cfg.cfg("NAME").setS(io(i_io)->name());
 	cfg.cfg("TYPE").setI(io(i_io)->type());
 	cfg.cfg("MODE").setI(io(i_io)->flg());
+	cfg.cfg("DEF").setNoTransl(io(i_io)->type()!=IO::String);
 	cfg.cfg("DEF").setS(io(i_io)->def());
 	cfg.cfg("HIDE").setB(io(i_io)->hide());
 	cfg.cfg("POS").setI(i_io);
 	SYS->db().at().dataSet(io_bd,io_cfgpath,cfg);
     }
 
-    //- Clear IO -
-    int fld_cnt=0;
-    cfg.cfg("ID").setS("");
+    //> Clear IO
     cfg.cfgViewAll(false);
-    while( SYS->db().at().dataSeek(io_bd,io_cfgpath,fld_cnt++,cfg ) )
-    {
+    for( int fld_cnt=0; SYS->db().at().dataSeek(io_bd,io_cfgpath,fld_cnt++,cfg ); )
 	if( ioId(cfg.cfg("ID").getS()) < 0 )
 	{
-	    SYS->db().at().dataDel(io_bd,io_cfgpath,cfg);
+	    SYS->db().at().dataDel(io_bd,io_cfgpath,cfg,true);
 	    fld_cnt--;
 	}
-	cfg.cfg("ID").setS("");
-    }
 }
 
 void Func::del( )
 {
     if( !owner().DB().size() )  return;
 
-    SYS->db().at().dataDel(owner().fullDB(),mod->nodePath()+owner().tbl(),*this);
+    SYS->db().at().dataDel(owner().fullDB(),mod->nodePath()+owner().tbl(),*this,true);
 
-    //- Delete io from DB -
+    //> Delete io from DB
     delIO( );
 }
 
 void Func::delIO( )
 {
     TConfig cfg(&mod->elFncIO());
-    cfg.cfg("F_ID").setS(id());
-    cfg.cfg("ID").setS("");
+    cfg.cfg("F_ID").setS(id(),true);
     SYS->db().at().dataDel(owner().fullDB()+"_io",mod->nodePath()+owner().tbl()+"_io",cfg);
 }
 
@@ -245,7 +238,7 @@ void Func::preIOCfgChange()
     if( be_start )
     {
 	setStart(false);
-	if( m_tval ) { delete m_tval; m_tval = NULL; }
+	if( mTVal ) { delete mTVal; mTVal = NULL; }
     }
     TFunction::preIOCfgChange();
 }
@@ -260,13 +253,13 @@ void Func::setStart( bool val )
 {
     if( val )
     {
-	//- Start calc -
+	//> Start calc
 	progCompile( );
 	run_st = true;
     }
     else
     {
-	//- Stop calc -
+	//> Stop calc
 	ResAlloc res(calc_res,true);
 	prg = "";
 	regClear();
@@ -300,7 +293,7 @@ void Func::ioMove( int pos, int to )
     if(!owner().DB().empty()) modif();
 }
 
-void Func::progCompile()
+void Func::progCompile( )
 {
     ResAlloc res(parse_res,true);
     ResAlloc res1(calc_res,true);
@@ -327,71 +320,79 @@ void Func::progCompile()
 
 int Func::funcGet( const string &path )
 {
+    string ns, f_path;
+    //> Check to correct function's path
     try
     {
-	//- Check to correct function's path -
-	if( !dynamic_cast<TFunction *>(&SYS->nodeAt(path,0,'.').at()) )
-	    return -1;
-	//- Get full path -
-	string f_path = SYS->nodeAt(path,0,'.').at().nodePath();
-	for( int i_fnc = 0; i_fnc < m_fncs.size(); i_fnc++ )
-	    if( f_path == m_fncs[i_fnc]->func().at().nodePath() )
-		return i_fnc;
-	m_fncs.push_back(new UFunc(path));
-	return m_fncs.size()-1; 
-    }catch(...){ return -1; }
+	if( dynamic_cast<TFunction*>(&SYS->nodeAt(path,0,'.').at()) )
+	    f_path = SYS->nodeAt(path,0,'.').at().nodePath();
+    }catch(...){ }
+    if( f_path.empty() )
+    {
+	for( int off = 0; !(ns=TSYS::strSepParse(mUsings,0,';',&off)).empty(); )
+	    try{ if( dynamic_cast<TFunction*>(&SYS->nodeAt(ns+"."+path,0,'.').at()) ) break; }
+	    catch(...){ continue; }
+	if( ns.empty() ) return -1;
+	f_path = SYS->nodeAt(ns+"."+path,0,'.').at().nodePath();
+    }
+    //> Search for already registered function
+    for( int i_fnc = 0; i_fnc < mFncs.size(); i_fnc++ )
+	if( f_path == mFncs[i_fnc]->func().at().nodePath() )
+	    return i_fnc;
+    mFncs.push_back(new UFunc(ns.empty()?path:ns+"."+path));
+    return mFncs.size()-1;
 }
 
 void Func::funcClear( )
 {
-    for( int i_fnc = 0; i_fnc < m_fncs.size(); i_fnc++ )
-        delete m_fncs[i_fnc];
-    m_fncs.clear();
+    for( int i_fnc = 0; i_fnc < mFncs.size(); i_fnc++ )
+	delete mFncs[i_fnc];
+    mFncs.clear();
 }
 
 int Func::regNew( bool var )
 {
     //- Get new register -
-    int i_rg = m_regs.size();
+    int i_rg = mRegs.size();
     if( !var )
-	for( i_rg = 0; i_rg < m_regs.size(); i_rg++ )
-	    if( !m_regs[i_rg]->lock() && m_regs[i_rg]->type() == Reg::Free )
+	for( i_rg = 0; i_rg < mRegs.size(); i_rg++ )
+	    if( !mRegs[i_rg]->lock() && mRegs[i_rg]->type() == Reg::Free )
 		break;
-    if( i_rg >= m_regs.size() ) m_regs.push_back(new Reg(i_rg));
+    if( i_rg >= mRegs.size() ) mRegs.push_back(new Reg(i_rg));
     return i_rg;
 }
 
 int Func::regGet( const char *nm )
 {
     //- Check allow registers -
-    for( int i_rg = 0; i_rg < m_regs.size(); i_rg++ )
-	if( m_regs[i_rg]->name() == nm )
+    for( int i_rg = 0; i_rg < mRegs.size(); i_rg++ )
+	if( mRegs[i_rg]->name() == nm )
 	    return i_rg;
     return -1;
 }
 
 void Func::regClear( )
 {
-    for( int i_rg = 0; i_rg < m_regs.size(); i_rg++ )
-        delete m_regs[i_rg];
-    m_regs.clear();
+    for( int i_rg = 0; i_rg < mRegs.size(); i_rg++ )
+        delete mRegs[i_rg];
+    mRegs.clear();
 }
 
 Reg *Func::regTmpNew( )
 {
     int i_rg;
-    for( i_rg = 0; i_rg < m_tmpregs.size(); i_rg++ )
-        if( m_tmpregs[i_rg]->type() == Reg::Free )
-            break;
-    if( i_rg >= m_tmpregs.size() ) m_tmpregs.push_back(new Reg());
-    return m_tmpregs[i_rg];
+    for( i_rg = 0; i_rg < mTmpRegs.size(); i_rg++ )
+	if( mTmpRegs[i_rg]->type() == Reg::Free )
+	    break;
+    if( i_rg >= mTmpRegs.size() ) mTmpRegs.push_back(new Reg());
+    return mTmpRegs[i_rg];
 }
 
 void Func::regTmpClean( )
 {
-    for( int i_rg = 0; i_rg < m_tmpregs.size(); i_rg++ )
-        delete m_tmpregs[i_rg];
-    m_tmpregs.clear();
+    for( int i_rg = 0; i_rg < mTmpRegs.size(); i_rg++ )
+	delete mTmpRegs[i_rg];
+    mTmpRegs.clear();
 }
 
 Reg *Func::cdMvi( Reg *op, bool no_code )
@@ -552,7 +553,7 @@ Reg *Func::cdBinaryOp( Reg::Code cod, Reg *op1, Reg *op2 )
 	    {
 		case Reg::BitShLeft:
 		case Reg::BitShRight:
-		    throw TError(nodePath().c_str(),_("Operation %d no support bool type"),cod);
+		    throw TError(nodePath().c_str(),_("Operation %d don't support bool type"),cod);
 	    }
 	    break;
 	case Reg::String:
@@ -573,7 +574,7 @@ Reg *Func::cdBinaryOp( Reg::Code cod, Reg *op1, Reg *op2 )
 		case Reg::GT:
 		case Reg::LER:
 		case Reg::GER:
-		    throw TError(nodePath().c_str(),_("Operation %d no support string type"),cod);
+		    throw TError(nodePath().c_str(),_("Operation %d don't support string type"),cod);
 	    }
 	    break;
     }
@@ -740,7 +741,7 @@ Reg *Func::cdUnaryOp( Reg::Code cod, Reg *op )
 		case Reg::Not:
 		case Reg::BitNot:
 		case Reg::Neg:
-		    throw TError(nodePath().c_str(),_("Operation %d no support string type"),cod);
+		    throw TError(nodePath().c_str(),_("Operation %d don't support string type"),cod);
 	    }
     }
 
@@ -894,7 +895,7 @@ Reg *Func::cdBldFnc( int f_cod, Reg *prm1, Reg *prm2 )
 
     if( (prm1 && prm1->vType(this) == Reg::String) ||
 	(prm2 && prm2->vType(this) == Reg::String) )
-	throw TError(nodePath().c_str(),_("Buildin functions no support string type"));
+	throw TError(nodePath().c_str(),_("Buildin functions don't support string type"));
     //- Free parameter's registers -
     if( prm1 )	{ prm1 = cdMvi( prm1 ); p1_pos = prm1->pos(); }
     if( prm2 )	{ prm2 = cdMvi( prm2 ); p2_pos = prm2->pos(); }
@@ -925,11 +926,11 @@ Reg *Func::cdExtFnc( int f_id, int p_cnt, bool proc )
 	{ ret_ok=true; break; }
     //- Check IO and parameters count -
     if( p_cnt > funcAt(f_id)->func().at().ioSize()-ret_ok )
-	throw TError(nodePath().c_str(),_("Request more %d parameters for function <%s>"),
+        throw TError(nodePath().c_str(),_("More than %d parameters are specified for function <%s>"),
 	    funcAt(f_id)->func().at().ioSize(),funcAt(f_id)->func().at().id().c_str());	
     //- Check the present return for fuction -
     if( !proc && !ret_ok )
-	throw TError(nodePath().c_str(),_("Request function <%s>, but it not have return IO"),funcAt(f_id)->func().at().id().c_str());
+	throw TError(nodePath().c_str(),_("Function is requested <%s>, but it doesn't have return of IO"),funcAt(f_id)->func().at().id().c_str());
     //- Mvi all parameters -
     for( int i_prm = 0; i_prm < p_cnt; i_prm++ )
 	f_prmst[i_prm] = cdMvi( f_prmst[i_prm] );
@@ -1071,25 +1072,25 @@ void Func::setValB( TValFunc *io, RegW &rg, char val )
 
 void Func::calc( TValFunc *val )
 {
-    calc_res.resRequestR( );
+    ResAlloc res(calc_res,false);
     if( !startStat( ) )	return;
 
-    //- Init list of registers -
-    RegW reg[m_regs.size()];
-    for( int i_rg = 0; i_rg < m_regs.size(); i_rg++ )
+    //> Init list of registers
+    RegW reg[mRegs.size()];
+    for( int i_rg = 0; i_rg < mRegs.size(); i_rg++ )
     {
-	reg[i_rg].setType(m_regs[i_rg]->type());
-	if(reg[i_rg].type() == Reg::Var) 
-	    reg[i_rg].val().io = m_regs[i_rg]->val().io;
+	reg[i_rg].setType(mRegs[i_rg]->type());
+	if(reg[i_rg].type() == Reg::Var)
+	    reg[i_rg].val().io = mRegs[i_rg]->val().io;
 	else if(reg[i_rg].type() == Reg::PrmAttr)
-	    *reg[i_rg].val().p_attr = *m_regs[i_rg]->val().p_attr;
+	    *reg[i_rg].val().p_attr = *mRegs[i_rg]->val().p_attr;
     }
 
-    //- Exec calc -
+    //> Exec calc
     ExecData dt = { 1, 0, 0 };
     try{ exec(val,reg,(const BYTE *)prg.c_str(),dt); }
     catch(TError err){ mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
-    calc_res.resReleaseR( );
+    res.release();
     if( dt.flg&0x07 == 0x01 )	setStart(false);
 }
 
@@ -1097,7 +1098,7 @@ void Func::exec( TValFunc *val, RegW *reg, const BYTE *cprg, ExecData &dt )
 {
     while( !(dt.flg&0x01) )
     {
-	//- Calc time control mechanism -
+	//> Calc time control mechanism
 	if( !((dt.com_cnt++)%1000) )
 	{
 	    if( !dt.start_tm )	dt.start_tm = time(NULL);
@@ -1108,234 +1109,235 @@ void Func::exec( TValFunc *val, RegW *reg, const BYTE *cprg, ExecData &dt )
 		return;
 	    }
 	}
-	//- Calc operation -
+	//> Calc operation
 	switch(*cprg)
 	{
+	    case Reg::EndFull: dt.flg |= 0x01;
 	    case Reg::End: return;
 	    //-- MVI codes --
 	    case Reg::MviB:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Load bool %d to reg %d.\n",*(BYTE *)(cprg+2),*(BYTE *)(cprg+1));
 #endif
 		reg[*(BYTE *)(cprg+1)] = *(char*)(cprg+2);
 		cprg+=3; break;
 	    case Reg::MviI:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Load integer %d to reg %d.\n",*(int *)(cprg+2),*(BYTE *)(cprg+1));
 #endif
 		reg[*(BYTE *)(cprg+1)] = *(int *)(cprg+2);
 		cprg+=2+sizeof(int); break;
 	    case Reg::MviR:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Load real %f to reg %d.\n",*(double *)(cprg+2),*(BYTE *)(cprg+1));
 #endif
 		reg[*(BYTE *)(cprg+1)] = *(double *)(cprg+2);
 		cprg+=2+sizeof(double); break;
 	    case Reg::MviS:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Load string %s(%d) to reg %d.\n",string((char *)cprg+3,*(BYTE *)(cprg+2)).c_str(),*(BYTE *)(cprg+2),*(BYTE *)(cprg+1));
 #endif
 		reg[*(BYTE *)(cprg+1)] = string((char *)cprg+3,*(BYTE *)(cprg+2));
 		cprg+=3+ *(BYTE *)(cprg+2); break;
 	    //-- Assign codes --
 	    case Reg::AssB:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Assign boolean from %d to %d.\n",*(BYTE *)(cprg+2),*(BYTE *)(cprg+1));
 #endif		
 		setValB(val,reg[*(BYTE *)(cprg+1)],getValB(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    case Reg::AssI:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Assign integer from %d to %d.\n",*(BYTE *)(cprg+2),*(BYTE *)(cprg+1));
 #endif
 		setValI(val,reg[*(BYTE *)(cprg+1)],getValI(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    case Reg::AssR:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Assign real from %d to %d.\n",*(BYTE *)(cprg+2),*(BYTE *)(cprg+1));
 #endif
 		setValR(val,reg[*(BYTE *)(cprg+1)],getValR(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    case Reg::AssS: 
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Assign string from %d to %d.\n",*(BYTE *)(cprg+2),*(BYTE *)(cprg+1));
 #endif
 		setValS(val,reg[*(BYTE *)(cprg+1)],getValS(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    //-- Mov codes --
 	    case Reg::MovB:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Move boolean from %d to %d.\n",*(BYTE *)(cprg+2),*(BYTE *)(cprg+1));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValB(val,reg[*(BYTE *)(cprg+2)]);
 		cprg+=3; break;
 	    case Reg::MovI:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Move integer from %d to %d.\n",*(BYTE *)(cprg+2),*(BYTE *)(cprg+1));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValI(val,reg[*(BYTE *)(cprg+2)]);
 		cprg+=3; break;
 	    case Reg::MovR:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Move real from %d to %d.\n",*(BYTE *)(cprg+2),*(BYTE *)(cprg+1));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValR(val,reg[*(BYTE *)(cprg+2)]);
 		cprg+=3; break;
 	    case Reg::MovS:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Move string from %d to %d.\n",*(BYTE *)(cprg+2),*(BYTE *)(cprg+1));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValS(val,reg[*(BYTE *)(cprg+2)]);
 		cprg+=3; break;
 	    //-- Binary operations --
 	    case Reg::AddR:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Real %d = %d + %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValR(val,reg[*(BYTE *)(cprg+2)]) + getValR(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::AddS:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: String %d = %d + %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValS(val,reg[*(BYTE *)(cprg+2)]) + getValS(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::Sub:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Real %d = %d - %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValR(val,reg[*(BYTE *)(cprg+2)]) - getValR(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::Mul:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Real %d = %d * %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValR(val,reg[*(BYTE *)(cprg+2)]) * getValR(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::Div:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Real %d = %d / %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValR(val,reg[*(BYTE *)(cprg+2)]) / getValR(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::RstI:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: %d = %d % %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValI(val,reg[*(BYTE *)(cprg+2)]) % getValI(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::BitOr:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: %d = %d | %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValI(val,reg[*(BYTE *)(cprg+2)]) | getValI(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::BitAnd:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: %d = %d & %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValI(val,reg[*(BYTE *)(cprg+2)]) & getValI(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::BitXor:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: %d = %d ^ %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValI(val,reg[*(BYTE *)(cprg+2)]) ^ getValI(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::BitShLeft:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: %d = %d << %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValI(val,reg[*(BYTE *)(cprg+2)]) << getValI(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::BitShRight:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: %d = %d >> %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValI(val,reg[*(BYTE *)(cprg+2)]) >> getValI(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::LOr:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: %d = %d || %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValB(val,reg[*(BYTE *)(cprg+2)]) || getValB(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::LAnd:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: %d = %d && %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValB(val,reg[*(BYTE *)(cprg+2)]) && getValB(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::LT:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Real %d = %d < %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValR(val,reg[*(BYTE *)(cprg+2)]) < getValR(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::GT:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Real %d = %d > %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValR(val,reg[*(BYTE *)(cprg+2)]) > getValR(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::LER:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Real %d = %d <= %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValR(val,reg[*(BYTE *)(cprg+2)]) <= getValR(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::GER:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Real %d = %d >= %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValR(val,reg[*(BYTE *)(cprg+2)]) >= getValR(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::EQR:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Real %d = %d == %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValR(val,reg[*(BYTE *)(cprg+2)]) == getValR(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::EQS:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: String %d = %d == %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValS(val,reg[*(BYTE *)(cprg+2)]) == getValS(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::NER:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Real %d = %d != %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValR(val,reg[*(BYTE *)(cprg+2)]) != getValR(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    case Reg::NES:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: String %d = %d != %d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValS(val,reg[*(BYTE *)(cprg+2)]) != getValS(val,reg[*(BYTE *)(cprg+3)]);
 		cprg+=4; break;
 	    //-- Unary operations --
 	    case Reg::Not: 
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: %d = !%d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = !getValB(val,reg[*(BYTE *)(cprg+2)]);
 		cprg+=3; break;
 	    case Reg::BitNot:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: %d = ~%d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = ~getValI(val,reg[*(BYTE *)(cprg+2)]);
 		cprg+=3; break;
 	    case Reg::Neg:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Real %d = -%d.\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = -getValR(val,reg[*(BYTE *)(cprg+2)]);
 		cprg+=3; break;
 	    //-- Condition --
 	    case Reg::If:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Condition %d: %d|%d|%d.\n",*(BYTE *)(cprg+1),6,*(WORD *)(cprg+2),*(WORD *)(cprg+4));
 #endif
 		if(getValB(val,reg[*(BYTE *)(cprg+1)]))
@@ -1345,7 +1347,7 @@ void Func::exec( TValFunc *val, RegW *reg, const BYTE *cprg, ExecData &dt )
 		cprg = cprg + *(WORD *)(cprg+4);
 		continue;
 	    case Reg::Cycle:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Cycle %d: %d|%d|%d|%d.\n",*(BYTE *)(cprg+1),8,*(WORD *)(cprg+2),*(WORD *)(cprg+4),*(WORD *)(cprg+6));
 #endif
 		while( !(dt.flg&0x01) )
@@ -1366,127 +1368,127 @@ void Func::exec( TValFunc *val, RegW *reg, const BYTE *cprg, ExecData &dt )
 	    case Reg::Continue:	dt.flg|=0x05;	break;
 	    //-- Buildin functions --
 	    case Reg::FSin:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=sin(%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = sin(getValR(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    case Reg::FCos:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=cos(%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = cos(getValR(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    case Reg::FTan:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=tan(%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = tan(getValR(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    case Reg::FSinh:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=sinh(%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = sinh(getValR(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    case Reg::FCosh:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=cosh(%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = cosh(getValR(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    case Reg::FTanh:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=tanh(%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = tanh(getValR(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    case Reg::FAsin:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=asin(%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = asin(getValR(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    case Reg::FAcos:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=acos(%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = acos(getValR(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    case Reg::FAtan:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=atan(%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = atan(getValR(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    case Reg::FRand:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=rand(%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = getValR(val,reg[*(BYTE *)(cprg+2)])*(double)rand()/(double)RAND_MAX;
 		cprg+=3; break;
 	    case Reg::FLg:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=lg(%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = log10(getValR(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    case Reg::FLn:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=ln(%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = log(getValR(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    case Reg::FExp:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=exp(%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = exp(getValR(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    case Reg::FPow:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=pow(%d,%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = pow(getValR(val,reg[*(BYTE *)(cprg+2)]),getValR(val,reg[*(BYTE *)(cprg+3)]));
 		cprg+=4; break;
 	    case Reg::FMin:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=min(%d,%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = vmin(getValR(val,reg[*(BYTE *)(cprg+2)]),getValR(val,reg[*(BYTE *)(cprg+3)]));
 		cprg+=4; break;
 	    case Reg::FMax:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=max(%d,%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2),*(BYTE *)(cprg+3));
 #endif
 		reg[*(BYTE *)(cprg+1)] = vmax(getValR(val,reg[*(BYTE *)(cprg+2)]),getValR(val,reg[*(BYTE *)(cprg+3)]));
 		cprg+=4; break;
 	    case Reg::FSqrt:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=sqrt(%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = sqrt(getValR(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    case Reg::FAbs:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=abs(%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = fabs(getValR(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    case Reg::FSign:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=sign(%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = (getValR(val,reg[*(BYTE *)(cprg+2)])>=0)?1:-1;
 		cprg+=3; break;
 	    case Reg::FCeil:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=ceil(%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = ceil(getValR(val,reg[*(BYTE *)(cprg+2)]));
 		cprg+=3; break;
 	    case Reg::FFloor:
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		printf("CODE: Function %d=floor(%d).\n",*(BYTE *)(cprg+1),*(BYTE *)(cprg+2));
 #endif
 		reg[*(BYTE *)(cprg+1)] = floor(getValR(val,reg[*(BYTE *)(cprg+2)]));
@@ -1495,7 +1497,7 @@ void Func::exec( TValFunc *val, RegW *reg, const BYTE *cprg, ExecData &dt )
 	    case Reg::CFunc:
 		{
 		    TValFunc vfnc("JavaLikeFuncCalc",&funcAt(*(BYTE *)(cprg+1))->func().at());
-#if DEBUG_VM
+#if OSC_DEBUG >= 5
 		    printf("CODE: Call function/procedure %d = %s(%d).\n",*(BYTE *)(cprg+3),vfnc.func()->id().c_str(),*(BYTE *)(cprg+2));
 #endif
 		    //--- Get return position ---
@@ -1515,12 +1517,12 @@ void Func::exec( TValFunc *val, RegW *reg, const BYTE *cprg, ExecData &dt )
 			}
 		    }
 		    //--- Make calc ---
-		    vfnc.calc();
+		    vfnc.calc(vfnc.user());
 		    //--- Process outputs ---
 		    for( i_p = 0; i_p < *(BYTE *)(cprg+2); i_p++ )
 		    {
 			p_p = (i_p>=r_pos)?i_p+1:i_p;
-		if( vfnc.ioFlg(p_p)&IO::Output )
+			if( vfnc.ioFlg(p_p)&IO::Output )
 			    switch(vfnc.ioType(p_p))
 			    {
 				case IO::String:  setValS(val,reg[*(BYTE *)(cprg+4+i_p)],vfnc.getS(p_p)); break;
@@ -1544,7 +1546,7 @@ void Func::exec( TValFunc *val, RegW *reg, const BYTE *cprg, ExecData &dt )
 		}
 	    default:
 		setStart(false);
-		throw TError(nodePath().c_str(),_("Operation %c(%xh) error. Function <%s> stoped."),*cprg,*cprg,id().c_str());
+		throw TError(nodePath().c_str(),_("Operation %c(%xh) error. Function <%s> is stoped."),*cprg,*cprg,id().c_str());
 	}
     }
 }
@@ -1556,8 +1558,8 @@ void Func::cntrCmdProc( XMLNode *opt )
     {
 	TFunction::cntrCmdProc(opt);
 	ctrMkNode("oscada_cntr",opt,-1,"/",_("Function: ")+name(),owner().DB().empty()?0444:0664,"root","root");
-	ctrMkNode("fld",opt,-1,"/func/cfg/name",_("Name"),owner().DB().empty()?0444:0664,"root","root",1,"tp","str");
-	ctrMkNode("fld",opt,-1,"/func/cfg/descr",_("Description"),owner().DB().empty()?0444:0664,"root","root",3,"tp","str","cols","90","rows","3");
+	ctrMkNode("fld",opt,-1,"/func/cfg/name",_("Name"),owner().DB().empty()?0444:0664,"root","root",2,"tp","str","len","50");
+	ctrMkNode("fld",opt,-1,"/func/cfg/descr",_("Description"),owner().DB().empty()?0444:0664,"root","root",3,"tp","str","cols","100","rows","5");
 	ctrMkNode("fld",opt,-1,"/func/cfg/m_calc_tm",_("Maximum calc time (sec)"),0664,"root","root",1,"tp","dec");
 	if(ctrMkNode("area",opt,-1,"/io",_("Programm")))
 	{
@@ -1574,7 +1576,7 @@ void Func::cntrCmdProc( XMLNode *opt )
 		ctrMkNode("list",opt,-1,"/io/io/4",_("Hide"),0664,"root","root",1,"tp","bool");
 		ctrMkNode("list",opt,-1,"/io/io/5",_("Default"),0664,"root","root",1,"tp","str");
 	    }
-	    ctrMkNode("fld",opt,-1,"/io/prog",_("Programm"),0664,"root","root",3,"tp","str","cols","90","rows","10");
+	    ctrMkNode("fld",opt,-1,"/io/prog",_("Programm"),0664,"root","root",2,"tp","str","rows","10");
 	}
 	return;
     }
@@ -1617,7 +1619,7 @@ void Func::cntrCmdProc( XMLNode *opt )
 	    int row = atoi(opt->attr("row").c_str());
 	    int col = atoi(opt->attr("col").c_str());
 	    if( (col == 0 || col == 1) && !opt->text().size() )
-	        throw TError(nodePath().c_str(),_("Empty value no valid."));
+	        throw TError(nodePath().c_str(),_("Empty value is not valid."));
 	    if( col == 0 )	io(row)->setId(opt->text());
 	    else if( col == 1 )	io(row)->setName(opt->text());
 	    else if( col == 2 )	io(row)->setType((IO::Type)atoi(opt->text().c_str()));
