@@ -22,8 +22,6 @@
 #ifndef FREEFUNC_H
 #define FREEFUNC_H
 
-#define DEBUG_VM 0
-
 typedef unsigned short int WORD;
 typedef unsigned char BYTE;
 
@@ -88,6 +86,7 @@ class Reg
 	enum Code	//Byte codes
 	{
 	    End,	//[E]: End programm.
+	    EndFull,	//[E]: Full end from programm.
 	    MviB,	//[CRB]: Load boolean <B> to register <R>.
 	    MviI,	//[CR____]: Load integer <____> to register <R>.
 	    MviR,	//[CR______]: Load real <______> to register <R>.
@@ -258,6 +257,12 @@ class Func : public TConfig, public TFunction
     friend int yyparse( );
     friend void yyerror(const char*);
     public:
+	//> Addition flags for IO
+	enum IOSpecFlgs
+	{
+	    SysAttr	= 0x10	//Lock attribute
+	};
+
 	//Attributes
 	Func( const char *, const char *name = "" );
 	~Func( );
@@ -267,15 +272,17 @@ class Func : public TConfig, public TFunction
 	Func &operator=( Func &func );
 
 	string name( );
-	string descr( )			{ return m_descr; }
+	string descr( )			{ return mDescr; }
 	int maxCalcTm( )		{ return max_calc_tm; }
 	const string &prog( )		{ return prg_src; }
+	const string &usings( )		{ return mUsings; }
 
 	void setName( const string &nm );
 	void setDescr( const string &dscr );
 	void setMaxCalcTm( int vl );
 	void setProg( const string &prg );
 	void setStart( bool val );
+	void setUsings( const string &val )	{ mUsings = val; }
 
 	void del( );
 
@@ -286,13 +293,13 @@ class Func : public TConfig, public TFunction
 
 	//- Functins` list functions -
 	int funcGet( const string &path );
-	UFunc *funcAt( int id )	{ return m_fncs.at(id); }
+	UFunc *funcAt( int id )	{ return mFncs.at(id); }
 	void funcClear( );
 
 	//- Registers` list functions -
 	int regNew( bool var = false );
 	int regGet( const char *nm );
-	Reg *regAt( int id )	{ return m_regs.at(id); }
+	Reg *regAt( int id )	{ return (id>=0) ? mRegs.at(id) : NULL; }
 	void regClear( );
 
 	//- Temporary registers` list functions -
@@ -361,8 +368,8 @@ class Func : public TConfig, public TFunction
 
     private:
 	//Attributes
-	string	&m_name;
-	string	&m_descr;
+	string	&mName;
+	string	&mDescr;
 	int	&max_calc_tm;
 	string	&prg_src;
 
@@ -373,9 +380,10 @@ class Func : public TConfig, public TFunction
 	string		prg;		//Build prog
 	int		la_pos;		//LA position
 	string		p_err;		//Parse error
-	vector<UFunc*>	m_fncs;		//Work functions list
-	vector<Reg*>	m_regs;		//Work registers list
-	vector<Reg*>	m_tmpregs;	//Constant temporary list
+	string		mUsings;	//Functions usings namespaces
+	vector<UFunc*>	mFncs;		//Work functions list
+	vector<Reg*>	mRegs;		//Work registers list
+	vector<Reg*>	mTmpRegs;	//Constant temporary list
 	deque<Reg*>	f_prmst;	//Function's parameters stack
 	Res		&parse_res;
 };
