@@ -32,13 +32,13 @@ using namespace JavaLikeCalc;
 //* Lib: Functions library                        *
 //*************************************************
 Lib::Lib( const char *id, const char *name, const string &lib_db ) :
-    TConfig(&mod->elLib()), m_id(cfg("ID").getSd()), m_name(cfg("NAME").getSd()),
-    m_descr(cfg("DESCR").getSd()), m_db(cfg("DB").getSd()), work_lib_db(lib_db)
+    TConfig(&mod->elLib()), mId(cfg("ID").getSd()), mName(cfg("NAME").getSd()),
+    mDescr(cfg("DESCR").getSd()), mDB(cfg("DB").getSd()), work_lib_db(lib_db)
 {
-    m_id = id;
-    m_name = name;
-    m_db = string("flb_")+id;
-    m_fnc = grpAdd("fnc_");
+    mId = id;
+    mName = name;
+    mDB = string("flb_")+id;
+    mFnc = grpAdd("fnc_");
     if( DB().empty() )	modifClr();
 }
 
@@ -55,7 +55,7 @@ TCntrNode &Lib::operator=( TCntrNode &node )
     //- Configuration copy -
     string tid = id();
     *(TConfig*)this = *(TConfig*)src_n;
-    m_id = tid;
+    mId = tid;
     work_lib_db = src_n->work_lib_db;
 
     //- Functions copy -
@@ -80,10 +80,10 @@ void Lib::postDisable( int flag )
 {
     if( flag && DB().size() )
     {
-	//- Delete libraries record -
-	SYS->db().at().dataDel(DB()+"."+mod->libTable(),mod->nodePath()+"lib/",*this);
+	//> Delete libraries record
+	SYS->db().at().dataDel(DB()+"."+mod->libTable(),mod->nodePath()+"lib/",*this,true);
 
-	//- Delete function's files -
+	//> Delete function's files
 	SYS->db().at().open(fullDB());
 	SYS->db().at().close(fullDB(),true);
 
@@ -94,34 +94,30 @@ void Lib::postDisable( int flag )
 
 string Lib::name( )
 {
-    return (m_name.size())?m_name:m_id;
+    return (mName.size())?mName:mId;
 }
 
 void Lib::setFullDB( const string &idb )
 {
     work_lib_db = TSYS::strSepParse(idb,0,'.')+"."+TSYS::strSepParse(idb,1,'.');
-    m_db = TSYS::strSepParse(idb,2,'.');
+    mDB = TSYS::strSepParse(idb,2,'.');
     modifG( );
 }
 
 void Lib::load_( )
 {
-    if( DB().empty() )	return;
+    if( DB().empty() || (!SYS->chkSelDB(DB())) )	return;
 
-    SYS->db().at().dataGet(work_lib_db+"."+mod->libTable(),mod->nodePath()+"lib/",*this);
+    SYS->db().at().dataGet(DB()+"."+mod->libTable(),mod->nodePath()+"lib/",*this);
 
-    //- Load functions -
+    //> Load functions
     TConfig c_el(&mod->elFnc());
     c_el.cfgViewAll(false);
-    int fld_cnt = 0;
-    while( SYS->db().at().dataSeek(fullDB(),mod->nodePath()+tbl(), fld_cnt++,c_el) )
+    for( int fld_cnt = 0; SYS->db().at().dataSeek(fullDB(),mod->nodePath()+tbl(), fld_cnt++,c_el); )
     {
 	string f_id = c_el.cfg("ID").getS();
-
 	if( !present(f_id) )	add(f_id.c_str());
 	at(f_id).at().load();
-
-	c_el.cfg("ID").setS("");
     }
 }
 
@@ -129,7 +125,7 @@ void Lib::save_( )
 {
     if( DB().empty() )    return;
 
-    SYS->db().at().dataSet(work_lib_db+"."+mod->libTable(),mod->nodePath()+"lib/",*this);
+    SYS->db().at().dataSet(DB()+"."+mod->libTable(),mod->nodePath()+"lib/",*this);
 }
 
 void Lib::setStart( bool val )
@@ -145,12 +141,12 @@ void Lib::setStart( bool val )
 
 void Lib::add( const char *id, const char *name )
 {
-    chldAdd(m_fnc,new Func(id,name));
+    chldAdd(mFnc,new Func(id,name));
 }
 
 void Lib::del( const char *id )
 {
-    chldDel(m_fnc,id);
+    chldDel(mFnc,id);
 }
 
 void Lib::cntrCmdProc( XMLNode *opt )
@@ -158,9 +154,10 @@ void Lib::cntrCmdProc( XMLNode *opt )
     //- Get page info -
     if( opt->name() == "info" )
     {
+	TCntrNode::cntrCmdProc(opt);
 	ctrMkNode("oscada_cntr",opt,-1,"/",_("Function's library: ")+id(),0664,"root","root");
-	if(ctrMkNode("branches",opt,-1,"/br","",0444))	
-	    ctrMkNode("grp",opt,-1,"/br/fnc_",_("Function"),0664,"root","root",1,"idm","1");
+	if(ctrMkNode("branches",opt,-1,"/br","",0444))
+	    ctrMkNode("grp",opt,-1,"/br/fnc_",_("Function"),0664,"root","root",2,"idm","1","idSz","20");
 	if(ctrMkNode("area",opt,-1,"/lib",_("Library")))
 	{
 	    if(ctrMkNode("area",opt,-1,"/lib/st",_("State")))
@@ -173,12 +170,12 @@ void Lib::cntrCmdProc( XMLNode *opt )
 	    if(ctrMkNode("area",opt,-1,"/lib/cfg",_("Config")))
 	    {
 		ctrMkNode("fld",opt,-1,"/lib/cfg/id",_("Id"),0444,"root","root",1,"tp","str");
-		ctrMkNode("fld",opt,-1,"/lib/cfg/name",_("Name"),DB().empty()?0444:0664,"root","root",1,"tp","str");
-		ctrMkNode("fld",opt,-1,"/lib/cfg/descr",_("Description"),DB().empty()?0444:0664,"root","root",3,"tp","str","cols","50","rows","3");
+		ctrMkNode("fld",opt,-1,"/lib/cfg/name",_("Name"),DB().empty()?0444:0664,"root","root",2,"tp","str","len","50");
+		ctrMkNode("fld",opt,-1,"/lib/cfg/descr",_("Description"),DB().empty()?0444:0664,"root","root",3,"tp","str","cols","100","rows","5");
 	    }
 	}
 	if(ctrMkNode("area",opt,-1,"/func",_("Functions")))
-	    ctrMkNode("list",opt,-1,"/func/func",_("Functions"),0664,"root","root",4,"tp","br","idm","1","s_com","add,del","br_pref","fnc_");
+	    ctrMkNode("list",opt,-1,"/func/func",_("Functions"),0664,"root","root",5,"tp","br","idm","1","s_com","add,del","br_pref","fnc_","idSz","20");
 	return;
     }
 
@@ -214,8 +211,8 @@ void Lib::cntrCmdProc( XMLNode *opt )
 	    for( unsigned i_f=0; i_f < lst.size(); i_f++ )
 		opt->childAdd("el")->setAttr("id",lst[i_f])->setText(at(lst[i_f]).at().name());
 	}
-	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )	add(opt->attr("id").c_str(),opt->text().c_str());
-	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )	chldDel(m_fnc,opt->attr("id"),-1,1);
+	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )	add(TSYS::strEncode(opt->attr("id"),TSYS::ID).c_str(),opt->text().c_str());
+	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )	chldDel(mFnc,opt->attr("id"),-1,1);
     }
     else if( a_path == "/func/ls_lib" && ctrChkNode(opt) )
     {
