@@ -36,9 +36,9 @@
 #define MOD_ID		"JavaLikeCalc"
 #define MOD_NAME	"Java-like based calculator"
 #define MOD_TYPE	"DAQ"
-#define VER_TYPE	VER_SPC
+#define VER_TYPE	VER_CNTR
 #define SUB_TYPE	"LIB"
-#define VERSION		"1.2.0"
+#define VERSION		"1.4.0"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Allow java-like based calculator and function's libraries engine. User can create and modify function and libraries."
 #define LICENSE		"GPL"
@@ -80,7 +80,7 @@ TipContr::TipContr( string src )
 
     mod		= this;
 
-    m_lib = grpAdd("lib_");
+    mLib = grpAdd("lib_");
 }
 
 TipContr::~TipContr()
@@ -90,79 +90,79 @@ TipContr::~TipContr()
 
 void TipContr::postEnable( int flag )
 {
-    TModule::postEnable( flag );
+    TTipDAQ::postEnable( flag );
 
-    //- Controller db structure -
-    fldAdd( new TFld("PRM_BD",_("Parameters table"),TFld::String,TFld::NoFlag,"30","system") );
-    fldAdd( new TFld("FUNC",_("Controller's function"),TFld::String,TFld::NoFlag,"20") );
+    //> Controller db structure
+    fldAdd( new TFld("PRM_BD",_("Parameters table"),TFld::String,TFld::NoFlag,"60","system") );
+    fldAdd( new TFld("FUNC",_("Controller's function"),TFld::String,TFld::NoFlag,"40") );
     fldAdd( new TFld("PERIOD",_("Calc period (ms)"),TFld::Integer,TFld::NoFlag,"7","1000","0;1000000") );
     fldAdd( new TFld("PRIOR",_("Calc task priority"),TFld::Integer,TFld::NoFlag,"2","0","0;100") );
-    fldAdd( new TFld("ITER",_("Iteration number into calc period"),TFld::Integer,TFld::NoFlag,"2","1","0;99") );
+    fldAdd( new TFld("ITER",_("Iteration number in the calc period"),TFld::Integer,TFld::NoFlag,"2","1","0;99") );
 
-    //- Controller value db structure -
-    val_el.fldAdd( new TFld("ID",_("IO ID"),TFld::String,TCfg::Key,"10") );
-    val_el.fldAdd( new TFld("VAL",_("IO value"),TFld::String,TFld::NoFlag,"20") );
+    //> Controller value db structure
+    val_el.fldAdd( new TFld("ID",_("IO ID"),TFld::String,TCfg::Key,"20") );
+    val_el.fldAdd( new TFld("VAL",_("IO value"),TFld::String,TFld::NoFlag,"50") );
 
-    //- Add parameter types -
+    //> Add parameter types
     int t_prm = tpParmAdd("std","PRM_BD",_("Standard"));
     tpPrmAt(t_prm).fldAdd( new TFld("FLD",_("Data fields(Sep - ';')"),TFld::String,TCfg::NoVal,"50") );
 
-    //- Lib's db structure -
+    //> Lib's db structure
     lb_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"20") );
-    lb_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TFld::NoFlag,"50") );
-    lb_el.fldAdd( new TFld("DESCR",_("Description"),TFld::String,TFld::NoFlag,"300") );
+    lb_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TCfg::TransltText,"50") );
+    lb_el.fldAdd( new TFld("DESCR",_("Description"),TFld::String,TCfg::TransltText,"300") );
     lb_el.fldAdd( new TFld("DB",_("Data base"),TFld::String,TFld::NoFlag,"30") );
 
-    //- Function's structure -
-    fnc_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"50") );
-    fnc_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TFld::NoFlag,"50") );
-    fnc_el.fldAdd( new TFld("DESCR",_("Description"),TFld::String,TFld::NoFlag,"300") );
+    //> Function's structure
+    fnc_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"20") );
+    fnc_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TCfg::TransltText,"50") );
+    fnc_el.fldAdd( new TFld("DESCR",_("Description"),TFld::String,TCfg::TransltText,"300") );
     fnc_el.fldAdd( new TFld("MAXCALCTM",_("Maximum calc time"),TFld::Integer,TFld::NoFlag,"3","10","0;999") );
-    fnc_el.fldAdd( new TFld("FORMULA",_("Formula"),TFld::String,TFld::NoFlag,"1000") );
+    fnc_el.fldAdd( new TFld("FORMULA",_("Formula"),TFld::String,TCfg::TransltText,"10000") );
 
-    //- Function's IO structure -
-    fncio_el.fldAdd( new TFld("F_ID",_("Function ID"),TFld::String,TCfg::Key,"50") );
+    //> Function's IO structure
+    fncio_el.fldAdd( new TFld("F_ID",_("Function ID"),TFld::String,TCfg::Key,"20") );
     fncio_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"20") );
-    fncio_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TFld::NoFlag,"50") );
+    fncio_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TCfg::TransltText,"50") );
     fncio_el.fldAdd( new TFld("TYPE",_("Type"),TFld::Integer,TFld::NoFlag,"1") );
     fncio_el.fldAdd( new TFld("MODE",_("Mode"),TFld::Integer,TFld::NoFlag,"1") );
-    fncio_el.fldAdd( new TFld("DEF",_("Default value"),TFld::String,TFld::NoFlag,"20") );
+    fncio_el.fldAdd( new TFld("DEF",_("Default value"),TFld::String,TCfg::TransltText,"20") );
     fncio_el.fldAdd( new TFld("HIDE",_("Hide"),TFld::Boolean,TFld::NoFlag,"1") );
     fncio_el.fldAdd( new TFld("POS",_("Position"),TFld::Integer,TFld::NoFlag,"3") );
 
-    //- Init named constant table -
+    //> Init named constant table
     double rvl;
-    rvl = 3.14159265358l; m_const.push_back(NConst(TFld::Real,"pi",string((char*)&rvl,sizeof(rvl))));
-    rvl = 2.71828182845l; m_const.push_back(NConst(TFld::Real,"e",string((char*)&rvl,sizeof(rvl))));
-    rvl = EVAL_REAL; m_const.push_back(NConst(TFld::Real,"EVAL_REAL",string((char*)&rvl,sizeof(rvl))));
+    rvl = 3.14159265358l; mConst.push_back(NConst(TFld::Real,"pi",string((char*)&rvl,sizeof(rvl))));
+    rvl = 2.71828182845l; mConst.push_back(NConst(TFld::Real,"e",string((char*)&rvl,sizeof(rvl))));
+    rvl = EVAL_REAL; mConst.push_back(NConst(TFld::Real,"EVAL_REAL",string((char*)&rvl,sizeof(rvl))));
     int ivl;
-    ivl = EVAL_INT; m_const.push_back(NConst(TFld::Integer,"EVAL_INT",string((char*)&ivl,sizeof(ivl))));
-    ivl = EVAL_BOOL; m_const.push_back(NConst(TFld::Boolean,"EVAL_BOOL",string((char*)&ivl,1)));
+    ivl = EVAL_INT; mConst.push_back(NConst(TFld::Integer,"EVAL_INT",string((char*)&ivl,sizeof(ivl))));
+    ivl = EVAL_BOOL; mConst.push_back(NConst(TFld::Boolean,"EVAL_BOOL",string((char*)&ivl,1)));
 
-    m_const.push_back(NConst(TFld::String,"EVAL_STR",EVAL_STR));
+    mConst.push_back(NConst(TFld::String,"EVAL_STR",EVAL_STR));
 
-    //- Init buildin functions list -
-    m_bfunc.push_back(BFunc("sin",Reg::FSin,1));
-    m_bfunc.push_back(BFunc("cos",Reg::FCos,1));
-    m_bfunc.push_back(BFunc("tan",Reg::FTan,1));
-    m_bfunc.push_back(BFunc("sinh",Reg::FSinh,1));
-    m_bfunc.push_back(BFunc("cosh",Reg::FCosh,1));
-    m_bfunc.push_back(BFunc("tanh",Reg::FTanh,1));
-    m_bfunc.push_back(BFunc("asin",Reg::FAsin,1));
-    m_bfunc.push_back(BFunc("acos",Reg::FAcos,1));
-    m_bfunc.push_back(BFunc("atan",Reg::FAtan,1));
-    m_bfunc.push_back(BFunc("rand",Reg::FRand,1));
-    m_bfunc.push_back(BFunc("lg",Reg::FLg,1));
-    m_bfunc.push_back(BFunc("ln",Reg::FLn,1));
-    m_bfunc.push_back(BFunc("exp",Reg::FExp,1));
-    m_bfunc.push_back(BFunc("pow",Reg::FPow,2));
-    m_bfunc.push_back(BFunc("min",Reg::FMin,2));
-    m_bfunc.push_back(BFunc("max",Reg::FMax,2));
-    m_bfunc.push_back(BFunc("sqrt",Reg::FSqrt,1));
-    m_bfunc.push_back(BFunc("abs",Reg::FAbs,1));
-    m_bfunc.push_back(BFunc("sign",Reg::FSign,1));
-    m_bfunc.push_back(BFunc("ceil",Reg::FCeil,1));
-    m_bfunc.push_back(BFunc("floor",Reg::FFloor,1));
+    //> Init buildin functions list
+    mBFunc.push_back(BFunc("sin",Reg::FSin,1));
+    mBFunc.push_back(BFunc("cos",Reg::FCos,1));
+    mBFunc.push_back(BFunc("tan",Reg::FTan,1));
+    mBFunc.push_back(BFunc("sinh",Reg::FSinh,1));
+    mBFunc.push_back(BFunc("cosh",Reg::FCosh,1));
+    mBFunc.push_back(BFunc("tanh",Reg::FTanh,1));
+    mBFunc.push_back(BFunc("asin",Reg::FAsin,1));
+    mBFunc.push_back(BFunc("acos",Reg::FAcos,1));
+    mBFunc.push_back(BFunc("atan",Reg::FAtan,1));
+    mBFunc.push_back(BFunc("rand",Reg::FRand,1));
+    mBFunc.push_back(BFunc("lg",Reg::FLg,1));
+    mBFunc.push_back(BFunc("ln",Reg::FLn,1));
+    mBFunc.push_back(BFunc("exp",Reg::FExp,1));
+    mBFunc.push_back(BFunc("pow",Reg::FPow,2));
+    mBFunc.push_back(BFunc("min",Reg::FMin,2));
+    mBFunc.push_back(BFunc("max",Reg::FMax,2));
+    mBFunc.push_back(BFunc("sqrt",Reg::FSqrt,1));
+    mBFunc.push_back(BFunc("abs",Reg::FAbs,1));
+    mBFunc.push_back(BFunc("sign",Reg::FSign,1));
+    mBFunc.push_back(BFunc("ceil",Reg::FCeil,1));
+    mBFunc.push_back(BFunc("floor",Reg::FFloor,1));
 }
 
 TController *TipContr::ContrAttach( const string &name, const string &daq_db )
@@ -176,24 +176,29 @@ void TipContr::compileFuncLangs( vector<string> &ls )
     ls.push_back("JavaScript");
 }
 
-string TipContr::compileFunc( const string &lang, TFunction &fnc_cfg, const string &prog_text )
+string TipContr::compileFunc( const string &lang, TFunction &fnc_cfg, const string &prog_text, const string &usings )
 {
-    if( lang != "JavaScript" )	throw TError(nodePath().c_str(),_("Compile the program language %s is no support."),lang.c_str());
-    if(!lbPresent("sys_compile"))	lbReg( new Lib("sys_compile","","") );
-    if(!lbAt("sys_compile").at().present(fnc_cfg.id()))
+    if( lang != "JavaScript" )	throw TError(nodePath().c_str(),_("Compilation with the help of the program language %s is not supported."),lang.c_str());
+    if( !lbPresent("sys_compile") )	lbReg( new Lib("sys_compile","","") );
+    if( !lbAt("sys_compile").at().present(fnc_cfg.id()) )
 	lbAt("sys_compile").at().add(fnc_cfg.id().c_str(),"");
+
     AutoHD<Func> func = lbAt("sys_compile").at().at(fnc_cfg.id());
     ((TFunction&)func.at()).operator=(fnc_cfg);
     func.at().setProg(prog_text.c_str());
     try
     {
 	if(func.at().startStat()) func.at().setStart(false);
-	func.at().setStart(true); 
+	func.at().setUsings(usings);
+	func.at().setStart(true);
     }
     catch(TError err)
     {
-	func.free();
-	lbAt("sys_compile").at().del(fnc_cfg.id().c_str());
+	if( !func.at().use() )
+	{
+	    func.free();
+	    lbAt("sys_compile").at().del(fnc_cfg.id().c_str());
+	}
 	throw TError(nodePath().c_str(),_("Compile error: %s\n"),err.mess.c_str());
     }
     return func.at().nodePath();
@@ -213,7 +218,7 @@ string TipContr::optDescr( )
 
 void TipContr::load_( )
 {
-    //- Load parameters from command line -
+    //> Load parameters from command line
     int next_opt;
     const char *short_opt="h";
     struct option long_opt[] =
@@ -228,42 +233,37 @@ void TipContr::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
 
-    //- Load parameters -
+    //> Load parameters
 
-    //- Load function's libraries -
+    //> Load function's libraries
     try
     {
-	//-- Search and create new libraries --
+	//>> Search and create new libraries
 	TConfig c_el(&elLib());
 	c_el.cfgViewAll(false);
 	vector<string> db_ls;
 
-	//-- Search into DB --
-	SYS->db().at().dbList(db_ls);
+	//>> Search into DB
+	SYS->db().at().dbList(db_ls,true);
 	for( int i_db = 0; i_db < db_ls.size(); i_db++ )
-	{
-	    int lib_cnt = 0;
-	    while(SYS->db().at().dataSeek(db_ls[i_db]+"."+libTable(),"",lib_cnt++,c_el) )
+	    for( int lib_cnt = 0; SYS->db().at().dataSeek(db_ls[i_db]+"."+libTable(),"",lib_cnt++,c_el); )
 	    {
 		string l_id = c_el.cfg("ID").getS();
 		if(!lbPresent(l_id)) lbReg(new Lib(l_id.c_str(),"",(db_ls[i_db]==SYS->workDB())?"*.*":db_ls[i_db]));
-		c_el.cfg("ID").setS("");
 	    }
-	}
 
-	//-- Search into config file --
-	int lib_cnt = 0;
-	while(SYS->db().at().dataSeek("",nodePath()+"lib/",lib_cnt++,c_el) )
-	{
-	    string l_id = c_el.cfg("ID").getS();
-	    if(!lbPresent(l_id)) lbReg(new Lib(l_id.c_str(),"","*.*"));
-	    c_el.cfg("ID").setS("");
-	}
+	//>> Search into config file
+	if( SYS->chkSelDB("<cfg>") )
+	    for( int lib_cnt = 0; SYS->db().at().dataSeek("",nodePath()+"lib/",lib_cnt++,c_el); )
+	    {
+		string l_id = c_el.cfg("ID").getS();
+		if(!lbPresent(l_id)) lbReg(new Lib(l_id.c_str(),"","*.*"));
+	    }
     }catch( TError err )
     {
 	mess_err(err.cat.c_str(),"%s",err.mess.c_str());
@@ -303,9 +303,9 @@ void TipContr::cntrCmdProc( XMLNode *opt )
     if( opt->name() == "info" )
     {
 	TTipDAQ::cntrCmdProc(opt);
-	ctrMkNode("grp",opt,-1,"/br/lib_",_("Library"),0664,"root","root",1,"idm","1");
+	ctrMkNode("grp",opt,-1,"/br/lib_",_("Library"),0664,"root","root",2,"idm","1","idSz","20");
 	if(ctrMkNode("area",opt,1,"/libs",_("Functions' Libraries")))
-	    ctrMkNode("list",opt,-1,"/libs/lb",_("Libraries"),0664,"root","root",4,"tp","br","idm","1","s_com","add,del","br_pref","lib_");
+	    ctrMkNode("list",opt,-1,"/libs/lb",_("Libraries"),0664,"root","root",5,"tp","br","idm","1","s_com","add,del","br_pref","lib_","idSz","20");
 	return;
     }
 
@@ -320,7 +320,7 @@ void TipContr::cntrCmdProc( XMLNode *opt )
 	    for( unsigned i_a=0; i_a < lst.size(); i_a++ )
 		opt->childAdd("el")->setAttr("id",lst[i_a])->setText(lbAt(lst[i_a]).at().name());
 	}
-	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )	lbReg(new Lib(opt->attr("id").c_str(),opt->text().c_str(),"*.*"));
+	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )	lbReg(new Lib(TSYS::strEncode(opt->attr("id"),TSYS::ID).c_str(),opt->text().c_str(),"*.*"));
 	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )	lbUnreg(opt->attr("id"),1);
     }
     else TTipDAQ::cntrCmdProc(opt);
@@ -328,15 +328,15 @@ void TipContr::cntrCmdProc( XMLNode *opt )
 
 NConst *TipContr::constGet( const char *nm )
 {
-    for( int i_cst = 0; i_cst < m_const.size(); i_cst++)
-	if( m_const[i_cst].name == nm ) return &m_const[i_cst];
+    for( int i_cst = 0; i_cst < mConst.size(); i_cst++)
+	if( mConst[i_cst].name == nm ) return &mConst[i_cst];
 	    return NULL;
 }
 
 BFunc *TipContr::bFuncGet( const char *nm )
 {
-    for( int i_bf = 0; i_bf < m_bfunc.size(); i_bf++)
-	if( m_bfunc[i_bf].name == nm ) return &m_bfunc[i_bf];
+    for( int i_bf = 0; i_bf < mBFunc.size(); i_bf++)
+	if( mBFunc[i_bf].name == nm ) return &mBFunc[i_bf];
 	    return NULL;
 }
 
@@ -346,8 +346,8 @@ BFunc *TipContr::bFuncGet( const char *nm )
 Contr::Contr( string name_c, const string &daq_db, ::TElem *cfgelem) :
     ::TController(name_c, daq_db, cfgelem), TValFunc(name_c.c_str(),NULL,false), prc_st(false),
     endrun_req(false),
-    m_per(cfg("PERIOD").getId()), m_prior(cfg("PRIOR").getId()),
-    m_iter(cfg("ITER").getId()), m_fnc(cfg("FUNC").getSd())
+    mPer(cfg("PERIOD").getId()), mPrior(cfg("PRIOR").getId()),
+    mIter(cfg("ITER").getId()), mFnc(cfg("FUNC").getSd())
 {
     cfg("PRM_BD").setS("JavaLikePrm_"+name_c);
     setDimens(true);
@@ -364,7 +364,7 @@ void Contr::postDisable(int flag)
     {
 	if( flag )
 	{
-	    //- Delete IO value's table -
+	    //> Delete IO value's table
 	    string db = DB()+"."+TController::id()+"_val";
 	    SYS->db().at().open(db);
 	    SYS->db().at().close(db,true);
@@ -375,16 +375,26 @@ void Contr::postDisable(int flag)
     TController::postDisable(flag);
 }
 
+string Contr::getStatus( )
+{
+    string val = TController::getStatus( );
+
+    if( startStat( ) && !redntUse( ) ) val += TSYS::strMess(_("Calc time %.6g mks. "),calcTm());
+
+    return val;
+}
+
+
 void Contr::enable_( )
 {
-    if( !mod->lbPresent(TSYS::strSepParse(m_fnc,0,'.')) )
-	throw TError(nodePath().c_str(),_("Functions library <%s> no present. Please, create functions library!"),TSYS::strSepParse(m_fnc,0,'.').c_str());
-    if( !mod->lbAt(TSYS::strSepParse(m_fnc,0,'.')).at().present(TSYS::strSepParse(m_fnc,1,'.')) )
+    if( !mod->lbPresent(TSYS::strSepParse(mFnc,0,'.')) )
+	throw TError(nodePath().c_str(),_("Functions library <%s> is not present. Please, create functions library!"),TSYS::strSepParse(mFnc,0,'.').c_str());
+    if( !mod->lbAt(TSYS::strSepParse(mFnc,0,'.')).at().present(TSYS::strSepParse(mFnc,1,'.')) )
     {
-	mess_info(nodePath().c_str(),_("Create new function <%s>."),m_fnc.c_str());
-	mod->lbAt(TSYS::strSepParse(m_fnc,0,'.')).at().add(TSYS::strSepParse(m_fnc,1,'.').c_str());
+	mess_info(nodePath().c_str(),_("Create new function <%s>."),mFnc.c_str());
+	mod->lbAt(TSYS::strSepParse(mFnc,0,'.')).at().add(TSYS::strSepParse(mFnc,1,'.').c_str());
     }
-    setFunc( &mod->lbAt(TSYS::strSepParse(m_fnc,0,'.')).at().at(TSYS::strSepParse(m_fnc,1,'.')).at() );
+    setFunc( &mod->lbAt(TSYS::strSepParse(mFnc,0,'.')).at().at(TSYS::strSepParse(mFnc,1,'.')).at() );
     try{ loadFunc( ); }
     catch(TError err)
     {
@@ -411,17 +421,21 @@ void Contr::loadFunc( )
     {
 	((Func *)func())->load();
 
-	//- Load values -
+	//> Creating special IO
+	if( func()->ioId("f_frq") < 0 ) func()->ioIns( new IO("f_frq",_("Function calculate frequency (Hz)"),IO::Real,Func::SysAttr,"1000",false),0);
+	if( func()->ioId("f_start") < 0 ) func()->ioIns( new IO("f_start",_("Function start flag"),IO::Boolean,Func::SysAttr,"0",false),1);
+	if( func()->ioId("f_stop") < 0 ) func()->ioIns( new IO("f_stop",_("Function stop flag"),IO::Boolean,Func::SysAttr,"0",false),2);
+
+	//> Load values
 	TConfig cfg(&mod->elVal());
 	string bd_tbl = TController::id()+"_val";
 	string bd = DB()+"."+bd_tbl;
 
-	int fld_cnt=0;
-	while( SYS->db().at().dataSeek(bd,mod->nodePath()+bd_tbl,fld_cnt++,cfg) )
+	for( int fld_cnt=0; SYS->db().at().dataSeek(bd,mod->nodePath()+bd_tbl,fld_cnt++,cfg); )
 	{
-	    if( func()->ioId(cfg.cfg("ID").getS()) >= 0 )
-		setS(func()->ioId(cfg.cfg("ID").getS()),cfg.cfg("VAL").getS());
-	    cfg.cfg("ID").setS("");
+	    int ioId = func()->ioId(cfg.cfg("ID").getS());
+	    if( ioId < 0 || func()->io(ioId)->flg()&Func::SysAttr ) continue;
+	    setS(ioId,cfg.cfg("VAL").getS());
 	}
     }
 }
@@ -434,30 +448,26 @@ void Contr::save_( )
     {
 	((Func *)func())->save();
 
-	//- Save values -
+	//> Save values
 	TConfig cfg(&mod->elVal());
 	string bd_tbl = TController::id()+"_val";
 	string val_bd = DB()+"."+bd_tbl;
 	for( int iio = 0; iio < ioSize(); iio++ )
 	{
+	    if( func()->io(iio)->flg()&Func::SysAttr ) continue;
 	    cfg.cfg("ID").setS(func()->io(iio)->id());
 	    cfg.cfg("VAL").setS(getS(iio));
 	    SYS->db().at().dataSet(val_bd,mod->nodePath()+bd_tbl,cfg);
 	}
 
-	//- Clear VAL -
-	int fld_cnt=0;
-	cfg.cfg("ID").setS("");
+	//> Clear VAL
 	cfg.cfgViewAll(false);
-	while( SYS->db().at().dataSeek(val_bd,mod->nodePath()+bd_tbl,fld_cnt++,cfg) )
-	{
+	for( int fld_cnt=0; SYS->db().at().dataSeek(val_bd,mod->nodePath()+bd_tbl,fld_cnt++,cfg); )
 	    if( ioId(cfg.cfg("ID").getS()) < 0 )
 	    {
-		SYS->db().at().dataDel(val_bd,mod->nodePath()+bd_tbl,cfg);
+		SYS->db().at().dataDel(val_bd,mod->nodePath()+bd_tbl,cfg,true);
 		fld_cnt--;
 	    }
-	    cfg.cfg("ID").setS("");
-	}
     }
 }
 
@@ -471,16 +481,16 @@ void Contr::start_( )
 	pthread_attr_t pthr_attr;
 	pthread_attr_init(&pthr_attr);
 	struct sched_param prior;
-	if( m_prior && SYS->user() == "root" )
+	if( mPrior && SYS->user() == "root" )
 	    pthread_attr_setschedpolicy(&pthr_attr,SCHED_RR);
 	else pthread_attr_setschedpolicy(&pthr_attr,SCHED_OTHER);
-	prior.__sched_priority=m_prior;
+	prior.__sched_priority=mPrior;
 	pthread_attr_setschedparam(&pthr_attr,&prior);
 
 	pthread_create(&procPthr,&pthr_attr,Contr::Task,this);
 	pthread_attr_destroy(&pthr_attr);
 	if( TSYS::eventWait(prc_st, true, nodePath()+"start",5) )
-	    throw TError(nodePath().c_str(),_("Acquisition task no started!"));
+	    throw TError(nodePath().c_str(),_("Acquisition task is not started!"));
     }
 }
 
@@ -492,41 +502,48 @@ void Contr::stop_( )
 	endrun_req = true;
 	pthread_kill( procPthr, SIGALRM );
 	if( TSYS::eventWait(prc_st,false,nodePath()+"stop",5) )
-	    throw TError(nodePath().c_str(),_("Acquisition task no stoped!"));
+	    throw TError(nodePath().c_str(),_("Acquisition task is not stopped!"));
 	pthread_join( procPthr, NULL );
     }
 }
 
 void *Contr::Task( void *icntr )
 {
-    long long work_tm, last_tm = 0;
-    struct timespec get_tm;
     Contr &cntr = *(Contr *)icntr;
 
+#if OSC_DEBUG >= 2
+    mess_debug(cntr.nodePath().c_str(),_("Thread <%u> is started. TID: %ld"),pthread_self(),(long int)syscall(224));
+#endif
+
     cntr.endrun_req = false;
     cntr.prc_st = true;
 
-    while(!cntr.endrun_req)
-    {
-	for( int i_it = 0; i_it < cntr.m_iter; i_it++ )
-	    try
-	    {
-		cntr.calc();
-		cntr.modif();
-	    }
-	    catch(TError err)
-	    {
-		mess_err(err.cat.c_str(),"%s",err.mess.c_str() ); 
-		mess_err(cntr.nodePath().c_str(),_("Calc controller's function error."));
-	    }
+    bool is_start = true;
+    bool is_stop  = false;
 
-	//- Calc next work time and sleep -
-	clock_gettime(CLOCK_REALTIME,&get_tm);
-	work_tm = (((long long)get_tm.tv_sec*1000000000+get_tm.tv_nsec)/((long long)cntr.period()*1000000) + 1)*(long long)cntr.period()*1000000;
-	if(last_tm == work_tm)  work_tm+=(long long)cntr.period()*1000000; //Fix early call
-	last_tm = work_tm;
-	get_tm.tv_sec = work_tm/1000000000; get_tm.tv_nsec = work_tm%1000000000;
-	clock_nanosleep(CLOCK_REALTIME,TIMER_ABSTIME,&get_tm,NULL);
+    while( true )
+    {
+	if( !cntr.redntUse( ) )
+	{
+	    //> Setting special IO
+	    int ioI = cntr.ioId("f_frq");	if( ioI >= 0 ) cntr.setR(ioI,(float)cntr.iterate()*1000/(float)cntr.period());
+	    ioI = cntr.ioId("f_start");		if( ioI >= 0 ) cntr.setB(ioI,is_start);
+	    ioI = cntr.ioId("f_stop");		if( ioI >= 0 ) cntr.setB(ioI,is_stop);
+
+	    for( int i_it = 0; i_it < cntr.mIter; i_it++ )
+		try { cntr.calc(); }
+		catch(TError err)
+		{
+		    mess_err(err.cat.c_str(),"%s",err.mess.c_str() ); 
+		    mess_err(cntr.nodePath().c_str(),_("Calc controller's function error."));
+		}
+	}
+
+	if( is_stop ) break;
+	TSYS::taskSleep((long long)cntr.period()*1000000);
+	if( cntr.endrun_req ) is_stop = true;
+	is_start = false;
+	cntr.modif();
     }
 
     cntr.prc_st = false;
@@ -534,6 +551,23 @@ void *Contr::Task( void *icntr )
     return NULL;
 }
 
+void Contr::redntDataUpdate( bool firstArchiveSync )
+{
+    TController::redntDataUpdate(firstArchiveSync);
+
+    //> Request for template's attributes values
+    XMLNode req("get"); req.setAttr("path",nodePath(0,true)+"/%2fserv%2ffncAttr");
+
+    //> Send request to first active station for this controller
+    try{ owner().owner().rdStRequest(workId(),req); }
+    catch(TError err) { return; }
+
+    //> Redirect respond to local controller
+    req.setName("set")->setAttr("path","/%2fserv%2ffncAttr");
+    cntrCmd(&req);
+}
+
+
 TParamContr *Contr::ParamAttach( const string &name, int type )
 {
     return new Prm(name,&owner().tpPrmAt(type));
@@ -541,14 +575,35 @@ TParamContr *Contr::ParamAttach( const string &name, int type )
 
 void Contr::cntrCmdProc( XMLNode *opt )
 {
-    //- Get page info -
+    //> Service commands process
+    string a_path = opt->attr("path");
+    if( a_path.substr(0,6) == "/serv/" )
+    {
+	if( a_path == "/serv/fncAttr" )
+	{
+	    if( !startStat( ) || !func( ) ) throw TError(nodePath().c_str(),_("No started or no present function."));
+	    if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEQ_RD) )
+		for( int i_a = 0; i_a < ioSize(); i_a++ )
+		    opt->childAdd("a")->setAttr("id",func()->io(i_a)->id())->setText(getS(i_a));
+	    if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEQ_WR) )
+		for( int i_a = 0; i_a < opt->childSize(); i_a++ )
+		{
+		    int io_id = -1;
+		    if( opt->childGet(i_a)->name() != "a" || (io_id=ioId(opt->childGet(i_a)->attr("id"))) < 0 ) continue;
+		    setS(io_id,opt->childGet(i_a)->text());
+		}
+	}
+	else TController::cntrCmdProc(opt);
+	return;
+    }
+
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TController::cntrCmdProc(opt);
 	ctrMkNode("fld",opt,-1,"/cntr/cfg/FUNC",cfg("FUNC").fld().descr(),0660,"root","root",3,"tp","str","dest","sel_ed","select","/cntr/flst");
 	if( enableStat() && ctrMkNode("area",opt,-1,"/fnc",_("Calcing")) )
 	{
-	    ctrMkNode("fld",opt,-1,"/fnc/clc_tm",_("Calc time (mks)"),0444,"root","root",1,"tp","real");
 	    if(ctrMkNode("table",opt,-1,"/fnc/io",_("Data"),0664,"root","root",2,"s_com","add,del,ins,move","rows","15"))
 	    {
 		ctrMkNode("list",opt,-1,"/fnc/io/0",_("Id"),0664,"root","root",1,"tp","str");
@@ -566,15 +621,14 @@ void Contr::cntrCmdProc( XMLNode *opt )
 	return;
     }
 
-    //- Process command to page -
-    string a_path = opt->attr("path");
+    //> Process command to page
     if( a_path == "/cntr/flst" && ctrChkNode(opt) )
     {
 	vector<string> lst;
 	int c_lv = 0;
 	string c_path = "", c_el;
 	opt->childAdd("el")->setText(c_path);
-	for( int c_off = 0; (c_el=TSYS::strSepParse(m_fnc,0,'.',&c_off)).size(); c_lv++ )
+	for( int c_off = 0; (c_el=TSYS::strSepParse(mFnc,0,'.',&c_off)).size(); c_lv++ )
 	{
 	    c_path += c_lv ? "."+c_el : c_el;
 	    opt->childAdd("el")->setText(c_path);
@@ -584,14 +638,13 @@ void Contr::cntrCmdProc( XMLNode *opt )
 	{
 	    case 0:	mod->lbList(lst); break;
 	    case 1:
-		if( mod->lbPresent(TSYS::strSepParse(m_fnc,0,'.')) )
-		    mod->lbAt(TSYS::strSepParse(m_fnc,0,'.')).at().list(lst);
+		if( mod->lbPresent(TSYS::strSepParse(mFnc,0,'.')) )
+		    mod->lbAt(TSYS::strSepParse(mFnc,0,'.')).at().list(lst);
 		break;
 	}
 	for( unsigned i_a=0; i_a < lst.size(); i_a++ )
 	    opt->childAdd("el")->setText(c_path+lst[i_a]);
     }
-    else if( a_path == "/fnc/clc_tm" && enableStat() && ctrChkNode(opt) )	opt->setText(TSYS::real2str(calcTm( ),6));
     else if( a_path == "/fnc/io" && enableStat() )
     {
 	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
@@ -616,7 +669,13 @@ void Contr::cntrCmdProc( XMLNode *opt )
 	if( ctrChkNode(opt,"ins",0664,"root","root",SEQ_WR) )
 	{ ((Func *)func())->ioIns( new IO("new","New IO",IO::Real,IO::Default), atoi(opt->attr("row").c_str()) ); modif(); }
 	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )
-	{ ((Func *)func())->ioDel( atoi(opt->attr("row").c_str()) ); modif(); }
+	{
+	    int row = atoi(opt->attr("row").c_str());
+	    if( func()->io(row)->flg()&Func::SysAttr )
+		throw TError(nodePath().c_str(),_("Deleting lock atribute in not allow."));
+	    ((Func *)func())->ioDel( row ); 
+	    modif();
+	}
 	if( ctrChkNode(opt,"move",0664,"root","root",SEQ_WR) )
 	{ ((Func *)func())->ioMove( atoi(opt->attr("row").c_str()), atoi(opt->attr("to").c_str()) ); modif(); }
 	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
@@ -624,8 +683,10 @@ void Contr::cntrCmdProc( XMLNode *opt )
 	    int row = atoi(opt->attr("row").c_str());
 	    int col = atoi(opt->attr("col").c_str());
 	    if( (col == 0 || col == 1) && !opt->text().size() )
-		throw TError(nodePath().c_str(),_("Empty value no valid."));
-	    switch(col)
+		throw TError(nodePath().c_str(),_("Empty value is not valid."));
+	    if( func()->io(row)->flg()&Func::SysAttr )
+		throw TError(nodePath().c_str(),_("Changing locked atribute is not allowed."));
+	    switch( col )
 	    {
 		case 0:	func()->io(row)->setId(opt->text());	break;
 		case 1:	func()->io(row)->setName(opt->text());	break;
@@ -637,7 +698,7 @@ void Contr::cntrCmdProc( XMLNode *opt )
 	    if( !((Func *)func())->owner().DB().empty() ) ((Func *)func())->modif();
 	}
     }
-    else if( a_path == "/fnc/prog" && enableStat() ) 
+    else if( a_path == "/fnc/prog" && enableStat() )
     {
 	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(((Func *)func())->prog());
 	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
@@ -721,9 +782,23 @@ void Prm::disable()
     TParamContr::disable();
 }
 
-void Prm::vlSet( TVal &val )
+Contr &Prm::owner( )	{ return (Contr&)TParamContr::owner(); }
+
+void Prm::vlSet( TVal &val, const TVariant &pvl )
 {
     if( !enableStat() ) return;
+
+    //> Send to active reserve station
+    if( owner().redntUse( ) )
+    {
+	if( val.getS() == pvl.getS() ) return;
+	XMLNode req("set");
+	req.setAttr("path",nodePath(0,true)+"/%2fserv%2fattr")->childAdd("el")->setAttr("id",val.name())->setText(val.getS());
+	SYS->daq().at().rdStRequest(owner().workId(),req);
+	return;
+    }
+
+    //> Direct write
     try
     {
 	int io_id = ((Contr &)owner()).ioId(val.name());
@@ -753,11 +828,12 @@ void Prm::vlGet( TVal &val )
 {
     if( val.name() == "err" )
     {
-	if( !owner().startStat() )	val.setS(_("2:Controller stoped"),0,true);
-	else if( !enableStat() )	val.setS(_("1:Parameter disabled"),0,true);
+	if( !owner().startStat() )	val.setS(_("2:Controller is stoped"),0,true);
+	else if( !enableStat() )	val.setS(_("1:Parameter is disabled"),0,true);
 	else val.setS("0",0,true);
 	return;
     }
+    if( owner().redntUse( ) ) return;
     try
     {
 	int io_id = ((Contr &)owner()).ioId(val.name());
