@@ -40,7 +40,7 @@
 #define MOD_NAME	"Logic level"
 #define MOD_TYPE	"DAQ"
 #define VER_TYPE	VER_CNTR
-#define VERSION		"0.9.1"
+#define VERSION		"1.0.0"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Allow logic level paramers."
 #define LICENSE		"GPL"
@@ -117,7 +117,7 @@ void TTpContr::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -125,7 +125,7 @@ void TTpContr::load_( )
 
 void TTpContr::postEnable( int flag )
 {
-    TModule::postEnable( flag );
+    TTipDAQ::postEnable( flag );
 
     //- Controler's bd structure -
     fldAdd( new TFld("PRM_BD",_("Parameteres table"),TFld::String,TFld::NoFlag,"30","") );
@@ -140,7 +140,7 @@ void TTpContr::postEnable( int flag )
     //- Logical level parameter IO BD structure -
     el_prm_io.fldAdd( new TFld("PRM_ID",_("Parameter ID"),TFld::String,TCfg::Key,"20") );
     el_prm_io.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"20") );
-    el_prm_io.fldAdd( new TFld("VALUE",_("Value"),TFld::String,TFld::NoFlag,"200") );
+    el_prm_io.fldAdd( new TFld("VALUE",_("Value"),TFld::String,TCfg::TransltText,"200") );
 }
 
 TController *TTpContr::ContrAttach( const string &name, const string &daq_db )
@@ -179,6 +179,13 @@ void TMdContr::postDisable(int flag)
     { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
 }
 
+string TMdContr::getStatus( )
+{
+    string rez = TController::getStatus( );
+    if( startStat() && !redntUse( ) ) rez += TSYS::strMess(_("Calc time %.6g ms. "),tm_calc);
+    return rez;
+}
+
 TParamContr *TMdContr::ParamAttach( const string &name, int type )
 {
     return new TMdPrm(name,&owner().tpPrmAt(type));
@@ -186,14 +193,14 @@ TParamContr *TMdContr::ParamAttach( const string &name, int type )
 
 void TMdContr::start_( )
 {
-    //- Former process parameters list -
+    //> Former process parameters list
     vector<string> list_p;
     list(list_p);
     for(int i_prm=0; i_prm < list_p.size(); i_prm++)
 	if( at(list_p[i_prm]).at().enableStat() )
 	    prmEn(list_p[i_prm],true);
 
-    //- Start the request data task -
+    //> Start the request data task
     if( !prc_st )
     {
 	pthread_attr_t pthr_attr;
@@ -208,23 +215,23 @@ void TMdContr::start_( )
 	pthread_create(&procPthr,&pthr_attr,TMdContr::Task,this);
 	pthread_attr_destroy(&pthr_attr);
 	if( TSYS::eventWait(prc_st, true, nodePath()+"start",5) )
-	    throw TError(nodePath().c_str(),_("Acquisition task no started!"));
+	    throw TError(nodePath().c_str(),_("Acquisition task is not started!"));
     }
 }
 
 void TMdContr::stop_( )
 {
-    //- Stop the request and calc data task -
+    //> Stop the request and calc data task
     if( prc_st )
     {
 	endrun_req = true;
 	pthread_kill( procPthr, SIGALRM );
 	if( TSYS::eventWait(prc_st,false,nodePath()+"stop",5) )
-	    throw TError(nodePath().c_str(),_("Acquisition task no stoped!"));
+	    throw TError(nodePath().c_str(),_("Acquisition task is not stopped!"));
 	pthread_join( procPthr, NULL );
     }
 
-    //- Clear process parameters list -
+    //> Clear process parameters list
     p_hd.clear();
 }
 
@@ -244,10 +251,12 @@ void TMdContr::prmEn( const string &id, bool val )
 
 void *TMdContr::Task( void *icntr )
 {
-    long long work_tm, last_tm = 0;
-    struct timespec get_tm;
     TMdContr &cntr = *(TMdContr *)icntr;
 
+#if OSC_DEBUG >= 2
+    mess_debug(cntr.nodePath().c_str(),_("Thread <%u> is started. TID: %ld"),pthread_self(),(long int)syscall(224));
+#endif
+
     cntr.endrun_req = false;
     cntr.prc_st = true;
 
@@ -256,28 +265,22 @@ void *TMdContr::Task( void *icntr )
 
     while(true)
     {
-	long long t_cnt = SYS->shrtCnt();
+	//> Update controller's data
+	if( !cntr.redntUse( ) )
+	{
+	    long long t_cnt = TSYS::curTime();
+	    cntr.en_res.resRequestR( );
+	    for( unsigned i_p=0; i_p < cntr.p_hd.size(); i_p++ )
+		try{ cntr.p_hd[i_p].at().calc(is_start,is_stop); }
+		catch(TError err)
+		{ mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
+	    cntr.en_res.resRelease( );
+	    cntr.tm_calc = 1e-3*(TSYS::curTime()-t_cnt);
+	}
 
-	//- Update controller's data -
-	cntr.en_res.resRequestR( );
-	for(unsigned i_p=0; i_p < cntr.p_hd.size(); i_p++)
-	    try{ cntr.p_hd[i_p].at().calc(is_start,is_stop); }
-	    catch(TError err)
-	    { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
-	cntr.en_res.resReleaseR( );
-	cntr.tm_calc = 1.0e3*((double)(SYS->shrtCnt()-t_cnt))/((double)SYS->sysClk());
-
-	if(is_stop) break;
-
-	//- Calc next work time and sleep -
-	clock_gettime(CLOCK_REALTIME,&get_tm);
-	work_tm = (((long long)get_tm.tv_sec*1000000000+get_tm.tv_nsec)/((long long)cntr.period()*1000000) + 1)*(long long)cntr.period()*1000000;
-	if(last_tm == work_tm)	work_tm+=(long long)cntr.period()*1000000;	//Fix early call
-	last_tm = work_tm;
-	get_tm.tv_sec = work_tm/1000000000; get_tm.tv_nsec = work_tm%1000000000;
-	clock_nanosleep(CLOCK_REALTIME,TIMER_ABSTIME,&get_tm,NULL);
-
-	if(cntr.endrun_req) is_stop = true;
+	if( is_stop ) break;
+	TSYS::taskSleep((long long)cntr.period()*1000000);
+	if( cntr.endrun_req ) is_stop = true;
 	is_start = false;
     }
 
@@ -286,26 +289,39 @@ void *TMdContr::Task( void *icntr )
     return NULL;
 }
 
-void TMdContr::cntrCmdProc( XMLNode *opt )
+void TMdContr::redntDataUpdate( bool firstArchiveSync )
 {
-    //- Get page info -
-    if( opt->name() == "info" )
+    TController::redntDataUpdate(firstArchiveSync);
+
+    vector<string> pls; list(pls);
+
+    //> Request for template's attributes values
+    XMLNode req("CntrReqs"); req.setAttr("path",nodePath(0,true));
+    for( int i_p = 0; i_p < pls.size(); i_p++ )
     {
-	TController::cntrCmdProc(opt);
-	ctrMkNode("fld",opt,-1,"/cntr/st/calc_tm",_("Calc template functions time (ms)"),0444,"root","root",1,"tp","real");
-	return;
+	if( !at(pls[i_p]).at().enableStat( ) ) continue;
+	req.childAdd("get")->setAttr("path","/prm_"+pls[i_p]+"/%2fserv%2ftmplAttr");
     }
-    //- Process command to page -
-    string a_path = opt->attr("path");
-    if( a_path == "/cntr/st/calc_tm" && ctrChkNode(opt) )	opt->setText(TSYS::real2str(tm_calc,6));
-    else TController::cntrCmdProc(opt);
+
+    //> Send request to first active station for this controller
+    try{ owner().owner().rdStRequest(workId(),req); }
+    catch(TError err) { return; }
+
+    //> Redirect respond to local parameters
+    req.setAttr("path","/");
+    for( int i_prm = 0; i_prm < req.childSize(); i_prm++ )
+    {
+	if( atoi(req.childGet(i_prm)->attr("err").c_str()) ) { req.childDel(i_prm--); continue; }
+	req.childGet(i_prm)->setName("set");
+    }
+    cntrCmd(&req);
 }
 
 //*************************************************
 //* TMdPrm                                        *
 //*************************************************
 TMdPrm::TMdPrm( string name, TTipParam *tp_prm ) : 
-    TParamContr(name,tp_prm), p_el("w_attr"), prm_refl(NULL), m_wmode(TMdPrm::Free), chk_lnk_need(false), 
+    TParamContr(name,tp_prm), p_el("w_attr"), prm_refl(NULL), m_wmode(TMdPrm::Free), chk_lnk_need(false),
     id_freq(-1), id_start(-1), id_stop(-1), m_mode(cfg("MODE").getId()), m_prm(cfg("PRM").getSd())
 {
 
@@ -332,14 +348,15 @@ void TMdPrm::postDisable(int flag)
 	{
 	    string io_bd = owner().DB()+"."+owner().cfg(type().db).getS()+"_io";
 	    TConfig cfg(&mod->prmIOE());
-	    cfg.cfg("PRM_ID").setS(id());
-	    cfg.cfg("ID").setS("");
+	    cfg.cfg("PRM_ID").setS(id(),true);
 	    SYS->db().at().dataDel(io_bd,owner().owner().nodePath()+owner().cfg(type().db).getS()+"_io",cfg);
 	}
     }catch(TError err)
     { mess_warning(err.cat.c_str(),"%s",err.mess.c_str()); }
 }
 
+TMdContr &TMdPrm::owner( )	{ return (TMdContr&)TParamContr::owner(); }
+
 void TMdPrm::enable()
 {
     if( enableStat() )	return;
@@ -352,7 +369,7 @@ void TMdPrm::enable()
 	loadIO();
 	if(mode() == TMdPrm::Template)
 	{
-	    //- Init system attributes identifiers -
+	    //> Init system attributes identifiers
 	    id_freq	= tmpl->val.func()->ioId("f_frq");
 	    id_start	= tmpl->val.func()->ioId("f_start");
 	    id_stop	= tmpl->val.func()->ioId("f_stop");
@@ -387,7 +404,7 @@ void TMdPrm::load_( )
 
 void TMdPrm::loadIO()
 {
-    //- Load IO and init links -
+    //> Load IO and init links
     if( mode() == TMdPrm::Template )
     {
 	TConfig cfg(&mod->prmIOE());
@@ -415,7 +432,7 @@ void TMdPrm::save_( )
 
 void TMdPrm::saveIO()
 {
-    //- Save IO and init links -
+    //> Save IO and init links
     if( mode() == TMdPrm::Template )
     {
 	TConfig cfg(&mod->prmIOE());
@@ -425,6 +442,7 @@ void TMdPrm::saveIO()
 	for( int i_io = 0; i_io < tmpl->val.func()->ioSize(); i_io++ )
 	{
 	    cfg.cfg("ID").setS(tmpl->val.func()->io(i_io)->id());
+	    cfg.cfg("VALUE").setNoTransl( !(tmpl->val.func()->io(i_io)->type()==IO::String && !(tmpl->val.func()->io(i_io)->flg()&TPrmTempl::CfgLink)) );
 	    if( tmpl->val.func()->io(i_io)->flg()&TPrmTempl::CfgLink )
 		cfg.cfg("VALUE").setS(lnk(lnkId(i_io)).prm_attr);
 	    else cfg.cfg("VALUE").setS(tmpl->val.getS(i_io));
@@ -441,7 +459,7 @@ void TMdPrm::mode( TMdPrm::Mode md, const string &prm )
 
     ResAlloc res(moderes,true);
 
-    //- Free old mode -
+    //> Free old mode
     if( md != mode() || prm != m_wprm )
     {
 	if( mode() == TMdPrm::DirRefl )
@@ -468,7 +486,7 @@ void TMdPrm::mode( TMdPrm::Mode md, const string &prm )
 	}
     }
 
-    //- Init/update new mode -
+    //> Init/update new mode
     if( md == TMdPrm::DirRefl )
     {
 	if( !prm_refl ) prm_refl = new AutoHD<TValue>;
@@ -504,9 +522,9 @@ void TMdPrm::mode( TMdPrm::Mode md, const string &prm )
 		tmpl->val.setVfName(id()+"_tmplprm");
 		to_make = true;
 	    }
-	    //-- Set mode --
+	    //>> Set mode
 	    m_wmode = md;
-	    //-- Init attrubutes --
+	    //>> Init attrubutes
 	    for( int i_io = 0; i_io < tmpl->val.func()->ioSize(); i_io++ )
 	    {
 		if( (tmpl->val.func()->io(i_io)->flg()&TPrmTempl::CfgLink) && lnkId(i_io) < 0 )
@@ -529,7 +547,7 @@ void TMdPrm::mode( TMdPrm::Mode md, const string &prm )
 		}
 		if( to_make && (tmpl->val.func()->io(i_io)->flg()&TPrmTempl::CfgLink) )	tmpl->val.setS(i_io,"0");
 	    }
-	    //-- Init links --
+	    //>> Init links
 	    initTmplLnks();
 	}catch(TError err)
 	{
@@ -569,12 +587,13 @@ void TMdPrm::vlGet( TVal &val )
 	if( val.name() == "err" )
 	{
 	    if(!enableStat())
-		val.setS(_("1:Parameter had disabled."),0,true);
+		val.setS(_("1:Parameter is disabled."),0,true);
 	    else if(!owner().startStat())
 		val.setS(_("2:Controller is stoped."),0,true);
 	}
 	return;
     }
+    if( owner().redntUse( ) ) return;
     try
     {
 	ResAlloc res(moderes,false);
@@ -629,8 +648,19 @@ void TMdPrm::vlGet( TVal &val )
     }
 }
 
-void TMdPrm::vlSet( TVal &val )
+void TMdPrm::vlSet( TVal &val, const TVariant &pvl )
 {
+    //> Send to active reserve station
+    if( owner().redntUse( ) )
+    {
+	if( val.getS() == pvl.getS() ) return;
+	XMLNode req("set");
+	req.setAttr("path",nodePath(0,true)+"/%2fserv%2fattr")->childAdd("el")->setAttr("id",val.name())->setText(val.getS());
+	SYS->daq().at().rdStRequest(owner().workId(),req);
+	return;
+    }
+
+    //> Direct write
     try
     {
 	ResAlloc res(moderes,false);
@@ -689,14 +719,14 @@ void TMdPrm::vlArchMake( TVal &val )
 int TMdPrm::lnkSize()
 {
     if( mode() != TMdPrm::Template )
-	throw TError(nodePath().c_str(),_("Parameter disabled or no template based."));
+        throw TError(nodePath().c_str(),_("Parameter is disabled or is not based on the template."));
     return tmpl->lnk.size();
 }
 
 int TMdPrm::lnkId( int id )
 {
     if( mode() != TMdPrm::Template )
-	throw TError(nodePath().c_str(),_("Parameter disabled or no template based."));
+        throw TError(nodePath().c_str(),_("Parameter is disabled or is not based on the template."));
     for( int i_l = 0; i_l < lnkSize(); i_l++ )
 	if( lnk(i_l).io_id == id )
 	    return i_l;
@@ -706,7 +736,7 @@ int TMdPrm::lnkId( int id )
 int TMdPrm::lnkId( const string &id )
 {
     if( mode() != TMdPrm::Template )
-	throw TError(nodePath().c_str(),_("Parameter disabled or no template based."));
+        throw TError(nodePath().c_str(),_("Parameter is disabled or is not based on the template."));
     for( int i_l = 0; i_l < lnkSize(); i_l++ )
 	if( tmpl->val.func()->io(lnk(i_l).io_id)->id() == id )
 	    return i_l;
@@ -716,7 +746,7 @@ int TMdPrm::lnkId( const string &id )
 TMdPrm::SLnk &TMdPrm::lnk( int num )
 {
     if( mode() != TMdPrm::Template )
-	throw TError(nodePath().c_str(),_("Parameter disabled or no template based."));
+        throw TError(nodePath().c_str(),_("Parameter is disabled or is not based on the template."));
     if( num < 0 || num >= tmpl->lnk.size() )
 	throw TError(nodePath().c_str(),_("Parameter id error."));
     return tmpl->lnk[num];
@@ -735,7 +765,7 @@ void TMdPrm::calc( bool first, bool last )
 	if(id_start>=0)	tmpl->val.setB(id_start,first);
 	if(id_stop>=0)	tmpl->val.setB(id_stop,last);
 
-        //- Get input links -
+	//- Get input links -
 	for( int i_l = 0; i_l < lnkSize(); i_l++ )
 	    if( lnk(i_l).aprm.freeStat() )	tmpl->val.setS(lnk(i_l).io_id,EVAL_STR);
 	    else switch(tmpl->val.ioType(lnk(i_l).io_id))
@@ -787,9 +817,28 @@ void TMdPrm::calc( bool first, bool last )
 
 void TMdPrm::cntrCmdProc( XMLNode *opt )
 {
-    //- Service commands process -
+    //> Service commands process
     string a_path = opt->attr("path");
-    if( a_path.substr(0,6) == "/serv/" )  { TParamContr::cntrCmdProc(opt); return; }
+    if( a_path.substr(0,6) == "/serv/" )
+    {
+	if( a_path == "/serv/tmplAttr" )
+	{
+	    ResAlloc res(moderes,false);
+	    if( mode( ) != TMdPrm::Template || !tmpl->val.func( ) ) throw TError(nodePath().c_str(),_("No template parameter or error."));
+	    if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEQ_RD) )
+		for( int i_a = 0; i_a < tmpl->val.ioSize(); i_a++ )
+		    opt->childAdd("ta")->setAttr("id",tmpl->val.func()->io(i_a)->id())->setText(tmpl->val.getS(i_a));
+	    if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEQ_WR) )
+		for( int i_a = 0; i_a < opt->childSize(); i_a++ )
+		{
+		    int io_id = -1;
+		    if( opt->childGet(i_a)->name() != "ta" || (io_id=tmpl->val.ioId(opt->childGet(i_a)->attr("id"))) < 0 ) continue;
+		    tmpl->val.setS(io_id,opt->childGet(i_a)->text());
+		}
+	}
+	else TParamContr::cntrCmdProc(opt);
+	return;
+    }
 
     vector<string> list;
     //- Get page info -
@@ -802,7 +851,7 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 	ctrMkNode("fld",opt,-1,"/prm/cfg/PRM",cfg("PRM").fld().descr(),0660,"root","root",3,"tp","str","dest","sel_ed","select","/prm/cfg/prmp_lst");
 	if( mode() == TMdPrm::Template && ctrMkNode("area",opt,-1,"/cfg",_("Template config")) )
 	{
-	    ctrMkNode("fld",opt,-1,"/cfg/attr_only",_("Only atributes show"),0664,"root","root",1,"tp","bool");
+	    ctrMkNode("fld",opt,-1,"/cfg/attr_only",_("Only atributes are to be shown"),0664,"root","root",1,"tp","bool");
 	    if(ctrMkNode("area",opt,-1,"/cfg/prm",_("Parameters")))
 		for( int i_io = 0; i_io < tmpl->val.ioSize(); i_io++ )
 		{
@@ -921,7 +970,7 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 		if( TSYS::strSepParse(p_vl,0,'.') == owner().owner().modId() &&
 			TSYS::strSepParse(p_vl,1,'.') == owner().id() &&
 			TSYS::strSepParse(p_vl,2,'.') == id() )
-		    throw TError(nodePath().c_str(),_("Self to self linking is error."));
+		    throw TError(nodePath().c_str(),_("Self to self linking error."));
 		prm = SYS->daq().at().at(TSYS::strSepParse(p_vl,0,'.')).at().
 				      at(TSYS::strSepParse(p_vl,1,'.')).at().
 				      at(TSYS::strSepParse(p_vl,2,'.'));
@@ -944,13 +993,13 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 		}
 	    if(!prm.freeStat())
 	    {
-		if( noonly_no_set )	throw TError(nodePath().c_str(),_("Parameter have not only atributes!"));
-		else if( no_set.size() )throw TError(nodePath().c_str(),_("Parameter have not atributes: %s !"),no_set.c_str());
+		if( noonly_no_set )	throw TError(nodePath().c_str(),_("Parameter has no one atribute!"));
+		else if( no_set.size() )throw TError(nodePath().c_str(),_("Parameter has not atributes: %s !"),no_set.c_str());
 		mode( (TMdPrm::Mode)m_mode, m_prm );
 	    }
 	}
     }
-    else if( (a_path.substr(0,12) == "/cfg/prm/pl_" || a_path.substr(0,12) == "/cfg/prm/ls_") && 
+    else if( (a_path.substr(0,12) == "/cfg/prm/pl_" || a_path.substr(0,12) == "/cfg/prm/ls_") &&
 	    mode() == TMdPrm::Template && ctrChkNode(opt) )
     {
 	int c_lv = 0;
@@ -1007,7 +1056,7 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 		if( TSYS::strSepParse(opt->text(),0,'.') == owner().owner().modId() &&
 			TSYS::strSepParse(opt->text(),1,'.') == owner().id() &&
 			TSYS::strSepParse(opt->text(),2,'.') == id() )
-		    throw TError(nodePath().c_str(),_("Self to self linking is error."));
+		    throw TError(nodePath().c_str(),_("Self to self linking error."));
 		lnk(lnkId(i_io)).prm_attr = opt->text();
 		mode( (TMdPrm::Mode)m_mode, m_prm );
 	    }
