@@ -43,7 +43,7 @@
 #define MOD_NAME	"Siemens DAQ"
 #define MOD_TYPE	"DAQ"
 #define VER_TYPE	VER_CNTR
-#define VERSION		"1.1.0"
+#define VERSION		"1.2.0"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Allow data source Siemens PLC by CP of Hilscher CIF cards use MPI protocol and library Libnodave for other."
 #define LICENSE		"GPL"
@@ -70,7 +70,7 @@ extern "C"
 using namespace Siemens;
 
 //************************************************
-//* TTpContr                                     * 
+//* TTpContr                                     *
 //************************************************
 TTpContr::TTpContr( string name ) : drv_CIF_OK(false)
 {
@@ -107,7 +107,7 @@ string TTpContr::optDescr( )
 
 void TTpContr::postEnable( int flag )
 {
-    TModule::postEnable(flag);
+    TTipDAQ::postEnable(flag);
 
     //- Controler's DB structure -
     fldAdd( new TFld("PRM_BD",_("Parameteres table"),TFld::String,TFld::NoFlag,"30","") );
@@ -162,7 +162,7 @@ void TTpContr::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -279,7 +279,7 @@ void TTpContr::initCIF( int dev )
     // Load parameters to board 
     //- Running board aplications -
     if( (sRet = DevSetHostState( dev, HOST_READY, 0L )) != DRV_NO_ERROR )
-	throw TError( nodePath().c_str(), _("CIF device %d running (DevSetHostState(HOST_READY)) is error!"), dev);
+	throw TError( nodePath().c_str(), _("CIF device %d running (DevSetHostState(HOST_READY)) error!"), dev);
     //- Load the protocol task parameters -
     memset(&DPParameter,0,sizeof(DPM_PLC_PARAMETER));
     DPParameter.bMode=DPM_SET_MODE_UNCONTROLLED;
@@ -348,9 +348,9 @@ void TTpContr::initCIF( int dev )
     ptBusDpm->abOctet[3] = 0;
     tMsg.ln = sizeof(DPM_BUS_DP)+sizeof(DDLM_DOWNLOAD_REQUEST)- DPM_MAX_LEN_DATA_UNIT;
     if((sRet=DevPutMessage(dev,(MSG_STRUC *)&tMsg, 200L))!=DRV_NO_ERROR)
-	throw TError( nodePath().c_str(), _("Put request is error: %d."),sRet);
+	throw TError( nodePath().c_str(), _("Sending request error: %d."),sRet);
     if((sRet = DevGetMessage(dev,sizeof(RCS_MESSAGE),(MSG_STRUC *)&tMsg, 200L))!=DRV_NO_ERROR)
-	throw TError( nodePath().c_str(), _("Get request is error: %d."),sRet);
+	throw TError( nodePath().c_str(), _("Getting request error: %d."),sRet);
     //-- Wait for operation start with new parameters of PLC task --
     do
     {
@@ -367,7 +367,7 @@ void TTpContr::getLifeListPB( unsigned board, string &buffer )
     int res;
 
     if( !cif_devs[board].present )
-	throw TError(nodePath().c_str(),_("15:Board %d no present."),board); 
+	throw TError(nodePath().c_str(),_("15:Board %d is not present."),board); 
 
     ResAlloc resource(cif_devs[board].res,true);
 
@@ -391,9 +391,9 @@ void TTpContr::getLifeListPB( unsigned board, string &buffer )
     ptRcsTelegramheader10->function   = TASK_TFC_READ;
     //- Put message -
     if((res = DevPutMessage(board,(MSG_STRUC *)&tMsg, 500L))!=DRV_NO_ERROR)
-	throw TError(nodePath().c_str(),_("12:Put request is error %d."),res);
+	throw TError(nodePath().c_str(),_("12:Sending request error %d."),res);
     if((res = DevGetMessage(board,sizeof(RCS_MESSAGE),(MSG_STRUC *)&tMsg, 200L))!=DRV_NO_ERROR)
-	throw TError(nodePath().c_str(),_("13:Get request is error %d."),res);
+	throw TError(nodePath().c_str(),_("13:Getting request error %d."),res);
 
     buffer.assign((char*)&tMsg.d[sizeof(RCS_TELEGRAMHEADER_10)],DPM_MAX_NUM_DEVICES+1);
 }
@@ -530,15 +530,24 @@ void TMdContr::postDisable( int flag )
     {
 	if( flag )
 	{
-	    //- Delete parameter's io table -
+	    //> Delete parameter's io table
 	    string tbl = DB()+"."+cfg("PRM_BD").getS()+"_io";
 	    SYS->db().at().open(tbl);
 	    SYS->db().at().close(tbl,true);
 	}
-    }catch(TError err)
-    { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
+    }
+    catch(TError err) { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
 }
 
+string TMdContr::getStatus( )
+{
+    string rez = TController::getStatus( );
+    if( startStat() && !redntUse( ) ) rez += TSYS::strMess(_("Acquisition time %.6g ms. "),tm_calc);
+    return rez;
+}
+
+TTpContr &TMdContr::owner( )	{ return *(TTpContr*)nodePrev(); }
+
 TParamContr *TMdContr::ParamAttach( const string &name, int type )
 {
     return new TMdPrm(name,&owner().tpPrmAt(type));
@@ -563,7 +572,7 @@ void TMdContr::enable_( )
 
 void TMdContr::disable_( )
 {
-    //- Clear acquisition data blocks and asynchronous write mode data blocks -
+    //> Clear acquisition data blocks and asynchronous write mode data blocks
     acqBlks.clear();
     writeBlks.clear();
 }
@@ -572,14 +581,14 @@ void TMdContr::start_( )
 {
     connectRemotePLC( );
 
-    //- Former proccess parameters list -
+    //> Former proccess parameters list
     vector<string> list_p;
     list(list_p);
     for(int i_prm=0; i_prm < list_p.size(); i_prm++)
 	if( at(list_p[i_prm]).at().enableStat() )
 	    prmEn( list_p[i_prm], true );
 
-    //- Start the request data task -
+    //> Start the request data task
     if( !prc_st )
     {
 	pthread_attr_t pthr_attr;
@@ -594,23 +603,23 @@ void TMdContr::start_( )
 	pthread_create(&procPthr,&pthr_attr,TMdContr::Task,this);
 	pthread_attr_destroy(&pthr_attr);
 	if( TSYS::eventWait(prc_st, true, nodePath()+"start",5) )
-	    throw TError(nodePath().c_str(),_("Acquisition task no started!"));
+	    throw TError(nodePath().c_str(),_("Acquisition task is not started!"));
     }
 }
 
 void TMdContr::stop_( )
 {
-    //- Stop the request and calc data task -
+    //> Stop the request and calc data task
     if( prc_st )
     {
 	endrun_req = true;
 	pthread_kill( procPthr, SIGALRM );
 	if( TSYS::eventWait(prc_st,false,nodePath()+"stop",5) )
-	    throw TError(nodePath().c_str(),_("Acquisition task no stoped!"));
+	    throw TError(nodePath().c_str(),_("Acquisition task is not stopped!"));
 	pthread_join( procPthr, NULL );
     }
-    //- Clear proccess parameters list -
-    p_hd.clear();
+    //> Clear proccess parameters list
+    pHd.clear();
 
     disconnectRemotePLC( );
 }
@@ -632,14 +641,12 @@ void TMdContr::prmEn( const string &id, bool val )
 {
     int i_prm;
 
-    ResAlloc res(en_res,true);
-    for( i_prm = 0; i_prm < p_hd.size(); i_prm++)
-	if( p_hd[i_prm].at().id() == id ) break;
+    ResAlloc res(nodeRes(),true);
+    for( i_prm = 0; i_prm < pHd.size(); i_prm++)
+	if( pHd[i_prm].at().id() == id ) break;
 
-    if( val && i_prm >= p_hd.size() )
-	p_hd.push_back(at(id));
-    if( !val && i_prm < p_hd.size() )
-	p_hd.erase(p_hd.begin()+i_prm);
+    if( val && i_prm >= pHd.size() )	pHd.push_back(at(id));
+    if( !val && i_prm < pHd.size() )	pHd.erase(pHd.begin()+i_prm);
 }
 
 void TMdContr::regVal( SValData ival, IO::Type itp, bool wr )
@@ -648,7 +655,7 @@ void TMdContr::regVal( SValData ival, IO::Type itp, bool wr )
 
     int iv_sz = valSize(itp,ival.sz); //Get real value's size
 
-    ResAlloc res(en_res,true);
+    ResAlloc res(nodeRes(),true);
 
     //- Register to acquisition block -
     int i_b;
@@ -700,7 +707,7 @@ void TMdContr::regVal( SValData ival, IO::Type itp, bool wr )
 		    {
 			writeBlks[i_b].val.append((ival.off+iv_sz)-(writeBlks[i_b].off+writeBlks[i_b].val.size()),0);
 			//- Check for allow mergin to next block -
-			if( i_b+1 < writeBlks.size() && writeBlks[i_b+1].db == ival.db && 
+			if( i_b+1 < writeBlks.size() && writeBlks[i_b+1].db == ival.db &&
 				(writeBlks[i_b].off+writeBlks[i_b].val.size()) >= writeBlks[i_b+1].off )
 			{
 			    writeBlks[i_b].val.append(writeBlks[i_b+1].val,writeBlks[i_b].off+writeBlks[i_b].val.size()-writeBlks[i_b+1].off,string::npos);
@@ -713,6 +720,7 @@ void TMdContr::regVal( SValData ival, IO::Type itp, bool wr )
 	    }
 	if( i_b >= writeBlks.size() )
 	    writeBlks.insert(writeBlks.begin()+i_b,SDataRec(ival.db,ival.off,iv_sz));
+	writeBlks[i_b].err = _("-1:No data");
     }
 }
 
@@ -722,14 +730,14 @@ void TMdContr::connectRemotePLC( )
     {
 	case CIF_PB:
 	    if( !(owner().cif_devs[0].present||owner().cif_devs[1].present||owner().cif_devs[2].present||owner().cif_devs[3].present) )
-		throw TError(nodePath().c_str(),_("Driver or no one boards no present."));
+		throw TError(nodePath().c_str(),_("No one driver or board are present."));
 	    break;
 	case ISO_TCP:
 	{
 	    //- Full Libnodave API -
 	    _daveOSserialType fds;
 	    fds.wfd = fds.rfd = openSocket(102,m_addr.c_str());
-	    if( fds.rfd <= 0 ) throw TError(nodePath().c_str(),_("Open socket of remote PLC is error."));
+	    if( fds.rfd <= 0 ) throw TError(nodePath().c_str(),_("Open socket of remote PLC error."));
 	    di = daveNewInterface(fds,(char*)(string("IF")+id()).c_str(),0,daveProtoISOTCP,daveSpeed187k);
 	    daveSetTimeout(di,5000000);
 	    dc = daveNewConnection(di,2,0,m_slot);
@@ -738,7 +746,7 @@ void TMdContr::connectRemotePLC( )
 		close(fds.wfd);
 		delete dc;
 		delete di;
-		throw TError(nodePath().c_str(),_("Connection to PLC is error."));
+		throw TError(nodePath().c_str(),_("Connection to PLC error."));
 	    }
 
 	    //- Self OpenSCADA API -
@@ -793,7 +801,7 @@ void TMdContr::connectRemotePLC( )
 
 	    break;
 	}
-	default: throw TError(nodePath().c_str(),_("Connection type '%d' no supported."),m_type);
+	default: throw TError(nodePath().c_str(),_("Connection type '%d' is not supported."),m_type);
     }
 }
 
@@ -819,8 +827,8 @@ void TMdContr::getDB( unsigned n_db, long offset, string &buffer )
 	    RCS_MESSAGE tMsg;
 	    int res, e_try = 4;
 
-	    if( buffer.size() > 240 )			throw TError(nodePath().c_str(),_("14:Request block too big."));
-	    if( !owner().cif_devs[m_dev].present )	throw TError(nodePath().c_str(),_("15:Board %d no present."),m_dev);
+	    if( buffer.size() > 240 )			throw TError(nodePath().c_str(),_("14:Request block is too big."));
+	    if( !owner().cif_devs[m_dev].present )	throw TError(nodePath().c_str(),_("15:Board %d is not present."),m_dev);
 	
 	    ResAlloc resource(owner().cif_devs[m_dev].res,true);
 
@@ -851,7 +859,7 @@ void TMdContr::getDB( unsigned n_db, long offset, string &buffer )
 		//- Put message to remote host -
 		res = DevPutMessage(m_dev,(MSG_STRUC *)&tMsg, 200L);
 		if( res == DRV_DEV_PUT_TIMEOUT )
-		    throw TError(nodePath().c_str(),_("12:Put request is timeouted."));
+		    throw TError(nodePath().c_str(),_("12:Sending request is timeouted."));
 		//- Get remote host's response -
 		if( res == DRV_NO_ERROR )
 		    res = DevGetMessage(m_dev,sizeof(RCS_MESSAGE),(MSG_STRUC *)&tMsg,200L);
@@ -861,8 +869,8 @@ void TMdContr::getDB( unsigned n_db, long offset, string &buffer )
 	    //- Process errors -
 	    if( res != DRV_NO_ERROR )		throw TError(nodePath().c_str(),_("19:Request to DB error %d."),res);
 	    if( res == DRV_DEV_GET_TIMEOUT )	throw TError(nodePath().c_str(),_("13:Get request is timeouted."));
-	    if( tMsg.f == 17 )	throw TError(nodePath().c_str(),_("17:No response of the remote station."));
-	    if( tMsg.f == 18 )	throw TError(nodePath().c_str(),_("18:Master not into the logical token ring."));
+	    if( tMsg.f == 17 )	throw TError(nodePath().c_str(),_("17:There is no response from the remote station."));
+	    if( tMsg.f == 18 )	throw TError(nodePath().c_str(),_("18:Master is out of the logical token ring."));
 	    if( tMsg.f == 0x85 )throw TError(nodePath().c_str(),_("20:Specified offset address or DB error."));
 
 	    //printf("Get DB %d:%d DB%d.%d(%d) -- %d\n",m_dev,vmax(0,vmin(126,atoi(m_addr.c_str()))),n_db,offset,buffer.size(),tMsg.f);
@@ -891,7 +899,7 @@ void TMdContr::getDB( unsigned n_db, long offset, string &buffer )
 	    int res;
 	    if( (res = daveReadBytes(dc,daveDB,n_db,offset,buffer.size(),NULL)) )
 	    {
-		mess_debug(nodePath().c_str(),_("Read block '%d' error: %d\n"),n_db,res);
+		mess_debug(nodePath().c_str(),_("Read block '%d' error: %d"),n_db,res);
 		throw TError(nodePath().c_str(),_("Read DB from controller error."));
 	    }
 	    buffer.assign((char*)dc->resultPointer,buffer.size());
@@ -954,8 +962,8 @@ void TMdContr::putDB( unsigned n_db, long offset, const string &buffer )
 	    RCS_MESSAGE tMsg;
 	    int res, e_try = 4;
 
-	    if( buffer.size() > 240 )			throw TError(nodePath().c_str(),_("16:Transmit block too big."));
-	    if( !owner().cif_devs[m_dev].present )	throw TError(nodePath().c_str(),_("15:Board %d no present."),m_dev);
+	    if( buffer.size() > 240 )			throw TError(nodePath().c_str(),_("16:Transmitted block is too big."));
+	    if( !owner().cif_devs[m_dev].present )	throw TError(nodePath().c_str(),_("15:Board %d is not present."),m_dev);
 
 	    ResAlloc resource(owner().cif_devs[m_dev].res,true);
 
@@ -988,7 +996,7 @@ void TMdContr::putDB( unsigned n_db, long offset, const string &buffer )
 		//- Put message to remote host -
 		res = DevPutMessage(m_dev,(MSG_STRUC *)&tMsg,200L);
 		if( res == DRV_DEV_PUT_TIMEOUT )
-		    throw TError(nodePath().c_str(),_("12:Put request is timeouted."));
+		    throw TError(nodePath().c_str(),_("12:Sending request is timeouted."));
 		//- Get remote host's response -
 		if( res == DRV_NO_ERROR)
 		    res = DevGetMessage(m_dev,sizeof(RCS_MESSAGE),(MSG_STRUC *)&tMsg, 200L);
@@ -999,9 +1007,9 @@ void TMdContr::putDB( unsigned n_db, long offset, const string &buffer )
 
 	    //- Process errors -
 	    if( res != DRV_NO_ERROR )		throw TError(nodePath().c_str(),_("19:Request to DB error %d."),res);
-	    if( res == DRV_DEV_GET_TIMEOUT )	throw TError(nodePath().c_str(),_("13:Get request is timeouted."));
-	    if( tMsg.f == 17 )	throw TError(nodePath().c_str(),_("17:No response of the remote station."));
-	    if( tMsg.f == 18 )	throw TError(nodePath().c_str(),_("18:Master not into the logical token ring."));
+	    if( res == DRV_DEV_GET_TIMEOUT )	throw TError(nodePath().c_str(),_("13:Getting request is timeouted."));
+	    if( tMsg.f == 17 )	throw TError(nodePath().c_str(),_("17:There is no response from the remote station."));
+	    if( tMsg.f == 18 )	throw TError(nodePath().c_str(),_("18:Master is out of the logical token ring."));
 	    if( tMsg.f == 0x85 )throw TError(nodePath().c_str(),_("20:Specified offset address or DB error."));
 	    break;
 	}
@@ -1010,7 +1018,7 @@ void TMdContr::putDB( unsigned n_db, long offset, const string &buffer )
 	    int res;
 	    if( (res = daveWriteBytes(dc,daveDB,n_db,offset,buffer.size(),(char*)buffer.c_str())) )
 	    {
-		mess_debug(nodePath().c_str(),_("Write block '%d' error: %d\n"),n_db,res);
+		mess_debug(nodePath().c_str(),_("Write block '%d' error: %d"),n_db,res);
 		throw TError(nodePath().c_str(),_("Write DB to controller error."));
 	    }
 
@@ -1063,7 +1071,7 @@ char TMdContr::getValB( SValData ival, string &err )
 	    else err = acqBlks[i_b].err;
 	    break;
 	}
-    err = err.size()?err:_("11:Value not gathered.");
+    err = err.size()?err:_("11:Value is not gathered.");
 
     return EVAL_BOOL;
 }
@@ -1085,7 +1093,7 @@ int TMdContr::getValI( SValData ival, string &err )
 	    else err = acqBlks[i_b].err;
 	    break;
 	}
-    err = err.size()?err:_("11:Value not gathered.");
+    err = err.size()?err:_("11:Value is not gathered.");
 
     return EVAL_INT;
 }
@@ -1106,7 +1114,7 @@ double TMdContr::getValR( SValData ival, string &err )
 	    else err = acqBlks[i_b].err;
 	    break;
 	}
-    err = err.size()?err:_("11:Value not gathered.");
+    err = err.size()?err:_("11:Value is not gathered.");
 
     return EVAL_REAL;
 }
@@ -1123,7 +1131,7 @@ string TMdContr::getValS( SValData ival, string &err )
 	    else err = acqBlks[i_b].err;
 	    break;
 	}
-    err = err.size()?err:_("11:Value not gathered.");
+    err = err.size()?err:_("11:Value is not gathered.");
 
     return EVAL_STR;
 }
@@ -1132,17 +1140,21 @@ void TMdContr::setValB( bool ivl, SValData ival, string &err )
 {
     int val = getValI(SValData(ival.db,ival.off,1),err);
     if(val==EVAL_INT || (bool)(val&(0x01<<ival.sz)) == ivl) return;
-    //- Write data to controller or write data block -
+    //> Write data to controller or write data block
     val^=(0x01<<ival.sz);
     try
     {
 	if( !assincWrite( ) )	putDB(ival.db,ival.off,string((char*)&val,1));
 	else
 	    for(int i_b = 0; i_b < writeBlks.size(); i_b++)
-		if( writeBlks[i_b].db == ival.db && ival.off >= writeBlks[i_b].off && 
+		if( writeBlks[i_b].db == ival.db && ival.off >= writeBlks[i_b].off &&
 			(ival.off+1) <= (writeBlks[i_b].off+writeBlks[i_b].val.size()) )
-		{ writeBlks[i_b].val[ival.off-writeBlks[i_b].off] = val; break; }
-	//- Set to DB buffer -
+		{
+		    writeBlks[i_b].val[ival.off-writeBlks[i_b].off] = val;
+		    if( atoi(writeBlks[i_b].err.c_str()) == -1 ) writeBlks[i_b].err = "";
+		    break;
+		}
+	//> Set to DB buffer
 	for(int i_b = 0; i_b < acqBlks.size(); i_b++)
 	    if(acqBlks[i_b].db == ival.db && ival.off >= acqBlks[i_b].off && 
 		    (ival.off+1) <= (acqBlks[i_b].off+acqBlks[i_b].val.size()) )
@@ -1155,7 +1167,7 @@ void TMdContr::setValI( int ivl, SValData ival, string &err )
 {
     int val = getValI(ival,err);
     if(val==EVAL_INT || val == ivl) return;
-    //- Write data to controller or write data block -
+    //> Write data to controller or write data block
     val = ivl;
     int iv_sz = valSize( IO::Integer, ival.sz );
     try
@@ -1163,12 +1175,16 @@ void TMdContr::setValI( int ivl, SValData ival, string &err )
 	if( !assincWrite( ) )	putDB(ival.db,ival.off,revers(string((char *)&val,iv_sz)));
 	else
  	    for(int i_b = 0; i_b < writeBlks.size(); i_b++)
-		if( writeBlks[i_b].db == ival.db && ival.off >= writeBlks[i_b].off && 
+		if( writeBlks[i_b].db == ival.db && ival.off >= writeBlks[i_b].off &&
 			(ival.off+iv_sz) <= (writeBlks[i_b].off+writeBlks[i_b].val.size()) )
-		{ writeBlks[i_b].val.replace(ival.off-writeBlks[i_b].off,iv_sz,revers(string((char *)&val,iv_sz))); break; }
-	//- Set to DB buffer -
+		{
+		    writeBlks[i_b].val.replace(ival.off-writeBlks[i_b].off,iv_sz,revers(string((char *)&val,iv_sz)));
+		    if( atoi(writeBlks[i_b].err.c_str()) == -1 ) writeBlks[i_b].err = "";
+		    break;
+		}
+	//> Set to DB buffer
 	for(int i_b = 0; i_b < acqBlks.size(); i_b++)
-	    if(acqBlks[i_b].db == ival.db && ival.off >= acqBlks[i_b].off && 
+	    if(acqBlks[i_b].db == ival.db && ival.off >= acqBlks[i_b].off &&
 		    (ival.off+iv_sz) <= (acqBlks[i_b].off+acqBlks[i_b].val.size()) )
 	    { acqBlks[i_b].val.replace(ival.off-acqBlks[i_b].off,iv_sz,revers(string((char *)&val,iv_sz))); break; }
     }
@@ -1179,8 +1195,8 @@ void TMdContr::setValR( double ivl, SValData ival, string &err )
 {
     double val = getValR(ival,err);
     float  val_4 = val;
-    if(val==EVAL_REAL || val == ivl) return;
-    //- Write data to controller or write data block -
+    if( val==EVAL_REAL || val == ivl ) return;
+    //> Write data to controller or write data block
     val = ivl;
     int iv_sz = valSize( IO::Real, ival.sz );
     try
@@ -1188,15 +1204,16 @@ void TMdContr::setValR( double ivl, SValData ival, string &err )
 	if( !assincWrite( ) )	putDB(ival.db,ival.off,revers(string((char *)&val,iv_sz)));
 	else
 	    for(int i_b = 0; i_b < writeBlks.size(); i_b++)
-		if(writeBlks[i_b].db == ival.db && ival.off >= writeBlks[i_b].off && 
+		if(writeBlks[i_b].db == ival.db && ival.off >= writeBlks[i_b].off &&
 			(ival.off+iv_sz) <= (writeBlks[i_b].off+writeBlks[i_b].val.size()) )
 		{
 		    writeBlks[i_b].val.replace(ival.off-writeBlks[i_b].off,iv_sz,revers(string(((iv_sz==4)?(char *)&val_4:(char *)&val),iv_sz)));
+		    if( atoi(writeBlks[i_b].err.c_str()) == -1 ) writeBlks[i_b].err = "";
 		    break;
 		}
-	//- Set to DB buffer -
+	//> Set to DB buffer
 	for(int i_b = 0; i_b < acqBlks.size(); i_b++)
-	    if(acqBlks[i_b].db == ival.db && ival.off >= acqBlks[i_b].off && 
+	    if(acqBlks[i_b].db == ival.db && ival.off >= acqBlks[i_b].off &&
 		    (ival.off+iv_sz) <= (acqBlks[i_b].off+acqBlks[i_b].val.size()) )
 	    {
 		acqBlks[i_b].val.replace(ival.off-acqBlks[i_b].off,iv_sz,revers(string(((iv_sz==4)?(char *)&val_4:(char *)&val),iv_sz))); 
@@ -1213,7 +1230,7 @@ void TMdContr::setValS( const string &ivl, SValData ival, string &err )
     string vali = ivl;
     vali.resize(iv_sz);
     if(val==EVAL_STR || val == vali) return;
-    //- Write data to controller or write data block -
+    //> Write data to controller or write data block
     try
     {
 	if( !assincWrite( ) )	putDB(ival.db,ival.off,vali);
@@ -1221,8 +1238,12 @@ void TMdContr::setValS( const string &ivl, SValData ival, string &err )
 	    for(int i_b = 0; i_b < writeBlks.size(); i_b++)
 		if(writeBlks[i_b].db == ival.db && ival.off >= writeBlks[i_b].off &&
 			(ival.off+iv_sz) <= (writeBlks[i_b].off+writeBlks[i_b].val.size()) )
-		{ writeBlks[i_b].val.replace(ival.off-writeBlks[i_b].off,iv_sz,vali.c_str()); break; }
-	//- Set to DB buffer -
+		{
+		    writeBlks[i_b].val.replace(ival.off-writeBlks[i_b].off,iv_sz,vali.c_str());
+		    if( atoi(writeBlks[i_b].err.c_str()) == -1 ) writeBlks[i_b].err = "";
+		    break;
+		}
+	//> Set to DB buffer
 	for(int i_b = 0; i_b < acqBlks.size(); i_b++)
 	    if(acqBlks[i_b].db == ival.db && ival.off >= acqBlks[i_b].off &&
 		    (ival.off+iv_sz) <= (acqBlks[i_b].off+acqBlks[i_b].val.size()) )
@@ -1240,15 +1261,17 @@ int TMdContr::valSize( IO::Type itp, int iv_sz )
 	case IO::Real:		return (iv_sz==4||iv_sz==8)?iv_sz:4;
 	case IO::Boolean:	return 1;
     }
-    throw TError(nodePath().c_str(),_("Value type is error."));
+    throw TError(nodePath().c_str(),_("Value type error."));
 }
 
 void *TMdContr::Task( void *icntr )
 {
-    long long work_tm, last_tm = 0;
-    struct timespec get_tm;
     TMdContr &cntr = *(TMdContr *)icntr;
 
+#if OSC_DEBUG >= 2
+    mess_debug(cntr.nodePath().c_str(),_("Thread <%u> is started. TID: %ld"),pthread_self(),(long int)syscall(224));
+#endif
+
     cntr.endrun_req = false;
     cntr.prc_st = true;
 
@@ -1257,25 +1280,28 @@ void *TMdContr::Task( void *icntr )
 
     while(true)
     {
-	long long t_cnt = SYS->shrtCnt();
+	long long t_cnt = TSYS::curTime();
 
-	//- Update controller's data -
-	cntr.en_res.resRequestR( );
-	//- Process write data blocks -
+	//> Update controller's data
+	cntr.nodeRes().resRequestR( );
+	//> Process write data blocks
 	if( cntr.assincWrite( ) )
 	    for(int i_b = 0; i_b < cntr.writeBlks.size(); i_b++)
 		try
 		{
+		    if( cntr.redntUse( ) ) { cntr.writeBlks[i_b].err = _("-1:No data"); continue; }
+		    if( atoi(cntr.writeBlks[i_b].err.c_str()) == -1 ) continue;
 		    //printf("TEST 00: Put %d: (%d:%d)\n",
 		    //	cntr.writeBlks[i_b].db, cntr.writeBlks[i_b].off, cntr.writeBlks[i_b].val.size() );
 		    cntr.putDB( cntr.writeBlks[i_b].db,cntr.writeBlks[i_b].off,cntr.writeBlks[i_b].val );
 		    cntr.writeBlks[i_b].err="";
 		}
-		catch(TError err) { cntr.writeBlks[i_b].err=err.mess; }	
-	//- Process acquisition data blocks -
+		catch(TError err) { cntr.writeBlks[i_b].err = err.mess; }
+	//> Process acquisition data blocks
 	for(int i_b = 0; i_b < cntr.acqBlks.size(); i_b++)
 	    try
 	    {
+		if( cntr.redntUse( ) ) { cntr.acqBlks[i_b].err = _("-1:No data"); continue; }
 		//printf("TEST 00: Get %d: (%d:%d)\n",
 		//    cntr.acqBlks[i_b].db, cntr.acqBlks[i_b].off, cntr.acqBlks[i_b].val.size() );
 		cntr.getDB(cntr.acqBlks[i_b].db, cntr.acqBlks[i_b].off, cntr.acqBlks[i_b].val);
@@ -1283,23 +1309,17 @@ void *TMdContr::Task( void *icntr )
 	    }
 	    catch(TError err) { cntr.acqBlks[i_b].err=err.mess; }
 
-	//- Calc parameters -
-	for(unsigned i_p=0; i_p < cntr.p_hd.size(); i_p++)
-	    try{ cntr.p_hd[i_p].at().calc(is_start,is_stop); }
+	//> Calc parameters
+	for( unsigned i_p=0; i_p < cntr.pHd.size() && !cntr.redntUse( ); i_p++)
+	    try{ cntr.pHd[i_p].at().calc(is_start,is_stop); }
 	    catch(TError err)
 	    { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
-	cntr.en_res.resReleaseR( );
-	cntr.tm_calc = 1.0e3*((double)(SYS->shrtCnt()-t_cnt))/((double)SYS->sysClk());
+	cntr.nodeRes().resRelease( );
+	cntr.tm_calc = 1e-3*(TSYS::curTime()-t_cnt);
 
 	if(is_stop) break;
 
-	//- Calc next work time and sleep -
-	clock_gettime(CLOCK_REALTIME,&get_tm);
-	work_tm = (((long long)get_tm.tv_sec*1000000000+get_tm.tv_nsec)/((long long)cntr.period()*1000000) + 1)*(long long)cntr.period()*1000000;
-	if(last_tm == work_tm)	work_tm+=(long long)cntr.period()*1000000;	//Fix early call
-	last_tm = work_tm;
-	get_tm.tv_sec = work_tm/1000000000; get_tm.tv_nsec = work_tm%1000000000;
-	clock_nanosleep(CLOCK_REALTIME,TIMER_ABSTIME,&get_tm,NULL);
+	TSYS::taskSleep((long long)cntr.period()*1000000);
 
 	if(cntr.endrun_req) is_stop = true;
 	is_start = false;
@@ -1310,20 +1330,32 @@ void *TMdContr::Task( void *icntr )
     return NULL;
 }
 
-void TMdContr::cntrCmdProc( XMLNode *opt )
+void TMdContr::redntDataUpdate( bool firstArchiveSync )
 {
-    //- Get page info -
-    if( opt->name() == "info" )
+    TController::redntDataUpdate(firstArchiveSync);
+
+    vector<string> pls; list(pls);
+
+    //> Request for template's attributes values
+    XMLNode req("CntrReqs"); req.setAttr("path",nodePath(0,true));
+    for( int i_p = 0; i_p < pls.size(); i_p++ )
     {
-	TController::cntrCmdProc(opt);
-	ctrMkNode("fld",opt,-1,"/cntr/st/acq_tm",_("Acquisition time (ms)"),0444,"root","root",1,"tp","real");
-	return;
+	if( !at(pls[i_p]).at().enableStat( ) ) continue;
+	req.childAdd("get")->setAttr("path","/prm_"+pls[i_p]+"/%2fserv%2ftmplAttr");
     }
 
-    //- Process command to page -
-    string a_path = opt->attr("path");
-    if( a_path == "/cntr/st/acq_tm" && ctrChkNode(opt) )	opt->setText(TSYS::real2str(tm_calc,6));
-    else TController::cntrCmdProc(opt);
+    //> Send request to first active station for this controller
+    try{ owner().owner().rdStRequest(workId(),req); }
+    catch(TError err) { return; }
+
+    //> Redirect respond to local parameters
+    req.setAttr("path","/");
+    for( int i_prm = 0; i_prm < req.childSize(); i_prm++ )
+    {
+	if( atoi(req.childGet(i_prm)->attr("err").c_str()) ) { req.childDel(i_prm--); continue; }
+	req.childGet(i_prm)->setName("set");
+    }
+    cntrCmd(&req);
 }
 
 //************************************************
@@ -1358,14 +1390,15 @@ void TMdPrm::postDisable(int flag)
 	{
 	    string io_bd = owner().DB()+"."+owner().cfg(type().db).getS()+"_io";
 	    TConfig cfg(&mod->prmIOE());
-	    cfg.cfg("PRM_ID").setS(id());
-	    cfg.cfg("ID").setS("");
+	    cfg.cfg("PRM_ID").setS(id(),true);
 	    SYS->db().at().dataDel(io_bd,owner().owner().nodePath()+owner().cfg(type().db).getS()+"_io",cfg);
 	}
     }catch(TError err)
     { mess_warning(err.cat.c_str(),"%s",err.mess.c_str()); }
 }
 
+TMdContr &TMdPrm::owner( )	{ return (TMdContr&)TParamContr::owner(); }
+
 void TMdPrm::enable()
 {
     if( enableStat() )	return;
@@ -1503,13 +1536,14 @@ void TMdPrm::vlGet( TVal &val )
 	if( val.name() == "err" )
 	{
 	    if(!enableStat())
-		val.setS(_("1:Parameter had disabled."),0,true);
+		val.setS(_("1:Parameter is disabled."),0,true);
 	    else if(!owner().startStat())
 		val.setS(_("2:Controller is stoped."),0,true);
 	}
 	else val.setS(EVAL_STR,0,true);
 	return;
     }
+    if( owner().redntUse( ) ) return;
     try
     {
 	int id_lnk = lnkId(val.name());
@@ -1545,8 +1579,19 @@ void TMdPrm::vlGet( TVal &val )
     }
 }
 
-void TMdPrm::vlSet( TVal &val )
+void TMdPrm::vlSet( TVal &val, const TVariant &pvl )
 {
+    //> Send to active reserve station
+    if( owner().redntUse( ) )
+    {
+	if( val.getS() == pvl.getS() ) return;
+	XMLNode req("set");
+	req.setAttr("path",nodePath(0,true)+"/%2fserv%2fattr")->childAdd("el")->setAttr("id",val.name())->setText(val.getS());
+	SYS->daq().at().rdStRequest(owner().workId(),req);
+	return;
+    }
+
+    //> Direct write
     try
     {
 	int id_lnk = lnkId(val.name());
@@ -1586,14 +1631,14 @@ void TMdPrm::vlArchMake( TVal &val )
 int TMdPrm::lnkSize()
 {
     if( !enableStat() )
-	throw TError(nodePath().c_str(),_("Parameter disabled."));
+	throw TError(nodePath().c_str(),_("Parameter is disabled."));
     return plnk.size();
 }
 
 int TMdPrm::lnkId( int id )
 {
     if( !enableStat() )
-	throw TError(nodePath().c_str(),_("Parameter disabled."));
+	throw TError(nodePath().c_str(),_("Parameter is disabled."));
     for( int i_l = 0; i_l < plnk.size(); i_l++ )
 	if( lnk(i_l).io_id == id )
 	    return i_l;
@@ -1603,7 +1648,7 @@ int TMdPrm::lnkId( int id )
 int TMdPrm::lnkId( const string &id )
 {
     if( !enableStat() )
-	throw TError(nodePath().c_str(),_("Parameter disabled."));
+	throw TError(nodePath().c_str(),_("Parameter is disabled."));
     for( int i_l = 0; i_l < plnk.size(); i_l++ )
     if( func()->io(lnk(i_l).io_id)->id() == id )
 	return i_l;
@@ -1613,7 +1658,7 @@ int TMdPrm::lnkId( const string &id )
 TMdPrm::SLnk &TMdPrm::lnk( int num )
 {
     if( !enableStat() || num < 0 || num >= plnk.size() )
-	throw TError(nodePath().c_str(),_("Parameter disabled or id error."));
+	throw TError(nodePath().c_str(),_("Parameter is disabled or id error."));
     return plnk[num];
 }
 
@@ -1639,11 +1684,11 @@ void TMdPrm::calc( bool first, bool last )
     try
     {
 	//- Proccess error hold -
-	if(acq_err.size())
+	if( acq_err.size() )
 	{
 	    time_t tm = time(NULL);
-	    if(!acq_err_tm)	acq_err_tm=tm+5;
-	    if(tm>acq_err_tm)	{ acq_err = ""; acq_err_tm=0; }
+	    if( !acq_err_tm )	acq_err_tm = tm+5;
+	    if( tm>acq_err_tm )	{ acq_err = ""; acq_err_tm=0; }
 	}
 
 	//- Set fixed system attributes -
@@ -1700,9 +1745,27 @@ void TMdPrm::calc( bool first, bool last )
 
 void TMdPrm::cntrCmdProc( XMLNode *opt )
 {
-    //- Service commands process -
+    //> Service commands process
     string a_path = opt->attr("path");
-    if( a_path.substr(0,6) == "/serv/" )  { TParamContr::cntrCmdProc(opt); return; }
+    if( a_path.substr(0,6) == "/serv/" )
+    {
+	if( a_path == "/serv/tmplAttr" )
+	{
+	    if( !enableStat() || !func() ) throw TError(nodePath().c_str(),_("Parameter disabled or error."));
+	    if( ctrChkNode(opt,"get",RWRWR_,"root","DAQ",SEQ_RD) )
+		for( int i_a = 0; i_a < ioSize(); i_a++ )
+		    opt->childAdd("a")->setAttr("id",func()->io(i_a)->id())->setText(getS(i_a));
+	    if( ctrChkNode(opt,"set",RWRWR_,"root","DAQ",SEQ_WR) )
+		for( int i_a = 0; i_a < opt->childSize(); i_a++ )
+		{
+		    int io_id = -1;
+		    if( opt->childGet(i_a)->name() != "a" || (io_id=ioId(opt->childGet(i_a)->attr("id"))) < 0 ) continue;
+		    setS(io_id,opt->childGet(i_a)->text());
+		}
+	}
+	else TParamContr::cntrCmdProc(opt);
+	return;
+    }
 
     vector<string> list;
     //- Get page info -
@@ -1712,7 +1775,7 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 	ctrMkNode("fld",opt,-1,"/prm/cfg/TMPL",cfg("TMPL").fld().descr(),0660,"root","root",3,"tp","str","dest","sel_ed","select","/cfg/prmp_lst");
 	if( enableStat() && ctrMkNode("area",opt,1,"/cfg",_("Template config")) )
 	{
-	    ctrMkNode("fld",opt,-1,"/cfg/only_off",_("Only DB offsets show"),0664,"root","root",1,"tp","bool");
+	    ctrMkNode("fld",opt,-1,"/cfg/only_off",_("Only DB offsets are to be shown"),0664,"root","root",1,"tp","bool");
 	    if(ctrMkNode("area",opt,-1,"/cfg/prm",_("Parameters")))
 		for( int i_io = 0; i_io < ioSize(); i_io++ )
 		{
@@ -1755,6 +1818,7 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
     {
 	m_tmpl = opt->text();
 	disable();
+	modif();
     }
     else if( a_path == "/cfg/prmp_lst" && ctrChkNode(opt) )
     {
