@@ -107,7 +107,7 @@ class TMdPrm : public TParamContr, public TValFunc
 	int lnkId( const string &id );
 	SLnk &lnk( int num );
 
-	TMdContr &owner( )	{ return (TMdContr&)TParamContr::owner(); }
+	TMdContr &owner( );
 
     protected:
 	//Methods
@@ -122,7 +122,7 @@ class TMdPrm : public TParamContr, public TValFunc
 	void cntrCmdProc( XMLNode *opt );	//Control interface command process
 
 	void vlGet( TVal &val );
-	void vlSet( TVal &val );
+	void vlSet( TVal &val, const TVariant &pvl );
 	void vlArchMake( TVal &val );
 
 	void loadIO( );
@@ -157,6 +157,8 @@ class TMdContr: public TController
 	TMdContr( string name_c, const string &daq_db, ::TElem *cfgelem );
 	~TMdContr( );
 
+	string getStatus( );
+
 	int period( )		{ return vmax(1,m_per); }
 	bool assincWrite( )	{ return m_assinc_wr; }
 
@@ -167,7 +169,11 @@ class TMdContr: public TController
 	void getDB( unsigned n_db, long offset, string &buffer );
 	void putDB( unsigned n_db, long offset, const string &buffer );
 
-	TTpContr &owner( )	{ return *(TTpContr *)nodePrev(); }
+	void redntDataUpdate( bool firstArchiveSync = false );
+
+	TTpContr &owner( );
+
+	Res &nodeRes( )		{ return nRes; }
 
     protected:
 	//Methods
@@ -192,9 +198,8 @@ class TMdContr: public TController
 	void setValR( double ivl, SValData ival, string &err );
 	void setValS( const string &ivl, SValData ival, string &err );
 
-	//- Service -
+	//> Service
 	void postDisable( int flag );				//Delete all DB if flag 1
-	void cntrCmdProc( XMLNode *opt );			//Control interface command process
 
     private:
 	//Methods
@@ -222,7 +227,6 @@ class TMdContr: public TController
 	};
 
 	//Attributes
-	Res	en_res;			// Resource for enable params
 	int	&m_per,			// ms
 		&m_prior,		// Process task priority
 		&m_type,		// Connection type
@@ -233,7 +237,7 @@ class TMdContr: public TController
 
 	bool	prc_st,			// Process task active
 		endrun_req;		// Request to stop of the Process task
-	vector< AutoHD<TMdPrm> > p_hd;	// Parameter's process list
+	vector< AutoHD<TMdPrm> > pHd;	// Parameter's process list
 	vector< SDataRec > acqBlks;	// Acquisition data blocks
 	vector< SDataRec > writeBlks;	// Data block for write to a data source, for asynchronous write mode
 
@@ -242,6 +246,8 @@ class TMdContr: public TController
 	daveInterface	*di;
 	daveConnection	*dc;
 
+	Res	nRes;
+
 	double	tm_calc;		// Template functions calc time
 };
 
@@ -273,6 +279,8 @@ class TTpContr: public TTipDAQ
 	void load_( );
 	void save_( );
 
+	bool redntAllow( )	{ return true; }
+
     private:
 	//Methods
 	TController *ContrAttach( const string &name, const string &daq_db );
