@@ -34,7 +34,7 @@
 #define MOD_NAME	"Self system OpenSCADA protocol"
 #define MOD_TYPE	"Protocol"
 #define VER_TYPE	VER_PROT
-#define VERSION		"0.8.0"
+#define VERSION		"0.9.0"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Self OpenSCADA protocol, support generic functions."
 #define LICENSE		"GPL"
@@ -161,7 +161,7 @@ void TProt::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -184,7 +184,7 @@ TProtocolIn *TProt::in_open( const string &name )
     return new TProtIn(name);
 }
 
-string TProt::outMess( const string &in, TTransportOut &tro )
+void TProt::outMess( XMLNode &io, TTransportOut &tro )
 {
     char buf[1000], buf1[256];
     string req, resp;
@@ -192,21 +192,21 @@ string TProt::outMess( const string &in, TTransportOut &tro )
 
     ResAlloc res( ores, true );
 
-    int in_off = 0;
-    bool   isDir= atoi(TSYS::strSepParse(in,0,'\n',&in_off).c_str());
-    string user = TSYS::strSepParse(in,0,'\n',&in_off);
-    string pass = TSYS::strSepParse(in,0,'\n',&in_off);
-    string data = in.substr(in_off);
+    bool   isDir = atoi(io.attr("rqDir").c_str()); io.attrDel("rqDir");
+    string user = io.attr("rqUser"); io.attrDel("rqUser");
+    string pass = io.attr("rqPass"); io.attrDel("rqPass");
+    string data = io.save();
+    io.clear();
 
     try
     {
 	while(true)
 	{
-	    //- Session open -
+	    //> Session open
 	    if( !isDir && tro.prm1() < 0 )
 	    {
 		req = "SES_OPEN "+user+" "+pass+"\n";
-		resp_len = tro.messIO(req.c_str(),req.size(),buf,sizeof(buf),20);
+		resp_len = tro.messIO(req.c_str(),req.size(),buf,sizeof(buf)-1,10000);
 		buf[resp_len] = 0;
 		buf1[0] = 0;
 		sscanf(buf,"REZ %d %255s\n",&rez,buf1);
@@ -214,44 +214,41 @@ string TProt::outMess( const string &in, TTransportOut &tro )
 		else if(rez > 0)throw TError(nodePath().c_str(),_("Station <%s> error: %s!"),tro.id().c_str(),buf1);
 		tro.setPrm1(atoi(buf1));
 	    }
-	    //- Request -
-	    //-- Compress data --
+	    //> Request
+	    //>> Compress data
 	    bool reqCompr = comprLev() && data.size() > comprBrd();
 	    if( reqCompr )	data = TSYS::strCompr(data,comprLev());
 
 	    if(isDir)	req = "REQDIR "+user+" "+pass+" "+TSYS::int2str(data.size()*(reqCompr?-1:1))+"\n"+data;
 	    else	req = "REQ "+TSYS::int2str(tro.prm1())+" "+TSYS::int2str(data.size()*(reqCompr?-1:1))+"\n"+data;
 	    buf[0] = 0;
-	    resp_len = tro.messIO(req.c_str(),req.size(),buf,sizeof(buf),20);
+	    resp_len = tro.messIO(req.c_str(),req.size(),buf,sizeof(buf),10000);
 	    resp.assign(buf,resp_len);
 
-	    //-- Get head --
+	    //>> Get head
 	    buf1[0] = 0;
-	    if(sscanf(resp.c_str(),"REZ %d %255s\n",&rez,buf1)<=0)
+	    if( sscanf(resp.c_str(),"REZ %d %255s\n",&rez,buf1) <= 0 )
 		throw TError(nodePath().c_str(),_("Station respond <%s> error!"),tro.id().c_str());
 	    if( rez == 1 )	{ tro.setPrm1(-1); if( isDir ) break; else continue; }
 	    if( rez > 0 )	throw TError(nodePath().c_str(),_("Station <%s> error: %d:%s!"),tro.id().c_str(),rez,buf1);
 	    int head_end = resp.find("\n",0);
 	    if( head_end == string::npos )
-		throw TError(nodePath().c_str(),_("Station <%s> error: Respond broken!"),tro.id().c_str());
+		throw TError(nodePath().c_str(),_("Station <%s> error: Respond is broken!"),tro.id().c_str());
 	    int resp_size = atoi(buf1);
 
-	    //-- Wait tail --
+	    //>> Wait tail
 	    while( resp.size() < abs(resp_size)+head_end+sizeof('\n') )
 	    {
-		resp_len = tro.messIO(NULL,0,buf,sizeof(buf),20);
+		resp_len = tro.messIO(NULL,0,buf,sizeof(buf),10000);
 		resp.append(buf,resp_len);
 	    }
 
-	    //-- Decompress data --
-	    if( resp_size < 0 )	return TSYS::strUncompr(resp.substr(head_end+sizeof('\n')));
-
-	    return resp.substr(head_end+sizeof('\n'));
+	    if( resp_size < 0 )	io.load(TSYS::strUncompr(resp.substr(head_end+sizeof('\n'))));
+	    else io.load(resp.substr(head_end+sizeof('\n')));
+	    return;
 	}
     }
     catch( TError err )	{ tro.stop(); throw; }
-
-    return "";
 }
 
 void TProt::cntrCmdProc( XMLNode *opt )
