@@ -85,7 +85,7 @@ class TProt: public TProtocol
 	void sesClose( int id_ses );
 	SAuth sesGet( int id_ses );
 
-	string outMess( const string &in, TTransportOut &tro );
+	void outMess( XMLNode &io, TTransportOut &tro );
 
     protected:
 	void load_( );
