@@ -31,6 +31,39 @@
 namespace FLibSYS
 {
 
+//*************************************************
+//* System access functions                       *
+
+//*************************************************
+//* System's commands call                        *
+//*************************************************
+class sysCall : public TFunction
+{
+    public:
+	sysCall( ) : TFunction("sysCall")
+	{
+	    ioAdd( new IO("rez",_("Rezult"),IO::String,IO::Return|IO::FullText) );
+	    ioAdd( new IO("com",_("Command"),IO::String,IO::Default) );
+	}
+
+	string name( )	{ return _("Sys: Call"); }
+	string descr( )	{ return _("System call commands."); }
+
+	void calc( TValFunc *val )
+	{
+	    FILE *fp = popen(val->getS(1).c_str(),"r");
+	    if( !fp ) return;
+
+	    char buf[STR_BUF_LEN];
+	    string rez;
+	    for( int r_cnt = 0; (r_cnt=fread(buf,1,sizeof(buf),fp)); )
+	    rez.append(buf,r_cnt);
+
+	    pclose(fp);
+	    val->setS(0,rez);
+	}
+};
+
 //*************************************************
 //* Archive subsystem's functions                 *
 
@@ -55,6 +88,31 @@ class varhOpen : public TFunction
 	}
 };
 
+//*************************************************
+//* Value buffer open                             *
+//*************************************************
+class varhBufOpen : public TFunction
+{
+    public:
+	varhBufOpen( ) : TFunction("varhBufOpen")
+	{
+	    ioAdd( new IO("id",_("Archive id"),IO::Integer,IO::Return) );
+	    ioAdd( new IO("tp",_("Type"),IO::Integer,IO::Default,"1") );
+	    ioAdd( new IO("sz",_("Size"),IO::Integer,IO::Default,"100") );
+	    ioAdd( new IO("per",_("Period (us)"),IO::Integer,IO::Default,"1000000") );
+	    ioAdd( new IO("hgrd",_("Hard grid"),IO::Boolean,IO::Default,"0") );
+	    ioAdd( new IO("hres",_("High resolution"),IO::Boolean,IO::Default,"0") );
+	}
+
+	string name( )	{ return _("Varch: Buffer open"); }
+	string descr( )	{ return _("Open value buffer for temporary values storing."); }
+
+	void calc( TValFunc *val )
+	{
+	    val->setI(0,mod->varchBufOpen((TFld::Type)val->getI(1),val->getI(2),val->getI(3),val->getB(4),val->getB(5)));
+	}
+};
+
 //*************************************************
 //* Value archive close                           *
 //*************************************************
@@ -87,6 +145,7 @@ class varhBeg : public TFunction
 	    ioAdd( new IO("sek",_("Seconds"),IO::Integer,IO::Output) );
 	    ioAdd( new IO("usek",_("Microseconds"),IO::Integer,IO::Output) );
 	    ioAdd( new IO("archtor",_("Archivator"),IO::String,IO::Default) );
+	    ioAdd( new IO("addr",_("Address of archive or atribute of parameter"),IO::String,IO::Default) );
 	}
 
 	string name( )	{ return _("Varch: Begin"); }
@@ -95,6 +154,9 @@ class varhBeg : public TFunction
 	void calc( TValFunc *val )
 	{
 	    int id = val->getI(0);
+	    string addr = val->getS(4);
+	    if( !addr.empty() )	id = mod->varchOpen(addr);
+
 	    long long vbg;
 	    if(mod->isArch(id))
 		vbg = mod->varch(id).at().begin(val->getS(3));
@@ -106,6 +168,8 @@ class varhBeg : public TFunction
 	    }
 	    val->setI(1,vbg/1000000);
 	    val->setI(2,vbg%1000000);
+
+	    if( !addr.empty() )	mod->varchClose(id);
 	}
 };
 
@@ -121,6 +185,7 @@ class varhEnd : public TFunction
 	    ioAdd( new IO("sek",_("Seconds"),IO::Integer,IO::Output) );
 	    ioAdd( new IO("usek",_("Microseconds"),IO::Integer,IO::Output) );
 	    ioAdd( new IO("archtor",_("Archivator"),IO::String,IO::Default) );
+	    ioAdd( new IO("addr",_("Address of archive or atribute of parameter"),IO::String,IO::Default) );
 	}
 
 	string name( )	{ return _("Varch: End"); }
@@ -129,6 +194,9 @@ class varhEnd : public TFunction
 	void calc( TValFunc *val )
 	{
 	    int id = val->getI(0);
+	    string addr = val->getS(4);
+	    if( !addr.empty() )	id = mod->varchOpen(addr);
+
 	    long long vend;
 	    if(mod->isArch(id))
 		vend = mod->varch(id).at().end(val->getS(3));
@@ -140,6 +208,8 @@ class varhEnd : public TFunction
 	    }
 	    val->setI(1,vend/1000000);
 	    val->setI(2,vend%1000000);
+
+	    if( !addr.empty() )	mod->varchClose(id);
 	}
 };
 
@@ -178,7 +248,7 @@ class varhCopyBuf : public TFunction
 		}
 		else vb = mod->vbuf(did);
 		if(!vb)     return;
-		mod->varch(sid).at().getVal(*vb,(long long)val->getI(2)*1000000+val->getI(3),
+		mod->varch(sid).at().getVals(*vb,(long long)val->getI(2)*1000000+val->getI(3),
 						(long long)val->getI(4)*1000000+val->getI(5),val->getS(6));
 	    }
 	    else if(mod->isArch(did))
@@ -191,7 +261,7 @@ class varhCopyBuf : public TFunction
 		}
 		else vb = mod->vbuf(sid);
 		if(!vb)     return;
-		mod->varch(did).at().setVal(*vb,(long long)val->getI(2)*1000000+val->getI(3),
+		mod->varch(did).at().setVals(*vb,(long long)val->getI(2)*1000000+val->getI(3),
 						(long long)val->getI(4)*1000000+val->getI(5),val->getS(6));
 	    }
 	    else
@@ -199,37 +269,12 @@ class varhCopyBuf : public TFunction
 		TValBuf* svb = mod->vbuf(sid);
 		TValBuf* dvb = mod->vbuf(did);
 		if(!svb || !dvb) return;
-		svb->getVal(*dvb,(long long)val->getI(2)*1000000+val->getI(3),
+		svb->getVals(*dvb,(long long)val->getI(2)*1000000+val->getI(3),
 				 (long long)val->getI(4)*1000000+val->getI(5));
 	    }
 	}
 };
 
-//*************************************************
-//* Value buffer open                             *
-//*************************************************
-class varhBufOpen : public TFunction
-{
-    public:
-	varhBufOpen( ) : TFunction("varhBufOpen")
-	{
-	    ioAdd( new IO("id",_("Archive id"),IO::Integer,IO::Return) );
-	    ioAdd( new IO("tp",_("Type"),IO::Integer,IO::Default,"1") );
-	    ioAdd( new IO("sz",_("Size"),IO::Integer,IO::Default,"100") );
-	    ioAdd( new IO("per",_("Period (us)"),IO::Integer,IO::Default,"1000000") );
-	    ioAdd( new IO("hgrd",_("Hard grid"),IO::Boolean,IO::Default,"0") );
-	    ioAdd( new IO("hres",_("High resolution"),IO::Boolean,IO::Default,"0") );
-	}
-
-	string name( )	{ return _("Varch: Buffer open"); }
-	string descr( )	{ return _("Open value buffer for temporary values storing."); }
-
-	void calc( TValFunc *val )
-	{
-	    val->setI(0,mod->varchBufOpen((TFld::Type)val->getI(1),val->getI(2),val->getI(3),val->getB(4),val->getB(5)));
-	}
-};
-
 //*************************************************
 //* Get integer                                   *
 //*************************************************
@@ -239,11 +284,12 @@ class varhGetI : public TFunction
 	varhGetI( ) : TFunction("varhGetI")
 	{
 	    ioAdd( new IO("val",_("Return value"),IO::Integer,IO::Return) );
-	    ioAdd( new IO("id",_("Buffer id"),IO::Integer,IO::Default) );
+	    ioAdd( new IO("id",_("Buffer id"),IO::Integer,IO::Default,"-1") );
 	    ioAdd( new IO("sec",_("Seconds"),IO::Integer,IO::Output) );
 	    ioAdd( new IO("usec",_("Microseconds"),IO::Integer,IO::Output) );
 	    ioAdd( new IO("up_ord",_("Up order"),IO::Boolean,IO::Default,"0") );
 	    ioAdd( new IO("archtor",_("Archivator"),IO::String,IO::Default) );
+	    ioAdd( new IO("addr",_("Address of archive or atribute of parameter"),IO::String,IO::Default) );
 	}
 
 	string name( )	{ return _("Varch: Get integer"); }
@@ -252,16 +298,25 @@ class varhGetI : public TFunction
 	void calc( TValFunc *val )
 	{
 	    int id = val->getI(1);
-	    long long vtm = (long long)val->getI(2)*1000000+val->getI(3);
-	    if(mod->isArch(id))
-		val->setI(0,mod->varch(id).at().getI(&vtm,val->getB(4),val->getS(5)));
-	    else
+	    string addr = val->getS(6);
+	    try
 	    {
-		TValBuf* vb = mod->vbuf(id);
-		if( !vb )	return;
-		val->setI(0,vb->getI(&vtm,val->getB(4)));
+		if( !addr.empty() )	id = mod->varchOpen(addr);
+
+		long long vtm = (long long)val->getI(2)*1000000+val->getI(3);
+		if(mod->isArch(id))
+		    val->setI(0,mod->varch(id).at().getVal(&vtm,val->getB(4),val->getS(5)).getI());
+		else
+		{
+		    TValBuf* vb = mod->vbuf(id);
+		    if( !vb )	return;
+		    val->setI(0,vb->getI(&vtm,val->getB(4)));
+		}
+		val->setI(2,vtm/1000000); val->setI(3,vtm%1000000);
+
+		if( !addr.empty() )	mod->varchClose(id);
 	    }
-	    val->setI(2,vtm/1000000); val->setI(3,vtm%1000000);
+	    catch(TError err) { val->setI(0,EVAL_INT); }
 	}
 };
 
@@ -274,11 +329,12 @@ class varhGetR : public TFunction
 	varhGetR( ) : TFunction("varhGetR")
 	{
 	    ioAdd( new IO("val",_("Return value"),IO::Real,IO::Return) );
-	    ioAdd( new IO("id",_("Buffer id"),IO::Integer,IO::Default) );
+	    ioAdd( new IO("id",_("Buffer id"),IO::Integer,IO::Default,"-1") );
 	    ioAdd( new IO("sec",_("Seconds"),IO::Integer,IO::Output) );
 	    ioAdd( new IO("usec",_("Microseconds"),IO::Integer,IO::Output) );
 	    ioAdd( new IO("up_ord",_("Up order"),IO::Boolean,IO::Default,"0") );
 	    ioAdd( new IO("archtor",_("Archivator"),IO::String,IO::Default) );
+	    ioAdd( new IO("addr",_("Address of archive or atribute of parameter"),IO::String,IO::Default) );
 	}
 	
 	string name( )	{ return _("Varch: Get real"); }
@@ -287,16 +343,25 @@ class varhGetR : public TFunction
 	void calc( TValFunc *val )
 	{
 	    int id = val->getI(1);
-	    long long vtm = (long long)val->getI(2)*1000000+val->getI(3);
-	    if(mod->isArch(id))
-		val->setR(0,mod->varch(id).at().getR(&vtm,val->getB(4),val->getS(5)));
-	    else
+	    string addr = val->getS(6);
+	    try
 	    {
-		TValBuf* vb = mod->vbuf(id);
-		if(!vb)     return;
-		val->setR(0,vb->getR(&vtm,val->getB(4)));
+		if( !addr.empty() )	id = mod->varchOpen(addr);
+
+		long long vtm = (long long)val->getI(2)*1000000+val->getI(3);
+		if( mod->isArch(id) )
+		    val->setR(0,mod->varch(id).at().getVal(&vtm,val->getB(4),val->getS(5)).getR());
+		else
+		{
+		    TValBuf* vb = mod->vbuf(id);
+		    if( !vb )	return;
+		    val->setR(0,vb->getR(&vtm,val->getB(4)));
+		}
+		val->setI(2,vtm/1000000); val->setI(3,vtm%1000000);
+
+		if( !addr.empty() )	mod->varchClose(id);
 	    }
-	    val->setI(2,vtm/1000000); val->setI(3,vtm%1000000);
+	    catch(TError err) { val->setR(0,EVAL_REAL); }
 	}
 };
 
@@ -309,11 +374,12 @@ class varhGetB : public TFunction
 	varhGetB( ) : TFunction("varhGetB")
 	{
 	    ioAdd( new IO("val",_("Return value"),IO::Boolean,IO::Return) );
-	    ioAdd( new IO("id",_("Buffer id"),IO::Integer,IO::Default) );
+	    ioAdd( new IO("id",_("Buffer id"),IO::Integer,IO::Default,"-1") );
 	    ioAdd( new IO("sec",_("Seconds"),IO::Integer,IO::Output) );
 	    ioAdd( new IO("usec",_("Microseconds"),IO::Integer,IO::Output) );
 	    ioAdd( new IO("up_ord",_("Up order"),IO::Boolean,IO::Default,"0") );
 	    ioAdd( new IO("archtor",_("Archivator"),IO::String,IO::Default) );
+	    ioAdd( new IO("addr",_("Address of archive or atribute of parameter"),IO::String,IO::Default) );
 	}
 
 	string name( )	{ return _("Varch: Get boolean"); }
@@ -322,16 +388,25 @@ class varhGetB : public TFunction
 	void calc( TValFunc *val )
 	{
 	    int id = val->getI(1);
-	    long long vtm = (long long)val->getI(2)*1000000+val->getI(3);
-	    if(mod->isArch(id))
-		val->setB(0,mod->varch(id).at().getB(&vtm,val->getB(4),val->getS(5)));
-	    else
+	    string addr = val->getS(6);
+	    try
 	    {
-		TValBuf* vb = mod->vbuf(id);
-		if( !vb )	return;
-		val->setB(0,vb->getB(&vtm,val->getB(4)));
+		if( !addr.empty() )	id = mod->varchOpen(addr);
+
+		long long vtm = (long long)val->getI(2)*1000000+val->getI(3);
+		if(mod->isArch(id))
+		    val->setB(0,mod->varch(id).at().getVal(&vtm,val->getB(4),val->getS(5)).getB());
+		else
+		{
+		    TValBuf* vb = mod->vbuf(id);
+		    if( !vb )	return;
+		    val->setB(0,vb->getB(&vtm,val->getB(4)));
+		}
+		val->setI(2,vtm/1000000); val->setI(3,vtm%1000000);
+
+		if( !addr.empty() )	mod->varchClose(id);
 	    }
-	    val->setI(2,vtm/1000000); val->setI(3,vtm%1000000);
+	    catch(TError err) { val->setB(0,EVAL_BOOL); }
 	}
 };
 
@@ -344,11 +419,12 @@ class varhGetS : public TFunction
 	varhGetS( ) : TFunction("varhGetS")
 	{
 	    ioAdd( new IO("val",_("Return value"),IO::String,IO::Return) );
-	    ioAdd( new IO("id",_("Buffer id"),IO::Integer,IO::Default) );
+	    ioAdd( new IO("id",_("Buffer id"),IO::Integer,IO::Default,"-1") );
 	    ioAdd( new IO("sec",_("Seconds"),IO::Integer,IO::Output) );
 	    ioAdd( new IO("usec",_("Microseconds"),IO::Integer,IO::Output) );
 	    ioAdd( new IO("up_ord",_("Up order"),IO::Boolean,IO::Default,"0") );
 	    ioAdd( new IO("archtor",_("Archivator"),IO::String,IO::Default) );
+	    ioAdd( new IO("addr",_("Address of archive or atribute of parameter"),IO::String,IO::Default) );
 	}
 
 	string name( )	{ return _("Varch: Get string"); }
@@ -357,16 +433,25 @@ class varhGetS : public TFunction
 	void calc( TValFunc *val )
 	{
 	    int id = val->getI(1);
-	    long long vtm = (long long)val->getI(2)*1000000+val->getI(3);
-	    if(mod->isArch(id))
-		val->setS(0,mod->varch(id).at().getS(&vtm,val->getB(4),val->getS(5)));
-	    else
+	    string addr = val->getS(6);
+	    try
 	    {
-		TValBuf* vb = mod->vbuf(id);
-		if( !vb )	return;
-		val->setS(0,vb->getS(&vtm,val->getB(4)));
+		if( !addr.empty() )	id = mod->varchOpen(addr);
+
+		long long vtm = (long long)val->getI(2)*1000000+val->getI(3);
+		if(mod->isArch(id))
+		val->setS(0,mod->varch(id).at().getVal(&vtm,val->getB(4),val->getS(5)).getS());
+		else
+		{
+		    TValBuf* vb = mod->vbuf(id);
+		    if( !vb )	return;
+		    val->setS(0,vb->getS(&vtm,val->getB(4)));
+		}
+		val->setI(2,vtm/1000000); val->setI(3,vtm%1000000);
+
+		if( !addr.empty() )	mod->varchClose(id);
 	    }
-	    val->setI(2,vtm/1000000); val->setI(3,vtm%1000000);
+	    catch(TError err) { val->setS(0,EVAL_STR); }
 	}
 };
 
@@ -675,6 +760,185 @@ class strParsePath : public TFunction
 	}
 };
 
+//*************************************************
+//* Path string convert to separated string       *
+//*************************************************
+class strPath2Sep : public TFunction
+{
+    public:
+	strPath2Sep( ) : TFunction("strPath2Sep")
+	{
+	    ioAdd( new IO("rez",_("Rezult"),IO::String,IO::Return) );
+	    ioAdd( new IO("src",_("Source"),IO::String,IO::Default) );
+	    ioAdd( new IO("sep",_("Separator"),IO::String,IO::Default,".") );
+	}
+
+	string name( )	{ return _("String: Path to separated string"); }
+	string descr( )	{ return _("Use for convert path to separated string."); }
+
+	void calc( TValFunc *val )
+	{
+	    val->setS(0,TSYS::path2sepstr(val->getS(1),val->getS(2).size()?val->getS(2)[0]:'.'));
+	}
+};
+
+//*************************************************
+//* String encode to HTML                         *
+//*************************************************
+class strEnc2HTML : public TFunction
+{
+    public:
+	strEnc2HTML( ) : TFunction("strEnc2HTML")
+	{
+	    ioAdd( new IO("rez",_("Rezult"),IO::String,IO::Return) );
+	    ioAdd( new IO("src",_("Source"),IO::String,IO::Default) );
+	}
+
+	string name( )	{ return _("String: Encode string to HTML"); }
+	string descr( )	{ return _("Use for encode string for use into HTML source."); }
+
+	void calc( TValFunc *val )
+	{
+	    val->setS(0,TSYS::strEncode(val->getS(1),TSYS::Html));
+	}
+};
+
+//*************************************************
+//* Convert real to string                        *
+//*************************************************
+class real2str : public TFunction
+{
+    public:
+	real2str( ) : TFunction("real2str")
+	{
+	    ioAdd( new IO("rez",_("Rezult"),IO::String,IO::Return) );
+	    ioAdd( new IO("val",_("Value"),IO::Real,IO::Default) );
+	    ioAdd( new IO("prc",_("Precision"),IO::Integer,IO::Default,"4") );
+	    ioAdd( new IO("tp",_("Type"),IO::String,IO::Default,"f") );
+	}
+
+	string name( )	{ return _("String: Real to string"); }
+	string descr( )	{ return _("Convert real to string."); }
+
+	void calc( TValFunc *val )
+	{
+	    val->setS(0,TSYS::real2str(val->getR(1),val->getI(2),val->getS(3).size()?val->getS(3)[0]:'f') );
+	}
+};
+
+//*************************************************
+//* Convert integer to string                     *
+//*************************************************
+class int2str : public TFunction
+{
+    public:
+	int2str( ) : TFunction("int2str")
+	{
+	    ioAdd( new IO("rez",_("Rezult"),IO::String,IO::Return) );
+	    ioAdd( new IO("val",_("Value"),IO::Integer,IO::Default) );
+	    ioAdd( new IO("base",_("Base"),IO::Integer,IO::Default,"10") );
+	}
+
+	string name( )	{ return _("String: Integer to string"); }
+	string descr( )	{ return _("Convert integer to string."); }
+
+	void calc( TValFunc *val )
+	{
+	    switch( val->getI(2) )
+	    {
+		case 8:	val->setS(0,TSYS::int2str(val->getI(1),TSYS::Oct));	break;
+		case 10:val->setS(0,TSYS::int2str(val->getI(1),TSYS::Dec));	break;
+		case 16:val->setS(0,TSYS::int2str(val->getI(1),TSYS::Hex));	break;
+		default: val->setS(0,"");
+	    }
+	}
+};
+
+//*************************************************
+//* Convert string to real                        *
+//*************************************************
+class str2real : public TFunction
+{
+    public:
+	str2real( ) : TFunction("str2real")
+	{
+	    ioAdd( new IO("rez",_("Rezult"),IO::Real,IO::Return) );
+	    ioAdd( new IO("val",_("Value"),IO::String,IO::Default) );
+	}
+
+	string name( )	{ return _("String: String to real"); }
+	string descr( )	{ return _("Convert string to real."); }
+
+	void calc( TValFunc *val )	{ val->setR(0,atof(val->getS(1).c_str())); }
+};
+
+//*************************************************
+//* Convert string to integer                     *
+//*************************************************
+class str2int : public TFunction
+{
+    public:
+	str2int( ) : TFunction("str2int")
+	{
+	    ioAdd( new IO("rez",_("Rezult"),IO::Integer,IO::Return) );
+	    ioAdd( new IO("val",_("Value"),IO::String,IO::Default) );
+	}
+
+	string name( )	{ return _("String: String to integer"); }
+	string descr( )	{ return _("Convert string to integer."); }
+
+	void calc( TValFunc *val )	{ val->setI(0,strtol(val->getS(1).c_str(),NULL,0)); }
+};
+
+//*************************************************
+//* Split float to words                          *
+//*************************************************
+class floatSplitWord : public TFunction
+{
+    public:
+	floatSplitWord( ) : TFunction("floatSplitWord")
+	{
+	    ioAdd( new IO("val",_("Value"),IO::Real,IO::Default) );
+	    ioAdd( new IO("w1",_("Word 1"),IO::Integer,IO::Output) );
+	    ioAdd( new IO("w2",_("Word 2"),IO::Integer,IO::Output) );
+	}
+
+	string name( )	{ return _("Float: Split to words"); }
+	string descr( )	{ return _("Split float (4 byte) to words (2 byte)."); }
+
+	void calc( TValFunc *val )
+	{
+	    float vl = val->getR(0);
+	    val->setI(1,*((ui16*)&vl));
+	    val->setI(2,*(((ui16*)&vl)+1));
+	}
+};
+
+//*************************************************
+//* Merge float from words                        *
+//*************************************************
+class floatMergeWord : public TFunction
+{
+    public:
+	floatMergeWord( ) : TFunction("floatMergeWord")
+	{
+	    ioAdd( new IO("rez",_("Rezult"),IO::Real,IO::Return) );
+	    ioAdd( new IO("w1",_("Word 1"),IO::Integer,IO::Default) );
+	    ioAdd( new IO("w2",_("Word 2"),IO::Integer,IO::Default) );
+	}
+
+	string name( )	{ return _("Float: Merge from words"); }
+	string descr( )	{ return _("Merge float (4 byte) from words (2 byte)."); }
+
+	void calc( TValFunc *val )
+	{
+	    float vl;
+	    *((ui16*)&vl) = val->getI(1);
+	    *(((ui16*)&vl)+1) = val->getI(2);
+	    val->setR(0,vl);
+	}
+};
+
 }
 
 #endif //SYSFNC_H
