@@ -145,7 +145,7 @@ string TTest::optDescr( )
 	"\n"),
 	MOD_TYPE,MOD_ID,nodePath().c_str());
 
-    return(buf);
+    return buf;
 }
 
 void TTest::load_( )
@@ -165,7 +165,7 @@ void TTest::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -183,7 +183,7 @@ void TTest::modStart(  )
     pthread_create(&pthr_tsk,&pthr_attr,Task,this);
     pthread_attr_destroy(&pthr_attr);
     if( TSYS::eventWait( run_st, true, string(MOD_ID)+": Is starting....",5) )
-	throw TError(nodePath().c_str(),_("No started!"));
+	throw TError(nodePath().c_str(),_("Not started!"));
 }
 
 void TTest::modStop(  )
@@ -191,8 +191,8 @@ void TTest::modStop(  )
     if( !run_st ) return;
 
     endrun = true;
-    if( TSYS::eventWait( run_st, false, string(MOD_ID)+": Is stoping....",5) )
-	throw TError(nodePath().c_str(),_("No stoped!"));
+    if( TSYS::eventWait( run_st, false, string(MOD_ID)+": Is stopping....",5) )
+	throw TError(nodePath().c_str(),_("Not stopped!"));
     pthread_join( pthr_tsk, NULL );
 }
 
@@ -204,9 +204,9 @@ void *TTest::Task( void *CfgM )
     tst->run_st = true;
     tst->endrun = false;
 
-//#if OSC_DEBUG
-//    mess_debug(tst->nodePath().c_str(),_("Thread <%d> started!"),gettid());
-//#endif
+#if OSC_DEBUG >= 2
+    mess_debug(tst->nodePath().c_str(),_("Thread <%u> is started. TID: %ld"),pthread_self(),(long int)syscall(224));
+#endif
 
     //Task counter
     while( !tst->endrun )
@@ -218,23 +218,17 @@ void *TTest::Task( void *CfgM )
 	    if( ++count == 1000000 ) count = 0;
 
 	    //Get All fields
-	    try
-	    {
-		int nd_cnt = 0;
-		while(true)
-		{
-		    XMLNode *t_n = tst->ctrId(&SYS->cfgRoot(),tst->nodePath())->childGet("prm",nd_cnt++);
-
-		    if( t_n->attr("on") == "1" && atoi(t_n->attr("per").c_str()) && 
-			    !( count % atoi(t_n->attr("per").c_str()) ) )
+	    ResAlloc res(SYS->nodeRes(),false);
+	    XMLNode *mn, *t_n;
+	    mn = TCntrNode::ctrId(&SYS->cfgRoot(),tst->nodePath(),true);
+	    if( mn )
+		for( int nd_cnt = 0; t_n=mn->childGet("prm",nd_cnt++,true); )
+		    if( t_n->attr("on") == "1" && atoi(t_n->attr("per").c_str()) && !( count % atoi(t_n->attr("per").c_str()) ) )
 		    {
 			string id = t_n->attr("id");
 			try{ tst->Test( id, t_n ); }
-			catch( TError err )
-			{ mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
+			catch( TError err ) { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
 		    }
-		}
-	    }catch(...){ }
 	}
 	usleep(STD_WAIT_DELAY*1000);
     }
@@ -524,9 +518,9 @@ void TTest::Test( const string &id, XMLNode *t_n )
 
 	//Delete fields
 	st_time = times(NULL);
-	for(int i_fld = 0; i_fld < experem; i_fld++)
+	for( int i_fld = 0; i_fld < experem; i_fld++ )
 	{
-	    bd_cfg.cfg("name").setS("Sh"+SYS->int2str(i_fld));
+	    bd_cfg.cfg("name").setS("Sh"+SYS->int2str(i_fld),true);
 	    tbl.at().fieldDel(bd_cfg);
 	}
 	mess_info(test_cat,"Del %d fields time <%f>sek!",experem,(float)(times(NULL)-st_time)/TSYS::HZ());
@@ -561,7 +555,7 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	    tr.at().outAt(addr).at().setAddr(addr);
 	}
 	if( !tr.at().outAt(addr).at().startStat() ) tr.at().outAt(addr).at().start();
-	int len = tr.at().outAt(addr).at().messIO(req.c_str(),req.size(),buf,sizeof(buf)-1,1);
+	int len = tr.at().outAt(addr).at().messIO(req.c_str(),req.size(),buf,sizeof(buf)-1,1000);
 	tr.at().outAt(addr).at().stop();
 	buf[len] = 0;
 	mess_info(test_cat,"%s: Put <%s>. Get: <%s>",addr.c_str(),req.c_str(),buf);
@@ -996,16 +990,16 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	mess_info(test_cat,"  Update buffer end ok.");
 
 	wtm += buf->period()*(buf->size()+5);
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	buf->setSize(1000);
 	for(int i=0; i<buf->size(); i++)
 	    buf->setS("Test: "+TSYS::int2str(i),wtm+i*buf->period());
-	mess_info(test_cat,"  Write 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Write 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	for(long long i = buf->begin(); i <= buf->end(); i+=buf->period())
 	{ rtm = i; buf->getS(&rtm); }
-	mess_info(test_cat,"  Read 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Read 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
 	mess_info(test_cat,"Test4 passed.");
 	delete buf;
@@ -1056,16 +1050,16 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	mess_info(test_cat,"  Update buffer end ok.");
 
 	wtm += buf->period()*(buf->size()+5);
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	buf->setSize(1000);
 	for(int i=0; i<buf->size(); i++)
 	    buf->setI(i,wtm+i*buf->period());
-	mess_info(test_cat,"  Write 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Write 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	for(long long i = buf->begin(); i <= buf->end(); i+=buf->period())
 	{ rtm = i; buf->getI(&rtm); }
-	mess_info(test_cat,"  Read 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Read 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
 	mess_info(test_cat,"Test5 passed.");
 	delete buf;
@@ -1118,16 +1112,16 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	mess_info(test_cat,"  Update buffer end ok.");
 
 	wtm += buf->period()*(buf->size()+5);
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	buf->setSize(1000);
 	for(int i=0; i<buf->size(); i++)
 	    buf->setS("Test: "+TSYS::int2str(i),wtm+i*buf->period());
-	mess_info(test_cat,"  Write 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Write 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	for(long long i = buf->begin(); i <= buf->end(); i+=buf->period())
 	{ rtm = i; buf->getS(&rtm); }
-	mess_info(test_cat,"  Read 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Read 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
 	mess_info(test_cat,"Test6 passed.");
 	delete buf;
@@ -1180,16 +1174,16 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	mess_info(test_cat,"  Update buffer end ok.");
 
 	wtm += buf->period()*(buf->size()+5);
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	buf->setSize(1000);
 	for(int i=0; i<buf->size(); i++)
 	    buf->setI(i,wtm+i*buf->period());
-	mess_info(test_cat,"  Write 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Write 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	for(long long i = buf->begin(); i <= buf->end(); i+=buf->period())
 	{ rtm = i; buf->getI(&rtm); }
-	mess_info(test_cat,"  Read 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Read 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
 	mess_info(test_cat,"Test7 passed.");
 	delete buf;
@@ -1242,16 +1236,16 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	mess_info(test_cat,"  Update buffer end ok.");
 
 	wtm += buf->period()*(buf->size()+5);
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	buf->setSize(1000);
 	for(int i=0; i<buf->size(); i++)
 	    buf->setS("Test: "+TSYS::int2str(i),wtm+i*buf->period());
-	mess_info(test_cat,"  Write 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Write 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	for(long long i = buf->begin(); i <= buf->end(); i+=buf->period())
 	{ rtm = i; buf->getS(&rtm); }
-	mess_info(test_cat,"  Read 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Read 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
 	mess_info(test_cat,"Test8 passed.");
 	delete buf;
@@ -1304,16 +1298,16 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	mess_info(test_cat,"  Update buffer end ok.");
 
 	wtm += buf->period()*(buf->size()+5);
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	buf->setSize(1000);
 	for(int i=0; i<buf->size(); i++)
 	    buf->setI(i,wtm+i*buf->period());
-	mess_info(test_cat,"  Write 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Write 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	for(long long i = buf->begin(); i <= buf->end(); i+=buf->period())
 	{ rtm = i; buf->getI(&rtm); }
-	mess_info(test_cat,"  Read 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Read 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
 	mess_info(test_cat,"Test9 passed.");
 	delete buf;
@@ -1364,16 +1358,16 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	mess_info(test_cat,"  Update buffer end ok.");
 
 	wtm += wper*(buf->size()+5);
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	buf->setSize(1000);
 	for(int i=0; i<buf->size(); i++)
 	    buf->setS("Test: "+TSYS::int2str(i),wtm+i*wper);
-	mess_info(test_cat,"  Write 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Write 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	for(long long i = buf->end(); i > buf->begin(); i=rtm-1)
 	{ rtm = i; buf->getS(&rtm); }
-	mess_info(test_cat,"  Read 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Read 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
 	mess_info(test_cat,"Test10 passed.");
 	delete buf;
@@ -1422,16 +1416,16 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	mess_info(test_cat,"  Update buffer end ok.");
 
 	wtm += wper*(buf->size()+5);
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	buf->setSize(1000);
 	for(int i=0; i<buf->size(); i++)
 	    buf->setI(i,wtm+i*wper);
-	mess_info(test_cat,"  Write 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Write 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	for(long long i = buf->end(); i > buf->begin(); i=rtm-1)
 	{ rtm = i; buf->getI(&rtm); }
-	mess_info(test_cat,"  Read 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Read 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
 	mess_info(test_cat,"Test11 passed.");
 	delete buf;
@@ -1480,16 +1474,16 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	mess_info(test_cat,"  Update buffer end ok.");
 
 	wtm += wper*(buf->size()+5);
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	buf->setSize(1000);
 	for(int i=0; i<buf->size(); i++)
 	    buf->setS("Test: "+TSYS::int2str(i),wtm+i*wper);
-	mess_info(test_cat,"  Write 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Write 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	for(long long i = buf->end(); i > buf->begin(); i=rtm-1)
 	{ rtm = i; buf->getS(&rtm); }
-	mess_info(test_cat,"  Read 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Read 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
 	mess_info(test_cat,"Test12 passed.");
 	delete buf;
@@ -1538,16 +1532,16 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	mess_info(test_cat,"  Update buffer end ok.");
 
 	wtm += wper*(buf->size()+5);
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	buf->setSize(1000);
 	for(int i=0; i<buf->size(); i++)
 	    buf->setI(i,wtm+i*wper);
-	mess_info(test_cat,"  Write 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Write 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	for(long long i = buf->end(); i > buf->begin(); i=rtm-1)
 	{ rtm = i; buf->getI(&rtm); }
-	mess_info(test_cat,"  Read 1000 values time %f ms!",1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Read 1000 values time %f ms!",1e-3*(TSYS::curTime()-st_cnt));
 
 	mess_info(test_cat,"Test13 passed.");
 	delete buf;
@@ -1569,7 +1563,7 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	mess_info(test_cat,"Test1. Simple fill and check archive.");
 	for( int i_el = 0; i_el < buf_sz; i_el++)
 	    buf.setI((int)pow(10,i_el),wtm+i_el*per);
-	o_arch.at().setVal(buf,buf.begin(),buf.end(),"");
+	o_arch.at().setVals(buf,buf.begin(),buf.end(),"");
 	for( int i_el = 0; i_el < buf_sz+2; i_el++)
 	{
 	    ttm = wtm+i_el*per;
@@ -1583,7 +1577,7 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	mess_info(test_cat,"Test2. Internal insert double value (down).");
 	buf.clear();
 	buf.setI((int)pow(10,2),wtm+3*per);
-	o_arch.at().setVal(buf,wtm+3*per,wtm+3*per,"");
+	o_arch.at().setVals(buf,wtm+3*per,wtm+3*per,"");
 	for( int i_el = 0; i_el < buf_sz+2; i_el++)
 	{
 	    ttm = wtm+i_el*per;
@@ -1597,7 +1591,7 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	mess_info(test_cat,"Test3. Internal insert double value (up).");
 	buf.clear();
 	buf.setI((int)pow(10,4),wtm+3*per);
-	o_arch.at().setVal(buf,wtm+3*per,wtm+3*per,"");
+	o_arch.at().setVals(buf,wtm+3*per,wtm+3*per,"");
 	for( int i_el = 0; i_el < buf_sz+2; i_el++)
 	{
 	    ttm = wtm+i_el*per;
@@ -1611,7 +1605,7 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	mess_info(test_cat,"Test4. Internal insert double value (down).");
 	buf.clear();
 	buf.setI((int)pow(10,2),wtm+3*per);
-	o_arch.at().setVal(buf,wtm+3*per,wtm+3*per,"");
+	o_arch.at().setVals(buf,wtm+3*per,wtm+3*per,"");
 	for( int i_el = 0; i_el < buf_sz+2; i_el++)
 	{
 	    ttm = wtm+i_el*per;
@@ -1625,7 +1619,7 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	mess_info(test_cat,"Test5. Internal insert no double value.");
 	buf.clear();
 	buf.setI((int)pow(10,9),wtm+per);
-	o_arch.at().setVal(buf,wtm+per,wtm+per,"");
+	o_arch.at().setVals(buf,wtm+per,wtm+per,"");
 	for( int i_el = 0; i_el < buf_sz+2; i_el++)
 	{
 	    ttm = wtm+i_el*per;
@@ -1640,7 +1634,7 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	mess_info(test_cat,"Test6. Internal insert double (up) value.");
 	buf.clear();
 	buf.setI((int)pow(10,2),wtm+per);
-	o_arch.at().setVal(buf,wtm+per,wtm+per,"");
+	o_arch.at().setVals(buf,wtm+per,wtm+per,"");
 	for( int i_el = 0; i_el < buf_sz+2; i_el++)
 	{
 	    ttm = wtm+i_el*per;
@@ -1655,7 +1649,7 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	mess_info(test_cat,"Test7. Internal insert value instaead double value.");
 	buf.clear();
 	buf.setI((int)pow(10,3),wtm+3*per);
-	o_arch.at().setVal(buf,wtm+3*per,wtm+3*per,"");
+	o_arch.at().setVals(buf,wtm+3*per,wtm+3*per,"");
 	for( int i_el = 0; i_el < buf_sz+2; i_el++)
 	{
 	    ttm = wtm+i_el*per;
@@ -1672,7 +1666,7 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	buf.clear();
 	for( int i_el = -1; i_el <= 1; i_el++ )
 	    buf.setI(i_el,wtm+i_el*per);
-	o_arch.at().setVal(buf,buf.begin(),buf.end(),"");
+	o_arch.at().setVals(buf,buf.begin(),buf.end(),"");
 	for( int i_el = -1; i_el <= 1; i_el++)
 	{
 	    ttm = wtm+i_el*per;
@@ -1690,20 +1684,17 @@ void TTest::Test( const string &id, XMLNode *t_n )
 	string inbuf, outbuf;
 	for(int i_s = 0; i_s < 256; i_s++) inbuf.push_back((unsigned char)i_s);
 
-	long long st_cnt = SYS->shrtCnt();
+	long long st_cnt = TSYS::curTime();
 	outbuf = TSYS::strEncode(inbuf,TSYS::base64);
-	mess_info(test_cat,"  Code %d size text time %f ms!",inbuf.size(),1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Code %d size text time %f ms!",inbuf.size(),1e-3*(TSYS::curTime()-st_cnt));
 	mess_info(test_cat,(string("  Coded text: ")+outbuf).c_str());
 
-	st_cnt = SYS->shrtCnt();
+	st_cnt = TSYS::curTime();
 	inbuf = TSYS::strDecode(outbuf,TSYS::base64);
-	mess_info(test_cat,"  Encode %d size text time %f ms!",outbuf.size(),1000.*(SYS->shrtCnt()-st_cnt)/SYS->sysClk());
+	mess_info(test_cat,"  Encode %d size text time %f ms!",outbuf.size(),1e-3*(TSYS::curTime()-st_cnt));
 	for(int i_s = 0; i_s < 256; i_s++) 
-	{
-	    //printf("TEST 00: %d -> %d\n",i_s,(unsigned char)inbuf[i_s]);
 	    if((unsigned char)inbuf[i_s] != i_s) 
 		throw TError("","Test1 failed! Coding error!" );
-	}
 	mess_info(test_cat,"Test1 passed.");
     }
 }
