@@ -48,11 +48,11 @@ class TSocketIn;
 class SSockIn
 {
     public:
-	SSockIn( TSocketIn *is_in, int icl_sock, string isender ) :
-	    s_in(is_in), cl_sock(icl_sock), sender(isender)	{  }
+	SSockIn( TSocketIn *is, int icSock, string isender ) :
+	    s(is), cSock(icSock), sender(isender)	{ }
 
-	TSocketIn	*s_in;
-	int		cl_sock;
+	TSocketIn	*s;
+	int		cSock;
 	string		sender;
 };
 
@@ -61,7 +61,7 @@ class SSockIn
 //************************************************
 struct SSockCl
 {
-    pid_t	cl_id;    // Client's pids
+    pthread_t	cl_id;    // Client's thread id
     int		cl_sock;
 };
 
@@ -81,13 +81,15 @@ class TSocketIn: public TTransportIn
 	TSocketIn( string name, const string &idb, TElem *el );
 	~TSocketIn( );
 
+	string getStatus( );
+
 	int maxQueue( )		{ return max_queue; }
-	int maxFork( )		{ return max_fork; }
-	int bufLen( )		{ return buf_len; }
+	int maxFork( )		{ return mMaxFork; }
+	int bufLen( )		{ return mBufLen; }
 
 	void setMaxQueue( int vl )	{ max_queue = vl; modif(); }
-	void setMaxFork( int vl )	{ max_fork = vl; modif(); }
-	void setBufLen( int vl )	{ buf_len = vl; modif(); }
+	void setMaxFork( int vl )	{ mMaxFork = vl; modif(); }
+	void setBufLen( int vl )	{ mBufLen = vl; modif(); }
 
 	void start( );
 	void stop( );
@@ -97,11 +99,10 @@ class TSocketIn: public TTransportIn
 	static void *Task( void * );
 	static void *ClTask( void * );
 
-	void ClSock( SSockIn &s_in );
-	void PutMess( int sock, string &request, string &answer, string sender, AutoHD<TProtocolIn> &prot_in );
+	void messPut( int sock, string &request, string &answer, string sender, AutoHD<TProtocolIn> &prot_in );
 
-	void RegClient( pthread_t thrid, int i_sock );
-	void UnregClient( pthread_t thrid );
+	void clientReg( pthread_t thrid, int i_sock );
+	void clientUnreg( pthread_t thrid );
 
 	void cntrCmdProc( XMLNode *opt );	//Control interface command process
 
@@ -114,8 +115,8 @@ class TSocketIn: public TTransportIn
 	bool		endrun_cl;		// Command for stop client tasks
 
 	int		&max_queue;		// max queue for TCP, UNIX sockets
-	int		&max_fork;		// maximum forking (opened sockets)
-	int		&buf_len;		// input buffer length
+	int		&mMaxFork;		// maximum forking (opened sockets)
+	int		&mBufLen;		// input buffer length
 
 	int		type;			// socket's types
 	string		path;			// path to file socket for UNIX socket
@@ -125,6 +126,10 @@ class TSocketIn: public TTransportIn
 
 	bool		cl_free;		// Clients stoped
 	vector<SSockCl>	cl_id;			// Client's pids
+
+	//- Status atributes -
+	float		trIn, trOut;		// Traffic in and out counter
+	int		connNumb, clsConnByLim;	// Connections number
 };
 
 //************************************************
@@ -143,6 +148,10 @@ class TSocketOut: public TTransportOut
 	TSocketOut( string name, const string &idb, TElem *el );
 	~TSocketOut( );
 
+	string getStatus( );
+	int timeout( );
+	void setTimeout( int vl );
+
 	void start( );
 	void stop( );
 
@@ -159,6 +168,9 @@ class TSocketOut: public TTransportOut
 	struct sockaddr_in	name_in;
 	struct sockaddr_un	name_un;
 	Res			wres;
+
+	//- Status atributes -
+	float	trIn, trOut;	// Traffic in and out counter
 };
 
 //************************************************
@@ -182,9 +194,6 @@ class TTransSock: public TTipTransport
 	void cntrCmdProc( XMLNode *opt );       //Control interface command process
 
 	void postEnable( int flag );
-
-	//Attributes
-	static const char	*i_cntr;
 };
 
 extern TTransSock *mod;
