@@ -39,7 +39,7 @@
 #define MOD_TYPE	"UI"
 #define VER_TYPE	VER_UI
 #define SUB_TYPE	"QT"
-#define VERSION		"1.6.0"
+#define VERSION		"1.8.0"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Allow the QT based OpenSCADA system configurator."
 #define LICENSE		"GPL"
@@ -119,7 +119,7 @@ string TUIMod::optDescr( )
 
 void TUIMod::load_( )
 {
-#if OSC_DEBUG
+#if OSC_DEBUG >= 1
     mess_debug(nodePath().c_str(),_("Load module."));
 #endif
 
@@ -138,7 +138,7 @@ void TUIMod::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -150,7 +150,7 @@ void TUIMod::load_( )
 
 void TUIMod::save_( )
 {
-#if OSC_DEBUG
+#if OSC_DEBUG >= 1
     mess_debug(nodePath().c_str(),_("Save module."));
 #endif
 
@@ -182,7 +182,7 @@ QMainWindow *TUIMod::openWindow( )
 	    if( rez == DlgUser::SelCancel )     return NULL;
 	    if( rez == DlgUser::SelErr )
 	    {
-		postMess(nodePath().c_str(),_("Auth wrong!!!"));
+		postMess(nodePath().c_str(),_("Auth is wrong!!!"));
 		continue;
 	    }
 	    user_open = d_usr.user().toAscii().data();
@@ -193,7 +193,7 @@ QMainWindow *TUIMod::openWindow( )
 
 void TUIMod::modStart( )
 {
-#if OSC_DEBUG
+#if OSC_DEBUG >= 1
     mess_debug(nodePath().c_str(),_("Start module."));
 #endif
 
@@ -203,7 +203,7 @@ void TUIMod::modStart( )
 
 void TUIMod::modStop( )
 {
-#if OSC_DEBUG
+#if OSC_DEBUG >= 1
     mess_debug(nodePath().c_str(),_("Stop module."));
 #endif
 
@@ -260,8 +260,8 @@ void TUIMod::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path == "/prm/cfg/host_lnk" && ctrChkNode(opt,"get",0660,"root","root",SEQ_RD) )
     {
-        SYS->transport().at().setSysHost(false);
-        opt->setText("/Transport");
+	SYS->transport().at().setSysHost(false);
+	opt->setText("/Transport");
     }
     else if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440) )   opt->setText(optDescr());
     else if( a_path == "/prm/cfg/u_lst" && ctrChkNode(opt) )
