@@ -23,6 +23,7 @@
 #include <unistd.h>
 
 #include <QMainWindow>
+#include <QMenuBar>
 #include <QToolBar>
 #include <QAction>
 #include <QImage>
@@ -44,7 +45,7 @@
 #define MOD_NAME	"QT GUI starter"
 #define MOD_TYPE	"UI"
 #define VER_TYPE	VER_UI
-#define VERSION		"1.5.1"
+#define VERSION		"1.5.3"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Allow QT GUI starter. It is single for all QT GUI modules!"
 #define LICENSE		"GPL"
@@ -85,6 +86,28 @@ TUIMod::TUIMod( string name ) : end_run(false), demon_mode(false), start_com(fal
     mSource	= name;
 
     mod		= this;
+
+    //> Massages not for compile but for indexing by gettext
+#if 0
+    char mess[][100] =
+    {
+	"&Yes","&No","Cancel","&OK","Apply","Close","Back","Forward","Parent Directory",
+	"Look in:","Computer","File","Folder","File &name:","Open","&Open","Cancel","Save","&Save","Date Modified","All Files (*)",
+	"Create New Folder","List View","Detail View","Files of type:","New Folder","&New Folder","Show &hidden files","&Delete","&Rename","Remove",
+	"&Undo","&Redo","Cu&t","&Copy","&Paste","Delete","Select All","Insert Unicode control character",
+	"%1 bytes","%1 KB",
+	"Are sure you want to delete '%1'?",
+	"<p>This program uses Qt Open Source Edition version %1.</p><p>Qt Open Source Edition is intended for the development of Open Source applications. You need a commercial Qt license for development of proprietary (closed source) applications.</p><p>Please see <a href=\"http://www.trolltech.com/company/model/\">www.trolltech.com/company/model/</a> for an overview of Qt licensing.</p>",
+	"<h3>About Qt</h3>%1<p>Qt is a C++ toolkit for cross-platform application development.</p><p>Qt provides single-source portability across MS&nbsp;Windows, Mac&nbsp;OS&nbsp;X, Linux, and all major commercial Unix variants. Qt is also available for embedded devices as Qt for Embedded Linux and Qt for Windows CE.</p><p>Qt is a Nokia product. See <a href=\"http://www.trolltech.com/qt/\">www.trolltech.com/qt/</a> for more information.</p>",
+	"Hu&e:","&Sat:","&Val:","&Red:","&Green:","Bl&ue:","A&lpha channel:","&Basic colors","&Custom colors","&Add to Custom Colors","Select color",
+	"Form","Printer","&Name:","P&roperties","Location:","Preview","Type:","Output &file:","Print range","Print all",
+	"Pages from","to","Selection","Output Settings","Copies:","Collate","Reverse","Copies","Color Mode","Color","Grayscale",
+	"Duplex Printing","None","Long side","Short side","Options","&Options >>","&Options <<","&Print","Print to File (PDF)","Print to File (Postscript)",
+	"Local file","Write %1 file","Paper","Page size:","Width:","Height:","Paper source:","Orientation","Portrait","Landscape","Reverse landscape",
+	"Reverse portrait","Margins","top margin","left margin","right margin","bottom margin","Points (pt)","Inches (in)",
+	"Millimeters (mm)","Centimeters (cm)","Page","Advanced"
+    };
+#endif
 }
 
 TUIMod::~TUIMod()
@@ -135,7 +158,7 @@ void TUIMod::postEnable( int flag )
 	    pthread_create(&pthr_tsk,&pthr_attr,Task,this);
 	    pthread_attr_destroy(&pthr_attr);
 	    if( TSYS::eventWait( run_st, true, nodePath()+"start",5) )
-		throw TError(nodePath().c_str(),_("QT main thread no started!"));
+		throw TError(nodePath().c_str(),_("QT main thread is not started!"));
 	}
     }
 }
@@ -146,14 +169,14 @@ void TUIMod::postDisable( int flag )
     {
 	end_run = true;
 	if( TSYS::eventWait( run_st, false, nodePath()+"stop",5) )
-	    throw TError(nodePath().c_str(),_("QT main thread no stoped!"));
+	    throw TError(nodePath().c_str(),_("QT main thread is not stopped!"));
 	pthread_join(pthr_tsk,NULL);
     }
 }
 
 void TUIMod::load_( )
 {
-#if OSC_DEBUG
+#if OSC_DEBUG >= 1
     mess_debug(nodePath().c_str(),_("Load module."));
 #endif
 
@@ -173,7 +196,7 @@ void TUIMod::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case 'd': demon_mode = true; break;
 	    case -1 : break;
 	}
@@ -185,7 +208,7 @@ void TUIMod::load_( )
 
 void TUIMod::save_( )
 {
-#if OSC_DEBUG
+#if OSC_DEBUG >= 1
     mess_debug(nodePath().c_str(),_("Save module."));
 #endif
 
@@ -194,7 +217,7 @@ void TUIMod::save_( )
 
 void TUIMod::modStart()
 {
-#if OSC_DEBUG
+#if OSC_DEBUG >= 1
     mess_debug(nodePath().c_str(),_("Start module."));
 #endif
 
@@ -203,7 +226,7 @@ void TUIMod::modStart()
 
 void TUIMod::modStop()
 {
-#if OSC_DEBUG
+#if OSC_DEBUG >= 1
     mess_debug(nodePath().c_str(),_("Stop module."));
 #endif
 
@@ -231,20 +254,24 @@ void *TUIMod::Task( void * )
     time_t st_time = time(NULL);
     vector<TMess::SRec> recs;
 
-//#if OSC_DEBUG
-//    mess_debug(mod->nodePath().c_str(),_("Thread <%d> started!"),gettid());
-//#endif
+#if OSC_DEBUG >= 2
+    mess_debug(mod->nodePath().c_str(),_("Thread <%u> is started. TID: %ld"),pthread_self(),(long int)syscall(224));
+#endif
 
-    //- QT application object init -
+    //> QT application object init
     QApplication *QtApp = new QApplication( (int&)SYS->argc,(char **)SYS->argv );
-    setlocale(LC_NUMERIC,"C");		//Standart numeric separator restore
     QtApp->setQuitOnLastWindowClosed(false);
     mod->run_st = true;
 
-    //- Start splash create -
+    //> Create I18N translator
+    I18NTranslator translator;
+    QtApp->installTranslator(&translator);
+
+    //> Start splash create
     if( !ico_t.load(TUIS::icoPath("splash").c_str()) )	ico_t.load(":/images/splash.png");
     QSplashScreen *splash = new QSplashScreen(QPixmap::fromImage(ico_t));
     splash->show();
+
     while( !mod->startCom( ) && !mod->endRun( ) )
     {
 	SYS->archive().at().messGet( st_time, time(NULL), recs, "", TMess::Debug, BUF_ARCH_NM );
@@ -257,7 +284,7 @@ void *TUIMod::Task( void * )
     }
     delete splash;
 
-    //- Start external modules -
+    //> Start external modules
     WinControl *winCntr = new WinControl( );
 
     int op_wnd = 0;
@@ -266,7 +293,7 @@ void *TUIMod::Task( void * )
 	if( mod->owner().modAt(list[i_l]).at().modInfo("SubType") == "QT" &&
 		mod->owner().modAt(list[i_l]).at().modFuncPresent("QMainWindow *openWindow();") )
 	{
-	    //-- Search module into start list --
+	    //>> Search module into start list
 	    int i_off = 0;
 	    string s_el;
 	    while( (s_el=TSYS::strSepParse(mod->start_mod,0,';',&i_off)).size() )
@@ -275,14 +302,15 @@ void *TUIMod::Task( void * )
 		if( winCntr->callQTModule(list[i_l]) ) op_wnd++;
 	}
 
-    //- Start call dialog -
-    if(!op_wnd) winCntr->startDialog( );
+    //> Start call dialog
+    if( QApplication::topLevelWidgets().isEmpty() ) winCntr->startDialog( );
 
     QObject::connect( QtApp, SIGNAL(lastWindowClosed()), winCntr, SLOT(lastWinClose()) );
+
     QtApp->exec();
     delete winCntr;
 
-    //- Stop splash create -
+    //> Stop splash create
     if( !ico_t.load(TUIS::icoPath("splash_exit").c_str()) )	ico_t.load(":/images/splash.png");
     splash = new QSplashScreen(QPixmap::fromImage(ico_t));
     splash->show();
@@ -299,7 +327,7 @@ void *TUIMod::Task( void * )
     }
     delete splash;
 
-    //- QT application object free -
+    //> QT application object free
     delete QtApp;
     first_ent = false;
 
@@ -377,11 +405,18 @@ bool WinControl::callQTModule( const string &nm )
     QMainWindow *new_wnd = ((&qt_mod.at())->*openWindow)( );
     if( !new_wnd ) return false;
 
-    //- Make QT starter toolbar -
-    QToolBar *toolBar = new QToolBar(_("QTStarter toolbar"), new_wnd);
-    toolBar->setObjectName("QTStarterTool");
-    new_wnd->addToolBar(toolBar);
-    //, Qt::DockTop );
+    //> Make QT starter toolbar
+    QToolBar *toolBar = NULL;
+    QMenu *menu = NULL;
+    if( !new_wnd->property("QTStarterToolDis").toBool() )
+    {
+	toolBar = new QToolBar("QTStarter",new_wnd);
+	toolBar->setObjectName("QTStarterTool");
+	new_wnd->addToolBar(toolBar);
+    }
+    if( !new_wnd->property("QTStarterMenuDis").toBool() && !new_wnd->menuBar()->actions().empty() )
+	menu = new_wnd->menuBar()->addMenu("QTStarter");
+
     mod->owner().modList(list);
     for( unsigned i_l = 0; i_l < list.size(); i_l++ )
 	if( mod->owner().modAt(list[i_l]).at().modInfo("SubType") == "QT" &&
@@ -399,12 +434,13 @@ bool WinControl::callQTModule( const string &nm )
 	else icon = QIcon(":/images/oscada_qt.png");
 	QAction *act_1 = new QAction(icon,qt_mod.at().modName().c_str(),new_wnd);
 	act_1->setObjectName(list[i_l].c_str());
-	act_1->setShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_1);
+	//act_1->setShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_1);
 	act_1->setToolTip(qt_mod.at().modName().c_str());
 	act_1->setWhatsThis(qt_mod.at().modInfo("Descript").c_str());
 	QObject::connect(act_1, SIGNAL(activated()), this, SLOT(callQTModule()));
 
-	toolBar->addAction(act_1);
+	if( toolBar ) toolBar->addAction(act_1);
+	if( menu ) menu->addAction(act_1);
     }
 
     new_wnd->show();
@@ -439,7 +475,7 @@ void WinControl::startDialog( )
 	}
 	else icon = QIcon(":/images/oscada_qt.png");
 
-	AutoHD<TModule> qt_mod = mod->owner().modAt(list[i_l]);	
+	AutoHD<TModule> qt_mod = mod->owner().modAt(list[i_l]);
 	QPushButton *butt = new QPushButton(icon,qt_mod.at().modName().c_str(),new_wnd->centralWidget());
 	butt->setObjectName(list[i_l].c_str());
 	QObject::connect(butt, SIGNAL(clicked(bool)), this, SLOT(callQTModule()));
@@ -460,3 +496,28 @@ void WinControl::startDialog( )
 
     new_wnd->show();
 }
+
+//*************************************************
+//* I18NTranslator                                *
+//*************************************************
+I18NTranslator::I18NTranslator( ) : QTranslator(0)
+{
+
+}
+
+bool I18NTranslator::isEmpty( ) const
+{
+    return false;
+}
+
+QString I18NTranslator::translate( const char *context, const char *sourceText, const char *comment ) const
+{
+    if( !sourceText ) return "";
+
+#if OSC_DEBUG >= 3
+    if( string(sourceText) == _(sourceText) )
+	mess_debug(mod->nodePath().c_str(),_("No translated QT message: '%s'"),sourceText);
+#endif
+
+    return _(sourceText);
+}
