@@ -34,10 +34,10 @@ using namespace VCA;
 //* WidgetLib: Widgets library                   *
 //************************************************
 WidgetLib::WidgetLib( const string &id, const string &name, const string &lib_db ) :
-    TConfig(&mod->elWdgLib()), m_enable(false), m_id(cfg("ID").getSd()), m_name(cfg("NAME").getSd()),
+    TConfig(&mod->elWdgLib()), mEnable(false), mId(cfg("ID").getSd()), m_name(cfg("NAME").getSd()),
     m_descr(cfg("DESCR").getSd()), m_ico(cfg("ICO").getSd()), m_dbt(cfg("DB_TBL").getSd()), work_lib_db(lib_db)
 {
-    m_id = id;
+    mId = id;
     m_name = name;
     m_dbt = string("wlb_")+id;
     m_wdg = grpAdd("wdg_",(id=="originals")?true:false);
@@ -54,9 +54,9 @@ TCntrNode &WidgetLib::operator=( TCntrNode &node )
     if( !src_n ) return *this;
 
     //- Copy generic configuration -
-    string tid = m_id;
+    string tid = mId;
     *(TConfig*)this = *(TConfig*)src_n;
-    m_id  = tid;
+    mId  = tid;
     m_dbt = string("wlb_")+tid;
     work_lib_db = src_n->work_lib_db;
 
@@ -98,23 +98,23 @@ void WidgetLib::postDisable( int flag )
 {
     if( flag )
     {
-	//- Delete libraries record -
-	SYS->db().at().dataDel(DB()+"."+mod->wlbTable(),mod->nodePath()+"LIB/",*this);
+	//> Delete libraries record
+	SYS->db().at().dataDel(DB()+"."+mod->wlbTable(),mod->nodePath()+"LIB/",*this,true);
 
-	//- Delete function's files -
-	//-- Delete widgets table --
+	//> Delete function's files
+	//>> Delete widgets table
 	SYS->db().at().open(fullDB());
 	SYS->db().at().close(fullDB(),true);
-	//-- Delete attributes table --
+	//>> Delete attributes table
 	SYS->db().at().open(fullDB()+"_io");
 	SYS->db().at().close(fullDB()+"_io",true);
-	//-- Delete user attributes table --
+	//>> Delete user attributes table
 	SYS->db().at().open(fullDB()+"_uio");
 	SYS->db().at().close(fullDB()+"_uio",true);
-	//-- Delete include widgets table --
+	//>> Delete include widgets table
 	SYS->db().at().open(fullDB()+"_incl");
 	SYS->db().at().close(fullDB()+"_incl",true);
-	//-- Delete mime-data table --
+	//>> Delete mime-data table
 	SYS->db().at().open(fullDB()+"_mime");
 	SYS->db().at().close(fullDB()+"_mime",true);
     }
@@ -122,7 +122,7 @@ void WidgetLib::postDisable( int flag )
 
 string WidgetLib::name( )
 {
-    return (m_name.size())?m_name:m_id;
+    return (m_name.size())?m_name:mId;
 }
 
 void WidgetLib::setFullDB( const string &it )
@@ -134,20 +134,18 @@ void WidgetLib::setFullDB( const string &it )
 
 void WidgetLib::load_( )
 {
-#if OSC_DEBUG
-    mess_debug(nodePath().c_str(),_("Load widget library."));
-#endif
+    if( !SYS->chkSelDB(DB()) ) return;
+
+    mess_info(nodePath().c_str(),_("Load widget library."));
 
     SYS->db().at().dataGet(DB()+"."+mod->wlbTable(),mod->nodePath()+"LIB/",*this);
 
-    //- Create new widgets -
+    //> Create new widgets
     TConfig c_el(&mod->elWdg());
     c_el.cfgViewAll(false);
-    int fld_cnt = 0;
-    while( SYS->db().at().dataSeek(fullDB(),mod->nodePath()+tbl(), fld_cnt++,c_el) )
+    for( int fld_cnt = 0; SYS->db().at().dataSeek(fullDB(),mod->nodePath()+tbl(), fld_cnt++,c_el); )
     {
 	string f_id = c_el.cfg("ID").getS();
-	c_el.cfg("ID").setS("");
 	if( !present(f_id) )	add(f_id,"","");
     }
 
@@ -156,9 +154,7 @@ void WidgetLib::load_( )
 
 void WidgetLib::save_( )
 {
-#if OSC_DEBUG
-    mess_debug(nodePath().c_str(),_("Save widget library."));
-#endif
+    mess_info(nodePath().c_str(),_("Save widget library."));
 
     SYS->db().at().dataSet(DB()+"."+mod->wlbTable(),mod->nodePath()+"LIB/",*this);
 
@@ -182,9 +178,7 @@ void WidgetLib::setEnable( bool val )
 {
     if( val == enable() )       return;
 
-#if OSC_DEBUG
-    mess_debug(nodePath().c_str(),val ? _("Enable widget library.") : _("Disable widget library."));
-#endif
+    mess_info(nodePath().c_str(),val ? _("Enable widget library.") : _("Disable widget library."));
 
     vector<string> f_lst;
     list(f_lst);
@@ -193,7 +187,7 @@ void WidgetLib::setEnable( bool val )
 	catch( TError err )
 	{ mess_err(nodePath().c_str(),_("Enable/disable widget '%s' error %s."),f_lst[i_ls].c_str(),err.mess.c_str()); }
 
-    m_enable = val;
+    mEnable = val;
 }
 
 void WidgetLib::mimeDataList( vector<string> &list, const string &idb )
@@ -205,10 +199,7 @@ void WidgetLib::mimeDataList( vector<string> &list, const string &idb )
 
     list.clear( );
     for( int fld_cnt = 0; SYS->db().at().dataSeek(wdb+"."+wtbl,mod->nodePath()+wtbl,fld_cnt,c_el); fld_cnt++ )
-    {
 	list.push_back( c_el.cfg("ID").getS() );
-	c_el.cfg("ID").setS("");
-    }
 }
 
 bool WidgetLib::mimeDataGet( const string &iid, string &mimeType, string *mimeData, const string &idb )
@@ -221,7 +212,7 @@ bool WidgetLib::mimeDataGet( const string &iid, string &mimeType, string *mimeDa
 	//- Get resource file from DB -
 	string dbid = is_res ? iid.substr(4) : iid;
 	string wtbl = tbl()+"_mime";
-	string wdb  = idb.empty() ? DB() : idb;	
+	string wdb  = idb.empty() ? DB() : idb;
 	TConfig c_el( &mod->elWdgData() );
 	if( !mimeData ) c_el.cfg("DATA").setView(false);
 	c_el.cfg("ID").setS( dbid );
@@ -270,7 +261,7 @@ void WidgetLib::mimeDataDel( const string &iid, const string &idb )
     string wtbl = tbl()+"_mime";
     string wdb  = idb.empty() ? DB() : idb;
     TConfig c_el( &mod->elWdgData() );
-    c_el.cfg("ID").setS(iid);
+    c_el.cfg("ID").setS(iid,true);
     SYS->db().at().dataDel( wdb+"."+wtbl, mod->nodePath()+wtbl, c_el );
 }
 
@@ -297,10 +288,11 @@ void WidgetLib::cntrCmdProc( XMLNode *opt )
     //- Get page info -
     if( opt->name() == "info" )
     {
+	TCntrNode::cntrCmdProc(opt);
 	ctrMkNode("oscada_cntr",opt,-1,"/",_("Widget's library: ")+id(),RWRWR_,"root","UI");
 	if(ico().size()) ctrMkNode("img",opt,-1,"/ico","",R_R_R_);
 	if(ctrMkNode("branches",opt,-1,"/br","",R_R_R_))
-	    ctrMkNode("grp",opt,-1,"/br/wdg_",_("Widget"),RWRWR_,"root","UI",1,"idm","1");
+	    ctrMkNode("grp",opt,-1,"/br/wdg_",_("Widget"),RWRWR_,"root","UI",2,"idm","1","idSz","30");
 	if(ctrMkNode("area",opt,-1,"/obj",_("Library")))
 	{
 	    if(ctrMkNode("area",opt,-1,"/obj/st",_("State")))
@@ -312,13 +304,13 @@ void WidgetLib::cntrCmdProc( XMLNode *opt )
 	    if(ctrMkNode("area",opt,-1,"/obj/cfg",_("Config")))
 	    {
 		ctrMkNode("fld",opt,-1,"/obj/cfg/id",_("Id"),R_R_R_,"root","UI",1,"tp","str");
-		ctrMkNode("fld",opt,-1,"/obj/cfg/name",_("Name"),RWRWR_,"root","UI",1,"tp","str");
-		ctrMkNode("fld",opt,-1,"/obj/cfg/descr",_("Description"),RWRWR_,"root","UI",3,"tp","str","cols","50","rows","3");
+		ctrMkNode("fld",opt,-1,"/obj/cfg/name",_("Name"),RWRWR_,"root","UI",2,"tp","str","len","50");
+		ctrMkNode("fld",opt,-1,"/obj/cfg/descr",_("Description"),RWRWR_,"root","UI",3,"tp","str","cols","100","rows","3");
 		ctrMkNode("img",opt,-1,"/obj/cfg/ico",_("Icon"),RWRWR_,"root","UI",2,"v_sz","64","h_sz","64");
 	    }
 	}
 	if(ctrMkNode("area",opt,-1,"/wdg",_("Widgets")))
-	    ctrMkNode("list",opt,-1,"/wdg/wdg",_("Widgets"),RWRWR_,"root","UI",4,"tp","br","idm","1","s_com","add,del","br_pref","wdg_");
+	    ctrMkNode("list",opt,-1,"/wdg/wdg",_("Widgets"),RWRWR_,"root","UI",5,"tp","br","idm","1","s_com","add,del","br_pref","wdg_","idSz","30");
 	if(ctrMkNode("area",opt,-1,"/mime",_("Mime data")))
 	    if(ctrMkNode("table",opt,-1,"/mime/mime",_("Mime data"),RWRWR_,"root","UI",2,"s_com","add,del","key","id"))
 	    {
@@ -368,8 +360,8 @@ void WidgetLib::cntrCmdProc( XMLNode *opt )
 	}
 	if( ctrChkNode(opt,"add",RWRWR_,"root","UI",SEQ_WR) )
 	{
-	    add(opt->attr("id").c_str(),opt->text().c_str());
-	    at(opt->attr("id")).at().setOwner(opt->attr("user"));
+	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::ID);
+	    add(vid,opt->text().c_str()); at(vid).at().setOwner(opt->attr("user"));
 	}
 	if( ctrChkNode(opt,"del",RWRWR_,"root","UI",SEQ_WR) ) del(opt->attr("id"),true);
     }
@@ -443,8 +435,8 @@ void WidgetLib::cntrCmdProc( XMLNode *opt )
 LWidget::LWidget( const string &iid, const string &isrcwdg ) :
 	Widget(iid), TConfig(&mod->elWdg()),
 	m_ico(cfg("ICO").getSd()), m_proc(cfg("PROC").getSd()), m_proc_per(cfg("PROC_PER").getId()),
-	m_owner(cfg("USER").getSd()), m_grp(cfg("GRP").getSd()), m_permit(cfg("PERMIT").getId()),
-	m_parent(cfg("PARENT").getSd()), m_attrs(cfg("ATTRS").getSd())
+	m_owner(cfg("USER").getSd()), m_grp(cfg("GRP").getSd()), mPermit(cfg("PERMIT").getId()),
+	mParent(cfg("PARENT").getSd()), m_attrs(cfg("ATTRS").getSd())
 {
     cfg("ID").setS(id());
 
@@ -463,7 +455,29 @@ WidgetLib &LWidget::ownerLib( )
 
 void LWidget::postDisable( int flag )
 {
+    if( flag )
+    {
+	string db  = ownerLib().DB();
+	string tbl = ownerLib().tbl();
 
+	//> Remove from library table
+	SYS->db().at().dataDel( db+"."+tbl, mod->nodePath()+tbl, *this, true );
+
+	//> Remove widget's IO from library IO table
+	TConfig c_el( &mod->elWdgIO() );
+	c_el.cfg("IDW").setS( id(), true );
+	SYS->db().at().dataDel( db+"."+tbl+"_io", mod->nodePath()+tbl+"_io", c_el );
+
+	//> Remove widget's user IO from library IO table
+	c_el.setElem( &mod->elWdgUIO() );
+	c_el.cfg("IDW").setS( id(), true );
+	SYS->db().at().dataDel( db+"."+tbl+"_uio", mod->nodePath()+tbl+"_uio", c_el );
+
+	//> Remove widget's included widgets from library include table
+	c_el.setElem( &mod->elInclWdg() );
+	c_el.cfg("IDW").setS( id(), true );
+	SYS->db().at().dataDel( db+"."+tbl+"_incl", mod->nodePath()+tbl+"_incl", c_el );
+    }
 }
 
 string LWidget::path( )
@@ -548,20 +562,13 @@ int LWidget::calcPer( )
 
 void LWidget::setCalcLang( const string &ilng )
 {
-    m_proc = m_proc.replace(0,m_proc.find("\n"),ilng);
+    m_proc = ilng+"\n"+calcProg();
     modif();
 }
 
 void LWidget::setCalcProg( const string &iprg )
 {
-    string tmp_prg = m_proc;
-    int lng_end = tmp_prg.find("\n");
-    if(lng_end == string::npos)
-    {
-	tmp_prg = tmp_prg+"\n";
-	lng_end = tmp_prg.find("\n");
-    }
-    m_proc = tmp_prg.replace(lng_end+1,string::npos,iprg);
+    m_proc = calcLang()+"\n"+iprg;
     modif();
 }
 
@@ -573,77 +580,88 @@ void LWidget::setCalcPer( int vl )
 
 void LWidget::load_( )
 {
-    //- Load generic widget's data -
+    if( !SYS->chkSelDB(ownerLib().DB()) ) return;
+
+    //> Load generic widget's data
     string db  = ownerLib().DB();
     string tbl = ownerLib().tbl();
     SYS->db().at().dataGet( db+"."+tbl, mod->nodePath()+tbl, *this );
 
+    //> Inherit modify attributes
+    vector<string> als;
+    attrList( als );
+    for( int i_a = 0; i_a < als.size(); i_a++ )
+    {
+	if( !attrPresent(als[i_a]) ) continue;
+	AutoHD<Attr> attr = attrAt(als[i_a]);
+	if( attr.at().modif() && m_attrs.find(als[i_a]) == string::npos )
+	{
+	    attr.at().setModif(0);
+	    inheritAttr(als[i_a]);
+	}
+    }
+
+    //> Load generic attributes
+    tbl = tbl+"_io";
+    TConfig c_el(&mod->elWdgIO());
+    c_el.cfg("IDW").setS(id());
+    string tstr;
+    for( int off = 0; !(tstr = TSYS::strSepParse(m_attrs,0,';',&off)).empty(); )
+    {
+	if( !attrPresent(tstr) )    continue;
+	AutoHD<Attr> attr = attrAt(tstr);
+	if( !(attr.at().flgGlob()&Attr::Generic) ) continue;
+	c_el.cfg("ID").setS(tstr);
+	if( !SYS->db().at().dataGet(db+"."+tbl,mod->nodePath()+tbl,c_el) ) continue;
+	attr.at().setS(c_el.cfg("IO_VAL").getS(),true);
+	attr.at().setFlgSelf((Attr::SelfAttrFlgs)c_el.cfg("SELF_FLG").getI());
+	attr.at().setCfgTempl(c_el.cfg("CFG_TMPL").getS());
+	attr.at().setCfgVal(c_el.cfg("CFG_VAL").getS());
+    }
+
+    //> Other attributes load
     loadIO();
 }
 
 void LWidget::loadIO( )
 {
-    string tstr;
-    vector<string> als;
-
     if( !enable() ) return;
 
-    //- Load widget's work attributes -
+    //> Load widget's work attributes
     string db  = ownerLib().DB();
     string tbl = ownerLib().tbl()+"_io";
 
-    //- Inherit modify attributes -
-    /*attrList( als );
-    for( int i_a = 0; i_a < als.size(); i_a++ )
-    {
-        AutoHD<Attr> attr = attrAt(als[i_a]);
-        if( attr.at().flgGlob()&Attr::IsInher && attr.at().modif() && m_attrs.find(als[i_a]) == string::npos )
-        {
-            attr.at().setModif(0);
-            inheritAttr(als[i_a]);
-        }
-    }
-    als.clear();*/
-    bool full_ls = false;
-    if( m_attrs == "*" )	{ attrList( als ); full_ls = true; }
-    else
-	for( int off = 0; !(tstr = TSYS::strSepParse(m_attrs,0,';',&off)).empty(); )
-	    als.push_back(tstr);
-
-    //-- Same attributes load --
+    //> Same attributes load
     TConfig c_el(&mod->elWdgIO());
     c_el.cfg("IDW").setS(id());
-    for( int i_a = 0; i_a < als.size(); i_a++ )
+    string tstr;
+    for( int off = 0; !(tstr = TSYS::strSepParse(m_attrs,0,';',&off)).empty(); )
     {
-	if( !attrPresent(als[i_a]) )    continue;
-	AutoHD<Attr> attr = attrAt(als[i_a]);
-	if( !(attr.at().flgGlob()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser ) continue;
-	c_el.cfg("ID").setS(als[i_a]);
+	if( !attrPresent(tstr) )    continue;
+	AutoHD<Attr> attr = attrAt(tstr);
+	if( attr.at().flgGlob()&Attr::Generic || (!(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) ) continue;
+	c_el.cfg("ID").setS(tstr);
 	if( !SYS->db().at().dataGet(db+"."+tbl,mod->nodePath()+tbl,c_el) ) continue;
 	attr.at().setS(c_el.cfg("IO_VAL").getS());
 	attr.at().setFlgSelf((Attr::SelfAttrFlgs)c_el.cfg("SELF_FLG").getI());
 	attr.at().setCfgTempl(c_el.cfg("CFG_TMPL").getS());
 	attr.at().setCfgVal(c_el.cfg("CFG_VAL").getS());
-	if( full_ls && attr.at().flgGlob()&Attr::Active ) attrList( als );
     }
 
-    //- Load widget's user attributes -
+    //> Load widget's user attributes
     tbl = ownerLib().tbl()+"_uio";
     c_el.setElem(&mod->elWdgUIO());
-    c_el.cfg("IDW").setS(id());
-    c_el.cfg("ID").setS("");
-    int fld_cnt = 0;
-    while( SYS->db().at().dataSeek(db+"."+tbl,mod->nodePath()+tbl,fld_cnt++,c_el) )
+    c_el.cfg("IDW").setS(id(),true);
+    for( int fld_cnt = 0; SYS->db().at().dataSeek(db+"."+tbl,mod->nodePath()+tbl,fld_cnt++,c_el); )
     {
 	string sid = c_el.cfg("ID").getS();
 	unsigned flg = c_el.cfg("IO_TYPE").getI();
-	c_el.cfg("ID").setS("");
 
 	if( !TSYS::pathLev(sid,1).empty() ) continue;
 	if( !attrPresent(sid) )
 	    attrAdd( new TFld(sid.c_str(),c_el.cfg("NAME").getS().c_str(),(TFld::Type)(flg&0x0f),flg>>4) );
 	AutoHD<Attr> attr = attrAt(sid);
-	if( !(!(attr.at().flgGlob()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) ) continue;
+	if( !(!(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) ) continue;
 	attr.at().setS(TSYS::strSepParse(c_el.cfg("IO_VAL").getS(),0,'|'));
 	attr.at().fld().setValues(TSYS::strSepParse(c_el.cfg("IO_VAL").getS(),1,'|'));
 	attr.at().fld().setSelNames(TSYS::strSepParse(c_el.cfg("IO_VAL").getS(),2,'|'));
@@ -652,17 +670,14 @@ void LWidget::loadIO( )
 	attr.at().setCfgVal(c_el.cfg("CFG_VAL").getS());
     }
 
-    //- Load cotainer widgets -
+    //> Load cotainer widgets
     if( !enable() || !isContainer() ) return;
     c_el.setElem(&mod->elInclWdg());
     tbl=ownerLib().tbl()+"_incl";
-    c_el.cfg("IDW").setS(id());
-    c_el.cfg("ID").setS("");
-    fld_cnt=0;
-    while( SYS->db().at().dataSeek(db+"."+tbl,mod->nodePath()+tbl,fld_cnt++,c_el) )
+    c_el.cfg("IDW").setS(id(),true);
+    for( int fld_cnt=0; SYS->db().at().dataSeek(db+"."+tbl,mod->nodePath()+tbl,fld_cnt++,c_el); )
     {
 	string sid  = c_el.cfg("ID").getS();
-	c_el.cfg("ID").setS("");
 	if( c_el.cfg("PARENT").getS() == "<deleted>" )
 	{
 	    if( wdgPresent(sid) )	wdgDel(sid);
@@ -678,49 +693,34 @@ void LWidget::save_( )
 {
     string db  = ownerLib().DB();
     string tbl = ownerLib().tbl();
+    string tbl_io = tbl+"_io";
 
-    //- Remove from DB -
-    if( nodeMode() == TCntrNode::Disable )
+    //> Save generic widget's data
+    m_attrs="";
+    vector<string> als;
+    attrList( als );
+    TConfig c_el(&mod->elWdgIO());
+    c_el.cfg("IDW").setS(id());
+    for( int i_a = 0; i_a < als.size(); i_a++ )
     {
-	//-- Remove from library table --
-	SYS->db().at().dataDel( db+"."+tbl, mod->nodePath()+tbl, *this );
-	
-	//-- Remove widget's IO from library IO table --
-	TConfig c_el( &mod->elWdgIO() );
-	c_el.cfg("IDW").setS( id() );
-	c_el.cfg("ID").setS( "" );
-	SYS->db().at().dataDel( db+"."+tbl+"_io", mod->nodePath()+tbl+"_io", c_el );
-
-	//-- Remove widget's user IO from library IO table --
-	c_el.setElem( &mod->elWdgUIO() );
-	c_el.cfg("IDW").setS( id() );
-	c_el.cfg("ID").setS( "" );
-	SYS->db().at().dataDel( db+"."+tbl+"_uio", mod->nodePath()+tbl+"_uio", c_el );
-
-	//-- Remove widget's included widgets from library include table --
-	c_el.setElem( &mod->elInclWdg() );
-	c_el.cfg("IDW").setS( id() );
-	c_el.cfg("ID").setS( "" );
-	SYS->db().at().dataDel( db+"."+tbl+"_incl", mod->nodePath()+tbl+"_incl", c_el );
-    }
-    //- Save widget's data -
-    else
-    {
-	//-- Save generic widget's data --
-	m_attrs="";
-	vector<string> als;
-	attrList( als );
-	for( int i_a = 0; i_a < als.size(); i_a++ )
+	AutoHD<Attr> attr = attrAt(als[i_a]);
+	if( !attr.at().modif() ) continue;
+	if( !(!(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) ) m_attrs+=als[i_a]+";";
+	if( attr.at().flgGlob()&Attr::Generic )
 	{
-	    AutoHD<Attr> attr = attrAt(als[i_a]);
-	    if( attr.at().modif() && !(!(attr.at().flgGlob()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) )
-		m_attrs+=als[i_a]+";";
+	    c_el.cfg("ID").setS(als[i_a]);
+	    c_el.cfg("IO_VAL").setNoTransl( !(attr.at().type() == TFld::String && !(attr.at().flgGlob()&(Attr::Image|Attr::DataTime|Attr::Color|Attr::Font|Attr::Address))) );
+	    c_el.cfg("IO_VAL").setS(attr.at().getS());
+	    c_el.cfg("SELF_FLG").setI(attr.at().flgSelf());
+	    c_el.cfg("CFG_TMPL").setS(attr.at().cfgTempl());
+	    c_el.cfg("CFG_VAL").setS(attr.at().cfgVal());
+	    SYS->db().at().dataSet(db+"."+tbl_io,mod->nodePath()+tbl_io,c_el);
 	}
-	SYS->db().at().dataSet( db+"."+tbl, mod->nodePath()+tbl, *this );
-
-	//-- Save widget's attributes --
-	saveIO();
     }
+    SYS->db().at().dataSet( db+"."+tbl, mod->nodePath()+tbl, *this );
+
+    //> Save widget's attributes
+    saveIO();
 }
 
 void LWidget::saveIO( )
@@ -729,7 +729,7 @@ void LWidget::saveIO( )
 
     if( !enable() ) return;
 
-    //- Save widget's attributes -
+    //> Save widget's attributes
     string db  = ownerLib().DB();
     string tbl = ownerLib().tbl()+"_io";
     string utbl = ownerLib().tbl()+"_uio";
@@ -738,15 +738,16 @@ void LWidget::saveIO( )
     TConfig c_el(&mod->elWdgIO());
     c_el.cfg("IDW").setS(id());
     TConfig c_elu(&mod->elWdgUIO());
-    c_elu.cfg("IDW").setS(id());
+    c_elu.cfg("IDW").setS(id(),true);
     for( int i_a = 0; i_a < als.size(); i_a++ )
     {
 	AutoHD<Attr> attr = attrAt(als[i_a]);
-	if( !attr.at().modif() )	continue;
-	if( !(attr.at().flgGlob()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser )
+	if( !attr.at().modif() || attr.at().flgGlob()&Attr::Generic )	continue;
+	if( !(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser )
 	{
-	    //-- User attribute store --
+	    //>> User attribute store
 	    c_elu.cfg("ID").setS(als[i_a]);
+	    c_elu.cfg("IO_VAL").setNoTransl( !(attr.at().type() == TFld::String && !(attr.at().flgGlob()&(Attr::Image|Attr::DataTime|Attr::Color|Attr::Font|Attr::Address))) );
 	    c_elu.cfg("IO_VAL").setS(attr.at().getS()+"|"+
 				    attr.at().fld().values()+"|"+
 				    attr.at().fld().selNames());
@@ -759,8 +760,9 @@ void LWidget::saveIO( )
 	}
 	else
 	{
-	    //-- Work attribute store --
+	    //>> Work attribute store
 	    c_el.cfg("ID").setS(als[i_a]);
+	    c_el.cfg("IO_VAL").setNoTransl( !(attr.at().type() == TFld::String && !(attr.at().flgGlob()&(Attr::Image|Attr::DataTime|Attr::Color|Attr::Font|Attr::Address))) );
 	    c_el.cfg("IO_VAL").setS(attr.at().getS());
 	    c_el.cfg("SELF_FLG").setI(attr.at().flgSelf());
 	    c_el.cfg("CFG_TMPL").setS(attr.at().cfgTempl());
@@ -769,44 +771,51 @@ void LWidget::saveIO( )
 	}
     }
 
-    //- Clear no present IO for user io table -
-    int fld_cnt=0;
-    c_elu.cfg("ID").setS("");
+    //> Clear no present IO for user io table
     c_elu.cfgViewAll(false);
-    while( SYS->db().at().dataSeek(db+"."+utbl,mod->nodePath()+utbl,fld_cnt++,c_elu) )
+    for( int fld_cnt=0; SYS->db().at().dataSeek(db+"."+utbl,mod->nodePath()+utbl,fld_cnt++,c_elu); )
     {
 	string sid = c_elu.cfg("ID").getS();
 	if( TSYS::pathLev(sid,1).empty() && !attrPresent(TSYS::pathLev(sid,0)) )
 	{
-	    SYS->db().at().dataDel(db+"."+utbl,mod->nodePath()+utbl,c_elu);
+	    SYS->db().at().dataDel(db+"."+utbl,mod->nodePath()+utbl,c_elu,true);
 	    fld_cnt--;
 	}
-	c_elu.cfg("ID").setS("");
     }
 }
 
-void LWidget::wdgAdd( const string &wid, const string &name, const string &path )
+void LWidget::wClear( )
 {
-    if( !isContainer() )  throw TError(nodePath().c_str(),_("No container widget!"));
+    Widget::wClear();
+
+    m_proc = m_attrs = "";
+}
+
+void LWidget::wdgAdd( const string &wid, const string &name, const string &path, bool force )
+{
+    if( !isContainer() )  throw TError(nodePath().c_str(),_("Widget is not container!"));
     if( wdgPresent(wid) ) return;
 
-    //- Check for label <deleted> -
-    string db  = ownerLib().DB();
-    string tbl = ownerLib().tbl()+"_incl";
-    TConfig c_el( &mod->elInclWdg() );
-    c_el.cfg("IDW").setS(id());
-    c_el.cfg("ID").setS(wid);
-    if( SYS->db().at().dataGet( db+"."+tbl, mod->nodePath()+tbl, c_el ) && c_el.cfg("PARENT").getS() == "<deleted>" )
+    //> Check for label <deleted>
+    if( !force )
     {
-	if( !parent().at().wdgPresent(wid) )	SYS->db().at().dataDel( db+"."+tbl, mod->nodePath()+tbl, c_el );
-	return;
+	string db  = ownerLib().DB();
+	string tbl = ownerLib().tbl()+"_incl";
+	TConfig c_el( &mod->elInclWdg() );
+	c_el.cfg("IDW").setS(id());
+	c_el.cfg("ID").setS(wid);
+	if( SYS->db().at().dataGet( db+"."+tbl, mod->nodePath()+tbl, c_el ) && c_el.cfg("PARENT").getS() == "<deleted>" )
+	{
+	    if( !parent().at().wdgPresent(wid) )	SYS->db().at().dataDel( db+"."+tbl, mod->nodePath()+tbl, c_el, true );
+	    return;
+	}
     }
 
-    //- Same widget add -
+    //> Same widget add
     chldAdd(inclWdg,new CWidget(wid,path));
     wdgAt(wid).at().setName(name);
 
-    //- Call heritors include widgets update -
+    //> Call heritors include widgets update
     for( int i_h = 0; i_h < m_herit.size(); i_h++ )
 	if( m_herit[i_h].at().enable( ) )
 	    m_herit[i_h].at().inheritIncl(wid);
@@ -838,7 +847,7 @@ void LWidget::inheritAttr( const string &attr )
 void LWidget::cntrCmdProc( XMLNode *opt )
 {
     if( cntrCmdServ(opt) ) return;
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	cntrCmdGeneric(opt);
@@ -857,8 +866,8 @@ void LWidget::cntrCmdProc( XMLNode *opt )
 //************************************************
 CWidget::CWidget( const string &iid, const string &isrcwdg ) :
         Widget(iid), TConfig(&mod->elInclWdg()),
-        m_parent(cfg("PARENT").getSd()), m_attrs(cfg("ATTRS").getSd()),
-        m_owner(cfg("USER").getSd()), m_grp(cfg("GRP").getSd()), m_permit(cfg("PERMIT").getId()),
+        mParent(cfg("PARENT").getSd()), m_attrs(cfg("ATTRS").getSd()),
+        m_owner(cfg("USER").getSd()), m_grp(cfg("GRP").getSd()), mPermit(cfg("PERMIT").getId()),
         delMark(false)
 {
     cfg("ID").setS(id());
@@ -883,9 +892,9 @@ LWidget &CWidget::ownerLWdg()
 
 void CWidget::postEnable( int flag )
 {
-    //- Call parent methos -
+    //> Call parent methos
     Widget::postEnable(flag);
-    //- Set container widget id -
+    //> Set container widget id
     cfg("IDW").setS(ownerLWdg().id());
 }
 
@@ -910,7 +919,7 @@ string CWidget::owner( )
 void CWidget::setOwner( const string &iown )
 {
     m_owner = iown;
-    //- Group update -
+    //> Group update
     if( SYS->security().at().grpAt("UI").at().user(iown) ) setGrp("UI");
     else
     {
@@ -938,7 +947,7 @@ void CWidget::setEnable( bool val )
 	    if( ownerLWdg().herit()[i_h].at().wdgPresent(id()) && !ownerLWdg().herit()[i_h].at().wdgAt(id()).at().enable( ) )
 	    try { ownerLWdg().herit()[i_h].at().wdgAt(id()).at().setEnable(true); }
 	    catch(...)
-	    { mess_err(nodePath().c_str(),_("Heritors widget <%s> enable error"),id().c_str()); }
+            { mess_err(nodePath().c_str(),_("Inheriting widget <%s> enable error"),id().c_str()); }
 }
 
 string CWidget::calcId( )
@@ -967,80 +976,89 @@ int CWidget::calcPer( )
 
 void CWidget::load_( )
 {
-    //- Load generic widget's data -
+    if( !SYS->chkSelDB(ownerLWdg().ownerLib().DB()) ) return;
+
+    //> Load generic widget's data
     string db  = ownerLWdg().ownerLib().DB();
     string tbl = ownerLWdg().ownerLib().tbl()+"_incl";
     SYS->db().at().dataGet(db+"."+tbl,mod->nodePath()+tbl,*this);
 
-    //- Load widget's attributes -
-    loadIO();
-}
-
-void CWidget::loadIO( )
-{
-    string tstr;
+    //> Inherit modify attributes
     vector<string> als;
-
-    if( !enable() ) return;
-
-    //- Load widget's work attributes -
-    string db  = ownerLWdg().ownerLib().DB();
-    string tbl = ownerLWdg().ownerLib().tbl()+"_io";
-
-    //- Inherit modify attributes -
-    /*attrList( als );
+    attrList( als );
     for( int i_a = 0; i_a < als.size(); i_a++ )
     {
+	if( !attrPresent(als[i_a]) ) continue;
 	AutoHD<Attr> attr = attrAt(als[i_a]);
-	if( attr.at().flgGlob()&Attr::IsInher && attr.at().modif() && m_attrs.find(als[i_a]) == string::npos )
+	if( attr.at().modif() && m_attrs.find(als[i_a]) == string::npos )
 	{
 	    attr.at().setModif(0);
 	    inheritAttr(als[i_a]);
 	}
     }
-    als.clear();*/
-    bool full_ls = false;
-    if( m_attrs == "*" )	{ attrList( als ); full_ls = true; }
-    else
-	for( int off = 0; !(tstr = TSYS::strSepParse(m_attrs,0,';',&off)).empty(); )
-	    als.push_back(tstr);
 
-    //-- Same load --
+    //> Load generic attributes
+    tbl = ownerLWdg().ownerLib().tbl()+"_io";
     TConfig c_el(&mod->elWdgIO());
     c_el.cfg("IDW").setS(ownerLWdg().id());
-    for( int i_a = 0; i_a < als.size(); i_a++ )
+    string tstr;
+    for( int off = 0; !(tstr = TSYS::strSepParse(m_attrs,0,';',&off)).empty(); )
     {
-	if( !attrPresent(als[i_a]) )    continue;
-	AutoHD<Attr> attr = attrAt(als[i_a]);
-	if( !(attr.at().flgGlob()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser ) continue;
-	c_el.cfg("ID").setS(id()+"/"+als[i_a]);
+	if( !attrPresent(tstr) ) continue;
+	AutoHD<Attr> attr = attrAt(tstr);
+	if( !(attr.at().flgGlob()&Attr::Generic) ) continue;
+	c_el.cfg("ID").setS(id()+"/"+tstr);
+	if( !SYS->db().at().dataGet(db+"."+tbl,mod->nodePath()+tbl,c_el) ) continue;
+	attr.at().setS(c_el.cfg("IO_VAL").getS(),true);
+	attr.at().setFlgSelf((Attr::SelfAttrFlgs)c_el.cfg("SELF_FLG").getI());
+	attr.at().setCfgTempl(c_el.cfg("CFG_TMPL").getS());
+	attr.at().setCfgVal(c_el.cfg("CFG_VAL").getS());
+    }
+
+    //> Load widget's attributes
+    loadIO();
+}
+
+void CWidget::loadIO( )
+{
+    if( !enable() ) return;
+
+    //> Load widget's work attributes
+    string db  = ownerLWdg().ownerLib().DB();
+    string tbl = ownerLWdg().ownerLib().tbl()+"_io";
+
+    //> Same load
+    TConfig c_el(&mod->elWdgIO());
+    c_el.cfg("IDW").setS(ownerLWdg().id());
+    string tstr;
+    for( int off = 0; !(tstr = TSYS::strSepParse(m_attrs,0,';',&off)).empty(); )
+    {
+	if( !attrPresent(tstr) )    continue;
+	AutoHD<Attr> attr = attrAt(tstr);
+	if( attr.at().flgGlob()&Attr::Generic || (!(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) ) continue;
+	c_el.cfg("ID").setS(id()+"/"+tstr);
 	if( !SYS->db().at().dataGet(db+"."+tbl,mod->nodePath()+tbl,c_el) ) continue;
 	attr.at().setS(c_el.cfg("IO_VAL").getS());
 	attr.at().setFlgSelf((Attr::SelfAttrFlgs)c_el.cfg("SELF_FLG").getI());
 	attr.at().setCfgTempl(c_el.cfg("CFG_TMPL").getS());
 	attr.at().setCfgVal(c_el.cfg("CFG_VAL").getS());
-	if( full_ls && attr.at().flgGlob()&Attr::Active ) attrList( als );
     }
-    //- Load widget's user attributes -
+    //> Load widget's user attributes
     tbl = ownerLWdg().ownerLib().tbl()+"_uio";
     c_el.setElem(&mod->elWdgUIO());
-    c_el.cfg("IDW").setS(ownerLWdg().id());
-    c_el.cfg("ID").setS("");
-    int fld_cnt = 0;
-    while( SYS->db().at().dataSeek(db+"."+tbl,mod->nodePath()+tbl,fld_cnt++,c_el) )
+    c_el.cfg("IDW").setS(ownerLWdg().id(),true);
+    for( int fld_cnt = 0; SYS->db().at().dataSeek(db+"."+tbl,mod->nodePath()+tbl,fld_cnt++,c_el); )
     {
 	string sid = c_el.cfg("ID").getS();
 	unsigned flg = c_el.cfg("IO_TYPE").getI();
-	c_el.cfg("ID").setS("");
 
-	if( TSYS::pathLev(sid,0) == id() && !TSYS::pathLev(sid,1).empty() )
-	    sid = TSYS::pathLev(sid,1);
+	if( TSYS::pathLev(sid,0) == id() && !TSYS::pathLev(sid,1).empty() ) sid = TSYS::pathLev(sid,1);
 	else continue;
 
 	if( !attrPresent(sid) )
 	    attrAdd( new TFld(sid.c_str(),c_el.cfg("NAME").getS().c_str(),(TFld::Type)(flg&0x0f),flg>>4) );
 	AutoHD<Attr> attr = attrAt(sid);
-	if( !(!(attr.at().flgGlob()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) ) continue;
+	if( !(!(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) ) continue;
 	attr.at().setS(TSYS::strSepParse(c_el.cfg("IO_VAL").getS(),0,'|'));
 	attr.at().fld().setValues(TSYS::strSepParse(c_el.cfg("IO_VAL").getS(),1,'|'));
 	attr.at().fld().setSelNames(TSYS::strSepParse(c_el.cfg("IO_VAL").getS(),2,'|'));
@@ -1054,59 +1072,61 @@ void CWidget::save_( )
 {
     string db  = ownerLWdg().ownerLib().DB();
     string tbl = ownerLWdg().ownerLib().tbl();
+    string tbl_io = tbl+"_io";
 
-    //- Delete from DB -
+    //> Delete from DB
     if( nodeMode() == TCntrNode::Disable )
     {
-	//-- Remove from library table --
+	//>> Remove from library table
 	if( delMark )
 	{
-	    m_parent = "<deleted>";
+	    mParent = "<deleted>";
 	    SYS->db().at().dataSet( db+"."+tbl+"_incl", mod->nodePath()+tbl+"_incl", *this );
 	}
-	else SYS->db().at().dataDel( db+"."+tbl+"_incl", mod->nodePath()+tbl+"_incl", *this );	
+	else SYS->db().at().dataDel( db+"."+tbl+"_incl", mod->nodePath()+tbl+"_incl", *this, true );
 
-	//-- Remove widget's work and users IO from library IO table --
+	//>> Remove widget's work and users IO from library IO table
 	TConfig c_el( &mod->elWdgIO() );
-	c_el.cfg("IDW").setS( ownerLWdg().id() );
+	c_el.cfg("IDW").setS( ownerLWdg().id(), true );
 	for( int i_a = 0; i_a < m_attrs.size(); i_a++ )
 	{
-	    c_el.cfg("ID").setS(id()+"/"+m_attrs[i_a]);
+	    c_el.cfg("ID").setS(id()+"/"+m_attrs[i_a],true);
 	    SYS->db().at().dataDel( db+"."+tbl+"_io", mod->nodePath()+tbl+"_io", c_el );
 	}
-	/*int io_cnt = 0;
-	while( SYS->db().at().dataSeek( db+"."+tbl+"_io", mod->nodePath()+tbl+"_io", io_cnt++, c_el ) )
-	{
-	    if( c_el.cfg("ID").getS().find(id()+"/") == 0 )
-	    { SYS->db().at().dataDel( db+"."+tbl+"_io", mod->nodePath()+tbl+"_io", c_el ); io_cnt--; }
-	    c_el.cfg("ID").setS("");
-	}*/
 	c_el.setElem(&mod->elWdgUIO());
-	c_el.cfg("IDW").setS( ownerLWdg().id() );
-	int io_cnt = 0;
-	while( SYS->db().at().dataSeek( db+"."+tbl+"_uio", mod->nodePath()+tbl+"_uio", io_cnt++, c_el ) )
-	{
+	c_el.cfg("IDW").setS( ownerLWdg().id(), true );
+	for( int io_cnt = 0; SYS->db().at().dataSeek( db+"."+tbl+"_uio", mod->nodePath()+tbl+"_uio", io_cnt++, c_el ); )
 	    if( c_el.cfg("ID").getS().find(id()+"/") == 0 )
-	    { SYS->db().at().dataDel( db+"."+tbl+"_uio", mod->nodePath()+tbl+"_uio", c_el ); io_cnt--; }
-	    c_el.cfg("ID").setS("");
-	}
+	    { SYS->db().at().dataDel( db+"."+tbl+"_uio", mod->nodePath()+tbl+"_uio", c_el, true ); io_cnt--; }
     }
-    //- Save widget's data -
+    //> Save widget's data
     else
     {
-	//-- Save generic widget's data --
+	//> Save generic widget's data
 	m_attrs="";
 	vector<string> als;
 	attrList( als );
+	TConfig c_el(&mod->elWdgIO());
+	c_el.cfg("IDW").setS(ownerLWdg().id());
 	for( int i_a = 0; i_a < als.size(); i_a++ )
 	{
 	    AutoHD<Attr> attr = attrAt(als[i_a]);
-	    if( attr.at().modif() && !(!(attr.at().flgGlob()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) )
-		m_attrs+=als[i_a]+";";
+	    if( !attr.at().modif() ) continue;
+	    if( !(!(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser) ) m_attrs+=als[i_a]+";";
+	    if( attr.at().flgGlob()&Attr::Generic )
+	    {
+		c_el.cfg("ID").setS( id()+"/"+als[i_a] );
+		c_el.cfg("IO_VAL").setNoTransl( !(attr.at().type() == TFld::String && !(attr.at().flgGlob()&(Attr::Image|Attr::DataTime|Attr::Color|Attr::Font|Attr::Address))) );
+		c_el.cfg("IO_VAL").setS(attr.at().getS());
+		c_el.cfg("SELF_FLG").setI(attr.at().flgSelf());
+		c_el.cfg("CFG_TMPL").setS(attr.at().cfgTempl());
+	        c_el.cfg("CFG_VAL").setS(attr.at().cfgVal());
+		SYS->db().at().dataSet(db+"."+tbl_io,mod->nodePath()+tbl_io,c_el);
+	    }
 	}
 	SYS->db().at().dataSet(db+"."+tbl+"_incl",mod->nodePath()+tbl+"_incl",*this);
 
-	//-- Save widget's attributes --
+	//>> Save widget's attributes
 	saveIO();
     }
 }
@@ -1124,15 +1144,16 @@ void CWidget::saveIO( )
     TConfig c_el(&mod->elWdgIO());
     c_el.cfg("IDW").setS(ownerLWdg().id());
     TConfig c_elu(&mod->elWdgUIO());
-    c_elu.cfg("IDW").setS(ownerLWdg().id());
+    c_elu.cfg("IDW").setS(ownerLWdg().id(),true);
     for( int i_a = 0; i_a < als.size(); i_a++ )
     {
 	AutoHD<Attr> attr = attrAt(als[i_a]);
-	if( !attr.at().modif() )	continue;
-	if( !(attr.at().flgGlob()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser )
+	if( !attr.at().modif() || attr.at().flgGlob()&Attr::Generic )	continue;
+	if( !(attr.at().flgSelf()&Attr::IsInher) && attr.at().flgGlob()&Attr::IsUser )
 	{
 	    //-- User attribute store --
 	    c_elu.cfg("ID").setS( id()+"/"+als[i_a] );
+	    c_elu.cfg("IO_VAL").setNoTransl( !(attr.at().type() == TFld::String && !(attr.at().flgGlob()&(Attr::Image|Attr::DataTime|Attr::Color|Attr::Font|Attr::Address))) );
 	    c_elu.cfg("IO_VAL").setS(attr.at().getS()+"|"+
 				    attr.at().fld().values()+"|"+
 				    attr.at().fld().selNames());
@@ -1147,6 +1168,7 @@ void CWidget::saveIO( )
 	{
 	    //-- Work attribute store --
 	    c_el.cfg("ID").setS( id()+"/"+als[i_a] );
+	    c_el.cfg("IO_VAL").setNoTransl( !(attr.at().type() == TFld::String && !(attr.at().flgGlob()&(Attr::Image|Attr::DataTime|Attr::Color|Attr::Font|Attr::Address))) );
 	    c_el.cfg("IO_VAL").setS(attr.at().getS());
 	    c_el.cfg("SELF_FLG").setI(attr.at().flgSelf());
 	    c_el.cfg("CFG_TMPL").setS(attr.at().cfgTempl());
@@ -1154,22 +1176,26 @@ void CWidget::saveIO( )
 	    SYS->db().at().dataSet(db+"."+tbl,mod->nodePath()+tbl,c_el);
 	}
     }
-    //- Clear no present IO for user io table -
-    int fld_cnt=0;
-    c_elu.cfg("ID").setS("");
+    //> Clear no present IO for user io table
     c_elu.cfgViewAll(false);
-    while( SYS->db().at().dataSeek(db+"."+utbl,mod->nodePath()+utbl,fld_cnt++,c_elu) )
+    for( int fld_cnt=0; SYS->db().at().dataSeek(db+"."+utbl,mod->nodePath()+utbl,fld_cnt++,c_elu); )
     {
 	string sid = c_elu.cfg("ID").getS();
 	if( TSYS::pathLev(sid,0) == id() && TSYS::pathLev(sid,1).size() && !attrPresent(TSYS::pathLev(sid,1)) )
 	{
-	    SYS->db().at().dataDel(db+"."+utbl,mod->nodePath()+utbl,c_elu);
+	    SYS->db().at().dataDel(db+"."+utbl,mod->nodePath()+utbl,c_elu,true);
 	    fld_cnt--;
 	}
-	c_elu.cfg("ID").setS("");
     }
 }
 
+void CWidget::wClear( )
+{
+    Widget::wClear();
+
+    m_attrs = "";
+}
+
 string CWidget::resourceGet( const string &id, string *mime )
 {
     string mimeType, mimeData;
