@@ -22,6 +22,8 @@
 #ifndef ORIGWIDG_H
 #define ORIGWIDG_H
 
+#include <string.h>
+
 #include "libwidg.h"
 
 namespace VCA
@@ -43,8 +45,8 @@ class PrWidget : public LWidget
 	string name( )		{ return "Root primitive"; }
 	string descr( )		{ return "Abstract root primitive"; }
 
-	void setName( const string &inm )	{ };
-	void setDescr( const string &idscr )	{ };
+	void setName( const string &inm )	{ }
+	void setDescr( const string &idscr )	{ }
 	void setEnable( bool val );
 
     protected:
@@ -74,7 +76,7 @@ class OrigElFigure : public PrWidget
     protected:
 	//Methods
 	void postEnable( int flag );
-	bool attrChange( Attr &cfg, void *prev );
+	bool attrChange( Attr &cfg, TVariant prev );
 };
 
 //*************************************************
@@ -92,7 +94,7 @@ class OrigFormEl : public PrWidget
     protected:
 	//Methods
 	void postEnable( int flag );
-	bool attrChange( Attr &cfg, void *prev );
+	bool attrChange( Attr &cfg, TVariant prev );
 };
 
 //************************************************
@@ -110,7 +112,7 @@ class OrigText : public PrWidget
      protected:
 	//Methods
 	void postEnable( int flag );
-	bool attrChange( Attr &cfg, void *prev );
+	bool attrChange( Attr &cfg, TVariant prev );
 };
 
 //************************************************
@@ -128,7 +130,7 @@ class OrigMedia : public PrWidget
     protected:
 	//Methods
 	void postEnable( int flag );
-	bool attrChange( Attr &cfg, void *prev );
+	bool attrChange( Attr &cfg, TVariant prev );
 };
 
 //************************************************
@@ -146,7 +148,7 @@ class OrigDiagram : public PrWidget
     protected:
 	//Methods
 	void postEnable( int flag );
-	bool attrChange( Attr &cfg, void *prev );
+	bool attrChange( Attr &cfg, TVariant prev );
 };
 
 //************************************************
@@ -164,7 +166,7 @@ class OrigProtocol : public PrWidget
     protected:
 	//Methods
 	void postEnable( int flag );
-	bool attrChange( Attr &cfg, void *prev );
+	bool attrChange( Attr &cfg, TVariant prev );
 };
 
 //************************************************
@@ -178,6 +180,19 @@ class OrigDocument : public PrWidget
 
 	string name( );
 	string descr( );
+
+    protected:
+	//Methods
+	void postEnable( int flag );
+	void calc( Widget *base );
+	bool attrChange( Attr &cfg, TVariant prev );
+
+	string makeDoc( const string &tmpl, Widget *wdg );
+	void nodeProcess( Widget *wdg, XMLNode *xcur, TValFunc &funcV, TFunction &funcIO, const string &iLang, bool instrDel = false );
+	void nodeClear( XMLNode *xcur );
+
+	//Attributes
+	static const char *XHTML_entity;
 };
 
 //************************************************
