@@ -27,6 +27,7 @@
 #include <tmess.h>
 
 #include "origwidg.h"
+#include "vcaFuncs.h"
 #include "vcaengine.h"
 
 //*************************************************
@@ -36,7 +37,7 @@
 #define MOD_TYPE	"UI"
 #define MOD_SUBTYPE	"VCAEngine"
 #define VER_TYPE	VER_UI
-#define VERSION		"0.7.0"
+#define VERSION		"0.8.10"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Generic visual control area engine."
 #define LICENSE		"GPL"
@@ -78,9 +79,10 @@ Engine::Engine( string name ) : mSynthCom("echo \"%t\" | ru_tts | sox -t raw -s
 
     mod		= this;
 
-    id_wlb = grpAdd("wlb_");
-    id_prj = grpAdd("prj_");
-    id_ses = grpAdd("ses_");
+    idWlb = grpAdd("wlb_");
+    idPrj = grpAdd("prj_");
+    idSes = grpAdd("ses_");
+    idFnc = grpAdd("vca",true);
 }
 
 Engine::~Engine( )
@@ -118,30 +120,30 @@ void Engine::postEnable( int flag )
 
     if( !(flag&TCntrNode::NodeConnect) ) return;
 
-    //- Make lib's DB structure -
+    //> Make lib's DB structure
     lbwdg_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"30") );
-    lbwdg_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TFld::NoFlag,"50") );
-    lbwdg_el.fldAdd( new TFld("DESCR",_("Description"),TFld::String,TFld::NoFlag,"300") );
+    lbwdg_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TCfg::TransltText,"50") );
+    lbwdg_el.fldAdd( new TFld("DESCR",_("Description"),TFld::String,TCfg::TransltText,"300") );
     lbwdg_el.fldAdd( new TFld("DB_TBL",_("Data base"),TFld::String,TFld::NoFlag,"30") );
     lbwdg_el.fldAdd( new TFld("ICO",_("Icon"),TFld::String,TFld::NoFlag,"10000") );
 
-    //- Make library widgets data container -
+    //> Make library widgets' data container
     wdgdata_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"30") );
     wdgdata_el.fldAdd( new TFld("MIME",_("Mime type"),TFld::String,TFld::NoFlag,"30") );
     wdgdata_el.fldAdd( new TFld("DATA",_("Mime data"),TFld::String,TFld::NoFlag,"500000") );
 
-    //- Make widgets DB structure -
+    //> Make widgets' DB structure
     wdg_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"30") );
     wdg_el.fldAdd( new TFld("ICO",_("Icon"),TFld::String,TFld::NoFlag,"10000") );
     wdg_el.fldAdd( new TFld("PARENT",_("Parent widget"),TFld::String,TFld::NoFlag,"200") );
-    wdg_el.fldAdd( new TFld("PROC",_("Procedure text and language"),TFld::String,TFld::NoFlag,"10000") );
+    wdg_el.fldAdd( new TFld("PROC",_("Procedure text and language"),TFld::String,TCfg::TransltText,"10000") );
     wdg_el.fldAdd( new TFld("PROC_PER",_("Procedure calc period"),TFld::Integer,TFld::NoFlag,"5","-1") );
     wdg_el.fldAdd( new TFld("USER",_("User"),TFld::String,TFld::NoFlag,"20","root") );
     wdg_el.fldAdd( new TFld("GRP",_("Group"),TFld::String,TFld::NoFlag,"20","UI") );
     wdg_el.fldAdd( new TFld("PERMIT",_("Permision"),TFld::Integer,TFld::OctDec,"3","436") );
     wdg_el.fldAdd( new TFld("ATTRS",_("Changed attributes"),TFld::String,TFld::NoFlag,"10000","*") );
 
-    //- Make include widgets DB structure -
+    //> Make include widgets' DB structure
     inclwdg_el.fldAdd( new TFld("IDW",_("IDW"),TFld::String,TCfg::Key,"100") );
     inclwdg_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"30") );
     inclwdg_el.fldAdd( new TFld("PARENT",_("Parent widget"),TFld::String,TFld::NoFlag,"200") );
@@ -150,28 +152,28 @@ void Engine::postEnable( int flag )
     inclwdg_el.fldAdd( new TFld("GRP",_("Group"),TFld::String,TFld::NoFlag,"20","UI") );
     inclwdg_el.fldAdd( new TFld("PERMIT",_("Permision"),TFld::Integer,TFld::OctDec,"3","436") );
 
-    //- Make widget's IO DB structure -
+    //> Make widget's IO DB structure
     wdgio_el.fldAdd( new TFld("IDW",_("Widget ID"),TFld::String,TCfg::Key,"100") );
     wdgio_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"61") );
-    wdgio_el.fldAdd( new TFld("IO_VAL",_("Attribute value"),TFld::String,TFld::NoFlag,"100000") );
+    wdgio_el.fldAdd( new TFld("IO_VAL",_("Attribute value"),TFld::String,TCfg::TransltText,"100000") );
     wdgio_el.fldAdd( new TFld("SELF_FLG",_("Attribute self flags"),TFld::Integer,TFld::NoFlag,"5") );
-    wdgio_el.fldAdd( new TFld("CFG_TMPL",_("Configuration template"),TFld::String,TFld::NoFlag,"30") );
+    wdgio_el.fldAdd( new TFld("CFG_TMPL",_("Configuration template"),TFld::String,TCfg::TransltText,"30") );
     wdgio_el.fldAdd( new TFld("CFG_VAL",_("Configuration value"),TFld::String,TFld::NoFlag,"1000") );
 
-    //- Make widget's user IO DB structure -
+    //> Make widget's user IO DB structure
     wdguio_el.fldAdd( new TFld("IDW",_("Widget ID"),TFld::String,TCfg::Key,"100") );
     wdguio_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"61") );
-    wdguio_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TFld::NoFlag,"50") );
+    wdguio_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TCfg::TransltText,"50") );
     wdguio_el.fldAdd( new TFld("IO_TYPE",_("Attribute generic flags and type"),TFld::Integer,TFld::NoFlag,"10") );
-    wdguio_el.fldAdd( new TFld("IO_VAL",_("Attribute value"),TFld::String,TFld::NoFlag,"100000") );
+    wdguio_el.fldAdd( new TFld("IO_VAL",_("Attribute value"),TFld::String,TCfg::TransltText,"100000") );
     wdguio_el.fldAdd( new TFld("SELF_FLG",_("Attribute self flags"),TFld::Integer,TFld::NoFlag,"5") );
-    wdguio_el.fldAdd( new TFld("CFG_TMPL",_("Configuration template"),TFld::String,TFld::NoFlag,"30") );
+    wdguio_el.fldAdd( new TFld("CFG_TMPL",_("Configuration template"),TFld::String,TCfg::TransltText,"30") );
     wdguio_el.fldAdd( new TFld("CFG_VAL",_("Configuration value"),TFld::String,TFld::NoFlag,"1000") );
 
-    //- Make project's DB structure -
+    //> Make project's DB structure
     prj_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"30") );
-    prj_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TFld::NoFlag,"50") );
-    prj_el.fldAdd( new TFld("DESCR",_("Description"),TFld::String,TFld::NoFlag,"300") );
+    prj_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TCfg::TransltText,"50") );
+    prj_el.fldAdd( new TFld("DESCR",_("Description"),TFld::String,TCfg::TransltText,"300") );
     prj_el.fldAdd( new TFld("DB_TBL",_("Data base"),TFld::String,TFld::NoFlag,"30") );
     prj_el.fldAdd( new TFld("ICO",_("Icon"),TFld::String,TFld::NoFlag,"10000") );
     prj_el.fldAdd( new TFld("USER",_("User"),TFld::String,TFld::NoFlag,"20","root") );
@@ -180,12 +182,12 @@ void Engine::postEnable( int flag )
     prj_el.fldAdd( new TFld("PER",_("Calc period (ms)"),TFld::Integer,TFld::NoFlag,"4","100") );
     prj_el.fldAdd( new TFld("FLGS",_("Flags"),TFld::Integer,TFld::NoFlag,"4") );
 
-    //- Make pages DB structure -
+    //> Make pages DB structure
     page_el.fldAdd( new TFld("OWNER",_("Owner"),TFld::String,TCfg::Key,"100") );
     page_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"30") );
     page_el.fldAdd( new TFld("ICO",_("Icon"),TFld::String,TFld::NoFlag,"10000") );
     page_el.fldAdd( new TFld("PARENT",_("Parent widget"),TFld::String,TFld::NoFlag,"200") );
-    page_el.fldAdd( new TFld("PROC",_("Procedure text and language"),TFld::String,TFld::NoFlag,"10000") );
+    page_el.fldAdd( new TFld("PROC",_("Procedure text and language"),TFld::String,TCfg::TransltText,"10000") );
     page_el.fldAdd( new TFld("PROC_PER",_("Procedure calc period"),TFld::Integer,TFld::NoFlag,"5","-1") );
     page_el.fldAdd( new TFld("USER",_("User"),TFld::String,TFld::NoFlag,"20","root") );
     page_el.fldAdd( new TFld("GRP",_("Group"),TFld::String,TFld::NoFlag,"20","UI") );
@@ -193,12 +195,17 @@ void Engine::postEnable( int flag )
     page_el.fldAdd( new TFld("FLGS",_("Flags"),TFld::Integer,TFld::NoFlag,"1","0") );
     page_el.fldAdd( new TFld("ATTRS",_("Changed attributes"),TFld::String,TFld::NoFlag,"10000","*") );
 
-    //- Init original widgets library -
+    //> Make sessions' IO values of projects DB structure
+    prj_ses_el.fldAdd( new TFld("IDW",_("Widget ID"),TFld::String,TCfg::Key,"200") );
+    prj_ses_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"30") );
+    prj_ses_el.fldAdd( new TFld("IO_VAL",_("Attribute value"),TFld::String,TFld::NoFlag,"100000") );
+
+    //> Init original widgets library
     wlbAdd("originals",_("Original widget's library"));
-    //-- Set default library icon --
+    //>> Set default library icon
     if( TUIS::icoPresent("VCA.lwdg_root") )
         wlbAt("originals").at().setIco(TSYS::strEncode(TUIS::icoGet("VCA.lwdg_root"),TSYS::base64));
-    //-- Add main original widgets --
+    //>> Add main original widgets
     wlbAt("originals").at().add( new OrigElFigure() );
     wlbAt("originals").at().add( new OrigFormEl() );
     wlbAt("originals").at().add( new OrigText() );
@@ -206,9 +213,15 @@ void Engine::postEnable( int flag )
     wlbAt("originals").at().add( new OrigDiagram() );
     wlbAt("originals").at().add( new OrigProtocol() );
     wlbAt("originals").at().add( new OrigDocument() );
-    wlbAt("originals").at().add( new OrigFunction() );
+    //wlbAt("originals").at().add( new OrigFunction() );
     wlbAt("originals").at().add( new OrigBox() );
-    wlbAt("originals").at().add( new OrigLink() );
+
+    //>> User functions
+    chldAdd( idFnc, new nodePresent() );
+    chldAdd( idFnc, new wdgList() );
+    chldAdd( idFnc, new attrList() );
+    chldAdd( idFnc, new attrGet() );
+    chldAdd( idFnc, new attrSet() );
 }
 
 void Engine::preDisable( int flag )
@@ -220,11 +233,9 @@ void Engine::preDisable( int flag )
 
 void Engine::load_( )
 {
-#if OSC_DEBUG
-    mess_debug(nodePath().c_str(),_("Load module."));
-#endif
+    mess_info(nodePath().c_str(),_("Load module."));
 
-    //- Load parameters from command line -
+    //> Load parameters from command line
     int next_opt;
     const char *short_opt="h";
     struct option long_opt[] =
@@ -239,46 +250,41 @@ void Engine::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
 
-    //- Load parameters from config file and DB -
+    //> Load parameters from config file and DB
     setSynthCom( TBDS::genDBGet(nodePath()+"SynthCom",synthCom()) );
     setSynthCode( TBDS::genDBGet(nodePath()+"SynthCode",synthCode()) );
 
-    //-- Load widget's libraries --
+    //>> Load widget's libraries
     try
     {
-	//--- Search and create new libraries ---
+	//>>> Search and create new libraries
 	TConfig c_el(&elWdgLib());
 	c_el.cfgViewAll(false);
 	vector<string> db_ls;
 
-	//---- Search into DB ----
-	SYS->db().at().dbList(db_ls);
+	//>>>> Search into DB
+	SYS->db().at().dbList(db_ls,true);
 	for( int i_db = 0; i_db < db_ls.size(); i_db++ )
-	{
-	    int lib_cnt = 0;
-	    while(SYS->db().at().dataSeek(db_ls[i_db]+"."+wlbTable(),"",lib_cnt++,c_el) )
+	    for( int lib_cnt = 0; SYS->db().at().dataSeek(db_ls[i_db]+"."+wlbTable(),"",lib_cnt++,c_el); )
 	    {
 		string l_id = c_el.cfg("ID").getS();
-		c_el.cfg("ID").setS("");
 		if(!wlbPresent(l_id)) wlbAdd(l_id,"",(db_ls[i_db]==SYS->workDB())?"*.*":db_ls[i_db]);
 	    }
-	}
 
-	//---- Search into config file ----
-	int lib_cnt = 0;
-	while(SYS->db().at().dataSeek("",nodePath()+"LIB/",lib_cnt++,c_el) )
-	{
-	    string l_id = c_el.cfg("ID").getS();
-	    c_el.cfg("ID").setS("");
-	    if( !wlbPresent(l_id) )	wlbAdd(l_id,"","*.*");
-	}
+	//>>>> Search into config file
+	if( SYS->chkSelDB("<cfg>") )
+	    for( int lib_cnt = 0; SYS->db().at().dataSeek("",nodePath()+"LIB/",lib_cnt++,c_el); )
+	    {
+		string l_id = c_el.cfg("ID").getS();
+		if( !wlbPresent(l_id) )	wlbAdd(l_id,"","*.*");
+	    }
 
-	//--- Load present libraries ---
+	//>>>> Load present libraries
 	wlbList(db_ls);
 	for( int l_id = 0; l_id < db_ls.size(); l_id++ )
 	    wlbAt(db_ls[l_id]).at().load();
@@ -288,38 +294,32 @@ void Engine::load_( )
 	mess_err(nodePath().c_str(),_("Load widget's libraries error."));
     }
 
-    //-- Load projects --
+    //>> Load projects
     try
     {
-	//--- Search and create new projects ---
+	//>>> Search and create new projects
 	TConfig c_el(&elProject());
 	c_el.cfgViewAll(false);
 	vector<string> db_ls;
 
-	//---- Search into DB ----
-	SYS->db().at().dbList(db_ls);
+	//>>>> Search into DB
+	SYS->db().at().dbList(db_ls,true);
 	for( int i_db = 0; i_db < db_ls.size(); i_db++ )
-	{
-	    int lib_cnt = 0;
-	    while(SYS->db().at().dataSeek(db_ls[i_db]+"."+prjTable(),"",lib_cnt++,c_el) )
+	    for( int lib_cnt = 0; SYS->db().at().dataSeek(db_ls[i_db]+"."+prjTable(),"",lib_cnt++,c_el); )
 	    {
 		string prj_id = c_el.cfg("ID").getS();
-		c_el.cfg("ID").setS("");
-		if( !prjPresent(prj_id) )
-		    prjAdd(prj_id,"",(db_ls[i_db]==SYS->workDB())?"*.*":db_ls[i_db]);
+		if( !prjPresent(prj_id) )	prjAdd(prj_id,"",(db_ls[i_db]==SYS->workDB())?"*.*":db_ls[i_db]);
 	    }
-	}
 
-	//---- Search into config file ----
-	int el_cnt = 0;
-	while(SYS->db().at().dataSeek("",nodePath()+"PRJ/",el_cnt++,c_el) )
-	{
-	    string prj_id = c_el.cfg("ID").getS();
-	    c_el.cfg("ID").setS("");
-	    if( !prjPresent(prj_id) )	prjAdd(prj_id,"","*.*");
-	}
+	//>>>> Search into config file
+	if( SYS->chkSelDB("<cfg>") )
+	    for( int el_cnt = 0; SYS->db().at().dataSeek("",nodePath()+"PRJ/",el_cnt++,c_el); )
+	    {
+		string prj_id = c_el.cfg("ID").getS();
+		if( !prjPresent(prj_id) )	prjAdd(prj_id,"","*.*");
+	    }
 
-	//--- Load present projects ---
+	//>>>> Load present projects
 	prjList(db_ls);
 	for( int el_id = 0; el_id < db_ls.size(); el_id++ )
 	    prjAt(db_ls[el_id]).at().load();
@@ -329,13 +329,13 @@ void Engine::load_( )
 	mess_err(nodePath().c_str(),_("Load projects error."));
     }
 
-    //- Libraries enable -
+    //> Libraries enable
     vector<string> ls;
     wlbList(ls);
     for( int l_id = 0; l_id < ls.size(); l_id++ )
 	wlbAt(ls[l_id]).at().setEnable(true);
 
-    //- Projects enable -
+    //> Projects enable
     prjList(ls);
     for( int l_id = 0; l_id < ls.size(); l_id++ )
 	prjAt(ls[l_id]).at().setEnable(true);
@@ -345,9 +345,7 @@ void Engine::load_( )
 
 void Engine::save_( )
 {
-#if OSC_DEBUG
-    mess_debug(nodePath().c_str(),_("Save module."));
-#endif
+    mess_info(nodePath().c_str(),_("Save module."));
 
     //- Save parameters to DB -
     TBDS::genDBSet( nodePath()+"SynthCom", synthCom() );
@@ -356,9 +354,7 @@ void Engine::save_( )
 
 void Engine::modStart()
 {
-#if OSC_DEBUG
-    mess_debug(nodePath().c_str(),_("Start module."));
-#endif
+    mess_info(nodePath().c_str(),_("Start module."));
 
     //- Libraries enable -
     vector<string> ls;
@@ -376,61 +372,64 @@ void Engine::modStart()
 
 void Engine::modStop()
 {
-#if OSC_DEBUG
-    mess_debug(nodePath().c_str(),_("Stop module."));
-#endif
+    mess_info(nodePath().c_str(),_("Stop module."));
 
     vector<string> ls;
 
-    //- Stop sessions -
+    //> Stop sessions
     sesList(ls);
     for( int l_id = 0; l_id < ls.size(); l_id++ )
 	sesAt(ls[l_id]).at().setEnable(false);
 
-    //- Libraries stop -
-    wlbList(ls);
-    for( int l_id = 0; l_id < ls.size(); l_id++ )
-	wlbAt(ls[l_id]).at().setEnable(false);
-
-    //- Projects stop -
+    //> Projects stop
     prjList(ls);
     for( int l_id = 0; l_id < ls.size(); l_id++ )
 	prjAt(ls[l_id]).at().setEnable(false);
 
+    //> Libraries stop
+    wlbList(ls);
+    for( int l_id = 0; l_id < ls.size(); l_id++ )
+	wlbAt(ls[l_id]).at().setEnable(false);
+
     run_st = false;
 }
 
 void Engine::wlbAdd( const string &iid, const string &inm, const string &idb )
 {
     if(wlbPresent(iid))	return;
-    chldAdd(id_wlb, new WidgetLib(iid,inm,idb));
+    chldAdd(idWlb, new WidgetLib(iid,inm,idb));
 }
 
 AutoHD<WidgetLib> Engine::wlbAt( const string &id )
 {
-    return chldAt(id_wlb,id);
+    return chldAt(idWlb,id);
 }
 
 void Engine::prjAdd( const string &iid, const string &inm, const string &idb )
 {
     if(prjPresent(iid))	return;
-    chldAdd(id_prj, new Project(iid,inm,idb));
+    chldAdd(idPrj, new Project(iid,inm,idb));
 }
 
 AutoHD<Project> Engine::prjAt( const string &id )
 {
-    return chldAt(id_prj,id);
+    return chldAt(idPrj,id);
 }
 
 void Engine::sesAdd( const string &iid, const string &iproj )
 {
     if(sesPresent(iid))	return;
-    chldAdd(id_ses, new Session(iid,iproj));
+    chldAdd(idSes, new Session(iid,iproj));
 }
 
 AutoHD<Session> Engine::sesAt( const string &id )
 {
-    return chldAt(id_ses,id);
+    return chldAt(idSes,id);
+}
+
+AutoHD<TFunction> Engine::fAt( const string &id )
+{
+    return chldAt(idFnc,id);
 }
 
 string Engine::callSynth( const string &itxt )
@@ -486,7 +485,7 @@ string Engine::callSynth( const string &itxt )
 void Engine::cntrCmdProc( XMLNode *opt )
 {
     string a_path = opt->attr("path");
-    //- Service commands process -
+    //> Service commands process
     if( a_path == "/serv/sess" )	//Session operation
     {
 	if( ctrChkNode(opt,"list",RWRWRW,"root","root",SEQ_RD) ) //List session for some project
@@ -507,16 +506,16 @@ void Engine::cntrCmdProc( XMLNode *opt )
 	    string sess = opt->attr("sess");
 	    string prj  = opt->attr("prj");
 
-	    //-- User permission check --
+	    //>> User permission check
 	    AutoHD<Project> wprj = (!sess.empty()) ? sesAt(sess).at().parent() : prjAt(prj);
 	    if( !SYS->security().at().access(opt->attr("user"),SEQ_RD,wprj.at().owner(),wprj.at().grp(),wprj.at().permit()) )
-		throw TError(nodePath().c_str(),_("Connection to session no permit for '%s'."),opt->attr("user").c_str());
-	    //-- Connect to present session --
+		throw TError(nodePath().c_str(),_("Connection to session is not permited for '%s'."),opt->attr("user").c_str());
+	    //>> Connect to present session
 	    if( !sess.empty() )	sesAt(sess).at().connect();
-	    //-- Create session --
+	    //>> Create session
 	    else if( !prj.empty() )
 	    {
-		//--- Prepare session name ---
+		//>>> Prepare session name
 		sess = prj;
 		for( int p_cnt = 0; sesPresent(sess); p_cnt++ )
 		    sess = prj+TSYS::int2str(p_cnt);
@@ -536,31 +535,77 @@ void Engine::cntrCmdProc( XMLNode *opt )
 	}
 	return;
     }
+    else if( a_path == "/serv/wlbBr" && ctrChkNode(opt,"get") )
+    {
+	string item = opt->attr("item");
+	string upd_lb   = TSYS::pathLev(item,0);
+	if( upd_lb.size() > 4 && upd_lb.substr(0,4) != "wlb_" )	return;
+	if( upd_lb.size() > 4 )		upd_lb = upd_lb.substr(4);
+	string upd_wdg  = TSYS::pathLev(item,1);
+	if( upd_wdg.size() > 4 )	upd_wdg = upd_wdg.substr(4);
+	string upd_wdgi = TSYS::pathLev(item,2);
+	if( upd_wdgi.size() > 4 )	upd_wdgi = upd_wdgi.substr(4);
+
+	//>> Widgets libraries
+	vector<string> ls;
+	wlbList(ls);
+	for( int i_wlb = 0; i_wlb < ls.size(); i_wlb++ )
+	{
+	    if( !upd_lb.empty() && upd_lb != ls[i_wlb] ) continue;
+	    AutoHD<WidgetLib> wlb = wlbAt(ls[i_wlb]);
+	    XMLNode *wlbN = opt->childAdd("wlb")->setAttr("id",ls[i_wlb])->setText(wlb.at().name());
+	    wlbN->childAdd("ico")->setText(wlb.at().ico());
+
+	    //>> Widgets
+	    vector<string> wls;
+	    wlb.at().list(wls);
+	    for( int i_w = 0; i_w < wls.size(); i_w++ )
+	    {
+		if( !upd_wdg.empty() && upd_wdg != wls[i_w] )	continue;
+		AutoHD<LWidget> w = wlb.at().at(wls[i_w]);
+		XMLNode *wN = wlbN->childAdd("w")->setAttr("id",wls[i_w])->setAttr("parent",w.at().parentNm())->setText(w.at().name());
+		wN->childAdd("ico")->setText(w.at().ico());
+
+		//>> Child widgets
+		vector<string> cwls;
+		w.at().wdgList(cwls);
+		if( cwls.size() < 1000 )
+		    for( int i_c = 0; i_c < cwls.size(); i_c++ )
+		    {
+			if( !upd_wdgi.empty() && upd_wdgi != cwls[i_c] )	continue;
+			AutoHD<CWidget> cw = w.at().wdgAt(cwls[i_c]);
+			wN->childAdd("cw")->setAttr("id",cwls[i_c])->setText(cw.at().name())->childAdd("ico")->setText((cwls.size()>=100)?"":cw.at().ico());
+		    }
+	    }
+	}
+	return;
+    }
 
     //- Get page info -
     if( opt->name() == "info" )
     {
 	TUI::cntrCmdProc(opt);
-	ctrMkNode("grp",opt,-1,"/br/prj_",_("Project"),0664,"root","UI",1,"idm","1");
-	ctrMkNode("grp",opt,-1,"/br/wlb_",_("Widget's library"),0664,"root","UI",1,"idm","1");
+	ctrMkNode("grp",opt,-1,"/br/prj_",_("Project"),0664,"root","UI",2,"idm","1","idSz","30");
+	ctrMkNode("grp",opt,-1,"/br/wlb_",_("Widget's library"),0664,"root","UI",2,"idm","1","idSz","30");
 	ctrMkNode("grp",opt,-1,"/br/ses_",_("Session"),0664,"root","UI");
+	ctrMkNode("grp",opt,-1,"/br/vca",_("Functions"),R_R_R_,"root","UI",1,"idm","1");
 	if(ctrMkNode("area",opt,-1,"/prm/cfg",_("Configuration"),0444,"root","UI"))
 	{
-	    ctrMkNode("list",opt,-1,"/prm/cfg/prj",_("Project"),0664,"root","UI",4,"tp","br","idm","1","s_com","add,del","br_pref","prj_");
-	    ctrMkNode("list",opt,-1,"/prm/cfg/wlb",_("Widget's libraries"),0664,"root","UI",4,"tp","br","idm","1","s_com","add,del","br_pref","wlb_");
+	    ctrMkNode("list",opt,-1,"/prm/cfg/prj",_("Project"),0664,"root","UI",5,"tp","br","idm","1","s_com","add,del","br_pref","prj_","idSz","30");
+	    ctrMkNode("list",opt,-1,"/prm/cfg/wlb",_("Widget's libraries"),0664,"root","UI",5,"tp","br","idm","1","s_com","add,del","br_pref","wlb_","idSz","30");
 	}
 	if(ctrMkNode("area",opt,1,"/ses",_("Sessions"),0444,"root","UI"))
 	    ctrMkNode("list",opt,-1,"/ses/ses",_("Sessions"),0664,"root","UI",3,"tp","br","s_com","add,del","br_pref","ses_");
 	if(ctrMkNode("area",opt,2,"/tts",_("Speech text synthesis"),0444,"root","UI"))
 	{
 	    ctrMkNode("fld",opt,-1,"/tts/comm",_("Command"),0664,"root","UI",4,"tp","str","dest","sel_ed","select","/tts/comm_ls","help",
-		_("Command line for call speech text synthesis engine.\n"
+		_("Command line for call of speech synthesis from the text engine.\n"
 		  "Use next words for replace:\n"
 		  "  %t - synthesis text;\n"
 		  "  %f - result file name.\n"
-		  "If result file name not used then result readed from pipe.\n"
-		  "If used result file name and not used %t then synthesis text sending to pipe."));
-	    ctrMkNode("fld",opt,-1,"/tts/code",_("Text code"),0664,"root","UI",2,"tp","str","help",_("Engine text codepage for text encode to it."));
+		  "If result file name is not used result is read from pipe.\n"
+		  "If result file name is used and  %t is not used synthesis text is sent to pipe."));
+	    ctrMkNode("fld",opt,-1,"/tts/code",_("Text code"),0664,"root","UI",2,"tp","str","help",_("Engine text codepage for text encode into it."));
 	}
 	return;
     }
@@ -586,8 +631,8 @@ void Engine::cntrCmdProc( XMLNode *opt )
 	}
 	if( ctrChkNode(opt,"add",0664,"root","UI",SEQ_WR) )
 	{
-	    prjAdd(opt->attr("id"),opt->text());
-	    prjAt(opt->attr("id")).at().setOwner(opt->attr("user"));
+	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::ID);
+	    prjAdd(vid,opt->text()); prjAt(vid).at().setOwner(opt->attr("user"));
 	}
 	if( ctrChkNode(opt,"del",0664,"root","UI",SEQ_WR) )   prjDel(opt->attr("id"),true);
     }
@@ -600,7 +645,7 @@ void Engine::cntrCmdProc( XMLNode *opt )
 	    for( unsigned i_a=0; i_a < lst.size(); i_a++ )
 		opt->childAdd("el")->setAttr("id",lst[i_a])->setText(wlbAt(lst[i_a]).at().name());
 	}
-	if( ctrChkNode(opt,"add",0664,"root","UI",SEQ_WR) )	wlbAdd(opt->attr("id"),opt->text());
+	if( ctrChkNode(opt,"add",0664,"root","UI",SEQ_WR) )	wlbAdd(TSYS::strEncode(opt->attr("id"),TSYS::ID),opt->text());
 	if( ctrChkNode(opt,"del",0664,"root","UI",SEQ_WR) )	wlbDel(opt->attr("id"),true);
     }
     else if( a_path == "/prm/cfg/cp/cp" && ctrChkNode(opt,"set",0660,"root","UI",SEQ_WR) )
@@ -620,17 +665,23 @@ void Engine::cntrCmdProc( XMLNode *opt )
 		    if( !SYS->security().at().access(opt->attr("user"),SEQ_RD,prj.at().owner(),prj.at().grp(),prj.at().permit()) )
 			continue;
 		}
-		opt->childAdd("el")->setText(lst[i_a]);
+		opt->childAdd("el")->setAttr("user",sesAt(lst[i_a]).at().user())->setText(lst[i_a]);
 	    }
 	}
 	if( ctrChkNode(opt,"add",0664,"root","UI",SEQ_WR) )
 	{
-	    sesAdd(opt->text());
-	    sesAt(opt->text()).at().setUser(opt->attr("user"));
-	    sesAt(opt->text()).at().setBackgrnd(true);
+	    string vid = TSYS::strEncode(opt->text(),TSYS::ID);
+	    sesAdd(vid); sesAt(vid).at().setUser(opt->attr("user")); sesAt(vid).at().setBackgrnd(true);
 	}
 	if( ctrChkNode(opt,"del",0664,"root","UI",SEQ_WR) )   sesDel(opt->text(),true);
     }
+    else if( a_path == "/br/vca" && ctrChkNode(opt) )
+    {
+	vector<string> lst;
+	fList(lst);
+	for( unsigned i_f=0; i_f < lst.size(); i_f++ )
+	    opt->childAdd("el")->setAttr("id",lst[i_f])->setText(fAt(lst[i_f]).at().name());
+    }
     else if( a_path == "/tts/code" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","UI",SEQ_RD) )	opt->setText(synthCode());
