@@ -27,6 +27,7 @@
 #include <tsys.h>
 #include <tmess.h>
 
+#include <QDesktopWidget>
 #include <QIcon>
 #include <QMessageBox>
 #include <QErrorMessage>
@@ -45,7 +46,7 @@
 #define MOD_TYPE	"UI"
 #define VER_TYPE	VER_UI
 #define SUB_TYPE	"QT"
-#define VERSION		"0.7.0"
+#define VERSION		"0.8.10"
 #define AUTORS		"Roman Savochenko"
 #define DEVELOPERS	"Roman Savochenko, Lysenko Maxim, Yashina Kseniya"
 #define DESCRIPTION	"Visual operation user interface."
@@ -123,7 +124,7 @@ void TVision::modInfo( vector<string> &list )
 string TVision::modInfo( const string &name )
 {
     if( name == "SubType" )		return SUB_TYPE;
-    else if( name == _("Developers") )	return DEVELOPERS;
+    else if( name == _("Developers") )	return _(DEVELOPERS);
     else return TModule::modInfo( name);
 }
 
@@ -146,7 +147,7 @@ string TVision::optDescr( )
 
 void TVision::load_( )
 {
-#if OSC_DEBUG
+#if OSC_DEBUG >= 1
     mess_debug(nodePath().c_str(),_("Load module."));
 #endif
 
@@ -165,7 +166,7 @@ void TVision::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -180,7 +181,7 @@ void TVision::load_( )
 
 void TVision::save_( )
 {
-#if OSC_DEBUG
+#if OSC_DEBUG >= 1
     mess_debug(nodePath().c_str(),_("Save module."));
 #endif
     //- Save parameters to DB -
@@ -215,15 +216,14 @@ QMainWindow *TVision::openWindow()
 	shapesWdg.push_back( new ShapeDiagram );
 	shapesWdg.push_back( new ShapeProtocol );
 	shapesWdg.push_back( new ShapeDocument );
-	shapesWdg.push_back( new ShapeFunction );
 	shapesWdg.push_back( new ShapeBox );
-	shapesWdg.push_back( new ShapeLink );
+	shapesWdg.push_back( new ShapeFunction );
     }
 
     string user_open = startUser( );
     string user_pass = userPass( );
 
-    //- Check for start user set OK -
+    //> Check for start user set OK
     int err;
     XMLNode req("get");
     req.setAttr("path",string("/Security/")+user_open+"/%2fauth")->setAttr("password",user_pass);
@@ -237,7 +237,7 @@ QMainWindow *TVision::openWindow()
 	    if( rez == DlgUser::SelCancel ) return NULL;
 	    if( rez == DlgUser::SelErr )
 	    {
-		postMess(nodePath().c_str(),_("Auth wrong!!!"));
+		postMess(nodePath().c_str(),_("Auth is wrong!!!"));
 		continue;
 	    }
 	    user_open = d_usr.user().toAscii().data();
@@ -245,30 +245,39 @@ QMainWindow *TVision::openWindow()
 	    break;
 	}
 
-    //- Check for run projects need -
-    bool runPrj = false;
+    //> Check for run projects need
     string sprj;
+    int  screen;
+    VisRun *fsess = NULL;
     for( int p_off = 0; (sprj=TSYS::strSepParse(run_prjs,0,';',&p_off)).size(); )
     {
-	//-- Find for already opened run window --
+	screen = 0;
+	int iSep = sprj.find("-");
+	if( iSep != string::npos ) screen = atoi(sprj.substr(iSep+1).c_str());
+	sprj = sprj.substr(0,iSep);
+
+	//QDesktopWidget().screen(1)
+	//>> Find for already opened run window
 	int i_w = 0;
 	for( ; i_w < mn_winds.size(); i_w++ )
-	    if( qobject_cast<VisRun*>(mn_winds[i_w]) && ((VisRun*)mn_winds[i_w])->srcProject( ) == sprj )
+	    if( qobject_cast<VisRun*>(mn_winds[i_w]) && ((VisRun*)mn_winds[i_w])->srcProject( ) == sprj &&
+		    QDesktopWidget().screenNumber(mn_winds[i_w]) == screen )
 		break;
-	if( i_w < mn_winds.size() ) continue;
-	VisRun *sess = new VisRun( "/prj_"+sprj, user_open, user_pass, VCAStation(), true );
+	if( i_w < mn_winds.size() || screen >= QDesktopWidget().numScreens() ) continue;
+	VisRun *sess = new VisRun( "/prj_"+sprj, user_open, user_pass, VCAStation(), true, QDesktopWidget().screen(screen) );
 	sess->show();
 	sess->raise();
 	sess->activateWindow();
-	runPrj = true;
+	if( !fsess ) fsess = sess;
     }
 
-    return runPrj ? NULL : new VisDevelop( user_open, user_pass, VCAStation() );
+    if( fsess ) return fsess;
+    return new VisDevelop( user_open, user_pass, VCAStation() );
 }
 
 void TVision::modStart()
 {
-#if OSC_DEBUG
+#if OSC_DEBUG >= 1
     mess_debug(nodePath().c_str(),_("Start module."));
 #endif
 
@@ -278,7 +287,7 @@ void TVision::modStart()
 
 void TVision::modStop()
 {
-#if OSC_DEBUG
+#if OSC_DEBUG >= 1
     mess_debug(nodePath().c_str(),_("Stop module."));
 #endif
     end_run = true;
@@ -330,13 +339,14 @@ void TVision::cntrCmdProc( XMLNode *opt )
 		ctrMkNode("fld",opt,-1,"/prm/cfg/start_user",_("Start user"),0664,"root","UI",1,"tp","str");
 		ctrMkNode("fld",opt,-1,"/prm/cfg/u_pass",_("User password"),0664,"root","UI",1,"tp","str");
 	    }
-	    ctrMkNode("fld",opt,-1,"/prm/cfg/run_prj",_("Run projects list (';' - sep)"),0664,"root","UI",1,"tp","str");
+	    ctrMkNode("fld",opt,-1,"/prm/cfg/run_prj",_("Run projects list"),0664,"root","UI",2,"tp","str",
+		"help",_("Automatic started project's list separated by symbol ';'.\nFor opening a project's window to need display (1) use project's name format: 'PrjName-1'."));
 	    ctrMkNode("comm",opt,-1,"/prm/cfg/host_lnk",_("Go to remote stations list configuration"),0660,"root","UI",1,"tp","lnk");
 	}
 	if(ctrMkNode("area",opt,2,"/alarm",_("Alarms"),0444,"root","UI"))
 	    ctrMkNode("fld",opt,-1,"/alarm/plComm",_("Play command"),0664,"root","UI",4,"tp","str","dest","sel_ed","select","/alarm/plComLs","help",
 		    _("Command line for call sounds play.\n"
-		    "Use %f for source file name insert. If source file not used then play sample send to pipe."));
+		    "Use %f for source file name inserting. If source file is not used play sample is sent to pipe."));
 	ctrMkNode("fld",opt,-1,"/help/g_help",_("Options help"),0440,"root","UI",3,"tp","str","cols","90","rows","5");
 	return;
     }
@@ -420,10 +430,10 @@ void TVision::postMess( const QString &cat, const QString &mess, TVision::MessLe
 
 int TVision::cntrIfCmd( XMLNode &node, const string &user, const string &password, const string &VCAStat, bool glob )
 {
-    //- Check for local VCAEngine path -
+    //> Check for local VCAEngine path
     if( !glob ) node.setAttr("path","/UI/VCAEngine"+node.attr("path"));
 
-    //- Local station request -
+    //> Local station request
     if( VCAStat.empty() || VCAStat == "." )
     {
 	node.setAttr("user",user);
@@ -431,19 +441,22 @@ int TVision::cntrIfCmd( XMLNode &node, const string &user, const string &passwor
 	return atoi(node.attr("rez").c_str());
     }
 
-    //- Request remote host -
+    //> Request remote host
     try
     {
 	TTransportS::ExtHost host = SYS->transport().at().extHostGet("*",VCAStat);
-	AutoHD<TTransportOut> tr = SYS->transport().at().extHost(host,"VCAStat");
+	AutoHD<TTransportOut> tr = SYS->transport().at().extHost(host,"UIVision");
 	if( !tr.at().startStat() )	tr.at().start();
-	if( user.empty() || user == host.user )
-	    node.load(tr.at().messProtIO("0\n"+host.user+"\n"+host.pass+"\n"+node.save(),"SelfSystem"));
-	else node.load(tr.at().messProtIO("1\n"+user+"\n"+password+"\n"+node.save(),"SelfSystem"));
+
+	bool trUser = (user.empty()||user==host.user);
+	node.setAttr("rqDir",trUser?"0":"1")->
+	    setAttr("rqUser",trUser?host.user:user)->
+	    setAttr("rqPass",trUser?host.pass:password);
+	tr.at().messProtIO(node,"SelfSystem");
 
 	return atoi(node.attr("rez").c_str());
     }
-    catch( TError err )	
+    catch( TError err )
     {
 	node.setAttr("rez","10");
 	node.setAttr("mcat",err.cat);
