@@ -100,6 +100,7 @@ LibProjProp::LibProjProp( VisDevelop *parent ) :
     lab->setSizePolicy( QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred) );
     glay->addWidget(lab,1,1);
     obj_db = new LineEdit(tab_w,LineEdit::Combo);
+    obj_db->setSizePolicy( QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed) );
     obj_db->setObjectName("/obj/st/db");
     connect(obj_db, SIGNAL(apply()), this, SLOT(isModify()));
     glay->addWidget(obj_db,1,2);
@@ -177,7 +178,7 @@ LibProjProp::LibProjProp( VisDevelop *parent ) :
     glay->addWidget(prj_runw,7,1,1,3);
 
     grp->setLayout(glay);
-    dlg_lay->addWidget(grp,1,0,1,2);
+    dlg_lay->addWidget(grp,1,0);
 
 
     //- Add tab 'Mime data' -
@@ -486,7 +487,7 @@ void LibProjProp::selectIco( )
     if(!ico_t.load(fileName))
     {
 	mod->postMess( mod->nodePath().c_str(), 
-		QString(_("Loaded icon image '%1' error.")).arg(fileName),TVision::Warning, this );
+		QString(_("Load icon image '%1' error.")).arg(fileName),TVision::Warning, this );
 	return;
     }
 
@@ -714,6 +715,7 @@ VisItProp::VisItProp( VisDevelop *parent ) :
     lab->setSizePolicy( QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred) );
     glay->addWidget(lab,1,1);
     obj_parent = new QComboBox(tab_w);
+    obj_parent->setSizePolicy( QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed) );
     obj_parent->setObjectName("/wdg/st/parent");
     connect(obj_parent, SIGNAL(activated(int)), this, SLOT(isModify()));
     glay->addWidget(obj_parent,1,2,1,4);
@@ -797,7 +799,7 @@ VisItProp::VisItProp( VisDevelop *parent ) :
     glay->addWidget(obj_accother,7,3);
 
     grp->setLayout(glay);
-    dlg_lay->addWidget(grp,1,0,1,2);
+    dlg_lay->addWidget(grp,1,0);
 
     //- Add tab 'Attributes' -
     //------------------------
@@ -931,7 +933,7 @@ void VisItProp::showDlg( const string &iit, bool reload )
     if( owner()->cntrIfCmd(info_req) )
     {
 	mod->postMess( mod->nodePath().c_str(),
-		QString(_("Get node '%1' information error.")).arg(ed_it.c_str()),TVision::Error, this );
+		QString(_("Getting node '%1' information error.")).arg(ed_it.c_str()),TVision::Error, this );
 	return;
     }
 
@@ -1139,7 +1141,7 @@ void VisItProp::tabChanged( int itb )
 	    if( owner()->cntrIfCmd(info_req) )
 	    {
 		mod->postMess( mod->nodePath().c_str(),
-			QString(_("Get node '%1' information error.")).arg(ed_it.c_str()),TVision::Error, this );
+			QString(_("Getting node '%1' information error.")).arg(ed_it.c_str()),TVision::Error, this );
 		return;
 	    }
 	    string sval;
@@ -1300,7 +1302,7 @@ void VisItProp::selectIco( )
     if(!ico_t.load(fileName))
     {
 	mod->postMess( mod->nodePath().c_str(), 
-		QString(_("Loaded icon image '%1' error.")).arg(fileName),TVision::Warning, this );
+		QString(_("Load icon image '%1' error.")).arg(fileName),TVision::Warning, this );
 	return;
     }
 
@@ -1394,7 +1396,7 @@ void VisItProp::addAttr( )
     //- Check current attribute -
     if( swdg.empty() )
     {
-	mod->postMess( mod->nodePath().c_str(), _("Valid widget no selected"),TVision::Warning,this);
+	mod->postMess( mod->nodePath().c_str(), _("Valid widget is not selected"),TVision::Warning,this);
 	return;
     }
 
@@ -1415,7 +1417,7 @@ void VisItProp::delAttr( )
     //- Check current attribute -
     if( !obj_attr_cfg->currentItem() || !obj_attr_cfg->currentItem()->parent() )
     {
-	mod->postMess( mod->nodePath().c_str(), _("Valid attribute no selected"),TVision::Warning,this);
+	mod->postMess( mod->nodePath().c_str(), _("Valid attribute is not selected"),TVision::Warning,this);
 	return;
     }
 
@@ -1439,7 +1441,7 @@ void VisItProp::changeAttr(QTreeWidgetItem *it, int col)
     //- Check current attribute -
     if( !it || !it->parent() )
     {
-	mod->postMess( mod->nodePath().c_str(), _("Valid attribute no selected"),TVision::Warning,this);
+	mod->postMess( mod->nodePath().c_str(), _("Valid attribute is not selected"),TVision::Warning,this);
 	return;
     }
 
