@@ -38,7 +38,7 @@ using namespace VISION;
 //* Shape widget view runtime mode                *
 //*************************************************
 RunWdgView::RunWdgView( const string &iwid, int ilevel, VisRun *mainWind, QWidget* parent, Qt::WindowFlags f ) :
-    WdgView(iwid,ilevel,(QMainWindow*)mainWind,parent,f), reqtm(1), mPermCntr(false), mPermView(true)
+    WdgView(iwid,ilevel,(QMainWindow*)mainWind,parent,f), mPermCntr(false), mPermView(true)
 {
     int endElSt = iwid.rfind("/");
     if( endElSt == string::npos ) return;
@@ -52,6 +52,14 @@ RunWdgView::~RunWdgView( )
 
 }
 
+string RunWdgView::name( )
+{
+    XMLNode req("get");
+    req.setAttr("path",id()+"/%2fattr%2fname");
+    if( !cntrIfCmd(req) )	return req.text();
+    return "";
+}
+
 string RunWdgView::user( )
 {
     return mainWin()->user();
@@ -88,116 +96,94 @@ WdgView *RunWdgView::newWdgItem( const string &iwid )
     return new RunWdgView(iwid,wLevel()+1,mainWin(),this);
 }
 
-void RunWdgView::update( bool full, const string &wpath )
+void RunWdgView::update( bool full, XMLNode *aBr )
 {
-    if( !wpath.empty() )
+    bool reqBrCr = false;
+    if( !aBr )
     {
-	int off = 0;
-	RunWdgView *wdg = findChild<RunWdgView*>(TSYS::pathLev(wpath,0,true,&off).c_str());
-	if( wdg && !qobject_cast<RunPageView*>(wdg) ) wdg->update(false,wpath.substr(off));
-	return;
+	aBr = new XMLNode("get");
+	aBr->setAttr("path",id()+"/%2fserv%2fattrBr")->setAttr("tm",TSYS::uint2str(full?0:mainWin()->reqTm()));
+	cntrIfCmd(*aBr);
+	reqBrCr = true;
     }
 
-    //- Request to widget for last attributes -
-    bool change = false;
-    XMLNode *req_el;
-    XMLNode req("get");
-    req.setAttr("path",id()+"/%2fserv%2fattr")->
-        setAttr("tm",TSYS::uint2str(full?0:reqtm));
-    if( !cntrIfCmd(req) )
-    {
-        if( full )	setAllAttrLoad(true);
-        for( int i_el = 0; i_el < req.childSize(); i_el++ )
-        {
-	    req_el = req.childGet(i_el);
-	    if( attrSet("",req_el->text(),atoi(req_el->attr("pos").c_str())) )
-		change = true;
-	}
-	if( full )
-	{
-	    setAllAttrLoad(false);
-	    attrSet("","load",-1);
-	    //- Childs update for permition change -
-	    childsUpdate(true);
-	    orderUpdate();
-	    QWidget::update();
-	}
-	reqtm = strtoul(req.attr("tm").c_str(),0,10);
-    }
-
-    //- Call childs for update -
+    if( full )	setAllAttrLoad(true);
+    for( int i_el = 0; i_el < aBr->childSize(); i_el++ )
+	if( aBr->childGet(i_el)->name() == "el" )
+	    attrSet("",aBr->childGet(i_el)->text(),atoi(aBr->childGet(i_el)->attr("p").c_str()));
     if( full )
-	for( int i_c = 0; i_c < children().size(); i_c++ )
+    {
+	setAllAttrLoad(false);
+
+	attrSet("","load",-1);
+
+	//> Delete child widgets
+	for( int i_c = 0, i_l = 0; i_c < children().size(); i_c++ )
+	{
+	    if( !qobject_cast<RunWdgView*>(children().at(i_c)) || qobject_cast<RunPageView*>(children().at(i_c)) ) continue;
+	    for( i_l = 0; i_l < aBr->childSize(); i_l++ )
+		if( aBr->childGet(i_l)->name() == "w" &&
+			((WdgView*)children().at(i_c))->id() == (id()+"/wdg_"+aBr->childGet(i_l)->attr("id")) )
+		    break;
+	    if( i_l >= aBr->childSize() ) children().at(i_c)->deleteLater();
+	}
+    }
+
+    //> Create new child widget
+    for( int i_l = 0, i_c = 0; i_l < aBr->childSize(); i_l++ )
+    {
+	if( aBr->childGet(i_l)->name() != "w" ) continue;
+
+	for( i_c = 0; i_c < children().size(); i_c++ )
 	    if( qobject_cast<RunWdgView*>(children().at(i_c)) && !qobject_cast<RunPageView*>(children().at(i_c)) &&
-		    ((RunWdgView*)children().at(i_c))->isEnabled() )
-		((RunWdgView*)children().at(i_c))->update(full);
+		    ((RunWdgView*)children().at(i_c))->id() == (id()+"/wdg_"+aBr->childGet(i_l)->attr("id")) )
+	    {
+		((RunWdgView*)children().at(i_c))->update(full,aBr->childGet(i_l));
+		break;
+	    }
+	if( i_c < children().size() ) continue;
+	WdgView *nwdg = newWdgItem(id()+"/wdg_"+aBr->childGet(i_l)->attr("id"));
+	nwdg->show();
+	nwdg->load("");
+    }
+
+    if( full )
+    {
+	orderUpdate();
+	QWidget::update();
+    }
+
+    if( reqBrCr ) delete aBr;
 }
 
-void RunWdgView::childsUpdate( bool newLoad )
+void RunWdgView::shapeList( const string &snm, vector<string> &ls )
 {
-    XMLNode req("get");
+    if( shape && snm == shape->id() )	ls.push_back(id());
 
-    string b_nm = id();
-    req.setAttr("path",id()+"/%2fwdg%2fcfg%2fpath")->setAttr("resLink","1");
-    if( !cntrIfCmd(req) ) b_nm = req.text();
-
-    vector<string> lst;
-    req.clear()->setAttr("path",b_nm+"/%2finclwdg%2fwdg")->setAttr("chkUserPerm","1");
-
-    if( !cntrIfCmd(req) )
-	for( int i_el = 0; i_el < req.childSize(); i_el++ )
-	    lst.push_back(req.childGet(i_el)->attr("id"));
-
-    //- Delete child widgets -
     for( int i_c = 0; i_c < children().size(); i_c++ )
-    {
-	if( !qobject_cast<RunWdgView*>(children().at(i_c)) || qobject_cast<RunPageView*>(children().at(i_c)) ) continue;
-	int i_l;
-	for( i_l = 0; i_l < lst.size(); i_l++ )
-	if( qobject_cast<WdgView*>(children().at(i_c))->id() == (b_nm+"/wdg_"+lst[i_l]) )
-	    break;
-	if( i_l >= lst.size() ) delete children().at(i_c);
-    }
+	if( qobject_cast<RunWdgView*>(children().at(i_c)) && !qobject_cast<RunPageView*>(children().at(i_c)) && ((RunWdgView*)children().at(i_c))->isEnabled() )
+	    ((RunWdgView*)children().at(i_c))->shapeList(snm,ls);
+}
 
-    //- Create new child widget -
-    for( int i_l = 0; i_l < lst.size(); i_l++ )
-    {
-	int i_c;
-	for( i_c = 0; i_c < children().size(); i_c++ )
-	if( qobject_cast<WdgView*>(children().at(i_c)) &&
-		qobject_cast<WdgView*>(children().at(i_c))->id() == (b_nm+"/wdg_"+lst[i_l]) )
-	    break;
-	if( i_c >= children().size() )
-	{
-	    WdgView *nwdg = newWdgItem(b_nm+"/wdg_"+lst[i_l]);
-	    if( nwdg )
-	    {
-		nwdg->show();
-		if( newLoad ) nwdg->load("");
-	    }
-	}
-    }
+RunWdgView *RunWdgView::findOpenWidget( const string &iwdg )
+{
+    //> Self check
+    if( id() == iwdg ) return this;
+    //> Check to included widgets
+    RunWdgView *wdg;
+    for( int i_ch = 0; i_ch < children().size(); i_ch++ )
+	if( qobject_cast<RunWdgView*>(children().at(i_ch)) && !qobject_cast<RunPageView*>(children().at(i_ch)) && ((RunWdgView*)children().at(i_ch))->isEnabled() &&
+		(wdg=((RunWdgView*)children().at(i_ch))->findOpenWidget(iwdg)) )
+	    return wdg;
+
+    return NULL;
 }
 
 void RunWdgView::orderUpdate( )
 {
-    //- Same order of included widgets is update -
-    WdgView *lw = NULL;
-    for( int i_c = 0; i_c < children().size(); i_c++ )
-    {
-	WdgView *cw = qobject_cast<RunWdgView*>(children().at(i_c));
-	if( !cw || qobject_cast<RunPageView*>(cw) ) continue;
-	//- Change order -
-	if( lw && (cw->z() < lw->z()) )
-	{
-	    cw->stackUnder(lw);
-	    i_c = -1;
-	    lw = NULL;
-	}
-	else lw = cw;
-    }
+    WdgView::orderUpdate( );
 
-    //- Update tab order -
+    //> Update tab order
     RunWdgView *prev_aw = NULL;
     for( int i_c = 0; i_c < children().size(); i_c++ )
     {
@@ -228,6 +214,12 @@ bool RunWdgView::attrSet( const string &attr, const string &val, int uiPrmPos )
 	case 3:		//pgOpenSrc
 	    setProperty("pgOpenSrc",val.c_str());
 	    return true;
+	case 6:		//active
+	    setProperty("active",(bool)atoi(val.c_str()));
+	    return true;
+	case 17:	//contextMenu
+	    setProperty("contextMenu",val.c_str());
+	    return true;
     }
 
     return rez;
@@ -245,26 +237,54 @@ string RunWdgView::resGet( const string &res )
 
 bool RunWdgView::event( QEvent *event )
 {
-    //- Paint message about access denied -
-    if( event->type() == QEvent::Paint && !permView() )
+    //> Force event's process
+    switch( event->type() )
     {
-	QPainter pnt(this);
-	//-- Fill page and draw border --
-	pnt.fillRect(rect(),QBrush(QColor("black"),Qt::Dense4Pattern));
-	pnt.setPen(QPen(QBrush(QColor("black")),1));
-	pnt.drawRect(rect().adjusted(0,0,-1,-1));
-	//-- Draw message --
-	QTextOption to;
-	pnt.setPen(QColor("red"));
-	to.setAlignment(Qt::AlignCenter);
-	to.setWrapMode(QTextOption::WordWrap);
-	pnt.drawText(rect(),QString(_("Widget: '%1'.\nView access is no permited.")).arg(id().c_str()),to);
-	return true;
+	case QEvent::Paint:
+	    if( permView() )	break;
+	    //> Paint message about access denied
+	    if( dynamic_cast<RunPageView*>(this) )
+	    {
+		QPainter pnt(this);
+		//>> Fill page and draw border
+		pnt.fillRect(rect(),QBrush(QColor("black"),Qt::Dense4Pattern));
+		pnt.setPen(QPen(QBrush(QColor("black")),1));
+		pnt.drawRect(rect().adjusted(0,0,-1,-1));
+		//>> Draw message
+		QTextOption to;
+		pnt.setPen(QColor("red"));
+		to.setAlignment(Qt::AlignCenter);
+		to.setWrapMode(QTextOption::WordWrap);
+		pnt.drawText(rect(),QString(_("Page: '%1'.\nView access is not permited.")).arg(id().c_str()),to);
+	    }
+	    return true;
+	case QEvent::MouseButtonPress:
+	    if( ((QMouseEvent*)event)->button() == Qt::RightButton && !property("contextMenu").toString().isEmpty() && property("active").toBool() && permCntr() )
+	    {
+		QAction *actTmp;
+		QMenu popup;
+		string sln;
+		for( int off = 0; (sln=TSYS::strSepParse(property("contextMenu").toString().toAscii().data(),0,'\n',&off)).size(); )
+		{
+		    actTmp = new QAction(TSYS::strSepParse(sln,0,':').c_str(),this);
+		    actTmp->setWhatsThis(TSYS::strSepParse(sln,1,':').c_str());
+		    popup.addAction(actTmp);
+		}
+		if( !popup.isEmpty() )
+		{
+		    actTmp = popup.exec(QCursor::pos());
+		    if( actTmp && !actTmp->whatsThis().isEmpty() ) attrSet("event","usr_"+actTmp->whatsThis().toStdString());
+		    popup.clear();
+		    return true;
+		}
+	    }
+	    break;
     }
 
+    //> Call to shape for event process
     if( WdgView::event(event) || (shape&&shape->event(this,event)) )	return true;
 
-    //- Key events process for send to model -
+    //> Key events process for send to model
     string mod_ev;
     switch( event->type() )
     {
@@ -443,21 +463,30 @@ RunPageView *RunPageView::parent( )
 
 RunPageView *RunPageView::findOpenPage( const string &ipg )
 {
-    //- Self check -
+    RunPageView *pg;
+    //> Self check
     if( id() == ipg ) return this;
-    //- Check to included widgets -
+    //> Check to included widgets
     for( int i_ch = 0; i_ch < children().size(); i_ch++ )
     {
 	if( qobject_cast<RunPageView*>(children().at(i_ch)) )
 	{
-	    RunPageView *pg = ((RunPageView*)children().at(i_ch))->findOpenPage(ipg);
+	    pg = ((RunPageView*)children().at(i_ch))->findOpenPage(ipg);
 	    if( pg ) return pg;
 	    continue;
 	}
 	if( !qobject_cast<RunWdgView*>(children().at(i_ch)) )	continue;
 	RunWdgView *rwdg = (RunWdgView*)children().at(i_ch);
-	if( rwdg->root() == "Box" && rwdg->pgOpenSrc() == ipg && !rwdg->property("inclPg").toString().isEmpty() )
-	    return (RunPageView*)TSYS::str2addr(rwdg->property("inclPg").toString().toAscii().data());
+	if( rwdg->root() == "Box" )
+	{
+	    if( rwdg->pgOpenSrc() == ipg && !rwdg->property("inclPg").toString().isEmpty() )
+		return (RunPageView*)TSYS::str2addr(rwdg->property("inclPg").toString().toAscii().data());
+	    if( ((ShapeBox::ShpDt*)rwdg->shpData)->inclWidget )
+	    {
+		pg = ((ShapeBox::ShpDt*)rwdg->shpData)->inclWidget->findOpenPage(ipg);
+		if( pg ) return pg;
+	    }
+	}
     }
 
     return NULL;
@@ -465,61 +494,59 @@ RunPageView *RunPageView::findOpenPage( const string &ipg )
 
 bool RunPageView::callPage( const string &pg_it, const string &pgGrp, const string &pgSrc )
 {
-    //printf("TEST 00: %s => %s\n",id().c_str(),pg_it.c_str());
-    //- Check for set include page -
+    //> Check for set include page
     for( int i_ch = 0; i_ch < children().size(); i_ch++ )
 	if( !pgGrp.empty() && !qobject_cast<RunPageView*>(children().at(i_ch)) &&
-		((RunWdgView *)children().at(i_ch))->root() == "Box" &&
-		((RunWdgView*)children().at(i_ch))->pgGrp() == pgGrp )
+		((RunWdgView*)children().at(i_ch))->root() == "Box" )
 	{
-	    string pg_it_prev = ((RunWdgView*)children().at(i_ch))->pgOpenSrc();
-	    if( pg_it != pg_it_prev )
+	    if( ((RunWdgView*)children().at(i_ch))->pgGrp() == pgGrp )
 	    {
-		if( !pg_it_prev.empty() )
+		string pg_it_prev = ((RunWdgView*)children().at(i_ch))->pgOpenSrc();
+		if( pg_it != pg_it_prev )
 		{
-		    XMLNode req("close");
-		    req.setAttr("path","/ses_"+mainWin()->workSess()+"/%2fserv%2fpg")->setAttr("pg",pg_it_prev);
-		    mainWin()->cntrIfCmd(req);
-		    //mainWin()->wAttrSet(pg_it_prev,"pgOpen","0");
+		    if( !pg_it_prev.empty() )
+		    {
+			XMLNode req("close");
+			req.setAttr("path","/ses_"+mainWin()->workSess()+"/%2fserv%2fpg")->setAttr("pg",pg_it_prev);
+			mainWin()->cntrIfCmd(req);
+		    }
+		    ((RunWdgView*)children().at(i_ch))->setPgOpenSrc(pg_it);
 		}
-		((RunWdgView*)children().at(i_ch))->setPgOpenSrc(pg_it);
+		return true;
 	    }
-	    return true;
+	    if( ((ShapeBox::ShpDt*)((RunWdgView*)children().at(i_ch))->shpData)->inclWidget &&
+		    ((ShapeBox::ShpDt*)((RunWdgView*)children().at(i_ch))->shpData)->inclWidget->callPage(pg_it,pgGrp,pgSrc) )
+		return true;
         }
-    //- Check for open child page -
-    if( pgSrc == id().c_str() )	{ pgOpen( pg_it ); return true; }
-    //- Put checking to self include pages -
+    //> Put checking to self include pages
     for( int i_ch = 0; i_ch < children().size(); i_ch++ )
 	if( qobject_cast<RunPageView*>(children().at(i_ch)) &&
 		((RunPageView *)children().at(i_ch))->callPage(pg_it,pgGrp,pgSrc))
 	    return true;
-    //- Unknown and empty source pages open as master page child windows -
-    if( !parent() )		{ pgOpen( pg_it ); return true; }
+    //> Check for open child page or for unknown and empty source pages open as master page child windows
+    if( pgSrc == id().c_str() || this == mainWin()->master_pg )
+    {
+	RunPageView *pg = new RunPageView(pg_it,mainWin(),this);
+	pg->setAttribute(Qt::WA_DeleteOnClose);
+	pg->setWindowFlags(Qt::Sheet);
+	pg->setWindowTitle(mainWin()->windowTitle());
+	pg->load("");
+	pg->moveF(QPointF(mapToGlobal(pos()).x()+sizeF().width()/2-pg->sizeF().width()/2,
+			  mapToGlobal(pos()).y()+sizeF().height()/2-pg->sizeF().height()/2));
+	return true;
+    }
 
     return false;
 }
 
-RunPageView *RunPageView::pgOpen( const string &ipg )
-{
-    RunPageView *pg = new RunPageView(ipg,mainWin(),this);
-    pg->setAttribute(Qt::WA_DeleteOnClose);
-    pg->setWindowFlags(Qt::Sheet);
-    pg->setWindowTitle(mainWin()->windowTitle());
-    pg->load("");
-    pg->moveF(QPointF(mapToGlobal(pos()).x()+sizeF().width()/2-pg->sizeF().width()/2,
-	     mapToGlobal(pos()).y()+sizeF().height()/2-pg->sizeF().height()/2));
-
-    return pg;
-}
-
 void RunPageView::closeEvent( QCloseEvent *event )
 {
-    //-- Send close command --
+    //> Send close command
     XMLNode req("close");
     req.setAttr("path","/ses_"+mainWin()->workSess()+"/%2fserv%2fpg")->setAttr("pg",id());
     mainWin()->cntrIfCmd(req);
 
-    //-- Close included pages --
+    //> Close included pages
     for( int i_ch = 0; i_ch < children().size(); i_ch++ )
 	if( !qobject_cast<RunPageView*>(children().at(i_ch)) && ((RunWdgView *)children().at(i_ch))->root() == "Box" &&
 		!((RunWdgView*)children().at(i_ch))->pgOpenSrc().empty() )
@@ -575,4 +602,3 @@ void SndPlay::run( )
 
     mPlayData.clear();
 };
-
