@@ -38,7 +38,7 @@
 #define MOD_TYPE	"UI"
 #define VER_TYPE	VER_UI
 #define SUB_TYPE	"WWW"
-#define MOD_VERSION	"1.5.1"
+#define MOD_VERSION	"1.5.3"
 #define AUTORS		"Roman Savochenko"
 #define DESCRIPTION	"Allow the WEB based OpenSCADA system configurator."
 #define LICENSE		"GPL"
@@ -67,7 +67,7 @@ using namespace WebCfg;
 //*************************************************
 //* TWEB                                          *
 //*************************************************
-TWEB::TWEB( string name ) : m_t_auth(10), lst_ses_chk(0)
+TWEB::TWEB( string name )
 {
     mId		= MOD_ID;
     mName	= MOD_NAME;
@@ -81,24 +81,27 @@ TWEB::TWEB( string name ) : m_t_auth(10), lst_ses_chk(0)
     mod		= this;
 
     //- Reg export functions -
-    modFuncReg( new ExpFunc("void HttpGet(const string&,string&,const string&,vector<string>&);",
+    modFuncReg( new ExpFunc("void HttpGet(const string&,string&,const string&,vector<string>&,const string&);",
 	"Process Get comand from http protocol's!",(void(TModule::*)( )) &TWEB::HttpGet) );
     modFuncReg( new ExpFunc("void HttpPost(const string&,string&,const string&,vector<string>&,const string&);",
 	"Process Set comand from http protocol's!",(void(TModule::*)( )) &TWEB::HttpPost) );
 
     //- Default CSS init -
-    m_CSStables =
-	"hr {width:100%}\n"
-	"body {background-color:#818181}\n"
-	"h1.head {text-align:center; color:#ffff00 }\n"
-	"h2.title {text-align:center; font-style:italic; margin: 0px; padding: 0px; border-width:0px }\n"
-	"table.page_head {background-color:#cccccc; border:3px ridge blue; width:100% }\n"
-	"table.page_head td.tool {text-align:center; border:1px solid blue; width:120px;  white-space: nowrap }\n"
-	"table.page_head td.user {text-align:left; border:1px solid blue; width:120px; white-space: nowrap }\n"
-	"table.page_area {background-color:#9999ff; border:3px ridge #a9a9a9; width:100%; padding:2px }\n"
-	"table.page_area tr.content {background-color:#cccccc; border:5px ridge #9999ff; padding:5px }\n"
-	"table.page_auth {background-color:#9999ff; border:3px ridge #a9a9a9; padding:2px }\n"
-	"table.page_auth tr.content {background-color:#cccccc; border:5px ridge #9999ff; padding:5px }\n";
+    mCSStables =
+	"hr { width: 100%; }\n"
+	"body { background-color: #818181; }\n"
+	"h1.head { text-align:center; color:#ffff00; }\n"
+	"h2.title { text-align:center; font-style:italic; margin: 0px; padding: 0px; border-width: 0px; }\n"
+	"table.work { background-color: #9999ff; border: 3px ridge #a9a9a9; padding: 2px;  }\n"
+	"table.work td { background-color:#cccccc; text-align: left; }\n"
+	"table.work td.content { padding: 5px; padding-bottom: 20px; }\n"
+	"table.work ul { margin: 0px; padding: 0px; padding-left: 20px; }\n"
+	"table.page_head { background-color: #cccccc; border: 3px ridge blue; width: 100%; }\n"
+	"table.page_head td.tool { text-align: center; border: 1px solid blue; width:120px; white-space: nowrap; }\n"
+	"table.page_head td.tool img { height: 32px; border: 0px; vertical-align: middle; }\n"
+	"table.page_head td.user { text-align: left; border: 1px solid blue; width: 120px; white-space: nowrap; }\n"
+	"table.page_area { background-color: #9999ff; border:3px ridge #a9a9a9; width:100%; padding:2px; }\n"
+	"table.page_area tr.content { background-color: #cccccc; border:5px ridge #9999ff; padding:5px; }\n";
 }
 
 TWEB::~TWEB()
@@ -108,14 +111,16 @@ TWEB::~TWEB()
 
 string TWEB::modInfo( const string &name )
 {
-    if( name == "SubType" ) return(SUB_TYPE);
-    else return( TModule::modInfo( name) );
+    if( name == "SubType" )	return SUB_TYPE;
+    else if( name == "Auth" )	return "1";
+    else return TModule::modInfo(name);
 }
 
 void TWEB::modInfo( vector<string> &list )
 {
     TModule::modInfo(list);
     list.push_back("SubType");
+    list.push_back("Auth");
 }
 
 string TWEB::optDescr( )
@@ -123,9 +128,7 @@ string TWEB::optDescr( )
     char buf[STR_BUF_LEN];
     snprintf(buf,sizeof(buf),_(
 	"======================= The module <%s:%s> options =======================\n"
-	"---------- Parameters of the module section <%s> in config file ----------\n"
-	"SessTimeLife <time>      Time of the sesion life, minutes (default 10).\n"
-	"CSSTables    <CSS>	  CSS for creating pages.\n\n"),
+	"---------- Parameters of the module section <%s> in config file ----------\n\n"),
 	MOD_TYPE,MOD_ID,nodePath().c_str());
 
     return buf;
@@ -148,20 +151,12 @@ void TWEB::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
 
     //- Load parameters from config file -
-    m_t_auth = atoi( TBDS::genDBGet(nodePath()+"SessTimeLife",TSYS::int2str(m_t_auth)).c_str() );
-    m_CSStables = TBDS::genDBGet(nodePath()+"CSSTables",m_CSStables);
-}
-
-void TWEB::save_( )
-{
-    TBDS::genDBSet(nodePath()+"SessTimeLife",TSYS::int2str(m_t_auth));
-    TBDS::genDBSet(nodePath()+"CSSTables",m_CSStables);
 }
 
 void TWEB::modStart()
@@ -176,13 +171,12 @@ void TWEB::modStop()
 
 string TWEB::httpHead( const string &rcode, int cln, const string &cnt_tp, const string &addattr )
 {
-    return  "HTTP/1.0 "+rcode+"\n"
-	    "Server: "+PACKAGE_STRING+"\n"
-	    "Accept-Ranges: bytes\n"
-	    "Content-Length: "+TSYS::int2str(cln)+"\n"
-	    "Connection: close\n"
-	    "Content-type: "+cnt_tp+"\n"
-	    "Charset="+Mess->charset()+"\n"+addattr+"\n";
+    return  "HTTP/1.0 "+rcode+"\r\n"
+	    "Server: "+PACKAGE_STRING+"\r\n"
+	    "Accept-Ranges: bytes\r\n"
+	    "Content-Length: "+TSYS::int2str(cln)+"\r\n"
+	    "Content-type: "+cnt_tp+"\r\n"
+	    "Charset="+Mess->charset()+"\r\n"+addattr+"\r\n";
 }
 
 string TWEB::pgHead( string head_els )
@@ -196,12 +190,12 @@ string TWEB::pgHead( string head_els )
 	"  <meta http-equiv='Content-Type' content='text/html; charset="+Mess->charset()+"'/>\n"
 	"  <meta http-equiv='Cache-Control' content='no-cache'/>\n"+
 	head_els+
-	"  <link rel='shortcut icon' href='/"+MOD_ID+"/%2fico' type='image' />\n"
-	"  <title>"+PACKAGE_NAME+". "+_(MOD_NAME)+"</title>\n"
-	"  <style type='text/css'>\n"+m_CSStables+"</style>\n"
+	"  <link rel='shortcut icon' href='/"MOD_ID"/ico' type='image' />\n"
+	"  <title>"PACKAGE_NAME". "+_(MOD_NAME)+"</title>\n"
+	"  <style type='text/css'>\n"+mCSStables+"</style>\n"
 	"</head>\n"
-	"<body alink='#33ccff' link='#3366ff' text='#000000' vlink='#339999'>\n"
-	"<h1 class='head'>"+PACKAGE_NAME+". "+_(MOD_NAME)+"</h1>\n"
+	"<body>\n"
+	"<h1 class='head'>"PACKAGE_NAME". "+_(MOD_NAME)+"</h1>\n"
 	"<hr size='3'/><br/>\n";
 }
 
@@ -210,58 +204,64 @@ string TWEB::pgTail( )
     return "<hr size='3'/>\n</body>\n</html>";
 }
 
-void TWEB::HttpGet( const string &urli, string &page, const string &sender, vector<string> &vars )
+void TWEB::HttpGet( const string &urli, string &page, const string &sender, vector<string> &vars, const string &user )
 {
-    map< string, string >::iterator prmEl;
-    SSess ses(TSYS::strDecode(urli,TSYS::HttpURL),page,sender,vars,"");
+    map<string,string>::iterator prmEl;
+    SSess ses(TSYS::strDecode(urli,TSYS::HttpURL),sender,user,vars,"");
     ses.page = pgHead();
 
     try
     {
 	string zero_lev = TSYS::pathLev(ses.url,0);
-	//- Get about module page -
+	//> Get about module page
 	if( zero_lev == "about" )	getAbout(ses);
-	//- Get global image -
-	else if( zero_lev.substr(0,4) == "img_" )
+	//> Get module icon and global image
+	else if( zero_lev == "ico" || zero_lev.substr(0,4) == "img_" )
 	{
 	    string itp;
-	    ses.page=TUIS::icoGet(zero_lev.substr(4),&itp);
+	    ses.page=TUIS::icoGet( zero_lev=="ico"?"UI."MOD_ID:zero_lev.substr(4), &itp );
 	    page = httpHead("200 OK",ses.page.size(),string("image/")+itp)+ses.page;
 	    return;
 	}
 	else
 	{
-	    sesCheck( ses );
-	    //- Auth dialog preparing -
-	    if( !ses.user.size() ) getAuth( ses );
+	    ses.pg_info.setName("info");
+	    ses.pg_info.setAttr("path",ses.url)->setAttr("user",ses.user);
+	    if(cntrIfCmd(ses.pg_info)) throw TError(ses.pg_info.attr("mcat").c_str(),"%s",ses.pg_info.text().c_str());
+	    ses.root = ses.pg_info.childGet(0);
+
+	    if( ses.root->name()=="img" )
+	    {
+		//>> Transfer page image
+		XMLNode req("get"); req.setAttr("path",ses.url)->setAttr("user",ses.user);
+		if(cntrIfCmd(req)) throw TError(req.attr("mcat").c_str(),"%s",req.text().c_str());
+		ses.page=TSYS::strDecode(req.text(),TSYS::base64);
+		page = httpHead("200 OK",ses.page.size(),string("image/")+req.attr("tp"))+ses.page;
+		return;
+	    }
 	    else
 	    {
-		ses.pg_info.setName("info");
-		ses.pg_info.setAttr("path",ses.url)->setAttr("user",ses.user);
-		if(cntrIfCmd(ses.pg_info)) throw TError(ses.pg_info.attr("mcat").c_str(),"%s",ses.pg_info.text().c_str());
-		ses.root = ses.pg_info.childGet(0);
-		
-		if(ses.root->name()=="img")
+		prmEl = ses.prm.find("com");
+		string wp_com = (prmEl!=ses.prm.end()) ? prmEl->second : "";
+		if( wp_com == "load" )
 		{
-		    //-- Transfer page image --
-		    XMLNode req("get"); req.setAttr("path",ses.url)->setAttr("user",ses.user);
-		    if(cntrIfCmd(req)) throw TError(req.attr("mcat").c_str(),"%s",req.text().c_str());
-		    ses.page=TSYS::strDecode(req.text(),TSYS::base64);
-		    page = httpHead("200 OK",ses.page.size(),string("image/")+req.attr("tp"))+ses.page;
-		    return;
+		    XMLNode reqc("load"); reqc.setAttr("path",ses.url+"/%2fobj")->setAttr("user",ses.user);
+		    cntrIfCmd(reqc);
 		}
-		else
+		else if( wp_com == "save" )
 		{
-		    //-- Get area --
-		    getHead( ses );
-		    getArea( ses, *ses.root, "/" );
+		    XMLNode reqc("save"); reqc.setAttr("path",ses.url+"/%2fobj")->setAttr("user",ses.user);
+		    cntrIfCmd(reqc);
 		}
+		//>> Get area
+		getHead( ses );
+		getArea( ses, *ses.root, "/" );
 	    }
 	}
     }catch(TError err)
     {
 	ses.page = "Page <"+ses.url+"> error: "+err.mess;
-	//postMess(ses.page,err.cat,err.mess,TWEB::Error); 
+	//postMess(ses.page,err.cat,err.mess,TWEB::Error);
 	page = httpHead("404 Not Found",ses.page.size())+ses.page;
 	return;
     }
@@ -274,25 +274,28 @@ void TWEB::HttpGet( const string &urli, string &page, const string &sender, vect
 
 void TWEB::getAbout( SSess &ses )
 {
-    ses.page = ses.page+"<center><table class='page_auth'>\n"
-	"<TR><TD>"+PACKAGE+" "+VERSION+"</TD></TR>\n"
-	"<TR class='content'><TD>\n"
-	"<table border='0'>\n"
-	"<TR><TD><font color='Blue'>"+_("Name: ")+"</font></TD><TD>"+_("Open Supervisory Control And Data Acquisition")+"</TD></TR>\n"
-	"<TR><TD><font color='Blue'>"+_("License: ")+"</font></TD><TD>GPL</TD></TR>\n"
-	"<TR><TD><font color='Blue'>"+_("Author: ")+"</font></TD><TD>Roman Savochenko</TD></TR>\n"
+    ses.page = ses.page+"<center>\n"
+	"<table class='work'>\n"
+	"<tr><th>"+MOD_ID+" v"+MOD_VERSION+"</th></tr>\n"
+	"<tr><td class='content'>\n"
+	"<table border='0px' cellspacing='3px'>\n"
+	"<TR><TD style='color: blue;'>"+_("Name: ")+"</TD><TD>"+_(MOD_NAME)+"</TD></TR>\n"
+	"<TR><TD style='color: blue;'>"+_("Desription: ")+"</TD><TD>"+_(DESCRIPTION)+"</TD></TR>\n"
+	"<TR><TD style='color: blue;'>"+_("License: ")+"</TD><TD>"+_(LICENSE)+"</TD></TR>\n"
+	"<TR><TD style='color: blue;'>"+_("Author: ")+"</TD><TD>"+_(AUTORS)+"</TD></TR>\n"
+	"</table>\n"
+	"</TD></TR>\n</table><br/>\n"
+	"<table class='work'>\n"
+	"<TR><th>"+PACKAGE+" v"+VERSION+"</th></TR>\n"
+	"<TR><TD class='content'>\n"
+	"<table border='0px' cellspacing='3px'>\n"
+	"<TR><TD style='color: blue;'>"+_("Name: ")+"</TD><TD>"+_(PACKAGE_DESCR)+"</TD></TR>\n"
+	"<TR><TD style='color: blue;'>"+_("License: ")+"</TD><TD>"+PACKAGE_LICENSE+"</TD></TR>\n"
+	"<TR><TD style='color: blue;'>"+_("Author: ")+"</TD><TD>"+_(PACKAGE_AUTHOR)+"</TD></TR>\n"
+	"<TR><TD style='color: blue;'>"+_("Web site: ")+"</TD><TD>"+PACKAGE_SITE+"</TD></TR>\n"
 	"</table>\n"
 	"</TD></TR></table><br/>\n"
-	"<table class='page_auth'>\n"
-	"<TR><TD>"+MOD_ID+" "+MOD_VERSION+"</TD></TR>\n"
-	"<TR class='content'><TD>\n"
-	"<table border='0'>\n"
-	"<TR><TD><font color='Blue'>"+_("Name: ")+"</font></TD><TD>"+_(MOD_NAME)+"</TD></TR>"
-	"<TR><TD><font color='Blue'>"+_("Desription: ")+"</font></TD><TD>"+_(DESCRIPTION)+"</TD></TR>"
-	"<TR><TD><font color='Blue'>"+_("License: ")+"</font></TD><TD>"+_(LICENSE)+"</TD></TR>"
-	"<TR><TD><font color='Blue'>"+_("Author: ")+"</font></TD><TD>"+_(AUTORS)+"</TD></TR>"
-	"</table>\n"
-	"</TD></TR>\n</table><br/></center>\n";
+	"</center>\n";
 }
 
 void TWEB::getHead( SSess &ses )
@@ -301,10 +304,14 @@ void TWEB::getHead( SSess &ses )
 
     ses.page = ses.page+"<table class='page_head'><tr>\n"
 	"<td class='tool'>\n"
-	"<a href='/"+MOD_ID+"'><img src='/"+MOD_ID+"/img_gohome' alt='"+_("Root page")+"' border='0'/></a>\n"
-	"<a href='"+path+"'><img src='/"+MOD_ID+"/img_reload' alt='" +_("Curent page")+"' border='0'/></a>\n"
-	"<a href='"+path.substr(0,path.rfind("/"))+"'><img src='/"+MOD_ID+"/img_up' alt='"+_("Previos page")+"' border='0'/></a>\n"
-	"<a href='/"+MOD_ID+"/about'><img src='/"+MOD_ID+"/img_help' alt='"+_("About")+"' border='0'/></a>\n"
+	"<a href='"+path+"?com=load' title='"+_("Load")+"'><img src='/"MOD_ID"/img_load' alt='"+_("Load")+"'/></a>\n"
+	"<a href='"+path+"?com=save' title='"+_("Save")+"'><img src='/"MOD_ID"/img_save' alt='"+_("Save")+"'/></a>\n"
+	"<img src='/"MOD_ID"/img_line'/>\n"
+	"<a href='/"MOD_ID"' title='"+_("Root page")+"'><img src='/"MOD_ID"/img_gohome' alt='"+_("Root page")+"'/></a>\n"
+	"<a href='"+path+"' title='"+_("Curent page")+"'><img src='/"MOD_ID"/img_reload' alt='" +_("Curent page")+"'/></a>\n"
+	"<a href='"+path.substr(0,path.rfind("/"))+"' title='"+_("Previous page")+"'><img src='/"MOD_ID"/img_up' alt='"+_("Previos page")+"'/></a>\n"
+	"<img src='/"MOD_ID"/img_line'/>\n"
+	"<a href='/"MOD_ID"/about' title='"+_("About")+"'><img src='/"MOD_ID"/img_help' alt='"+_("About")+"'/></a>\n"
 	"</td>\n"
 	"<td>";
     if(ses.root->childGet("id","ico",true))
@@ -313,10 +320,8 @@ void TWEB::getHead( SSess &ses )
 	"<td class='user'";
     if(ses.user == "root") ses.page = ses.page + " bgcolor='red'";
     else                   ses.page = ses.page + " bgcolor='LawnGreen'";
-    ses.page = ses.page+">"+_("user:")+" <b>"+ses.user+"</b><br/>"+_("from:")+" <b>"+ses.sender+"</b>\n"
-	"<form action='"+path+"' method='post' enctype='multipart/form-data'>\n"
-	"<input name='auth_ch' type='submit' value='"+_("Change")+"'/>\n"
-	"</form></td>\n"
+    ses.page = ses.page+">"+_("user:")+" <b>"+ses.user+"</b><br/>"+_("from:")+" <b>"+ses.sender+"</b><br/>\n"
+	"(<a href='/login'>"+_("Change")+"</a>, <a href='/logout'>"+_("Exit")+"</a>)\n"
 	"</tr></table><br/>\n";
 }
 
@@ -527,11 +532,11 @@ bool TWEB::getVal( SSess &ses, XMLNode &node, string a_path, bool rd )
 		}
 		else if( node.attr("tp") == "time" )
 		{
-		    struct tm *tm_tm;
+		    struct tm tm_tm;
 		    time_t tm_t;
 		    if( dt_req.text().size() ) tm_t = atoi(dt_req.text().c_str());
 		    else                       tm_t = time(NULL);
-		    tm_tm = localtime(&tm_t);
+		    localtime_r(&tm_t,&tm_tm);
 		    if( !wr )
 		    {
 			char *c_tm = ctime( &tm_t );
@@ -542,13 +547,13 @@ bool TWEB::getVal( SSess &ses, XMLNode &node, string a_path, bool rd )
 		    else
 		    {
 			string s_id = node.attr("id");
-			ses.page = ses.page+"<input type='text' name='"+s_id+"_d' value='"+TSYS::int2str(tm_tm->tm_mday)+"' maxlength='2' size='2'/>\n";
-			ses.page = ses.page+"<input type='text' name='"+s_id+"_ms' value='"+TSYS::int2str(tm_tm->tm_mon+1)+"' maxlength='2' size='2'/>\n";
-			ses.page = ses.page+"<input type='text' name='"+s_id+"_y' value='"+TSYS::int2str(tm_tm->tm_year+1900)+"' maxlength='4' size='4'/>\n";
+			ses.page = ses.page+"<input type='text' name='"+s_id+"_d' value='"+TSYS::int2str(tm_tm.tm_mday)+"' maxlength='2' size='2'/>\n";
+			ses.page = ses.page+"<input type='text' name='"+s_id+"_ms' value='"+TSYS::int2str(tm_tm.tm_mon+1)+"' maxlength='2' size='2'/>\n";
+			ses.page = ses.page+"<input type='text' name='"+s_id+"_y' value='"+TSYS::int2str(tm_tm.tm_year+1900)+"' maxlength='4' size='4'/>\n";
 			ses.page = ses.page+ " , ";
-			ses.page = ses.page+"<input type='text' name='"+s_id+"_h' value='"+TSYS::int2str(tm_tm->tm_hour)+"' maxlength='2' size='2'/>\n";
-			ses.page = ses.page+"<input type='text' name='"+s_id+"_m' value='"+TSYS::int2str(tm_tm->tm_min)+"' maxlength='2' size='2'/>\n";
-			ses.page = ses.page+"<input type='text' name='"+s_id+"_s' value='"+TSYS::int2str(tm_tm->tm_sec)+"' maxlength='2' size='2'/>\n";
+			ses.page = ses.page+"<input type='text' name='"+s_id+"_h' value='"+TSYS::int2str(tm_tm.tm_hour)+"' maxlength='2' size='2'/>\n";
+			ses.page = ses.page+"<input type='text' name='"+s_id+"_m' value='"+TSYS::int2str(tm_tm.tm_min)+"' maxlength='2' size='2'/>\n";
+			ses.page = ses.page+"<input type='text' name='"+s_id+"_s' value='"+TSYS::int2str(tm_tm.tm_sec)+"' maxlength='2' size='2'/>\n";
 		    }
 		}
 		else
@@ -581,7 +586,7 @@ bool TWEB::getVal( SSess &ses, XMLNode &node, string a_path, bool rd )
 	if( node.attr("tp") == "br" || wr )
 	    ses.page = ses.page+"<form action='"+path+"' method='post' enctype='multipart/form-data'>\n";
 
-	XMLNode req("get"); 
+	XMLNode req("get");
 	req.setAttr("path",ses.url+"/"+TSYS::strEncode( a_path, TSYS::PathEl ))->
 	    setAttr("user",ses.user);
 	if(cntrIfCmd(req)) ses.mess.push_back(req.text().c_str());
@@ -787,58 +792,12 @@ bool TWEB::getVal( SSess &ses, XMLNode &node, string a_path, bool rd )
     return wr;
 }
 
-void TWEB::getAuth( SSess &ses )
-{
-    ses.page = ses.page+"<center><table class='page_auth'>"
-		"<tr><td><b>"+_("Enter to module")+"</b></td></tr>\n"
-		"<tr class='content'> <td align='center'>\n"
-		"<form method='post' action='/"+MOD_ID+ses.url+"' enctype='multipart/form-data'>\n"
-		"<table cellpadding='3'>\n"
-		"<tr><td><b>"+_("User name")+"</b></td><td><input type='text' name='user' size='20'/></td></tr>\n"
-		"<tr><td><b>"+_("Password")+"</b></td><td><input type='password' name='pass' size='20'/></td></tr>\n"
-		"<tr><td colspan='2' align='center'><input type='submit' name='auth_enter' value='"+_("Enter")+"'/>\n"
-		"<input type='reset' name='clean' value='"+_("Clean")+"'/>\n"
-		"</td></tr></table>\n</form>\n"
-		"</td></tr></table></center>\n";
-}
-
-void TWEB::HttpPost( const string &url, string &page, const string &sender, vector<string> &vars, const string &contain )
+void TWEB::HttpPost( const string &url, string &page, const string &sender, vector<string> &vars, const string &user )
 {
     map< string, string >::iterator cntEl;
-    SSess ses(TSYS::strDecode(url,TSYS::HttpURL),page,sender,vars,contain);
+    SSess ses(TSYS::strDecode(url,TSYS::HttpURL),sender,user,vars,page);
 
-    //- Check for autentification POST requests -
-    if( ses.cnt.find("auth_enter") != ses.cnt.end() )
-    {
-	string pass;
-	if( (cntEl=ses.cnt.find("user")) != ses.cnt.end() )	ses.user = cntEl->second;
-	if( (cntEl=ses.cnt.find("pass")) != ses.cnt.end() )	pass = cntEl->second;
-	if( SYS->security().at().usrPresent(ses.user) && SYS->security().at().usrAt(ses.user).at().auth(pass) )
-	{
-	    ses.page = pgHead("<META HTTP-EQUIV='Refresh' CONTENT='0; URL=/"MOD_ID"/"+url+"'/>")+pgTail();
-	    page=httpHead("200 OK",ses.page.size(),"text/html",
-		"Set-Cookie: oscd_u_id="+TSYS::int2str(sesOpen(ses.user))+"; path=/;\n")+ses.page;
-	    return;
-	}
-	ses.page = pgHead();
-	messPost(ses.page,nodePath(),_("Auth wrong! Retry please."),TWEB::Error);
-	ses.page += "\n";
-	getAuth( ses );
-	ses.page += pgTail();
-	page = httpHead("200 OK",ses.page.size(),"text/html")+ses.page;
-	return;
-    }
-    //- Session check -
-    sesCheck( ses );
-    if( !ses.user.size() || ses.cnt.find("auth_ch") != ses.cnt.end() )
-    {
-	ses.page = pgHead("<META HTTP-EQUIV='Refresh' CONTENT='0; URL=/"MOD_ID"/"+url+"'/>")+pgTail();
-	page=httpHead("200 OK",ses.page.size(),"text/html",
-	"Set-Cookie: oscd_u_id=""; path=/;\n")+ses.page;
-	return;
-    }
-
-    //- Commands process -
+    //> Commands process
     ses.page = pgHead();
     try
     {
@@ -847,10 +806,10 @@ void TWEB::HttpPost( const string &url, string &page, const string &sender, vect
 	if(cntrIfCmd(ses.pg_info)) throw TError(ses.pg_info.attr("mcat").c_str(),"%s",ses.pg_info.text().c_str());
 	ses.root = ses.pg_info.childGet(0);
 
-	//-- Parse post category and path to area --
+	//>> Parse post category and path to area
 	string prs_cat, prs_path;
 	unsigned i_el;
- 
+
 	string prs_comm;
 	for( cntEl = ses.cnt.begin(); cntEl != ses.cnt.end(); cntEl++ )
 	{
@@ -874,7 +833,7 @@ void TWEB::HttpPost( const string &url, string &page, const string &sender, vect
 		getArea( ses, *ses.root, "/" );
 	    }
 	}
-    }catch(TError err) 
+    }catch(TError err)
     { messPost(ses.page,err.cat,err.mess,TWEB::Error); }
 
     colontDown( ses );
@@ -926,15 +885,15 @@ int  TWEB::postVal( SSess &ses, XMLNode &node, string prs_path)
 	XMLNode *t_c = node.childGet(i_cf);
 	if( t_c->name() == "fld" && (atoi(t_c->attr("acs").c_str())&SEQ_WR) )
 	{
-	    if( !valPrepare( ses, *t_c,prs_path, true ) ) 
+	    if( !valPrepare( ses, *t_c,prs_path, true ) )
 		continue;
 	    mess_info(nodePath().c_str(),_("%s| Change <%s:%s> to %s"),
 		ses.user.c_str(),
 		t_c->attr("id").c_str(),
 		t_c->attr("dscr").c_str(),
 		t_c->text().c_str());
-		
-	    XMLNode req("set"); 
+
+	    XMLNode req("set");
 	    req.setAttr("path",ses.url+"/"+TSYS::strEncode(prs_path+t_c->attr("id"), TSYS::PathEl))->
 		setAttr("user",ses.user)->setText(t_c->text());
 	    if(cntrIfCmd(req)) { ses.mess.push_back(req.text().c_str()); return 0x01; }
@@ -1114,7 +1073,7 @@ int TWEB::postList( SSess &ses, XMLNode &node, string prs_path )
 	    ses.user.c_str(), f_path.c_str(), i_pos.c_str(), i_pos_to.c_str());
     }
 
-    if(cntrIfCmd(n_el1)) ses.mess.push_back(n_el1.text().c_str());    
+    if(cntrIfCmd(n_el1)) ses.mess.push_back(n_el1.text().c_str());
 
     return 0x01;
 }
@@ -1282,7 +1241,7 @@ bool TWEB::valPrepare( SSess &ses, XMLNode &node, string prs_path, bool compare
 {
     unsigned i_cnt;
     string   val;
-    map< string, string >::iterator cntEl;
+    map<string,string>::iterator cntEl;
 
     XMLNode req("get");
     req.setAttr("path",ses.url+"/"+TSYS::strEncode(prs_path+node.attr("id"),TSYS::PathEl))->
@@ -1299,9 +1258,7 @@ bool TWEB::valPrepare( SSess &ses, XMLNode &node, string prs_path, bool compare
     }
     else if( node.attr("tp") == "time" )
     {
-	//time_t c_tm = time(NULL);
 	struct tm tm_tm;
-	//tm_tm = *localtime(&c_tm);
 	string s_id = node.attr("id");
 	tm_tm.tm_isdst = -1;
 	if( (cntEl=ses.cnt.find(s_id+"_d")) != ses.cnt.end() )
@@ -1347,56 +1304,15 @@ int TWEB::cntrIfCmd( XMLNode &node )
     return atoi(node.attr("rez").c_str());
 }
 
-int TWEB::sesOpen( string name )
-{
-    int sess_id;
-    ResAlloc res(m_res,true);
-
-    //- Get free identifier -
-    do{ sess_id = rand(); }
-    while( sess_id == 0 || m_auth.find(sess_id) != m_auth.end() );
-
-    //- Add new session authentification -
-    m_auth[sess_id] = SAuth(name,time(NULL));
-
-    return sess_id;
-}
-
-void TWEB::sesCheck( SSess &ses )
-{
-    time_t cur_tm = time(NULL);
-    map< int, SAuth >::iterator authEl;
-
-    ResAlloc res(m_res,false);
-
-    //- Check for close old sessions -
-    if( cur_tm > lst_ses_chk+10 )
-    {
-	for( authEl = m_auth.begin(); authEl != m_auth.end(); )
-	    if( cur_tm > authEl->second.t_auth+m_t_auth*60 )
-		m_auth.erase(authEl++);
-	    else authEl++;
-	lst_ses_chk = cur_tm;
-    }
-
-    //- Check for session and close old sessions -
-    authEl = m_auth.find(atoi(getCookie( "oscd_u_id", ses.vars ).c_str()));
-    if( authEl != m_auth.end() )
-    {
-	ses.user = authEl->second.name;
-	authEl->second.t_auth = cur_tm;
-    }
-}
-
 string TWEB::getCookie( string name, vector<string> &vars )
 {
     for( unsigned i_var = 0; i_var < vars.size(); i_var++)
 	if( vars[i_var].substr(0, vars[i_var].find(":",0)) == "Cookie" )
 	{
 	    int i_beg = vars[i_var].find(name+"=",0);
-	    if( i_beg == string::npos ) return("");
+	    if( i_beg == string::npos ) return "";
 	    i_beg += name.size()+1;
-	    return( vars[i_var].substr(i_beg,vars[i_var].find(";",i_beg)-i_beg) );
+	    return vars[i_var].substr(i_beg,vars[i_var].find(";",i_beg)-i_beg);
 	}
     return "";
 }
@@ -1418,48 +1334,16 @@ void TWEB::colontDown( SSess &ses )
 
 string TWEB::cntGet( SSess &ses, const string &nm )
 {
-    map< string, string >::iterator prmEl = ses.cnt.find(nm);
+    map<string,string>::iterator prmEl = ses.cnt.find(nm);
     if( prmEl != ses.cnt.end() ) return prmEl->second;
     return "<empty>";
 }
 
-void TWEB::cntrCmdProc( XMLNode *opt )
-{
-    //-- Get page info --
-    if( opt->name() == "info" )
-    {
-	TUI::cntrCmdProc(opt);
-	if(ctrMkNode("area",opt,1,"/prm/cfg",_("Module options")))
-	{
-	    ctrMkNode("fld",opt,-1,"/prm/cfg/lf_tm",_("Life time of auth sesion(min)"),0660,"root","root",1,"tp","dec");
-	    ctrMkNode("fld",opt,-1,"/prm/cfg/CSS",_("CSS"),0660,"root","root",3,"tp","str","cols","90","rows","7");
-	}
-	ctrMkNode("fld",opt,-1,"/help/g_help",_("Options help"),0440,"root","root",3,"tp","str","cols","90","rows","5");
-        return;
-    }
-
-    //-- Process command to page --
-    string a_path = opt->attr("path");
-    if( a_path == "/prm/cfg/lf_tm" )
-    {
-	if( ctrChkNode(opt,"get",0660,"root","root",SEQ_RD) )	opt->setText( TSYS::int2str(authTime()) );
-	if( ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )	setAuthTime( atoi(opt->text().c_str()) );
-    }
-    else if( a_path == "/prm/cfg/CSS" )
-    {
-	if( ctrChkNode(opt,"get",0660,"root","root",SEQ_RD) )	opt->setText( CSStables() );
-	if( ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )	setCSStables( opt->text() );
-    }
-    else if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440) )	opt->setText(optDescr());
-    else TUI::cntrCmdProc(opt);
-}
-
 //*************************************************
 //* SSess                                         *
 //*************************************************
-SSess::SSess( const string &iurl, const string &ipage, const string &isender,
-	vector<string> &ivars, const string &icontent ) :
-    url(iurl), page(ipage), sender(isender), vars(ivars), content(icontent)
+SSess::SSess( const string &iurl, const string &isender, const string &iuser, vector<string> &ivars, const string &icontent ) :
+    url(iurl), sender(isender), user(iuser), vars(ivars), content(icontent)
 {
     //- URL parameters parse -
     int prmSep = iurl.find("?");
