@@ -24,8 +24,8 @@
 #define VCA_SESS_H
 
 #include <gd.h>
-#include <gdfonts.h>
-#include <gdfontt.h>
+
+#include <fftw3.h>
 
 #include <string>
 #include <vector>
@@ -45,6 +45,11 @@ class Point
 };
 
 typedef map<int,Point> PntMap;
+typedef map<int,int> WidthMap;
+typedef map<int,int> ColorMap;
+typedef map<int,string> ImageMap;
+typedef map<int,int> StyleMap;
+
 
 using std::string;
 using std::vector;
@@ -73,6 +78,7 @@ class VCAObj : public TCntrNode
 	virtual void setAttrs( XMLNode &node, const string &user ) = 0;
 
 	VCASess &owner( );
+
     protected:
 	string imgDef;
 
@@ -136,7 +142,7 @@ class VCAElFigure : public VCAObj
 
 	bool lineIntersect( double x1, double y1, double x2, double y2,
 			    double x3, double y3, double x4, double y4 );
-	//Point line_intersect_point( Point pt1, Point pt2, Point pt3, Point pt4 );
+        //vector<Point> floodFill( gdImagePtr im, int xMin, int yMin, int xMax, int yMax, int x, int y, int border_color);
 	double angle( const Point p1, const Point p2, const Point p3, const Point p4 );
 	double length( const Point pt1, const Point pt2 );
 	Point arc( double t, double a, double b );
@@ -146,30 +152,38 @@ class VCAElFigure : public VCAObj
 	double bezierDeltaT( Point p1, Point p2, Point p3, Point p4 );
 	double ABS(double var);
 	bool isPaintable( ShapeItem item, double xScale, double yScale );
-	void paintFigure( gdImagePtr im, ShapeItem item, double xScale, double yScale, bool flag_allocate, bool flag_style, bool flag_clr_ln );
+	void paintFigure( gdImagePtr im, ShapeItem item, double xScale, double yScale, bool flag_allocate, bool flag_style );
 	void paintFigureBorders( gdImagePtr im, Point el_p1, Point el_p2, Point el_p3, Point el_p4, Point el_p5, Point el_p6, int  clr_el, int clr_el_line, double el_width, double el_border_width, int type, double xScale, double yScale );
 	void dashDot( gdImagePtr im, Point el_p1, Point el_p2, Point el_p3, Point el_p4, Point el_p5, Point el_p6, int  clr_el, double el_width, int type, int style  );
         void dashDotFigureBorders( gdImagePtr im, Point el_p1, Point el_p2, Point el_p3, Point el_p4, Point el_p5, Point el_p6, int  clr_el, int clr_el_line, double el_width, double el_border_width, int type, double wdt, double wdt_1, double xScale, double yScale  );
-	void paintFill( gdImagePtr im, Point pnt, InundationItem &in_item, int color );
+	void paintFill( gdImagePtr im, Point pnt, InundationItem &in_item );
 	Point unscaleUnrotate( Point point, double xScale, double yScale, bool flag_scale, bool flag_rotate );
         Point scaleRotate( Point point, double xScale, double yScale, bool flag_scale, bool flag_rotate );
+        int drawElF( SSess &ses, double xSc, double ySc, Point clickPnt );
 	//Attributes
 	double	width,		//Widget geometry
 		height;
 	int	geomMargin,	//Margin
 		lineClr,
-		lineDecor,
 		bordWdth,
 		bordClr,
 		lineWdth,
-		fillClr;
-	string	elLst, lineStyle;
+		fillClr,
+                lineStyle,
+                scaleHeight,
+                scaleWidth;
+	string	elLst;
 	double	orient;
-	bool	active,		//Active diagram
+	bool	active,         //Active diagram
 		rel_list;
         gdImagePtr im;
 
 	PntMap pnts;
+        WidthMap widths;
+        ColorMap colors;
+        ImageMap images;
+        StyleMap styles;
+
 	vector<ShapeItem> shapeItems;
 	vector<InundationItem> inundationItems;
 
@@ -190,22 +204,29 @@ class VCADiagram : public VCAObj
 	void postReq( SSess &ses );
 	void setAttrs( XMLNode &node, const string &user );
 
+	void makeTrendsPicture( SSess &ses );
+	void makeSpectrumPicture( SSess &ses );
+
 	//Attributes
 	int		width,height,		//Widget geometry
-			tArX,tArY,tArW,tArH,	//Trend area geometry
-			geomMargin,		//Margin
-			bordWidth;		//Border width
-	bool		active,			//Active diagram
-			tTimeCurent;		//Curent time
+			tArX,tArY,tArW,tArH;	//Trend area geometry
+	short		active		:1;	//Active diagram
+	short		type		:3;	//Diagram type
+	short		bordWidth	:8;	//Border width
+	short		geomMargin	:8;	//Margin
+	short		tTimeCurent	:1;	//Curent time
+	short		trcPer		:10;	//Tracing period
+	short		sclHor		:4;	//Horisontal scale mode
+	short		sclVer		:4;	//Vertical scale mode
 	long long	tTime, curTime, tPict;	//Trend time, trend cursor's time position and picture time
-	int		trcPer;			//Tracing period
 	float		tSize;			//Trend size (s)
+	float		fftBeg, fftEnd;
 	int		curColor, 		//Cursor line color
 			sclColor,		//Scale grid color
-			sclMarkColor;		//Scale markers color
+			sclMarkColor,		//Scale markers color
+			sclMarkFontSize;	//Scale markers font size
 	string		sclMarkFont,		//Scale markers font
 			valArch;		//Value archivator
-	int		sclHor, sclVer;		//Horisontal and Vertical scale mode 
 	time_t		lstTrc;			//Last trace
 
     private:
@@ -225,6 +246,7 @@ class VCADiagram : public VCAObj
 
 		//Methods
 		TrendObj( VCADiagram *owner );
+		~TrendObj( );
 
 		string addr( )		{ return m_addr; }
 		double bordL( )		{ return m_bord_low; }
@@ -244,9 +266,16 @@ class VCADiagram : public VCAObj
 		void setCurVal( double vl )	{ m_curvl = vl; }
 
 		void loadData( const string &user, bool full = false );
+		void loadTrendsData( const string &user, bool full );
+		void loadSpectrumData( const string &user, bool full );
 
 		VCADiagram &owner( );
 
+		//Attributes
+		//> FFT
+		int	fftN;			//Spectrum samples number
+		fftw_complex	*fftOut;	//Spectrum out buffer, size = fftN/2+1
+
 	    private:
 		//Attributes
 		string		m_addr;		//A parameter or an archive item address
@@ -288,23 +317,45 @@ class VCASess : public TCntrNode
 	void getReq( SSess &ses );
 	void postReq( SSess &ses );
 
-	//- Objects -
+	//> Objects
 	void objList( vector<string> &list )		{ chldList(id_objs,list); }
 	bool objPresent( const string &name )		{ return chldPresent(id_objs,name); }
 	void objAdd( VCAObj *obj );
 	void objDel( const string &name )		{ chldDel(id_objs,name); }
 	AutoHD<VCAObj> objAt( const string &name )	{ return chldAt(id_objs,name); }
 
+	string resGet( const string &res, const string &path, const string &user, string *mime = NULL );
+
+	string cacheResGet( const string &res, string *mime = NULL );
+	void cacheResSet( const string &res, const string &val, const string &mime );
+
+	Res &nodeRes( )			{ return nRes; }
+
     private:
+	//Data
+	class CacheEl
+	{
+	    public:
+	    CacheEl( time_t itm, const string &ival, const string &imime ) : tm(itm), val(ival), mime(imime)	{ }
+	    CacheEl( )	{ }
+
+	    time_t	tm;
+	    string	val;
+	    string	mime;
+	};
+
 	//Methods
 	string nodeName( )		{ return m_id; }
 	void postDisable( int flag );
 
 	//Attributes
 	string	m_id;
-	int	id_objs;		//Primitive object's container identifier
+	int	id_objs;			//Primitive object's container identifier
 	time_t	lst_ses_req;
 	bool	mIsCreate;
+	map<string,CacheEl>	mCacheRes;	//Resources cache
+
+	Res	nRes;
 };
 
 }
