@@ -27,18 +27,16 @@
 #include <map>
 
 #include "resalloc.h"
+#include "tvariant.h"
 
 using std::string;
 using std::vector;
 using std::map;
 
-#define BUF_ARCH_NM "<bufer>"
-
 //*************************************************
 //* Value archivator                              *
 //*************************************************
 
-
 //*************************************************
 //* TValBuf                                       *
 //*   Value buffer object. Contain a massif of    *
@@ -52,17 +50,20 @@ class TValBuf
 	TValBuf( TFld::Type vtp, int isz, long long ipr, bool ihgrd = false, bool ihres = false );
 	~TValBuf( );
 
+	TValBuf &operator=( TValBuf &src );
+
 	void clear( );
 
-	TFld::Type valType( )	{ return m_val_tp; }
-	bool hardGrid( )	{ return m_hrd_grd; }
-	bool highResTm( )	{ return m_hg_res_tm; }
-	int size( )		{ return m_size; }
+	TFld::Type valType( )	{ return mValTp; }
+	bool hardGrid( )	{ return mHrdGrd; }
+	bool highResTm( )	{ return mHgResTm; }
+	int size( )		{ return mSize; }
 	int realSize( );
-	long long period( )	{ return m_per; }
+	unsigned int evalCnt( )	{ return mEvalCnt; }
 
-	long long begin( )	{ return m_beg; }
-	long long end( )	{ return m_end; }
+	long long begin( )	{ return mBeg; }
+	long long end( )	{ return mEnd; }
+	long long period( )	{ return mPer; }
 
 	bool vOK( long long ibeg, long long iend );
 
@@ -72,31 +73,31 @@ class TValBuf
 	void setSize( int vl );
 	void setPeriod( long long vl );
 
-	//Get value
-	virtual void getVal( TValBuf &buf, long long beg = 0, long long end = 0 );
+	//> Get value
+	virtual void getVals( TValBuf &buf, long long beg = 0, long long end = 0 );
 	virtual string getS( long long *tm = NULL, bool up_ord = false );
 	virtual double getR( long long *tm = NULL, bool up_ord = false );
 	virtual int    getI( long long *tm = NULL, bool up_ord = false );
 	virtual char   getB( long long *tm = NULL, bool up_ord = false );
 
-	//Set value
-	virtual void setVal( TValBuf &buf, long long beg = 0, long long end = 0 );
+	//> Set value
+	virtual void setVals( TValBuf &buf, long long beg = 0, long long end = 0 );
 	virtual void setS( const string &value, long long tm = 0 );
 	virtual void setR( double value, long long tm = 0 );
 	virtual void setI( int value, long long tm = 0 );
 	virtual void setB( char value, long long tm = 0 );
 
-    protected:
-	//Protected methods
 	void makeBuf( TFld::Type v_tp, int isz, long long ipr, bool hd_grd, bool hg_res );	//Create new or change buffer mode (all data into buffer will lost)
 
     private:
 	//Private data and attributes
 	template <class TpVal> class TBuf
 	{
+	    friend class TValBuf;
 	    public:
 		//Public methods
-		TBuf( TpVal eval, int &isz, long long &ipr, bool &ihgrd, bool &ihres, long long& iend, long long& ibeg );
+		TBuf( TpVal eval, int &isz, long long &ipr, bool &ihgrd, bool &ihres,
+		    long long& iend, long long& ibeg, unsigned int &iEvalCnt );
 		~TBuf( );
 
 		void clear( );
@@ -106,7 +107,7 @@ class TValBuf
 		TpVal get( long long *tm = NULL, bool up_ord = false );
 		void  set( TpVal value, long long tm = 0 );
 
-		//- Create new or change buffer mode (all data into buffer will lost) -
+		//> Create new or change buffer mode (all data into buffer will lost)
 		void makeBuf( int isz, long long ipr, bool hd_grd, bool hg_res );
 
 	    private:
@@ -118,6 +119,7 @@ class TValBuf
 		int	&size,
 			cur;		//Curent position.
 		TpVal	eval;		//Error element value
+		unsigned int &mEvalCnt;
 
 		struct SLw  { time_t tm; TpVal val; };
 		struct SHg  { long long tm; TpVal val; };
@@ -129,8 +131,8 @@ class TValBuf
 		}buf;
 	};
 
-	Res		b_res;		//Access resource
-	TFld::Type	m_val_tp;	//Store values type
+	Res		bRes;		//Access resource
+	TFld::Type	mValTp;		//Store values type
 	union
 	{
 	    TBuf<char>	*bl;
@@ -138,12 +140,13 @@ class TValBuf
 	    TBuf<double>*real;
 	    TBuf<string>*str;
 	} buf;
-	
-	bool	m_hg_res_tm,	//High resolution time use (microseconds)
-		m_hrd_grd;	//Set hard griding. It mode no support the zero periodic.
-	long long m_end, m_beg,	//Time of buffer begin and end (64 bit usec)
-		m_per;		//Periodic grid value (usec). If value = 0 then it no periodic buffer, error for time griding mode!
-	int	m_size;		//Buffer size limit.
+
+	bool	mHgResTm,		//High resolution time use (microseconds)
+		mHrdGrd;		//Set hard griding. It mode no support the zero periodic.
+	long long mEnd, mBeg,		//Time of buffer begin and end (64 bit usec)
+		mPer;			//Periodic grid value (usec). If value = 0 then it no periodic buffer, error for time griding mode!
+	int	mSize;			//Buffer size limit.
+	unsigned int mEvalCnt;
 };
 
 
@@ -166,33 +169,33 @@ class TVArchive : public TCntrNode, public TValBuf, public TConfig
 
 	TCntrNode &operator=( TCntrNode &node );
 
-	//- Base functions -
-	const string &id( )	{ return m_id; }
+	//> Base functions
+	const string &id( )	{ return mId; }
 	string name( );
-	string dscr( )		{ return m_dscr; }
-	SrcMode srcMode( )	{ return (TVArchive::SrcMode)m_srcmode; }
-	string  srcData( )	{ return m_dsourc; }
-	bool toStart( )  	{ return m_start; }
+	string dscr( )		{ return mDscr; }
+	SrcMode srcMode( )	{ return (TVArchive::SrcMode)mSrcMode; }
+	string  srcData( )	{ return mDSourc; }
+	bool toStart( )  	{ return mStart; }
 	bool startStat( )	{ return run_st; }
 
-	string DB( )		{ return m_db; }
+	string DB( )		{ return mDB; }
 	string tbl( );
 	string fullDB( )	{ return DB()+'.'+tbl(); }
 
 	long long end( const string &arch = BUF_ARCH_NM );
 	long long begin( const string &arch = BUF_ARCH_NM );
+	long long period( const string &arch = BUF_ARCH_NM );
 	TFld::Type valType( )	{ return TValBuf::valType(); }
 	bool hardGrid( )	{ return TValBuf::hardGrid(); }
 	bool highResTm( )	{ return TValBuf::highResTm(); }
 	int size( )		{ return TValBuf::size(); }
-	long long period( )	{ return TValBuf::period(); }
 
-	void setName( const string &inm )	{ m_name = inm; modif(); }
-	void setDscr( const string &idscr )	{ m_dscr = idscr; modif(); }
+	void setName( const string &inm )	{ mName = inm; modif(); }
+	void setDscr( const string &idscr )	{ mDscr = idscr; modif(); }
 	void setSrcMode( SrcMode vl, const string &isrc = "" );
-	void setToStart( bool vl )		{ m_start = vl; modif(); }
+	void setToStart( bool vl )		{ mStart = vl; modif(); }
 
-	void setDB( const string &idb )		{ m_db = idb; modifG(); }
+	void setDB( const string &idb )		{ mDB = idb; modifG(); }
 
 	void setValType( TFld::Type vl );
 	void setHardGrid( bool vl );
@@ -200,24 +203,20 @@ class TVArchive : public TCntrNode, public TValBuf, public TConfig
 	void setSize( int vl );
 	void setPeriod( long long vl );
 
-	//- Service -
+	//> Service
 	void start( );
 	void stop( bool full_del = false );
 
-	//- Get value -
-	void getVal( TValBuf &buf, long long beg = 0, long long end = 0,
-		const string &arch = "", int limit = 100000 );
-	string getS( long long *tm = NULL, bool up_ord = false, const string &arch = "" );
-	double getR( long long *tm = NULL, bool up_ord = false, const string &arch = "" );
-	int    getI( long long *tm = NULL, bool up_ord = false, const string &arch = "" );
-	char   getB( long long *tm = NULL, bool up_ord = false, const string &arch = "" );
+	//> Get value
+	TVariant getVal( long long *tm = NULL, bool up_ord = false, const string &arch = "", bool onlyLocal = false );
+	void getVals( TValBuf &buf, long long beg = 0, long long end = 0,
+		const string &arch = "", int limit = 100000, bool onlyLocal = false );
+	void setVals( TValBuf &buf, long long beg, long long end, const string &arch );
 
-	void setVal( TValBuf &buf, long long beg, long long end, const string &arch );
-
-	//- Active get data from atribute -
+	//> Active get data from atribute
 	void getActiveData( );
 
-	//- Phisical archivator's functions -
+	//> Phisical archivator's functions
 	void archivatorList( vector<string> &ls );
 	bool archivatorPresent( const string &arch );
 	void archivatorAttach( const string &arch );
@@ -241,29 +240,29 @@ class TVArchive : public TCntrNode, public TValBuf, public TConfig
     private:
 	//Private methods
 	void setUpBuf( );
-	string nodeName( )	{ return m_id; }
+	string nodeName( )	{ return mId; }
 
 	//Private attributes
 	Res	a_res;
 	bool	run_st;
-	string	m_db;
-	//- Base params -
-	string  &m_id,		//ID
-		&m_name,	//Name
-		&m_dscr,	//Description
-		&m_dsourc,	//Data source (parameter atribute)
-		&m_archs;	//Use archivators list (separated whith ';')
-	bool	&m_start;	//Starting flag
-	int	&m_srcmode;	//Source mode
-	//- Buffer params -
-	int	&m_vtype;	//Value type (int, real, bool, string)
-	double	&m_bper;	//Buffer period
-	int	&m_bsize;	//Buffer size
-	bool	&m_bhgrd,	//Buffer use hard time griding
-		&m_bhres;	//Buffer use high time resolution
-	//- Mode params -
+	string	mDB;
+	//> Base params
+	string  &mId,		//ID
+		&mName,		//Name
+		&mDscr,		//Description
+		&mDSourc,	//Data source (parameter atribute)
+		&mArchs;	//Use archivators list (separated whith ';')
+	bool	&mStart;	//Starting flag
+	int	&mSrcMode;	//Source mode
+	//> Buffer params
+	int	&mVType;	//Value type (int, real, bool, string)
+	double	&mBPer;		//Buffer period
+	int	&mBSize;	//Buffer size
+	bool	&mBHGrd,	//Buffer use hard time griding
+		&mBHRes;	//Buffer use high time resolution
+	//> Mode params
 	AutoHD<TVal>	pattr_src;
-	//- Phisical archive's elements -
+	//> Phisical archive's elements
 	vector<TVArchEl*> arch_el;	//Links
 };
 
@@ -285,34 +284,34 @@ class TVArchivator : public TCntrNode, public TConfig
 
 	TCntrNode &operator=( TCntrNode &node );
 
-	const string &id( )	{ return m_id; }
+	const string &id( )	{ return mId; }
 	string workId( );
 	string name( );
-	string dscr( )		{ return m_dscr; }
-	string addr( )		{ return m_addr; }
-	double valPeriod( )	{ return m_v_per; }
-	int    archPeriod( )	{ return m_a_per; }
+	string dscr( )		{ return mDscr; }
+	string addr( )		{ return mAddr; }
+	double valPeriod( )	{ return mVPer; }
+	int    archPeriod( )	{ return mAPer; }
 
-	bool toStart( )		{ return m_start; }
+	bool toStart( )		{ return mStart; }
 	bool startStat( )	{ return run_st; }
 
-	string DB( )		{ return m_db; }
+	string DB( )		{ return mDB; }
 	string tbl( );
 	string fullDB( )		{ return DB()+'.'+tbl(); }
 
-	void setName( const string &inm )	{ m_name = inm; modif(); }
-	void setDscr( const string &idscr )	{ m_dscr = idscr; modif(); }
-	void setAddr( const string &vl )	{ m_addr = vl; modif(); }
+	void setName( const string &inm )	{ mName = inm; modif(); }
+	void setDscr( const string &idscr )	{ mDscr = idscr; modif(); }
+	void setAddr( const string &vl )	{ mAddr = vl; modif(); }
 	virtual void setValPeriod( double per );
-	virtual void setArchPeriod( int per );
-	void setToStart( bool vl )		{ m_start = vl; modif(); }
+	virtual void setArchPeriod( int per )	{ mAPer = (per?per:1); modif(); }
+	void setToStart( bool vl )		{ mStart = vl; modif(); }
 
-	void setDB( const string &idb )		{ m_db = idb; modif(); }
+	void setDB( const string &idb )		{ mDB = idb; modif(); }
 
 	virtual void start( );
 	virtual void stop( bool full_del = false );
 
-	//- Place archive functions -
+	//> Place archive functions
 	void archiveList( vector<string> &ls );
 	bool archivePresent( const string &iid );
 
@@ -320,7 +319,7 @@ class TVArchivator : public TCntrNode, public TConfig
 
     protected:
 	//Protected methods
-	//- Protected place archive functions -
+	//> Protected place archive functions
 	TVArchEl *archivePlace( TVArchive &item );
 	void archiveRemove( const string &id, bool full = false );
 
@@ -336,31 +335,29 @@ class TVArchivator : public TCntrNode, public TConfig
 
 	//Protected attributes
 	Res	a_res;
-	bool	run_st;
-	//- Phisical elements storing -
+	bool	run_st, endrunReq;
+	//> Phisical elements storing
 	map<string,TVArchEl*> archEl;
 
     private:
 	//Private methods
-	string nodeName( )	{ return m_id; }
-	static void Task( union sigval obj );	//Process task
+	string nodeName( )	{ return mId; }
+	static void *Task( void *param );	//Process task
 
 	//Private attributes
-	string	&m_id,		//Var arch id
-		&m_name,	//Var arch name
-		&m_dscr,	//Var arch description
-		&m_addr;	//Mess arch phisical address
-	bool	&m_start;	//Var arch starting flag
-	double	&m_v_per;	//Value period (sec)
-	int	&m_a_per;	//Archivation period
-	string	m_db;
-	//- Archivate process -
+	string	&mId,		//Var arch id
+		&mName,		//Var arch name
+		&mDscr,		//Var arch description
+		&mAddr;		//Mess arch phisical address
+	bool	&mStart;	//Var arch starting flag
+	double	&mVPer;		//Value period (sec)
+	int	&mAPer;		//Archivation period
+	string	mDB;
+	//> Archivate process
 	double	tm_calc;	//Archiving time
-	bool	prc_st;		//Process stat
-	timer_t	tmId;		//Thread timer
+	pthread_t mPthr;
 };
 
-
 //*************************************************
 //* TVArchEl                                      *
 //*************************************************
@@ -376,30 +373,31 @@ class TVArchEl
 
 	virtual long long end( )	{ return 0; }		//Archive end
 	virtual long long begin( )	{ return 0; }		//Archive begin
-	long long lastGet( )		{ return m_last_get; }	//Last getted value time
+	long long lastGet( )		{ return mLastGet; }	//Last getted value time
 
-	virtual void getVal( TValBuf &buf, long long beg = 0, long long end = 0 ) { }
-	virtual string getS( long long *tm, bool up_ord ) { }
-	virtual double getR( long long *tm, bool up_ord ) { }
-	virtual int    getI( long long *tm, bool up_ord ) { }
-	virtual char   getB( long long *tm, bool up_ord ) { }
-
-	virtual void setVal( TValBuf &buf, long long beg = 0, long long end = 0 ) { }
+	TVariant getVal( long long *tm, bool up_ord, bool onlyLocal = false );
+	void getVals( TValBuf &buf, long long beg = 0, long long end = 0, bool onlyLocal = false );
+	void setVals( TValBuf &buf, long long beg = 0, long long end = 0 );
 
 	TVArchive &archive( );
 	TVArchivator &archivator( );
 
-	//Public atributes
-	//- Previous averaging value -
+    protected:
+	//Protected methods
+	virtual TVariant getValProc( long long *tm, bool up_ord );
+	virtual void getValsProc( TValBuf &buf, long long beg, long long end )	{ }
+	virtual void setValsProc( TValBuf &buf, long long beg, long long end )	{ }
+
+	//> Previous averaging value
 	long long prev_tm;
 	string prev_val;
 
     private:
 	//Private attributes
-	TVArchive	&m_achive;
-	TVArchivator	&m_archivator;
+	TVArchive	&mArchive;
+	TVArchivator	&mArchivator;
 
-	long long m_last_get;
+	long long mLastGet;
 };
 
 #endif // TArchVal_H
