@@ -32,16 +32,16 @@
 //************************************************
 TBDS::TBDS( ) : TSubSYS("BD","Data Bases",true)
 {
-    //- Generic system DB -
+    //> Generic system DB
     fldAdd( new TFld("user","User",TFld::String,TCfg::Key,"20") );
     fldAdd( new TFld("id","Value ID",TFld::String,TCfg::Key,"100") );
-    fldAdd( new TFld("val","Value"  ,TFld::String,TFld::NoFlag,"1000") );
+    fldAdd( new TFld("val","Value"  ,TFld::String,TCfg::TransltText,"1000") );
 
-    //- Open data bases DB structure -
+    //> Open data bases DB structure
     el_db.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"20") );
     el_db.fldAdd( new TFld("TYPE",_("DB type (module)"),TFld::String,TCfg::Key,"20") );
-    el_db.fldAdd( new TFld("NAME",_("Name"),TFld::String,TFld::NoFlag,"50") );
-    el_db.fldAdd( new TFld("DESCR",_("Description"),TFld::String,TFld::NoFlag,"200") );
+    el_db.fldAdd( new TFld("NAME",_("Name"),TFld::String,TCfg::TransltText,"50") );
+    el_db.fldAdd( new TFld("DESCR",_("Description"),TFld::String,TCfg::TransltText,"200") );
     el_db.fldAdd( new TFld("ADDR",_("Address"),TFld::String,TFld::NoFlag,"100") );
     el_db.fldAdd( new TFld("CODEPAGE",_("Code page"),TFld::String,TFld::NoFlag,"20") );
     el_db.fldAdd( new TFld("EN",_("To enable"),TFld::Boolean,TFld::NoFlag,"1","1") );
@@ -49,10 +49,10 @@ TBDS::TBDS( ) : TSubSYS("BD","Data Bases",true)
 
 TBDS::~TBDS(  )
 {
-    ResAlloc res(genDBCacheRes,true);
-    while(genDBCache.size())
+    ResAlloc res(nodeRes(),true);
+    while( genDBCache.size() )
     {
-	delete genDBCache[0];
+	delete genDBCache.front();
 	genDBCache.pop_front();
     }
     res.release();
@@ -68,12 +68,17 @@ string TBDS::realDBName( const string &bdn )
 	   ((bd_n=="*") ? TSYS::strSepParse(SYS->workDB(),1,'.') : bd_n)+(bd_tbl.empty() ? "" : "."+bd_tbl);
 }
 
-void TBDS::dbList( vector<string> &ls )
+void TBDS::dbList( vector<string> &ls, bool checkSel )
 {
-    vector<string> tdb_ls, db_ls;
-
     ls.clear();
 
+    if( checkSel && !SYS->selDB().empty() )
+    {
+	if( SYS->selDB() != "<cfg>" ) ls.push_back(SYS->selDB());
+	return;
+    }
+
+    vector<string> tdb_ls, db_ls;
     modList(tdb_ls);
     for( int i_tp = 0; i_tp < tdb_ls.size(); i_tp++ )
     {
@@ -83,6 +88,34 @@ void TBDS::dbList( vector<string> &ls )
     }
 }
 
+void TBDS::closeOldTables( int secOld )
+{
+    vector<string> tdbs, dbs, tbls;
+
+    try
+    {
+	modList(tdbs);
+	for( int i_tdb = 0; i_tdb < tdbs.size(); i_tdb++ )
+	{
+	    at(tdbs[i_tdb]).at().list(dbs);
+	    for( int i_db = 0; i_db < dbs.size(); i_db++ )
+	    {
+		AutoHD<TBD> db = at(tdbs[i_tdb]).at().at(dbs[i_db]);
+		db.at().list(tbls);
+		for( int i_tbl = 0; i_tbl < tbls.size(); i_tbl++ )
+		{
+		    AutoHD<TTable> tbl = db.at().at(tbls[i_tbl]);
+		    if( (time(NULL)-tbl.at().lstUse()) > secOld )
+		    {
+			tbl.free();
+			db.at().close(tbls[i_tbl]);
+		    }
+		}
+	    }
+	}
+    }catch(...){  }
+}
+
 AutoHD<TTable> TBDS::open( const string &bdn, bool create )
 {
     bool bd_op = false;
@@ -148,21 +181,22 @@ bool TBDS::dataSeek( const string &bdn, const string &path, int lev, TConfig &cf
     {
 	try
 	{
+	    ResAlloc res(SYS->nodeRes(),false);
 	    XMLNode *nd = ctrId(&SYS->cfgRoot(),path);
 	    for( int i_fld = 0; i_fld < nd->childSize(); i_fld++ )
 	    {
 		XMLNode *el = nd->childGet(i_fld);
 		if( el->name() == "fld" )
 		{
-		    //- Check keywords -
+		    //> Check keywords
 		    vector<string> cf_el;
 		    cfg.cfgList(cf_el);
 		
-		    //- Check keywords -
+		    //> Check keywords
 		    int i_el;
 		    for( i_el = 0; i_el < cf_el.size(); i_el++ )
 			if( cfg.cfg(cf_el[i_el]).fld().flg()&TCfg::Key &&
-			    cfg.cfg(cf_el[i_el]).getS().size() && 
+			    cfg.cfg(cf_el[i_el]).keyUse() &&
 			    cfg.cfg(cf_el[i_el]).getS() != el->attr(cf_el[i_el]) ) break;
 		    if( i_el == cf_el.size() && lev <= c_lev++ )
 		    {
@@ -182,8 +216,7 @@ bool TBDS::dataSeek( const string &bdn, const string &path, int lev, TConfig &cf
 	if( !tbl.freeStat() )
 	{
 	    bool rez = tbl.at().fieldSeek(lev-c_lev,cfg);
-	    tbl.free();
-	    close(bdn);
+	    //tbl.free(); close(bdn);
 	    return rez;
 	}
     }
@@ -195,7 +228,7 @@ bool TBDS::dataGet( const string &bdn, const string &path, TConfig &cfg )
 {
     AutoHD<TTable> tbl = open(bdn);
 
-    //- Load from DB -
+    //> Load from DB
     if( !tbl.freeStat() )
     {
 	bool db_true = true;
@@ -206,16 +239,15 @@ bool TBDS::dataGet( const string &bdn, const string &path, TConfig &cfg )
 		mess_warning(err.cat.c_str(),"%s",err.mess.c_str());
 	    db_true = false;
 	}
-	tbl.free();
-	close(bdn);
 	if(db_true) return true;
     }
-    //- Load from Config file if tbl no present -
+    //> Load from Config file if tbl no present
     XMLNode *nd;
+    ResAlloc res(SYS->nodeRes(),false);
     try{ nd = ctrId(&SYS->cfgRoot(),path); }
     catch(...){ return false; }
 
-    //-- Scan fields and fill Config --
+    //>> Scan fields and fill Config
     for( int i_fld = 0; i_fld < nd->childSize(); i_fld++ )
     {
 	XMLNode *el = nd->childGet(i_fld);
@@ -227,12 +259,21 @@ bool TBDS::dataGet( const string &bdn, const string &path, TConfig &cfg )
 
 	    int i_el;
 	    for( i_el = 0; i_el < cf_el.size(); i_el++ )
-		if( cfg.cfg(cf_el[i_el]).fld().flg()&TCfg::Key && 
+		if( cfg.cfg(cf_el[i_el]).fld().flg()&TCfg::Key &&
 		    cfg.cfg(cf_el[i_el]).getS() != el->attr(cf_el[i_el]) ) break;
 	    if( i_el == cf_el.size() )
 	    {
 		for( int i_el = 0; i_el < cf_el.size(); i_el++ )
-		    cfg.cfg(cf_el[i_el]).setS(el->attr(cf_el[i_el]));
+		{
+		    TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
+		    u_cfg.setS(el->attr(cf_el[i_el]));
+		    if( u_cfg.fld().flg()&TCfg::TransltText && !Mess->lang2CodeBase().empty() &&
+			((!cfg.lang2Code().empty() && cfg.lang2Code() != Mess->lang2CodeBase()) || (cfg.lang2Code().empty() && Mess->lang2Code() != Mess->lang2CodeBase())) )
+		    {
+			string vl = el->attr(cf_el[i_el]+"_"+(cfg.lang2Code().empty()?Mess->lang2Code():cfg.lang2Code()));
+			if( !vl.empty() ) u_cfg.setS(vl);
+		    }
+		}
 		return true;
 	    }
 	}
@@ -250,33 +291,48 @@ bool TBDS::dataSet( const string &bdn, const string &path, TConfig &cfg )
 	bool db_true = true;
 	try{ tbl.at().fieldSet(cfg); }
 	catch(TError err) { mess_warning(err.cat.c_str(),"%s",err.mess.c_str()); db_true = false; }
-	tbl.free();
-	try{ close(bdn); } 
-	catch(TError err) { mess_warning(err.cat.c_str(),"%s",err.mess.c_str()); }
 	return db_true;
     }
     //throw TError("BD","Field no present.");
     return false;
 }
 
-bool TBDS::dataDel( const string &bdn, const string &path, TConfig &cfg )
+bool TBDS::dataDel( const string &bdn, const string &path, TConfig &cfg, bool useKeyAll )
 {
+    vector<string> cels;
     AutoHD<TTable> tbl = open(bdn);
 
     if( !tbl.freeStat() )
     {
 	bool db_true = true;
-	try{ tbl.at().fieldDel(cfg); }
+	try
+	{
+	    //> Select for using all keys
+	    if( useKeyAll )
+	    {
+		cfg.cfgList(cels);
+		for( int i_el = 0; i_el < cels.size(); i_el++ )
+		    if( (cfg.cfg(cels[i_el]).fld().flg()&TCfg::Key) && !cfg.cfg(cels[i_el]).keyUse() )
+			cfg.cfg(cels[i_el]).setKeyUse(true);
+		    else { cels.erase(cels.begin()+i_el); i_el--; }
+	    }
+	
+	    tbl.at().fieldDel(cfg);
+
+	    //> Restore not using keys selection
+	    if( useKeyAll )
+		for( int i_el = 0; i_el < cels.size(); i_el++ )
+		    cfg.cfg(cels[i_el]).setKeyUse(false);
+	}
 	catch(TError err) { mess_warning(err.cat.c_str(),"%s",err.mess.c_str()); db_true = false; }
-	tbl.free();
-	close(bdn);
+	//tbl.free(); close(bdn);
 	return db_true;
     }
 
     return false;
 }
 
-void TBDS::genDBSet(const string &path, const string &val, const string &user)
+void TBDS::genDBSet(const string &path, const string &val, const string &user, char rFlg )
 {
     string rez;
 
@@ -288,97 +344,83 @@ void TBDS::genDBSet(const string &path, const string &val, const string &user)
 	if( !tbl.freeStat() )
 	{
 	    TConfig db_el(&dbs.at());
+	    if( !(rFlg&TBDS::UseTranslate) ) db_el.setLang2Code(Mess->lang2CodeBase());
 	    db_el.cfg("user").setS(user);
 	    db_el.cfg("id").setS(path);
 	    db_el.cfg("val").setS(val);
-	
+
 	    try{ tbl.at().fieldSet(db_el); }
 	    catch(TError err){ }
-	
-	    tbl.free();
-	    dbs.at().close(dbs.at().fullDBSYS());
 	}
-
-	//Put to cache
-	ResAlloc res(dbs.at().genDBCacheRes,true);
-	for( int i_cel = 0; i_cel < dbs.at().genDBCache.size(); i_cel++ )
-	    if( dbs.at().genDBCache[i_cel]->cfg("user").getS() == user &&
-		dbs.at().genDBCache[i_cel]->cfg("id").getS() == path )
-	    {
-		dbs.at().genDBCache[i_cel]->cfg("val").setS(val);
-		return;
-	    }
-	TConfig *db_el = new TConfig(&dbs.at());
-	db_el->cfg("user").setS(user);
-	db_el->cfg("id").setS(path);
-	db_el->cfg("val").setS(val);
-	dbs.at().genDBCache.push_front(db_el);
-	if(dbs.at().genDBCache.size() > 100)
+	else
 	{
-	    delete dbs.at().genDBCache[0];
-	    dbs.at().genDBCache.pop_back();
+	    //Put to cache
+	    ResAlloc res(dbs.at().nodeRes(),true);
+	    for( int i_cel = 0; i_cel < dbs.at().genDBCache.size(); i_cel++ )
+		if( dbs.at().genDBCache[i_cel]->cfg("user").getS() == user &&
+		    dbs.at().genDBCache[i_cel]->cfg("id").getS() == path )
+		{ dbs.at().genDBCache[i_cel]->cfg("val").setS(val); return; }
+	    TConfig *db_el = new TConfig(&dbs.at());
+	    db_el->cfg("user").setS(user);
+	    db_el->cfg("id").setS(path);
+	    db_el->cfg("val").setS(val);
+	    dbs.at().genDBCache.push_front(db_el);
+	    while( dbs.at().genDBCache.size() > 100 )
+	    {
+		delete dbs.at().genDBCache.back();
+		dbs.at().genDBCache.pop_back();
+	    }
 	}
     }
 }
 
-string TBDS::genDBGet(const string &path, const string &oval, const string &user, bool onlyCfg )
+string TBDS::genDBGet(const string &path, const string &oval, const string &user, char rFlg )
 {
     bool bd_ok=false;
     string rez = oval;
 
-    if( SYS->present("BD") )
+    //> Get from generic DB
+    if( SYS->present("BD") && !(rFlg&TBDS::OnlyCfg) )
     {
 	AutoHD<TBDS> dbs = SYS->db();
-
-	//- Get from cache -
-	ResAlloc res(dbs.at().genDBCacheRes,false);
-	for( int i_cel = 0; i_cel < dbs.at().genDBCache.size(); i_cel++ )
-	    if( dbs.at().genDBCache[i_cel]->cfg("user").getS() == user &&
-			dbs.at().genDBCache[i_cel]->cfg("id").getS() == path )
-		return dbs.at().genDBCache[i_cel]->cfg("val").getS();
-	res.release();
-
-	//- Get from generic DB -
-	if(!onlyCfg)
+	AutoHD<TTable> tbl = dbs.at().open(SYS->db().at().fullDBSYS());
+	if( !tbl.freeStat() )
 	{
-	    AutoHD<TTable> tbl = dbs.at().open(SYS->db().at().fullDBSYS());
-	    if( !tbl.freeStat() )
+	    TConfig db_el(&dbs.at());
+	    if( !(rFlg&TBDS::UseTranslate) ) db_el.setLang2Code(Mess->lang2CodeBase());
+	    db_el.cfg("user").setS(user);
+	    db_el.cfg("id").setS(path);
+	    try
 	    {
-		TConfig db_el(&dbs.at());
-		db_el.cfg("user").setS(user);
-		db_el.cfg("id").setS(path);
-		try
-		{
-		    tbl.at().fieldGet(db_el);
-		    rez = db_el.cfg("val").getS();
-		    bd_ok = true;
-		}
-		catch(TError err){  }
-
-		tbl.free();
-		dbs.at().close(dbs.at().fullDBSYS());
+		tbl.at().fieldGet(db_el);
+		rez = db_el.cfg("val").getS();
+		bd_ok = true;
 	    }
+	    catch(TError err){  }
 	}
     }
-    //- Get from config file -
-    if(!bd_ok)
-	try{ rez = SYS->ctrId(&SYS->cfgRoot(),path)->text(); }
-	catch(...){ }
 
-    //Put to cache
-    if( SYS->present("BD") )
+    if(!bd_ok)
     {
-	AutoHD<TBDS> dbs = SYS->db();
-	ResAlloc res(dbs.at().genDBCacheRes,true);
-	TConfig *db_el = new TConfig(&dbs.at());
-	db_el->cfg("user").setS(user);
-	db_el->cfg("id").setS(path);
-	db_el->cfg("val").setS(rez);
-	dbs.at().genDBCache.push_front(db_el);
-	if(dbs.at().genDBCache.size() > 100)
+	//> Get from cache
+	if( SYS->present("BD") )
 	{
-	    delete dbs.at().genDBCache[0];
-	    dbs.at().genDBCache.pop_back();
+	    AutoHD<TBDS> dbs = SYS->db();
+	    ResAlloc res(dbs.at().nodeRes(),false);
+	    for( int i_cel = 0; i_cel < dbs.at().genDBCache.size(); i_cel++ )
+		if( dbs.at().genDBCache[i_cel]->cfg("user").getS() == user &&
+			dbs.at().genDBCache[i_cel]->cfg("id").getS() == path )
+		    return dbs.at().genDBCache[i_cel]->cfg("val").getS();
+	}
+
+	//> Get from config file
+	ResAlloc res(SYS->nodeRes(),false);
+	XMLNode *tgtN = TCntrNode::ctrId(&SYS->cfgRoot(),path,true);
+	if( tgtN ) rez = tgtN->text();
+	if( rFlg&TBDS::UseTranslate && !Mess->lang2CodeBase().empty() && Mess->lang2Code() != Mess->lang2CodeBase() )
+	{
+	    tgtN = TCntrNode::ctrId(&SYS->cfgRoot(),path+"_"+Mess->lang2Code(),true);
+	    if( tgtN ) rez = tgtN->text();
 	}
     }
 
@@ -398,7 +440,7 @@ string TBDS::optDescr(  )
 
 void TBDS::load_( )
 {
-    //- Load parameters from command line -
+    //> Load parameters from command line
     int next_opt;
     const char *short_opt="h";
     struct option long_opt[] =
@@ -413,15 +455,15 @@ void TBDS::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
 
-    //- Load parameters from config file -
+    //> Load parameters from config file
 
-    //- Open DB load -
-    //Load and enable other DB store DB
+    //> Open DB load
+    //>> Load and enable other DB stored into DB
     string db_tp = TSYS::strSepParse(fullDB(),0,'.');
     string db_nm = TSYS::strSepParse(fullDB(),1,'.');
     if( !at(db_tp).at().openStat(db_nm) )
@@ -432,20 +474,20 @@ void TBDS::load_( )
     }
 
     string id,type;
-    //- Search and open new DB -
+    //> Search and open new DB
     try
     {
-	TConfig c_el(&el_db);
-	c_el.cfgViewAll(false);
-	int fld_cnt = 0;
-	while( SYS->db().at().dataSeek(fullDB(),nodePath()+"DB/",fld_cnt++,c_el) )
+	if( SYS->chkSelDB(fullDB()) )
 	{
-	    id = c_el.cfg("ID").getS();
-	    type = c_el.cfg("TYPE").getS();
-	    if( (type+"."+id) != SYS->workDB() && modPresent(type) && !at(type).at().openStat(id) )
-		at(type).at().open(id);
-	    c_el.cfg("ID").setS("");
-	    c_el.cfg("TYPE").setS("");
+	    TConfig c_el(&el_db);
+	    c_el.cfgViewAll(false);
+	    for( int fld_cnt = 0; SYS->db().at().dataSeek(fullDB(),nodePath()+"DB/",fld_cnt++,c_el); )
+	    {
+		id = c_el.cfg("ID").getS();
+		type = c_el.cfg("TYPE").getS();
+		if( (type+"."+id) != SYS->workDB() && modPresent(type) && !at(type).at().openStat(id) )
+		    at(type).at().open(id);
+	    }
 	}
     }catch( TError err )
     {
@@ -488,18 +530,20 @@ void TTipBD::open( const string &iid )
     chldAdd(m_db,openBD(iid));
 }
 
+TBDS &TTipBD::owner( )	{ return (TBDS&)TModule::owner(); }
+
 void TTipBD::cntrCmdProc( XMLNode *opt )
 {
     //Get page info
     if( opt->name() == "info" )
     {
 	TModule::cntrCmdProc(opt);
-	ctrMkNode("grp",opt,-1,"/br/db_",_("DB"),0664,"root","BD",1,"idm","1");
+	ctrMkNode("grp",opt,-1,"/br/db_",_("DB"),0664,"root","BD",2,"idm","1","idSz","20");
 	if(ctrMkNode("area",opt,0,"/db",_("DB"),0444))
 	{
 	    ctrMkNode("fld",opt,-1,"/db/ful_db_del",_("Full DB delete"),0660,"root","BD",2,
-		"tp","bool","help",_("Select for full deletion DB on DB close. Else DB will be simple closed."));
-	    ctrMkNode("list",opt,-1,"/db/odb",_("DB"),0664,"root","BD",4,"tp","br","idm","1","s_com","add,del","br_pref","db_");
+		"tp","bool","help",_("Select for full deletion DB on DB close. Else DB will be simply closed."));
+	    ctrMkNode("list",opt,-1,"/db/odb",_("DB"),0664,"root","BD",5,"tp","br","idm","1","s_com","add,del","br_pref","db_","idSz","20");
 	}
 	return;
     }
@@ -522,8 +566,8 @@ void TTipBD::cntrCmdProc( XMLNode *opt )
 	}
 	if( ctrChkNode(opt,"add",0664,"root","BD",SEQ_WR) )
 	{
-	    open(opt->attr("id"));
-	    at(opt->attr("id")).at().setName(opt->text());
+	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::ID);
+	    open(vid); at(vid).at().setName(opt->text());
 	}
 	if( ctrChkNode(opt,"del",0664,"root","BD",SEQ_WR) )	close(opt->attr("id"),true);
     }
@@ -534,11 +578,11 @@ void TTipBD::cntrCmdProc( XMLNode *opt )
 //* TBD                                          *
 //************************************************
 TBD::TBD( const string &iid, TElem *cf_el ) : TConfig( cf_el ),
-    m_id(cfg("ID").getSd()), m_name(cfg("NAME").getSd()), m_dscr(cfg("DESCR").getSd()),
-    m_addr(cfg("ADDR").getSd()), m_codepage(cfg("CODEPAGE").getSd()), m_toen(cfg("EN").getBd()), m_en(false)
+    mId(cfg("ID").getSd()), mName(cfg("NAME").getSd()), mDscr(cfg("DESCR").getSd()),
+    mAddr(cfg("ADDR").getSd()), mCodepage(cfg("CODEPAGE").getSd()), mToEn(cfg("EN").getBd()), mEn(false)
 {
-    m_id = iid;
-    m_tbl = grpAdd("tbl_");
+    mId = iid;
+    mTbl = grpAdd("tbl_");
 }
 
 TCntrNode &TBD::operator=( TCntrNode &node )
@@ -551,7 +595,7 @@ TCntrNode &TBD::operator=( TCntrNode &node )
 	string tid = id();
 	*(TConfig*)this = *(TConfig*)src_n;
 	cfg("TYPE").setS(owner().modId());
-	m_id = tid;
+	mId = tid;
     }
 
     if( src_n->enableStat() && enableStat() )
@@ -594,21 +638,20 @@ void TBD::postDisable(int flag)
     try
     {
 	if( flag )
-	    SYS->db().at().dataDel(owner().owner().fullDB(),SYS->db().at().nodePath()+"DB/",*this);
+	    SYS->db().at().dataDel(owner().owner().fullDB(),SYS->db().at().nodePath()+"DB/",*this,true);
     }catch(TError err)
     { mess_warning(err.cat.c_str(),"%s",err.mess.c_str()); }
 }
 
-string TBD::name()
-{
-    return (m_name.size())?m_name:id();
-}
+TTipBD &TBD::owner( )	{ return *(TTipBD*)nodePrev(); }
+
+string TBD::name( )	{ return (mName.size())?mName:id(); }
 
 void TBD::enable( )
 {
     if( enableStat() ) return;
 
-    m_en = true;
+    mEn = true;
 }
 
 void TBD::disable( )
@@ -620,17 +663,18 @@ void TBD::disable( )
     for(int i_l = 0; i_l < t_list.size(); i_l++)
 	close(t_list[i_l]);
 
-    m_en = false;
+    mEn = false;
 }
 
 void TBD::open( const string &table, bool create )
 {
-    if( chldPresent(m_tbl,table) ) return;
-    chldAdd(m_tbl,openTable(table, create));
+    if( chldPresent(mTbl,table) ) return;
+    chldAdd(mTbl,openTable(table, create));
 }
 
 void TBD::load_( )
 {
+    if( !SYS->chkSelDB(SYS->workDB()) ) return;
     SYS->db().at().dataGet(owner().owner().fullDB(),SYS->db().at().nodePath()+"DB/",*this);
     if( !enableStat() && toEnable() )	enable();
 }
@@ -645,6 +689,7 @@ void TBD::cntrCmdProc( XMLNode *opt )
     //Get page info
     if( opt->name() == "info" )
     {
+	TCntrNode::cntrCmdProc(opt);
 	ctrMkNode("oscada_cntr",opt,-1,"/",_("Data base: ")+name(),0664,"root","BD");
 	ctrMkNode("branches",opt,-1,"/br","",0444);
 	ctrMkNode("grp",opt,-1,"/br/tbl_",_("Opened table"),0664,"root","BD");
@@ -653,14 +698,15 @@ void TBD::cntrCmdProc( XMLNode *opt )
 	    if(ctrMkNode("area",opt,-1,"/prm/st",_("State")))
 	    {
 		ctrMkNode("fld",opt,-1,"/prm/st/st",_("Enable"),0664,"root","BD",1,"tp","bool");
-		ctrMkNode("list",opt,-1,"/prm/st/allow_tbls",_("Allow tables"),0664,"root","BD",3,
-		    "tp","str","s_com","del","help",_("Tables contained into DB, no opened tables."));
+		ctrMkNode("list",opt,-1,"/prm/st/allow_tbls",_("Accessible tables"),0664,"root","BD",3,
+		    "tp","str","s_com","del","help",_("Tables which are in the DB, tables which are not opened at that moment."));
+		ctrMkNode("comm",opt,-1,"/prm/st/load",_("Load system from this DB"),0660,"root","root");
 	    }
 	    if(ctrMkNode("area",opt,-1,"/prm/cfg",_("Config")))
 	    {
 		ctrMkNode("fld",opt,-1,"/prm/cfg/id",cfg("ID").fld().descr(),0444,"root","BD",1,"tp","str");
-		ctrMkNode("fld",opt,-1,"/prm/cfg/nm",cfg("NAME").fld().descr(),0664,"root","BD",1,"tp","str");
-		ctrMkNode("fld",opt,-1,"/prm/cfg/dscr",cfg("DESCR").fld().descr(),0664,"root","BD",3,"tp","str","cols","50","rows","3");
+		ctrMkNode("fld",opt,-1,"/prm/cfg/nm",cfg("NAME").fld().descr(),0664,"root","BD",2,"tp","str","len","50");
+		ctrMkNode("fld",opt,-1,"/prm/cfg/dscr",cfg("DESCR").fld().descr(),0664,"root","BD",3,"tp","str","cols","100","rows","3");
 		ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),0664,"root","BD",1,"tp","str");
 		ctrMkNode("fld",opt,-1,"/prm/cfg/codep",cfg("CODEPAGE").fld().descr(),0664,"root","BD",2,
 		    "tp","str","help",_("Codepage of data into DB. For example: UTF-8, KOI8-R, KOI8-U ... ."));
@@ -669,7 +715,7 @@ void TBD::cntrCmdProc( XMLNode *opt )
 	}
 	if(ctrMkNode("area",opt,1,"/tbls",_("Tables"),0444))
 	    ctrMkNode("list",opt,-1,"/tbls/otbl",_("Opened tables"),0664,"root","BD",4,
-		"tp","br","s_com","add,del","br_pref","tbl_","help",_("Opened table list.\nAdd and delete table operations is real open and close tables operations."));
+		"tp","br","s_com","add,del","br_pref","tbl_","help",_("Opened table list.\nAdding and deleting tables operations are really open and close tables operations."));
 	return;
     }
 
@@ -695,6 +741,13 @@ void TBD::cntrCmdProc( XMLNode *opt )
 	    close( opt->text(), true );
 	}
     }
+    else if( a_path == "/prm/st/load" && ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )
+    {
+	SYS->setSelDB(owner().modId()+"."+id());
+	SYS->modifG();
+	SYS->load();
+	SYS->setSelDB("");
+    }
     else if( a_path == "/prm/cfg/id" && ctrChkNode(opt) )		opt->setText(id());
     else if( a_path == "/prm/cfg/nm" )
     {
@@ -739,9 +792,10 @@ void TBD::cntrCmdProc( XMLNode *opt )
 //************************************************
 //* TTable                                       *
 //************************************************
-TTable::TTable( const string &name ) :  m_name(name)
+TTable::TTable( const string &name ) :  mName(name)
 {
     modifClr();
+    mLstUse = time(NULL);
 }
 
 TTable::~TTable()
@@ -754,27 +808,23 @@ TCntrNode &TTable::operator=( TCntrNode &node )
     TTable *src_n = dynamic_cast<TTable*>(&node);
     if( !src_n || !src_n->owner().enableStat() || !owner().enableStat() ) return *this;
 
-    //- Table content copy -
+    //> Table content copy
     TConfig req;
     src_n->fieldStruct(req);
-    //-- Scan source table and write to destination table
-    for( int row = 0; src_n->fieldSeek(row,req); row++ )
-    {
-	fieldSet(req);
-	//--- Clear key fields ---
-	for( int i_e = 0; i_e < req.elem().fldSize(); i_e++ )
-	    if( req.elem().fldAt(i_e).flg()&TCfg::Key )
-		req.cfg(req.elem().fldAt(i_e).name()).setS("");
-    }
+    //>> Scan source table and write to destination table
+    for( int row = 0; src_n->fieldSeek(row,req); row++ ) fieldSet(req);
 
     return *this;
 }
 
+TBD &TTable::owner( )	{ return *(TBD*)nodePrev(); }
+
 void TTable::cntrCmdProc( XMLNode *opt )
 {
     //Get page info
     if( opt->name() == "info" )
     {
+	TCntrNode::cntrCmdProc(opt);
 	ctrMkNode("oscada_cntr",opt,-1,"/",_("Table: ")+name(),0664,"root","BD");
 	if(ctrMkNode("area",opt,0,"/prm",_("Table")))
 	{
@@ -815,8 +865,6 @@ void TTable::cntrCmdProc( XMLNode *opt )
 		    if( i_r == 0 )	//Prepare columns
 			ctrMkNode("list",opt,-1,("/prm/tbl/"+eid).c_str(),"",0664);
 		    opt->childGet(i_f)->childAdd("el")->setText(req.cfg(eid).getS());
-		    if( req.elem().fldAt(i_f).flg()&TCfg::Key )
-			req.cfg(eid).setS("");
 		}
 	if( ctrChkNode(opt,"add",0664,"root","BD",SEQ_WR) )
 	{
@@ -832,25 +880,25 @@ void TTable::cntrCmdProc( XMLNode *opt )
 	if( ctrChkNode(opt,"del",0664,"root","BD",SEQ_WR) )
 	{
 	    for( int i_f = 0; i_f < req.elem().fldSize(); i_f++ )
-	    {
-	        eid = req.elem().fldAt(i_f).name();
-	        if( !(req.elem().fldAt(i_f).flg()&TCfg::Key) ) continue;
-	        req.cfg(eid).setS(opt->attr("key_"+eid));
-	    }
+		if( req.elem().fldAt(i_f).flg()&TCfg::Key )
+		{
+		    eid = req.elem().fldAt(i_f).name();
+		    req.cfg(eid).setS(opt->attr("key_"+eid),true);
+		}
 	    fieldDel(req);
 	}
 	if( ctrChkNode(opt,"set",0664,"root","BD",SEQ_WR) )
 	{
-	    //- Get structure -
+	    //> Get structure
 	    string  col = opt->attr("col");
 	    bool key_chng = false;
 	    for( int i_f = 0; i_f < req.elem().fldSize(); i_f++ )
-	    {
-	        eid = req.elem().fldAt(i_f).name();
-	        if( !(req.elem().fldAt(i_f).flg()&TCfg::Key) ) continue;
-	        req.cfg(eid).setS(opt->attr("key_"+eid));
-	        if( eid == col ) key_chng = true;
-	    }
+		if( req.elem().fldAt(i_f).flg()&TCfg::Key )
+		{
+		    eid = req.elem().fldAt(i_f).name();
+		    req.cfg(eid).setS(opt->attr("key_"+eid),true);
+		    if( eid == col ) key_chng = true;
+		}
 	    if( key_chng )
 	    {
 	        fieldGet(req);
