@@ -22,7 +22,7 @@
 #ifndef TBDS_H
 #define TBDS_H
 
-#define  VER_BD 1    //BDS type modules version
+#define  VER_BD 2    //BDS type modules version
 
 #include <stdio.h>
 
@@ -52,7 +52,8 @@ class TTable : public TCntrNode
 
 	TCntrNode &operator=( TCntrNode &node );
 
-	string &name( )		{ return m_name; }
+	string &name( )		{ return mName; }
+	time_t lstUse( )	{ return mLstUse; }
 
 	virtual void fieldStruct( TConfig &cfg )
 	{ throw TError(nodePath().c_str(),_("Function <%s> no support!"),"fieldStruct"); }
@@ -65,18 +66,21 @@ class TTable : public TCntrNode
 	virtual void fieldDel( TConfig &cfg )
 	{ throw TError(nodePath().c_str(),_("Function <%s> no support!"),"fieldDel"); }
 
-	TBD &owner( )		{ return *(TBD *)nodePrev(); }
+	TBD &owner( );
 
     protected:
 	//Protected methods
 	void cntrCmdProc( XMLNode *opt );       //Control interface command process
 
+	//Protected attributes
+	time_t	mLstUse;
+
     private:
 	//Private methods
-	string nodeName( )	{ return m_name; }
+	string nodeName( )	{ return mName; }
 
 	//Private attributes
-	string m_name;
+	string	mName;
 };
 
 //************************************************
@@ -93,38 +97,38 @@ class TBD : public TCntrNode, public TConfig
 
 	TCntrNode &operator=( TCntrNode &node );
 
-	const string	&id( )		{ return m_id; }
+	const string	&id( )		{ return mId; }
 	string		name( );
-	const string	&dscr( )	{ return m_dscr; }
-	const string	&addr( )	{ return m_addr; }
-	const string	&codePage( )	{ return m_codepage; }
+	const string	&dscr( )	{ return mDscr; }
+	const string	&addr( )	{ return mAddr; }
+	const string	&codePage( )	{ return mCodepage; }
 
-	bool enableStat( )		{ return m_en; }
-	bool toEnable( )		{ return m_toen; }
+	bool enableStat( )		{ return mEn; }
+	bool toEnable( )		{ return mToEn; }
 
-	void setName( const string &inm )	{ m_name = inm; modif(); }
-	void setDscr( const string &idscr )	{ m_dscr = idscr; modif(); }
-	void setAddr( const string &iaddr )	{ m_addr = iaddr; modif(); }
-	void setCodePage( const string &icp )	{ m_codepage = icp; modif(); }
-	void setToEnable( bool ivl )		{ m_toen = ivl; modif(); }
+	void setName( const string &inm )	{ mName = inm; modif(); }
+	void setDscr( const string &idscr )	{ mDscr = idscr; modif(); }
+	void setAddr( const string &iaddr )	{ mAddr = iaddr; modif(); }
+	void setCodePage( const string &icp )	{ mCodepage = icp; modif(); }
+	void setToEnable( bool ivl )		{ mToEn = ivl; modif(); }
 
 	virtual void enable( );
 	virtual void disable( );
 
-	//- Opened DB tables -
+	//> Opened DB tables
 	virtual void allowList( vector<string> &list )
 	{ throw TError(nodePath().c_str(),_("Function <%s> no support!"),"allowList"); }
-	void list( vector<string> &list )	{ chldList(m_tbl,list); }
-	bool openStat( const string &table )	{ return chldPresent(m_tbl,table); }
+	void list( vector<string> &list )	{ chldList(mTbl,list); }
+	bool openStat( const string &table )	{ return chldPresent(mTbl,table); }
 	void open( const string &table, bool create );
-	void close( const string &table, bool del = false )	{ chldDel(m_tbl,table,-1,del); }
-	AutoHD<TTable> at( const string &name )	{ return chldAt(m_tbl,name); }
+	void close( const string &table, bool del = false )	{ chldDel(mTbl,table,-1,del); }
+	AutoHD<TTable> at( const string &name )	{ return chldAt(mTbl,name); }
 
-	//- SQL request interface -
+	//> SQL request interface
 	virtual void sqlReq( const string &req, vector< vector<string> > *tbl = NULL )
 	{ throw TError(nodePath().c_str(),_("Function <%s> no support!"),"sqlReq"); }
 
-	TTipBD &owner( )		{ return *(TTipBD *)nodePrev(); }
+	TTipBD &owner( );
 
     protected:
 	//Protected methods
@@ -142,20 +146,20 @@ class TBD : public TCntrNode, public TConfig
     private:
 	//Private methods
 	void postEnable( int flag );
-	string nodeName( )	{ return m_id; }
+	string nodeName( )	{ return mId; }
 
 	//Private attributes
-	//- Base options -
-	string	&m_id,		//ID
-		&m_name,	//Name
-		&m_dscr,	//Description
-		&m_addr,	//Individual address
-		&m_codepage;	//DB codepage
-	bool	&m_toen;
+	//> Base options
+	string	&mId,		//ID
+		&mName,		//Name
+		&mDscr,		//Description
+		&mAddr,		//Individual address
+		&mCodepage;	//DB codepage
+	bool	&mToEn;
 
-	bool	m_en;
-	//- Special options -
-	int	m_tbl;
+	bool	mEn;
+	//> Special options
+	int	mTbl;
 };
 
 //************************************************
@@ -172,14 +176,14 @@ class TTipBD : public TModule
 
 	bool fullDeleteDB( )	{ return full_db_del; }
 
-	//- Opened DB -
+	//> Opened DB
 	void list( vector<string> &list )	{ chldList(m_db,list); }
 	bool openStat( const string &idb )	{ return chldPresent(m_db,idb); }
 	void open( const string &iid );
 	void close( const string &iid, bool erase = false )	{ chldDel(m_db,iid,-1,erase); }
 	AutoHD<TBD> at( const string &name )	{ return chldAt(m_db,name); }
 
-	TBDS &owner( )		{ return (TBDS&)TModule::owner(); }
+	TBDS &owner( );
 
     private:
 	//Private methods
@@ -195,12 +199,19 @@ class TTipBD : public TModule
 
 //************************************************
 //* TBDS                                         *
-//************************************************ 
+//************************************************
 class TSYS;
 
 class TBDS : public TSubSYS, public TElem
 {
     public:
+	//Data
+	enum ReqGen
+	{
+	    OnlyCfg	= 0x01,		//Only from cinfig request
+	    UseTranslate= 0x02		//Use translation for request
+	};
+
 	//Public methods
 	TBDS( );
 	~TBDS( );
@@ -208,22 +219,23 @@ class TBDS : public TSubSYS, public TElem
 	int subVer( )		{ return VER_BD; }
 
 	static string realDBName( const string &bdn );
-	void dbList( vector<string> &ls );
+	void dbList( vector<string> &ls, bool checkSel = false );
 
-	//- Open/close table. -
+	void closeOldTables( int secOld = 60 );
+
+	//> Open/close table.
 	AutoHD<TTable> open( const string &bdn, bool create = false );
 	void close( const string &bdn, bool del = false );
 
-	//- Get Data from DB or config file. If <tbl> cleaned then load from config file -
+	//> Get Data from DB or config file. If <tbl> cleaned then load from config file
 	bool dataSeek( const string &bdn, const string &path, int lev, TConfig &cfg );
 	bool dataGet( const string &bdn, const string &path, TConfig &cfg );
 	bool dataSet( const string &bdn, const string &path, TConfig &cfg );
-	bool dataDel( const string &bdn, const string &path, TConfig &cfg );
+	bool dataDel( const string &bdn, const string &path, TConfig &cfg, bool useKeyAll = false );
 
-	//- Generic DB table -
-	static string genDBGet( const string &path, const string &oval = "",
-		const string &user = "root", bool onlyCfg = false );
-	static void genDBSet( const string &path, const string &val, const string &user = "root" );
+	//> Generic DB table
+	static string genDBGet( const string &path, const string &oval = "", const string &user = "root", char rFlg = 0 );
+	static void genDBSet( const string &path, const string &val, const string &user = "root", char rFlg = 0 );
 
 	string fullDBSYS( );
 	string fullDB( );
@@ -234,6 +246,8 @@ class TBDS : public TSubSYS, public TElem
 
 	string optDescr( );
 
+	Res &nodeRes( )		{ return nRes; }
+
     protected:
 	void load_( );
 
@@ -243,8 +257,9 @@ class TBDS : public TSubSYS, public TElem
 
 	//Private attributes
 	TElem	el_db;
-	Res	genDBCacheRes;
 	deque<TConfig*> genDBCache;
+
+	Res	nRes;
 };
 
 #endif // TBDS_H
