@@ -36,10 +36,11 @@
 //*************************************************
 //* TCntrNode                                     *
 //*************************************************
+pthread_mutex_t TCntrNode::connM = PTHREAD_MUTEX_INITIALIZER;
 
 //*************************************************
 //* Controll scenaries language section           *
-TCntrNode::TCntrNode( TCntrNode *iprev ) : m_use(0), m_flg(0), m_oi(USHRT_MAX)
+TCntrNode::TCntrNode( TCntrNode *iprev ) : mUse(0), m_flg(0), mOi(USHRT_MAX), chGrp(NULL)
 {
     setNodeMode(Disable);
     prev.node = iprev;
@@ -50,6 +51,7 @@ TCntrNode::TCntrNode( TCntrNode *iprev ) : m_use(0), m_flg(0), m_oi(USHRT_MAX)
 TCntrNode::~TCntrNode()
 {
     nodeDelAll();
+    if( chGrp ) delete chGrp;
 }
 
 TCntrNode &TCntrNode::operator=( TCntrNode &node )
@@ -62,11 +64,11 @@ void TCntrNode::nodeDelAll( )
     if( nodeMode() != Disable )     nodeDis( );
 
     TMap::iterator p;
-    for( unsigned i_g = 0; i_g < chGrp.size(); i_g++ )
-	while( (p=chGrp[i_g].elem.begin())!=chGrp[i_g].elem.end() )
+    for( unsigned i_g = 0; chGrp && i_g < chGrp->size(); i_g++ )
+	while( (p = (*chGrp)[i_g].elem.begin()) != (*chGrp)[i_g].elem.end() )
 	{
 	    delete p->second;
-	    chGrp[i_g].elem.erase(p);
+	    (*chGrp)[i_g].elem.erase(p);
 	}
 }
 
@@ -104,29 +106,35 @@ void TCntrNode::cntrCmd( XMLNode *opt, int lev, const string &ipath, int off )
     {
 	if( !s_br.empty() && s_br[0] != '/' )
 	{
-	    for( int i_g = 0; i_g < chGrp.size(); i_g++ )
-		if( s_br.substr(0,chGrp[i_g].id.size()) == chGrp[i_g].id )
+	    for( int i_g = 0; chGrp && i_g < chGrp->size(); i_g++ )
+		if( s_br.substr(0,(*chGrp)[i_g].id.size()) == (*chGrp)[i_g].id )
 		{
-		    chldAt(i_g,s_br.substr(chGrp[i_g].id.size())).at().cntrCmd(opt,0,path,off);
+		    chldAt(i_g,s_br.substr((*chGrp)[i_g].id.size())).at().cntrCmd(opt,0,path,off);
 		    return;
 		}
-	    //- Go to default thread -
-	    if( !chGrp.empty() ) chldAt(0,s_br).at().cntrCmd(opt,0,path,off);
+	    //> Go to default thread
+	    if( chGrp ) chldAt(0,s_br).at().cntrCmd(opt,0,path,off);
 	    return;
 	}
-	//- Post command to node -
-	opt->setAttr("path",s_br);
-	cntrCmdProc(opt);
-	if( opt->attr("rez") != "0" )
-	    throw TError("ContrItfc",_("%s:%s:> Control element <%s> error!"),opt->name().c_str(),path.c_str(),s_br.c_str());
+	//> Post command to node
+	if( opt->name() == "CntrReqs" )
+	    for( int i_n = 0; i_n < opt->childSize(); i_n++ )
+		cntrCmd(opt->childGet(i_n));
+	else
+	{
+	    opt->setAttr("path",s_br);
+	    cntrCmdProc(opt);
+	    if( opt->attr("rez") != "0" )
+		throw TError("ContrItfc",_("%s:%s:> Control element <%s> error!"),opt->name().c_str(),path.c_str(),s_br.c_str());
+	}
     }
     catch(TError err)
     {
 	if( err.cat == "warning" )	opt->setAttr("rez","1");
 	else opt->setAttr("rez","2");
-	opt->childClean();
+	opt->childClear();
 	opt->setAttr("mcat",err.cat);
-	opt->setText(err.mess);	
+	opt->setText(err.mess);
     }
     opt->setAttr("path",path);
 }
@@ -145,8 +153,8 @@ void TCntrNode::nodeEn( int flag )
     preEnable( flag );
 
     TMap::iterator p;
-    for( unsigned i_g = 0; i_g < chGrp.size(); i_g++ )
-	for( p=chGrp[i_g].elem.begin(); p != chGrp[i_g].elem.end(); p++ )
+    for( unsigned i_g = 0; chGrp && i_g < chGrp->size(); i_g++ )
+	for( p = (*chGrp)[i_g].elem.begin(); p != (*chGrp)[i_g].elem.end(); ++p )
 	    if( p->second->nodeMode( ) == Disable )
 		p->second->nodeEn( flag );
 
@@ -172,8 +180,8 @@ void TCntrNode::nodeDis( long tm, int flag )
     try
     {
 	TMap::iterator p;
-	for( unsigned i_g = 0; i_g < chGrp.size(); i_g++ )
-	    for( p=chGrp[i_g].elem.begin(); p != chGrp[i_g].elem.end(); p++ )
+	for( unsigned i_g = 0; chGrp && i_g < chGrp->size(); i_g++ )
+	    for( p = (*chGrp)[i_g].elem.begin(); p != (*chGrp)[i_g].elem.end(); ++p )
 		if( p->second->nodeMode() == Enable )
 		    p->second->nodeDis( tm, flag );
 
@@ -181,15 +189,15 @@ void TCntrNode::nodeDis( long tm, int flag )
 	time_t t_cur = time(NULL);
 	while(1)
 	{
-	    if( !m_use )	break;
-#if OSC_DEBUG
-            mess_debug(nodePath().c_str(),_("Wait of free %d users!"),m_use);
+	    if( !mUse )	break;
+#if OSC_DEBUG >= 1
+            mess_debug(nodePath().c_str(),_("Waiting for freeing by %d users!"),mUse);
 #endif
 	    //Check timeout
 	    if( tm && time(NULL) > t_cur+tm)
 	    {
 		if( !TSYS::finalKill )
-		    throw TError(nodePath().c_str(),_("Timeouted of wait. Object used by %d users. Free object first!"),m_use);
+		    throw TError(nodePath().c_str(),_("Timeouted of wait. Object is used by %d users. Free object first!"),mUse);
 		mess_err(nodePath().c_str(),_("Blocking node error. Inform developpers please!"));
 		break;
 	    }
@@ -216,12 +224,12 @@ void TCntrNode::nodeList( vector<string> &list, const string &gid )
 {
     vector<string> tls;
     list.clear();
-    for( int i_gr = 0; i_gr < chGrp.size(); i_gr++ )
-	if( gid.empty() || gid == chGrp[i_gr].id )
+    for( int i_gr = 0; chGrp && i_gr < chGrp->size(); i_gr++ )
+	if( gid.empty() || gid == (*chGrp)[i_gr].id )
 	{
 	    chldList( i_gr, tls );
 	    for( int i_l = 0; i_l < tls.size(); i_l++ )
-		list.push_back(chGrp[i_gr].id+tls[i_l]);
+		list.push_back((*chGrp)[i_gr].id+tls[i_l]);
 	    if( !gid.empty() )	break;
 	}
 }
@@ -236,11 +244,11 @@ AutoHD<TCntrNode> TCntrNode::nodeAt( const string &path, int lev, char sep, int
 	return this;
     }
     ResAlloc res(hd_res,false);
-    for( int i_g = 0; i_g < chGrp.size(); i_g++ )
-	if( s_br.substr(0,chGrp[i_g].id.size()) == chGrp[i_g].id )
-	    return chldAt(i_g,s_br.substr(chGrp[i_g].id.size())).at().nodeAt(path,0,sep,off);
-    //Go to default group
-    if( chGrp.size() )	return chldAt(0,s_br).at().nodeAt(path,0,sep,off);
+    for( int i_g = 0; chGrp && i_g < chGrp->size(); i_g++ )
+	if( s_br.substr(0,(*chGrp)[i_g].id.size()) == (*chGrp)[i_g].id )
+	    return chldAt(i_g,s_br.substr((*chGrp)[i_g].id.size())).at().nodeAt(path,0,sep,off);
+    //> Go to default group
+    if( chGrp )	return chldAt(0,s_br).at().nodeAt(path,0,sep,off);
 }
 
 void TCntrNode::nodeDel( const string &path, char sep, int flag, bool shDel )
@@ -264,7 +272,7 @@ void TCntrNode::nodeCopy( const string &src, const string &dst, const string &us
     int n_del = 0;
     for( int off = 0; !(t_el=TSYS::pathLev(dst,0,true,&off)).empty(); n_del++ )
     { if( n_del ) d_elp += ("/"+d_el); d_el = t_el; }
-    if( !n_del ) throw TError(SYS->nodePath().c_str(),_("Copy from '%s' to '%s' impossible"),src.c_str(),dst.c_str());
+    if( !n_del ) throw TError(SYS->nodePath().c_str(),_("Copy from '%s' to '%s' is impossible"),src.c_str(),dst.c_str());
 
     //- Connect to destination containers node -
     AutoHD<TCntrNode> dst_n = SYS->nodeAt(d_elp);
@@ -274,7 +282,7 @@ void TCntrNode::nodeCopy( const string &src, const string &dst, const string &us
     br_req.setAttr("user",user)->setAttr("path","/%2fbr");
     dst_n.at().cntrCmd(&br_req);
     if( atoi(br_req.attr("rez").c_str()) || !br_req.childGet(0,true) )
-	throw TError(SYS->nodePath().c_str(),_("Destination node not have branches."));
+	throw TError(SYS->nodePath().c_str(),_("Destination node doesn't have branches."));
     XMLNode *branch = br_req.childGet(0);
     int i_b;
     for( i_b = 0; i_b < branch->childSize(); i_b++ )
@@ -282,12 +290,12 @@ void TCntrNode::nodeCopy( const string &src, const string &dst, const string &us
 		atoi(branch->childGet(i_b)->attr("acs").c_str())&SEQ_WR )
 	    break;
     if( i_b >= branch->childSize() )
-	throw TError(SYS->nodePath().c_str(),_("Destination node not have want branche."));
+        throw TError(SYS->nodePath().c_str(),_("Destination node doesn't have necessary branche."));
     bool idm = atoi(branch->childGet(i_b)->attr("idm").c_str());
     string n_grp = branch->childGet(i_b)->attr("id");
     d_el = d_el.substr(n_grp.size());
     i_b = dst_n.at().grpId(n_grp);
-    if( i_b < 0 ) throw TError(SYS->nodePath().c_str(),_("Destination node not have want branche."));
+    if( i_b < 0 ) throw TError(SYS->nodePath().c_str(),_("Destination node doesn't have necessary branche."));
 
     //- Connect or create new destination node -
     if( !dst_n.at().chldPresent( i_b, d_el ) )
@@ -305,20 +313,24 @@ void TCntrNode::nodeCopy( const string &src, const string &dst, const string &us
 
 unsigned TCntrNode::grpAdd( const string &iid, bool iordered )
 {
+    if( !chGrp ) chGrp = new vector<GrpEl>;
+
     int g_id;
-    for( g_id = 0; g_id < chGrp.size(); g_id++ )
-	if( chGrp[g_id].id == iid ) break;
-    if( g_id == chGrp.size() )	chGrp.push_back( GrpEl() );
-    chGrp[g_id].id = iid;
-    chGrp[g_id].ordered = iordered;
+    for( g_id = 0; g_id < chGrp->size(); g_id++ )
+	if( (*chGrp)[g_id].id == iid ) break;
+    if( g_id == chGrp->size() )	chGrp->push_back( GrpEl() );
+    (*chGrp)[g_id].id = iid;
+    (*chGrp)[g_id].ordered = iordered;
 
     return g_id;
 }
 
+char TCntrNode::grpSize( )	{ return chGrp ? chGrp->size() : 0; }
+
 char TCntrNode::grpId( const string &sid )
 {
-    for( int g_id = 0; g_id < chGrp.size(); g_id++ )
-	if( chGrp[g_id].id == sid )	return g_id;
+    for( int g_id = 0; chGrp && g_id < chGrp->size(); g_id++ )
+	if( (*chGrp)[g_id].id == sid )	return g_id;
 
     return -1;
 }
@@ -326,31 +338,32 @@ char TCntrNode::grpId( const string &sid )
 TCntrNode::GrpEl &TCntrNode::grpAt( char iid )
 {
     if( iid < 0 || iid >= grpSize( ) )	throw TError(nodePath().c_str(),_("Branche group '%d' error."),iid);
-    return chGrp[iid];
+    return (*chGrp)[iid];
 }
 
 void TCntrNode::chldList( char igr, vector<string> &list )
 {
     ResAlloc res(hd_res,false);
-    if( igr >= chGrp.size() )	throw TError(nodePath().c_str(),"Group of childs %d error!",igr);
+    if( !chGrp || igr >= chGrp->size() ) throw TError(nodePath().c_str(),"Group of childs %d error!",igr);
     if( nodeMode() == Disable )	throw TError(nodePath().c_str(),"Node is disabled!");
 
     list.clear();
-    if( !chGrp[igr].ordered )
+    list.reserve((*chGrp)[igr].elem.size());
+    if( !(*chGrp)[igr].ordered )
     {
-	for( TMap::iterator p=chGrp[igr].elem.begin(); p!=chGrp[igr].elem.end(); p++ )
+	for( TMap::iterator p = (*chGrp)[igr].elem.begin(); p != (*chGrp)[igr].elem.end(); ++p )
 	    if( p->second->nodeMode() != Disable )
 		list.push_back(p->first);
     }
     else
     {
 	bool disYes = false;
-	for( TMap::iterator p=chGrp[igr].elem.begin(); p != chGrp[igr].elem.end(); p++ )
+	for( TMap::iterator p = (*chGrp)[igr].elem.begin(); p != (*chGrp)[igr].elem.end(); ++p )
 	    if( p->second->nodeMode() == Disable )	disYes = true;
 	    else
 	    {
-		while( p->second->m_oi >= list.size() )	list.push_back("");
-		list[p->second->m_oi] = p->first;
+		while( p->second->mOi >= list.size() )	list.push_back("");
+		list[p->second->mOi] = p->first;
 	    }
 	if( disYes )
 	    for( int i_p = 0; i_p < list.size(); i_p++ )
@@ -362,11 +375,11 @@ void TCntrNode::chldList( char igr, vector<string> &list )
 bool TCntrNode::chldPresent( char igr, const string &name )
 {
     ResAlloc res(hd_res,false);
-    if( igr >= chGrp.size() )	throw TError(nodePath().c_str(),"Group of childs %d error!",igr);
+    if( !chGrp || igr >= chGrp->size() ) throw TError(nodePath().c_str(),"Group of childs %d error!",igr);
     if( nodeMode() == Disable )	throw TError(nodePath().c_str(),"Node is disabled!");
 
-    TMap::iterator p=chGrp[igr].elem.find(name);
-    if( p != chGrp[igr].elem.end() && p->second->nodeMode() == Enable ) return true;
+    TMap::iterator p = (*chGrp)[igr].elem.find(name);
+    if( p != (*chGrp)[igr].elem.end() && p->second->nodeMode() == Enable ) return true;
 
     return false;
 }
@@ -374,16 +387,17 @@ bool TCntrNode::chldPresent( char igr, const string &name )
 void TCntrNode::chldAdd( char igr, TCntrNode *node, int pos )
 {
     ResAlloc res(hd_res,false);
-    if( igr >= chGrp.size() )	throw TError(nodePath().c_str(),"Group of childs %d error!",igr);
+    if( !chGrp || igr >= chGrp->size() ) throw TError(nodePath().c_str(),"Group of childs %d error!",igr);
     if( nodeMode() != Enable )	throw TError(nodePath().c_str(),"Node is not enabled!");
 
     TMap::iterator p;
-    if( TSYS::strEmpty(node->nodeName()) )
+    if( TSYS::strNoSpace(node->nodeName()).empty() )
     {
 	delete node;
 	throw TError(nodePath().c_str(),"Add child id is empty!");
     }
-    if( (p=chGrp[igr].elem.find(node->nodeName())) != chGrp[igr].elem.end() )
+
+    if( (p = (*chGrp)[igr].elem.find(node->nodeName())) != (*chGrp)[igr].elem.end() )
     {
 	delete node;
 	if( p->second->nodeMode() == Disable )	p->second->nodeEn( TCntrNode::NodeRestore );
@@ -393,14 +407,14 @@ void TCntrNode::chldAdd( char igr, TCntrNode *node, int pos )
     res.request(true);
     node->prev.node = this;
     node->prev.grp = igr;
-    if( chGrp[igr].ordered )
+    if( (*chGrp)[igr].ordered )
     {
-	pos = (pos<0||pos>chGrp[igr].elem.size()) ? chGrp[igr].elem.size() : pos;
-	node->m_oi = pos;
-	for( p = chGrp[igr].elem.begin(); p != chGrp[igr].elem.end(); p++ )
-	    if( p->second->m_oi >= pos ) p->second->m_oi++;
+	pos = (pos<0||pos>(*chGrp)[igr].elem.size()) ? (*chGrp)[igr].elem.size() : pos;
+	node->mOi = pos;
+	for( p = (*chGrp)[igr].elem.begin(); p != (*chGrp)[igr].elem.end(); p++ )
+	    if( p->second->mOi >= pos ) p->second->mOi++;
     }
-    chGrp[igr].elem.insert(std::pair<string,TCntrNode*>(node->nodeName(),node));
+    (*chGrp)[igr].elem.insert(std::pair<string,TCntrNode*>(node->nodeName(),node));
     res.release();
 
     if( node->nodeMode() == Disable )	node->nodeEn( TCntrNode::NodeConnect );
@@ -410,13 +424,13 @@ void TCntrNode::chldDel( char igr, const string &name, long tm, int flag, bool s
 {
     if( tm < 0 )	tm = DEF_TIMEOUT;
     ResAlloc res(hd_res,false);
-    if( igr >= chGrp.size() )	throw TError(nodePath().c_str(),"Group of childs %d error!",igr);
+    if( !chGrp || igr >= chGrp->size() ) throw TError(nodePath().c_str(),"Group of childs %d error!",igr);
     if( !(nodeMode() == Enable || nodeMode() == Disable) )
-	throw TError(nodePath().c_str(),"Node is proced now!");
+        throw TError(nodePath().c_str(),"Node is beign processed now!");
 
-    TMap::iterator p=chGrp[igr].elem.find(name);
-    if( p == chGrp[igr].elem.end() )
-	throw TError(nodePath().c_str(),"Child <%s> no present!", name.c_str());
+    TMap::iterator p = (*chGrp)[igr].elem.find(name);
+    if( p == (*chGrp)[igr].elem.end() )
+	throw TError(nodePath().c_str(),"Child <%s> is not present!", name.c_str());
 
     if( p->second->nodeMode() == Enable )
 	p->second->nodeDis( tm, (flag<<8)|(shDel?NodeShiftDel:0) );
@@ -424,14 +438,16 @@ void TCntrNode::chldDel( char igr, const string &name, long tm, int flag, bool s
     if( !shDel )
     {
 	res.request( true );
-	if( chGrp[igr].ordered )
+	p = (*chGrp)[igr].elem.find(name);
+	if( p == (*chGrp)[igr].elem.end() ) return;
+	if( (*chGrp)[igr].ordered )
 	{
-	    int pos = p->second->m_oi;
-	    for( TMap::iterator p1=chGrp[igr].elem.begin(); p1 != chGrp[igr].elem.end(); p1++ )
-		if( p1->second->m_oi > pos ) p1->second->m_oi--;
+	    int pos = p->second->mOi;
+	    for( TMap::iterator p1 = (*chGrp)[igr].elem.begin(); p1 != (*chGrp)[igr].elem.end(); ++p1 )
+		if( p1->second->mOi > pos ) p1->second->mOi--;
 	}
 	delete p->second;
-	chGrp[igr].elem.erase(p);
+	(*chGrp)[igr].elem.erase(p);
     }
 }
 
@@ -440,10 +456,10 @@ unsigned TCntrNode::nodeUse(  )
     ResAlloc res(hd_res,false);
     if( nodeMode() == Disable )	throw TError(nodePath().c_str(),"Node is disabled!");
 
-    unsigned i_use = m_use;
+    unsigned i_use = mUse;
     TMap::iterator p;
-    for( unsigned i_g = 0; i_g < chGrp.size(); i_g++ )
-     for( p=chGrp[i_g].elem.begin(); p!=chGrp[i_g].elem.end(); p++ )
+    for( unsigned i_g = 0; chGrp && i_g < chGrp->size(); i_g++ )
+     for( p = (*chGrp)[i_g].elem.begin(); p != (*chGrp)[i_g].elem.end(); ++p )
 	if( p->second->nodeMode() != Disable )
 	    i_use+=p->second->nodeUse();
 
@@ -457,10 +473,10 @@ string TCntrNode::nodePath( char sep, bool from_root )
 	if( prev.node )
 	{
 	    if( from_root && !prev.node->prev.node )
-		return ((prev.grp<0)?"":prev.node->chGrp[prev.grp].id)+nodeName();
+		return ((prev.grp<0)?"":(*(prev.node->chGrp))[prev.grp].id)+nodeName();
 	    else
 		return prev.node->nodePath(sep,from_root)+string(1,sep)+
-			((prev.grp<0)?"":prev.node->chGrp[prev.grp].id)+nodeName();
+			((prev.grp<0)?"":(*(prev.node->chGrp))[prev.grp].id)+nodeName();
 	}
 	else return nodeName();
     }
@@ -468,7 +484,7 @@ string TCntrNode::nodePath( char sep, bool from_root )
     {
 	if( prev.node )
 	    return prev.node->nodePath(sep,from_root)+
-		    ((prev.grp<0)?"":prev.node->chGrp[prev.grp].id)+nodeName()+"/";
+		    ((prev.grp<0)?"":(*(prev.node->chGrp))[prev.grp].id)+nodeName()+"/";
 	else return from_root?"/":("/"+nodeName()+"/");
     }
 }
@@ -477,18 +493,18 @@ TCntrNode *TCntrNode::nodePrev( bool noex )
 {
     if( prev.node ) return prev.node;
     if( noex )	return NULL;
-    throw TError(nodePath().c_str(),"Node is it root or no connect!");
+    throw TError(nodePath().c_str(),"Node is the root or is not connected!");
 }
 
 AutoHD<TCntrNode> TCntrNode::chldAt( char igr, const string &name, const string &user )
 {
     ResAlloc res( hd_res, false );
-    if( igr >= chGrp.size() ) 	throw TError(nodePath().c_str(),"Group of childs %d error!",igr);
+    if( !chGrp || igr >= chGrp->size() ) throw TError(nodePath().c_str(),"Group of childs %d error!",igr);
     if( nodeMode() == Disable )	throw TError(nodePath().c_str(),"Node is disabled!");
 
-    TMap::iterator p=chGrp[igr].elem.find(name);
-    if(p == chGrp[igr].elem.end() || p->second->nodeMode() == Disable)
-	throw TError(nodePath().c_str(),_("Element <%s> no present or disabled!"), name.c_str());
+    TMap::iterator p = (*chGrp)[igr].elem.find(name);
+    if( p == (*chGrp)[igr].elem.end() || p->second->nodeMode() == Disable )
+	throw TError(nodePath().c_str(),_("Element <%s> is not present or disabled!"), name.c_str());
 
     return AutoHD<TCntrNode>(p->second,user);
 }
@@ -500,12 +516,12 @@ int TCntrNode::isModify( int f )
 
     if( f&Self && m_flg&SelfModify )	rflg |= Self;
     if( f&Child )
-	for( unsigned i_g = 0; i_g < chGrp.size(); i_g++ )
+	for( unsigned i_g = 0; chGrp && i_g < chGrp->size(); i_g++ )
 	{
 	    TMap::iterator p;
-	    for( p=chGrp[i_g].elem.begin(); p != chGrp[i_g].elem.end(); p++ )
+	    for( p = (*chGrp)[i_g].elem.begin(); p != (*chGrp)[i_g].elem.end(); ++p )
 		if( p->second->isModify(Self|Child) )	{ rflg |= Child; break; }
-	    if( p != chGrp[i_g].elem.end() )	break;
+	    if( p != (*chGrp)[i_g].elem.end() )	break;
 	}
 
     return rflg;
@@ -515,8 +531,8 @@ void TCntrNode::modifG( )
 {
     ResAlloc res( hd_res, false );
     modif( );
-    for( unsigned i_g = 0; i_g < chGrp.size(); i_g++ )
-	for( TMap::iterator p=chGrp[i_g].elem.begin(); p != chGrp[i_g].elem.end(); p++ )
+    for( unsigned i_g = 0; chGrp && i_g < chGrp->size(); i_g++ )
+	for( TMap::iterator p = (*chGrp)[i_g].elem.begin(); p != (*chGrp)[i_g].elem.end(); ++p )
 	    p->second->modifG( );
 }
 
@@ -524,8 +540,8 @@ void TCntrNode::modifGClr( )
 {
     ResAlloc res( hd_res, false );
     modifClr( );
-    for( unsigned i_g = 0; i_g < chGrp.size(); i_g++ )
-	for( TMap::iterator p=chGrp[i_g].elem.begin(); p != chGrp[i_g].elem.end(); p++ )
+    for( unsigned i_g = 0; chGrp && i_g < chGrp->size(); i_g++ )
+	for( TMap::iterator p = (*chGrp)[i_g].elem.begin(); p != (*chGrp)[i_g].elem.end(); ++p )
 	    p->second->modifGClr( );
 }
 
@@ -541,14 +557,14 @@ void TCntrNode::load( )
 	catch( TError err )
 	{
 	    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
-	    mess_err(nodePath().c_str(),_("Load node is error."));
+	    mess_err(nodePath().c_str(),_("Loading node error."));
 	}
     //- Childs load process -
     if( isModify(Child)&Child )
     {
 	ResAlloc res( hd_res, false );
-	for( unsigned i_g = 0; i_g < chGrp.size(); i_g++ )
-	    for( TMap::iterator p = chGrp[i_g].elem.begin(); p != chGrp[i_g].elem.end(); p++ )
+	for( unsigned i_g = 0; chGrp && i_g < chGrp->size(); i_g++ )
+	    for( TMap::iterator p = (*chGrp)[i_g].elem.begin(); p != (*chGrp)[i_g].elem.end(); ++p )
 		if( p->second->isModify(Self|Child) )	p->second->load( );
     }
     modifClr( );
@@ -562,16 +578,16 @@ void TCntrNode::save( )
 	catch(TError err)
 	{
 	    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
-	    mess_err(nodePath().c_str(),_("Save node is error."));
+	    mess_err(nodePath().c_str(),_("Saving node error."));
 	}
     //- Childs load process -
     if( isModify(Child)&Child )
     {
 	ResAlloc res( hd_res, false );
-	for( unsigned i_g = 0; i_g < chGrp.size(); i_g++ )
-	    for( int i_p = 0; i_p < chGrp[i_g].elem.size(); i_p++ )
+	for( unsigned i_g = 0; chGrp && i_g < chGrp->size(); i_g++ )
+	    for( int i_p = 0; i_p < (*chGrp)[i_g].elem.size(); i_p++ )
 	    {
-		TMap::iterator p = chGrp[i_g].elem.begin();
+		TMap::iterator p = (*chGrp)[i_g].elem.begin();
 		for( int i = 0; i < i_p; i++ )	p++;
 		
 		if( p->second->isModify(Self|Child) )	p->second->save( );
@@ -590,16 +606,17 @@ void TCntrNode::save( )
 
 void TCntrNode::AHDConnect()
 {
-    conn_res.resRequestW( );
-    m_use++;
-    conn_res.resReleaseW( );
+    pthread_mutex_lock(&connM);
+    mUse++;
+    if( mUse > 65000 ) mess_err(nodePath().c_str(),_("Too more users for node!!!"));
+    pthread_mutex_unlock(&connM);
 }
 
 void TCntrNode::AHDDisConnect()
 {
-    conn_res.resRequestW( );
-    m_use--;
-    conn_res.resReleaseW( );
+    pthread_mutex_lock(&connM);
+    mUse--;
+    pthread_mutex_unlock(&connM);
 }
 
 XMLNode *TCntrNode::ctrMkNode( const char *n_nd, XMLNode *nd, int pos, const char *path, const string &dscr,
@@ -639,7 +656,7 @@ XMLNode *TCntrNode::ctrMkNode( const char *n_nd, XMLNode *nd, int pos, const cha
 	if( obj1 ) { obj = obj1; continue; }
 	//int wofft = woff;
 	if( TSYS::pathLev(path,0,true,&woff).size() )
-	    throw TError("ContrItfc",_("Some tags on path <%s> missed!"),req.c_str());
+	    throw TError("ContrItfc",_("Some tags on path <%s> are missed!"),req.c_str());
 	obj = obj->childIns(pos);
     }
     obj->setName(n_nd)->setAttr("id",reqt1)->setAttr("dscr",dscr)->setAttr("acs",TSYS::int2str(n_acs));
@@ -678,15 +695,53 @@ void TCntrNode::cntrCmdProc( XMLNode *opt )
     if( opt->name() == "info" )
 	ctrMkNode("oscada_cntr",opt,-1,"/",_("Node: ")+nodeName(),0444,"root","root");
 
-    //- Process command to page -
+    //> Process command to page
     string a_path = opt->attr("path");
     if( a_path == "/obj" )
     {
-	if( ctrChkNode(opt,"modify",RWRWRW) )	opt->setText( isModify(TCntrNode::All) ? "1" : "0" );
+	//>> Get node modify flag
+	if( ctrChkNode(opt,"modify",R_R_R_) )	opt->setText(isModify(TCntrNode::All)?"1":"0");
+	//>> Do load node
 	else if( ctrChkNode(opt,"load",RWRWRW,"root","root",SEQ_WR) )	load( );
+	//>> Do save node
 	else if( ctrChkNode(opt,"save",RWRWRW,"root","root",SEQ_WR) )	save( );
+	//>> Do copy node
 	else if( ctrChkNode(opt,"copy",RWRWRW,"root","root",SEQ_WR) )
 	    nodeCopy(opt->attr("src"),opt->attr("dst"),opt->attr("user"));
+	//>> Request node childs parameters
+	else if( ctrChkNode(opt,"chlds",R_R_R_,"root","root",SEQ_RD) )
+	{
+	    string chGrp = opt->attr("grp");
+	    bool icoCheck = atoi(opt->attr("icoCheck").c_str());
+	    vector<string> ls;
+	    XMLNode req("get"); req.setAttr("path","/br/"+chGrp)->setAttr("user",opt->attr("user"));
+	    cntrCmdProc(&req);
+	    int chGrpId = grpId(chGrp);
+	    if( chGrpId >= 0 )
+		for( int i_ch = 0; i_ch < req.childSize(); i_ch++ )
+		{
+		    XMLNode *chN = opt->childAdd();
+		    *chN = *req.childGet(i_ch);
+		    //>>> Connect to child and get info from it
+		    AutoHD<TCntrNode> ch = chldAt(chGrpId,chN->attr("id").empty()?chN->text():chN->attr("id"));
+		    //>>>> Check icon
+		    XMLNode reqIco("get"); reqIco.setAttr("path","/ico")->setAttr("user",opt->attr("user"));
+		    ch.at().cntrCmdProc(&reqIco);
+		    if( icoCheck ) chN->setAttr("icoSize",TSYS::int2str(reqIco.text().size()));
+		    else chN->childAdd("ico")->setText(reqIco.text());
+		    //>>>> Process groups
+		    XMLNode brReq("info"); brReq.setAttr("path","/br")->setAttr("user",opt->attr("user"));
+		    ch.at().cntrCmdProc(&brReq);
+		    for( int i_br = 0; brReq.childSize() && i_br < brReq.childGet(0)->childSize(); i_br++ )
+		    {
+			XMLNode *chB = chN->childAdd();
+			*chB = *brReq.childGet(0)->childGet(i_br);
+			int grpBrId = ch.at().grpId(chB->attr("id"));
+			ch.at().chldList(grpBrId,ls);
+			chB->setAttr("chPresent",ls.size()?"1":"0");
+		    }
+		}
+	}
     }
     else if( (a_path == "/db/list" || a_path == "/db/tblList") && ctrChkNode(opt) )
     {
