@@ -22,16 +22,19 @@
 #ifndef TCNTRNODE_H
 #define TCNTRNODE_H
 
+#include <pthread.h>
+
 #include <string>
 #include <vector>
 
-//- Security standard permissions -
+//> Security standard permissions
 #define R_R_R_  0444
 #define R_R___  0440
 #define R_____  0400
 #define RWRWRW  0666
 #define RWRWR_  0664
 #define RWR_R_  0644
+#define RWRW__  0660
 #define RWR___  0640
 #define RW____  0600
 
@@ -41,7 +44,14 @@ using std::vector;
 
 class TCntrNode;
 
-#if OSC_HASHMAP
+#if OSC_ORDLISTMAP
+
+#include <map>
+using std::map;
+typedef map<string, TCntrNode* > TMap;
+
+#else
+
 #include <ext/hash_map>
 using __gnu_cxx::hash_map;
 
@@ -61,10 +71,7 @@ typedef hash_map<string, TCntrNode*, __gnu_cxx::hash<string> > TMap;
 //#include <unordered_map>
 //using std::unordered_map;
 //typedef unordered_map<string, TCntrNode* > TMap;
-#else
-#include <map>
-using std::map;
-typedef map<string, TCntrNode* > TMap;
+
 #endif
 
 #include "xml.h"
@@ -88,10 +95,10 @@ class TCntrNode
 
 	void cntrCmd( XMLNode *opt, int lev = 0, const string &path = "", int off = 0 );
 
-	//- Static functions -
+	//> Static functions
 	static XMLNode *ctrId( XMLNode *inf, const string &n_id, bool noex = false );      //get node for it full identifier
 
-	//-- Controll Field --
+	//> Controll Field
 	static XMLNode *ctrMkNode( const char *n_nd, XMLNode *nd, int pos, const char *req, const string &dscr,
 	    int perm=0777, const char *user="root", const char *grp="root", int n_attr=0, ... );
 	static bool ctrChkNode( XMLNode *nd, const char *cmd="get", int perm=0444, const char *user="root",
@@ -106,12 +113,12 @@ class TCntrNode
 	//Data
 	enum Flag
 	{
-	    //- Modes -
+	    //> Modes
 	    MkDisable	= 0x00,		//Node make disable
 	    Disable	= 0x01,		//Node disabled
 	    MkEnable	= 0x02,		//Node make enable
 	    Enable	= 0x03,		//Node enabled
-	    //- Flags -
+	    //> Flags
 	    SelfModify	= 0x04,		//Self modify
 	};
 	enum EnFlag
@@ -123,6 +130,7 @@ class TCntrNode
 	enum ModifFlag	{ Self = 0x01, Child = 0x02, All = 0x03 };
 
 	//Methods
+	virtual Res &nodeRes( )		{ return hd_res; }
 	virtual string nodeName( )	{ return "NO Named!"; }
 	string nodePath( char sep = 0, bool from_root = false );
 
@@ -135,8 +143,9 @@ class TCntrNode
 	char	 nodeFlg( )		{ return m_flg; }
 	char	 nodeMode( )		{ return m_flg&0x3; }
 	unsigned nodeUse( );
+	unsigned nodePos( )		{ return mOi; }
 
-	//- Modify process methods -
+	//> Modify process methods
 	int  isModify( int mflg = TCntrNode::All );		//Check for modify want
 	void modif( )	{ m_flg |= SelfModify; }		//Set local modify
 	void modifG( );						//Set group modify
@@ -145,8 +154,8 @@ class TCntrNode
 	void load( );						//Load node, if modified
 	void save( );						//Save node, if modified
 
-	void AHDConnect( );
-	void AHDDisConnect( );
+	virtual void AHDConnect( );
+	virtual void AHDDisConnect( );
 
     protected:
 	//Data
@@ -158,7 +167,7 @@ class TCntrNode
 	};
 
 	//Methods
-	//- Commands -
+	//> Commands
 	void nodeEn( int flag = 0 );
 	void nodeDis( long tm = 0, int flag = 0 );
 
@@ -167,16 +176,16 @@ class TCntrNode
 	void setNodePrev( TCntrNode *node )	{ prev.node = node; }
 	void setNodeMode( char mode )		{ m_flg = (m_flg&(~0x03))|(mode&0x03); }
 
-	//- Childs -
+	//> Childs
 	AutoHD<TCntrNode> chldAt( char igr, const string &name, const string &user = "" );
 	void chldList( char igr, vector<string> &list );
 	bool chldPresent( char igr, const string &name );
 	void chldAdd( char igr, TCntrNode *node, int pos = -1 );
 	void chldDel( char igr, const string &name, long tm = -1, int flag = 0, bool shDel = false );
 
-	//- Conteiners -
-	char	grpSize( )	{ return chGrp.size(); }
-	char	 grpId( const string &sid );
+	//> Conteiners
+	char	grpSize( );
+	char	grpId( const string &sid );
 	GrpEl	&grpAt( char id );
 	unsigned grpAdd( const string &id, bool ordered = false );
 
@@ -198,15 +207,15 @@ class TCntrNode
 	} prev;
 
 	//Attributes
-	//- Childs -
-	Res	hd_res,			//Resource HD
-		conn_res;		//Connect resource
+	//> Childs
+	Res	hd_res;			//Resource HD
+	static pthread_mutex_t connM;	//Connection mutex
 
-	vector<GrpEl>	chGrp;		//Child groups
+	vector<GrpEl>	*chGrp;		//Child groups
 
-	//- Curent node -
-	unsigned char		m_use;	//Use counter
-	unsigned short int	m_oi;	//Order index
+	//> Curent node
+	unsigned short int	mUse;	//Use counter
+	unsigned short int	mOi;	//Order index
 
 	char	m_flg;
 };
