@@ -31,7 +31,7 @@ TConfig::TConfig( TElem *Elements ) : m_elem(NULL)
 
 TConfig::~TConfig()
 {
-    //- Deinit value -
+    //> Deinit value
     TCfgMap::iterator p;
     while( (p=value.begin())!=value.end() )
     {
@@ -76,7 +76,7 @@ void TConfig::addFld( TElem *el, unsigned id )
 void TConfig::delFld( TElem *el, unsigned id )
 {
     TCfgMap::iterator p=value.find(m_elem->fldAt(id).name());
-    if(p==value.end())	return;
+    if( p==value.end() ) return;
     delete p->second;
     value.erase(p);
 }
@@ -84,10 +84,18 @@ void TConfig::delFld( TElem *el, unsigned id )
 TCfg &TConfig::cfg( const string &n_val )
 {
     TCfgMap::iterator p=value.find(n_val);
-    if(p==value.end())	throw TError("TConfig",_("Attribute '%s' no present!"),n_val.c_str());
+    if( p==value.end() ) throw TError("TConfig",_("Attribute '%s' is not present!"),n_val.c_str());
     return *p->second;
 }
 
+TCfg *TConfig::at( const string &n_val, bool noExpt )
+{
+    TCfgMap::iterator p=value.find(n_val);
+    if( p != value.end() ) return p->second;
+    if( noExpt ) return NULL;
+    throw TError("TConfig",_("Attribute '%s' is not present!"),n_val.c_str());
+}
+
 void TConfig::cfgList( vector<string> &list )
 {
     list.clear();
@@ -97,21 +105,28 @@ void TConfig::cfgList( vector<string> &list )
 bool TConfig::cfgPresent( const string &n_val )
 {
     TCfgMap::iterator p=value.find(n_val);
-    if(p==value.end()) return false;
+    if( p==value.end() ) return false;
     return true;
 }
 
 void TConfig::cfgViewAll( bool val )
 {
-    for( TCfgMap::iterator p = value.begin(); p != value.end(); p++ )
+    for( TCfgMap::iterator p = value.begin(); p != value.end(); ++p )
 	p->second->setView(val);
 }
 
+void TConfig::cfgKeyUseAll( bool val )
+{
+    for( TCfgMap::iterator p = value.begin(); p != value.end(); ++p )
+	if( p->second->fld().flg()&TCfg::Key )
+	    p->second->setKeyUse(val);
+}
+
 void TConfig::setElem(TElem *Elements, bool first)
 {
-    if(m_elem == Elements && !first ) return;
+    if( m_elem == Elements && !first ) return;
 
-    //- Clear previos setting -
+    //> Clear previos setting
     if(m_elem)
     {
 	TCfgMap::iterator p;
@@ -124,7 +139,7 @@ void TConfig::setElem(TElem *Elements, bool first)
 	if(single) delete m_elem;
     }
 
-    //- Set new setting -
+    //> Set new setting
     if( !Elements )
     {
 	m_elem = new TElem("single");
@@ -180,225 +195,206 @@ void TConfig::cntrCmdProc( XMLNode *opt, const string &elem, const string &user,
 //*************************************************
 //* TCfg                                          *
 //*************************************************
-TCfg::TCfg( TFld &fld, TConfig &owner ) : m_view(true), m_owner(owner)
+TCfg::TCfg( TFld &fld, TConfig &owner ) : mView(true), mKeyUse(false), mOwner(owner), mNoTransl(false)
 {
-    //- Chek for self field for dinamic elements -
+    //> Chek for self field for dinamic elements
     if( fld.flg()&TFld::SelfFld )
     {
-	m_fld = new TFld();
-	*m_fld = fld;
+	mFld = new TFld();
+	*mFld = fld;
     }
-    else m_fld = &fld;
+    else mFld = &fld;
 
-    switch(m_fld->type())
+    switch(mFld->type())
     {
 	case TFld::String:
-	    m_val.s_val    = new string;
-	    *(m_val.s_val) = m_fld->def();
+	    m_val.s_val    = new ResString();
+	    m_val.s_val->setVal(mFld->def());
 	    break;
-	case TFld::Integer:	m_val.i_val = atoi(m_fld->def().c_str());	break;
-	case TFld::Real:	m_val.r_val = atof(m_fld->def().c_str());	break;
-	case TFld::Boolean:	m_val.b_val = atoi(m_fld->def().c_str());	break;
+	case TFld::Integer:	m_val.i_val = atoi(mFld->def().c_str());	break;
+	case TFld::Real:	m_val.r_val = atof(mFld->def().c_str());	break;
+	case TFld::Boolean:	m_val.b_val = atoi(mFld->def().c_str());	break;
     }
-    if( fld.flg()&TCfg::Hide )	m_view = false;
+    if( fld.flg()&TCfg::Hide )	mView = false;
 }
 
 TCfg::~TCfg(  )
 {
-    if( m_fld->type() == TFld::String )	delete m_val.s_val;
-    if( m_fld->flg()&TFld::SelfFld )   	delete m_fld;
+    if( mFld->type() == TFld::String )	delete m_val.s_val;
+    if( mFld->flg()&TFld::SelfFld )	delete mFld;
 }
 
-const string &TCfg::name()
-{
-    return m_fld->name();
-}
-
-string TCfg::getSEL( )
-{
-    if( !(m_fld->flg()&TFld::Selected) )
-	throw TError("Cfg",_("Element type no select!"));
-    switch( m_fld->type() )
-    {
-	case TFld::String:	return m_fld->selVl2Nm(*m_val.s_val);
-	case TFld::Integer:	return m_fld->selVl2Nm(m_val.i_val);
-	case TFld::Real:	return m_fld->selVl2Nm(m_val.r_val);
-	case TFld::Boolean:	return m_fld->selVl2Nm(m_val.b_val);
-    }
-}
+const string &TCfg::name()	{ return mFld->name(); }
 
 string &TCfg::getSd( )
 {
-    if( m_fld->type()!=TFld::String )
-	throw TError("Cfg",_("Element type no string!"));
-
-    return *m_val.s_val;
+    if( mFld->type()!=TFld::String )	throw TError("Cfg",_("Element type is not string!"));
+    return m_val.s_val->str;
 }
 
 double &TCfg::getRd( )
 {
-    if( m_fld->type()!=TFld::Real )
-	throw TError("Cfg",_("Element type no real!"));
-
+    if( mFld->type()!=TFld::Real )	throw TError("Cfg",_("Element type is not real!"));
     return m_val.r_val;
 }
 
 int &TCfg::getId( )
 {
-    if( m_fld->type()!=TFld::Integer )
-	throw TError("Cfg",_("Element type no int!"));
-
+    if( mFld->type()!=TFld::Integer )	throw TError("Cfg",_("Element type is not int!"));
     return m_val.i_val;
 }
 
 bool &TCfg::getBd( )
 {
-    if( m_fld->type()!=TFld::Boolean )
-	throw TError("Cfg",_("Element type no boolean!"));
-
+    if( mFld->type()!=TFld::Boolean )	throw TError("Cfg",_("Element type is not boolean!"));
     return m_val.b_val;
 }
 
-string TCfg::getS( )
+string TCfg::getSEL( char RqFlg )
 {
-    switch(m_fld->type())
+    if( !(mFld->flg()&TFld::Selected) )	throw TError("Cfg",_("Element type is not selected!"));
+    switch( mFld->type() )
     {
-	case TFld::String:	return *m_val.s_val;
-	case TFld::Integer:	return TSYS::int2str(m_val.i_val);
-	case TFld::Real:	return TSYS::real2str(m_val.r_val);
-	case TFld::Boolean:	return TSYS::int2str(m_val.b_val);
+	case TFld::String:	return mFld->selVl2Nm(getS(RqFlg));
+	case TFld::Integer:	return mFld->selVl2Nm(getI(RqFlg));
+	case TFld::Real:	return mFld->selVl2Nm(getR(RqFlg));
+	case TFld::Boolean:	return mFld->selVl2Nm(getB(RqFlg));
     }
 }
 
-double TCfg::getR( )
+string TCfg::getS( char RqFlg )
 {
-    switch(m_fld->type())
+    switch( mFld->type() )
     {
-	case TFld::String:	return atof(m_val.s_val->c_str());
-	case TFld::Integer:	return m_val.i_val;
+	case TFld::Integer:	return TSYS::int2str(getI(RqFlg));
+	case TFld::Real:	return TSYS::real2str(getR(RqFlg));
+	case TFld::Boolean:	return TSYS::int2str(getB(RqFlg));
+	case TFld::String:	return m_val.s_val->getVal();
+    }
+}
+
+double TCfg::getR( char RqFlg )
+{
+    switch( mFld->type() )
+    {
+	case TFld::String:	return atof(getS(RqFlg).c_str());
+	case TFld::Integer:	return getI(RqFlg);
+	case TFld::Boolean:	return getB(RqFlg);
 	case TFld::Real:	return m_val.r_val;
-	case TFld::Boolean:	return m_val.b_val;
     }
 }
 
-int TCfg::getI( )
+int TCfg::getI( char RqFlg )
 {
-    switch(m_fld->type())
+    switch( mFld->type() )
     {
-	case TFld::String:	return atoi(m_val.s_val->c_str());
+	case TFld::String:	return atoi(getS(RqFlg).c_str());
+	case TFld::Real:	return (int)getR(RqFlg);
+	case TFld::Boolean:	return getB(RqFlg);
 	case TFld::Integer:	return m_val.i_val;
-	case TFld::Real:	return (int)m_val.r_val;
+    }
+}
+
+bool TCfg::getB( char RqFlg )
+{
+    switch( mFld->type() )
+    {
+	case TFld::String:	return atoi(getS(RqFlg).c_str());
+	case TFld::Integer:	return getI(RqFlg);
+	case TFld::Real:	return (int)getR(RqFlg);
 	case TFld::Boolean:	return m_val.b_val;
     }
 }
 
-bool TCfg::getB( )
+void TCfg::setSEL( const string &val, char RqFlg )
 {
-    switch(m_fld->type())
+    if( !(mFld->flg()&TFld::Selected) ) throw TError("Cfg",_("Element type is not selected!"));
+    switch( mFld->type() )
     {
-	case TFld::String:	return atoi(m_val.s_val->c_str());
-	case TFld::Integer:	return m_val.i_val;
-	case TFld::Real:	return (int)m_val.r_val;
-	case TFld::Boolean:	return m_val.b_val;
+	case TFld::String:	setS( mFld->selNm2VlS(val), RqFlg );	break;
+	case TFld::Integer:	setI( mFld->selNm2VlI(val), RqFlg );	break;
+	case TFld::Real:	setR( mFld->selNm2VlR(val), RqFlg );	break;
+	case TFld::Boolean:	setB( mFld->selNm2VlB(val), RqFlg );	break;
     }
 }
 
-void TCfg::setSEL( const string &val, bool forcView )
+void TCfg::setS( const string &val, char RqFlg )
 {
-    if( !(m_fld->flg()&TFld::Selected) ) 
-	throw TError("Cfg",_("Element type no select!"));
-    switch( m_fld->type() )
-    {
-	case TFld::String:	setS( m_fld->selNm2VlS(val),forcView );	break;
-	case TFld::Integer:	setI( m_fld->selNm2VlI(val),forcView );	break;
-	case TFld::Real:	setR( m_fld->selNm2VlR(val),forcView );	break;
-	case TFld::Boolean:	setB( m_fld->selNm2VlB(val),forcView );	break;
-    }
-}
-
-void TCfg::setS( const string &val, bool forcView )
-{
-    switch( m_fld->type() )
+    switch( mFld->type() )
     {
+	case TFld::Integer:	setI( atoi(val.c_str()), RqFlg );	break;
+	case TFld::Real:	setR( atof(val.c_str()), RqFlg );	break;
+	case TFld::Boolean:	setB( atoi(val.c_str()), RqFlg );	break;
 	case TFld::String:
 	{
-	    string t_str = *(m_val.s_val);
-	    *(m_val.s_val) = val;
-	    if( !m_owner.cfgChange(*this) )	*(m_val.s_val) = t_str;
-	    if( forcView ) setView(true);
+	    string t_str = m_val.s_val->getVal();
+	    m_val.s_val->setVal(val);
+	    if( !mOwner.cfgChange(*this) ) m_val.s_val->setVal(t_str);
+	    if( RqFlg&TCfg::ForceUse )	{ setView(true); setKeyUse(true); }
 	    break;
 	}
-	case TFld::Integer:	setI( atoi(val.c_str()),forcView );	break;
-	case TFld::Real:	setR( atof(val.c_str()),forcView );	break;
-	case TFld::Boolean:	setB( atoi(val.c_str()),forcView );	break;
     }
 }
 
-void TCfg::setR( double val, bool forcView )
+void TCfg::setR( double val, char RqFlg )
 {
-    switch( m_fld->type() )
+    switch( mFld->type() )
     {
-	case TFld::String:	setS( TSYS::real2str(val),forcView );	break;
-	case TFld::Integer:	setI( (int)val, forcView );	break;
+	case TFld::String:	setS( TSYS::real2str(val), RqFlg );	break;
+	case TFld::Integer:	setI( (int)val, RqFlg );	break;
+	case TFld::Boolean:	setB( val, RqFlg );	break;
 	case TFld::Real:
 	{
-	    if( !(m_fld->flg()&TFld::Selected) && m_fld->selValR()[0] < m_fld->selValR()[1] )
-	    {
-		if( val < m_fld->selValR()[0] )	val = m_fld->selValR()[0];
-		if( val > m_fld->selValR()[1] )	val = m_fld->selValR()[1];
-	    }
+	    if( !(mFld->flg()&TFld::Selected) && mFld->selValR()[0] < mFld->selValR()[1] )
+		val = vmin(mFld->selValR()[1],vmax(mFld->selValR()[0],val));
 	    double t_val = m_val.r_val;
 	    m_val.r_val = val;
-	    if( !m_owner.cfgChange(*this) )	m_val.r_val = t_val;
-	    if( forcView ) setView(true);
+	    if( !mOwner.cfgChange(*this) )	m_val.r_val = t_val;
+	    if( RqFlg&TCfg::ForceUse )	{ setView(true); setKeyUse(true); }
 	    break;
 	}
-	case TFld::Boolean:	setB( val, forcView );	break;
     }
 }
 
-void TCfg::setI( int val, bool forcView )
+void TCfg::setI( int val, char RqFlg )
 {
-    switch( m_fld->type() )
+    switch( mFld->type() )
     {
-	case TFld::String:	setS( TSYS::int2str(val),forcView );	break;
+	case TFld::String:	setS( TSYS::int2str(val), RqFlg );	break;
+	case TFld::Real:	setR( val, RqFlg );	break;
+	case TFld::Boolean:	setB( val, RqFlg );	break;
 	case TFld::Integer:
 	{
-	    if( !(m_fld->flg()&TFld::Selected) && m_fld->selValI()[0] < m_fld->selValI()[1] )
-	    {
-		if( val < m_fld->selValI()[0] )	val = m_fld->selValI()[0];
-		if( val > m_fld->selValI()[1] )	val = m_fld->selValI()[1];
-	    }
+	    if( !(mFld->flg()&TFld::Selected) && mFld->selValI()[0] < mFld->selValI()[1] )
+		val = vmin(mFld->selValI()[1],vmax(mFld->selValI()[0],val));
 	    int t_val = m_val.i_val;
 	    m_val.i_val = val;
-	    if( !m_owner.cfgChange(*this) )	m_val.i_val = t_val;
-	    if( forcView ) setView(true);
+	    if( !mOwner.cfgChange(*this) )	m_val.i_val = t_val;
+	    if( RqFlg&TCfg::ForceUse )	{ setView(true); setKeyUse(true); }
 	    break;
 	}
-	case TFld::Real:	setR( val,forcView );	break;
-	case TFld::Boolean:	setB( val,forcView );	break;
     }
 }
 
-void TCfg::setB( bool val, bool forcView )
+void TCfg::setB( bool val, char RqFlg )
 {
-    switch( m_fld->type() )
+    switch( mFld->type() )
     {
-	case TFld::String:	setS( TSYS::int2str(val),forcView );	break;
-	case TFld::Integer:	setI( val,forcView );	break;
-	case TFld::Real:	setR( val,forcView );	break;
+	case TFld::String:	setS( TSYS::int2str(val), RqFlg );	break;
+	case TFld::Integer:	setI( val, RqFlg );	break;
+	case TFld::Real:	setR( val, RqFlg );	break;
 	case TFld::Boolean:
 	{
 	    bool t_val = m_val.b_val;
 	    m_val.b_val = val;
-	    if( !m_owner.cfgChange(*this) )	m_val.b_val = t_val;
-	    if( forcView ) setView(true);
+	    if( !mOwner.cfgChange(*this) )	m_val.b_val = t_val;
+	    if( RqFlg&TCfg::ForceUse )	{ setView(true); setKeyUse(true); }
 	    break;
 	}
     }
 }
 
-bool TCfg::operator==(TCfg & cfg)
+bool TCfg::operator==( TCfg &cfg )
 {
     switch(fld().type())
     {
