@@ -43,10 +43,15 @@ class TCfg
 	//Data
 	enum AttrFlg
 	{
+	    TransltText = 0x100,	//String value type: Use translation variable texts mechanism
 	    NoVal	= 0x200,	//No value mirrored
 	    Key		= 0x400,	//Primary key
 	    Hide	= 0x800		//Attribute hide
 	};
+	enum ReqFlg
+	{
+	    ForceUse	= 0x01,		//Force use flag
+	};
 
 	//Methods
 	TCfg( TFld &fld, TConfig &owner );
@@ -57,44 +62,50 @@ class TCfg
 	bool operator==( TCfg &cfg );
 	TCfg &operator=( TCfg &cfg );
 
-	bool	view( )			{ return m_view; }
-	void	setView( bool vw )	{ m_view = vw; }
+	bool	view( )			{ return mView; }
+	bool	keyUse( )		{ return mKeyUse; }
+	bool	noTransl( )		{ return mNoTransl; }
+	void	setView( bool vw )	{ mView = vw; }
+	void	setKeyUse( bool vl )	{ if( fld().flg()&Key ) mKeyUse = vl; }
+	void	setNoTransl( bool vl )	{ mNoTransl = vl; }
 
-	TFld &fld( )			{ return *m_fld; }
+	TFld &fld( )			{ return *mFld; }
 
-	//- Universal access -
-	string	getSEL( );
-	string	getS( );
-	double	getR( );
-	int	getI( );
-	bool	getB( );
-	//- Direct access -
+	//> Universal access
+	string	getSEL( char RqFlg = 0 );
+	string	getS( char RqFlg = 0 );
+	double	getR( char RqFlg = 0 );
+	int	getI( char RqFlg = 0 );
+	bool	getB( char RqFlg = 0 );
+	//> Direct access
 	string	&getSd( );
 	double	&getRd( );
 	int	&getId( );
 	bool	&getBd( );
 
-	void setSEL( const string &val, bool forcView = false );
-	void setS( const string &val, bool forcView = false );
-	void setR( double val, bool forcView = false );
-	void setI( int val, bool forcView = false );
-	void setB( bool val, bool forcView = false );
+	void setSEL( const string &val, char RqFlg = 0 );
+	void setS( const string &val, char RqFlg = 0 );
+	void setR( double val, char RqFlg = 0 );
+	void setI( int val, char RqFlg = 0 );
+	void setB( bool val, char RqFlg = 0 );
 
     private:
 	//Data
 	union
 	{
-	    string	*s_val;
+	    ResString	*s_val;
 	    double	r_val;
 	    int		i_val;
 	    bool	b_val;
 	}m_val;
 
 	//Attributes
-	bool m_view;
+	char mView	: 1;
+	char mKeyUse	: 1;
+	char mNoTransl	: 1;
 
-	TFld	*m_fld;
-	TConfig	&m_owner;
+	TFld	*mFld;
+	TConfig	&mOwner;
 };
 
 //*************************************************
@@ -116,7 +127,9 @@ class TConfig: public TValElem
 	void cfgList( vector<string> &list );
 	bool cfgPresent( const string &n_val );
 	TCfg &cfg( const string &n_val );
+	TCfg *at( const string &n_val, bool noExpt = false );
 	void cfgViewAll( bool val = true );	//Hide all no key elements
+	void cfgKeyUseAll( bool val );
 
 	TElem &elem( );
 	void setElem( TElem *Elements, bool first = false );
@@ -126,6 +139,9 @@ class TConfig: public TValElem
         void cntrCmdProc( XMLNode *fld, const string &elem,
 		const string &user = "root", const string &grp = "root", int perm = 0664 );
 
+	string lang2Code( )	{ return mLang2Code; }
+	void setLang2Code( const string &vl )	{ mLang2Code = vl; }
+
     protected:
 	//Methods
 	virtual bool cfgChange( TCfg &cfg )	{ return true; }
@@ -138,7 +154,8 @@ class TConfig: public TValElem
 	//Attributes
 	TCfgMap		value;
 	TElem		*m_elem;
-	bool		single;
+	char		single		: 1;
+	string		mLang2Code;
 };
 
 #endif // TCONFIG_H
