@@ -28,12 +28,11 @@
 //* TController					  *
 //*************************************************
 TController::TController( const string &id_c, const string &daq_db, TElem *cfgelem ) :
-    m_db(daq_db), TConfig(cfgelem), run_st(false), en_st(false),
-    m_id(cfg("ID").getSd()), m_name(cfg("NAME").getSd()), m_descr(cfg("DESCR").getSd()),
-    m_aen(cfg("ENABLE").getBd()), m_astart(cfg("START").getBd())
+    mDB(daq_db), TConfig(cfgelem), run_st(false), en_st(false), mRedntUse(false), mRedntFirst(true),
+    mId(cfg("ID").getSd()), mAEn(cfg("ENABLE").getBd()), mAStart(cfg("START").getBd())
 {
-    m_id = id_c;
-    m_prm = grpAdd("prm_");
+    mId = id_c;
+    mPrm = grpAdd("prm_");
 }
 
 TController::~TController(  )
@@ -46,13 +45,13 @@ TCntrNode &TController::operator=( TCntrNode &node )
     TController *src_n = dynamic_cast<TController*>(&node);
     if( !src_n ) return *this;
 
-    //- Configuration copy -
+    //> Configuration copy
     string tid = id();
     *(TConfig*)this = *(TConfig*)src_n;
-    m_id = tid;
-    setDB(src_n->m_db);
+    mId = tid;
+    setDB(src_n->mDB);
 
-    //- Parameters copy -
+    //> Parameters copy
     if( src_n->enableStat( ) )
     {
 	if( !enableStat( ) )	enable();
@@ -82,10 +81,10 @@ void TController::postDisable(int flag)
     {
 	if( flag )
 	{
-	    //- Delete DB record -
-	    SYS->db().at().dataDel(fullDB(),owner().nodePath()+"DAQ",*this);
+	    //> Delete DB record
+	    SYS->db().at().dataDel(fullDB(),owner().nodePath()+"DAQ",*this,true);
 
-	    //- Delete parameter's tables -
+	    //> Delete parameter's tables
 	    for(unsigned i_tp = 0; i_tp < owner().tpPrmSize(); i_tp++)
 	    {
 		string tbl = DB()+"."+cfg(owner().tpPrmAt(i_tp).db).getS();
@@ -97,18 +96,38 @@ void TController::postDisable(int flag)
     { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
 }
 
-string TController::name()
-{
-    return (m_name.size())?m_name:id();
-}
+TTipDAQ &TController::owner( )	{ return *(TTipDAQ*)nodePrev(); }
 
-string TController::tbl( )
+string TController::workId( )	{ return owner().modId()+"."+id(); }
+
+string TController::name( )	{ string nm = cfg("NAME").getS(); return nm.empty() ? id() : nm; }
+
+void TController::setName( const string &nm )		{ cfg("NAME").setS(nm);  }
+
+string TController::descr( )	{ return cfg("DESCR").getS(); }
+
+void TController::setDescr( const string &dscr )	{ cfg("DESCR").setS(dscr); }
+
+string TController::tbl( )	{ return owner().owner().subId()+"_"+owner().modId(); }
+
+string TController::getStatus( )
 {
-    return owner().owner().subId()+"_"+owner().modId();
+    string rez;
+    if( startStat() )
+    {
+	rez = _("Started. ");
+	if( owner().redntAllow( ) && redntUse( ) ) rez += _("Geting data from remote station. ");
+    }
+    else if( enableStat() ) rez = _("Enabled. ");
+    else rez = _("Disabled. ");
+
+    return rez;
 }
 
 void TController::load_( )
 {
+    if( !SYS->chkSelDB(DB()) ) return;
+
     mess_info(nodePath().c_str(),_("Load controller's configs!"));
 
     SYS->db().at().dataGet(fullDB(),owner().nodePath()+"DAQ",*this);
@@ -134,13 +153,16 @@ bool TController::cfgChange( TCfg &cfg )
 
 void TController::start( )
 {
-    //- Enable if no enabled -
+    //> Enable if no enabled
     if( run_st ) return;
     if( !en_st ) enable();
 
     mess_info(nodePath().c_str(),_("Start controller!"));
 
-    //- Start for children -
+    //> First archives synchronization
+    if( owner().redntAllow() && redntMode( ) != TController::Off ) redntDataUpdate(true);
+
+    //> Start for children
     start_();
 
     run_st = true;
@@ -225,25 +247,22 @@ void TController::LoadParmCfg(  )
 	    TConfig c_el(&owner().tpPrmAt(i_tp));
 	    c_el.cfgViewAll(false);
 
-	    int fld_cnt = 0;
-	    while( SYS->db().at().dataSeek(DB()+"."+cfg(owner().tpPrmAt(i_tp).db).getS(),
-					   owner().nodePath()+cfg(owner().tpPrmAt(i_tp).db).getS(),fld_cnt++,c_el) )
+	    for( int fld_cnt = 0; SYS->db().at().dataSeek(DB()+"."+cfg(owner().tpPrmAt(i_tp).db).getS(),
+					   owner().nodePath()+cfg(owner().tpPrmAt(i_tp).db).getS(),fld_cnt++,c_el); )
 	    {
 		try
 		{
-		    if( !present(c_el.cfg("SHIFR").getS()) ) 
-			add( c_el.cfg("SHIFR").getS(), i_tp ); 
+		    if( !present(c_el.cfg("SHIFR").getS()) ) add( c_el.cfg("SHIFR").getS(), i_tp );
 		}
 		catch(TError err)
 		{
-		    mess_err(err.cat.c_str(),"%s",err.mess.c_str()); 
+		    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
 		    mess_err(nodePath().c_str(),_("Add parameter <%s> error."),c_el.cfg("SHIFR").getS().c_str());
 		}
-		c_el.cfg("SHIFR").setS("");
 	    }
 	}catch(TError err)
 	{
-	    mess_err(err.cat.c_str(),"%s",err.mess.c_str()); 
+	    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
 	    mess_err(nodePath().c_str(),_("Search and create new parameters error."));
 	}
     }
@@ -260,8 +279,8 @@ void TController::LoadParmCfg(  )
 
 void TController::add( const string &name, unsigned type )
 {
-    if( chldPresent(m_prm,name) ) return;
-    chldAdd(m_prm,ParamAttach( name, type ));
+    if( chldPresent(mPrm,name) ) return;
+    chldAdd(mPrm,ParamAttach( name, type ));
 }
 
 TParamContr *TController::ParamAttach( const string &name, int type)
@@ -269,55 +288,140 @@ TParamContr *TController::ParamAttach( const string &name, int type)
     return new TParamContr(name, &owner().tpPrmAt(type));
 }
 
+TController::Redundant TController::redntMode( )	{ return (TController::Redundant)cfg("REDNT").getI(); }
+
+void TController::setRedntMode( Redundant vl )		{ cfg("REDNT").setI(vl); modif(); }
+
+string TController::redntRun( )				{ return cfg("REDNT_RUN").getS(); }
+
+void TController::setRedntRun( const string &vl )	{ cfg("REDNT_RUN").setS(vl); modif(); }
+
+void TController::setRedntUse( bool vl )
+{
+    if( mRedntUse == vl ) return;
+    mRedntUse = mRedntFirst = vl;
+}
+
+void TController::redntDataUpdate( bool firstArchiveSync )
+{
+    vector<string> pls;
+    list(pls);
+
+    //> Prepare group request to parameters
+    AutoHD<TParamContr> prm;
+    XMLNode req("CntrReqs"); req.setAttr("path",nodePath(0,true));
+    if( firstArchiveSync || mRedntFirst ) req.setAttr("all","1");
+    for( int i_p = 0; i_p < pls.size(); i_p++ )
+    {
+	prm = at(pls[i_p]);
+	if( !prm.at().enableStat( ) ) { pls.erase(pls.begin()+i_p); i_p--; continue; }
+	//>> Check attributes last present data time into archives
+	if( firstArchiveSync || mRedntFirst )
+	{
+	    prm.at().mRedntTmLast = 0;
+	    vector<string> listV;
+	    prm.at().vlList(listV);
+	    for( int iV = 0; iV < listV.size(); iV++ )
+	    {
+		AutoHD<TVal> vl = prm.at().vlAt(listV[iV]);
+		prm.at().mRedntTmLast = vmax(prm.at().mRedntTmLast,vl.at().time());
+		if( !vl.at().arch().freeStat() )
+		    prm.at().mRedntTmLast = vmax(prm.at().mRedntTmLast,vl.at().arch().at().end(""));
+	    }
+	    if( prm.at().mRedntTmLast )
+		prm.at().mRedntTmLast = vmax(prm.at().mRedntTmLast,TSYS::curTime()-(long long)(3.6e9*owner().owner().rdRestDtTm()));
+	    else prm.at().mRedntTmLast = TSYS::curTime();
+	}
+	req.childAdd("get")->setAttr("path","/prm_"+pls[i_p]+"/%2fserv%2fattr")->
+	    setAttr("tm",TSYS::ll2str(prm.at().mRedntTmLast));
+    }
+
+    //> Send request to first active station for this controller
+    try{ owner().owner().rdStRequest(workId(),req); }
+    catch(TError err) { return; }
+
+    //> Write data to parameters
+    for( int i_p = 0; i_p < pls.size(); i_p++ )
+    {
+	prm = at(pls[i_p]);
+	prm.at().mRedntTmLast = atoll(req.childGet(i_p)->attr("tm").c_str());
+	for( int i_a = 0; i_a < req.childGet(i_p)->childSize(); i_a++ )
+	{
+	    XMLNode *aNd = req.childGet(i_p)->childGet(i_a);
+	    if( !prm.at().vlPresent(aNd->attr("id")) ) continue;
+	    AutoHD<TVal> vl = prm.at().vlAt(aNd->attr("id"));
+
+	    if( aNd->attr("tm").empty() ) vl.at().setS(aNd->text(),prm.at().mRedntTmLast,true);
+	    else if( aNd->childSize() )
+	    {
+		long long btm = atoll(aNd->attr("tm").c_str());
+		long long per = atoll(aNd->attr("per").c_str());
+		if( !vl.at().arch().freeStat() )
+		{
+		    TValBuf buf(vl.at().arch().at().valType(),0,per,true,true);
+		    for( int i_v = 0; i_v < aNd->childSize(); i_v++ )
+			buf.setS(aNd->childGet(i_v)->text(),btm+per*i_v);
+		    vl.at().arch().at().setVals(buf,buf.begin(),buf.end(),"");
+		}
+		vl.at().setS(aNd->childGet(aNd->childSize()-1)->text(),btm+per*(aNd->childSize()-1),true);
+	    }
+	}
+    }
+
+    mRedntFirst = false;
+}
+
 void TController::cntrCmdProc( XMLNode *opt )
 {
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
+	TCntrNode::cntrCmdProc(opt);
 	ctrMkNode("oscada_cntr",opt,-1,"/",_("Controller: ")+name(),0664,"root","root");
 	ctrMkNode("branches",opt,-1,"/br","",0444);
 	if(ctrMkNode("area",opt,-1,"/cntr",_("Controller")))
 	{
 	    if(ctrMkNode("area",opt,-1,"/cntr/st",_("State")))
 	    {
+		ctrMkNode("fld",opt,-1,"/cntr/st/status",_("Status"),0444,"root","root",1,"tp","str");
 		ctrMkNode("fld",opt,-1,"/cntr/st/en_st",_("Enable"),0664,"root","root",1,"tp","bool");
 		ctrMkNode("fld",opt,-1,"/cntr/st/run_st",_("Run"),0664,"root","root",1,"tp","bool");
 		ctrMkNode("fld",opt,-1,"/cntr/st/db",_("Controller DB"),0664,"root","root",4,"tp","str","dest","select","select","/db/list",
 		    "help",_("DB address in format [<DB module>.<DB name>].\nFor use main work DB set '*.*'."));
 	    }
-	    if(ctrMkNode("area",opt,-1,"/cntr/cfg",_("Config")))
+	    if( ctrMkNode("area",opt,-1,"/cntr/cfg",_("Config")) )
+	    {
 		TConfig::cntrCmdMake(opt,"/cntr/cfg",0,"root","root",0664);
+		//>> Append configuration properties
+		XMLNode *xt = ctrId(opt->childGet(0),"/cntr/cfg/REDNT_RUN",true);
+		if( xt ) xt->setAttr("dest","select")->setAttr("select","/cntr/redRunLst");
+	    }
 	}
 	if( owner().tpPrmSize() )
 	{
-	    ctrMkNode("grp",opt,-1,"/br/prm_",_("Parameter"),0660,"root","root",1,"idm","1");
+	    ctrMkNode("grp",opt,-1,"/br/prm_",_("Parameter"),0660,"root","root",2,"idm","1","idSz","20");
 	    if(ctrMkNode("area",opt,-1,"/prm",_("Parameters")))
 	    {
 		if( owner().tpPrmSize() > 1 )
 		    ctrMkNode("fld",opt,-1,"/prm/t_prm",_("To add parameters"),0660,"root","root",3,"tp","str","dest","select","select","/prm/t_lst");
-		ctrMkNode("list",opt,-1,"/prm/prm",_("Parameters"),0660,"root","root",4,"tp","br","idm","1","s_com","add,del","br_pref","prm_");
-		ctrMkNode("fld",opt,-1,"/prm/nmb",_("Number"),0444,"root","root",1,"tp","dec");
-		ctrMkNode("fld",opt,-1,"/prm/en",_("Enabled"),0444,"root","root",1,"tp","dec");
+		ctrMkNode("fld",opt,-1,"/prm/nmb",_("Number"),0444,"root","root",1,"tp","str");
+		ctrMkNode("list",opt,-1,"/prm/prm",_("Parameters"),0660,"root","root",5,"tp","br","idm","1","s_com","add,del","br_pref","prm_","idSz","20");
 	    }
 	}
 	return;
     }
 
-    //- Process command to page -
+    //> Process command to page
     vector<string> c_list;
     string a_path = opt->attr("path");
-    if( a_path == "/prm/nmb" && ctrChkNode(opt) )
-    {
-	list(c_list);
-	opt->setText(TSYS::int2str(c_list.size()));
-    }
-    else if( a_path == "/prm/en" && ctrChkNode(opt) )
+    if( a_path == "/cntr/st/status" && ctrChkNode(opt) )	opt->setText(getStatus());
+    else if( a_path == "/prm/nmb" && ctrChkNode(opt) )
     {
 	list(c_list);
 	int e_c = 0;
 	for( int i_a = 0; i_a < c_list.size(); i_a++ )
 	    if( at(c_list[i_a]).at().enableStat( ) )	e_c++;
-	opt->setText(TSYS::int2str(e_c));
+	opt->setText(TSYS::strMess(_("All: %d; Enabled: %d"),c_list.size(),e_c));
     }
     else if( a_path == "/prm/t_prm" && owner().tpPrmSize() )
     {
@@ -336,8 +440,9 @@ void TController::cntrCmdProc( XMLNode *opt )
 	}
 	if( ctrChkNode(opt,"add",0660,"root","root",SEQ_WR) )
 	{
-	    add(opt->attr("id"),owner().tpPrmToId(TBDS::genDBGet(owner().nodePath()+"addType",owner().tpPrmAt(0).name,opt->attr("user"))));
-	    at(opt->attr("id")).at().setName(opt->text());
+	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::ID);
+	    add(vid,owner().tpPrmToId(TBDS::genDBGet(owner().nodePath()+"addType",owner().tpPrmAt(0).name,opt->attr("user"))));
+	    at(vid).at().setName(opt->text());
 	}
 	if( ctrChkNode(opt,"del",0660,"root","root",SEQ_WR) )	del(opt->attr("id"),true);
     }
@@ -369,5 +474,15 @@ void TController::cntrCmdProc( XMLNode *opt )
 		if( owner().tpPrmAt(i_t).db == TSYS::pathLev(a_path,2) )
 		     modifG( );
     }
+    else if( a_path == "/cntr/redRunLst" && ctrChkNode(opt) )
+    {
+	opt->childAdd("el")->setAttr("id","<high>")->setText(_("<High level>"));
+	opt->childAdd("el")->setAttr("id","<low>")->setText(_("<Low level>"));
+	opt->childAdd("el")->setAttr("id","<optimal>")->setText(_("<Optimal>"));
+	vector<string> sls;
+	owner().owner().rdStList(sls);
+	for( int i_s = 0; i_s < sls.size(); i_s++ )
+	    opt->childAdd("el")->setAttr("id",sls[i_s])->setText(SYS->transport().at().extHostGet("*",sls[i_s]).name);
+    }
     else TCntrNode::cntrCmdProc(opt);
 }
