@@ -25,6 +25,8 @@
 #include <string>
 #include <vector>
 
+#include "resalloc.h"
+
 using std::string;
 using std::vector;
 
@@ -50,9 +52,10 @@ class TFld
 	    Selected	= 0x01,		//Connnect to simple elements
 	    SelfFld	= 0x02,		//Create self field
 	    NoWrite	= 0x04,		//No writeable
-	    HexDec	= 0x08,		//Hexodecimal view prefer for decimal value type
-	    OctDec	= 0x10,		//Octal view prefer for decimal value type
-	    FullText	= 0x08		//Full, multiline text for string type
+	    HexDec	= 0x08,		//Decimal value type: hexodecimal view prefer
+	    OctDec	= 0x10,		//Decimal value type: octal view prefer
+	    DateTimeDec	= 0x20,		//Decimal value type: UTC date seconds counter
+	    FullText	= 0x08		//String value type: full, multiline text
 	};
 
 	//Methods
@@ -152,14 +155,15 @@ class TElem
 	void fldDel( unsigned int id );
 	TFld &fldAt( unsigned int id );
 
-	void valAtt( TValElem *cnt ); 
-	void valDet( TValElem *cnt ); 
+	void valAtt( TValElem *cnt );
+	void valDet( TValElem *cnt );
 
     private:
 	//Attributes
 	string			m_name;
 	vector<TFld*>		elem;
 	vector<TValElem*>	cont;		//Conteiners
+	Res			mResEl;
 };
 
 //*************************************************
