@@ -46,7 +46,8 @@ class IO
 	{
 	    Default = 0x00,	//Default mode (Input IO)
 	    Output  = 0x01,
-	    Return  = 0x02
+	    Return  = 0x02,
+	    FullText= 0x04
 	};
 
 	//Methods
@@ -55,13 +56,13 @@ class IO
 
 	IO &operator=( IO &iio );
 
-	const string &id( )	{ return m_id; }
-	const string &name( )	{ return m_name; }
-	const Type &type( )	{ return m_type; }
-	unsigned flg( )		{ return m_flg; }
-	const string &def( )	{ return m_def; }
-	bool  hide( )		{ return m_hide; }
-	const string &rez( )	{ return m_rez; }
+	const string &id( )	{ return mId; }
+	const string &name( )	{ return mName; }
+	const Type &type( )	{ return mType; }
+	unsigned flg( )		{ return mFlg; }
+	const string &def( )	{ return mDef; }
+	bool  hide( )		{ return mHide; }
+	const string &rez( )	{ return mRez; }
 
 	void setId( const string &val );
 	void setName( const string &val );
@@ -73,13 +74,13 @@ class IO
 
     private:
 	//Attributes
-	string	m_id;
-	string	m_name;
-	Type	m_type;
-	unsigned m_flg;
-	string	m_def;
-	bool	m_hide;
-	string	m_rez;
+	string	mId;
+	string	mName;
+	Type	mType;
+	unsigned mFlg;
+	string	mDef;
+	bool	mHide;
+	string	mRez;
 
 	TFunction *owner;
 };
@@ -98,10 +99,11 @@ class TFunction : public TCntrNode
 
 	TFunction &operator=( TFunction &func );
 
-	const string &id( )		{ return m_id; };
+	const string &id( )		{ return mId; };
 	virtual string name( )		{ return ""; }
 	virtual string descr( )		{ return ""; }
 	bool startStat( )		{ return run_st; }
+	int use( )			{ return used.size(); }
 
 	virtual void setStart( bool val )  { run_st = val; }
 
@@ -111,7 +113,7 @@ class TFunction : public TCntrNode
 	int ioSize( );
 	IO *io( int id );
 	void ioAdd( IO *io );
-	void ioIns( IO *io, int pos );
+	int ioIns( IO *io, int pos );
 	void ioDel( int pos );
 	void ioMove( int pos, int to );
 
@@ -130,9 +132,9 @@ class TFunction : public TCntrNode
 	void preDisable( int flag );
 
 	//Attributes
-	string		m_id;
+	string		mId;
 	bool		run_st;
-	TValFunc	*m_tval;
+	TValFunc	*mTVal;
 
     private:
 	//Methods
@@ -140,7 +142,7 @@ class TFunction : public TCntrNode
 
 	//Attributes
 	Res		f_res;
-	vector<IO*>	m_io;
+	vector<IO*>	mIO;
 	vector<TValFunc*>	used;
 };
 
@@ -151,11 +153,14 @@ class TValFunc
 {
     public:
 	//Methods
-	TValFunc( const string &iname = "", TFunction *ifunc = NULL, bool iblk = true );
+	TValFunc( const string &iname = "", TFunction *ifunc = NULL, bool iblk = true, const string &iuser = "root" );
 	virtual ~TValFunc( );
 
-	const string &vfName( )			{ return m_name; }
-	void setVfName( const string &inm )	{ m_name = inm; }
+	string user( )				{ return mUser; }
+	const string &vfName( )			{ return mName; }
+
+	void setUser( const string &iuser )	{ mUser = iuser; }
+	void setVfName( const string &inm )	{ mName = inm; }
 
 	void ioList( vector<string> &list );
 	int  ioId( const string &id );
@@ -163,17 +168,17 @@ class TValFunc
 	IO::Type ioType( unsigned id )
 	{
 	    if( id >= m_val.size() )    throw TError("ValFunc",_("Id or IO %d error!"),id);
-	    return m_func->io(id)->type();
+	    return mFunc->io(id)->type();
 	}
 	unsigned ioFlg( unsigned id )
 	{
 	    if( id >= m_val.size() )    throw TError("ValFunc",_("Id or IO %d error!"),id);
-	    return m_func->io(id)->flg();
+	    return mFunc->io(id)->flg();
 	}
 	bool ioHide( unsigned id )
 	{
 	    if( id >= m_val.size() )    throw TError("ValFunc",_("Id or IO %d error!"),id);
-	    return m_func->io(id)->hide();
+	    return mFunc->io(id)->hide();
 	}
 
 	//- get IO value -
@@ -189,20 +194,20 @@ class TValFunc
 	void setB( unsigned id, char val );
 
 	//- Blocked values screen -
-	bool	blk( )			{ return m_blk; }
+	bool	blk( )			{ return mBlk; }
 
 	//- Dimension controll -
-	bool	dimens( )		{ return m_dimens; }
-	void	setDimens( bool set )	{ m_dimens = set; }
+	bool	dimens( )		{ return mDimens; }
+	void	setDimens( bool set )	{ mDimens = set; }
 
 	//- Calc function -
-	virtual void calc( );
+	virtual void calc( const string &user = "" );
 	//- Calc time function -
 	double  calcTm( )		{ return tm_calc; }
 	void setCalcTm( double ivl )	{ tm_calc = ivl; }
 
 	//- Attached function -
-	TFunction *func( )		{ return m_func; }
+	TFunction *func( )		{ return mFunc; }
 	void setFunc( TFunction *func, bool att_det = true );
 
 	virtual void preIOCfgChange( );
@@ -215,10 +220,10 @@ class TValFunc
 	    IO::Type	tp;
 	    union
 	    {
-	        string *s;
-	        double r;
-	        int    i;
-	        char   b;
+		ResString *s;
+		double r;
+		int    i;
+		char   b;
 	    }val;
 	};
 
@@ -230,13 +235,14 @@ class TValFunc
 	void funcDisConnect( bool det = true );
 
 	//Attributes
-	string	m_name;		//Value name
-	bool	m_blk,		//Blocked values screen
-		m_dimens;	//Make dimension of the calc time
+	string	mName,		//Value name
+		mUser;		//Link to user
+	bool	mBlk,		//Blocked values screen
+		mDimens;	//Make dimension of the calc time
 
 	double tm_calc;		//Calc time in mikroseconds
 
-	TFunction	*m_func;
+	TFunction	*mFunc;
 };
 
 #endif //TFUNCTIONS_H
