@@ -43,13 +43,17 @@
 //*************************************************
 TMess::TMess(  ) : IOCharSet("UTF-8"), m_mess_level(0), log_dir(0x2)
 {
+    setenv("LC_NUMERIC","C",1);
     openlog(PACKAGE,0,LOG_USER);
     setlocale(LC_ALL,"");
-    setlocale(LC_NUMERIC,"C");	//For true float type conversion
     IOCharSet = nl_langinfo(CODESET);
 
     bindtextdomain(PACKAGE,LOCALEDIR);
     textdomain(PACKAGE);
+
+    mLang2Code = lang();
+    if( mLang2Code.size() < 2 || mLang2Code == "POSIX" || mLang2Code == "C" ) mLang2Code = "en";
+    else mLang2Code = mLang2Code.substr(0,2);
 }
 
 TMess::~TMess(  )
@@ -98,7 +102,11 @@ void TMess::put( const char *categ, Type level, const char *fmt,  ... )
     if(log_dir&1) syslog(level_sys,TSYS::strEncode(s_mess,TSYS::FormatPrint).c_str());
     if(log_dir&2) fprintf(stdout,"%s \n",s_mess.c_str());
     if(log_dir&4) fprintf(stderr,"%s \n",s_mess.c_str());
-    if((log_dir&8) && SYS->present("Archive") )	SYS->archive().at().messPut(time(NULL),categ,level,mess);
+    if((log_dir&8) && SYS->present("Archive") )
+    {
+	long long ctm = TSYS::curTime();
+	SYS->archive().at().messPut(ctm/1000000,ctm%1000000,categ,level,mess);
+    }
 }
 
 void TMess::get( time_t b_tm, time_t e_tm, vector<TMess::SRec> & recs, const string &category, Type level )
@@ -108,25 +116,31 @@ void TMess::get( time_t b_tm, time_t e_tm, vector<TMess::SRec> & recs, const str
 
 string TMess::lang( )
 {
-    if( getenv("LANGUAGE") )		return getenv("LANGUAGE");    
-    else if( getenv("LC_MESSAGES") )	return getenv("LC_MESSAGES");
-    else if( getenv("LANG") )		return getenv("LANG");
-    else return("C");
+    char *lng = NULL;
+    if( ((lng=getenv("LANGUAGE")) && strlen(lng)) ||
+	    ((lng=getenv("LC_MESSAGES")) && strlen(lng)) ||
+	    ((lng=getenv("LANG")) && strlen(lng)) )
+	return lng;
+    else return "C";
 }
 
 void TMess::setLang( const string &lng )
 {
-    if( getenv("LANGUAGE") ) setenv("LANGUAGE", lng.c_str(), 1);
-    else setenv("LC_MESSAGES", lng.c_str(), 1);
+    char *prvLng = NULL;
+    if( (prvLng=getenv("LANGUAGE")) && strlen(prvLng) ) setenv( "LANGUAGE", lng.c_str(), 1 );
+    else setenv( "LC_MESSAGES", lng.c_str(), 1 );
     setlocale(LC_ALL,"");
 
     IOCharSet = nl_langinfo(CODESET);
-    setlocale(LC_NUMERIC,"C");
+
+    mLang2Code = lang();
+    if( mLang2Code.size() < 2 || mLang2Code == "POSIX" || mLang2Code == "C" ) mLang2Code = "en";
+    else mLang2Code = mLang2Code.substr(0,2);
 }
 
 string TMess::codeConv( const string &fromCH, const string &toCH, const string &mess)
 {
-    //- Make convert to blocks 100 bytes !!! -
+    //> Make convert to blocks 100 bytes !!!
     string buf = "";
     char   *ibuf, outbuf[100], *obuf;
     size_t ilen, olen;
@@ -169,6 +183,15 @@ const char *TMess::I18N( const char *mess, const char *d_name )
     return dgettext(d_name, mess);
 }
 
+void TMess::setLang2CodeBase( const string &vl )
+{
+    mLang2CodeBase = vl;
+    if( mLang2CodeBase.size() < 2 || mLang2CodeBase == "POSIX" || mLang2CodeBase == "C" ) mLang2CodeBase = "en";
+    else mLang2CodeBase = mLang2CodeBase.substr(0,2);
+
+    SYS->modif();
+}
+
 bool TMess::chkPattern( const string &val, const string &patt )
 {
     bool mult_s = false;
@@ -189,7 +212,7 @@ bool TMess::chkPattern( const string &val, const string &patt )
 	}
 	else
 	{
-	    if( mult_s ) 
+	    if( mult_s )
 	    {
 		if( v_bck >= 0 ) { v_cnt = v_bck; p_cnt = p_bck; v_bck = -1; }
 		else v_cnt++;
@@ -226,14 +249,16 @@ void TMess::load()
 	}
     } while(next_opt != -1);
 
-    //- Load params config file -
-    i = atoi(TBDS::genDBGet(SYS->nodePath()+"MessLev",TSYS::int2str(messLevel()),"root",SYS->sysOptCfg()).c_str());
+    //> Load params config file
+    i = atoi(TBDS::genDBGet(SYS->nodePath()+"MessLev",TSYS::int2str(messLevel())).c_str());
     if( i >= 0 && i <= 7 ) setMessLevel(i);
-    setLogDirect(atoi(TBDS::genDBGet(SYS->nodePath()+"LogTarget",TSYS::int2str(logDirect()),"root",SYS->sysOptCfg()).c_str()));
+    setLogDirect(atoi(TBDS::genDBGet(SYS->nodePath()+"LogTarget",TSYS::int2str(logDirect())).c_str()));
+    mLang2CodeBase = TBDS::genDBGet(SYS->nodePath()+"Lang2CodeBase",mLang2CodeBase);
 }
 
 void TMess::save()
 {
     TBDS::genDBSet(SYS->nodePath()+"MessLev",TSYS::int2str(messLevel()));
     TBDS::genDBSet(SYS->nodePath()+"LogTarget",TSYS::int2str(logDirect()));
+    TBDS::genDBSet(SYS->nodePath()+"Lang2CodeBase",mLang2CodeBase);
 }
