@@ -28,7 +28,10 @@
 #include <vector>
 #include <exception>
 
+#include "tconfig.h"
+
 #define _(mess) Mess->I18N(mess)
+#define FTM(rec) ((long long)rec.time*1000000 + rec.utime)
 #define message(cat,lev,fmt,args...) Mess->put(cat,lev,fmt,##args)
 #define mess_debug(cat,fmt,args...) Mess->put(cat,TMess::Debug,fmt,##args)
 #define mess_info(cat,fmt,args...) Mess->put(cat,TMess::Info,fmt,##args)
@@ -55,14 +58,15 @@ class TMess
 	class SRec
 	{
 	    public:
-		SRec() : time(0), categ(""), level(TMess::Debug), mess("") { }
-		SRec(time_t itm, const string &icat, Type ilev, const string &imess) :
-		    time(itm), categ(icat), level(ilev), mess(imess) { }
+		SRec( ) : time(0), categ(""), level(TMess::Debug), mess("") { }
+		SRec( time_t itm, int iutime, const string &icat, Type ilev, const string &imess ) :
+		    time(itm), utime(iutime), categ(icat), level(ilev), mess(imess) { }
 
-		time_t time;
-		string categ;
-		Type   level;
-		string mess;
+		time_t	time;
+		int	utime;
+		string	categ;
+		Type	level;
+		string	mess;
 	};
 
 	//Methods
@@ -85,11 +89,14 @@ class TMess
 	static bool chkPattern( const string &val, const string &patern );
 
 	string lang( );
+	string lang2Code( )	{ return mLang2Code; }
+	string lang2CodeBase( )	{ return mLang2CodeBase; }
 	string &charset( )	{ return IOCharSet; }
 	int logDirect( )	{ return log_dir; }
 	int messLevel( )	{ return m_mess_level; }
 
 	void setLang( const string &lang );
+	void setLang2CodeBase( const string &vl );
 	void setLogDirect( int dir );
 	void setMessLevel( int level );
 
@@ -97,10 +104,15 @@ class TMess
 	void get( time_t b_tm, time_t e_tm, vector<TMess::SRec> & recs, const string &category = "", Type level = Debug );
 
     private:
+	//Methods
+	void varTextActivate( );
+
 	//Attributes
 	string	IOCharSet;		//Internal charset
 	int	m_mess_level;		//Work messages level
 	int	log_dir;		//Log direction
+
+	string	mLang2CodeBase, mLang2Code;
 };
 
 extern TMess *Mess;
