@@ -1,7 +1,7 @@
 
 //OpenSCADA system file: tmodschedul.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2008 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -27,6 +27,7 @@
 #include <stdlib.h>
 #include <signal.h>
 #include <getopt.h>
+#include <string.h>
 
 #include <string>
 
@@ -36,27 +37,15 @@
 //*************************************************
 //* TModSchedul                                   *
 //*************************************************
-TModSchedul::TModSchedul( ) : 
-    TSubSYS("ModSched","Modules sheduler",false), prc_st(false), m_mod_path("./"), m_per(10)
+TModSchedul::TModSchedul( ) : TSubSYS("ModSched","Modules sheduler",false), mPer(10), mAllow("*")
 {
-    //- Create calc timer -
-    struct sigevent sigev;
-    sigev.sigev_notify = SIGEV_THREAD;
-    sigev.sigev_value.sival_ptr = this;
-    sigev.sigev_notify_function = SchedTask;
-    sigev.sigev_notify_attributes = NULL;
-    timer_create(CLOCK_REALTIME,&sigev,&tmId);
-}
 
-TModSchedul::~TModSchedul(  )
-{
-    timer_delete(tmId);
 }
 
 void TModSchedul::preDisable(int flag)
 {
-    //- Detach all share libs -
-    ResAlloc res(hd_res,true);
+    //> Detach all share libs
+    ResAlloc res(nodeRes(),true);
     for( unsigned i_sh = 0; i_sh < SchHD.size(); i_sh++ )
 	if( SchHD[i_sh].hd )
 	{
@@ -79,77 +68,23 @@ string TModSchedul::optDescr( )
 	"    --ModPath=<path>   Modules <path> (/var/os/modules/).\n"
 	"------------ Parameters of section <%s> in config file -----------\n"
 	"ModPath  <path>        Path to shared libraries(modules).\n"
-	"ModAuto  <list>        List of automatic loaded, attached and started shared libraries (direct_dbf.so;virt.so).\n"
+	"ModAllow <list>        List of shared libraries allowed for automatic loading, attaching and starting (bd_DBF.so;daq_JavaLikeCalc.so).\n"
+	"                       Use '*' value for allow all modules.\n"
+	"ModDeny  <list>        List of shared libraries deny for automatic loading, attaching and starting (bd_DBF.so;daq_JavaLikeCalc.so).\n"
 	"ChkPer   <sec>         Period of checking at new shared libraries(modules).\n\n"
 	),nodePath().c_str());
 
     return(buf);
 }
 
-void TModSchedul::subStart(  )
-{ 
-#if OSC_DEBUG
-    mess_debug(nodePath().c_str(),_("Start subsystem."));
-#endif
-
-    //- Start interval timer for periodic thread creating -
-    struct itimerspec itval;
-    itval.it_interval.tv_sec = itval.it_value.tv_sec = chkPer();
-    itval.it_interval.tv_nsec = itval.it_value.tv_nsec = 0;
-    timer_settime(tmId, 0, &itval, NULL);
-}
-
-void TModSchedul::subStop(  )
-{
-#if OSC_DEBUG
-    mess_debug(nodePath().c_str(),_("Stop subsystem."));
-#endif
-
-    //- Stop interval timer for periodic thread creating -
-    struct itimerspec itval;
-    itval.it_interval.tv_sec = itval.it_interval.tv_nsec =
-	itval.it_value.tv_sec = itval.it_value.tv_nsec = 0;
-    timer_settime(tmId, 0, &itval, NULL);
-    if( TSYS::eventWait( prc_st, false, nodePath()+"stop",20) )
-	throw TError(nodePath().c_str(),_("Module scheduler thread no stoped!"));
-
-#if OSC_DEBUG
-    mess_debug(nodePath().c_str(),_("Stop subsystem. OK"));
-#endif
-}
-
-void TModSchedul::setChkPer( int per )
-{
-    m_per = per;
-    struct itimerspec itval;
-    itval.it_interval.tv_sec = itval.it_value.tv_sec = chkPer();
-    itval.it_interval.tv_nsec = itval.it_value.tv_nsec = 0;
-    timer_settime(tmId, 0, &itval, NULL);
-    modif();
-}
-
-void TModSchedul::SchedTask(union sigval obj)
-{
-    TModSchedul  *shed = (TModSchedul *)obj.sival_ptr;
-    if( shed->prc_st )  return;
-    shed->prc_st = true;
-
-    try
-    {
-	shed->libLoad(shed->modPath(),true);
-    } catch(TError err){ mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
-
-    shed->prc_st = false;
-}
-
 void TModSchedul::loadLibS(  )
 {
-    libLoad(modPath(),false);
+    libLoad(SYS->modDir(),false);
 }
 
 void TModSchedul::load_( )
 {
-    //- Load parameters from command line -
+    //> Load parameters from command line
     int next_opt;
     const char *short_opt="h";
     struct option long_opt[] =
@@ -165,31 +100,22 @@ void TModSchedul::load_( )
 	next_opt=getopt_long(SYS->argc,( char *const * ) SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
-	    case 'm': setModPath(optarg); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
+	    case 'm': SYS->setModDir(optarg); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
 
-    //- Load parameters from command line -
-    setChkPer( atoi(TBDS::genDBGet(nodePath()+"ChkPer",TSYS::int2str(m_per)).c_str()) );
-    setModPath( TBDS::genDBGet(nodePath()+"ModPath",m_mod_path) );
-
-    string opt = TBDS::genDBGet(nodePath()+"ModAuto");
-    string ovl;
-    m_am_list.clear();
-    for( int el_off = 0; (ovl=TSYS::strSepParse(opt,0,';',&el_off)).size(); )
-        m_am_list.push_back(ovl);
+    //> Load parameters from command line
+    setChkPer( atoi(TBDS::genDBGet(nodePath()+"ChkPer",TSYS::int2str(chkPer())).c_str()) );
+    SYS->setModDir( TBDS::genDBGet(nodePath()+"ModPath",SYS->modDir()) );
+    setAllowList(TBDS::genDBGet(nodePath()+"ModAllow",allowList(),"root",true));
+    setDenyList(TBDS::genDBGet(nodePath()+"ModDeny",denyList(),"root",true));
 }
 
 void TModSchedul::save_( )
 {
     TBDS::genDBSet(nodePath()+"ChkPer",TSYS::int2str(chkPer()));
-    TBDS::genDBSet(nodePath()+"ModPath",modPath());
-    string m_auto;
-    for(int i_a = 0; i_a < m_am_list.size(); i_a++ )
-	m_auto+=m_am_list[i_a]+";";
-    TBDS::genDBSet(nodePath()+"ModAuto",m_auto);
 }
 
 void TModSchedul::ScanDir( const string &Paths, vector<string> &files )
@@ -213,8 +139,8 @@ void TModSchedul::ScanDir( const string &Paths, vector<string> &files )
 	while((scan_dirent = readdir(IdDir)) != NULL)
 	{
 	    if( string("..") == scan_dirent->d_name || string(".") == scan_dirent->d_name ) continue;
-	    NameMod=Path+"/"+scan_dirent->d_name;
-	    if( CheckFile(NameMod) ) files.push_back(NameMod); 
+	    NameMod = Path+"/"+scan_dirent->d_name;
+	    if( CheckFile(NameMod) ) files.push_back(NameMod);
 	}
 	closedir(IdDir);
 
@@ -228,21 +154,22 @@ bool TModSchedul::CheckFile( const string &iname )
 
     stat(iname.c_str(),&file_stat);
 
-    if( (file_stat.st_mode&S_IFMT) != S_IFREG ) return false;
+    if( /*iname.size() <= 3 || iname.substr(iname.size()-3,3) != ".so" ||*/ (file_stat.st_mode&S_IFMT) != S_IFREG ) return false;
     if( access(iname.c_str(),F_OK|R_OK) != 0 )  return false;
     NameMod=iname;
 
     void *h_lib = dlopen(iname.c_str(),RTLD_LAZY|RTLD_GLOBAL);
-    if(h_lib == NULL)
+    if( h_lib == NULL )
     {
-	mess_warning(nodePath().c_str(),_("SO <%s> error: %s !"),iname.c_str(),dlerror());
-	return(false);
+	//mess_warning(nodePath().c_str(),_("Module <%s> error: %s !"),iname.c_str(),dlerror());
+	return false;
     }
     else dlclose(h_lib);
 
-    for(unsigned i_sh=0; i_sh < SchHD.size(); i_sh++)
-	if(SchHD[i_sh].name == iname )
-	    if(file_stat.st_mtime > SchHD[i_sh].m_tm) return true;
+    ResAlloc res(nodeRes(),false);
+    for( int i_sh=0; i_sh < SchHD.size(); i_sh++ )
+	if( SchHD[i_sh].name == iname )
+	    if(file_stat.st_mtime > SchHD[i_sh].tm) return true;
 	    else return false;
 
     return true;
@@ -252,20 +179,20 @@ int TModSchedul::libReg( const string &name )
 {
     struct stat file_stat;
 
-    ResAlloc res(hd_res,true);
+    ResAlloc res(nodeRes(),true);
     stat(name.c_str(),&file_stat);
     unsigned i_sh;
     for( i_sh = 0; i_sh < SchHD.size(); i_sh++ )
 	if( SchHD[i_sh].name == name ) break;
-    if( i_sh == SchHD.size() )	SchHD.push_back( SHD(NULL,file_stat.st_mtime,name) );
-    else SchHD[i_sh].m_tm = file_stat.st_mtime;
+    if( i_sh >= SchHD.size() )	SchHD.push_back( SHD(NULL,file_stat.st_mtime,name) );
+    else SchHD[i_sh].tm = file_stat.st_mtime;
 
     return i_sh;
 }
 
 void TModSchedul::libUnreg( const string &iname )
 {
-    ResAlloc res(hd_res,true);
+    ResAlloc res(nodeRes(),true);
     for(unsigned i_sh = 0; i_sh < SchHD.size(); i_sh++)
 	if( SchHD[i_sh].name == iname ) 
 	{
@@ -273,23 +200,23 @@ void TModSchedul::libUnreg( const string &iname )
 	    SchHD.erase(SchHD.begin()+i_sh);
 	    return;
 	}
-    throw TError(nodePath().c_str(),_("SO <%s> no present!"),iname.c_str());
+    throw TError(nodePath().c_str(),_("SO <%s> is not present!"),iname.c_str());
 }
 
 void TModSchedul::libAtt( const string &iname, bool full )
 {
-    ResAlloc res(hd_res,true);
-    for(unsigned i_sh = 0; i_sh < SchHD.size(); i_sh++)
-	if( SchHD[i_sh].name == iname ) 
+    ResAlloc res(nodeRes(),true);
+    for( int i_sh = 0; i_sh < SchHD.size(); i_sh++ )
+	if( SchHD[i_sh].name == iname )
 	{
-	    if( SchHD[i_sh].hd ) 
-		throw TError(nodePath().c_str(),_("SO <%s> already attached!"),iname.c_str());
+	    if( SchHD[i_sh].hd )
+		throw TError(nodePath().c_str(),_("SO <%s> is already attached!"),iname.c_str());
 
 	    void *h_lib = dlopen(iname.c_str(),RTLD_LAZY|RTLD_GLOBAL);
 	    if( !h_lib )
 		throw TError(nodePath().c_str(),_("SO <%s> error: %s !"),iname.c_str(),dlerror());
 
-	    //- Connect to module function -
+	    //> Connect to module function
 	    TModule::SAt (*module)( int );
 	    module = (TModule::SAt (*)(int)) dlsym(h_lib,"module");
 	    if( dlerror() != NULL )
@@ -298,7 +225,7 @@ void TModSchedul::libAtt( const string &iname, bool full )
 		throw TError(nodePath().c_str(),_("SO <%s> error: %s !"),iname.c_str(),dlerror());
 	    }
 
-	    //- Connect to attach function -
+	    //> Connect to attach function
 	    TModule *(*attach)( const TModule::SAt &, const string & );
 	    attach = (TModule * (*)(const TModule::SAt &, const string &)) dlsym(h_lib,"attach");
 	    if( dlerror() != NULL )
@@ -324,13 +251,13 @@ void TModSchedul::libAtt( const string &iname, bool full )
 			//-- Check type module version --
 			if( AtMod.t_ver != owner().at(list[i_sub]).at().subVer() )
 			{
-			    mess_warning(nodePath().c_str(),_("%s for type <%s> no support module version: %d!"),
+			    mess_warning(nodePath().c_str(),_("%s for type <%s> doesn't support module version: %d!"),
 				AtMod.id.c_str(),AtMod.type.c_str(),AtMod.t_ver);
 			    break;
 			}
 			//-- Check module present --
 			if( owner().at(list[i_sub]).at().modPresent(AtMod.id) )
-			    mess_warning(nodePath().c_str(),_("Module <%s> already present!"),AtMod.id.c_str());
+			    mess_warning(nodePath().c_str(),_("Module <%s> is already present!"),AtMod.id.c_str());
 			else
 			{
 			    //-- Attach new module --
@@ -358,14 +285,13 @@ void TModSchedul::libAtt( const string &iname, bool full )
 	    else SchHD[i_sh].hd = h_lib;
 	    return;
 	}
-    throw TError(nodePath().c_str(),_("SO <%s> no present!"),iname.c_str());
+    throw TError(nodePath().c_str(),_("SO <%s> is not present!"),iname.c_str());
 }
 
 void TModSchedul::libDet( const string &iname )
 {
-    ResAlloc res(hd_res,true);
-    for(unsigned i_sh = 0; i_sh < SchHD.size(); i_sh++)
-    {
+    ResAlloc res(nodeRes(),true);
+    for( int i_sh = 0; i_sh < SchHD.size(); i_sh++ )
 	if( SchHD[i_sh].name == iname && SchHD[i_sh].hd )
 	{
 	    while( SchHD[i_sh].use.size() )
@@ -389,23 +315,32 @@ void TModSchedul::libDet( const string &iname )
 	    SchHD[i_sh].hd = NULL;
 	    return;
 	}
-    }
-    throw TError(nodePath().c_str(),_("SO <%s> no present!"),iname.c_str());
+    throw TError(nodePath().c_str(),_("SO <%s> is not present!"),iname.c_str());
 }
 
-bool TModSchedul::CheckAuto( const string &name ) const
+bool TModSchedul::chkAllowMod( const string &name )
 {
-    if( m_am_list.size() == 1 && m_am_list[0] == "*") return(true);
-    else
-	for( unsigned i_au = 0; i_au < m_am_list.size(); i_au++)
-	    if( name == m_am_list[i_au] ) return true;
+    string sel, nmFile;
+    int path_els = 0;
 
-    return false;
+    for( int off = 0; (sel=TSYS::pathLev(name,0,true,&off)).size(); path_els++ ) nmFile = sel;
+
+    if( allowList().empty() )	return false;
+    if( TSYS::strNoSpace(allowList()) != "*" )
+    {
+	for( int off = 0; (sel=TSYS::strSepParse(allowList(),0,';',&off)).size(); )
+	    if( TSYS::strNoSpace(sel) == name || TSYS::strNoSpace(sel) == nmFile ) break;
+	if( sel.empty() ) return false;
+    }
+    for( int off = 0; (sel=TSYS::strSepParse(denyList(),0,';',&off)).size(); )
+	if( TSYS::strNoSpace(sel) == name || TSYS::strNoSpace(sel) == nmFile ) return false;
+
+    return true;
 }
 
 void TModSchedul::libList( vector<string> &list )
 {
-    ResAlloc res(hd_res,false);
+    ResAlloc res(nodeRes(),false);
     list.clear();
     for(unsigned i_sh = 0; i_sh < SchHD.size(); i_sh++)
 	list.push_back( SchHD[i_sh].name );
@@ -413,26 +348,27 @@ void TModSchedul::libList( vector<string> &list )
 
 TModSchedul::SHD &TModSchedul::lib( const string &iname )
 {
-    ResAlloc res(hd_res,false);
+    ResAlloc res(nodeRes(),false);
     //string nm_t = SYS->fNameFix(name);
     for(unsigned i_sh = 0; i_sh < SchHD.size(); i_sh++)
-	if( SchHD[i_sh].name == iname ) 
+	if( SchHD[i_sh].name == iname )
 	    return SchHD[i_sh];
-    throw TError(nodePath().c_str(),_("SO <%s> no present!"),iname.c_str());
+    throw TError(nodePath().c_str(),_("SO <%s> is not present!"),iname.c_str());
 }
 
-void TModSchedul::libLoad( const string &iname, bool full)
+void TModSchedul::libLoad( const string &iname, bool full )
 {
     vector<string> files;
 
     ScanDir( iname, files );
-    for(unsigned i_f = 0; i_f < files.size(); i_f++)
+    for( int i_f = 0; i_f < files.size(); i_f++ )
     {
-	unsigned i_sh;
-	bool st_auto = CheckAuto(files[i_f]);
+	int i_sh;
+	bool st_auto = chkAllowMod(files[i_f]);
+	ResAlloc res(nodeRes(),false);
 	for( i_sh = 0; i_sh < SchHD.size(); i_sh++ )
 	    if( SchHD[i_sh].name == files[i_f] ) break;
-	if(i_sh < SchHD.size())
+	if( i_sh < SchHD.size() )
 	{
 	    try { if(st_auto) libDet(files[i_f]); }
 	    catch(TError err)
@@ -442,6 +378,7 @@ void TModSchedul::libLoad( const string &iname, bool full)
 		continue;
 	    }
 	}
+	res.release();
 	libReg(files[i_f]);
 	if(st_auto)
 	{
@@ -459,10 +396,20 @@ void TModSchedul::cntrCmdProc( XMLNode *opt )
 	TSubSYS::cntrCmdProc(opt);
 	if(ctrMkNode("area",opt,0,"/ms",_("Subsystem"),0444,"root","root"))
 	{
+	    ctrMkNode("fld",opt,-1,"/ms/mod_path",_("Path to shared libs(modules)"),0444,"root","root",1,"tp","str");
+	    ctrMkNode("fld",opt,-1,"/ms/mod_allow",_("Allowed modules"),0444,"root","root",2,"tp","str",
+		"help",_("List of shared libs(modules) allow for auto conection.\nElements separated by symbol ';'.\nValue '*' is used for allow all modules."));
+	    ctrMkNode("fld",opt,-1,"/ms/mod_deny",_("Denied modules"),0444,"root","root",2,"tp","str",
+		"help",_("List of shared libs(modules) deny for auto conection.\nElements separated by symbol ';'."));
 	    ctrMkNode("fld",opt,-1,"/ms/chk_per",_("Check modules period (sec)"),0664,"root","root",1,"tp","dec");
 	    ctrMkNode("comm",opt,-1,"/ms/chk_now",_("Check modules now."),0660,"root","root");
-	    ctrMkNode("fld",opt,-1,"/ms/mod_path",_("Path to shared libs(modules)"),0664,"root","root",1,"tp","str");
-	    ctrMkNode("list",opt,-1,"/ms/mod_auto",_("List of auto conected shared libs(modules)"),0664,"root","root",2,"tp","str","s_com","add,ins,edit,del");
+	    if(ctrMkNode("table",opt,-1,"/ms/libs",_("Shared libs(modules)"),0664,"root","root",1,"key","path"))
+	    {
+		ctrMkNode("list",opt,-1,"/ms/libs/path",_("Path"),0444,"root","root",1,"tp","str");
+		ctrMkNode("list",opt,-1,"/ms/libs/tm",_("Time"),0444,"root","root",1,"tp","time");
+		ctrMkNode("list",opt,-1,"/ms/libs/mods",_("Modules"),0444,"root","root",1,"tp","str");
+		ctrMkNode("list",opt,-1,"/ms/libs/en",_("Enable"),0664,"root","root",1,"tp","bool");
+	    }
 	}
 	ctrMkNode("fld",opt,-1,"/help/g_help",_("Options help"),0440,"root","root",3,"tp","str","cols","90","rows","10");
 	return;
@@ -470,27 +417,45 @@ void TModSchedul::cntrCmdProc( XMLNode *opt )
 
     //- Process command to page -
     string a_path = opt->attr("path");
-    if( a_path == "/ms/chk_per" )
+    if( a_path == "/ms/mod_path" && ctrChkNode(opt,"get") )		opt->setText( SYS->modDir() );
+    else if( a_path == "/ms/mod_allow" && ctrChkNode(opt,"get") )	opt->setText(allowList());
+    else if( a_path == "/ms/mod_deny" && ctrChkNode(opt,"get") )	opt->setText(denyList());
+    else if( a_path == "/ms/chk_per" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(TSYS::int2str(chkPer()));
 	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setChkPer(atoi(opt->text().c_str()));
     }
-    else if( a_path == "/ms/mod_path" )	
-    {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText( modPath() );
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setModPath( opt->text() );
-    }
-    else if( a_path == "/ms/mod_auto" )
+    else if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440,"root","root",SEQ_RD) )	opt->setText(optDescr());
+    else if( a_path == "/ms/chk_now" && ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )	libLoad(SYS->modDir(),true);
+    else if( a_path == "/ms/libs" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
-	    for( unsigned i_a=0; i_a < m_am_list.size(); i_a++ )
-		opt->childAdd("el")->setText(m_am_list[i_a]);
-	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )	{ m_am_list.push_back(opt->text()); modif(); }
-	if( ctrChkNode(opt,"ins",0664,"root","root",SEQ_WR) )	{ m_am_list.insert(m_am_list.begin()+atoi(opt->attr("pos").c_str()),opt->text()); modif(); }
-	if( ctrChkNode(opt,"edit",0664,"root","root",SEQ_WR) )	{ m_am_list[atoi(opt->attr("pos").c_str())] = opt->text(); modif(); }
-	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )	{ m_am_list.erase(m_am_list.begin()+atoi(opt->attr("pos").c_str())); modif(); }
+	{
+	    XMLNode *n_nm	= ctrMkNode("list",opt,-1,"/ms/libs/path","");
+	    XMLNode *n_tm	= ctrMkNode("list",opt,-1,"/ms/libs/tm","");
+	    XMLNode *n_mods	= ctrMkNode("list",opt,-1,"/ms/libs/mods","");
+	    XMLNode *n_en	= ctrMkNode("list",opt,-1,"/ms/libs/en","");
+	    ResAlloc res(nodeRes(),false);
+	    for( int i_sl = 0; i_sl < SchHD.size(); i_sl++ )
+	    {
+		if( n_nm ) n_nm->childAdd("el")->setText(SchHD[i_sl].name);
+		if( n_tm ) n_tm->childAdd("el")->setText(TSYS::TSYS::int2str(SchHD[i_sl].tm));
+		if( n_mods )
+		{
+		    string useLs;
+		    for( int i_el = 0; i_el < SchHD[i_sl].use.size(); i_el++ )
+			useLs += SchHD[i_sl].use[i_el]+"; ";
+		    n_mods->childAdd("el")->setText(useLs);
+		}
+		if( n_en ) n_en->childAdd("el")->setText(TSYS::TSYS::int2str((bool)SchHD[i_sl].hd));
+	    }
+	}
+	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
+	{
+	    if( opt->attr("col") == "en" )
+		if( atoi(opt->text().c_str()) ) libAtt(opt->attr("key_path"),true);
+		else libDet(opt->attr("key_path"));
+	}
     }
-    else if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440,"root","root",SEQ_RD) )	opt->setText(optDescr());
-    else if( a_path == "/ms/chk_now" && ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )	libLoad(modPath(),true);
     else TSubSYS::cntrCmdProc(opt);
 }
