@@ -1,7 +1,7 @@
 
 //OpenSCADA system file: tmodschedul.h
 /***************************************************************************
- *   Copyright (C) 2003-2007 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2009 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -37,30 +37,28 @@ class TModSchedul : public TSubSYS
 	{
 	    public:
 		//Methods
-		SHD( ) : hd(NULL), m_tm(0)	{ }
-		SHD( void *ihd, time_t itm, const string &iname ) : hd(ihd), m_tm(itm), name(iname)	{ }
-		
+		SHD( ) : hd(NULL), tm(0)	{ }
+		SHD( void *ihd, time_t itm, const string &iname ) : hd(ihd), tm(itm), name(iname)	{ }
+
 		//Attributes
 		void		*hd;		// NULL - share lib present but no attached
 		vector<string>	use;		// if share lib attached to show who modules used
-		time_t		m_tm;		// data modify of share lib for automatic update
+		time_t		tm;		// data modify of share lib for automatic update
 		string		name;		// share lib path
 	};
 
 	//Methods
 	TModSchedul( );
-	~TModSchedul( );
 
-	int chkPer( )		{ return m_per; }
-	string modPath( )	{ return m_mod_path; }
+	string allowList( )	{ return mAllow; }
+	string denyList( )	{ return mDeny; }
+	int chkPer( )		{ return mPer; }
 
-	void setChkPer( int per );
-	void setModPath( const string &vl )	{ m_mod_path = vl; modif(); }
+	void setAllowList( const string &vl )	{ mAllow = vl; modif(); }
+	void setDenyList( const string &vl )	{ mDeny = vl; modif(); }
+	void setChkPer( int per )		{ mPer = per; modif(); }
 
-	void subStart( );
-	void subStop( );
-
-	void loadLibS( );				// Load/Init/Start all share libs and registry moduls into TSubSYS	
+	void loadLibS( );				// Load/Init/Start all share libs and registry moduls into TSubSYS
 
 	SHD &lib( const string &name );			// Get stat share lib <name>
 	void libList( vector<string> &list );		// List present share libs
@@ -68,6 +66,8 @@ class TModSchedul : public TSubSYS
 	void libAtt( const string &name, bool full = false);	// Attach share libs
 	void libDet( const string &name );		// Detach share libs
 
+	Res &nodeRes( )		{ return nRes; }
+
     protected:
 	//Methods
 	void load_( );
@@ -81,21 +81,19 @@ class TModSchedul : public TSubSYS
 	bool CheckFile( const string &name );			//Check file to OpenSCADA share libs
 	int  libReg( const string &name );			//Registre present share lib
 	void libUnreg( const string &name );			//Unreg deleted share lib
-	bool CheckAuto( const string &name) const;		//Check file to auto attaching
+	bool chkAllowMod( const string &name );			//Check for allow file to attaching
 
 	void cntrCmdProc( XMLNode *opt );		//Control interface command process
 
 	static void SchedTask( union sigval obj );
 
 	//Attributes
-	string		m_mod_path;
-	vector<string>	m_am_list;
+	string	mAllow, mDeny;
 
-	Res		hd_res;
-	int		m_per;				//Check to new modules period
+	int		mPer;				//Check to new modules period
 	vector<SHD>	SchHD;
-	timer_t		tmId;				//Thread timer
-	bool		prc_st;
+
+	Res		nRes;
 };
 
 #endif // TMODSCHEDUL_H
