@@ -66,6 +66,8 @@ void TModule::postEnable( int flag )
     bindtextdomain(lc_id.c_str(),LOCALEDIR);
 }
 
+TSubSYS &TModule::owner( )	{ return *(TSubSYS*)nodePrev(); }
+
 void TModule::modFuncList( vector<string> &list )
 {
     list.clear();
@@ -85,7 +87,7 @@ TModule::ExpFunc &TModule::modFunc( const string &prot )
 {
     for(int i=0; i < m_efunc.size(); i++)
 	if( m_efunc[i]->prot == prot ) return *m_efunc[i];
-    throw TError(nodePath().c_str(),_("Function <%s> no present into module!"),prot.c_str());
+    throw TError(nodePath().c_str(),_("Function <%s> is not present in the module!"),prot.c_str());
 }
 
 void TModule::modFunc( const string &prot, void (TModule::**offptr)() )
@@ -120,6 +122,7 @@ void TModule::cntrCmdProc( XMLNode *opt )
     //- Get page info -
     if( opt->name() == "info" )
     {
+	TCntrNode::cntrCmdProc(opt);
 	ctrMkNode("oscada_cntr",opt,-1,"/",_("Module: ")+modId(),0444);
 	ctrMkNode("branches",opt,-1,"/br","",0444);
 	if(TUIS::icoPresent(owner().subId()+"."+modId())) ctrMkNode("img",opt,-1,"/ico","",0444);
