@@ -18,7 +18,7 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
- 
+
 #include "tbds.h"
 #include "tsys.h"
 #include "tmess.h"
@@ -32,10 +32,10 @@
 //* TParamContr                                   *
 //*************************************************
 TParamContr::TParamContr( const string &name, TTipParam *tpprm ) :
-    TConfig(tpprm), tipparm(tpprm), m_en(false),
-    m_id(cfg("SHIFR").getSd()), m_name(cfg("NAME").getSd()), m_descr(cfg("DESCR").getSd()), m_aen(cfg("EN").getBd())
+    TConfig(tpprm), tipparm(tpprm), m_en(false), mRedntTmLast(0), m_id(cfg("SHIFR").getSd()), m_aen(cfg("EN").getBd())
 {
-    m_id = m_name = name;
+    m_id = name;
+    setName(name);
 }
 
 TParamContr::~TParamContr( )
@@ -48,7 +48,7 @@ TCntrNode &TParamContr::operator=( TCntrNode &node )
     TParamContr *src_n = dynamic_cast<TParamContr*>(&node);
     if( !src_n ) return *this;
 
-    //- Configuration copy -
+    //> Configuration copy
     string tid = id();
     *(TConfig*)this = *(TConfig*)src_n;
     m_id = tid;
@@ -58,10 +58,15 @@ TCntrNode &TParamContr::operator=( TCntrNode &node )
     return *this;
 }
 
-string TParamContr::name()
-{
-    return (m_name.size())?m_name:m_id;
-}
+TController &TParamContr::owner( )	{ return *(TController*)nodePrev(); }
+
+string TParamContr::name( )	{ string nm = cfg("NAME").getS(); return nm.size() ? nm : id(); }
+
+void TParamContr::setName( const string &inm )	{ cfg("NAME").setS(inm); }
+
+string TParamContr::descr( )	{ return cfg("DESCR").getS(); }
+
+void TParamContr::setDescr( const string &idsc ){ cfg("DESCR").setS(idsc); }
 
 void TParamContr::postEnable(int flag)
 {
@@ -76,10 +81,10 @@ void TParamContr::preDisable(int flag)
 {
     if( flag )
     {
-	//- Delete archives -
+	//> Delete archives
 	vector<string> a_ls;
 	vlList(a_ls);
-	for(int i_a = 0; i_a < a_ls.size(); i_a++)
+	for( int i_a = 0; i_a < a_ls.size(); i_a++ )
 	    if( !vlAt(a_ls[i_a]).at().arch().freeStat() )
 	    {
 		string arh_id = vlAt(a_ls[i_a]).at().arch().at().id();
@@ -94,27 +99,29 @@ void TParamContr::postDisable(int flag)
 {
     if( flag )
     {
-	//- Delete parameter from DB -
+	//> Delete parameter from DB
 	try
 	{
 	    SYS->db().at().dataDel(owner().DB()+"."+owner().cfg(type().db).getS(),
-				   owner().owner().nodePath()+owner().cfg(type().db).getS(),*this);
+				   owner().owner().nodePath()+owner().cfg(type().db).getS(),*this,true);
 	}catch(TError err) { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
     }
 }
 
 void TParamContr::load_( )
 {
+    if( !SYS->chkSelDB(owner().DB()) ) return;
+
     SYS->db().at().dataGet(owner().DB()+"."+owner().cfg(type().db).getS(),
 			   owner().owner().nodePath()+owner().cfg(type().db).getS(),*this);
 }
 
 void TParamContr::save_( )
 {
-    SYS->db().at().dataSet(owner().DB()+"."+owner().cfg(type().db).getS(),
-			   owner().owner().nodePath()+owner().cfg(type().db).getS(),*this);
+    SYS->db().at().dataSet( owner().DB()+"."+owner().cfg(type().db).getS(),
+			    owner().owner().nodePath()+owner().cfg(type().db).getS(),*this );
 
-    //- Save archives -
+    //> Save archives
     vector<string> a_ls;
     vlList(a_ls);
     for(int i_a = 0; i_a < a_ls.size(); i_a++)
@@ -122,12 +129,7 @@ void TParamContr::save_( )
 	    vlAt(a_ls[i_a]).at().arch().at().save();
 }
 
-bool TParamContr::cfgChange( TCfg &cfg )
-{
-    modif( );
-
-    return true;
-}
+bool TParamContr::cfgChange( TCfg &cfg )	{ modif( ); return true; }
 
 TParamContr & TParamContr::operator=( TParamContr & PrmCntr )
 {
@@ -151,7 +153,7 @@ void TParamContr::vlGet( TVal &val )
     if(val.name() == "err" )
     {
 	if( enableStat() ) val.setS("0",0,true);
-	else val.setS(_("1:Parameter had disabled."),0,true);
+	else val.setS(_("1:Parameter is disabled."),0,true);
     }
 }
 
@@ -159,10 +161,10 @@ void TParamContr::cntrCmdProc( XMLNode *opt )
 {
     string a_path = opt->attr("path");
 
-    //- Service commands process -
+    //> Service commands process
     if( a_path.substr(0,6) == "/serv/" ) { TValue::cntrCmdProc(opt); return; }
 
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TValue::cntrCmdProc(opt);
@@ -172,7 +174,7 @@ void TParamContr::cntrCmdProc( XMLNode *opt )
 	    if(ctrMkNode("area",opt,-1,"/prm/st",_("State")))
 	    {
 		ctrMkNode("fld",opt,-1,"/prm/st/type",_("Type"),0444,"root","root",1,"tp","str");
-		if( owner().enableStat() ) 
+		if( owner().enableStat() )
 		    ctrMkNode("fld",opt,-1,"/prm/st/en",_("Enable"),0664,"root","root",1,"tp","bool");
 	    }
 	    if(ctrMkNode("area",opt,-1,"/prm/cfg",_("Config")))
@@ -180,14 +182,14 @@ void TParamContr::cntrCmdProc( XMLNode *opt )
 	}
         return;
     }
-    //- Process command to page -
+    //> Process command to page
     if( a_path == "/prm/st/type" && ctrChkNode(opt) )	opt->setText(type().name);
     else if( a_path == "/prm/st/en" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(enableStat()?"1":"0");
 	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
 	{
-	    if( !owner().enableStat() )	throw TError(nodePath().c_str(),"Controller no started!");
+	    if( !owner().enableStat() )	throw TError(nodePath().c_str(),"Controller is not started!");
 	    else atoi(opt->text().c_str())?enable():disable();
 	}
     }
