@@ -38,20 +38,21 @@ class TTipParam;
 class TParamContr : public TConfig, public TValue
 {
     public:
-	TParamContr( const string &name, TTipParam *tpprm ); 
+	//Methods
+	TParamContr( const string &name, TTipParam *tpprm );
 	virtual ~TParamContr( );
 
 	TCntrNode &operator=( TCntrNode &node );
 
 	const string &id( )	{ return m_id; }
 	string name( );
-	string descr( )		{ return m_descr; }
+	string descr( );
 	bool toEnable( )	{ return m_aen; }
 	bool enableStat( )	{ return m_en; }
 
-	void setName( const string &inm )	{ m_name = inm; }
-	void setDescr( const string &idsc )	{ m_descr = idsc; }
-	void setToEnable( bool vl )		{ m_aen = vl; }
+	void setName( const string &inm );
+	void setDescr( const string &idsc );
+	void setToEnable( bool vl )		{ m_aen = vl; modif(); }
 
 	TTipParam &type( )	{ return *tipparm; }
 
@@ -63,7 +64,10 @@ class TParamContr : public TConfig, public TValue
 
 	TParamContr &operator=( TParamContr & PrmCntr );
 
-	TController &owner( ) 	{ return *(TController *)nodePrev(); }
+	TController &owner( );
+
+	//Attributes
+	long long mRedntTmLast;
 
     protected:
 	//Methods
@@ -84,11 +88,11 @@ class TParamContr : public TConfig, public TValue
 	string nodeName( )	{ return m_id; }
 
 	//Attributes
-	string	&m_id, &m_name,	&m_descr;
+	string	&m_id;
 	bool	&m_aen, m_en;
 	TElem	el_err;		//Error atributes
 
-	TTipParam   *tipparm;
+	TTipParam	*tipparm;
 };
 
 #endif // TPARAMCONTR_H
