@@ -26,12 +26,10 @@
 //* TPrmTempl                                     *
 //*************************************************
 TPrmTempl::TPrmTempl( const char *iid, const char *iname ) :
-    TFunction(string("tmpl_")+iid), TConfig(&SYS->daq().at().tplE()),
-    m_id(cfg("ID").getSd()), m_name(cfg("NAME").getSd()),
-    m_descr(cfg("DESCR").getSd()), m_prog(cfg("PROGRAM").getSd())
+    TFunction(string("tmpl_")+iid), TConfig(&SYS->daq().at().tplE()), m_id(cfg("ID").getSd()), m_prog(cfg("PROGRAM").getSd())
 {
     m_id = iid;
-    m_name = iname;
+    setName(iname);
 }
 
 TPrmTempl::~TPrmTempl(  )
@@ -54,7 +52,7 @@ TCntrNode &TPrmTempl::operator=( TCntrNode &node )
 
 void TPrmTempl::postEnable( int flag )
 {
-    //- Create default IOs -
+    //> Create default IOs
     if( flag&TCntrNode::NodeConnect )
     {
 	ioIns( new IO("f_frq",_("Function calculate frequency (Hz)"),IO::Real,TPrmTempl::LockAttr,"1000",false),0);
@@ -70,55 +68,47 @@ void TPrmTempl::postDisable(int flag)
     {
 	if( flag )
 	{
-	    SYS->db().at().dataDel(owner().fullDB(),owner().owner().nodePath()+owner().tbl(),*this);
-	    //- Delete template's IO -
+	    SYS->db().at().dataDel(owner().fullDB(),owner().owner().nodePath()+owner().tbl(),*this,true);
+	    //> Delete template's IO
 	    TConfig cfg(&owner().owner().tplIOE());
-	    cfg.cfg("TMPL_ID").setS(id());
-	    cfg.cfg("ID").setS("");
+	    cfg.cfg("TMPL_ID").setS(id(),true);
 	    SYS->db().at().dataDel(owner().fullDB()+"_io",owner().owner().nodePath()+owner().tbl()+"_io/",cfg);
 	}
     }catch(TError err)
     { mess_warning(err.cat.c_str(),"%s",err.mess.c_str()); }
 }
 
-string TPrmTempl::name()
-{
-    return (m_name.size())?m_name:m_id;
-}
+TPrmTmplLib &TPrmTempl::owner( )	{ return *(TPrmTmplLib*)nodePrev(); }
 
-string TPrmTempl::progLang()
-{
-    if(m_prog.find("\n")==string::npos)	m_prog=m_prog+"\n";
-    return m_prog.substr(0,m_prog.find("\n"));
-}
+string TPrmTempl::name( )		{ string nm = cfg("NAME").getS(); return nm.size() ? nm : id(); }
 
-string TPrmTempl::prog()
+void TPrmTempl::setName( const string &inm )	{ cfg("NAME").setS(inm); modif(); }
+
+string TPrmTempl::descr( )		{ return cfg("DESCR").getS(); }
+
+void TPrmTempl::setDescr( const string &idsc )	{ cfg("DESCR").setS(idsc); modif(); }
+
+string TPrmTempl::progLang()		{ return m_prog.substr(0,m_prog.find("\n")); }
+
+string TPrmTempl::prog( )
 {
-    int lng_end = m_prog.find("\n");
-    if(lng_end == string::npos)	lng_end=0;
-    else lng_end++;
-    return m_prog.substr(lng_end);
+    int lngEnd = m_prog.find("\n");
+    return m_prog.substr( (lngEnd==string::npos)?0:lngEnd+1 );
 }
 
 void TPrmTempl::setProgLang( const string &ilng )
 {
-    if(startStat()) setStart(false);
+    if( startStat() ) setStart(false);
 
-    m_prog.replace(0,m_prog.find("\n"),ilng);
+    m_prog = ilng+"\n"+prog();
     modif();
 }
 
 void TPrmTempl::setProg( const string &iprg )
 {
-    if(startStat()) setStart(false);
+    if( startStat() ) setStart(false);
 
-    int lng_end = m_prog.find("\n");
-    if(lng_end == string::npos)
-    {
-	m_prog=m_prog+"\n";
-	lng_end=m_prog.find("\n");
-    }
-    m_prog.replace(lng_end+1,string::npos,iprg);
+    m_prog = progLang()+"\n"+iprg;
     modif();
 }
 
@@ -146,20 +136,20 @@ AutoHD<TFunction> TPrmTempl::func()
 
 void TPrmTempl::load_( )
 {
-    //- Self load -
+    if( !SYS->chkSelDB(owner().DB()) ) return;
+
+    //> Self load
     SYS->db().at().dataGet(owner().fullDB(),owner().owner().nodePath()+owner().tbl(),*this);
 
-    //- Load IO -
+    //> Load IO
     vector<string> u_pos;
     TConfig cfg(&owner().owner().tplIOE());
-    cfg.cfg("TMPL_ID").setS(id());
-    int io_cnt = 0;
-    while(SYS->db().at().dataSeek(owner().fullDB()+"_io",owner().owner().nodePath()+owner().tbl()+"_io",io_cnt++,cfg) )
+    cfg.cfg("TMPL_ID").setS(id(),true);
+    for( int io_cnt = 0; SYS->db().at().dataSeek(owner().fullDB()+"_io",owner().owner().nodePath()+owner().tbl()+"_io",io_cnt++,cfg); )
     {
 	string sid = cfg.cfg("ID").getS();
-	cfg.cfg("ID").setS("");
 
-	//- Position storing -
+	//> Position storing
 	int pos = cfg.cfg("POS").getI();
 	while( u_pos.size() <= pos )	u_pos.push_back("");
 	u_pos[pos] = sid;
@@ -181,7 +171,7 @@ void TPrmTempl::load_( )
     {
 	if( u_pos[i_p].empty() ) continue;
 	int iid = ioId(u_pos[i_p]);
-	if( iid != i_p ) 
+	if( iid != i_p )
 	    try{ ioMove(iid,i_p); } catch(...){ }
     }
 }
@@ -190,38 +180,34 @@ void TPrmTempl::save_( )
 {
     string w_db = owner().fullDB();
     string w_cfgpath = owner().owner().nodePath()+owner().tbl();
-    //- Self save -
+    //> Self save
     SYS->db().at().dataSet(w_db,w_cfgpath,*this);
 
-    //- Save IO -
+    //> Save IO
     TConfig cfg(&owner().owner().tplIOE());
-    cfg.cfg("TMPL_ID").setS(id());
+    cfg.cfg("TMPL_ID").setS(id(),true);
     for(int i_io = 0; i_io < ioSize(); i_io++)
     {
-	if( io(i_io)->id() == "f_frq" || io(i_io)->id() == "f_start" ||
-		io(i_io)->id() == "f_stop" || io(i_io)->id() == "f_err" ) continue;
+	if( io(i_io)->flg()&TPrmTempl::LockAttr ) continue;
 	cfg.cfg("ID").setS(io(i_io)->id());
 	cfg.cfg("NAME").setS(io(i_io)->name());
 	cfg.cfg("TYPE").setI(io(i_io)->type());
 	cfg.cfg("FLAGS").setI(io(i_io)->flg());
+	cfg.cfg("VALUE").setNoTransl( !(io(i_io)->type()==IO::String || io(i_io)->flg()&TPrmTempl::CfgLink) );
 	cfg.cfg("VALUE").setS(io(i_io)->def());
 	cfg.cfg("POS").setI(i_io);
 	SYS->db().at().dataSet(w_db+"_io",w_cfgpath+"_io",cfg);
     }
-    //- Clear IO -
-    int fld_cnt=0;
-    cfg.cfg("ID").setS("");
+    //> Clear IO
     cfg.cfgViewAll(false);
-    while( SYS->db().at().dataSeek(w_db+"_io",w_cfgpath+"_io",fld_cnt++,cfg ) )
+    for( int fld_cnt=0; SYS->db().at().dataSeek(w_db+"_io",w_cfgpath+"_io",fld_cnt++,cfg); )
     {
 	string sio = cfg.cfg("ID").getS( );
-	if( ioId(sio) < 0 || sio == "f_frq" || sio == "f_start" ||
-		sio == "f_stop" || sio == "f_err" )
+	if( ioId(sio) < 0 || io(ioId(sio))->flg()&TPrmTempl::LockAttr )
 	{
-	    SYS->db().at().dataDel(w_db+"_io",w_cfgpath+"_io",cfg);
+	    SYS->db().at().dataDel(w_db+"_io",w_cfgpath+"_io",cfg,true);
 	    fld_cnt--;
 	}
-	cfg.cfg("ID").setS("");
     }
 }
 
@@ -235,16 +221,20 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
     //- Get page info -
     if( opt->name() == "info" )
     {
+	TCntrNode::cntrCmdProc(opt);
 	ctrMkNode("oscada_cntr",opt,-1,"/",_("Parameter template: ")+name(),0664,"root","root");
 	if(ctrMkNode("area",opt,-1,"/tmpl",_("Template")))
 	{
 	    if(ctrMkNode("area",opt,-1,"/tmpl/st",_("State")))
+	    {
 		ctrMkNode("fld",opt,-1,"/tmpl/st/st",_("Accessing"),0664,"root","root",1,"tp","bool");
+		ctrMkNode("fld",opt,-1,"/tmpl/st/use",_("Used"),0444,"root","root",1,"tp","dec");
+	    }
 	    if(ctrMkNode("area",opt,-1,"/tmpl/cfg",_("Config")))
 	    {
 		ctrMkNode("fld",opt,-1,"/tmpl/cfg/id",_("Id"),0444,"root","root",1,"tp","str");
-		ctrMkNode("fld",opt,-1,"/tmpl/cfg/name",_("Name"),0664,"root","root",1,"tp","str");
-		ctrMkNode("fld",opt,-1,"/tmpl/cfg/descr",_("Description"),0664,"root","root",3,"tp","str","cols","70","rows","4");
+		ctrMkNode("fld",opt,-1,"/tmpl/cfg/name",_("Name"),0664,"root","root",2,"tp","str","len","50");
+		ctrMkNode("fld",opt,-1,"/tmpl/cfg/descr",_("Description"),0664,"root","root",3,"tp","str","cols","100","rows","4");
 	    }
 	}
 	if( ctrMkNode("area",opt,-1,"/io",_("IO")) )
@@ -253,14 +243,21 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 	    {
 		ctrMkNode("list",opt,-1,"/io/io/0",_("Id"),0664,"root","root",1,"tp","str");
 		ctrMkNode("list",opt,-1,"/io/io/1",_("Name"),0664,"root","root",1,"tp","str");
-		ctrMkNode("list",opt,-1,"/io/io/2",_("Type"),0664,"root","root",4,"tp","dec","idm","1","dest","select","select","/io/tp");
-		ctrMkNode("list",opt,-1,"/io/io/3",_("Mode"),0664,"root","root",4,"tp","dec","idm","1","dest","select","select","/io/md");
-		ctrMkNode("list",opt,-1,"/io/io/4",_("Attribute"),0664,"root","root",4,"tp","dec","idm","1","dest","select","select","/io/attr_mods");
-		ctrMkNode("list",opt,-1,"/io/io/5",_("Access"),0664,"root","root",4,"tp","dec","idm","1","dest","select","select","/io/accs_mods");
+		ctrMkNode("list",opt,-1,"/io/io/2",_("Type"),0664,"root","root",5,"tp","dec","idm","1","dest","select",
+		    "sel_id",(TSYS::int2str(IO::Real)+";"+TSYS::int2str(IO::Integer)+";"+TSYS::int2str(IO::Boolean)+";"+TSYS::int2str(IO::String)).c_str(),
+		    "sel_list",_("Real;Integer;Boolean;String"));
+		ctrMkNode("list",opt,-1,"/io/io/3",_("Mode"),0664,"root","root",5,"tp","dec","idm","1","dest","select",
+		    "sel_id",(TSYS::int2str(IO::Default)+";"+TSYS::int2str(IO::Output)).c_str(),"sel_list",_("Input;Output"));
+		ctrMkNode("list",opt,-1,"/io/io/4",_("Attribute"),0664,"root","root",5,"tp","dec","idm","1","dest","select",
+		    "sel_id",(TSYS::int2str(IO::Default)+";"+TSYS::int2str(TPrmTempl::AttrRead)+";"+TSYS::int2str(TPrmTempl::AttrFull)).c_str(),
+		    "sel_list",_("No attribute;Read only;Full access"));
+		ctrMkNode("list",opt,-1,"/io/io/5",_("Access"),0664,"root","root",5,"tp","dec","idm","1","dest","select",
+		    "sel_id",(TSYS::int2str(IO::Default)+";"+TSYS::int2str(TPrmTempl::CfgPublConst)+";"+TSYS::int2str(TPrmTempl::CfgLink)).c_str(),
+		    "sel_list",_("Constant;Public constant;Link"));
 		ctrMkNode("list",opt,-1,"/io/io/6",_("Value"),0664,"root","root",1,"tp","str");
 	    }
 	    ctrMkNode("fld",opt,-1,"/io/prog_lang",_("Programm language"),0664,"root","root",3,"tp","str","dest","sel_ed","select","/io/plang_ls");
-	    ctrMkNode("fld",opt,-1,"/io/prog",_("Programm"),0664,"root","root",3,"tp","str","cols","90","rows","8");
+	    ctrMkNode("fld",opt,-1,"/io/prog",_("Programm"),0664,"root","root",2,"tp","str","rows","10");
 	}
 	return;
     }
@@ -273,6 +270,7 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(startStat()?"1":"0");
 	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setStart(atoi(opt->text().c_str()));
     }
+    else if( a_path == "/tmpl/st/use" && ctrChkNode(opt) )	opt->setText(TSYS::int2str(startStat()?func().at().use():0));
     else if( a_path == "/tmpl/cfg/id" && ctrChkNode(opt) )	opt->setText(id());
     else if( a_path == "/tmpl/cfg/name" )
     {
@@ -312,9 +310,9 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )
 	{
 	    int row = atoi(opt->attr("row").c_str());
-	    if(io(row)->flg()&TPrmTempl::LockAttr)
+	    if( io(row)->flg()&TPrmTempl::LockAttr )
 		throw TError(nodePath().c_str(),_("Deleting lock atribute in not allow."));
-	    ioDel( atoi(opt->attr("row").c_str()) );
+	    ioDel( row );
 	    modif();
 	}
 	if( ctrChkNode(opt,"move",0664,"root","root",SEQ_WR) )  { ioMove( atoi(opt->attr("row").c_str()), atoi(opt->attr("to").c_str()) ); modif(); }
@@ -322,10 +320,8 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 	{
 	    int row = atoi(opt->attr("row").c_str());
 	    int col = atoi(opt->attr("col").c_str());
-	    if( (col == 0 || col == 1) && !opt->text().size() )
-		throw TError(nodePath().c_str(),_("Empty value no valid."));
-	    if( io(row)->flg()&TPrmTempl::LockAttr )
-		throw TError(nodePath().c_str(),_("Change lock atribute in not allow."));
+	    if( io(row)->flg()&TPrmTempl::LockAttr )	throw TError(nodePath().c_str(),_("Changing locked atribute is not allowed."));
+	    if( (col == 0 || col == 1) && !opt->text().size() )	throw TError(nodePath().c_str(),_("Empty value is not valid."));
 	    switch(col)
 	    {
 		case 0:	io(row)->setId(opt->text());	break;
@@ -373,31 +369,6 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 	for(int i_l = 0; i_l < ls.size(); i_l++)
 	    opt->childAdd("el")->setText(c_path+ls[i_l]);
     }
-    else if( a_path == "/io/tp" && ctrChkNode(opt) )
-    {
-	opt->childAdd("el")->setAttr("id",TSYS::int2str(IO::Real))->setText(_("Real"));
-	opt->childAdd("el")->setAttr("id",TSYS::int2str(IO::Integer))->setText(_("Integer"));
-	opt->childAdd("el")->setAttr("id",TSYS::int2str(IO::Boolean))->setText(_("Boolean"));
-	opt->childAdd("el")->setAttr("id",TSYS::int2str(IO::String))->setText(_("String"));
-    }
-    else if( a_path == "/io/md" && ctrChkNode(opt) )
-    {
-	opt->childAdd("el")->setAttr("id",TSYS::int2str(IO::Default))->setText(_("Input"));
-	opt->childAdd("el")->setAttr("id",TSYS::int2str(IO::Output))->setText(_("Output"));
-	opt->childAdd("el")->setAttr("id",TSYS::int2str(IO::Return))->setText(_("Return"));
-    }
-    else if( a_path == "/io/attr_mods" && ctrChkNode(opt) )
-    {
-	opt->childAdd("el")->setAttr("id",TSYS::int2str(IO::Default))->setText(_("No attribute"));
-	opt->childAdd("el")->setAttr("id",TSYS::int2str(TPrmTempl::AttrRead))->setText(_("Read only"));
-	opt->childAdd("el")->setAttr("id",TSYS::int2str(TPrmTempl::AttrFull))->setText(_("Full access"));
-    }
-    else if( a_path == "/io/accs_mods" && ctrChkNode(opt) )
-    {
-	opt->childAdd("el")->setAttr("id",TSYS::int2str(IO::Default))->setText(_("Constant"));
-	opt->childAdd("el")->setAttr("id",TSYS::int2str(TPrmTempl::CfgPublConst))->setText(_("Public constant"));
-	opt->childAdd("el")->setAttr("id",TSYS::int2str(TPrmTempl::CfgLink))->setText(_("Link"));
-    }
     else TCntrNode::cntrCmdProc(opt);
 }
 
@@ -405,8 +376,7 @@ void TPrmTempl::cntrCmdProc( XMLNode *opt )
 //* TPrmTmplLib                                   *
 //*************************************************
 TPrmTmplLib::TPrmTmplLib( const char *id, const char *name, const string &lib_db ) :
-    TConfig(&SYS->daq().at().elLib()), run_st(false), m_id(cfg("ID").getSd()), m_name(cfg("NAME").getSd()),
-    m_descr(cfg("DESCR").getSd()), m_db(cfg("DB").getSd()), work_lib_db(lib_db)
+    TConfig(&SYS->daq().at().elLib()), run_st(false), m_id(cfg("ID").getSd()), m_db(cfg("DB").getSd()), work_lib_db(lib_db)
 {
     m_id = id;
     setName( name );
@@ -452,10 +422,10 @@ void TPrmTmplLib::postDisable(int flag)
 {
     if( flag )
     {
-	//- Delete libraries record -
-	SYS->db().at().dataDel(work_lib_db+"."+owner().tmplLibTable(),owner().nodePath()+"tmplib",*this);
+	//> Delete libraries record
+	SYS->db().at().dataDel(work_lib_db+"."+owner().tmplLibTable(),owner().nodePath()+"tmplib",*this,true);
 
-	//- Delete temlate librarie's DBs -
+	//> Delete temlate librarie's DBs
 	SYS->db().at().open(fullDB());
 	SYS->db().at().close(fullDB(),true);
 
@@ -464,10 +434,15 @@ void TPrmTmplLib::postDisable(int flag)
     }
 }
 
-string TPrmTmplLib::name()
-{
-    return (m_name.size())?m_name:m_id;
-}
+TDAQS &TPrmTmplLib::owner( )	{ return *(TDAQS*)nodePrev(); }
+
+string TPrmTmplLib::name( )	{ string nm = cfg("NAME").getS(); return nm.size() ? nm : id(); }
+
+void TPrmTmplLib::setName( const string &vl )	{ cfg("NAME").setS(vl); modif(); }
+
+string TPrmTmplLib::descr( )	{ return cfg("DESCR").getS(); }
+
+void TPrmTmplLib::setDescr( const string &vl )	{ cfg("DESCR").setS(vl); modif(); }
 
 void TPrmTmplLib::setFullDB( const string &vl )
 {
@@ -478,24 +453,23 @@ void TPrmTmplLib::setFullDB( const string &vl )
 
 void TPrmTmplLib::load_( )
 {
-    SYS->db().at().dataGet(work_lib_db+"."+owner().tmplLibTable(),owner().nodePath()+"tmplib",*this);
+    if( !SYS->chkSelDB(DB()) ) return;
 
-    //- Load templates -
+    SYS->db().at().dataGet(DB()+"."+owner().tmplLibTable(),owner().nodePath()+"tmplib",*this);
+
+    //> Load templates
     TConfig c_el(&owner().tplE());
     c_el.cfgViewAll(false);
-    int fld_cnt = 0;
-    while( SYS->db().at().dataSeek(fullDB(),owner().nodePath()+tbl(), fld_cnt++,c_el) )
+    for( int fld_cnt = 0; SYS->db().at().dataSeek(fullDB(),owner().nodePath()+tbl(), fld_cnt++,c_el); )
     {
 	string f_id = c_el.cfg("ID").getS();
-	c_el.cfg("ID").setS("");
-
 	if( !present(f_id) )	add(f_id.c_str());
     }
 }
 
 void TPrmTmplLib::save_( )
 {
-    SYS->db().at().dataSet(work_lib_db+"."+owner().tmplLibTable(),owner().nodePath()+"tmplib",*this);
+    SYS->db().at().dataSet(DB()+"."+owner().tmplLibTable(),owner().nodePath()+"tmplib",*this);
 }
 
 void TPrmTmplLib::start( bool val )
@@ -514,7 +488,7 @@ void TPrmTmplLib::start( bool val )
 
     run_st = val;
 
-    if( isErr )	throw TError(nodePath().c_str(),_("Some templates start is error."));
+    if( isErr )	throw TError(nodePath().c_str(),_("Some templates start error."));
 }
 
 void TPrmTmplLib::add( const char *id, const char *name )
@@ -527,9 +501,10 @@ void TPrmTmplLib::cntrCmdProc( XMLNode *opt )
     //- Get page info -
     if( opt->name() == "info" )
     {
+	TCntrNode::cntrCmdProc(opt);
 	ctrMkNode("oscada_cntr",opt,-1,"/",_("Parameter templates library: ")+id(),0664,"root","root");
-	if(ctrMkNode("branches",opt,-1,"/br","",0444))	
-	    ctrMkNode("grp",opt,-1,"/br/tmpl_",_("Template"),0664,"root","root",1,"idm","1");
+	if(ctrMkNode("branches",opt,-1,"/br","",0444))
+	    ctrMkNode("grp",opt,-1,"/br/tmpl_",_("Template"),0664,"root","root",2,"idm","1","idSz","20");
 	if(ctrMkNode("area",opt,-1,"/lib",_("Library")))
 	{
 	    if(ctrMkNode("area",opt,-1,"/lib/st",_("State")))
@@ -541,12 +516,12 @@ void TPrmTmplLib::cntrCmdProc( XMLNode *opt )
 	    if(ctrMkNode("area",opt,-1,"/lib/cfg",_("Config")))
 	    {
 		ctrMkNode("fld",opt,-1,"/lib/cfg/id",_("Id"),0444,"root","root",1,"tp","str");
-		ctrMkNode("fld",opt,-1,"/lib/cfg/name",_("Name"),0664,"root","root",1,"tp","str");
-		ctrMkNode("fld",opt,-1,"/lib/cfg/descr",_("Description"),0664,"root","root",3,"tp","str","cols","50","rows","3");
+		ctrMkNode("fld",opt,-1,"/lib/cfg/name",_("Name"),0664,"root","root",2,"tp","str","len","50");
+		ctrMkNode("fld",opt,-1,"/lib/cfg/descr",_("Description"),0664,"root","root",3,"tp","str","cols","100","rows","3");
 	    }
 	}
 	if(ctrMkNode("area",opt,-1,"/tmpl",_("Parameter templates")))
-	    ctrMkNode("list",opt,-1,"/tmpl/tmpl",_("Templates"),0664,"root","root",4,"tp","br","idm","1","s_com","add,del","br_pref","tmpl_");
+	    ctrMkNode("list",opt,-1,"/tmpl/tmpl",_("Templates"),0664,"root","root",5,"tp","br","idm","1","s_com","add,del","br_pref","tmpl_","idSz","20");
 	return;
     }
     //- Process command to page -
@@ -581,7 +556,7 @@ void TPrmTmplLib::cntrCmdProc( XMLNode *opt )
 	    for( unsigned i_f=0; i_f < lst.size(); i_f++ )
 		opt->childAdd("el")->setAttr("id",lst[i_f])->setText(at(lst[i_f]).at().name());
 	}
-	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )	add(opt->attr("id").c_str(),opt->text().c_str());
+	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )	add(TSYS::strEncode(opt->attr("id"),TSYS::ID).c_str(),opt->text().c_str());
 	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )	del(opt->attr("id").c_str(),true);
     }
     else TCntrNode::cntrCmdProc(opt);
