@@ -46,7 +46,7 @@ string TProtocolS::optDescr(  )
 
 void TProtocolS::load_()
 {
-    //- Load parameters from command line -
+    //> Load parameters from command line
     int next_opt;
     const char *short_opt="h";
     struct option long_opt[] =
@@ -61,25 +61,25 @@ void TProtocolS::load_()
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
 
-    //- Load parameters from config file -
+    //> Load parameters from config file
 
 }
 
 void TProtocolS::cntrCmdProc( XMLNode *opt )
 {
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TSubSYS::cntrCmdProc(opt);
 	ctrMkNode("fld",opt,-1,"/help/g_help",_("Options help"),0440,"root",subId().c_str(),3,"tp","str","cols","90","rows","10");
 	return;
     }
-    //- Process command to page -
+    //> Process command to page
     string a_path = opt->attr("path");
     if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440,"root",subId().c_str()) )	opt->setText(optDescr());
     else TSubSYS::cntrCmdProc(opt);
@@ -96,12 +96,14 @@ TProtocol::TProtocol()
 
 TProtocol::~TProtocol()
 {
+
 }
 
-void TProtocol::open( const string &name )
+void TProtocol::open( const string &name, const string &tr )
 {
     if( chldPresent(m_pr,name) ) return;
     chldAdd(m_pr,in_open(name));
+    at(name).at().setSrcTr(tr);
 }
 
 void TProtocol::close( const string &name )
@@ -112,7 +114,7 @@ void TProtocol::close( const string &name )
 //************************************************
 //* TProtocolIn                                  *
 //************************************************
-TProtocolIn::TProtocolIn( const string &name ) : m_name(name)
+TProtocolIn::TProtocolIn( const string &name ) : mName(name)
 {
     modifClr( );
 }
@@ -121,3 +123,5 @@ TProtocolIn::~TProtocolIn()
 {
 
 }
+
+TProtocol &TProtocolIn::owner( )	{ return *(TProtocol*)nodePrev(); }
