@@ -22,7 +22,7 @@
 #ifndef TPROTOCOLS_H
 #define TPROTOCOLS_H
 
-#define  VER_PROT 1    //ProtocolS type modules version
+#define  VER_PROT 2    //ProtocolS type modules version
 
 #include <string>
 
@@ -32,29 +32,34 @@ using std::string;
 
 //************************************************
 //* TProtocolIn                                  *
-//************************************************ 
+//************************************************
 class TProtocol;
 
 class TProtocolIn : public TCntrNode
 {
     public:
+	//Methods
 	TProtocolIn( const string &name );
 	virtual ~TProtocolIn( );
 
-	string &name( )		{ return m_name; }
+	const string &name( )		{ return mName; }
+	const string &srcTr( )		{ return mSrcTr; }
 
-	//- Process input messages -
+	void setSrcTr( const string &vl )	{ mSrcTr = vl; }
+
+	//> Process input messages
 	virtual bool mess( const string &request, string &answer, const string &sender )
 	{ answer = ""; }
 
-	TProtocol &owner( )	{ return *(TProtocol *)nodePrev(); }
+	TProtocol &owner( );
 
     private:
 	//Methods
-	string nodeName( )	{ return m_name; }
+	string nodeName( )	{ return mName; }
 
 	//Attributes
-	string            m_name;
+	string		mName;
+	string		mSrcTr;
 };
 
 //************************************************
@@ -68,15 +73,15 @@ class TProtocol: public TModule
 	TProtocol( );
 	virtual ~TProtocol( );
 
-	//- Input protocol -
+	//> Input protocol
 	void list( vector<string> &list )		{ chldList(m_pr,list); }
 	bool openStat( const string &name )		{ return chldPresent(m_pr,name); } 
-	void open( const string &name );
+	void open( const string &name, const string &tr );
 	void close( const string &name );
 	AutoHD<TProtocolIn> at( const string &name )	{ return chldAt(m_pr,name); }
 
-	//- Output protocol -
-	virtual string outMess( const string &in, TTransportOut &tro )
+	//> Output protocol
+	virtual void outMess( XMLNode &io, TTransportOut &tro )
 	{ throw TError(nodePath().c_str(),"Function <%s> no support!","outMess"); }
 
     private:
