@@ -34,14 +34,14 @@ TSecurity::TSecurity( ) : TSubSYS("Security","Security",false)
     m_usr = TCntrNode::grpAdd("usr_");
     m_grp = TCntrNode::grpAdd("grp_");
 
-    //- User BD structure -
+    //> User BD structure
     user_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TCfg::Key,"20") );
-    user_el.fldAdd( new TFld("DESCR",_("Full name"),TFld::String,0,"50") );
+    user_el.fldAdd( new TFld("DESCR",_("Full name"),TFld::String,TCfg::TransltText,"50") );
     user_el.fldAdd( new TFld("PASS",_("Password"),TFld::String,0,"20") );
     user_el.fldAdd( new TFld("PICTURE",_("User picture"),TFld::String,0,"100000") );
-    //- Group BD structure -
+    //> Group BD structure
     grp_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TCfg::Key,"20") );
-    grp_el.fldAdd( new TFld("DESCR",_("Full name"),TFld::String,0,"50") );
+    grp_el.fldAdd( new TFld("DESCR",_("Full name"),TFld::String,TCfg::TransltText,"50") );
     grp_el.fldAdd( new TFld("USERS",_("Users"),TFld::String,0,"200") );
 }
 
@@ -120,14 +120,14 @@ char TSecurity::access( const string &user, char mode, const string &owner, cons
 {
     char rez = 0;
 
-    //- Check owner permision -
+    //> Check owner permision
     if( user == "root" || user == owner )
 	rez = ((access&0700)>>6)&mode;
-    //- Check other permision -
+    //> Check other permision
     if( rez == mode )	return rez;
     rez|=(access&07)&mode;
     if( rez == mode )	return rez;
-    //- Check groupe permision -
+    //> Check groupe permision
     if( grpAt(group).at().user(user) || grpAt("root").at().user(user) )
 	rez|=((access&070)>>3)&mode;
 
@@ -136,7 +136,7 @@ char TSecurity::access( const string &user, char mode, const string &owner, cons
 
 void TSecurity::load_( )
 {
-    //- Load commandline data -
+    //> Load commandline data
     int next_opt;
     const char *short_opt="h";
     struct option long_opt[] =
@@ -151,88 +151,68 @@ void TSecurity::load_( )
 	next_opt=getopt_long(SYS->argc,( char *const * ) SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
 
-    //- Load parametrs -
+    //> Load parameters
 
-    //- Load DB -
+    //> Load DB
     string	name;
 
-    //-- Search and create new users --
+    //>> Search and create new users
     try
     {
 	TConfig g_cfg(&user_el);
 	g_cfg.cfgViewAll(false);
-	vector<string> tdb_ls, db_ls;
+	vector<string> db_ls;
 
-	//--- Search into DB ---
-	SYS->db().at().modList(tdb_ls);
-	for( int i_tp = 0; i_tp < tdb_ls.size(); i_tp++ )
-	{
-	    SYS->db().at().at(tdb_ls[i_tp]).at().list(db_ls);
-	    for( int i_db = 0; i_db < db_ls.size(); i_db++ )
+	//>>> Search into DB
+	SYS->db().at().dbList(db_ls,true);
+	for( int i_db = 0; i_db < db_ls.size(); i_db++ )
+	    for( int fld_cnt=0; SYS->db().at().dataSeek(db_ls[i_db]+"."+subId()+"_user","",fld_cnt++,g_cfg); )
 	    {
-		string wbd = tdb_ls[i_tp]+"."+db_ls[i_db];
-		int fld_cnt=0;
-		while( SYS->db().at().dataSeek(wbd+"."+subId()+"_user","",fld_cnt++,g_cfg) )
-		{
-		    name = g_cfg.cfg("NAME").getS();
-		    if( !usrPresent(name) )	usrAdd(name,(wbd==SYS->workDB())?"*.*":wbd);
-		    g_cfg.cfg("NAME").setS("");
-		}
+		name = g_cfg.cfg("NAME").getS();
+		if( !usrPresent(name) )	usrAdd(name,(db_ls[i_db]==SYS->workDB())?"*.*":db_ls[i_db]);
 	    }
-	}
 
-	//--- Search into config file ---
-	int fld_cnt=0;
-	while( SYS->db().at().dataSeek("",nodePath()+subId()+"_user",fld_cnt++,g_cfg) )
-	{
-	    name = g_cfg.cfg("NAME").getS();
-	    if( !usrPresent(name) )	usrAdd(name,"*.*");
-	    g_cfg.cfg("NAME").setS("");
-	}
+	//>>> Search into config file
+	if( SYS->chkSelDB("<cfg>") )
+	    for( int fld_cnt=0; SYS->db().at().dataSeek("",nodePath()+subId()+"_user",fld_cnt++,g_cfg); )
+	    {
+		name = g_cfg.cfg("NAME").getS();
+		if( !usrPresent(name) )	usrAdd(name,"*.*");
+	    }
     }catch(TError err)
     {
 	mess_err(err.cat.c_str(),"%s",err.mess.c_str());
 	mess_err(nodePath().c_str(),_("Search and create new users error."));
     }
 
-    //-- Search and create new user groups --
+    //>> Search and create new user groups
     try
     {
 	TConfig g_cfg(&grp_el);
 	g_cfg.cfgViewAll(false);
-	vector<string> tdb_ls, db_ls;
+	vector<string> db_ls;
 
-	//--- Search into DB ---
-	SYS->db().at().modList(tdb_ls);
-	for( int i_tp = 0; i_tp < tdb_ls.size(); i_tp++ )
-	{
-	    SYS->db().at().at(tdb_ls[i_tp]).at().list(db_ls);
-	    for( int i_db = 0; i_db < db_ls.size(); i_db++ )
+	//>>> Search into DB
+	SYS->db().at().dbList(db_ls,true);
+	for( int i_db = 0; i_db < db_ls.size(); i_db++ )
+	    for( int fld_cnt=0; SYS->db().at().dataSeek(db_ls[i_db]+"."+subId()+"_grp","",fld_cnt++,g_cfg); )
 	    {
-		string wbd = tdb_ls[i_tp]+"."+db_ls[i_db];
-		int fld_cnt=0;
-		while( SYS->db().at().dataSeek(wbd+"."+subId()+"_grp","",fld_cnt++,g_cfg) )
-		{
-		    name = g_cfg.cfg("NAME").getS();
-		    if( !grpPresent(name) )	grpAdd(name,(wbd==SYS->workDB())?"*.*":wbd);
-		    g_cfg.cfg("NAME").setS("");
-		}
+		name = g_cfg.cfg("NAME").getS();
+		if( !grpPresent(name) )	grpAdd(name,(db_ls[i_db]==SYS->workDB())?"*.*":db_ls[i_db]);
 	    }
-	}
 
-	//--- Search into config file ---
-	int fld_cnt=0;
-	while( SYS->db().at().dataSeek("",nodePath()+subId()+"_grp",fld_cnt++,g_cfg) )
-	{
-	    name = g_cfg.cfg("NAME").getS();
-	    if( !grpPresent(name) )	grpAdd(name,"*.*");
-	    g_cfg.cfg("NAME").setS("");
-	}
+	//>>> Search into config file
+	if( SYS->chkSelDB("<cfg>") )
+	    for( int fld_cnt=0; SYS->db().at().dataSeek("",nodePath()+subId()+"_grp",fld_cnt++,g_cfg); )
+	    {
+		name = g_cfg.cfg("NAME").getS();
+		if( !grpPresent(name) )	grpAdd(name,"*.*");
+	    }
     }catch(TError err)
     {
 	mess_err(err.cat.c_str(),"%s",err.mess.c_str());
@@ -257,12 +237,12 @@ void TSecurity::cntrCmdProc( XMLNode *opt )
     if( opt->name() == "info" )
     {
 	TSubSYS::cntrCmdProc(opt);
-	ctrMkNode("grp",opt,-1,"/br/usr_",_("User"),0664,"root",subId().c_str());
-	ctrMkNode("grp",opt,-1,"/br/grp_",_("Group"),0664,"root",subId().c_str());
+	ctrMkNode("grp",opt,-1,"/br/usr_",_("User"),0664,"root",subId().c_str(),1,"idSz","20");
+	ctrMkNode("grp",opt,-1,"/br/grp_",_("Group"),0664,"root",subId().c_str(),1,"idSz","20");
 	if(ctrMkNode("area",opt,1,"/usgr",_("Users and groups")))
 	{
-	    ctrMkNode("list",opt,-1,"/usgr/users",_("Users"),0664,"root",subId().c_str(),3,"tp","br","s_com","add,del","br_pref","usr_");
-	    ctrMkNode("list",opt,-1,"/usgr/grps",_("Groups"),0664,"root",subId().c_str(),3,"tp","br","s_com","add,del","br_pref","grp_");
+	    ctrMkNode("list",opt,-1,"/usgr/users",_("Users"),0664,"root",subId().c_str(),4,"tp","br","s_com","add,del","br_pref","usr_","idSz","20");
+	    ctrMkNode("list",opt,-1,"/usgr/grps",_("Groups"),0664,"root",subId().c_str(),4,"tp","br","s_com","add,del","br_pref","grp_","idSz","20");
 	}
 	ctrMkNode("fld",opt,-1,"/help/g_help",_("Options help"),0440,"root",subId().c_str(),3,"tp","str","cols","90","rows","10");
 
@@ -281,7 +261,7 @@ void TSecurity::cntrCmdProc( XMLNode *opt )
 	    for( unsigned i_a=0; i_a < list.size(); i_a++ )
 	        opt->childAdd("el")->setText(list[i_a]);
 	}
-	if( ctrChkNode(opt,"add",0664,"root",subId().c_str(),SEQ_WR) )	usrAdd(opt->text());
+	if( ctrChkNode(opt,"add",0664,"root",subId().c_str(),SEQ_WR) )	usrAdd(TSYS::strEncode(opt->text(),TSYS::ID));
 	if( ctrChkNode(opt,"del",0664,"root",subId().c_str(),SEQ_WR) )	usrDel(opt->text(),true);
     }
     else if( a_path == "/br/grp_" || a_path == "/usgr/grps" )
@@ -293,7 +273,7 @@ void TSecurity::cntrCmdProc( XMLNode *opt )
 	    for( unsigned i_a=0; i_a < list.size(); i_a++ )
 	        opt->childAdd("el")->setText(list[i_a]);
 	}
-	if( ctrChkNode(opt,"add",0664,"root",subId().c_str(),SEQ_WR) )	grpAdd(opt->text());
+	if( ctrChkNode(opt,"add",0664,"root",subId().c_str(),SEQ_WR) )	grpAdd(TSYS::strEncode(opt->text(),TSYS::ID));
 	if( ctrChkNode(opt,"del",0664,"root",subId().c_str(),SEQ_WR) )	grpDel(opt->text(),true);
     }
     else TSubSYS::cntrCmdProc(opt);
@@ -343,8 +323,8 @@ void TUser::postDisable(int flag)
     try
     {
 	if( flag )
-	    SYS->db().at().dataDel(fullDB(),owner().nodePath()+tbl(),*this);
-	//- Remove user from groups -
+	    SYS->db().at().dataDel(fullDB(),owner().nodePath()+tbl(),*this,true);
+	//> Remove user from groups
 	vector<string> gls;
 	owner().usrGrpList(name(),gls);
 	for(int i_g = 0; i_g < gls.size(); i_g++)
@@ -353,13 +333,13 @@ void TUser::postDisable(int flag)
     { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
 }
 
-string TUser::tbl( )
-{
-    return owner().subId()+"_user";
-}
+TSecurity &TUser::owner( )	{ return *(TSecurity*)nodePrev(); }
+
+string TUser::tbl( )		{ return owner().subId()+"_user"; }
 
 void TUser::load_( )
 {
+    if( !SYS->chkSelDB(DB()) ) return;
     SYS->db().at().dataGet(fullDB(),owner().nodePath()+tbl(),*this);
 }
 
@@ -373,12 +353,13 @@ void TUser::cntrCmdProc( XMLNode *opt )
     //- Get page info -
     if( opt->name() == "info" )
     {
+	TCntrNode::cntrCmdProc(opt);
 	ctrMkNode("oscada_cntr",opt,-1,"/",_("User ")+name(),0664,name().c_str(),"Security");
 	if(picture().size()) ctrMkNode("img",opt,-1,"/ico","",0444);
 	if(ctrMkNode("area",opt,-1,"/prm",_("User")))
 	{
 	    ctrMkNode("fld",opt,-1,"/prm/name",cfg("NAME").fld().descr(),0444,"root","root",1,"tp","str");
-	    ctrMkNode("fld",opt,-1,"/prm/dscr",cfg("DESCR").fld().descr(),0664,name().c_str(),"Security",1,"tp","str");
+	    ctrMkNode("fld",opt,-1,"/prm/dscr",cfg("DESCR").fld().descr(),0664,name().c_str(),"Security",2,"tp","str","len","50");
 	    ctrMkNode("img",opt,-1,"/prm/pct",cfg("PICTURE").fld().descr(),0664,name().c_str(),"Security",1,"v_sz","100");
 	    ctrMkNode("fld",opt,-1,"/prm/db",_("User DB"),0664,"root",SYS->db().at().subId().c_str(),4,"tp","str","dest","select","select","/db/list",
 		"help",_("DB address in format [<DB module>.<DB name>].\nFor use main work DB set '*.*'."));
@@ -472,18 +453,18 @@ void TGroup::postDisable(int flag)
     try
     {
 	if( flag )
-	    SYS->db().at().dataDel(fullDB(),owner().nodePath()+tbl(),*this);
+	    SYS->db().at().dataDel(fullDB(),owner().nodePath()+tbl(),*this,true);
     }catch(TError err)
     { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
 }
 
-string TGroup::tbl( )
-{
-    return owner().subId()+"_grp";
-}
+TSecurity &TGroup::owner( )	{ return *(TSecurity*)nodePrev(); }
+
+string TGroup::tbl( )		{ return owner().subId()+"_grp"; }
 
 void TGroup::load_( )
 {
+    if( !SYS->chkSelDB(DB()) ) return;
     SYS->db().at().dataGet(fullDB(),owner().nodePath()+tbl(),*this);
 }
 
@@ -524,10 +505,11 @@ void TGroup::cntrCmdProc( XMLNode *opt )
     //- Get page info -
     if( opt->name() == "info" )
     {
+	TCntrNode::cntrCmdProc(opt);
 	ctrMkNode("oscada_cntr",opt,-1,"/",_("Group ")+name(),0664,"root",owner().subId().c_str());
 	ctrMkNode("area",opt,-1,"/prm",_("Group"));
 	ctrMkNode("fld",opt,-1,"/prm/name",cfg("NAME").fld().descr(),0444,"root",owner().subId().c_str(),1,"tp","str");
-	ctrMkNode("fld",opt,-1,"/prm/dscr",cfg("DESCR").fld().descr(),0664,"root",owner().subId().c_str(),1,"tp","str");
+	ctrMkNode("fld",opt,-1,"/prm/dscr",cfg("DESCR").fld().descr(),0664,"root",owner().subId().c_str(),2,"tp","str","len","50");
 	ctrMkNode("fld",opt,-1,"/prm/db",_("User group DB"),0664,"root",SYS->db().at().subId().c_str(),4,"tp","str","dest","select","select","/db/list",
 	    "help",_("DB address in format [<DB module>.<DB name>].\nFor use main work DB set '*.*'."));
 	ctrMkNode("list",opt,-1,"/prm/users",cfg("USERS").fld().descr(),0664,"root",owner().subId().c_str(),2,"tp","str","s_com","add,del");
