@@ -29,8 +29,7 @@
 //*************************************************
 //* TSubSYS                                       *
 //*************************************************
-TSubSYS::TSubSYS( const char *id, const char *name, bool modi ) :
-	m_id(id), m_name(name), m_mod_sys(modi), m_mod(-1)
+TSubSYS::TSubSYS( const char *id, const char *name, bool modi ) : m_id(id), m_name(name), m_mod_sys(modi), m_mod(-1), mStart(false)
 {
     if(subModule()) m_mod = grpAdd("mod_");
 }
@@ -40,29 +39,28 @@ TSubSYS::~TSubSYS(  )
     nodeDelAll();
 }
 
-string TSubSYS::subName()
-{
-    return m_name.size()?_(m_name.c_str()):m_id;
-}
+TSYS &TSubSYS::owner( )		{ return *(TSYS*)nodePrev(); }
+
+string TSubSYS::subName( )	{ return m_name.size()?_(m_name.c_str()):m_id; }
 
 void TSubSYS::modList( vector<string> &list )
 {
-    if( !subModule() ) throw TError(nodePath().c_str(),_("No modules subsystem!"));
+    if( !subModule() ) throw TError(nodePath().c_str(),_("The subsystem is not modular!"));
     chldList(m_mod,list);
 }
 
 bool TSubSYS::modPresent( const string &name )
 {
-    if( !subModule() ) throw TError(nodePath().c_str(),_("No modules subsystem!")); 
+    if( !subModule() ) throw TError(nodePath().c_str(),_("The subsystem is not modular!")); 
     return chldPresent(m_mod,name); 
 }
 
 void TSubSYS::modAdd( TModule *modul )
 {
-    if( !subModule() ) throw TError(nodePath().c_str(),_("No modules subsystem!"));
+    if( !subModule() ) throw TError(nodePath().c_str(),_("The subsystem is not modular!"));
     if( chldPresent(m_mod,modul->modId()) ) return;
     chldAdd(m_mod,modul);
-#if OSC_DEBUG
+#if OSC_DEBUG >= 1
     vector<string> list;
     modul->modInfo( list );
     for( unsigned i_opt = 0; i_opt < list.size(); i_opt++)
@@ -72,14 +70,14 @@ void TSubSYS::modAdd( TModule *modul )
 
 void TSubSYS::modDel( const string &name )
 {
-    if( !subModule() ) throw TError(nodePath().c_str(),_("No modules subsystem!"));
-    mess_info(nodePath().c_str(),_("Disconnect modul <%s>!"),name.c_str());
+    if( !subModule() ) throw TError(nodePath().c_str(),_("The subsystem is not modular!"));
+    mess_info(nodePath().c_str(),_("Disconnect module <%s>!"),name.c_str());
     chldDel(m_mod,name);
 }
 
 AutoHD<TModule> TSubSYS::modAt( const string &name )
 {
-    if( !subModule() ) throw TError(nodePath().c_str(),_("No modules subsystem!")); 
+    if( !subModule() ) throw TError(nodePath().c_str(),_("The subsystem is not modular!")); 
     return chldAt(m_mod,name);
 }
 
@@ -101,8 +99,10 @@ void TSubSYS::subStart( )
 	catch(TError err)
 	{
 	    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
-	    mess_err(nodePath().c_str(),_("Start module '%s' is error."),list[i_m].c_str());
+	    mess_err(nodePath().c_str(),_("Start module '%s' error."),list[i_m].c_str());
 	}
+
+    mStart = true;
 }
 
 void TSubSYS::subStop( )
@@ -114,9 +114,11 @@ void TSubSYS::subStop( )
 	try{ modAt(list[i_m]).at().modStop( ); }
 	catch(TError err)
 	{
-	    mess_err(err.cat.c_str(),"%s",err.mess.c_str()); 
-	    mess_err(nodePath().c_str(),_("Stop module '%s' is error."),list[i_m].c_str());
+	    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
+	    mess_err(nodePath().c_str(),_("Stop module '%s' error."),list[i_m].c_str());
 	}
+
+    mStart = false;
 }
 
 void TSubSYS::cntrCmdProc( XMLNode *opt )
@@ -124,6 +126,7 @@ void TSubSYS::cntrCmdProc( XMLNode *opt )
     //Get page info
     if( opt->name() == "info" )
     {
+	TCntrNode::cntrCmdProc(opt);
 	ctrMkNode("oscada_cntr",opt,-1,"/",_("Subsystem: ")+subName(),0444);
 	ctrMkNode("branches",opt,-1,"/br","",0444);
 	if(TUIS::icoPresent(subId()))	ctrMkNode("img",opt,-1,"/ico","",0444);
