@@ -45,6 +45,7 @@ class TSubSYS : public TCntrNode
 
 	string subId( )		{ return m_id; }
 	string subName( );
+	bool subStartStat( )	{ return mStart; }
 
 	bool subModule( )	{ return m_mod_sys; }	//Module subsystem
 
@@ -61,12 +62,15 @@ class TSubSYS : public TCntrNode
 	void modDel( const string &name );
 	AutoHD<TModule> modAt( const string &name );
 
-	TSYS &owner( )		{ return *(TSYS *)nodePrev(); }
+	TSYS &owner( );
 
     protected:
 	//Protected methods
 	void cntrCmdProc( XMLNode *opt );       //Control interface command process
 
+	//Private attributes
+	bool	mStart;
+
     private:
 	//Private methods
 	string nodeName( )	{ return subId(); }
