@@ -48,18 +48,17 @@ TSYS	*SYS;
 bool TSYS::finalKill = false;
 
 TSYS::TSYS( int argi, char ** argb, char **env ) :
-    m_confFile("/etc/oscada.xml"), m_id("EmptySt"), m_name("Empty Station"),
-    m_user("root"),argc(argi), envp((const char **)env), argv((const char **)argb), stop_signal(0),
-    m_sysOptCfg(false), mWorkDB(""), mSaveAtExit(false), mSavePeriod(0)
+    mConfFile("/etc/oscada.xml"), mId("EmptySt"), mName(_("Empty Station")), mIcoDir("./icons/"), mModDir("./"),
+    mUser("root"),argc(argi), envp((const char **)env), argv((const char **)argb), mStopSignal(0), mWorkDB(""), mSaveAtExit(false), mSavePeriod(0)
 {
     finalKill = false;
     SYS = this;		//Init global access value
-    m_subst = grpAdd("sub_",true);
+    mSubst = grpAdd("sub_",true);
     nodeEn();
 
     Mess = new TMess();
 
-    if( getenv("USER") ) m_user = getenv("USER");
+    if( getenv("USER") ) mUser = getenv("USER");
 
     //Init system clock
     clkCalc();
@@ -98,18 +97,19 @@ string TSYS::workDir( )
     return getcwd(buf,sizeof(buf));
 }
 
-string TSYS::setWorkDir( const string &wdir )
+void TSYS::setWorkDir( const string &wdir )
 {
-    chdir( wdir.c_str() );
+    if( chdir(wdir.c_str()) != 0 )
+	mess_err(nodePath().c_str(),_("Change work directory to '%s' error: %s"),wdir.c_str(),strerror(errno));
     modif( );
 }
 
 string TSYS::int2str( int val, TSYS::IntView view )
 {
     char buf[STR_BUF_LEN];
-    if( view == TSYS::Dec )      snprintf(buf,sizeof(buf),"%d",val);
-    else if( view == TSYS::Oct ) snprintf(buf,sizeof(buf),"%o",val);
-    else if( view == TSYS::Hex ) snprintf(buf,sizeof(buf),"%x",val);
+    if( view == TSYS::Dec )		snprintf(buf,sizeof(buf),"%d",val);
+    else if( view == TSYS::Oct )	snprintf(buf,sizeof(buf),"%o",val);
+    else if( view == TSYS::Hex )	snprintf(buf,sizeof(buf),"%x",val);
 
     return buf;
 }
@@ -117,9 +117,9 @@ string TSYS::int2str( int val, TSYS::IntView view )
 string TSYS::uint2str( unsigned val, IntView view )
 {
     char buf[STR_BUF_LEN];
-    if( view == TSYS::Dec )      snprintf(buf,sizeof(buf),"%u",val);
-    else if( view == TSYS::Oct ) snprintf(buf,sizeof(buf),"%o",val);
-    else if( view == TSYS::Hex ) snprintf(buf,sizeof(buf),"%x",val);
+    if( view == TSYS::Dec )		snprintf(buf,sizeof(buf),"%u",val);
+    else if( view == TSYS::Oct )	snprintf(buf,sizeof(buf),"%o",val);
+    else if( view == TSYS::Hex )	snprintf(buf,sizeof(buf),"%x",val);
 
     return buf;
 }
@@ -127,17 +127,17 @@ string TSYS::uint2str( unsigned val, IntView view )
 string TSYS::ll2str( long long val, IntView view )
 {
     char buf[STR_BUF_LEN];
-    if( view == TSYS::Dec )      snprintf(buf,sizeof(buf),"%lld",val);
-    else if( view == TSYS::Oct ) snprintf(buf,sizeof(buf),"%llo",val);
-    else if( view == TSYS::Hex ) snprintf(buf,sizeof(buf),"%llx",val);
+    if( view == TSYS::Dec )		snprintf(buf,sizeof(buf),"%lld",val);
+    else if( view == TSYS::Oct )	snprintf(buf,sizeof(buf),"%llo",val);
+    else if( view == TSYS::Hex )	snprintf(buf,sizeof(buf),"%llx",val);
 
     return buf;
 }
 
-string TSYS::real2str( double val, int prec )
+string TSYS::real2str( double val, int prec, char tp )
 {
     char buf[STR_BUF_LEN];
-    snprintf(buf,sizeof(buf),"%.*g",prec,val);
+    snprintf(buf,sizeof(buf),(tp=='g')?"%.*g":"%.*f",prec,val);
 
     return buf;
 }
@@ -155,23 +155,37 @@ void *TSYS::str2addr( const string &str )
     return (void *)strtoul(str.c_str(),NULL,16);
 }
 
-bool TSYS::strEmpty( const string &val )
+string TSYS::strNoSpace( const string &val )
 {
-    if( !val.size() )	return true;
+    int beg = -1, end = -1;
+
     for( int i_s = 0; i_s < val.size(); i_s++ )
 	if( val[i_s] != ' ' && val[i_s] != '\n' && val[i_s] != '\t' )
-	    return false;
+	{
+	    if( beg < 0 ) beg = i_s;
+	    end = i_s;
+	}
 
-    return true;
+    return (beg>=0) ? val.substr(beg,end-beg+1) : "";
+}
+
+string TSYS::strMess( const char *fmt, ... )
+{
+    char str[STR_BUF_LEN];
+    va_list argptr;
+
+    va_start(argptr,fmt);
+    vsnprintf(str,sizeof(str),fmt,argptr);
+    va_end(argptr);
+
+    return str;
 }
 
 string TSYS::optDescr( )
 {
-    char s_buf[STR_BUF_LEN];
     utsname buf;
-
     uname(&buf);
-    snprintf(s_buf,sizeof(s_buf),_(
+    return TSYS::strMess(_(
 	"***************************************************************************\n"
 	"********** %s v%s (%s-%s). *********\n"
 	"***************************************************************************\n\n"
@@ -180,7 +194,7 @@ string TSYS::optDescr( )
 	"===========================================================================\n"
 	"-h, --help             Info message about system options.\n"
 	"    --Config=<path>    Config file path.\n"
-	"    --Station=<name>   Station name.\n"
+	"    --Station=<id>     Station identifier.\n"
 	"    --demon            Start into demon mode.\n"
 	"    --MessLev=<level>  Process messages <level> (0-7).\n"
 	"    --log=<direct>     Direct messages to:\n"
@@ -189,21 +203,21 @@ string TSYS::optDescr( )
 	"                         <direct> & 4 - stderr;\n"
 	"                         <direct> & 8 - archive.\n"
 	"----------- The config file station <%s> parameters -----------\n"
+	"StName     <nm>	Station name.\n"
+	"WorkDB     <Type.Name> Work DB (type and name).\n"
 	"Workdir    <path>	Work directory.\n"
+	"IcoDir     <path>	Icons directory.\n"
+	"ModDir     <path>	Modules directory.\n"
 	"MessLev    <level>     Messages <level> (0-7).\n"
 	"LogTarget  <direction> Direct messages to:\n"
 	"                           <direct> & 1 - syslogd;\n"
 	"                           <direct> & 2 - stdout;\n"
 	"                           <direct> & 4 - stderr;\n"
 	"                           <direct> & 8 - archive.\n"
-	"SysLang    <lang>	Internal language.\n"
-	"WorkDB     <Type.Name> Work DB (type and name).\n"
+	"Lang2CodeBase <lang>	Base language for variable texts translation, two symbols code.\n"
 	"SaveAtExit <true>      Save system at exit.\n"
-	"SavePeriod <sec>	Save system period.\n"
-	"SYSOptCfg  <true>      Get system options from DB.\n\n"),
+	"SavePeriod <sec>	Save system period.\n\n"),
 	PACKAGE_NAME,VERSION,buf.sysname,buf.release,nodePath().c_str());
-
-    return s_buf;
 }
 
 bool TSYS::cfgFileLoad()
@@ -228,20 +242,19 @@ bool TSYS::cfgFileLoad()
 	switch(next_opt)
 	{
 	    case 'h':
-		fprintf(stdout,optDescr().c_str());
+		fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str());
 		Mess->setMessLevel(7);
 		cmd_help = true;
 		break;
-	    case 'f': m_confFile = optarg; break;
-	    case 's': m_id = optarg; break;
+	    case 'f': mConfFile = optarg; break;
+	    case 's': mId = optarg; break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
 
     //Load config file
-    int hd = open(m_confFile.c_str(),O_RDONLY);
-    if( hd < 0 )
-	mess_err(nodePath().c_str(),_("Config file <%s> error: %s"),m_confFile.c_str(),strerror(errno));
+    int hd = open(mConfFile.c_str(),O_RDONLY);
+    if( hd < 0 ) mess_err(nodePath().c_str(),_("Config file <%s> error: %s"),mConfFile.c_str(),strerror(errno));
     else
     {
 	int cf_sz = lseek(hd,0,SEEK_END);
@@ -254,27 +267,27 @@ bool TSYS::cfgFileLoad()
 	free(buf);
 	try
 	{
-	    root_n.load(s_buf);
-	    if( root_n.name() == "OpenSCADA" )
+	    ResAlloc res(nodeRes(),true);
+	    rootN.load(s_buf);
+	    if( rootN.name() == "OpenSCADA" )
 	    {
 		XMLNode *stat_n = NULL;
-		for( int i_st = root_n.childSize()-1; i_st >= 0; i_st-- )
-		    if( root_n.childGet(i_st)->name() == "station" )
+		for( int i_st = rootN.childSize()-1; i_st >= 0; i_st-- )
+		    if( rootN.childGet(i_st)->name() == "station" )
 		    {
-			stat_n = root_n.childGet(i_st);
-			if( stat_n->attr("id") == m_id ) break;
+			stat_n = rootN.childGet(i_st);
+			if( stat_n->attr("id") == mId ) break;
 		    }
-		if( stat_n && stat_n->attr("id") != m_id )
+		if( stat_n && stat_n->attr("id") != mId )
 		{
-		    mess_warning(nodePath().c_str(),_("Station <%s> into config file no present. Use <%s> station config!"),
-			m_id.c_str(), stat_n->attr("id").c_str() );
-		    m_id	= stat_n->attr("id");
-		    m_name	= stat_n->attr("name");
+                    mess_warning(nodePath().c_str(),_("Station <%s> is not present in the config file. Use <%s> station config!"),
+			mId.c_str(), stat_n->attr("id").c_str() );
+		    mId	= stat_n->attr("id");
 		}
-		if( !stat_n )	root_n.clear();
-	    } else root_n.clear();
-	    if( !root_n.childSize() )
-		mess_err(nodePath().c_str(),_("Config <%s> error!"),m_confFile.c_str());
+		if( !stat_n )	rootN.clear();
+	    } else rootN.clear();
+	    if( !rootN.childSize() )
+		mess_err(nodePath().c_str(),_("Config <%s> error!"),mConfFile.c_str());
 	}
 	catch( TError err ) { mess_err(nodePath().c_str(),_("Load config file error: %s"),err.mess.c_str() ); }
     }
@@ -285,12 +298,13 @@ bool TSYS::cfgFileLoad()
 void TSYS::cfgPrmLoad()
 {
     //System parameters
-    m_sysOptCfg = atoi(TBDS::genDBGet(nodePath()+"SYSOptCfg",TSYS::int2str(m_sysOptCfg),"root",true).c_str());
-    chdir(TBDS::genDBGet(nodePath()+"Workdir","","root",sysOptCfg()).c_str());
-
-    mWorkDB = TBDS::genDBGet(nodePath()+"WorkDB","*.*","root",sysOptCfg());
-    setSaveAtExit( atoi(TBDS::genDBGet(nodePath()+"SaveAtExit","0","root").c_str()) );
-    setSavePeriod( atoi(TBDS::genDBGet(nodePath()+"SavePeriod","0","root").c_str()) );
+    mName = TBDS::genDBGet(nodePath()+"StName",name(),"root",TBDS::UseTranslate);
+    mWorkDB = TBDS::genDBGet(nodePath()+"WorkDB","*.*","root",TBDS::OnlyCfg);
+    setWorkDir( TBDS::genDBGet(nodePath()+"Workdir").c_str() );
+    setIcoDir( TBDS::genDBGet(nodePath()+"IcoDir",icoDir()) );
+    setModDir( TBDS::genDBGet(nodePath()+"ModDir",modDir()) );
+    setSaveAtExit( atoi(TBDS::genDBGet(nodePath()+"SaveAtExit","0").c_str()) );
+    setSavePeriod( atoi(TBDS::genDBGet(nodePath()+"SavePeriod","0").c_str()) );
 }
 
 void TSYS::load_()
@@ -304,7 +318,7 @@ void TSYS::load_()
 
     if( first_load )
     {
-	//- Create subsystems -
+	//> Create subsystems
 	add( new TBDS() );
 	add( new TSecurity() );
 	add( new TTransportS() );
@@ -315,19 +329,20 @@ void TSYS::load_()
 	add( new TUIS() );
 	add( new TModSchedul() );
 
-	//- Load modules -
+	//> Load modules
 	modSchedul().at().load();
 	modSchedul().at().loadLibS();
 
-	//- First DB subsystem load -
+	//> First DB subsystem load
 	db().at().load();
+	if( !cmd_help ) modSchedul().at().modifG();	// For try reload from DB
 
-	//- Second load for load from generic DB -
-	cfgPrmLoad();
+	//> Second load for load from generic DB
 	Mess->load();
+	cfgPrmLoad();
     }
 
-    //- Direct load subsystems and modules -
+    //> Direct load subsystems and modules
     vector<string> lst;
     list(lst);
     for( unsigned i_a=0; i_a < lst.size(); i_a++ )
@@ -348,14 +363,16 @@ void TSYS::save_( )
 
     mess_info(nodePath().c_str(),_("Save!"));
 
-    //System parameters
+    //> System parameters
     getcwd(buf,sizeof(buf));
+    TBDS::genDBSet(nodePath()+"StName",mName,"root",TBDS::UseTranslate);
     TBDS::genDBSet(nodePath()+"Workdir",buf);
-    TBDS::genDBSet(nodePath()+"WorkDB",mWorkDB);
+    TBDS::genDBSet(nodePath()+"IcoDir",icoDir());
+    TBDS::genDBSet(nodePath()+"ModDir",modDir());
     TBDS::genDBSet(nodePath()+"SaveAtExit",TSYS::int2str(saveAtExit()));
     TBDS::genDBSet(nodePath()+"SavePeriod",TSYS::int2str(savePeriod()));
 
-    Mess->save();       //Messages load
+    Mess->save();	//Messages load
 }
 
 int TSYS::start(  )
@@ -374,13 +391,18 @@ int TSYS::start(  )
 
     cfgFileScan( true );
     unsigned int i_cnt = 1;
-    while( !stop_signal )
+    while( !mStopSignal )
     {
-	//- CPU frequency calc
+	//> CPU frequency calc
 	if( !(i_cnt%(10*1000/STD_WAIT_DELAY)) )	clkCalc( );
-	//- Config file change periodic check -
+	//> Config file change periodic check
 	if( !(i_cnt%(10*1000/STD_WAIT_DELAY)) )	cfgFileScan( );
-	//- Periodic changes saving to DB -
+	//> Periodic shared libraries checking
+	if( modSchedul( ).at().chkPer() && !(i_cnt%(modSchedul( ).at().chkPer()*1000/STD_WAIT_DELAY)) )
+	    modSchedul( ).at().libLoad(modDir(),true);
+	//> Old tables closing
+	if( !(i_cnt%(10*1000/STD_WAIT_DELAY)) )	db().at().closeOldTables();
+	//> Periodic changes saving to DB
 	if( savePeriod() && !(i_cnt%(savePeriod()*1000/STD_WAIT_DELAY)) )
 	    save();
 
@@ -398,12 +420,20 @@ int TSYS::start(  )
 	    mess_err(nodePath().c_str(),_("Error stop subsystem <%s>."),lst[i_a].c_str());
 	}
 
-    return stop_signal;
+    return mStopSignal;
 }
 
 void TSYS::stop( )
-{ 
-    stop_signal = SIGINT;
+{
+    mStopSignal = SIGINT;
+}
+
+bool TSYS::chkSelDB( const string& wDB )
+{
+    if( selDB().empty() ) return true;
+    if( selDB() == "<cfg>" && (wDB == "<cfg>" || TBDS::realDBName(wDB) == workDB()) ) return true;
+    if( SYS->selDB( ) == TBDS::realDBName(wDB) ) return true;
+    return false;
 }
 
 void TSYS::sighandler( int signal )
@@ -411,14 +441,14 @@ void TSYS::sighandler( int signal )
     switch(signal)
     {
 	case SIGINT:
-	    SYS->stop_signal=signal;
+	    SYS->mStopSignal=signal;
 	    break;
 	case SIGTERM:
-	    mess_warning(SYS->nodePath().c_str(),_("Have get a Terminate signal. Server been stoped!"));
-	    SYS->stop_signal=signal;
+            mess_warning(SYS->nodePath().c_str(),_("The Terminate signal is received.. Server is being stoped!"));
+	    SYS->mStopSignal=signal;
 	    break;
 	case SIGFPE:
-	    mess_warning(SYS->nodePath().c_str(),_("Float point exeption catch!"));
+            mess_warning(SYS->nodePath().c_str(),_("Floating point exception is catched!"));
 	    break;
 	case SIGCHLD:
 	{
@@ -435,7 +465,7 @@ void TSYS::sighandler( int signal )
 	    mess_emerg(SYS->nodePath().c_str(),_("Segmentation fault signal!"));
 	    break;
 	case SIGABRT:
-	    mess_emerg(SYS->nodePath().c_str(),_("OpenSCADA aborted!"));
+	    mess_emerg(SYS->nodePath().c_str(),_("OpenSCADA is aborted!"));
 	    break;
 	case SIGALRM:	break;
 	default:
@@ -445,22 +475,21 @@ void TSYS::sighandler( int signal )
 
 void TSYS::cfgFileScan( bool first )
 {
-    static string cfg_fl;
-    static struct stat f_stat;
+    struct stat f_stat;
 
-    struct stat f_stat_t;
-    bool   up = false;
+    if( stat(cfgFile().c_str(),&f_stat) != 0 ) return;
+    bool up = false;
+    if( rootCfgFl != cfgFile() || rootFlTm != f_stat.st_mtime ) up = true;
+    rootCfgFl = cfgFile();
+    rootFlTm = f_stat.st_mtime;
 
-    if( cfg_fl == cfgFile() )
+    if( up && !first )
     {
-	stat(cfg_fl.c_str(),&f_stat_t);
-	if( f_stat.st_mtime != f_stat_t.st_mtime ) up = true;
+	modifG();
+	setSelDB("<cfg>");
+	load();
+	setSelDB("");
     }
-    else up = true;
-    cfg_fl = cfgFile();
-    stat(cfg_fl.c_str(),&f_stat);
-
-    if( up && !first )	{ modifG(); load(); }
 }
 
 long long TSYS::curTime()
@@ -475,7 +504,7 @@ string TSYS::fNameFix( const string &fname )
     string tmp;
     char   buf[STR_BUF_LEN];
 
-    for( string::const_iterator it = fname.begin(); it != fname.end(); it++ )
+    for( string::const_iterator it = fname.begin(); it != fname.end(); ++it )
     {
 	if( *(it) == '.' && *(it+1) == '.' && *(it+2) == '/')
 	{
@@ -713,6 +742,33 @@ string TSYS::strEncode( const string &in, TSYS::Code tp, const string &symb )
 	    for( i_sz = 0; i_sz < sout.size(); i_sz++ )
 		if( sout[i_sz] == '%' ) { sout.replace(i_sz,1,"%%"); i_sz++; }
 	    break;
+	case TSYS::ID:
+	    sout.reserve(in.size());
+	    for( i_sz = 0; i_sz < in.size(); i_sz++ )
+		switch( in[i_sz] )
+		{
+		    case ' ': case '/': case '\\': case '&': case '(':
+		    case ')': case '[': case ']': case '!': case '~':
+		    case '`': case '@': case '%': case '^': case '-':
+		    case '+': case '=': case '*': case '{': case '}':
+		    case ':': case ';': case '"': case '\'': case '<':
+		    case '>': case '?':
+			sout+="_";	break;
+		    default:	sout+=in[i_sz];
+		}
+	    break;
+	case TSYS::Bin:
+	{
+	    string svl, evl;
+	    sout.reserve(in.size());
+	    for( int off = 0; (svl=TSYS::strSepParse(in,0,'\n',&off)).size(); )
+		for( int offE = 0; (evl=TSYS::strSepParse(svl,0,' ',&offE)).size(); )
+		    sout+=(char)strtol(evl.c_str(),NULL,16);
+	    break;
+	}
+	case TSYS::Reverse:
+	    for( i_sz = in.size()-1; i_sz >= 0; i_sz-- ) sout += in[i_sz];
+	    break;
     }
     return sout;
 }
@@ -772,6 +828,11 @@ string TSYS::strDecode( const string &in, TSYS::Code tp )
 		i_sz+=4;
 	    }
 	    break;
+	case TSYS::Bin:
+	    sout.reserve(in.size());
+	    for( i_sz = 0; i_sz < in.size(); i_sz++ )
+		sout += TSYS::strMess(((i_sz+1)%16)?"%0.2x ":"%0.2x\n",(unsigned char)in[i_sz]);
+	    break;
     }
 
     return sout;
@@ -849,6 +910,20 @@ long TSYS::HZ()
     return sysconf(_SC_CLK_TCK);
 }
 
+void TSYS::taskSleep( long long per )
+{
+    struct timespec sp_tm;
+
+    clock_gettime(CLOCK_REALTIME,&sp_tm);
+    long long pnt_tm = ( ((long long)sp_tm.tv_sec*1000000000+sp_tm.tv_nsec)/per + 1)*per;
+    do
+    {
+	sp_tm.tv_sec = pnt_tm/1000000000; sp_tm.tv_nsec = pnt_tm%1000000000;
+	if( clock_nanosleep(CLOCK_REALTIME,TIMER_ABSTIME,&sp_tm,NULL) )	return;
+	clock_gettime(CLOCK_REALTIME,&sp_tm);
+    }while( ((long long)sp_tm.tv_sec*1000000000+sp_tm.tv_nsec) < pnt_tm );
+}
+
 void TSYS::cntrCmdProc( XMLNode *opt )
 {
     char buf[STR_BUF_LEN];
@@ -856,6 +931,7 @@ void TSYS::cntrCmdProc( XMLNode *opt )
     //Get page info
     if( opt->name() == "info" )
     {
+	TCntrNode::cntrCmdProc(opt);
 	snprintf(buf,sizeof(buf),_("%s station: \"%s\""),PACKAGE_NAME,name().c_str());
 	ctrMkNode("oscada_cntr",opt,-1,"/",buf,0444);
 	if(ctrMkNode("branches",opt,-1,"/br","",0444))
@@ -863,7 +939,8 @@ void TSYS::cntrCmdProc( XMLNode *opt )
 	if(TUIS::icoPresent(id())) ctrMkNode("img",opt,-1,"/ico","",0444);
 	if(ctrMkNode("area",opt,-1,"/gen",_("Station"),0444))
 	{
-	    ctrMkNode("fld",opt,-1,"/gen/stat",_("Station"),0444,"root","root",1,"tp","str");
+	    ctrMkNode("fld",opt,-1,"/gen/id",_("ID"),R_R_R_,"root","root",1,"tp","str");
+	    ctrMkNode("fld",opt,-1,"/gen/stat",_("Station"),RWRWR_,"root","root",1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/gen/prog",_("Programm"),0444,"root","root",1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/gen/ver",_("Version"),0444,"root","root",1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/gen/host",_("Host name"),0444,"root","root",1,"tp","str");
@@ -873,14 +950,18 @@ void TSYS::cntrCmdProc( XMLNode *opt )
 	    ctrMkNode("fld",opt,-1,"/gen/clk_res",_("Realtime clock resolution (msec)"),0444,"root","root",1,"tp","real");
 	    ctrMkNode("fld",opt,-1,"/gen/in_charset",_("Internal charset"),0440,"root","root",1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/gen/config",_("Config file"),0440,"root","root",1,"tp","str");
-	    ctrMkNode("fld",opt,-1,"/gen/workdir",_("Work directory"),0440,"root","root",1,"tp","str");
+	    ctrMkNode("fld",opt,-1,"/gen/workdir",_("Work directory"),0660,"root","root",1,"tp","str");
+	    ctrMkNode("fld",opt,-1,"/gen/icodir",_("Icons directory"),0660,"root","root",1,"tp","str");
+	    ctrMkNode("fld",opt,-1,"/gen/moddir",_("Modules directory"),0660,"root","root",1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/gen/wrk_db",_("Work DB"),0664,"root",db().at().subId().c_str(),4,"tp","str","dest","select","select","/db/list",
 		"help",_("Work DB address in format [<DB module>.<DB name>].\nChange it field if you want save or reload all system from other DB."));
 	    ctrMkNode("fld",opt,-1,"/gen/saveExit",_("Save system at exit"),0664,"root","root",2,"tp","bool",
 		"help",_("Select for automatic system saving to DB on exit."));
 	    ctrMkNode("fld",opt,-1,"/gen/savePeriod",_("Save system period"),0664,"root","root",2,"tp","dec",
 		"help",_("Use no zero period (seconds) for periodic saving of changed systems parts to DB."));
-	    ctrMkNode("fld",opt,-1,"/gen/lang",_("Language"),0664,"root","root",1,"tp","str");
+	    ctrMkNode("fld",opt,-1,"/gen/lang",_("Language"),RWRWR_,"root","root",1,"tp","str");
+	    ctrMkNode("fld",opt,-1,"/gen/baseLang",_("Variable texts' base language"),RWRWR_,"root","root",4,"tp","str","dest","select","select","/gen/baseLangLs",
+		"help",_("Multilangual for variable texts support enabling by base language selection."));
 	    if(ctrMkNode("area",opt,-1,"/gen/mess",_("Messages"),0444))
 	    {
 		ctrMkNode("fld",opt,-1,"/gen/mess/lev",_("Least level"),0664,"root","root",3,
@@ -916,10 +997,15 @@ void TSYS::cntrCmdProc( XMLNode *opt )
 	utsname ubuf; uname(&ubuf);
 	opt->setText(string(ubuf.sysname)+"-"+ubuf.release);
     }
-    else if( a_path == "/gen/user" && ctrChkNode(opt) )	opt->setText(m_user);
+    else if( a_path == "/gen/user" && ctrChkNode(opt) )	opt->setText(mUser);
     else if( a_path == "/gen/prog" && ctrChkNode(opt) )	opt->setText(PACKAGE_NAME);
     else if( a_path == "/gen/ver" && ctrChkNode(opt) )	opt->setText(VERSION);
-    else if( a_path == "/gen/stat" && ctrChkNode(opt) )	opt->setText(name());
+    else if( a_path == "/gen/id" && ctrChkNode(opt) )	opt->setText(id());
+    else if( a_path == "/gen/stat" )
+    {
+	if( ctrChkNode(opt,"get",RWRWR_,"root","root",SEQ_RD) )	opt->setText(name());
+	if( ctrChkNode(opt,"set",RWRWR_,"root","root",SEQ_WR) )	setName(opt->text());
+    }
     else if( a_path == "/gen/frq" && ctrChkNode(opt) )	opt->setText(TSYS::real2str((float)sysClk()/1000000.,6));
     else if( a_path == "/gen/clk_res" && ctrChkNode(opt) )
     {
@@ -928,7 +1014,7 @@ void TSYS::cntrCmdProc( XMLNode *opt )
 	opt->setText(TSYS::real2str((float)tmval.tv_nsec/1000000.,4));
     }
     else if( a_path == "/gen/in_charset" && ctrChkNode(opt) )	opt->setText(Mess->charset());
-    else if( a_path == "/gen/config" && ctrChkNode(opt) )	opt->setText(m_confFile);
+    else if( a_path == "/gen/config" && ctrChkNode(opt) )	opt->setText(mConfFile);
     else if( a_path == "/gen/wrk_db" )
     {
 	if( ctrChkNode(opt,"get",0664,"root",db().at().subId().c_str(),SEQ_RD) ) opt->setText(mWorkDB);
@@ -949,12 +1035,34 @@ void TSYS::cntrCmdProc( XMLNode *opt )
 	if( ctrChkNode(opt,"get",0440,"root","root",SEQ_RD) )	opt->setText(workDir());
 	if( ctrChkNode(opt,"set",0440,"root","root",SEQ_WR) )	setWorkDir(opt->text().c_str());
     }
+    else if( a_path == "/gen/icodir" )
+    {
+	if( ctrChkNode(opt,"get",0440,"root","root",SEQ_RD) )	opt->setText(icoDir());
+	if( ctrChkNode(opt,"set",0440,"root","root",SEQ_WR) )	setIcoDir(opt->text().c_str());
+    }
+    else if( a_path == "/gen/moddir" )
+    {
+	if( ctrChkNode(opt,"get",0440,"root","root",SEQ_RD) )	opt->setText(modDir());
+	if( ctrChkNode(opt,"set",0440,"root","root",SEQ_WR) )	setModDir(opt->text().c_str());
+    }
     else if( a_path == "/gen/lang" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(Mess->lang());
 	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	Mess->setLang(opt->text());
     }
-    else if( a_path == "/gen/mess/lev" ) 
+    else if( a_path == "/gen/baseLang" )
+    {
+	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(Mess->lang2CodeBase());
+	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	Mess->setLang2CodeBase(opt->text());
+    }
+    else if( a_path == "/gen/baseLangLs" && ctrChkNode(opt) )
+    {
+	opt->childAdd("el")->setText(Mess->lang2Code());
+	if( !Mess->lang2CodeBase().empty() && Mess->lang2CodeBase() != Mess->lang2Code() )
+	    opt->childAdd("el")->setText(Mess->lang2CodeBase());
+	opt->childAdd("el")->setText("");
+    }
+    else if( a_path == "/gen/mess/lev" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(TSYS::int2str(Mess->messLevel()));
 	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	Mess->setMessLevel(atoi(opt->text().c_str()));
