@@ -22,6 +22,12 @@
 #ifndef TSYS_H
 #define TSYS_H
 
+//- Programm constants -
+#define PACKAGE_LICENSE	"GPL v2"
+#define PACKAGE_DESCR	"Open Supervisory Control And Data Acquisition"
+#define PACKAGE_AUTHOR	"Roman Savochenko"
+#define PACKAGE_SITE	"http://oscada.org.ua"
+
 //- Hardware independent types -
 typedef char si8;
 typedef short si16;
@@ -37,6 +43,7 @@ typedef unsigned long long ui64;
 #define STR_BUF_LEN	3000	// Len of string buffers (no string class)
 #define STD_WAIT_DELAY	100	// Standart wait dalay (ms)
 #define STD_WAIT_TM	10	// Standart timeouts length (s)
+#define BUF_ARCH_NM	"<buffer>"
 
 #include <math.h>
 #include <stdio.h>
@@ -70,7 +77,7 @@ class TSYS : public TCntrNode
 
     public:
 	//Data
-	enum Code	{ PathEl, HttpURL, Html, JavaSc, SQL, Custom, base64, FormatPrint };
+	enum Code	{ PathEl, HttpURL, Html, JavaSc, SQL, Custom, base64, FormatPrint, ID, Bin, Reverse };
 	enum IntView	{ Dec, Oct, Hex };
 	enum Errors
 	{
@@ -83,7 +90,8 @@ class TSYS : public TCntrNode
 	    DBOpenTable,	//open table error
 	    DBClose,		//close DB or table error
 	    DBTableEmpty,	//table is empty
-	    DBRowNoPresent	//no present of requested row
+	    DBRowNoPresent,	//no present of requested row
+	    DBReadOnly		//read only db
 	};
 
 	//Public methods
@@ -93,18 +101,19 @@ class TSYS : public TCntrNode
 	int  start( );
 	void stop( );
 
-	int stopSignal( )	{ return stop_signal; }
+	int stopSignal( )	{ return mStopSignal; }
 
-        //- Programms options -
-	string id( )		{ return m_id; }
-	string name( )		{ return m_name; }
-	string user( )		{ return m_user; }               //Run user name
+	//> Programms options
+	string id( )		{ return mId; }
+	string name( )		{ return mName; }
+	void setName( const string &vl )	{ mName = vl; modif(); }
+	string user( )		{ return mUser; }	//Run user name
 
-	void list( vector<string> &list )	{ chldList(m_subst,list); }
-	bool present( const string &name )	{ return chldPresent(m_subst,name); }
-	void add( TSubSYS *sub )		{ chldAdd(m_subst,sub); }
-	void del( const string &name )		{ chldDel(m_subst,name); }
-	AutoHD<TSubSYS> at( const string &name ){ return chldAt(m_subst,name); }
+	void list( vector<string> &list )	{ chldList(mSubst,list); }
+	bool present( const string &name )	{ return chldPresent(mSubst,name); }
+	void add( TSubSYS *sub )		{ chldAdd(mSubst,sub); }
+	void del( const string &name )		{ chldDel(mSubst,name); }
+	AutoHD<TSubSYS> at( const string &name ){ return chldAt(mSubst,name); }
 
 	AutoHD<TUIS>		ui( )		{ return at("UI"); }
 	AutoHD<TArchiveS>	archive( )	{ return at("Archive"); }
@@ -117,34 +126,37 @@ class TSYS : public TCntrNode
 	AutoHD<TSecurity>	security( )	{ return at("Security"); }
 
 	string workDir( );
-	string setWorkDir( const string &wdir );
+	string icoDir( )	{ return mIcoDir; }
+	string modDir( )	{ return mModDir; }
+	void setWorkDir( const string &wdir );
+	void setIcoDir( const string &idir )	{ mIcoDir = idir; modif(); }
+	void setModDir( const string &mdir )	{ mModDir = mdir; modif(); }
 
 	//- Config file functions -
-	string cfgFile( )	{ return m_confFile; }
-	XMLNode &cfgRoot( )	{ return root_n; }
+	string cfgFile( )	{ return mConfFile; }
+	XMLNode &cfgRoot( )	{ return rootN; }
 
-	//- Default DB -
 	string workDB( )	{ return mWorkDB; }
+	string selDB( )		{ return mSelDB; }
+	bool chkSelDB( const string& wDB );
 	void setWorkDB( const string &wdb )	{ mWorkDB = wdb; modifG(); }
+	void setSelDB( const string &vl )	{ mSelDB = vl; }
 	bool saveAtExit( )	{ return mSaveAtExit; }
 	void setSaveAtExit( bool vl )		{ mSaveAtExit = vl; modif(); }
 	int  savePeriod( )	{ return mSavePeriod; }
 	void setSavePeriod( int vl )		{ mSavePeriod = vl; modif(); }
 
-	//- Get system options from DB -
-	bool sysOptCfg( )	{ return m_sysOptCfg; }
-
 	string optDescr( );	//print comand line options
 
 	static void sighandler( int signal );
 
         //- Short time dimensions -
-	unsigned long long sysClk( )		{ return m_sysclc; }
+	unsigned long long sysClk( )		{ return mSysclc; }
 	void clkCalc( )
 	{
 	    unsigned long long st_pnt = shrtCnt( );
 	    usleep(100000);
-	    m_sysclc = 10*(shrtCnt( )-st_pnt);
+	    mSysclc = 10*(shrtCnt( )-st_pnt);
 	}
 	unsigned long long shrtCnt( )
 	{
@@ -158,6 +170,8 @@ class TSYS : public TCntrNode
 	//- Current system time (usec) -
 	static long long curTime( );
 
+	static void taskSleep( long long per );
+
 	//- Wait event with timeout support -
 	static bool eventWait( bool &m_mess_r_stat, bool exempl, const string &loc, time_t time = 0 );
 
@@ -165,7 +179,7 @@ class TSYS : public TCntrNode
 	static string int2str( int val, IntView view = Dec );
 	static string uint2str( unsigned val, IntView view = Dec );
 	static string ll2str( long long val, IntView view = Dec );
-	static string real2str( double val, int prec = 15 );
+	static string real2str( double val, int prec = 15, char tp = 'g' );
 	static double realRound( double val, int dig = 0, bool toint = false )
 	{
 	    double rez = floor(val*pow(10,dig)+0.5)/pow(10,dig);
@@ -177,15 +191,16 @@ class TSYS : public TCntrNode
 	static string addr2str( void *addr );
 	static void *str2addr( const string &str );
 
-	//- Path and string parse -
+	//> Path and string parse
 	static string fNameFix( const string &fname );
-	static bool strEmpty( const string &val );
+	static string strNoSpace( const string &val );
 	static string strSepParse( const string &str, int level, char sep, int *off = NULL );
 	static string pathLev( const string &path, int level, bool encode = true, int *off = NULL );
 	static string path2sepstr( const string &path, char sep = '.' );
 	static string sepstr2path( const string &str, char sep = '.' );
 	static string strEncode( const string &in, Code tp, const string &symb = " \t\n");
 	static string strDecode( const string &in, Code tp = Custom );
+	static string strMess( const char *fmt, ... );
 
 	static string strCompr( const string &in, int lev = -1 );
 	static string strUncompr( const string &in );
@@ -196,6 +211,8 @@ class TSYS : public TCntrNode
 	const char **argv;	//Comand line seting buffer.
 	const char **envp;	//System environment.
 
+	Res &nodeRes( )		{ return nRes; }
+
     protected:
 	//Protected methods
 	void load_( );
@@ -212,23 +229,27 @@ class TSYS : public TCntrNode
 	static unsigned char getBase64Code( unsigned char asymb );
 
 	//Private attributes
-	string	m_user,		// A owner user name!
-	 	m_confFile,	// Config file name
-		m_id,		// Station id
-		m_name;		// Station name
+	string	mUser,		// A owner user name!
+	 	mConfFile,	// Config file name
+		mId,		// Station id
+		mName,		// Station name
+		mIcoDir,	// Icons directory
+		mModDir;	// Modules directory
 
-	bool	m_sysOptCfg;	// Get system options from config only
-
-	string	mWorkDB;	// Work DB
+	string	mWorkDB,mSelDB;	// Work and selected DB
 	bool	mSaveAtExit;	// Save at exit
 	int	mSavePeriod;	// Save period (s) for periodic system saving to DB
 
-	XMLNode root_n;		// Root of the config file tree
+	XMLNode rootN;		// Root of the config file tree
+	string	rootCfgFl;	// Root node's config file name
+	time_t	rootFlTm;	// Root node's config file's modify time
 
-	int	stop_signal,	// Stop station signal
-		m_subst;	// Subsystem tree id
+	int	mStopSignal,	// Stop station signal
+		mSubst;		// Subsystem tree id
 
-	unsigned long long m_sysclc;
+	Res	nRes;
+
+	unsigned long long mSysclc;
 };
 
 extern TSYS *SYS;
