@@ -38,8 +38,8 @@ TTipDAQ::TTipDAQ( )
     m_cntr = grpAdd("cntr_");
 
     fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key|TFld::NoWrite,"20") );
-    fldAdd( new TFld("NAME",_("Name"),TFld::String,0,"50") );
-    fldAdd( new TFld("DESCR",_("Description"),TFld::String,TFld::FullText,"300") );
+    fldAdd( new TFld("NAME",_("Name"),TFld::String,TCfg::TransltText,"50") );
+    fldAdd( new TFld("DESCR",_("Description"),TFld::String,TFld::FullText|TCfg::TransltText,"300") );
     fldAdd( new TFld("ENABLE",_("To enable"),TFld::Boolean,0,"1","0") );
     fldAdd( new TFld("START",_("To start"),TFld::Boolean,0,"1","0") );
 }
@@ -55,6 +55,21 @@ TTipDAQ::~TTipDAQ( )
     }
 }
 
+TDAQS &TTipDAQ::owner( )	{ return *(TDAQS*)nodePrev(); }
+
+void TTipDAQ::postEnable( int flag )
+{
+    TModule::postEnable(flag);
+
+    if( redntAllow( ) )
+    {
+	fldAdd( new TFld("REDNT",_("Redundant"),TFld::Integer,TFld::Selected,"1","0",
+	    (TSYS::int2str(TController::Off)+";"+TSYS::int2str(TController::Asymmetric)+";"+TSYS::int2str(TController::Symmetric)).c_str(),
+	    _("Off;Asymmetric;Symmetric")) );
+	fldAdd( new TFld("REDNT_RUN",_("Preferable run"),TFld::String,0,"20","<high>") );
+    }
+}
+
 void TTipDAQ::modStart( )
 {
     vector<string> lst;
@@ -73,7 +88,7 @@ void TTipDAQ::modStart( )
 void TTipDAQ::modStop( )
 {
     vector<string> lst;
-    //- Stop all controllers -
+    //> Stop all controllers
     list(lst);
     for(int i_l=0; i_l < lst.size(); i_l++)
         at(lst[i_l]).at().stop( );
@@ -103,13 +118,13 @@ int TTipDAQ::tpParmAdd( const char *id, const char *n_db, const char *name )
 {
     if( tpPrmPresent(id) )	return tpPrmToId(id);
 
-    //- Add type -
+    //> Add type
     int i_t = paramt.size();
     paramt.push_back(new TTipParam(id, name, n_db) );
-    //- Add structure fields -
+    //> Add structure fields
     paramt[i_t]->fldAdd( new TFld("SHIFR",_("ID"),TFld::String,TCfg::Key|TFld::NoWrite,"20") );
-    paramt[i_t]->fldAdd( new TFld("NAME",_("Name"),TFld::String,0,"50") );
-    paramt[i_t]->fldAdd( new TFld("DESCR",_("Description"),TFld::String,TFld::FullText,"200") );
+    paramt[i_t]->fldAdd( new TFld("NAME",_("Name"),TFld::String,TCfg::TransltText,"50") );
+    paramt[i_t]->fldAdd( new TFld("DESCR",_("Description"),TFld::String,TFld::FullText|TCfg::TransltText,"200") );
     paramt[i_t]->fldAdd( new TFld("EN",_("To enable"),TFld::Boolean,TCfg::NoVal,"1","0") );
 
     return i_t;
@@ -119,7 +134,7 @@ int TTipDAQ::tpPrmToId( const string &name_t)
 {
     for(unsigned i_t=0; i_t < paramt.size(); i_t++)
 	if(paramt[i_t]->name == name_t) return i_t;
-    throw TError(nodePath().c_str(),_("Parameter type is no present."));
+    throw TError(nodePath().c_str(),_("Parameter type is not present."));
 }
 
 TController *TTipDAQ::ContrAttach( const string &name, const string &daq_db )
@@ -127,23 +142,23 @@ TController *TTipDAQ::ContrAttach( const string &name, const string &daq_db )
 	throw TError(nodePath().c_str(),"Error attach new controller %s.",name.c_str()); 
 }
 
-string TTipDAQ::compileFunc( const string &lang, TFunction &fnc_cfg, const string &prog_text )
+string TTipDAQ::compileFunc( const string &lang, TFunction &fnc_cfg, const string &prog_text, const string &usings )
 {
-	throw TError(nodePath().c_str(),"Module is no support of function compile languages.");
+	throw TError(nodePath().c_str(),"Module doesn't support the function for compilation programming languages.");
 }
 
 void TTipDAQ::cntrCmdProc( XMLNode *opt )
 {
-    //- Get page info -
+    //> Get page info
     if( opt->name() == "info" )
     {
 	TModule::cntrCmdProc(opt);
-	ctrMkNode("grp",opt,-1,"/br/cntr_",_("Controller"),0664,"root","root",1,"idm","1");
+	ctrMkNode("grp",opt,-1,"/br/cntr_",_("Controller"),0664,"root","root",2,"idm","1","idSz","20");
 	if(ctrMkNode("area",opt,0,"/tctr",_("Controllers")))
-	    ctrMkNode("list",opt,-1,"/tctr/ctr",_("Controllers"),0664,"root","root",4,"tp","br","idm","1","s_com","add,del","br_pref","cntr_");
+	    ctrMkNode("list",opt,-1,"/tctr/ctr",_("Controllers"),0664,"root","root",5,"tp","br","idm","1","s_com","add,del","br_pref","cntr_","idSz","20");
 	return;
     }
-    //- Process command to page -
+    //> Process command to page
     string a_path = opt->attr("path");
     if( a_path == "/br/cntr_" || a_path == "/tctr/ctr" )
     {
@@ -156,8 +171,8 @@ void TTipDAQ::cntrCmdProc( XMLNode *opt )
 	}
 	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )
 	{
-	    add(opt->attr("id"));
-	    at(opt->attr("id")).at().setName(opt->text());
+	    string vid = TSYS::strEncode(opt->attr("id"),TSYS::ID);
+	    add(vid); at(vid).at().setName(opt->text());
 	}
 	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )	chldDel(m_cntr,opt->attr("id"),-1,1);
     }
