@@ -45,10 +45,12 @@ class TTransportIn : public TCntrNode, public TConfig
 	TCntrNode &operator=( TCntrNode &node );
 
 	const string &id( )	{ return m_id; }
+	string workId( );
 	string name( );
 	string dscr( )		{ return m_dscr; }
 	string addr( )		{ return m_addr; }
 	string protocol( )	{ return m_prot; }
+	virtual string getStatus( );
 
 	bool toStart( ) 	{ return m_start; }
 	bool startStat( )	{ return run_st; }
@@ -57,18 +59,18 @@ class TTransportIn : public TCntrNode, public TConfig
 	string tbl( );
 	string fullDB( )	{ return DB()+'.'+tbl(); }
 
-	void setName( const string &inm )	{ m_name = inm; modif(); }
-	void setDscr( const string &idscr )	{ m_dscr = idscr; modif(); }
-	void setAddr( const string &addr )	{ m_addr = addr; modif(); }
-	void setProtocol( const string &prt )	{ m_prot = prt; modif(); }
-	void setToStart( bool val )             { m_start = val; modif(); }
+	void setName( const string &inm )		{ m_name = inm; modif(); }
+	void setDscr( const string &idscr )		{ m_dscr = idscr; modif(); }
+	virtual void setAddr( const string &addr )	{ m_addr = addr; modif(); }
+	void setProtocol( const string &prt )		{ m_prot = prt; modif(); }
+	void setToStart( bool val )			{ m_start = val; modif(); }
 
-	void setDB( const string &vl )          { m_db = vl; modifG(); }
+	void setDB( const string &vl )			{ m_db = vl; modifG(); }
 
 	virtual void start( )	{ };
 	virtual void stop( )	{ };
 
-	TTipTransport &owner( )	{ return *(TTipTransport*)nodePrev(); }
+	TTipTransport &owner( );
 
     protected:
 	//Methods
@@ -110,6 +112,7 @@ class TTransportOut : public TCntrNode, public TConfig
 	TCntrNode &operator=( TCntrNode &node );
 
 	const string &id( )	{ return m_id; }
+	string	workId( );
 	string	name( );
 	string	dscr( )		{ return m_dscr; }
 	string	addr( )		{ return m_addr; }
@@ -117,19 +120,20 @@ class TTransportOut : public TCntrNode, public TConfig
 	int	prm2( )		{ return m_prm2; }
 	bool	toStart()	{ return m_start; }
 	bool	startStat()	{ return run_st; }
+	virtual	string getStatus( );
 
 	string DB( )		{ return m_db; }
 	string tbl( );
 	string fullDB( )	{ return DB()+'.'+tbl(); }
 
-	void setName( const string &inm )	{ m_name = inm; modif(); }
-	void setDscr( const string &idscr )	{ m_dscr = idscr; modif(); }
-	void setAddr( const string &addr )	{ m_addr = addr; modif(); }
-	void setPrm1( int vl )			{ m_prm1 = vl; }
-	void setPrm2( int vl )			{ m_prm2 = vl; }
-	void setToStart( bool val )		{ m_start = val; modif(); }
+	void setName( const string &inm )		{ m_name = inm; modif(); }
+	void setDscr( const string &idscr )		{ m_dscr = idscr; modif(); }
+	virtual void setAddr( const string &addr )	{ m_addr = addr; modif(); }
+	void setPrm1( int vl )				{ m_prm1 = vl; }
+	void setPrm2( int vl )				{ m_prm2 = vl; }
+	void setToStart( bool val )			{ m_start = val; modif(); }
 
-	void setDB( const string &vl )		{ m_db = vl; modifG(); }
+	void setDB( const string &vl )			{ m_db = vl; modifG(); }
 
 	virtual void start( )			{ };
 	virtual void stop( )			{ };
@@ -137,9 +141,9 @@ class TTransportOut : public TCntrNode, public TConfig
 	virtual int messIO( const char *obuf, int len_ob, char *ibuf = NULL, int len_ib = 0, int time = 0 )
 	{ return 0; }
 
-	string messProtIO( const string &in, const string &prot );
+	void messProtIO( XMLNode &io, const string &prot );
 
-	TTipTransport &owner( )	{ return *(TTipTransport*)nodePrev(); }
+	TTipTransport &owner( );
 
     protected:
 	//Methods
@@ -152,7 +156,7 @@ class TTransportOut : public TCntrNode, public TConfig
 	void save_( );
 
 	//Attributes
-	bool    run_st;
+	bool	run_st;
 
     private:
 	//Methods
@@ -182,21 +186,21 @@ class TTipTransport: public TModule
 	TTipTransport( );
 	virtual ~TTipTransport( );
 
-	//- Input transports -
+	//> Input transports
 	void inList( vector<string> &list )			{ chldList(m_in,list); }
 	bool inPresent( const string &name )			{ return chldPresent(m_in,name); }
 	void inAdd( const string &name, const string &db = "*.*" );
 	void inDel( const string &name, bool complete = false )	{ chldDel(m_in,name,-1,complete); }
 	AutoHD<TTransportIn> inAt( const string &name )		{ return chldAt(m_in,name); }
 
-	//- Output transports -
+	//> Output transports
 	void outList( vector<string> &list )			{ chldList(m_out,list); }
 	bool outPresent( const string &name )			{ return chldPresent(m_out,name); }
 	void outAdd( const string &name, const string &idb = "*.*" );
 	void outDel( const string &name, bool complete = false ){ chldDel(m_out,name,-1,complete); }
 	AutoHD<TTransportOut> outAt( const string &name )	{ return chldAt(m_out,name); }
 
-	TTransportS &owner( )					{ return (TTransportS&)TModule::owner(); }
+	TTransportS &owner( );
 
     protected:
 	//Methods
@@ -244,8 +248,10 @@ class TTransportS : public TSubSYS
 	~TTransportS( );
 
 	int subVer( )			{ return VER_TR; }
+	void inTrList( vector<string> &ls );
+	void outTrList( vector<string> &ls );
 
-	//- External hosts -
+	//> External hosts
 	bool sysHost( )			{ return sys_host; }
 	void setSysHost( bool vl )	{ sys_host = vl; }
 	string extHostsDB( );
@@ -256,6 +262,9 @@ class TTransportS : public TSubSYS
 	void extHostSet( const ExtHost &host );
 	void extHostDel( const string &user, const string &id );
 
+	//> Request to remote or local OpenSCADA control interface
+	int cntrIfCmd( XMLNode &node, const string &senderPref, const string &user = "" );
+
 	void subStart( );
 	void subStop( );
 
