@@ -23,6 +23,8 @@
 #include <fcntl.h>
 #include <getopt.h>
 
+#include <gd.h>
+
 #include "tsys.h"
 #include "tmess.h"
 #include "tuis.h"
@@ -32,7 +34,7 @@
 //*************************************************
 TUIS::TUIS( ) : TSubSYS("UI","User interfaces",true)
 {
-
+    gdFTUseFontConfig(1);
 }
 
 string TUIS::optDescr( )
@@ -58,7 +60,7 @@ void TUIS::load_( )
 	next_opt=getopt_long(SYS->argc,(char * const *)SYS->argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': fprintf(stdout,optDescr().c_str()); break;
+	    case 'h': fprintf(stdout,TSYS::strEncode(optDescr(),TSYS::FormatPrint).c_str()); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
@@ -69,7 +71,7 @@ void TUIS::load_( )
 
 void TUIS::subStart(  )
 {
-#if OSC_DEBUG
+#if OSC_DEBUG >= 1
     mess_debug(nodePath().c_str(),_("Start subsystem."));
 #endif
 
@@ -78,7 +80,7 @@ void TUIS::subStart(  )
 
 void TUIS::subStop( )
 {
-#if OSC_DEBUG
+#if OSC_DEBUG >=1
     mess_debug(nodePath().c_str(),_("Stop subsystem."));
 #endif
 
@@ -99,14 +101,19 @@ bool TUIS::icoPresent(const string &inm, string *tp)
 
 string TUIS::icoGet(const string &inm, string *tp )
 {
-    int len;
+    int len, hd = -1, i_t;
     char buf[STR_BUF_LEN];
     string rez;
+    char types[][5] = {"png","gif","jpg","jpeg"};
 
-    int hd = open(icoPath(inm).c_str(),O_RDONLY);
+    for( i_t = 0; i_t < sizeof(types)/5; i_t++ )
+    {
+	hd = open(icoPath(inm,types[i_t]).c_str(),O_RDONLY);
+	if( hd != -1 ) break;
+    }
     if( hd != -1 )
     {
-	if( tp ) *tp = "png";
+	if( tp ) *tp = types[i_t];
 	while( len = read(hd,buf,sizeof(buf)) )
 	    rez.append(buf,len);
 	close(hd);
@@ -115,9 +122,9 @@ string TUIS::icoGet(const string &inm, string *tp )
     return rez;
 }
 
-string TUIS::icoPath(const string &ico)
+string TUIS::icoPath( const string &ico, const string &tp )
 {
-    return "./icons/"+ico+".png";
+    return SYS->icoDir()+"/"+ico+"."+tp;
 }
 
 void TUIS::cntrCmdProc( XMLNode *opt )
