@@ -26,18 +26,12 @@
 #include <vector>
 
 #include "terror.h"
+#include "tvariant.h"
 #include "tconfig.h"
 
 using std::string;
 using std::vector;
 
-//Error values
-#define EVAL_BOOL 2
-#define EVAL_INT  -2147483647
-#define EVAL_REAL -3.3E308
-#define EVAL_STR  "<EVAL>"
-
-
 //*************************************************
 //* TVal                                          *
 //*************************************************
@@ -55,22 +49,25 @@ class TVal : public TCntrNode
 	};
 
 	//Methods
-	TVal( TFld &fld, TValue *owner );
-	TVal( TCfg &cfg, TValue *owner );
+	TVal( );
+	TVal( TFld &fld );
+	TVal( TCfg &cfg );
 	~TVal( );
 
+	void setFld( TFld &fld );
+	void setCfg( TCfg &cfg );
+
 	const string &name( );
+	long long time( )	{ return mTime; }
 
-	TFld &fld( );
-
-	//- Read curent value (direct) -
+	//> Read curent value (direct)
 	string	getSEL( long long *tm = NULL, bool sys = false );
 	string	getS( long long *tm = NULL, bool sys = false );
 	double	getR( long long *tm = NULL, bool sys = false );
 	int	getI( long long *tm = NULL, bool sys = false );
 	char	getB( long long *tm = NULL, bool sys = false );
 
-	//- Set curent value -
+	//> Set curent value
 	void setSEL( const string &value, long long tm = 0, bool sys = false );
 	void setS( const string &value, long long tm = 0, bool sys = false );
 	void setR( double value, long long tm = 0, bool sys = false );
@@ -80,10 +77,12 @@ class TVal : public TCntrNode
 	AutoHD<TVArchive> arch( );
 	void setArch( const AutoHD<TVArchive> &vl );
 
+	TValue &owner( );
+	TFld &fld( );
+
     protected:
+	//Methods
 	void cntrCmdProc( XMLNode *opt );
-	void vlSet( );
-	void vlGet( );
 
     private:
 	//Methods
@@ -92,20 +91,20 @@ class TVal : public TCntrNode
 	//Attributes
 	union
 	{
-	    string	*val_s;		//string value
+	    ResString	*val_s;		//string value
 	    double	val_r;		//real value
 	    int		val_i;		//integer value
 	    char	val_b;		//boolean value
 	} val;
 
-	bool     m_cfg;		//Config id
+	bool	mCfg;		//Config id
 	union
 	{
 	    TFld *fld;
 	    TCfg *cfg;
 	} src;
-	long long time;		//Last value's time (usec)
-	AutoHD<TVArchive>	m_arch;
+	long long mTime;		//Last value's time (usec)
+	AutoHD<TVArchive>	mArch;
 };
 
 //*************************************************
@@ -121,48 +120,43 @@ class TValue: public TCntrNode, public TValElem
 	TValue( );
 	virtual ~TValue();
 
-	//- Atributes -
+	//> Atributes
 	void vlList( vector<string> &list )	{ chldList(m_vl,list); }
 	bool vlPresent( const string &name )	{ return chldPresent(m_vl,name); }
 	AutoHD<TVal> vlAt( const string &name )	{ return chldAt(m_vl,name); }
 
     protected:
+	//Methods
 	void cntrCmdProc( XMLNode *opt );	//Control interface command process
 
-	//- Manipulation for config element -
-	TConfig *vlCfg( )			{ return m_cfg; }
+	//> Manipulation for config element
+	TConfig *vlCfg( )			{ return mCfg; }
 	void setVlCfg( TConfig *cfg );		//Set configs. NULL - clear configs.
 
-	//- Manipulation for elements of value -
+	//> Manipulation for elements of value
 	bool vlElemPresent( TElem *ValEl );
 	void vlElemAtt( TElem *ValEl );
 	void vlElemDet( TElem *ValEl );
 	TElem &vlElem( const string &name );
 
-	virtual void vlSet( TVal &val )		{ };
+	virtual TVal* vlNew( );
+	virtual void vlSet( TVal &val, const TVariant &pvl )		{ };
 	virtual void vlGet( TVal &val )		{ };
 	virtual void vlArchMake( TVal &val )	{ };
 
     private:
 	//Methods
-	//- Set value direct into controller param's -
-	virtual void vlSet( int id_elem )
-	{ throw TError(nodePath().c_str(),_("Direct access to write value no admit.")); }
-	//- Get value direct from controller param's -
-	virtual void vlGet( int id_elem )
-	{ throw TError(nodePath().c_str(),_("Direct access to read value no admit.")); }
-
-	//- TElem commands -
+	//> TElem commands
 	void detElem( TElem *el );
-	void addFld( TElem *el, unsigned id_val); 
-	void delFld( TElem *el, unsigned id_val); 
+	void addFld( TElem *el, unsigned id_val);
+	void delFld( TElem *el, unsigned id_val);
 
 	//Attributes
 	char		m_vl;
-	vector<TElem*>	elem;   // Elements (dinamic parts)
+	vector<TElem*>	elem;		// Elements (dinamic parts)
 
-	short int	l_cfg;  // Config len
-	TConfig*	m_cfg;  // Configs (static parts)
+	short int	l_cfg;		// Config len
+	TConfig*	mCfg;		// Configs (static parts)
 };
 
 #endif // TVALUE_H
