@@ -29,6 +29,7 @@
 
 using std::string;
 using std::vector;
+using std::pair;
 
 //*************************************************
 //* XMLNode                                       *
@@ -39,65 +40,76 @@ class XMLNode
 	//Data
 	enum SaveView
 	{
-	    BrOpenPrev	= 0x01,		//Break preview open tag
-	    BrOpenPast	= 0x02,		//Break past open tag
-	    BrClosePast	= 0x04,		//Break past close tag
-	    BrTextPast	= 0x08		//Break past text
+	    BrOpenPrev		= 0x01,		//Break preview open tag
+	    BrOpenPast		= 0x02,		//Break past open tag
+	    BrClosePast		= 0x04,		//Break past close tag
+	    BrTextPast		= 0x08,		//Break past text
+	    BrPrcInstrPast	= 0x10,		//Break past process instruction
+	    BrAllPast		= 0x1E,		//Break past all
+	    XMLHeader		= 0x20,		//Include XML header
+	    InclNode		= 0x80		//Included node, not root
 	};
 
 	//Methods
-	XMLNode( const string &name = "" ) : m_name(name), m_text(""), current_node(NULL)	{  }
+	XMLNode( const string &name = "" ) : mName(name), mText(""), mParent(NULL)	{  }
 	~XMLNode( )				{ clear(); }
 
 	XMLNode &operator=( XMLNode &prm );
 
-	string	name( ) const			{ return m_name; }
-	XMLNode* setName( const string &s )	{ m_name = s; return this; }
+	string	name( ) const			{ return mName; }
+	XMLNode* setName( const string &s )	{ mName = s; return this; }
 
-	string	text( ) const			{ return m_text; }
-	XMLNode* setText( const string &s )	{ m_text = s; return this; }
+	string	text( ) const			{ return mText; }
+	XMLNode* setText( const string &s )	{ mText = s; return this; }
 
 	void	attrList( vector<string> & list ) const;
+	XMLNode* attrDel( const string &name );
 	void	attrClear( );
 	string	attr( const string &name ) const;
 	XMLNode* setAttr( const string &name, const string &val );
-	XMLNode* setAttr_( const char *name, const char *val );
+
+	void	prcInstrList( vector<string> & list ) const;
+	void	prcInstrDel( const string &target );
+	void	prcInstrClear( );
+	string	prcInstr( const string &target ) const;
+	XMLNode* setPrcInstr( const string &target, const string &val );
 
 	void	load( const string &vl );
 	string	save( unsigned char flgs = 0 );
 	XMLNode* clear( );
 
-	int	childSize( ) const		{ return m_children.size(); }
+	int	childSize( ) const		{ return mChildren.size(); }
 	void	childAdd( XMLNode *nd );
 	XMLNode* childAdd( const string &name = "" );
 	int	childIns( unsigned id, XMLNode *nd );
 	XMLNode* childIns( unsigned id, const string &name = "" );
 	void	childDel( const unsigned id );
-	void	childClean( const string &name = "" );
+	void	childDel( XMLNode *nd );
+	void	childClear( const string &name = "" );
 	XMLNode* childGet( const int, bool noex = false ) const;
-	XMLNode* childGet( const string &name, const int numb = 0 ) const;
+	XMLNode* childGet( const string &name, const int numb = 0, bool noex = false ) const;
 	XMLNode* childGet( const string &attr, const string &name, bool noex = false ) const;
 
+	XMLNode* parent( )			{ return mParent; }
     private:
 	//Methods
-	string encode ( const string &s ) const;
+	string encode( const string &s, bool text = false ) const;
 
-	static void start_element ( void *data, const char *el, const char **attr );
-	static void end_element ( void *data, const char *el );
-	static void characters ( void *userData, const XML_Char *s, int len );
+	static void start_element( void *data, const char *el, const char **attr );
+	static void end_element( void *data, const char *el );
+	static void characters( void *userData, const XML_Char *s, int len );
+	static void instrHandler( void *userData, const XML_Char *target, const XML_Char *data );
 
 	//Attributes
-	string m_name;
-	string m_text;
-	vector<XMLNode *> m_children;
-	vector<string>   n_attr;
-	vector<string>   v_attr;
-
-	static const char *o_name;
+	string mName;
+	string mText;
+	vector< XMLNode* >		mChildren;
+	vector< pair<string,string> >	mAttr;
+	vector< pair<string,string> >	mPrcInstr;
+	XMLNode* mParent;
 
 	//- Parse/load XML attributes -
 	vector<XMLNode*> node_stack;
-	XMLNode* current_node;
 };
 
 #endif  //XML_H
